unit BasicData;

interface

uses Windows, StdCtrls, Buttons, ComCtrls, Forms, Gauges,
     EditControls, DataTypes, HartModules, Controls, Classes;

type
  TfmBasicData = class(TForm)
    FixedDataBox: TGroupBox;
    ManufacturerDisplay: TLabelStringEdit;
    DeviceCodeDisplay: TLabelStringEdit;
    SoftwareRevisionDisplay: TLabelStringEdit;
    HardwareRevisionDisplay: TLabelStringEdit;
    ConfigurableDataBox: TGroupBox;
    TAG_Edit: TLabelStringEdit;
    DESCRIPTION_Edit: TLabelStringEdit;
    DateLabel: TLabel;
    Date_Picker: TDateTimePicker;
    CommGauge: TGauge;
    OK_Button: TBitBtn;
    Close_Button: TBitBtn;
    Today_Button: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure TAG_EditEnter(Sender: TObject);
    procedure TAG_EditChange(Sender: TObject);
    procedure DESCRIPTION_EditEnter(Sender: TObject);
    procedure DESCRIPTION_EditChange(Sender: TObject);
    procedure Date_PickerChange(Sender: TObject);
    procedure Today_ButtonClick(Sender: TObject);
  private
    { Private declarations }
    fDialogMode: TDialogMode;
    fSaveMode: TSaveMode;
    pHartModule: PBasicHartModule;
    procedure SetAllLabels;
    procedure ArrangeDisplayModeLook;
    procedure ArrangeConfigurationModeLook;
    procedure SetAllDisplayValues;
    procedure IndicateReceivedChar;
    procedure UpdateAcceptButton;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
    procedure SetMode(DMode: TDialogMode; SMode: TSaveMode);
  end;

var
  fmBasicData: TfmBasicData;

implementation

{$R *.dfm}

uses SysUtils, Graphics, Dialogs, ProcessUtils, StringUtils, DataInstances, LabelNumbers,
     DataConstants, HartDataTypes, HartDataConstants, HelpContextNumbers, ConversionProcs;

//****************************************************************************//

procedure TfmBasicData.SetAllLabels;
begin
  // Self.Caption will be set later according to the fDialogMode

  ManufacturerDisplay.LabelCaption:=LanguageStrings[flManufacturer];
  DeviceCodeDisplay.LabelCaption:=LanguageStrings[flDeviceCode];
  SoftwareRevisionDisplay.LabelCaption:=LanguageStrings[flSWRevision];
  HardwareRevisionDisplay.LabelCaption:=LanguageStrings[flHWRevision];

  TAG_Edit.LabelCaption:=DeleteAcceleratorKey(LanguageStrings[flTag]);
  DESCRIPTION_Edit.LabelCaption:=DeleteAcceleratorKey(LanguageStrings[flDescription]);
  DateLabel.Caption:=DeleteAcceleratorKey(LanguageStrings[flDate]);

  OK_Button.Caption:=LanguageStrings[blSetting];
  Close_Button.Caption:=LanguageStrings[blClose];
  Today_Button.Caption:=LanguageStrings[blSetToday];
end;

procedure TfmBasicData.ArrangeDisplayModeLook;
begin
  Caption:=' '+LanguageStrings[flDisplayBasic];
  HelpContext:=hcDisplayBasic;

  TAG_Edit.EditColor:=clSilver;
  DESCRIPTION_Edit.EditColor:=clSilver;
  Date_Picker.Color:=clSilver;

  ConfigurableDataBox.Enabled:=false;

  CommGauge.Visible:=false;
  
  OK_Button.Visible:=false;

  Close_Button.Top:=ConfigurableDataBox.Top+ConfigurableDataBox.Height+20;
  Close_Button.Left:=(Self.Width-Close_Button.Width) DIV 2;
  Today_Button.Visible:=false;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmBasicData.ArrangeConfigurationModeLook;
var lButtonGap: integer;
begin
  Caption:=' '+LanguageStrings[flConfigBasic];
  HelpContext:=hcConfigBasic;

  TAG_Edit.CharValidation:=cvEnabled;
  TAG_Edit.ControlChars:=CHARTAB;
  DESCRIPTION_Edit.CharValidation:=cvEnabled;
  DESCRIPTION_Edit.ControlChars:=CHARTAB;

  TAG_Edit.LabelCaption:=LanguageStrings[flTag];
  DESCRIPTION_Edit.LabelCaption:=LanguageStrings[flDescription];
  DateLabel.Caption:=LanguageStrings[flDate];

  ConfigurableDataBox.Enabled:=true;

  CommGauge.Visible:=true;
  CommGauge.Top:=ConfigurableDataBox.Top+ConfigurableDataBox.Height+20;

  OK_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Top:=OK_Button.Top;
  Today_Button.Top:=OK_Button.Top;

  lButtonGap:=(CommGauge.Width-OK_Button.Width-Close_Button.Width-Today_Button.Width) DIV 2;
  OK_Button.Left:=CommGauge.Left;
  Close_Button.Left:=CommGauge.Left+OK_Button.Width+lButtonGap;
  Today_Button.Left:=CommGauge.Left+OK_Button.Width+lButtonGap+Close_Button.Width+lButtonGap;

  Self.Height:=OK_Button.Top+80;

  // fConfigResult:=mrIgnore;
end;

procedure TfmBasicData.SetAllDisplayValues;
var lBasicDevicedata: THBasicDeviceData;
begin
  if (pHartModule<>nil) then
     begin
     lBasicDevicedata:=pHartModule^.BasicDeviceData;
     ManufacturerDisplay.EditText:=ManufacturersIdentNameFromHART(lBasicDeviceData.UID.ManufacturerIdentCode);
     DeviceCodeDisplay.EditText:=CDeviceShortNames[IdentToType(lBasicDeviceData.UID)];
     SoftwareRevisionDisplay.EditText:=IntToStr(lBasicDeviceData.UID.SoftwareRevision);
     HardwareRevisionDisplay.EditText:=HARTHardwareRevisionToString(lBasicDeviceData.UID.HardwareRevision);

     TAG_Edit.EditText:=lBasicDeviceData.TDD.TagName;
     DESCRIPTION_Edit.EditText:=lBasicDeviceData.TDD.Descriptor;
     Date_Picker.Date:=EncodeHARTDate(lBasicDevicedata.TDD.Day,lBasicDevicedata.TDD.Month,lBasicDevicedata.TDD.Year);
     end;
end;

procedure TfmBasicData.IndicateReceivedChar;
begin
  CommGauge.AddProgress(1);
  if CommGauge.Progress >= CommGauge.MaxValue then
     begin
     WaitMilliseconds(100);
     CommGauge.Progress:=0;
     end;
end;

procedure TfmBasicData.UpdateAcceptButton;
begin
  if pHartModule<>nil then
     OK_Button.Enabled:=not(pHartModule^.VerifyDeviceData);
end;

//****************************************************************************//

procedure TfmBasicData.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmBasicData.SetMode(DMode: TDialogMode; SMode: TSaveMode);
begin
  fDialogMode:=DMode; fSaveMode:=SMode;
end;

//****************************************************************************//

procedure TfmBasicData.FormCreate(Sender: TObject);
begin
  fDialogMode:=dmDisplay;
  fSaveMode:=smToMemory;

  pHartModule:=nil;

  ManufacturerDisplay.EditAlignment:=taLeftJustify;
  DeviceCodeDisplay.EditAlignment:=taLeftJustify;
  SoftwareRevisionDisplay.EditAlignment:=taLeftJustify;
  HardwareRevisionDisplay.EditAlignment:=taLeftJustify;

  ManufacturerDisplay.ReadOnly:=true;
  DeviceCodeDisplay.ReadOnly:=true;
  SoftwareRevisionDisplay.ReadOnly:=true;
  HardwareRevisionDisplay.ReadOnly:=true;

  ManufacturerDisplay.EditColor:=clSilver;
  DeviceCodeDisplay.EditColor:=clSilver;
  SoftwareRevisionDisplay.EditColor:=clSilver;
  HardwareRevisionDisplay.EditColor:=clSilver;

  FixedDataBox.Enabled:=false;

  TAG_Edit.EditAlignment:=taLeftJustify;
  DESCRIPTION_Edit.EditAlignment:=taLeftJustify;

  Date_Picker.MinDate:=CMinimalHARTDate; // date not before 01.01.1900
  Date_Picker.MaxDate:=CMaximalHARTDate; //   and not after 31.12.2155

  SetAllLabels;
end;

procedure TfmBasicData.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin
     if fDialogMode=dmDisplay then ArrangeDisplayModeLook;
     if fDialogMode=dmConfig then ArrangeConfigurationModeLook;

     SetAllDisplayValues;

     if fDialogMode=dmConfig then OK_Button.Enabled:=false;
     end
  else Close;
end;

procedure TfmBasicData.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var fExpectedTransferDataLength: integer;
    lUploadRequired: boolean;
begin
  CanClose:=(ModalResult<>mrOK);
  lUploadRequired:=false;
  if (pHartModule<>nil) and (fDialogMode=dmConfig) then
     begin
     // OriginalCursor:=Cursor;
     ConfigurableDataBox.Enabled:=false;
     OK_Button.Enabled:=false;
     Close_Button.Enabled:=false;
     Today_Button.Enabled:=false;
     if ((ModalResult<>mrOK) and not(pHartModule^.VerifyDeviceData)) then
        begin
        case (MessageDlg(LanguageStrings[mlChangedNotSaved],mtWarning,[mbYes,mbNo,mbCancel],hcNoHelpTopic)) of
             mrYes: lUploadRequired:=true;
             mrNo:  ModalResult:=mrIgnore;
             else CanClose:=false;
             end;
        end;

     if ((ModalResult=mrOK) or lUploadRequired) then
        begin
        pHartModule^.EvaluateRequiredDeviceDataStatusFlags;
        fExpectedTransferDataLength:=pHartModule^.GetModuleTransferDataLength(tdUpload);
        if (fExpectedTransferDataLength>0) then
           begin
           if (fSaveMode=smToDevice) then
              begin
              CommGauge.MaxValue:=pHartModule^.GetModuleTransferDataLength(tdUpload);
              CommGauge.Progress:=0;

              Cursor:=crHourGlass;
              Application.ProcessMessages;
              pHartModule^.UploadRequiredDeviceData;

              if not(pHartModule^.VerifyDeviceData) then
                 begin
                 MessageDlg(LanguageStrings[msDifferentResponse],mtError,[mbOk],hcNoHelpTopic);

                 SetAllDisplayValues;
                 pHartModule^.SynchronizeRequiredDeviceData;
                 UpdateAcceptButton;

                 ModalResult:=mrCancel;
                 end;

              end
           else // (fSaveMode=smToMemory)
              begin
              pHartModule^.SynchronizeActualDeviceData;
              MessageDlg(LanguageStrings[msDataSavedToMemory],mtConfirmation,[mbOk],hcNoHelpTopic);
              end;
           end;
        end;

     if not(CanClose) then
        begin
        ConfigurableDataBox.Enabled:=true;
        OK_Button.Enabled:=not(pHartModule^.VerifyDeviceData);
        Close_Button.Enabled:=true;
        Today_Button.Enabled:=true;
        end;
     Cursor:=OriginalCursor;
     end;
end;

procedure TfmBasicData.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmBasicData.TAG_EditEnter(Sender: TObject);
var S: string;
begin
  S:=DeleteTrailingChars(TAG_Edit.EditText,' ');
  if TAG_Edit.EditText<>S then TAG_Edit.EditText:=S;
end;

procedure TfmBasicData.TAG_EditChange(Sender: TObject);
var lBasicDeviceData: THBasicDeviceData;
begin
  if (pHartModule<>nil) then
     begin
     lBasicDeviceData:=pHartModule^.BasicDeviceData;
     lBasicDeviceData.TDD.TagName:=TAG_Edit.EditText;
     lBasicDeviceData.TDD.Descriptor:=DESCRIPTION_Edit.EditText;
     pHartModule^.BasicDeviceData:=lBasicDeviceData;
     UpdateAcceptButton;
     end;
end;

procedure TfmBasicData.DESCRIPTION_EditEnter(Sender: TObject);
var S: string;
begin
  S:=DeleteTrailingChars(DESCRIPTION_Edit.EditText,' ');
  if DESCRIPTION_Edit.EditText<>S then DESCRIPTION_Edit.EditText:=S;
end;

procedure TfmBasicData.DESCRIPTION_EditChange(Sender: TObject);
var lBasicDeviceData: THBasicDeviceData;
begin
  if (pHartModule<>nil) then
     begin
     lBasicDeviceData:=pHartModule^.BasicDeviceData;
     lBasicDeviceData.TDD.TagName:=TAG_Edit.EditText;
     lBasicDeviceData.TDD.Descriptor:=DESCRIPTION_Edit.EditText;
     pHartModule^.BasicDeviceData:=lBasicDeviceData;
     UpdateAcceptButton;
     end;
end;

procedure TfmBasicData.Date_PickerChange(Sender: TObject);
var lBasicDeviceData: THBasicDeviceData;
begin
  if (pHartModule<>nil) then
     begin
     lBasicDeviceData:=pHartModule^.BasicDeviceData;
     DecodeHARTDate(Date_Picker.Date,lBasicDeviceData.TDD.Day,lBasicDeviceData.TDD.Month,lBasicDeviceData.TDD.Year);
     pHartModule^.BasicDeviceData:=lBasicDeviceData;
     UpdateAcceptButton;
     end;
end;

procedure TfmBasicData.Today_ButtonClick(Sender: TObject);
var lBasicDeviceData: THBasicDeviceData;
begin
  if (Date_Picker.Date<>Date) then
     begin
     Date_Picker.Date:=Date;
     if (pHartModule<>nil) then
        begin
        lBasicDeviceData:=pHartModule^.BasicDeviceData;
        DecodeHARTDate(Date_Picker.Date,lBasicDeviceData.TDD.Day,lBasicDeviceData.TDD.Month,lBasicDeviceData.TDD.Year);
        pHartModule^.BasicDeviceData:=lBasicDeviceData;
        UpdateAcceptButton;
      end;
     end;
end;

end.
