unit Calculator;

interface

uses
  Windows, Forms, StdCtrls, Buttons, GroupControls, DataTypes, TempUtils, EditControls,
  Controls, Classes;

type
  TfmCalculator = class(TForm)
    InputTypeSelector: TMyRadioGroup;
    InputSelectorComboBox: TComboBox;
    Close_Button: TBitBtn;
    TemperatureEdit: TLabelRealEdit;
    ResultDisplay: TLabelDisplay;
    procedure FormCreate(Sender: TObject);
    procedure InputTypeSelectorClick(Sender: TObject);
    procedure InputSelectorComboBoxChange(Sender: TObject);
    procedure TemperatureEditChange(Sender: TObject);
  private
    { Private declarations }
    fInputType     : TTempInputType;
    fTHCType: TTHCType;
    fRTDType: TRTDType;
    fWrongType: boolean;
  public
    { Public declarations }
    procedure ArrangeInputSelectorComboBox;
  end;

var
  fmCalculator: TfmCalculator;

implementation

{$R *.dfm}

uses DataInstances, LabelNumbers, HelpContextNumbers;

procedure TfmCalculator.ArrangeInputSelectorComboBox;
begin
  with InputSelectorComboBox do
       begin
       Items.Clear;
       Text:=' please select';
       case fInputType of
            itTHC: begin
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_B]);
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_E]);
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_J]);
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_K]);
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_N]);
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_R]);
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_S]);
                   Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_T]);
                   // Items.Add(' '+LanguageStrings[flTType]+' '+LanguageStrings[flTHC_XK]);
                   end;
            itRTD: begin
                   Items.Add(' '+LanguageStrings[flPt100]);
                   {Items.Add(' '+LanguageStrings[flPt500]);
                   Items.Add(' '+LanguageStrings[flPt1000]);
                   Items.Add(' '+LanguageStrings[flNi100]);
                   Items.Add(' '+LanguageStrings[flNi500]);
                   Items.Add(' '+LanguageStrings[flNi1000]);
                   Items.Add(' '+LanguageStrings[flCu50]);
                   Items.Add(' '+LanguageStrings[flCu53]);}
                   end;
            end;
       end;
end;

procedure TfmCalculator.FormCreate(Sender: TObject);
begin
  Self.Caption:='Calculator';
  HelpContext:=hcCalculator;

  InputTypeSelector.Caption:=LanguageStrings[flInputType];
  InputTypeSelector.Items.Clear;
  InputTypeSelector.Items.Add(LanguageStrings[flTHC]);
  InputTypeSelector.Items.Add(LanguageStrings[flRTD]);

  fInputType:=itNone;
  fWrongType:=true;

  ArrangeInputSelectorComboBox;

  Close_Button.Caption:=LanguageStrings[blClose];
end;

procedure TfmCalculator.InputTypeSelectorClick(Sender: TObject);
begin
  case InputTypeSelector.ItemIndex of
       0: begin
          fInputType:=itTHC;
          ResultDisplay.LabelCaption:='Voltage';
          ResultDisplay.UnitText:='mV';
          ResultDisplay.RealPrecision:=3;
          end;
       1: begin
          fInputType:=itRTD;
          ResultDisplay.LabelCaption:='Resistance';
          ResultDisplay.UnitText:='Ohm';
          ResultDisplay.RealPrecision:=1;
          end
       else
          begin
          fInputType:=itNone;
          ResultDisplay.LabelCaption:='';
          ResultDisplay.UnitText:='';
          ResultDisplay.RealPrecision:=0;
          end;
       end;
  ArrangeInputSelectorComboBox;
  ResultDisplay.SetDisplay(0.0);
end;

procedure TfmCalculator.InputSelectorComboBoxChange(Sender: TObject);
begin
  fWrongType:=false;
  case fInputType of
       itTHC: case InputSelectorComboBox.ItemIndex of
                   0: fTHCType:=ttB;
                   1: fTHCType:=ttE;
                   2: fTHCType:=ttJ;
                   3: fTHCType:=ttK;
                   4: fTHCType:=ttN;
                   5: fTHCType:=ttR;
                   6: fTHCType:=ttS;
                   7: fTHCType:=ttT;
                   // 8: fTHCType:=ttXK;
                   else fWrongType:=true;
                   end;
       itRTD: case InputSelectorComboBox.ItemIndex of
                   0: fRTDType:=rtPlatin;
                   else fWrongType:=true;
                   end;
       end;
end;

procedure TfmCalculator.TemperatureEditChange(Sender: TObject);
begin
  if not(fWrongType) then
     case fInputType of
          itTHC: ResultDisplay.SetDisplay(GetITS90THCVoltage(fTHCType,TemperatureEdit.Value));
          itRTD: ResultDisplay.SetDisplay(GetRTDResistance(fRTDType,100.0,TemperatureEdit.Value));
          end;
end;

end.
