unit DataLogger;

interface

uses Windows, ExtCtrls, Buttons, Controls, StdCtrls, Gauges, Classes, SysUtils, Forms,
     Dialogs, EditControls, ArraysAndGrids, DataTypes, HartDataTypes, HartModules;

type
  TWhatLogger = (wlCurrent, wlProcess);
  TUnitMinSec = (tuSecond, tuMinute);

  PSetupLoggerData = ^TSetupLoggerData;
  TSetupLoggerData = record
    Signal: TWhatLogger;
    MinSec: TUnitMinSec;
    Interval: word;
    PointNumber: word;
    SaveName: TFileName;
    end;

  TControlLoggerData = record
    Running: boolean;
    RecordingStartTime,
    ActualRecordingTime: TDateTime;
    RecordingStartTickCount: dword;
    ActualPoint, WhichOV: word;
    end;

  TfmDataLogger = class(TForm)
    LoggerSettingBox: TGroupBox;
    MeasureValueSelection: TRadioGroup;
    IntervalValueEdit: TLabelIntegerEdit;
    IntervalUnitSelection: TComboBox;
    RecordNumberEdit: TLabelIntegerEdit;
    ChannelSelectionBox: TGroupBox;
    ChannelLabel: TLabel;
    ChannelSelection: TComboBox;
    FileNameEdit: TLabelStringEdit;
    FileName_Button: TButton;

    LoggerControlBox: TGroupBox;
    MeasureGauge: TGauge;
    RecordValueDisplay: TLabelDisplay;
    Panel1: TPanel;
    ElapsedDisplay: TLabelDisplay;
    RemainingDisplay: TLabelDisplay;
    Start_Stop_Button: TButton;
    CommGauge: TGauge;
    Close_Button: TBitBtn;
    LoggerTimer: TTimer;

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure MeasureValueSelectionClick(Sender: TObject);
    procedure IntervalValueEditRangeValidationError(Sender: TObject);
    procedure IntervalUnitSelectionXClick(Sender: TObject);
    procedure RecordNumberEditRangeValidationError(Sender: TObject);
    procedure ChannelSelectionChange(Sender: TObject);
    procedure FileName_ButtonClick(Sender: TObject);
    procedure Start_Stop_ButtonClick(Sender: TObject);
    procedure LoggerTimerTimer(Sender: TObject);
  private
    { Private declarations }
    fDeviceType: TDeviceType;
    pHartModule: PBasicHartModule;
    fFreeNameSelection,
    fFullScale,
    fFirstRequest,
    fTimerExecution: boolean;
    fDefaultPath: TFileName;
    fSetupLoggerData: TSetupLoggerData;
    fControlLoggerData: TControlLoggerData;
    fLoggerDataResults: TRealDynGrid;
    LoggerSaveDialog: TSaveDialog;
    procedure IndicateReceivedChar;
    procedure CommonPrefixLoggerLook;
    procedure LoggerLook_x073;
    procedure LoggerLook_x576;
    procedure LoggerLook_x532;
    procedure LoggerLook_1003;
    procedure CommonSuffixLoggerLook;

    function GetLoggerData: THDynamicVariables;
    procedure RefreshLogger(ProcessData, CurrentData: real);

    function StartDataRecording: boolean;
    procedure ContinueDataRecording;
    procedure CloseRecordingMode;
    procedure FinishDataRecording;
    procedure LoggerSaveDialogSelectionChange(Sender: TObject);
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
  end;

var
  fmDataLogger: TfmDataLogger;

implementation

{$R *.dfm}

uses Types, ProcessUtils, StringUtils, LnCRCFileStream, DataInstances, DataConstants,
     HelpContextNumbers, LabelNumbers, HartDataConstants, ConversionProcs;

const
  CDefLogger: TSetupLoggerData = (
              Signal: wlProcess;
              MinSec:  tuSecond;
              Interval: 10;
              PointNumber: 100);

  CLoggerInterval = 1000;
  CLog576Interval = 2000;

function GetMillisecondValue(Mode: TUnitMinSec; Value: TDateTime): integer;
begin
  Result:=0;
end;

procedure TfmDataLogger.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(10);
     end
  else WaitMilliseconds(100);
end;

procedure TfmDataLogger.CommonPrefixLoggerLook;
begin
  // Captions and Labels

  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlDataLogger]);

  LoggerSettingBox.Caption:=LanguageStrings[flLoggerSetup];

  with MeasureValueSelection do
       begin
       Caption:=LanguageStrings[flMeasureValue];
       Items.Clear;
       Items.Add(LanguageStrings[flProcessValueShort]);
       Items.Add(LanguageStrings[flCurrentValue]);
       if fSetupLoggerData.Signal=wlProcess then ItemIndex:=0 else ItemIndex:=1;
       end;

  with IntervalValueEdit do
       begin
       LabelCaption:=LanguageStrings[flIntervalValue];
       Value:=fSetupLoggerData.Interval;
       end;

  with IntervalUnitSelection do
       begin
       Items.Clear;
       Items.Add(LanguageStrings[flSecUnits]);
       Items.Add(LanguageStrings[flMinUnits]);
       if fSetupLoggerData.MinSec=tuSecond then ItemIndex:=0 else ItemIndex:=1;
       end;

  with RecordNumberEdit do
       begin
       LabelCaption:=LanguageStrings[flRecordCount];
       Value:=fSetupLoggerData.PointNumber;
       end;

  ChannelSelectionBox.Visible:=false;

  FileNameEdit.LabelCaption:=LanguageStrings[blFileName];
  FileNameEdit.EditAlignment:=taLeftJustify;

  if ExtractFilePath(fSetupLoggerData.SaveName)=fDefaultPath then
     FileNameEdit.EditText:=ExtractFileName(fSetupLoggerData.SaveName)
  else FileNameEdit.EditText:=StripPathString(fSetupLoggerData.SaveName,30);

  LoggerControlBox.Caption:=LanguageStrings[flLoggerControl];

  RecordValueDisplay.LabelCaption:=LanguageStrings[flProcessValueShort];
  ElapsedDisplay.LabelCaption:=LanguageStrings[flElapsedTime];
  RemainingDisplay.LabelCaption:=LanguageStrings[flRemainingTime];

  Start_Stop_Button.Caption:=LanguageStrings[blStart];

  Close_Button.Caption:=LanguageStrings[blClose];

  // Dimensions

  Self.Width:=325;

  LoggerSettingBox.Top:=15;
  LoggerSettingBox.Width:=290;

  LoggerControlBox.Width:=290;

end;

procedure TfmDataLogger.LoggerLook_x073;
begin
  // Captions and Labels

  RecordValueDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
  RecordValueDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));

  // Dimensions

  CommGauge.MaxValue:=22;
end;

procedure TfmDataLogger.LoggerLook_x576;
var lx576Channel: integer;
begin
  // Captions and Labels

  if (THartModule_x576(pHartModule^).ChannelNumber in [0,1]) then
     lx576Channel:=THartModule_x576(pHartModule^).ChannelNumber
  else lx576Channel:=0;

  RecordValueDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[lx576Channel].PVUnits)];
  RecordValueDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[lx576Channel].PVUnits));

  ChannelSelectionBox.Visible:=true;

  ChannelLabel.Caption:=LanguageStrings[flChannel];

  ChannelSelection.ItemIndex:=lx576Channel;

  // Dimensions

  CommGauge.MaxValue:=33;
end;

procedure TfmDataLogger.LoggerLook_x532;
begin
  // Captions and Labels

  RecordValueDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
  RecordValueDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));

  // Dimensions

  CommGauge.MaxValue:=22;
end;

procedure TfmDataLogger.LoggerLook_1003;
begin
  // Captions and Labels

  RecordValueDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
  RecordValueDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));

  // Dimensions

  CommGauge.MaxValue:=22;
end;

procedure TfmDataLogger.CommonSuffixLoggerLook;
begin
  // Captions and Labels

  RecordValueDisplay.SetDisplay(0.0);
  ElapsedDisplay.SetDisplay(0.0);
  RemainingDisplay.SetDisplay(0.0);

  // Dimensions

  if not(ChannelSelectionBox.Visible) then
     begin
     FileNameEdit.Top:=MeasureValueSelection.Top+MeasureValueSelection.Height+13;
     FileName_Button.Top:=MeasureValueSelection.Top+MeasureValueSelection.Height+15;
     end
  else
     begin
     FileNameEdit.Top:=ChannelSelectionBox.Top+ChannelSelectionBox.Height+13;
     FileName_Button.Top:=ChannelSelectionBox.Top+ChannelSelectionBox.Height+15;
     end;

  LoggerSettingBox.Height:= FileName_Button.Top+ FileName_Button.Height+13;

  LoggerControlBox.Top:=LoggerSettingBox.Top + LoggerSettingBox.Height + 10;

  Close_Button.Top:=LoggerControlBox.Top+LoggerControlBox.Height+20;
  Close_Button.Left:=(Self.Width-Close_Button.Width) DIV 2;

  Self.Height:=Close_Button.Top+80;
end;

function TfmDataLogger.GetLoggerData: THDynamicVariables;
var NewLoggerData: THDynamicVariables;
begin
  if (pHartModule<>nil) and
     ((fDeviceType<>dtx576) and (pHartModule^.GeTHDynamicVariables(NewLoggerData)))  or
     ((fDeviceType=dtx576) and (THartModule_x576(pHartModule^).GeTHDynamicVariables(NewLoggerData))) then
     Result:=NewLoggerData
  else Result:=CDef_DynamicVariables;
end;

procedure TfmDataLogger.RefreshLogger(ProcessData, CurrentData: real);
begin
  if fSetupLoggerData.Signal=wlProcess then
     RecordValueDisplay.SetDisplay(ProcessData)
  else RecordValueDisplay.SetDisplay(CurrentData);
  if not(fFullScale) then
     MeasureGauge.Progress:=round((CurrentData-4.0)*6.25)
  else MeasureGauge.Progress:=round(CurrentData*5.0);
end;

function TfmDataLogger.StartDataRecording: boolean;
var lLoggerData: THDynamicVariables;
    lServiceDuration: dword;
    lResults: TRealDynArray;
    LDuration: TDateTime;
begin
  Result:=false;

  LoggerSettingBox.Enabled:=false;
  Start_Stop_Button.Caption:=LanguageStrings[blFinish];
  Close_Button.Enabled:=false;

  with fSetupLoggerData do
       begin
       ClearDynGrid(fLoggerDataResults);
       SetLength(fLoggerDataResults,0);

       PointNumber:=RecordNumberEdit.Value;
       fControlLoggerData.ActualPoint:=0;
       fControlLoggerData.Running:=true;
       fControlLoggerData.RecordingStartTime:=Now;

       fControlLoggerData.RecordingStartTickCount:=GetTickCount;
       lLoggerData:=GetLoggerData;
       CommGauge.Progress:=0;

       RefreshLogger (lLoggerData.PrimaryVariable,lLoggerData.LoopCurrent);

       SetLength(lResults,2);
       lResults[0]:=0.0;
       if Signal=wlProcess then
          lResults[1]:=lLoggerData.PrimaryVariable
       else lResults[1]:=lLoggerData.LoopCurrent;
       AppendArrayToGrid(lResults,fLoggerDataResults);

       Interval:=IntervalValueEdit.Value;
       if Interval>60 then Interval:=60;
       if MinSec=tuSecond then
          lDuration:=PointNumber * Interval * CSecond
       else lDuration:=PointNumber * Interval * CMinute;

       ElapsedDisplay.SetDisplay(0.0);
       RemainingDisplay.SetDisplay(lDuration);

       lServiceDuration:=GetTickCount-fControlLoggerData.RecordingStartTickCount;
       if (MinSec=tuMinute) or
          not((Interval*1000-lServiceDuration)<500) then // at least 500ms free time
          begin
          if MinSec=tuSecond then
             LoggerTimer.Interval:=(Interval*1000-lServiceDuration)
          else
             begin
             if (Interval>1) then
                LoggerTimer.Interval:=60000
             else LoggerTimer.Interval:=60000-lServiceDuration;
             LoggerTimer.Interval:=60000;
             fControlLoggerData.WhichOV:=0;
             end;

          Result:=true;
          LoggerTimer.Enabled:=true;
          end;
       end;
end;

procedure TfmDataLogger.ContinueDataRecording;
var lLoggerData: THDynamicVariables;
    lResults: TRealDynArray;
    lDuration: TDateTime;
begin
  fControlLoggerData.ActualRecordingTime:=Now;
  lLoggerData:=GetLoggerData;
  CommGauge.Progress:=0;

  with fSetupLoggerData do
       begin
       RefreshLogger (lLoggerData.PrimaryVariable,lLoggerData.LoopCurrent);

       SetLength(lResults,2);
       if MinSec=tuSecond then
          lResults[0]:=(fControlLoggerData.ActualRecordingTime-fControlLoggerData.RecordingStartTime)*24*60*60
       else lResults[0]:=(fControlLoggerData.ActualRecordingTime-fControlLoggerData.RecordingStartTime)*24*60;
       if Signal=wlProcess then
          lResults[1]:=lLoggerData.PrimaryVariable
       else lResults[1]:=lLoggerData.LoopCurrent;
       AppendArrayToGrid(lResults,fLoggerDataResults);

       if MinSec=tuSecond then
          lDuration:=(PointNumber-fControlLoggerData.ActualPoint) * Interval * CSecond
       else lDuration:=(PointNumber-fControlLoggerData.ActualPoint) * Interval * CMinute;

       ElapsedDisplay.SetDisplay(fControlLoggerData.ActualRecordingTime-fControlLoggerData.RecordingStartTime);
       RemainingDisplay.SetDisplay(lDuration);
       end;
end;

procedure TfmDataLogger.CloseRecordingMode;
begin
  if fControlLoggerData.Running then fControlLoggerData.Running:=false;

  LoggerSettingBox.Enabled:=true;
  Start_Stop_Button.Caption:=LanguageStrings[blStart];
  FileName_Button.Enabled:=true;
  Close_Button.Enabled:=true;
end;

procedure TfmDataLogger.FinishDataRecording;
var TargetStream: TLnCRCFileStream;
    lAppendLine: TStringDynArray;
    lResultInfo,
    lFooterInfo: TStringDynGrid;
    I, RowCount: integer;
{$IFDEF ExcelSupport}
    SheetName: string;
{$ENDIF}
    SepChar: char;
begin
  RowCount:=System.Length(fLoggerDataResults);
  if (fControlLoggerData.ActualPoint+1)<RowCount then RowCount:=fControlLoggerData.ActualPoint+1;

  if (RowCount>1) then
     begin

    for I:=0 to RowCount-1 do
        begin
        SetLength(lAppendLine,2);
        lAppendLine[0]:=FloatToStrF(fLoggerDataResults[I,0],ffFixed,6,2);  // what precision is required here ?
        lAppendLine[1]:=FloatToStrF(fLoggerDataResults[I,1],ffFixed,6,4);
        AppendArrayToGrid(lAppendLine,lResultInfo);
        end;

     SetLength(lAppendLine,1);
     lAppendLine[0]:='#';
     AppendArrayToGrid(lAppendLine,lFooterInfo);

     SetLength(lAppendLine,3);
     lAppendLine[0]:=DeleteAcceleratorKey(LanguageStrings[blStart]);
     lAppendLine[1]:=DateToStr(fControlLoggerData.RecordingStartTime);
     lAppendLine[2]:=TimeToStr(fControlLoggerData.RecordingStartTime);
     AppendArrayToGrid(lAppendLine,lFooterInfo);

     SetLength(lAppendLine,2);
     lAppendLine[0]:=DeleteAcceleratorKey(LanguageStrings[flMeasureValue]);
     case fSetupLoggerData.Signal of
          wlProcess: lAppendLine[1]:=LanguageStrings[flProcessValue];
          wlCurrent: lAppendLine[1]:=LanguageStrings[flCurrentValue];
          else lAppendLine[1]:='';
          end;
     AppendArrayToGrid(lAppendLine,lFooterInfo);

     SetLength(lAppendLine,3);
     lAppendLine[0]:=LanguageStrings[flIntervalValue];
     lAppendLine[1]:=IntToStr(fSetupLoggerData.Interval);
     case fSetupLoggerData.MinSec of
          tuSecond: lAppendLine[2]:=LanguageStrings[flSecUnits];
          tuMinute: lAppendLine[2]:=LanguageStrings[flMinUnits];
          else lAppendLine[2]:='';
          end;
     AppendArrayToGrid(lAppendLine,lFooterInfo);

     SetLength(lAppendLine,2);
     lAppendLine[0]:=DeleteAcceleratorKey(LanguageStrings[flRecordCount]);
     lAppendLine[1]:=IntToStr(RowCount);
     AppendArrayToGrid(lAppendLine,lFooterInfo);
     end;

  if (LowerCase(ExtractFileExt(fSetupLoggerData.SaveName))<>LowerCase(CDef_ExcelExt))then
     begin
     if (LowerCase(ExtractFileExt(fSetupLoggerData.SaveName))=LowerCase(CDef_CsvIExExt)) then SepChar:=ListSeparator
     else SepChar:=CTAB;

     TargetStream:=TLnCRCFileStream.Create(fSetupLoggerData.SaveName,fmCreate);

     TargetStream.WriteStringDynGridLn(lResultInfo,SepChar);
     TargetStream.WriteStringDynGridLn(lFooterInfo,SepChar);

     TargetStream.Free;
{$IFDEF ExcelSupport}
     end
  else // save data to Excel file
     begin
     SheetName:='Logger data'; // DemoData.MDD.TDD.TagName + '  ' + FloatToStr(HeaderInfo.RecordingStartTime);

     if OpenExistingOrCreateNewExcelWorkbookFile(ExcelApp,fSetupLoggerData.SaveName,SheetName) then
        begin
        WriteDynRealGridToSheetRangeInActiveExcelWorkbook(ExcelApp,fLoggerDataResults,1,1,1);
        WriteDynStringGridToSheetRangeInActiveExcelWorkbook(ExcelApp,lAppendInfo,1,1,RowCount+1);
        SaveActiveExcelWorkbook(ExcelApp);
        CloseActiveExcelWorkbook(ExcelApp);
        end;
{$ENDIF}
     end;

  CloseRecordingMode;
end;

procedure TfmDataLogger.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmDataLogger.FormCreate(Sender: TObject);
begin
  fDeviceType:=dtNone;
  pHartModule:=nil;

  HelpContext:=hcDataLogger;
  fSetupLoggerData:=CDefLogger;

  fTimerExecution:=false;
  fFreeNameSelection:=MOD45.PasswordReleased;

  if (MOD45.LastLoggerPath<>'') then
     fDefaultPath:=MOD45.LastLoggerPath
  else fDefaultPath:=GetMyDocumentsFolder;

  if ExcelAvailable then fSetupLoggerData.SaveName:=fDefaultPath+'\'+Def_ExcelFile
  else fSetupLoggerData.SaveName:=fDefaultPath+'\'+CDef_CsvIExFile;

  LoggerTimer.Enabled:=false;
end;

procedure TfmDataLogger.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin
     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);
     if (fDeviceType<>dtNone) then
        begin

        CommonPrefixLoggerLook;

        case fDeviceType of
             dt507x,
             dtx073,
             dt5074XK,
             dt4073XK,
             dt4075,
             dtx575,
             dtx573: LoggerLook_x073;
             dtx576: LoggerLook_x576;
             dtx532: LoggerLook_x532;
             dt1003: LoggerLook_1003;
             end;

        CommonSuffixLoggerLook;

        if (fDeviceType in CFrequencyConverter) and
           (THartModule_Freq(pHartModule^).FullScale>0) then
           fFullScale:=true
        else fFullScale:=false;

        fFirstRequest:=true;
        LoggerTimer.Interval:=100;
        LoggerTimer.Enabled:=true;
        end
     else Close;
     end
  else Close;
end;

procedure TfmDataLogger.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not(fControlLoggerData.Running);
  if not(CanClose) then
     MessageDlg (LanguageStrings[msCannotBreakLogger],mtConfirmation,[mbOk],hcNoHelpTopic);
end;

procedure TfmDataLogger.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmDataLogger.MeasureValueSelectionClick(Sender: TObject);
begin
  if MeasureValueSelection.ItemIndex=0 then fSetupLoggerData.Signal:=wlProcess
  else fSetupLoggerData.Signal:=wlCurrent;

  if fSetupLoggerData.Signal=wlProcess then
     begin
     RecordValueDisplay.LabelCaption:=LanguageStrings[flProcessValueShort];
     if (fDeviceType<>dtx576) then
        begin
        RecordValueDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
        RecordValueDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));
        end
     else
        begin
        RecordValueDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[THartModule_x576(pHartModule^).ChannelNumber].PVUnits)];
        RecordValueDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[THartModule_x576(pHartModule^).ChannelNumber].PVUnits));
        end;
     end
  else
     begin
     RecordValueDisplay.LabelCaption:=LanguageStrings[flCurrentValue];
     RecordValueDisplay.UnitText:=LanguageStrings[flmiliAmp];
     RecordValueDisplay.RealPrecision:=3;
     end;
  RecordValueDisplay.SetDisplay(0.0);
end;

procedure TfmDataLogger.IntervalUnitSelectionXClick(Sender: TObject);
begin
  if IntervalUnitSelection.ItemIndex=0 then fSetupLoggerData.MinSec:=tuSecond
  else fSetupLoggerData.MinSec:=tuMinute;

  if fSetupLoggerData.MinSec=tuSecond then
     with IntervalValueEdit do
          begin
          Min:=2;
          Max:=60;
          if Value<Min then Value:=Min;
          end
  else // fSetupLoggerData.MinSec=tuMinute
     with IntervalValueEdit do
          begin
          Min:=1;
          Max:=30;
          if Value>Max then Value:=Max;
          end;
end;

procedure TfmDataLogger.IntervalValueEditRangeValidationError(Sender: TObject);
begin
  with IntervalValueEdit do
       if Value<Min then Value:=Min
       else if Value>Max then Value:=Max;
end;

procedure TfmDataLogger.RecordNumberEditRangeValidationError(Sender: TObject);
begin
  with RecordNumberEdit do
       if Value<Min then Value:=Min
       else if Value>Max then Value:=Max;
end;

procedure TfmDataLogger.ChannelSelectionChange(Sender: TObject);
begin
  if (fDeviceType=dtx576) and
     (ChannelSelection.ItemIndex in [0,1]) then
     begin
     THartModule_x576(pHartModule^).ChannelNumber:=(ChannelSelection.ItemIndex);

     if fSetupLoggerData.Signal=wlProcess then
        begin
        RecordValueDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[THartModule_x576(pHartModule^).ChannelNumber].PVUnits)];
        RecordValueDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[THartModule_x576(pHartModule^).ChannelNumber].PVUnits));
        end;
     end;
end;

procedure TfmDataLogger.LoggerSaveDialogSelectionChange(Sender: TObject);
begin
  if (LoggerSaveDialog<>nil) and ExcelAvailable then
     begin
     if LoggerSaveDialog.FilterIndex=1 then LoggerSaveDialog.DefaultExt:=LowerCase(CDef_ExcelExt)
     else LoggerSaveDialog.DefaultExt:=LowerCase(CDef_CsvIExExt);
     end
  else LoggerSaveDialog.DefaultExt:=LowerCase(CDef_CsvIExExt);
end;

procedure TfmDataLogger.FileName_ButtonClick(Sender: TObject);
begin
  LoggerSaveDialog:=TSaveDialog.Create(Self);
  with LoggerSaveDialog do
       begin
       Title:=' '+LanguageStrings[dlSaveFile];
       OnSelectionChange:=LoggerSaveDialogSelectionChange;
       if ExcelAvailable then
          begin
          Filter:=LanguageStrings[dsXlsCsvFilterList];
          DefaultExt:=LowerCase(CDef_ExcelExt);
          end
       else
          begin
          Filter:=LanguageStrings[dsCsvFilterList];
          DefaultExt:=LowerCase(CDef_CsvIExExt);
          end;
       InitialDir:=fDefaultPath;

       if Execute then
          begin
          if fFreeNameSelection then fSetupLoggerData.SaveName:=FileName
          else fSetupLoggerData.SaveName:=ExtractFilePath(FileName)+CDef_FileName+DefaultExt;
          if not(LowerCase(ExtractFileExt(fSetupLoggerData.SaveName))=LowerCase(DefaultExt)) then
             fSetupLoggerData.SaveName:=ChangeFileExt(fSetupLoggerData.SaveName,DefaultExt);
          if ExtractFilePath(fSetupLoggerData.SaveName)=fDefaultPath then
             FileNameEdit.EditText:=ExtractFileName(fSetupLoggerData.SaveName)
          else FileNameEdit.EditText:=StripPathString(fSetupLoggerData.SaveName,30);
          MOD45.LastLoggerPath:=ExtractFilePath(FileName);
          end;
       end;
  LoggerSaveDialog.Free;
end;

procedure TfmDataLogger.Start_Stop_ButtonClick(Sender: TObject);
var MessageInfo: string;
begin
  if not(fControlLoggerData.Running) then
     begin
     if not(StartDataRecording) then
        begin
        MessageDlg (LanguageStrings[msShortInterval],mtError,[mbCancel],hcNoHelpTopic);
        CloseRecordingMode;
        end;
     end
  else  // fControlLoggerData.Running
     begin
     while fTimerExecution do;
     LoggerTimer.Enabled:=false;

     FinishDataRecording;
     FmtStr(MessageInfo,LanguageStrings[msRecordBreak],[ExtractFileName(fSetupLoggerData.SaveName)]);
     MessageDlg (MessageInfo,mtInformation,[mbOk],hcNoHelpTopic);
     end;
end;

procedure TfmDataLogger.LoggerTimerTimer(Sender: TObject);
var lLoggerData: THDynamicVariables;
    lExpectedTickCount: dword;
    MessageInfo: string;
begin
  LoggerTimer.Enabled:=false;
  if fFirstRequest then
     begin
     fFirstRequest:=false;
     lLoggerData:=GetLoggerData;
     CommGauge.Progress:=0;
     end
  else // not(fFirstRequest)
     begin
     fTimerExecution:=true;

     if (fSetupLoggerData.MinSec=tuSecond) or ((fControlLoggerData.WhichOV+1)=fSetupLoggerData.Interval) then
        begin
        System.Inc(fControlLoggerData.ActualPoint);

        ContinueDataRecording;

        if (fControlLoggerData.ActualPoint<fSetupLoggerData.PointNumber) then
           begin
           if (fSetupLoggerData.MinSec=tuSecond) then
              begin
              lExpectedTickCount:=fControlLoggerData.RecordingStartTickCount+(fControlLoggerData.ActualPoint+1)*fSetupLoggerData.Interval*1000;

              LoggerTimer.Interval:=lExpectedTickCount-GetTickCount;
              end
           else // (fSetupLoggerData.MinSec=tuMinute)
              begin
              if (fSetupLoggerData.Interval>1) then
                 LoggerTimer.Interval:=60000
              else
                 begin
                 lExpectedTickCount:=fControlLoggerData.RecordingStartTickCount+(fControlLoggerData.ActualPoint+1)*fSetupLoggerData.Interval*60000;
                 LoggerTimer.Interval:=lExpectedTickCount-GetTickCount;
                 end;
              fControlLoggerData.WhichOV:=0;
              end;

           LoggerTimer.Enabled:=true;
           end
        else // not(fControlLoggerData.ActualPoint<fSetupLoggerData.PointNumber)
           begin
           FinishDataRecording;
           FmtStr(MessageInfo,LanguageStrings[msRecordComplete],[ExtractFileName(fSetupLoggerData.SaveName)]);
           MessageDlg (MessageInfo,mtInformation,[mbOk],hcNoHelpTopic);
           end;
        fControlLoggerData.WhichOV:=0;
        end
     else // (fSetupLoggerData.MinSec=tuMinute) and ((fControlLoggerData.WhichOV+1)<fSetupLoggerData.Interval)
        begin
        System.Inc(fControlLoggerData.WhichOV);
        if ((fControlLoggerData.WhichOV+1)=fSetupLoggerData.Interval) then
           begin
           lExpectedTickCount:=fControlLoggerData.RecordingStartTickCount+(fControlLoggerData.ActualPoint+1)*fSetupLoggerData.Interval*60000;
           LoggerTimer.Interval:=lExpectedTickCount-GetTickCount;
           end;
        LoggerTimer.Enabled:=true;
        end;
     fTimerExecution:=false;
     end;
end;

end.
