unit DigitalDiagnostics;

interface

uses Windows, Buttons, StdCtrls, Controls, Classes, Gauges, Forms, EditControls,
     DataTypes, HartModules;

type
  TfmDigitalDiagnostics = class(TForm)
    AlarmGroupBox: TGroupBox;
    AlarmOnButton: TButton;
    AlarmOffButton: TButton;
    AlarmPulseButton: TButton;
    PulseGroupBox: TGroupBox;
    PulseOnButton: TButton;
    PulseOffButton: TButton;
    PulsePulseButton: TButton;
    PulseFrequencyEdit: TLabelIntegerEdit;
    FrequencyButton: TButton;
    ErrorLEDGroupBox: TGroupBox;
    ErrorLEDOnButton: TButton;
    ErrorLEDOffButton: TButton;
    ErrorLEDPulseButton: TButton;
    CommGauge: TGauge;
    Close_Button: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure AlarmOnButtonClick(Sender: TObject);
    procedure AlarmOffButtonClick(Sender: TObject);
    procedure AlarmPulseButtonClick(Sender: TObject);
    procedure PulseOnButtonClick(Sender: TObject);
    procedure PulseOffButtonClick(Sender: TObject);
    procedure PulsePulseButtonClick(Sender: TObject);
    procedure FrequencyButtonClick(Sender: TObject);
    procedure ErrorLEDOnButtonClick(Sender: TObject);
    procedure ErrorLEDOffButtonClick(Sender: TObject);
    procedure ErrorLEDPulseButtonClick(Sender: TObject);
  private
    { Private declarations }
    fDeviceType: TDeviceType;
    pHartModule: PBasicHartModule;
    fTransferActive,
    fFrequencyPulseActivated: boolean;
    procedure IndicateReceivedChar;
    procedure CommonPrefixDigitalDiagnosticsLook;
    procedure DigitalDiagnosticsLook_x532;
    procedure DigitalDiagnosticsLook_1003;
    procedure CommonSuffixDigitalDiagnosticsLook;
    procedure PerformDiagnosticFunction(Number: byte);
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
  end;

var
  fmDigitalDiagnostics: TfmDigitalDiagnostics;

implementation

{$R *.dfm}

uses StringUtils, ProcessUtils, DataInstances, LabelNumbers, HelpContextNumbers,
     ConversionProcs;

procedure TfmDigitalDiagnostics.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(10);
     end
  else WaitMilliseconds(100);
end;

procedure TfmDigitalDiagnostics.CommonPrefixDigitalDiagnosticsLook;
begin
  // Captions and Displays

  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlSetDigitalOutputs]);

  AlarmGroupBox.Caption:=LanguageStrings[flAlarmOutput];
  AlarmOnButton.Caption:=LanguageStrings[blOn];
  AlarmOffButton.Caption:=LanguageStrings[blOff];
  AlarmPulseButton.Caption:=LanguageStrings[blPulse];

  PulseGroupBox.Caption:=LanguageStrings[flPulseOutput];
  PulseOnButton.Caption:=LanguageStrings[blOn];
  PulseOffButton.Caption:=LanguageStrings[blOff];
  PulsePulseButton.Caption:=LanguageStrings[blPulse];
  PulseFrequencyEdit.LabelCaption:=LanguageStrings[flFreq];
  PulseFrequencyEdit.UnitText:=LanguageStrings[flHerz];
  FrequencyButton.Caption:=LanguageStrings[blSet];

  ErrorLEDGroupBox.Caption:=LanguageStrings[flLEDOutput];
  ErrorLEDOnButton.Caption:=LanguageStrings[blOn];
  ErrorLEDOffButton.Caption:=LanguageStrings[blOff];
  ErrorLEDPulseButton.Caption:=LanguageStrings[blPulse];

  Close_Button.Caption:=LanguageStrings[blClose];

  // Dimensions

  Self.Width:=275;

  AlarmGroupBox.Top:=15;
  AlarmGroupBox.Width:=240;

  PulseGroupBox.Top:=80;
  PulseGroupBox.Width:=240;

  ErrorLEDGroupBox.Width:=240;

  CommGauge.Width:=240;
end;

procedure TfmDigitalDiagnostics.DigitalDiagnosticsLook_x532;
begin
  PulseFrequencyEdit.Visible:=true;
  FrequencyButton.Visible:=true;

  PulseGroupBox.Height:=85;

  ErrorLEDGroupBox.Top:=175;

end;

procedure TfmDigitalDiagnostics.DigitalDiagnosticsLook_1003;
begin
  PulseFrequencyEdit.Visible:=false;
  FrequencyButton.Visible:=false;

  PulseGroupBox.Height:=55;

  ErrorLEDGroupBox.Top:=145;

end;

procedure TfmDigitalDiagnostics.CommonSuffixDigitalDiagnosticsLook;
begin
  CommGauge.Top:=ErrorLEDGroupBox.Top+ErrorLEDGroupBox.Height+15;

  Close_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Left:=CommGauge.Left+(CommGauge.Width-Close_Button.Width) DIV 2;

  Close_Button.Enabled:=false;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmDigitalDiagnostics.PerformDiagnosticFunction(Number: byte);
begin
  AlarmGroupBox.Enabled:=false;
  PulseGroupBox.Enabled:=false;
  ErrorLEDGroupBox.Enabled:=false;

  Close_Button.Enabled:=false;
  CommGauge.MaxValue:=0;
  Application.ProcessMessages;

  if (Number<>39) then
     begin
     fTransferActive:=true;
     if (Number in [33,35,37]) and fFrequencyPulseActivated then
        begin
        CommGauge.MaxValue:=CommGauge.MaxValue+8;
        THartModule_Freq(pHartModule^).SetDigitalDiagnostic(13);
        fFrequencyPulseActivated:=false;
        end;

     CommGauge.MaxValue:=CommGauge.MaxValue+8;
     THartModule_Freq(pHartModule^).SetDigitalDiagnostic(Number);
     end;

  WaitMilliseconds(50);
  CommGauge.Progress:=0;

  fTransferActive:=false;

  AlarmGroupBox.Enabled:=true;
  PulseGroupBox.Enabled:=true;
  ErrorLEDGroupBox.Enabled:=true;
  Close_Button.Enabled:=true;
end;

procedure TfmDigitalDiagnostics.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmDigitalDiagnostics.FormCreate(Sender: TObject);
begin
  fDeviceType:=dtNone;
  pHartModule:=nil;

  fTransferActive:=false;
  fFrequencyPulseActivated:=false;

  HelpContext:=hcSetDigital;
end;

procedure TfmDigitalDiagnostics.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin
     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);
     if (fDeviceType<>dtNone) then
        begin

        CommonPrefixDigitalDiagnosticsLook;

        case fDeviceType of
             dtx532: DigitalDiagnosticsLook_x532;
             dt1003: DigitalDiagnosticsLook_1003;
             else Close;
             end;

        CommonSuffixDigitalDiagnosticsLook;
        end
     else Close;
     end
  else Close;
end;

procedure TfmDigitalDiagnostics.FormActivate(Sender: TObject);
begin
  PerformDiagnosticFunction(13);
end;

procedure TfmDigitalDiagnostics.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not(fFrequencyPulseActivated or fTransferActive);
end;

procedure TfmDigitalDiagnostics.FormDestroy(Sender: TObject);
begin
  PerformDiagnosticFunction(15);

  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmDigitalDiagnostics.AlarmOnButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(43);
end;

procedure TfmDigitalDiagnostics.AlarmOffButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(45);
end;

procedure TfmDigitalDiagnostics.AlarmPulseButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(47);
end;

procedure TfmDigitalDiagnostics.PulseOnButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(33);
end;

procedure TfmDigitalDiagnostics.PulseOffButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(35);
end;

procedure TfmDigitalDiagnostics.PulsePulseButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(37);
end;

procedure TfmDigitalDiagnostics.FrequencyButtonClick(Sender: TObject);
begin
  CommGauge.MaxValue:=8;
  Application.ProcessMessages;
  if (fDeviceType=dtx532) then
     THartModule_x532(pHartModule^).SetDigitalDiagnostic(39,PulseFrequencyEdit.Value);
  fFrequencyPulseActivated:=true;

  WaitMilliseconds(100);
  CommGauge.Progress:=0;
end;

procedure TfmDigitalDiagnostics.ErrorLEDOnButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(23);
end;

procedure TfmDigitalDiagnostics.ErrorLEDOffButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(25);
end;

procedure TfmDigitalDiagnostics.ErrorLEDPulseButtonClick(Sender: TObject);
begin
  PerformDiagnosticFunction(27);
end;

end.
