unit HardwareConfig;

interface

uses Windows, Forms, StdCtrls, Buttons, ExtCtrls, Controls, Gauges, Classes,
     HartModules, DataTypes, ComCtrls, EditControls, HartDataTypes;

type
  THWConfigMode = (hmNone, hmCircuitry,hmTrimmer);

  TfmHardwareConfig = class(TForm)
    PageControl1: TPageControl;
    HardwareCircuitryTabSheet: TTabSheet;
    LFDRadioGroup: TRadioGroup;
    RelayRadioGroup: TRadioGroup;
    TrimmerCalibrationTabSheet: TTabSheet;
    ActualTrimmerConfigurationGroupBox: TGroupBox;
    ConfigMiddleOffsetDisplay: TLabelDisplay;
    ConfigMiddleGainDisplay: TLabelDisplay;
    ConfigLimitOffsetDisplay: TLabelDisplay;
    ConfigLimitGainDisplay: TLabelDisplay;

    ActualTrimmerGroupBox: TGroupBox;
    ActualOffsetDisplay: TLabelDisplay;
    ActualGainDisplay: TLabelDisplay;
    MinAndMaxTrimmerConfigurationGroupBox: TGroupBox;
    MinOffsetDisplay: TLabelDisplay;
    MinGainDisplay: TLabelDisplay;
    MaxOffsetDisplay: TLabelDisplay;
    MaxGainDisplay: TLabelDisplay;
    EvaluatedTrimmerConfigurationGroupBox: TGroupBox;
    EvaluatedMiddleOffsetDisplay: TLabelDisplay;
    EvaluatedMiddleGainDisplay: TLabelDisplay;
    EvaluatedLimitOffsetDisplay: TLabelDisplay;
    EvaluatedLimitGainDisplay: TLabelDisplay;

    CommGauge: TGauge;
    OK_Button: TBitBtn;
    Close_Button: TBitBtn;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure PageControl1Changing(Sender: TObject; var AllowChange: Boolean);
    procedure PageControl1Change(Sender: TObject);
    procedure PageControl1DrawTab(Control: TCustomTabControl; TabIndex: Integer;
      const Rect: TRect; Active: Boolean);
    procedure HardwareRadioGroupClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    fDeviceType: TDeviceType;
    pHartModule: PBasicHartModule;
    fHWConfigMode: THWConfigMode;
    fTransferActive,
    fTrimmerReady,
    fPageChangeRequest: boolean;
    fActualHardwareCircuitry,
    fRequiredHardwareCircuitry: byte;
    fActualTrimmerValues,
    fActualConfigMiddle,
    fActualConfigLimit,
    fEvaluatedMin,
    fEvaluatedMax,
    fEvaluatedMiddle,
    fEvaluatedLimit: THTrimmerVariables_1003;

    procedure IndicateReceivedChar;

    procedure SetAllLabels;
    procedure ArrangeControls;

    procedure SetDefaultMinMaxValues;
    procedure RefreshActualTrimmer;
    procedure CalculateAndRefreshConfig;
    procedure RefreshConfigTrimmerValues;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
  end;

var
  fmHardwareConfig: TfmHardwareConfig;

implementation

{$R *.dfm}

uses Dialogs, Graphics, ProcessUtils, DataInstances, LabelNumbers, HelpContextNumbers,
     HartDataConstants,  ConversionProcs;

type
  TMinMax = record
    MinValue             : word;
    MaxValue             : word;
    end;

  TUpperLower = record
    LowerValue           : word;
    UpperValue           : word;
    end;

  TEvaluationFlags = record
    MinOffset, MaxOffset,
    MinGain, MaxGain,
    EvaluatedMiddleOffset,
    EvaluatedLimitOffset,
    EvaluatedMiddleGain,
    EvaluatedLimitGain,
    ConfigMiddleOffset,
    ConfigLimitOffset,
    ConfigMiddleGain,
    ConfigLimitGain,
    ActualOffset,
    ActualGain: boolean;
    end;

const
  CAbsoluteTrimmerRange: TMinMax = (MinValue: 100; MaxValue: 5000);

  CExpectedTrimmerRange: TMinMax = (MinValue: 600; MaxValue: 1900);

  CAcceptableTrimmerDeviation:  TUpperLower = (LowerValue: 250; UpperValue: 450);
  CAcceptableConfigDeviation:   TUpperLower = (LowerValue: 25; UpperValue: 45);

procedure TfmHardwareConfig.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(10);
     end
  else WaitMilliseconds(100);
end;

procedure TfmHardwareConfig.SetAllLabels;
begin
  Caption:='Hardware Configuration'; // ' '+DeleteAcceleratorKey(LanguageStrings[mlHardwareConfig]);

  HardwareCircuitryTabSheet.Caption:='Hardware Circuitry';

  LFDRadioGroup.Caption:='LFD Input circuit';
  LFDRadioGroup.Items.Clear;
  LFDRadioGroup.Items.Add('not present'); // LanguageStrings[flNotPresent]);
  LFDRadioGroup.Items.Add('present');     // LanguageStrings[flPresent]);

  RelayRadioGroup.Caption:='Relay Output';
  RelayRadioGroup.Items.Clear;
  RelayRadioGroup.Items.Add('not present'); // LanguageStrings[flNotPresent]);
  RelayRadioGroup.Items.Add('present');     // LanguageStrings[flPresent]);

  TrimmerCalibrationTabSheet.Caption:='Trimmer Calibration';

  ActualTrimmerConfigurationGroupBox.Caption:='Actual Trimmer Configuration';
  ActualTrimmerGroupBox.Caption:='Actual Trimmer Values';
  MinAndMaxTrimmerConfigurationGroupBox.Caption:='Min-Max Trimmer Configuration';
  EvaluatedTrimmerConfigurationGroupBox.Caption:='Evaluated Trimmer Configuration';

  ConfigMiddleOffsetDisplay.LabelCaption:='Mid. Offset';
  ConfigMiddleGainDisplay.LabelCaption:='Mid. Gain';
  ConfigLimitOffsetDisplay.LabelCaption:='Lim. Offset';
  ConfigLimitGainDisplay.LabelCaption:='Lim. Gain';

  ActualOffsetDisplay.LabelCaption:='Offset';
  ActualGainDisplay.LabelCaption:='Gain';

  MinOffsetDisplay.LabelCaption:='Min. Offset';
  MinGainDisplay.LabelCaption:='Min. Gain';
  MaxOffsetDisplay.LabelCaption:='Max. Offset';
  MaxGainDisplay.LabelCaption:='Max. Gain';

  EvaluatedMiddleOffsetDisplay.LabelCaption:='Mid. Offset';
  EvaluatedMiddleGainDisplay.LabelCaption:='Mid. Gain';
  EvaluatedLimitOffsetDisplay.LabelCaption:='Lim. Offset';
  EvaluatedLimitGainDisplay.LabelCaption:='Lim. Gain';

  OK_Button.Caption:=LanguageStrings[blSetting];
  Close_Button.Caption:=LanguageStrings[blClose];
end;

procedure TfmHardwareConfig.ArrangeControls;
var lButtonGap: integer;
begin
  CommGauge.Top:=PageControl1.Top+PageControl1.Height+15;

  OK_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Top:=OK_Button.Top;

  lButtonGap:=(CommGauge.Width-OK_Button.Width-Close_Button.Width) DIV 3;
  OK_Button.Left:=CommGauge.Left+lButtonGap;
  Close_Button.Left:=CommGauge.Left+OK_Button.Width+2*lButtonGap;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmHardwareConfig.SetDefaultMinMaxValues;
begin
  fEvaluatedMin.OffsetValue:=CAbsoluteTrimmerRange.MaxValue;
  fEvaluatedMax.OffsetValue:=CAbsoluteTrimmerRange.MinValue;
  fEvaluatedMin.GainValue:=CAbsoluteTrimmerRange.MaxValue;
  fEvaluatedMax.GainValue:=CAbsoluteTrimmerRange.MinValue;

  MinOffsetDisplay.SetDisplay(fEvaluatedMin.OffsetValue);
  MaxOffsetDisplay.SetDisplay(fEvaluatedMax.OffsetValue);
  MinGainDisplay.SetDisplay(fEvaluatedMin.GainValue);
  MaxGainDisplay.SetDisplay(fEvaluatedMax.GainValue);
end;

procedure TfmHardwareConfig.RefreshActualTrimmer;
begin
  THartModule_1003(pHartModule^).GetTrimmerInformation(1,fActualTrimmerValues);
  ActualOffsetDisplay.SetDisplay(fActualTrimmerValues.OffsetValue);
  ActualGainDisplay.SetDisplay(fActualTrimmerValues.GainValue);
end;

procedure TfmHardwareConfig.CalculateAndRefreshConfig;
  function Difference (Value1, Value2: word): word;
  begin
    Difference:=System.Abs(longint(Value1) - longint(Value2));
  end;
var lEvaluationFlags: TEvaluationFlags;
    lChangedFlag: boolean;
begin
  lChangedFlag:=false;
  if (fActualTrimmerValues.OffsetValue>CAbsoluteTrimmerRange.MinValue) and (fActualTrimmerValues.OffsetValue<fEvaluatedMin.OffsetValue) then
     begin
     fEvaluatedMin.OffsetValue:=fActualTrimmerValues.OffsetValue;
     MinOffsetDisplay.SetDisplay(fEvaluatedMin.OffsetValue);
     lChangedFlag:=true;
     end;
  lEvaluationFlags.MinOffset:=not(Difference(fEvaluatedMin.OffsetValue,CExpectedTrimmerRange.MinValue)>CAcceptableTrimmerDeviation.LowerValue);
  if lEvaluationFlags.MinOffset then
     MinOffsetDisplay.DisplayColor:=clLime
  else MinOffsetDisplay.DisplayColor:=clRed;

  if (fActualTrimmerValues.OffsetValue<CAbsoluteTrimmerRange.MaxValue) and (fActualTrimmerValues.OffsetValue>fEvaluatedMax.OffsetValue) then
     begin
     fEvaluatedMax.OffsetValue:=fActualTrimmerValues.OffsetValue;
     MaxOffsetDisplay.SetDisplay(fEvaluatedMax.OffsetValue);
     lChangedFlag:=true;
     end;
  lEvaluationFlags.MaxOffset:=not(Difference(fEvaluatedMax.OffsetValue,CExpectedTrimmerRange.MaxValue)>CAcceptableTrimmerDeviation.UpperValue);
  if lEvaluationFlags.MaxOffset then
     MaxOffsetDisplay.DisplayColor:=clLime
  else MaxOffsetDisplay.DisplayColor:=clRed;

  if lChangedFlag then
     begin
     fEvaluatedMiddle.OffsetValue:=(fEvaluatedMax.OffsetValue + fEvaluatedMin.OffsetValue) DIV 2;
     EvaluatedMiddleOffsetDisplay.SetDisplay(fEvaluatedMiddle.OffsetValue);
     fEvaluatedLimit.OffsetValue:=(fEvaluatedMax.OffsetValue - fEvaluatedMin.OffsetValue) DIV 2;
     EvaluatedLimitOffsetDisplay.SetDisplay(fEvaluatedLimit.OffsetValue);
     end;

  lEvaluationFlags.EvaluatedMiddleOffset:=not(Difference(fEvaluatedMiddle.OffsetValue,(CExpectedTrimmerRange.MinValue + CExpectedTrimmerRange.MaxValue) DIV 2)>CAcceptableTrimmerDeviation.LowerValue);
  if lEvaluationFlags.EvaluatedMiddleOffset then
     EvaluatedMiddleOffsetDisplay.DisplayColor:=clLime
  else EvaluatedMiddleOffsetDisplay.DisplayColor:=clRed;
  lEvaluationFlags.ConfigMiddleOffset:=not(Difference(fEvaluatedMiddle.OffsetValue,fActualConfigMiddle.OffsetValue)>CAcceptableConfigDeviation.LowerValue);
  if lEvaluationFlags.ConfigMiddleOffset then
     ConfigMiddleOffsetDisplay.DisplayColor:=clLime
  else ConfigMiddleOffsetDisplay.DisplayColor:=clRed;
  lEvaluationFlags.EvaluatedLimitOffset:=not(Difference(fEvaluatedLimit.OffsetValue,(CExpectedTrimmerRange.MaxValue - CExpectedTrimmerRange.MinValue) DIV 2)>CAcceptableTrimmerDeviation.LowerValue);
  if lEvaluationFlags.EvaluatedLimitOffset then
     EvaluatedLimitOffsetDisplay.DisplayColor:=clLime
  else EvaluatedLimitOffsetDisplay.DisplayColor:=clRed;
  lEvaluationFlags.ConfigLimitOffset:=not(Difference(fEvaluatedLimit.OffsetValue,fActualConfigLimit.OffsetValue)>CAcceptableConfigDeviation.LowerValue);
  if lEvaluationFlags.ConfigLimitOffset then
     ConfigLimitOffsetDisplay.DisplayColor:=clLime
  else ConfigLimitOffsetDisplay.DisplayColor:=clRed;
  lEvaluationFlags.ActualOffset:=not(Difference(fActualTrimmerValues.OffsetValue,fEvaluatedMiddle.OffsetValue)>CAcceptableConfigDeviation.LowerValue);
  if lEvaluationFlags.ActualOffset then
     ActualOffsetDisplay.DisplayColor:=clLime
  else ActualOffsetDisplay.DisplayColor:=clRed;

  lChangedFlag:=false;
  if (fActualTrimmerValues.GainValue>CAbsoluteTrimmerRange.MinValue) and (fActualTrimmerValues.GainValue<fEvaluatedMin.GainValue) then
     begin
     fEvaluatedMin.GainValue:=fActualTrimmerValues.GainValue;
     MinGainDisplay.SetDisplay(fEvaluatedMin.GainValue);
     lChangedFlag:=true;
     end;
  lEvaluationFlags.MinGain:=not(Difference(fEvaluatedMin.GainValue,CExpectedTrimmerRange.MinValue)>CAcceptableTrimmerDeviation.LowerValue);
  if lEvaluationFlags.MinGain then
     MinGainDisplay.DisplayColor:=clLime
  else MinGainDisplay.DisplayColor:=clRed;

  if (fActualTrimmerValues.GainValue<CAbsoluteTrimmerRange.MaxValue) and (fActualTrimmerValues.GainValue>fEvaluatedMax.GainValue) then
     begin
     fEvaluatedMax.GainValue:=fActualTrimmerValues.GainValue;
     MaxGainDisplay.SetDisplay(fEvaluatedMax.GainValue);
     lChangedFlag:=true;
     end;
  lEvaluationFlags.MaxGain:=not(Difference(fEvaluatedMax.GainValue,CExpectedTrimmerRange.MaxValue)>CAcceptableTrimmerDeviation.UpperValue);
  if lEvaluationFlags.MaxGain then
     MaxGainDisplay.DisplayColor:=clLime
  else MaxGainDisplay.DisplayColor:=clRed;

  if lChangedFlag then begin
     fEvaluatedMiddle.GainValue:=(fEvaluatedMax.GainValue + fEvaluatedMin.GainValue) DIV 2;
     EvaluatedMiddleGainDisplay.SetDisplay(fEvaluatedMiddle.GainValue);
     fEvaluatedLimit.GainValue:=(fEvaluatedMax.GainValue - fEvaluatedMin.GainValue) DIV 2;
     EvaluatedLimitGainDisplay.SetDisplay(fEvaluatedLimit.GainValue);
     end;

  lEvaluationFlags.EvaluatedMiddleGain:=not(Difference(fEvaluatedMiddle.GainValue,(CExpectedTrimmerRange.MinValue + CExpectedTrimmerRange.MaxValue) DIV 2)>CAcceptableTrimmerDeviation.LowerValue);
  if lEvaluationFlags.EvaluatedMiddleGain then
     EvaluatedMiddleGainDisplay.DisplayColor:=clLime
  else EvaluatedMiddleGainDisplay.DisplayColor:=clRed;
  lEvaluationFlags.ConfigMiddleGain:=not(Difference(fEvaluatedMiddle.GainValue,fActualConfigMiddle.GainValue)>CAcceptableConfigDeviation.UpperValue);
  if lEvaluationFlags.ConfigMiddleGain then
     ConfigMiddleGainDisplay.DisplayColor:=clLime
  else ConfigMiddleGainDisplay.DisplayColor:=clRed;
  lEvaluationFlags.EvaluatedLimitGain:=not(Difference(fEvaluatedLimit.GainValue,(CExpectedTrimmerRange.MaxValue - CExpectedTrimmerRange.MinValue) DIV 2)>CAcceptableTrimmerDeviation.LowerValue);
  if lEvaluationFlags.EvaluatedLimitGain then
     EvaluatedLimitGainDisplay.DisplayColor:=clLime
  else EvaluatedLimitGainDisplay.DisplayColor:=clRed;
  lEvaluationFlags.ConfigLimitGain:=not(Difference(fEvaluatedLimit.GainValue,fActualConfigLimit.GainValue)>CAcceptableConfigDeviation.UpperValue);
  if lEvaluationFlags.ConfigLimitGain then
     ConfigLimitGainDisplay.DisplayColor:=clLime
  else ConfigLimitGainDisplay.DisplayColor:=clRed;
  lEvaluationFlags.ActualGain:=not(Difference(fActualTrimmerValues.GainValue,fEvaluatedMiddle.GainValue)>CAcceptableConfigDeviation.LowerValue);
  if lEvaluationFlags.ActualGain then
     ActualGainDisplay.DisplayColor:=clLime
  else ActualGainDisplay.DisplayColor:=clRed;

  with lEvaluationFlags do
       fTrimmerReady:=MinOffset and
                      MaxOffset and
                      MinGain and
                      MaxGain and
                      EvaluatedMiddleOffset and
                      EvaluatedLimitOffset and
                      EvaluatedMiddleGain and
                      EvaluatedLimitGain and
                      ActualOffset and
                      ActualGain;

  OK_Button.Enabled:=fTrimmerReady;
end;

procedure TfmHardwareConfig.RefreshConfigTrimmerValues;
begin
  THartModule_1003(pHartModule^).GetTrimmerInformation(2,fActualConfigMiddle);
  ConfigMiddleOffsetDisplay.SetDisplay(fActualConfigMiddle.OffsetValue);
  ConfigMiddleGainDisplay.SetDisplay(fActualConfigMiddle.GainValue);

  THartModule_1003(pHartModule^).GetTrimmerInformation(3,fActualConfigLimit);
  ConfigLimitOffsetDisplay.SetDisplay(fActualConfigLimit.OffsetValue);
  ConfigLimitGainDisplay.SetDisplay(fActualConfigLimit.GainValue);
end;

procedure TfmHardwareConfig.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmHardwareConfig.FormCreate(Sender: TObject);
begin
  fDeviceType:=dtNone;
  pHartModule:=nil;

  HelpContext:=hcHardwareConfig;

  fActualHardwareCircuitry:=0;
  fRequiredHardwareCircuitry:=0;

  PageControl1.ActivePage:=HardwareCircuitryTabSheet;
  fHWConfigMode:=hmCircuitry;

  fTransferActive:=false;
  fTrimmerReady:=false;
  fPageChangeRequest:=false;

  PageControl1.OwnerDraw:=true;

  CommGauge.MaxValue:=8;
end;

procedure TfmHardwareConfig.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin
     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);

     if (fDeviceType=dt1003) then
        begin
        SetAllLabels;
        ArrangeControls;

        fActualHardwareCircuitry:=THartModule_1003(pHartModule^).HardwareCircuitry;
        fRequiredHardwareCircuitry:=fActualHardwareCircuitry;

        if ((fActualHardwareCircuitry and CP3_LFDPresent)>0) then
           LFDRadioGroup.ItemIndex:=1
        else LFDRadioGroup.ItemIndex:=0;
        if ((fActualHardwareCircuitry and CP3_RelayPresent)>0) then
           RelayRadioGroup.ItemIndex:=1
        else RelayRadioGroup.ItemIndex:=0;

        SetDefaultMinMaxValues;

        OK_Button.Enabled:=false;
        Close_Button.Enabled:=true;
        end
     else Close;
     end
  else Close;
end;

procedure TfmHardwareConfig.FormActivate(Sender: TObject);
begin
  fTransferActive:=true;
  CommGauge.MaxValue:=24;
  RefreshConfigTrimmerValues;
  WaitMilliseconds(100);
  CommGauge.Progress:=0;
  fTransferActive:=false;

  CommGauge.MaxValue:=8;
end;

procedure TfmHardwareConfig.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var lActualHardwareCircuitry: byte;
begin
  CanClose:=false;

  if not(fTransferActive) then
     begin
     fTransferActive:=true;

     case ModalResult of
          mrOK: if (fHWConfigMode=hmCircuitry) then
                   begin
                   CommGauge.MaxValue:=8;
                   if THartModule_1003(pHartModule^).SetHardwareCircuitryConfiguration(fRequiredHardwareCircuitry) and
                      THartModule_1003(pHartModule^).GetHardwareCircuitryConfiguration(lActualHardwareCircuitry) then
                      begin
                      fActualHardwareCircuitry:=lActualHardwareCircuitry;
                      OK_Button.Enabled:=false;
                      WaitMilliseconds(250);
                      CommGauge.Progress:=0;
                      end;
                   end
                else if (fHWConfigMode=hmTrimmer) then
                        begin
                        Timer1.Enabled:=false;
                        CommGauge.MaxValue:=16;
                        if THartModule_1003(pHartModule^).SetTrimmerInformation(2,fEvaluatedMiddle) and
                           THartModule_1003(pHartModule^).SetTrimmerInformation(3,fEvaluatedLimit) then
                           begin
                           OK_Button.Enabled:=false;
                           WaitMilliseconds(250);
                           CommGauge.Progress:=0;
                           CommGauge.MaxValue:=24;
                           RefreshConfigTrimmerValues;
                           WaitMilliseconds(100);
                           CommGauge.Progress:=0;
                           end;
                        Timer1.Enabled:=true;
                        end;
          else  if ((fHWConfigMode=hmCircuitry) and (fRequiredHardwareCircuitry<>fActualHardwareCircuitry)) or
                   ((fHWConfigMode=hmTrimmer) and (fTrimmerReady))  then
                   begin
                   case (MessageDlg(LanguageStrings[mlChangedNotSaved],mtWarning,[mbYes,mbNo,mbCancel],hcNoHelpTopic)) of
                        mrYes: begin
                               if (fHWConfigMode=hmCircuitry) then
                                  begin
                                  CommGauge.MaxValue:=8;
                                  if THartModule_1003(pHartModule^).SetHardwareCircuitryConfiguration(fRequiredHardwareCircuitry) and
                                     THartModule_1003(pHartModule^).GetHardwareCircuitryConfiguration(lActualHardwareCircuitry) then
                                     begin
                                     WaitMilliseconds(250);
                                     ModalResult:=mrOK;
                                     CanClose:=true;
                                     end
                                  else
                                     begin
                                     MessageDlg('Unexpected error. Parameter change was not saved to device.',mtError,[mbCancel],hcNoHelpTopic);
                                     CanClose:=false;
                                     end;
                                  end
                               else if (fHWConfigMode=hmTrimmer) then
                                       begin
                                       Timer1.Enabled:=false;
                                       CommGauge.MaxValue:=16;
                                       if THartModule_1003(pHartModule^).SetTrimmerInformation(2,fEvaluatedMiddle) and
                                          THartModule_1003(pHartModule^).SetTrimmerInformation(3,fEvaluatedLimit) then
                                          begin
                                          WaitMilliseconds(250);
                                          ModalResult:=mrOK;
                                          CanClose:=true;
                                          end
                                       else
                                          begin
                                          MessageDlg('Unexpected error. Parameter change was not saved to device.',mtError,[mbCancel],hcNoHelpTopic);
                                          CanClose:=false;
                                          end;
                                       end;
                               end;
                        mrNo:  begin
                               ModalResult:=mrIgnore;
                               CanClose:=true;
                               end;
                        end;
                   end
                else CanClose:=true;
             end;
     fTransferActive:=false;
     end;
end;

procedure TfmHardwareConfig.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmHardwareConfig.PageControl1Changing(Sender: TObject;
  var AllowChange: Boolean);
begin
  fPageChangeRequest:=fTransferActive;
  AllowChange:=not(fTransferActive);
end;

procedure TfmHardwareConfig.PageControl1Change(Sender: TObject);
begin
  Timer1.Enabled:=false;
  if (PageControl1.ActivePage=HardwareCircuitryTabSheet) then
     begin
     fHWConfigMode:=hmCircuitry;
     CommGauge.MaxValue:=8;
     OK_Button.Enabled:=(fRequiredHardwareCircuitry<>fActualHardwareCircuitry);
     end
  else if (PageControl1.ActivePage=TrimmerCalibrationTabSheet) then
          begin
          fHWConfigMode:=hmTrimmer;
          CommGauge.MaxValue:=12;
          OK_Button.Enabled:=false;
          Timer1.Enabled:=true;
          end
       else fHWConfigMode:=hmNone;
end;

procedure TfmHardwareConfig.PageControl1DrawTab(Control: TCustomTabControl;
  TabIndex: Integer; const Rect: TRect; Active: Boolean);
begin
  if (Active) then
     Control.Canvas.Font.Style:=Control.Canvas.Font.Style+[fsBold]
  else Control.Canvas.Font.Style:=Control.Canvas.Font.Style-[fsBold];
  Control.Canvas.TextOut(Rect.Left+5,Rect.Top+3,PageControl1.Pages[TabIndex].Caption);
end;

procedure TfmHardwareConfig.HardwareRadioGroupClick(Sender: TObject);
begin
  if not(fTransferActive) then
     begin
     if (LFDRadioGroup.ItemIndex>0) then
        fRequiredHardwareCircuitry:=CP3_LFDPresent
     else fRequiredHardwareCircuitry:=0;
     if (RelayRadioGroup.ItemIndex>0) then
        fRequiredHardwareCircuitry:=fRequiredHardwareCircuitry or CP3_RelayPresent;

     OK_Button.Enabled:=(fRequiredHardwareCircuitry<>fActualHardwareCircuitry);
     end;
end;

procedure TfmHardwareConfig.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled:=false;
  fTransferActive:=true;

  RefreshActualTrimmer;
  CalculateAndRefreshConfig;

  WaitMilliseconds(100);
  CommGauge.Progress:=0;

  fTransferActive:=false;

  if fPageChangeRequest then
     begin
     fPageChangeRequest:=false;
     PageControl1.ActivePage:=HardwareCircuitryTabSheet;
     PageControl1Change(Sender);
     end
  else Timer1.Enabled:=true;
end;

end.
