unit Inspection;

interface

uses Windows, Forms, Classes, Controls, StdCtrls, Buttons,
     ExtCtrls, Gauges, EditControls, HartModules, DataTypes;

type
  TfmInspection = class(TForm)
    FlagsAndModesGroupBox: TGroupBox;
    FirstFlagDisplay: TLabelDisplay;
    SecondFlagDisplay: TLabelDisplay;
    ThirdFlagDisplay: TLabelDisplay;
    FourthFlagDisplay: TLabelDisplay;
    FifthFlagDisplay: TLabelDisplay;
    SixthFlagDisplay: TLabelDisplay;
    SeventhFlagDisplay: TDisplay;

    CalibrationParametersGroupBox: TGroupBox;
    LowCalibration: TDisplay;
    HighCalibration: TDisplay;
    AdjustmentParametersGroupBox: TGroupBox;
    LowAdjustment: TDisplay;
    HighAdjustment: TDisplay;
    OtherParametersGroupBox: TGroupBox;
    Label2: TLabel;
    Label1: TLabel;
    EighthDisplay: TDisplay;
    NinthDisplay: TDisplay;

    CommGauge: TGauge;
    Timer1: TTimer;
    Close_Button: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    fDeviceType: TDeviceType;
    pHartModule: PBasicHartModule;
    fTransferActive: boolean;

    procedure IndicateReceivedChar;

    procedure SetAllLabels;
    procedure ArrangeControls;

    procedure RefreshInternals_x532;
    procedure RefreshInternals_1003;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
  end;

var
  fmInspection: TfmInspection;

implementation

{$R *.dfm}

uses Types, ConversionUtils, ProcessUtils, HartDataTypes, HelpContextNumbers, ConversionProcs;

{ TfmInspection }

procedure TfmInspection.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(10);
     end
  else WaitMilliseconds(100);
end;

procedure TfmInspection.SetAllLabels;
begin
  Self.Caption:='Internal Variables Inspection'; // ' '+DeleteAcceleratorKey(LanguageStrings[mlInspection]);

  FlagsAndModesGroupBox.Caption:='Operation Flags';
  CalibrationParametersGroupBox.Caption:='Calibration Parameters';
  AdjustmentParametersGroupBox.Caption:='Adjustment Parameters';
  OtherParametersGroupBox.Caption:='Other Parameters';

  FirstFlagDisplay.LabelCaption:='Op. Syst. Flags';
  SecondFlagDisplay.LabelCaption:='Config. Flags';
  SixthFlagDisplay.LabelCaption:='Error Inform.:';

  if (fDeviceType=dtx532) then
     begin
     ThirdFlagDisplay.LabelCaption:='Ana. Out. Mode';
     FourthFlagDisplay.LabelCaption:='Pulse Out. Mode';
     FifthFlagDisplay.LabelCaption:='Alarm Out. Mode';

     Label1.Caption:='Thresh. Op.';
     Label2.Caption:='Deb. Inputs';
     EighthDisplay.DisplayFormat:=tfString;
     end;

  if (fDeviceType=dt1003) then
     begin
     ThirdFlagDisplay.LabelCaption:='Ana. M&&A Flags';
     FourthFlagDisplay.LabelCaption:='Dig. Mode Flags';
     FifthFlagDisplay.LabelCaption:='Dig. Action Flags';

     Label1.Caption:='Stack Reserve';
     Label2.Caption:='Port A Inp.';
     EighthDisplay.DisplayFormat:=tfDecimal;
     end;

end;

procedure TfmInspection.ArrangeControls;
begin
  FirstFlagDisplay.SetDisplay('00000000');
  SecondFlagDisplay.SetDisplay('00000000');
  ThirdFlagDisplay.SetDisplay('00000000');
  FourthFlagDisplay.SetDisplay('00000000');
  FifthFlagDisplay.SetDisplay('00000000');
  SixthFlagDisplay.SetDisplay('00000000');
  SeventhFlagDisplay.SetDisplay('00000000');

  LowCalibration.SetDisplay(0);
  HighCalibration.SetDisplay(0);
  LowAdjustment.SetDisplay(0);
  HighAdjustment.SetDisplay(0);

  NinthDisplay.SetDisplay('00000000');

  if (fDeviceType=dtx532) then
     begin
     EighthDisplay.SetDisplay('00000000');
     Label2.Left:=90;
     end;

  if (fDeviceType=dt1003) then
     begin
     EighthDisplay.SetDisplay(0);
     Label2.Left:=100;
     end;

  Close_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Left:=CommGauge.Left+(CommGauge.Width-Close_Button.Width) DIV 2;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmInspection.RefreshInternals_x532;
var lFlagInfos,
    lCalibInputInfos,
    lAdjustEighthDisplayInfos,
    lErrorOtherInfos: TByteDynArray;
    L: integer;
begin
  if (THartModule_Freq(pHartModule^).GetInternalValues(0,lFlagInfos)) then
     begin
     L:=System.Length(lFlagInfos);
     if (not(L<1)) then // OpSys
        FirstFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[0]));
     if (not(L<2)) then // Config
        SecondFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[1]));
     if (not(L<3)) then // Ana Out Mode
        ThirdFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[2]));
     if (not(L<4)) then // Pulse Out Mode
        FourthFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[3]));
     if (not(L<5)) then // Alarm Out Mode
        FifthFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[4]));
     // if (not(L<6)) then
     //    InputOutputDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[5]));
     end;

  if (THartModule_Freq(pHartModule^).GetInternalValues(1,lErrorOtherInfos)) then
     begin
     L:=System.Length(lErrorOtherInfos);
     if (not(L<1)) then // Device Errors (low)
        SixthFlagDisplay.SetDisplay(ByteToBinaryString(lErrorOtherInfos[0]));
     if (not(L<2)) then // Device Errors (high)
        SeventhFlagDisplay.SetDisplay(ByteToBinaryString(lErrorOtherInfos[1]));
     if (not(L<3)) then // Threshold Op. Flags
        EighthDisplay.SetDisplay(ByteToBinaryString(lErrorOtherInfos[2]));
     if (not(L<5)) then // Debounced Input Status
        NinthDisplay.SetDisplay(ByteToBinaryString(lErrorOtherInfos[4]));
     end;

  if (THartModule_Freq(pHartModule^).GetInternalValues(2,lCalibInputInfos)) then
     begin
     L:=System.Length(lCalibInputInfos);
     if (not(L<2)) then // Low Calibration
        LowCalibration.SetDisplay(lCalibInputInfos[0]+lCalibInputInfos[1]*256);
     if (not(L<4)) then // High Calibration
        HighCalibration.SetDisplay(lCalibInputInfos[2]+lCalibInputInfos[3]*256);
     end;
  if (THartModule_Freq(pHartModule^).GetInternalValues(3,lAdjustEighthDisplayInfos)) then
     begin
     L:=System.Length(lAdjustEighthDisplayInfos);
     if (not(L<2)) then // Low Adjustment
        LowAdjustment.SetDisplay(lAdjustEighthDisplayInfos[0]+lAdjustEighthDisplayInfos[1]*256);
     if (not(L<4)) then // High Adjustment
        HighAdjustment.SetDisplay(lAdjustEighthDisplayInfos[2]+lAdjustEighthDisplayInfos[3]*256);
     end;
end;

procedure TfmInspection.RefreshInternals_1003;
var lFlagInfos,
    lCalibInputInfos,
    lAdjustAndStackInfos,
    lErrorOtherInfos: TByteDynArray;
    L: integer;
begin
  if (THartModule_Freq(pHartModule^).GetInternalValues(0,lFlagInfos)) then
     begin
     L:=System.Length(lFlagInfos);
     if (not(L<1)) then // OpSys
        FirstFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[0]));
     if (not(L<5)) then // Config
        SecondFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[4]));
     if (not(L<2)) then // Ana M&A
        ThirdFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[1]));
     if (not(L<3)) then // Dig Mode
        FourthFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[2]));
     if (not(L<4)) then // Dig Action
        FifthFlagDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[3]));
     // if (not(L<6)) then
     //    ChecksumDisplay.SetDisplay(ByteToBinaryString(lFlagInfos[5]));
     end;
  if (THartModule_Freq(pHartModule^).GetInternalValues(1,lCalibInputInfos)) then
     begin
     L:=System.Length(lCalibInputInfos);
     if (not(L<2)) then // Low Calibration
        LowCalibration.SetDisplay(lCalibInputInfos[0]+lCalibInputInfos[1]*256);
     if (not(L<4)) then // High Calibration
        HighCalibration.SetDisplay(lCalibInputInfos[2]+lCalibInputInfos[3]*256);
     if (not(L<5)) then // Port A Inputs
        NinthDisplay.SetDisplay(ByteToBinaryString(lCalibInputInfos[4]));
     // if (not(L<6)) then
     //    PortAOutput.SetDisplay(ByteToBinaryString(lCalibInputInfos[5]));
     end;
  if (THartModule_Freq(pHartModule^).GetInternalValues(2,lAdjustAndStackInfos)) then
     begin
     L:=System.Length(lAdjustAndStackInfos);
     if (not(L<2)) then // Low Adjustment
        LowAdjustment.SetDisplay(lAdjustAndStackInfos[0]+lAdjustAndStackInfos[1]*256);
     if (not(L<4)) then // High Adjustment
        HighAdjustment.SetDisplay(lAdjustAndStackInfos[2]+lAdjustAndStackInfos[3]*256);
     if (not(L<5)) then // Stack Reserve
        EighthDisplay.SetDisplay(lAdjustAndStackInfos[4]);
     //
     end;
  if (THartModule_Freq(pHartModule^).GetInternalValues(3,lErrorOtherInfos)) then
     begin
     L:=System.Length(lErrorOtherInfos);
     if (not(L<1)) then // Device Errors (low)
        SixthFlagDisplay.SetDisplay(ByteToBinaryString(lErrorOtherInfos[0]));
     if (not(L<2)) then // Device Errors (high)
        SeventhFlagDisplay.SetDisplay(ByteToBinaryString(lErrorOtherInfos[1]));
     // if (not(L<3)) then
     //    TransFuncDisplay2.SetDisplay(ByteToBinaryString(lErrorOtherInfos[2]));
     end;
end;

procedure TfmInspection.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmInspection.FormCreate(Sender: TObject);
begin
  fDeviceType:=dtNone;
  pHartModule:=nil;

  HelpContext:=hcInspection;

  fTransferActive:=false;

  Timer1.Interval:=1000;
  Timer1.Enabled:=false;

  CommGauge.MaxValue:=56;
end;

procedure TfmInspection.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin
     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);

     if (fDeviceType in [dtx532,dt1003]) then
        begin
        SetAllLabels;
        ArrangeControls;
        end
     else Close;
     end
  else Close;
end;

procedure TfmInspection.FormActivate(Sender: TObject);
begin
  fTransferActive:=true;
  if (fDeviceType=dtx532) then
     RefreshInternals_x532;
  if (fDeviceType=dt1003) then
     RefreshInternals_1003;
  WaitMilliseconds(100);
  CommGauge.Progress:=0;
  fTransferActive:=false;

  Timer1.Enabled:=true;
end;

procedure TfmInspection.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not(fTransferActive);
end;

procedure TfmInspection.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmInspection.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled:=false;

  fTransferActive:=true;
  if (fDeviceType=dtx532) then
     RefreshInternals_x532;
  if (fDeviceType=dt1003) then
     RefreshInternals_1003;
  WaitMilliseconds(100);
  CommGauge.Progress:=0;
  fTransferActive:=false;

  Timer1.Enabled:=true;
end;

end.
