unit Language;

interface

uses Windows, Buttons, Classes, Controls, ExtCtrls, Forms, StdCtrls;

type
  TfmLanguage = class(TForm)
    SelectLanguage: TRadioGroup;
    OK_Button: TBitBtn;
    Close_Button: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SelectLanguageClick(Sender: TObject);
    procedure OK_ButtonClick(Sender: TObject);
  private
    { Private declarations }
    LanguageCount: word;
    LangSelection: LANGID;
    fLangIndex: integer;
  public
    { Public declarations }
    procedure SetLanguage(Value: LANGID);
    function GetLanguage: LANGID;
  end;

var
  fmLanguage: TfmLanguage;

implementation

{$R *.dfm}

uses StringUtils, DataInstances, DataConstants, LabelNumbers, HelpContextNumbers;

procedure TfmLanguage.SetLanguage(Value: LANGID);
begin
  LangSelection:=Value;
end;

function TfmLanguage.GetLanguage: LANGID;
begin
  Result:=LangSelection;
end;

procedure TfmLanguage.FormCreate(Sender: TObject);
begin
  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlLanguage]);
  SelectLanguage.Caption:='';

  HelpContext:=hcLanguage;

  LanguageCount:=0;
  LangSelection:=CDefaultLanguage;

  fLangIndex:=-1;
  OK_Button.Enabled:=false;

  OK_Button.Caption:=LanguageStrings[blSetting];
  Close_Button.Caption:=LanguageStrings[blClose];
end;

procedure TfmLanguage.FormShow(Sender: TObject);
var I: integer;
begin
  LanguageCount:=System.Length(LanguageInfos);
  SelectLanguage.Height:=(LanguageCount+1)*24;
  with SelectLanguage do
       begin
       Items.Clear;
       for I:=0 to LanguageCount-1 do
           Items.Add(LanguageInfos[I].LangName);
       for I:=0 to LanguageCount-1 do
           if LanguageInfos[I].LangNumber=LangSelection then
              begin
              ItemIndex:=I;
              fLangIndex:=ItemIndex;
              break;
              end;
       end;

  OK_Button.Top:=SelectLanguage.Top+SelectLanguage.Height+20;
  OK_Button.Left:=SelectLanguage.Left+(SelectLanguage.Width-OK_Button.Width) DIV 2;
  Close_Button.Top:=OK_Button.Top+OK_Button.Height+15;
  Close_Button.Left:=OK_Button.Left;

  Self.Height:=Close_Button.Top+80;

  FocusControl(SelectLanguage);
end;

procedure TfmLanguage.SelectLanguageClick(Sender: TObject);
begin
  OK_Button.Enabled:=(SelectLanguage.ItemIndex<>fLangIndex);
end;

procedure TfmLanguage.OK_ButtonClick(Sender: TObject);
begin
  LangSelection:=LanguageInfos[SelectLanguage.ItemIndex].LangNumber;
end;

end.
