unit MasterData_73;

interface

uses
  Windows, Messages, SysUtils, Variants, Types, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, Buttons, Gauges, GroupControls,
  EditControls, DataTypes, HartDataTypes, HartModules;

type
  TMasterDataRec = record
    InputType     : TInputType;
    TypeNumber    : word;
    ColdJunction  : TOnOffSet;
    WiringNumber  : word;
    Linearisation : TOnOffSet;
    PVUnits       : word;

    Point4mA  : real;
    Point20mA : real;
    FilterFrequencySetting : TFrequency;

    DampingTimeConstant    : real;
    SmoothingTimeConstant  : LongInt;

    NamurOutput            : TNamurSet;
    ResponseSpeed          : TSpeedSet;

    TransmitterFailureAlarmLevel : TAlarmSet;
    ColdJunctionFailureAlarmLevel: TAlarmSet;
    OpenCircuitAlarmLevel        : TAlarmSet;
    ShortCircuitAlarmLevel       : TAlarmSet;

    TripOutputSetting     : Tx075TripType;
    TripThreshold         : real;
    TripHysteresis        : real;
    end;

  TMasterLimitRec = record
    Limits4mA : TRealLimits;
    Limits20mA: TRealLimits;
    LimitsDampingTC       : TRealLimits;
    LimitsSmoothingTC     : TLongLimits;
    LimitsThreshold       : TRealLimits;
    LimitsHysteresis      : TRealLimits;
    end;

  TUnitEnableArray = array [0..4] of boolean;

  TfmModuleData_73 = class(TForm)
    PageControl1: TPageControl;

    // Input Tab Sheet
    InputTabSheet: TTabSheet;
    InputTypeSelector: TMyRadioGroup;
    InputNumberSelector: TMyRadioGroup;
    CJCSelector: TMyRadioGroup;
    RTDWiringSelector: TMyRadioGroup;
    LinearisationSelector: TMyRadioGroup;
    ProcessUnitSelector: TMyRadioGroup;
    SignalRangeBox: TGroupBox;
    Point4mA_Edit: TLabelRealEdit;
    Point20mA_Edit: TLabelRealEdit;
    FrequencySelector: TMyRadioGroup;

    // Output Tab Sheet
    OutputTabSheet: TTabSheet;
    OutputFilteringBox: TGroupBox;
    DampingConstant_Edit: TLabelRealEdit;
    SmoothingConstant_Edit: TLabelIntegerEdit;
    OutputModeBox: TGroupBox;
    NE43OnButton: TRadioButton;
    NE43OffButton: TRadioButton;
    ResponseTimeSelector: TMyRadioGroup;

    // Alarm Tab Sheet
    AlarmTabSheet: TTabSheet;

    AlarmLevelBox: TGroupBox;
    TX_Alarm_Selector: TMyRadioGroup;
    OC_Alarm_Selector: TMyRadioGroup;
    CJC_Alarm_Selector: TMyRadioGroup;
    SC_Alarm_Selector: TMyRadioGroup;
    EBDTripDisplayBevel: TGroupBox;
    TripOnButton: TRadioButton;
    EBDOnButton: TRadioButton;
    EBDOffButton: TRadioButton;
    Threshold_Edit: TLabelRealEdit;
    Hysteresis_Edit: TLabelRealEdit;

    // Other components
    CommGauge: TGauge;
    OK_Button: TBitBtn;
    Close_Button: TBitBtn;

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);

    procedure PageControl1DrawTab(Control: TCustomTabControl; TabIndex: Integer;
      const Rect: TRect; Active: Boolean);

    procedure InputTypeSelectorClick(Sender: TObject);
    procedure InputNumberSelectorClick(Sender: TObject);

    procedure CJCSelectorClick(Sender: TObject);

    procedure RTDWiringSelectorClick(Sender: TObject);

    procedure LinearisationSelectorClick(Sender: TObject);

    procedure ProcessUnitSelectorClick(Sender: TObject);

    procedure Point4mA_EditChange(Sender: TObject);
    procedure Point20mA_EditChange(Sender: TObject);

    procedure FrequencySelectorClick(Sender: TObject);

    procedure DampingConstant_EditChange(Sender: TObject);
    procedure SmoothingConstant_EditChange(Sender: TObject);

    procedure NE43OnButtonClick(Sender: TObject);
    procedure NE43OffButtonClick(Sender: TObject);

    procedure ResponseTimeSelectorClick(Sender: TObject);
    procedure TX_Alarm_SelectorClick(Sender: TObject);
    procedure OC_Alarm_SelectorClick(Sender: TObject);
    procedure CJC_Alarm_SelectorClick(Sender: TObject);
    procedure SC_Alarm_SelectorClick(Sender: TObject);
    procedure EBDOnButtonClick(Sender: TObject);
    procedure EBDOffButtonClick(Sender: TObject);
  private
    { Private declarations }
    fDialogMode: TDialogMode;
    fSaveMode: TSaveMode;
    pHartModule: PBasicHartModule;
    fDeviceType: TDeviceType;
    fMasterDataRec: TMasterDataRec;
    fMasterLimitRec: TMasterLimitRec;
    fProtectedPath,
    fUnusualLinConfirmed,
    fUnusualCJCConfirmed: boolean;

    procedure EncodeModuleData;
    procedure DecodeModuleData;

    procedure SetInputTypeSelectorLabels;
    procedure SetInputTypeSpecificInputNumberSelectorLabels;
    procedure SetInputTypeItem;
    procedure SetInputNumberSelectorItem;
    procedure SetCJCSelectorLabels;
    procedure SetDeviceSpecificRTDWiringSelectorLabels;
    procedure SetCJCorRTDWiringSelectorItems;
    procedure SetLinearisationSelectorLabels;
    procedure SetLinearisationSelectorItem;
    procedure SetInputTypeSpecificProcessUnitSelectorLabels;
    procedure SetInputTypeSpecificLastProcessUnitSelectorLabel;
    procedure SetInputTypeSpecificProcessUnitSelectorItem;
    procedure SetSignalRangeEditLabels;
    procedure SetSignalRangeEditValues;
    procedure SetSignalRangeEditUnits;
    procedure SetFilterFrequencyLabels;
    procedure SetFilterFrequencySelectorItems;
    procedure SetOutputFilteringEditLabels;
    procedure SetOutputFilteringEditValues;
    procedure SetOutputLimitationSelectorLabels;
    procedure SetOutputLimitationSelectorItems;
    procedure SetResponseTimeSelectorLabels;
    procedure SetResponseTimeSelectorItems;
    procedure SetAlarmLevelSelectorLabels;
    procedure SetTransmitterFailureAlarmLevelSelectorItems;
    procedure SetColdJunctionFailureAlarmLevelItems;
    procedure SetOpenCircuitAlarmLevelItems;
    procedure SetShortCircuitAlarmLevelItems;
    procedure SetAlarmLevelSelectorItems;
    procedure SetEBDTripSelectorLabels;
    procedure SetEBDTripSelectorItems;

    procedure SetAllLabels;
    procedure ArrangeVisibleControls;
    procedure ArrangeDisplayModeLook;
    procedure ArrangeConfigurationModeLook;
    procedure SetAllDisplayValues;
    procedure IndicateReceivedChar;
    procedure UpdateAcceptButton;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
    procedure SetMode(DMode: TDialogMode; SMode: TSaveMode);
  end;

var
  fmModuleData_73: TfmModuleData_73;

implementation

{$R *.dfm}

uses StringUtils, ProcessUtils, DataInstances, DataConstants, LabelNumbers, HelpContextNumbers,
     ConversionProcs, LanguageStringProcs;

const
                                   // C    F    R     K   mV/Ohm
  CAllUnitsOn:  TUnitEnableArray = (true, true, true, true, true);
  CNonLinUnits: TUnitEnableArray = (true, true, true, true, false); // non linear sensor, means that linearisation is on
  CLinUnits:    TUnitEnableArray = (false,false,false,false,true);  // linear sensor, means that linearisation is off

function GetDynArray(Source: TUnitEnableArray): TBooleanDynArray;
var NewBoolArray: TBooleanDynArray;
    I, L: integer;
begin
  L:=SizeOf(TUnitEnableArray);
  SetLength(NewBoolArray,L);
  for I:=0 to L-1 do
      NewBoolArray[I]:=Source[I];
  Result:=NewBoolArray;
end;

function GetInputLimits(DeviceType: TDeviceType; HART_TransFunction,HART_InputType,HART_PVUnits: byte): TRealLimits;
var InputLimits: TRealLimits;
    Linearisation: TOnOffSet;
    lType: word;
begin
  lType:=TypeNumberFromHART (DeviceType,HART_InputType);
  Linearisation:=LinearFromHART (HART_TransFunction);
  case InputTypeFromHART (DeviceType,HART_InputType) of
       itTHC: begin
              case lType of
                   0: InputLimits:=Lim_B;
                   1: InputLimits:=Lim_E;
                   2: InputLimits:=Lim_J;
                   3: InputLimits:=Lim_K;
                   4: InputLimits:=Lim_N;
                   5: if not(DeviceType in [dt4075,dtx575]) then InputLimits:=Lim_R
                      else InputLimits:=Lim_R75;
                   6: begin
                      if (DeviceType=dt4073XK) or (DeviceType=dt5074XK) then InputLimits:=Lim_XK
                      else if not(DeviceType in [dt4075,dtx575]) then InputLimits:=Lim_S
                           else InputLimits:=Lim_S75;
                      end;
                   7: InputLimits:=Lim_T;
                   8: if not(DeviceType in [dt4075,dtx575]) then InputLimits:=Lim_V
                      else InputLimits:=Lim_XK;
                   9: if (DeviceType in [dt4075,dtx575]) then InputLimits:=Lim_V;
                   end;
              end;
       itRTD: case DeviceType of
                   dt4075: case lType of
                                0: if Linearisation=tsOn then
                                      InputLimits:=Lim_Pt
                                   else InputLimits:=Lim_RLin;
                                1: InputLimits:=Lim_Cu50;
                                2: InputLimits:=Lim_Cu53;
                                3: InputLimits:=Lim_RLin;
                                end;
                   dtx575: case lType of
                                0: if Linearisation=tsOn then
                                      InputLimits:=Lim_Pt
                                   else InputLimits:=Lim_RLin;
                                1: InputLimits:=Lim_Pt;
                                2: InputLimits:=Lim_Pt1T;
                                3,4,5: InputLimits:=Lim_Ni;
                                6: InputLimits:=Lim_Cu50;
                                7: InputLimits:=Lim_Cu53;
                                8: InputLimits:=Lim_RLin;
                                end;
                   else begin
                        if lType=0 then InputLimits:=Lim_Pt
                        else InputLimits:=Lim_RLin;
                        end;
                   end;
       end;

  case PVUnitsFromHART(HART_PVUnits) of
       1: begin             // Fahrenheit
          InputLimits.LLim:=CelsiusToFahrenheit(InputLimits.LLim);
          InputLimits.ULim:=CelsiusToFahrenheit(InputLimits.ULim);
          end;
       2: begin             // Rankine
          InputLimits.LLim:=CelsiusToRankine(InputLimits.LLim);
          InputLimits.ULim:=CelsiusToRankine(InputLimits.ULim);
          end;
       3: begin             // Kelvin
          InputLimits.LLim:=CelsiusToKelvin(InputLimits.LLim);
          InputLimits.ULim:=CelsiusToKelvin(InputLimits.ULim);
          end;
       4: begin             // mV
          InputLimits.LLim:=InputLimits.LLim;               // ???
          InputLimits.ULim:=InputLimits.ULim;
          end;
       5: begin             // Ohm
          InputLimits.LLim:=InputLimits.LLim;               // ???
          InputLimits.ULim:=InputLimits.ULim;
          end;
       end;
  Result:=InputLimits;
end;

//****************************************************************************//

procedure TfmModuleData_73.EncodeModuleData;
var lPrimaryVariableOutputData: THPrimaryVariableOutput;
    lx073ParameterSet: THx073ParameterSet;
    lx075TripParameters: THx075TripParameters;
    lResponseSpeed,lNamurOutput,lShortAlarm: byte;
begin
  if (pHartModule<>nil) then
     begin
     lPrimaryVariableOutputData:=THartModule_x073(pHartModule^).PrimaryVariableOutputData;
     lx073ParameterSet:=THartModule_x073(pHartModule^).x073ParameterSet;

     with fMasterDataRec do
          begin
          InputType:=InputTypeFromHART(fDeviceType,lx073ParameterSet.InputType);
          TypeNumber:=TypeNumberFromHART(fDeviceType,lx073ParameterSet.InputType);
          ColdJunction:=CJCompFromHART(lx073ParameterSet.ColdJunctionCompensationOnOff);
          WiringNumber:=WiringNumberFromHART(fDeviceType,lx073ParameterSet.InputType);
          Linearisation:=LinearFromHART(lPrimaryVariableOutputData.TransferFunction);

          PVUnits:=PVUnitsFromHART(lPrimaryVariableOutputData.PVUnits);
          Point4mA:=lPrimaryVariableOutputData.Point4mA;
          Point20mA:=lPrimaryVariableOutputData.Point20mA;
          FilterFrequencySetting:=FrequencyFromHART(lx073ParameterSet.FilterFrequencySelector);
          DampingTimeConstant:=lPrimaryVariableOutputData.DampingTimeConstant;
          SmoothingTimeConstant:=lx073ParameterSet.SmoothingTimeConstant;

          TransmitterFailureAlarmLevel:=AlarmFromHART(lx073ParameterSet.TransmitterFailureAlarmLevel);
          ColdJunctionFailureAlarmLevel:=AlarmFromHART(lx073ParameterSet.ColdJunctionFailureAlarmLevel);
          OpenCircuitAlarmLevel:=AlarmFromHART(lx073ParameterSet.OpenCircuitAlarmLevel);

          if (fDeviceType in [dt4075,dtx575,dtx573]) then
             begin
             lx075TripParameters:=THartModule_x075(pHartModule^).x075TripParameters;

             TripOutputSetting:=x075TripTypeFromHART(lx075TripParameters.TripOutputFunction);
             TripThreshold:=lx075TripParameters.TripValue;
             TripHysteresis:=lx075TripParameters.Hysteresis;
             end;

          if (fDeviceType in [dtx575,dtx573]) then
             begin
             lResponseSpeed:=THartModule_x575(pHartModule^).ResponseTime;
             lNamurOutput:=THartModule_x575(pHartModule^).NE43Mode;

             ResponseSpeed:=x575SpeedFromHART(lResponseSpeed);
             NamurOutput:=x575NAMURFromHART(lNamurOutput);
             end;

          if (fDeviceType=dtx573) then
             begin
             lShortAlarm:=THartModule_x573(pHartModule^).ShortCircuitAlarmLevel;
             ShortCircuitAlarmLevel:=AlarmFromHART(lShortAlarm);
             end;
          end;

     with fMasterLimitRec do
          begin
          Limits4mA:=GetInputLimits(fDeviceType,lPrimaryVariableOutputData.TransferFunction,lx073ParameterSet.InputType,lPrimaryVariableOutputData.PVUnits);
          Limits20mA:=GetInputLimits(fDeviceType,lPrimaryVariableOutputData.TransferFunction,lx073ParameterSet.InputType,lPrimaryVariableOutputData.PVUnits);
          LimitsDampingTC:=LDamp;
          LimitsSmoothingTC:=LSmoo;
          LimitsThreshold:=LDamp;
          LimitsHysteresis:=LDamp;
          end;
     end;
end;

procedure TfmModuleData_73.DecodeModuleData;
var lPrimaryVariableOutputData: THPrimaryVariableOutput;
    lx073ParameterSet: THx073ParameterSet;
    lx075TripParameters: THx075TripParameters;
    lResponseSpeed,lNamurOutput,lShortAlarm: byte;
begin
  if (pHartModule<>nil) then
     begin
     lx073ParameterSet.InputType:=InputTypeAndNumberAndWiringToHART(fDeviceType,fMasterDataRec.InputType,fMasterDataRec.TypeNumber,fMasterDataRec.WiringNumber);
     lx073ParameterSet.ColdJunctionCompensationOnOff:=CJCompToHART(fMasterDataRec.ColdJunction);
     lx073ParameterSet.OpenCircuitAlarmLevel:=AlarmToHART(fMasterDataRec.OpenCircuitAlarmLevel);
     lx073ParameterSet.TransmitterFailureAlarmLevel:=AlarmToHART(fMasterDataRec.TransmitterFailureAlarmLevel);
     lx073ParameterSet.FilterFrequencySelector:=FrequencyToHART(fMasterDataRec.FilterFrequencySetting);
     lx073ParameterSet.SmoothingTimeConstant:=word(fMasterDataRec.SmoothingTimeConstant);
     lx073ParameterSet.ColdJunctionFailureAlarmLevel:=AlarmToHART(fMasterDataRec.ColdJunctionFailureAlarmLevel);

     lPrimaryVariableOutputData.PVAlarm:=0; // not used in MTL products, set to 0
     lPrimaryVariableOutputData.TransferFunction:=LinearToHART(fMasterDataRec.Linearisation);
     lPrimaryVariableOutputData.PVUnits:=PVUnitsToHART(fMasterDataRec.PVUnits);
     lPrimaryVariableOutputData.Point4mA:=fMasterDataRec.Point4mA;
     lPrimaryVariableOutputData.Point20mA:=fMasterDataRec.Point20mA;
     lPrimaryVariableOutputData.DampingTimeConstant:=fMasterDataRec.DampingTimeConstant;

     THartModule_x073(pHartModule^).PrimaryVariableOutputData:=lPrimaryVariableOutputData;
     THartModule_x073(pHartModule^).x073ParameterSet:=lx073ParameterSet;

     if (fDeviceType in [dt4075,dtx575,dtx573]) then
        begin
        lx075TripParameters.TripOutputFunction:=x075TripTypeToHART(fMasterDataRec.TripOutputSetting);
        lx075TripParameters.TripValue:=fMasterDataRec.TripThreshold;
        lx075TripParameters.Hysteresis:=fMasterDataRec.TripHysteresis;

        THartModule_x075(pHartModule^).x075TripParameters:=lx075TripParameters;
        end;

     if (fDeviceType in [dtx575,dtx573]) then
        begin
        lResponseSpeed:=x575SpeedToHART(fMasterDataRec.ResponseSpeed);
        lNamurOutput:=x575NAMURToHART(fMasterDataRec.NamurOutput);

        THartModule_x575(pHartModule^).ResponseTime:=lResponseSpeed;
        THartModule_x575(pHartModule^).NE43Mode:=lNamurOutput;
        end;

     if (fDeviceType=dtx573) then
        begin
        lShortAlarm:=AlarmToHART(fMasterDataRec.ShortCircuitAlarmLevel);

        THartModule_x573(pHartModule^).ShortCircuitAlarmLevel:=lShortAlarm;
        end;
     end;
end;

//****************************************************************************//

procedure TfmModuleData_73.SetInputTypeSelectorLabels;
begin
  InputTypeSelector.Caption:=LanguageStrings[flInputType];
  InputTypeSelector.Items.Clear;
  InputTypeSelector.Items.Add('&'+LanguageStrings[flTHCtype]);
  InputTypeSelector.Items.Add('&'+LanguageStrings[flRTDtype]);
end;

procedure TfmModuleData_73.SetInputTypeSpecificInputNumberSelectorLabels;
var I: integer;
begin
  InputNumberSelector.Items.Clear;
  if fMasterDataRec.InputType=itTHC then
     begin
     InputNumberSelector.Caption:=LanguageStrings[flTHCtype];

     InputNumberSelector.Height:=200;
     
     for I:=0 to 5 do InputNumberSelector.Items.Add(LanguageStrings[flTHC_B+I]);
     if (fDeviceType=dt5074XK) or (fDeviceType=dt4073XK) then
        InputNumberSelector.Items.Add(LanguageStrings[flTHC_XK])
     else InputNumberSelector.Items.Add(LanguageStrings[flTHC_B+6]);
     InputNumberSelector.Items.Add(LanguageStrings[flTHC_T]);
     if (fDeviceType in [dt4075,dtx575]) then InputNumberSelector.Items.Add(LanguageStrings[flTHC_XK]);

     InputNumberSelector.Items.Add(LanguageStrings[flmV]);
     end
  else // fMasterDataRec.InputType=itRTD
     begin
     InputNumberSelector.Caption:=LanguageStrings[flRTDtype];

     InputNumberSelector.Items.Add(LanguageStrings[flPt100]);
     if not(fDeviceType in [dt4075,dtx575,dtx573]) then begin
        InputNumberSelector.Height:=65;
        end
     else begin // fDeviceType in [dt4075,dtx575,dtx573]
        if (fDeviceType=dt4075) then begin
           InputNumberSelector.Height:=140;
           InputNumberSelector.Items.Add(LanguageStrings[flCu50]);
           InputNumberSelector.Items.Add(LanguageStrings[flCu53]);
           end
        else begin // fDeviceType=dtx575 or fDeviceType=dtx573
           InputNumberSelector.Height:=200;
           InputNumberSelector.Items.Add(LanguageStrings[flPt500]);
           InputNumberSelector.Items.Add(LanguageStrings[flPt1000]);
           InputNumberSelector.Items.Add(LanguageStrings[flNi100]);
           InputNumberSelector.Items.Add(LanguageStrings[flNi500]);
           InputNumberSelector.Items.Add(LanguageStrings[flNi1000]);
           InputNumberSelector.Items.Add(LanguageStrings[flCu50]);
           InputNumberSelector.Items.Add(LanguageStrings[flCu53]);
           end;
        end;
     InputNumberSelector.Items.Add(LanguageStrings[flResistor]);
     end;
end;

procedure TfmModuleData_73.SetInputTypeItem;
begin
  case fMasterDataRec.InputType  of
       itTHC: InputTypeSelector.ItemIndex:=0;
       itRTD: InputTypeSelector.ItemIndex:=1;
       else InputTypeSelector.ItemIndex:=-1;
       end;
  if fDialogMode=dmDisplay then InputTypeSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetInputNumberSelectorItem;
begin
  // detect resistor selection - Pt100 with linearisation off
  if ((fMasterDataRec.InputType=itRTD) and (fMasterDataRec.TypeNumber=0) and (fMasterDataRec.Linearisation=tsOff)) then
     InputNumberSelector.ItemIndex:=InputNumberSelector.Items.Count-1
  else // not resistor selection
     begin
     if fMasterDataRec.TypeNumber<InputNumberSelector.Items.Count then
        InputNumberSelector.ItemIndex:=fMasterDataRec.TypeNumber
     else InputNumberSelector.ItemIndex:=0;
     end;
  if fDialogMode=dmDisplay then InputNumberSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetCJCSelectorLabels;
begin
  CJCSelector.Caption:=LanguageStrings[flColdJunctionCompensation];

  CJCSelector.Items.Clear;
  CJCSelector.Items.Add(LanguageStrings[flOn]);
  CJCSelector.Items.Add(LanguageStrings[flOff]);
end;

procedure TfmModuleData_73.SetDeviceSpecificRTDWiringSelectorLabels;
begin
  RTDWiringSelector.Caption:=LanguageStrings[flWiring];

  // check how many items are required for the actual module type !!!
  RTDWiringSelector.Items.Clear;
  RTDWiringSelector.Items.Add(LanguageStrings[fl2Wire]);
  RTDWiringSelector.Items.Add(LanguageStrings[fl3Wire]);
  RTDWiringSelector.Items.Add(LanguageStrings[fl4Wire]);
end;

procedure TfmModuleData_73.SetCJCorRTDWiringSelectorItems;
begin
  if fMasterDataRec.InputType=itTHC then
     begin
     CJCSelector.Visible:=true;
     RTDWiringSelector.Visible:=false;
     CJCSelector.EnableAllButtons;
     if fMasterDataRec.ColdJunction in [tsOn..tsOff] then
        CJCSelector.ItemIndex:=integer(fMasterDataRec.ColdJunction)
     else CJCSelector.ItemIndex:=1;
     if fDialogMode=dmDisplay then CJCSelector.DisableNotCheckedButtons;
     end
  else
     begin
     CJCSelector.Visible:=false;
     RTDWiringSelector.Visible:=true;
     RTDWiringSelector.EnableAllButtons;
     if fMasterDataRec.WiringNumber in [0..3] then
        RTDWiringSelector.ItemIndex:=fMasterDataRec.WiringNumber
     else RTDWiringSelector.ItemIndex:=0;
     if fDialogMode=dmDisplay then RTDWiringSelector.DisableNotCheckedButtons;
     end;
end;

procedure TfmModuleData_73.SetLinearisationSelectorLabels;
begin
  LinearisationSelector.Caption:=LanguageStrings[flLinearisation];

  LinearisationSelector.Items.Clear;
  LinearisationSelector.Items.Add(LanguageStrings[flOn]);
  LinearisationSelector.Items.Add(LanguageStrings[flOff]);
end;

procedure TfmModuleData_73.SetLinearisationSelectorItem;
begin
  LinearisationSelector.EnableAllButtons;
  if fMasterDataRec.Linearisation in [tsOn..tsOff] then
     LinearisationSelector.ItemIndex:=integer(fMasterDataRec.Linearisation)
  else LinearisationSelector.ItemIndex:=1;
  if fDialogMode=dmDisplay then LinearisationSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetInputTypeSpecificProcessUnitSelectorLabels;
var I: integer;
begin
  ProcessUnitSelector.Caption:=LanguageStrings[flProcessUnits];

  ProcessUnitSelector.Items.Clear;
  for I:=0 to 3 do ProcessUnitSelector.Items.Add(LanguageStrings[flCelsius+I]);
  if fMasterDataRec.InputType=itTHC then
     ProcessUnitSelector.Items.Add(LanguageStrings[flCelsius+4])     // mV
  else ProcessUnitSelector.Items.Add(LanguageStrings[flCelsius+5]);  // Ohm
end;

procedure TfmModuleData_73.SetInputTypeSpecificLastProcessUnitSelectorLabel;
begin
  if (fMasterDataRec.InputType=itTHC) then
     ProcessUnitSelector.Items.Strings[4]:=LanguageStrings[flCelsius+4]    //mV
  else ProcessUnitSelector.Items.Strings[4]:=LanguageStrings[flCelsius+5]; // Ohm
end;

procedure TfmModuleData_73.SetInputTypeSpecificProcessUnitSelectorItem;
begin
  ProcessUnitSelector.EnableAllButtons;
  if (fMasterDataRec.PVUnits in [0..3]) or ((fMasterDataRec.PVUnits=4) and(fMasterDataRec.InputType=itTHC)) then
     ProcessUnitSelector.ItemIndex:=fMasterDataRec.PVUnits
  else
     if (fMasterDataRec.PVUnits=5) and (fMasterDataRec.InputType=itRTD) then
        ProcessUnitSelector.ItemIndex:=4
     else ProcessUnitSelector.ItemIndex:=0; // default units = C, however be careful here !!!
  if fDialogMode=dmDisplay then ProcessUnitSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetSignalRangeEditLabels;
begin
  SignalRangeBox.Caption:=LanguageStrings[flSignalRange];
  Point4mA_Edit.LabelCaption:=LanguageStrings[fl4mAPoint];
  Point20mA_Edit.LabelCaption:=LanguageStrings[fl20mAPoint];
end;

procedure  TfmModuleData_73.SetSignalRangeEditValues;
begin
  Point4mA_Edit.Value:=fMasterDataRec.Point4mA;
  Point20mA_Edit.Value:=fMasterDataRec.Point20mA;
end;

procedure TfmModuleData_73.SetSignalRangeEditUnits;
begin
  Point4mA_Edit.UnitText:=LanguageStrings[flCelsius+fMasterDataRec.PVUnits];
  Point20mA_Edit.UnitText:=LanguageStrings[flCelsius+fMasterDataRec.PVUnits];
end;

procedure TfmModuleData_73.SetFilterFrequencyLabels;
begin
  FrequencySelector.Caption:=LanguageStrings[flFilterFreq];

  FrequencySelector.Items.Clear;
  FrequencySelector.Items.Add(LanguageStrings[fl50Hz]);
  FrequencySelector.Items.Add(LanguageStrings[fl60Hz]);
end;

procedure TfmModuleData_73.SetFilterFrequencySelectorItems;
begin
  if fMasterDataRec.FilterFrequencySetting in [tf50..tf60] then
     FrequencySelector.ItemIndex:=integer(fMasterDataRec.FilterFrequencySetting)
  else FrequencySelector.ItemIndex:=0;
  if fDialogMode=dmDisplay then FrequencySelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetOutputFilteringEditLabels;
begin
  OutputFilteringBox.Caption:=LanguageStrings[flOutputFiltering];

  DampingConstant_Edit.LabelCaption:=LanguageStrings[flDampingConst];
  SmoothingConstant_Edit.LabelCaption:=LanguageStrings[flSmoothingConst];
end;

procedure TfmModuleData_73.SetOutputFilteringEditValues;
begin
  DampingConstant_Edit.Value:=fMasterDataRec.DampingTimeConstant;
  SmoothingConstant_Edit.Value:=fMasterDataRec.SmoothingTimeConstant;
end;

procedure TfmModuleData_73.SetOutputLimitationSelectorLabels;
begin
  OutputModeBox.Caption:=LanguageStrings[flOutputCharacteristic];

  NE43OnButton.Caption:=LanguageStrings[flOutNE43];
  NE43OffButton.Caption:=LanguageStrings[flOutMTL];
end;

procedure TfmModuleData_73.SetOutputLimitationSelectorItems;
begin
  case fMasterDataRec.NamurOutput of
       nsOff: begin
              NE43OffButton.Enabled:=true;
              NE43OffButton.Checked:=true;
              if fDialogMode=dmDisplay then NE43OnButton.Enabled:=false;
              end;
       nsOn:  begin
              NE43OnButton.Enabled:=true;
              NE43OnButton.Checked:=true;
              if fDialogMode=dmDisplay then NE43OffButton.Enabled:=false;
              end;
       nsNotAvailable: begin
                       end;
       end;
end;

procedure TfmModuleData_73.SetResponseTimeSelectorLabels;
begin
  ResponseTimeSelector.Caption:=LanguageStrings[flResponseTime];

  ResponseTimeSelector.Items.Clear;
  ResponseTimeSelector.Items.Add(LanguageStrings[flSlow]);
  ResponseTimeSelector.Items.Add(LanguageStrings[flMedium]);
  ResponseTimeSelector.Items.Add(LanguageStrings[flFast]);
end;

procedure TfmModuleData_73.SetResponseTimeSelectorItems;
begin
  if fMasterDataRec.ResponseSpeed in [tsMiddle..tsFast] then
     ResponseTimeSelector.ItemIndex:=integer(fMasterDataRec.ResponseSpeed)
  else ResponseTimeSelector.ItemIndex:=0;
  if fDialogMode=dmDisplay then ResponseTimeSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetAlarmLevelSelectorLabels;
begin
  AlarmLevelBox.Caption:=LanguageStrings[flAlarmLevels];

  TX_Alarm_Selector.Caption:=LanguageStrings[flAlarmTX];
  TX_Alarm_Selector.Items.Clear;
  TX_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  TX_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  TX_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);

  CJC_Alarm_Selector.Caption:=LanguageStrings[flAlarmCJC];
  CJC_Alarm_Selector.Items.Clear;
  CJC_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  CJC_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  CJC_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);

  OC_Alarm_Selector.Caption:=LanguageStrings[flAlarmOC];
  OC_Alarm_Selector.Items.Clear;
  OC_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  OC_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  OC_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);

  SC_Alarm_Selector.Caption:=LanguageStrings[flAlarmSC];
  SC_Alarm_Selector.Items.Clear;
  SC_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  SC_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  SC_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);
end;

procedure TfmModuleData_73.SetTransmitterFailureAlarmLevelSelectorItems;
begin
  if fMasterDataRec.TransmitterFailureAlarmLevel in [taLow..taHigh] then
     TX_Alarm_Selector.ItemIndex:=integer(fMasterDataRec.TransmitterFailureAlarmLevel)
  else TX_Alarm_Selector.ItemIndex:=2;
  if fDialogMode=dmDisplay then TX_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetColdJunctionFailureAlarmLevelItems;
begin
  CJC_Alarm_Selector.EnableAllButtons;
  if fMasterDataRec.ColdJunctionFailureAlarmLevel in [taLow..taHigh] then
     CJC_Alarm_Selector.ItemIndex:=integer(fMasterDataRec.ColdJunctionFailureAlarmLevel)
  else CJC_Alarm_Selector.ItemIndex:=2;
  if fDialogMode=dmDisplay then CJC_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetOpenCircuitAlarmLevelItems;
begin
  if fMasterDataRec.OpenCircuitAlarmLevel in [taLow..taHigh] then
     OC_Alarm_Selector.ItemIndex:=integer(fMasterDataRec.OpenCircuitAlarmLevel)
  else OC_Alarm_Selector.ItemIndex:=2;
  if fDialogMode=dmDisplay then OC_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetShortCircuitAlarmLevelItems;
begin
  if fMasterDataRec.ShortCircuitAlarmLevel in [taLow..taHigh] then
     SC_Alarm_Selector.ItemIndex:=integer(fMasterDataRec.ShortCircuitAlarmLevel)
  else SC_Alarm_Selector.ItemIndex:=2;
  if fDialogMode=dmDisplay then SC_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetAlarmLevelSelectorItems;
begin
  SetTransmitterFailureAlarmLevelSelectorItems;

  SetColdJunctionFailureAlarmLevelItems;

  SetOpenCircuitAlarmLevelItems;

  SetShortCircuitAlarmLevelItems;
end;

procedure TfmModuleData_73.SetEBDTripSelectorLabels;
begin
  EBDTripDisplayBevel.Caption:=LanguageStrings[flEBDdetection];

  EBDOnButton.Caption:=LanguageStrings[flEBD];
  EBDOffButton.Caption:=LanguageStrings[flOff];
end;

procedure TfmModuleData_73.SetEBDTripSelectorItems;
begin
  case fMasterDataRec.TripOutputSetting of
       ttOff: begin
              EBDOffButton.Enabled:=true;
              EBDOffButton.Checked:=true;
              if fDialogMode=dmDisplay then EBDOnButton.Enabled:=false;
              end;
       ttEBD: begin
              EBDOnButton.Enabled:=true;
              EBDOnButton.Checked:=true;
              if fDialogMode=dmDisplay then EBDOffButton.Enabled:=false;
              end;
       ttTrip: ;
       end;
end;

//****************************************************************************//

procedure TfmModuleData_73.SetAllLabels;
begin
  // Self.Caption will be set later according to the fDialogMode

  InputTabSheet.Caption:=LanguageStrings[flInputTab];

  SetInputTypeSelectorLabels;
  // InputNumber are module specific
  SetCJCSelectorLabels;
  // RTD wiring labels may be module specific

  SetLinearisationSelectorLabels;
  // Process Units are Input Type specific

  SetSignalRangeEditLabels;

  SetFilterFrequencyLabels;

  OutputTabSheet.Caption:=LanguageStrings[flOutputTab];

  SetOutputFilteringEditLabels;
  SetOutputLimitationSelectorLabels;
  SetResponseTimeSelectorLabels;

  AlarmTabSheet.Caption:=LanguageStrings[flAlarmsTab];

  SetAlarmLevelSelectorLabels;
  // Note: the EBD Trip lables may be module specific !!!
  SetEBDTripSelectorLabels;

  OK_Button.Caption:=LanguageStrings[blSetting];
  Close_Button.Caption:=LanguageStrings[blClose];
end;

procedure TfmModuleData_73.ArrangeVisibleControls;
var I: integer;
begin
  if (fDeviceType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx575]) then
     SC_Alarm_Selector.Visible:=false;

  if (fDeviceType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075]) then
     begin
     OutputModeBox.Visible:=false;
     ResponseTimeSelector.Visible:=false;
     end;

  if (fDeviceType in [dt507x,dtx073,dt5074XK,dt4073XK]) then
     EBDTripDisplayBevel.Visible:=false;

  if fMasterDataRec.InputType=itTHC then
     begin
     CJCSelector.Visible:=true;
     RTDWiringSelector.Visible:=false;
     end
  else // fMasterDataRec.InputType=itRTD
     begin
     CJCSelector.Visible:=false;
     RTDWiringSelector.Visible:=true;
     end;
end;

procedure TfmModuleData_73.ArrangeDisplayModeLook;
var I: integer;
begin
  Caption:=' '+LanguageStrings[flDisplayModule]; // DeleteAllChars(LanguageStrings[flDisplayInput],'&');
  HelpContext:=hcDisplayInput73;

  // Input Configuration Tab Sheet

  Point4mA_Edit.DisableEdition:=true;
  Point20mA_Edit.DisableEdition:=true;

  DampingConstant_Edit.DisableEdition:=true;
  SmoothingConstant_Edit.DisableEdition:=true;

  {NE43OnButton.Enabled:=false;
  NE43OffButton.Enabled:=false;

  EBDOnButton.Enabled:=false;
  EBDOffButton.Enabled:=false;}

  CommGauge.Visible:=false;
  OK_Button.Visible:=false;

  Close_Button.Left:=(Self.Width-Close_Button.Width) DIV 2;
  Close_Button.Top:=PageControl1.Top+PageControl1.Height+18;

  Self.Height:=Close_Button.Top+88;
end;

procedure TfmModuleData_73.ArrangeConfigurationModeLook;
begin
  Caption:=' '+LanguageStrings[flConfigModule]; // DeleteAllChars(LanguageStrings[flConfigInput],'&');
  HelpContext:=hcConfigInput73;

  CommGauge.Visible:=true;

  // fConfigResult:=mrIgnore;
end;

procedure TfmModuleData_73.SetAllDisplayValues;
begin
  // Input Configuration Tab Sheet

  fProtectedPath:=true;

  SetInputTypeItem;

  SetInputNumberSelectorItem;

  SetCJCorRTDWiringSelectorItems;
  if (fMasterDataRec.InputType=itTHC) then
     CJCSelector.DisableNotCheckedButtons;

  SetLinearisationSelectorItem;
  LinearisationSelector.DisableNotCheckedButtons;

  SetInputTypeSpecificProcessUnitSelectorItem;
  if fDialogMode=dmConfig then
     begin
     if (fMasterDataRec.PVUnits<4) and (fMasterDataRec.Linearisation=tsOn) then
        ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits))
     else if not(fMasterDataRec.PVUnits<4) and (fMasterDataRec.Linearisation=tsOff) then
             ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CLinUnits));
     end;

  SetSignalRangeEditValues;
  SetSignalRangeEditUnits;

  SetFilterFrequencySelectorItems;

  SetOutputFilteringEditValues;

  SetOutputLimitationSelectorItems;

  SetResponseTimeSelectorItems;

  SetAlarmLevelSelectorItems;
  if (fDialogMode=dmConfig) and
     (fMasterDataRec.InputType=itTHC) then
     SC_Alarm_Selector.DisableNotCheckedButtons
  else CJC_Alarm_Selector.DisableNotCheckedButtons;

  SetEBDTripSelectorItems;
  if (fDialogMode=dmConfig) and
     (fMasterDataRec.InputType=itRTD) then
     EBDOnButton.Enabled:=false;

  fProtectedPath:=false;

  // Output Configuration Tab Sheet
end;

procedure TfmModuleData_73.IndicateReceivedChar;
begin
  CommGauge.AddProgress(1);
  if CommGauge.Progress >= CommGauge.MaxValue then
     begin
     WaitMilliseconds(100);
     CommGauge.Progress:=0;
     end;
end;

procedure TfmModuleData_73.UpdateAcceptButton;
begin
  if pHartModule<>nil then
     OK_Button.Enabled:=not(pHartModule^.VerifyDeviceData);
end;

//****************************************************************************//

procedure TfmModuleData_73.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmModuleData_73.SetMode(DMode: TDialogMode; SMode: TSaveMode);
begin
  fDialogMode:=DMode; fSaveMode:=SMode;
end;

//****************************************************************************//

procedure TfmModuleData_73.FormCreate(Sender: TObject);
begin
  fDialogMode:=dmDisplay;
  fSaveMode:=smToDevice;

  fDeviceType:=dtNone;
  pHartModule:=nil;

  HelpContext:=hcMonitor;

  with fMasterDataRec do
       begin
       InputType:=itNone;
       TypeNumber:=0;
       WiringNumber:=0;
       PVUnits:=0;
       Linearisation:=tsOff;
       ColdJunction:=tsOff;
       end;

  fUnusualLinConfirmed:=false;
  fUnusualCJCConfirmed:=false;

  SetAllLabels;
end;

procedure TfmModuleData_73.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin

     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);
     if (fDeviceType<>dtNone) then
        begin
        EncodeModuleData;

        SetInputTypeSpecificInputNumberSelectorLabels;
        SetDeviceSpecificRTDWiringSelectorLabels;
        SetInputTypeSpecificProcessUnitSelectorLabels;

        ArrangeVisibleControls;

        if fDialogMode=dmDisplay then ArrangeDisplayModeLook;
        if fDialogMode=dmConfig then ArrangeConfigurationModeLook;

        SetAllDisplayValues;

        if fDialogMode=dmConfig then OK_Button.Enabled:=false;
        end
     else
        begin
        MessageDlg('Unexpected error',mtError,[mbCancel],0);
        Close;
        end;
     end
  else Close;
end;

procedure TfmModuleData_73.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var fExpectedTransferDataLength: integer;
    lUploadRequired: boolean;
begin
  CanClose:=(ModalResult<>mrOK);
  lUploadRequired:=false;
  if (pHartModule<>nil) and (fDialogMode=dmConfig) then
     begin
     OK_Button.Enabled:=false;
     Close_Button.Enabled:=false;
     if ((ModalResult<>mrOK) and not(pHartModule^.VerifyDeviceData)) then
        begin
        case (MessageDlg(LoadDllString(mlChangedNotSaved),mtWarning,[mbYes,mbNo,mbCancel],hcNoHelpTopic)) of
             mrYes: lUploadRequired:=true;
             mrNo:  ModalResult:=mrIgnore;
             else CanClose:=false;
             end;
        end;

     if ((ModalResult=mrOK) or lUploadRequired) then
        begin
        pHartModule^.EvaluateRequiredDeviceDataStatusFlags;
        fExpectedTransferDataLength:=pHartModule^.GetModuleTransferDataLength(tdUpload);
        if (fExpectedTransferDataLength>0) then
           begin
           if (fSaveMode=smToDevice) then
              begin
              CommGauge.MaxValue:=pHartModule^.GetModuleTransferDataLength(tdUpload);
              CommGauge.Progress:=0;

              Cursor:=crHourGlass;
              Application.ProcessMessages;
              pHartModule^.UploadRequiredDeviceData;

              if not(pHartModule^.VerifyDeviceData) then
                 begin
                 MessageDlg(LoadDllString(msDifferentResponse),mtError,[mbOk],hcNoHelpTopic);
                 MessageDlg(LoadDllString(msTransferError),mtError,[mbOk],hcNoHelpTopic);
                 ModalResult:=mrCancel;
                 end;

              end
           else  // (fSaveMode=smToMemory)
              MessageDlg(LoadDllString(msDataPreSaved),mtConfirmation,[mbOk],hcNoHelpTopic);
           end;
        end;

     if not(CanClose) then
        begin
        UpdateAcceptButton;
        Close_Button.Enabled:=true;
        end;
     Cursor:=OriginalCursor;
     end;
end;

procedure TfmModuleData_73.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

//****************************************************************************//

procedure TfmModuleData_73.PageControl1DrawTab(Control: TCustomTabControl;
  TabIndex: Integer; const Rect: TRect; Active: Boolean);
begin
  Control.Canvas.Brush.Color:=clBlue;
  Control.Canvas.TextOut(Rect.left+5,Rect.top+3,PageControl1.Pages[tabindex].Caption);
  PageControl1.Pages[TabIndex].Brush.Color:=Control.Canvas.Brush.Color;
end;

procedure TfmModuleData_73.InputTypeSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     case InputTypeSelector.ItemIndex of
          0: fMasterDataRec.InputType:=itTHC;
          1: fMasterDataRec.InputType:=itRTD;
          else fMasterDataRec.InputType:=itNone;
          end;

     fMasterDataRec.TypeNumber:=0;         // Type B or Pt100
     fMasterDataRec.Linearisation:=tsOn;   // linearisation on
     fMasterDataRec.PVUnits:=0;            // PVUnits C
     fMasterDataRec.Point4mA:=0.0;
     fMasterDataRec.Point20mA:=100.0;

     if fMasterDataRec.InputType=itTHC then
        begin
        fMasterDataRec.ColdJunction:=tsOn;
        // RTD wiring number is not relevant
        // ColdJunctionFailureAlarmLevel remain not affected
        fMasterDataRec.ShortCircuitAlarmLevel:=taOff;
        end
     else // fMasterDataRec.InputType=itRTD
        begin
        // Cold Junction Selection is not relevant
        fMasterDataRec.WiringNumber:=0;    // 2-wire
        fMasterDataRec.ColdJunctionFailureAlarmLevel:=taOff;
        // ShortCircuitAlarmLevel remain unchanged
        end;

     // TransmitterFailureAlarmLevel and OpenCircuitAlarmLevel remain not affected

     fProtectedPath:=true;

     SetInputTypeSpecificInputNumberSelectorLabels;
     SetInputNumberSelectorItem;

     SetCJCorRTDWiringSelectorItems;
     CJCSelector.DisableNotCheckedButtons;

     SetLinearisationSelectorItem;
     LinearisationSelector.DisableNotCheckedButtons;
     
     SetInputTypeSpecificLastProcessUnitSelectorLabel;
     SetInputTypeSpecificProcessUnitSelectorItem;
     ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits));

     SetSignalRangeEditValues;
     SetSignalRangeEditUnits;

     if fMasterDataRec.InputType=itTHC then
        begin
        CJC_Alarm_Selector.EnableAllButtons;
        EBDOnButton.Enabled:=true;
        end
     else // fMasterDataRec.InputType=itRTD
        begin
        SetColdJunctionFailureAlarmLevelItems;
        CJC_Alarm_Selector.DisableNotCheckedButtons;
        end;

     fProtectedPath:=false;
     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.InputNumberSelectorClick(Sender: TObject);
var lTypeNumber, lWiringNumber, lLastTypeNumber: word;
    fFirstTHC, fFirstmV: boolean;
    fFirstRTD, fFirstRes: boolean;
begin
  if not(fProtectedPath) then
     begin
     lTypeNumber:=InputNumberSelector.ItemIndex;
     // note: this is always the "mV" or "resistor" selection
     lLastTypeNumber:=InputNumberSelector.Items.Count-1;

     if fMasterDataRec.InputType=itTHC then
        begin
        if (lTypeNumber<>lLastTypeNumber) then
           begin
           // This is the first input type change from itmV to itTHC
           if (fMasterDataRec.TypeNumber=lLastTypeNumber) then
              begin
              fProtectedPath:=true;

              fMasterDataRec.ColdJunction:=tsOn;
              SetCJCorRTDWiringSelectorItems;
              CJCSelector.DisableNotCheckedButtons;

              fMasterDataRec.Linearisation:=tsOn;
              SetLinearisationSelectorItem;
              LinearisationSelector.DisableNotCheckedButtons;

              fMasterDataRec.PVUnits:=0; // set default selection to "C"
              SetInputTypeSpecificProcessUnitSelectorItem;
              ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits));
              SetSignalRangeEditUnits;

              CJC_Alarm_Selector.EnableAllButtons;

              EBDOnButton.Enabled:=true;

              fProtectedPath:=false;
              end;
           end
        else // the new selection is "mV"
           begin
           fProtectedPath:=true;

           fMasterDataRec.ColdJunction:=tsOff;
           SetCJCorRTDWiringSelectorItems;
           CJCSelector.DisableNotCheckedButtons;

           fMasterDataRec.Linearisation:=tsOff;
           SetLinearisationSelectorItem;
           LinearisationSelector.DisableNotCheckedButtons;

           fMasterDataRec.PVUnits:=4; // set default selection to "mV"
           SetInputTypeSpecificProcessUnitSelectorItem;
           ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CLinUnits));
           SetSignalRangeEditUnits;

           fMasterDataRec.ColdJunctionFailureAlarmLevel:=taOff;
           SetColdJunctionFailureAlarmLevelItems;
           CJC_Alarm_Selector.DisableNotCheckedButtons;

           fMasterDataRec.TripOutputSetting:=ttOff;
           SetEBDTripSelectorItems;
           EBDOnButton.Enabled:=false;

           fProtectedPath:=false;
           end;
        end
     else // fMasterDataRec.InputType=itRTD
        begin
        if (lTypeNumber<>lLastTypeNumber) then
           begin
           // This is the first input type change from "resistor" to "true RTD"
           // if (fMasterDataRec.TypeNumber=lLastTypeNumber) then
           if (fMasterDataRec.Linearisation=tsOff) then
              begin
              fProtectedPath:=true;

              // RTD wiring remain unchanged

              fMasterDataRec.Linearisation:=tsOn;
              SetLinearisationSelectorItem;
              LinearisationSelector.DisableNotCheckedButtons;

              fMasterDataRec.PVUnits:=0; // set default selection to "C"
              SetInputTypeSpecificProcessUnitSelectorItem;
              ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits));
              SetSignalRangeEditUnits;

              fProtectedPath:=false;
              end;
           end
        else // the new selection is "resistor"
           begin
           fProtectedPath:=true;

           // RTD wiring remain unchanged

           fMasterDataRec.Linearisation:=tsOff;
           SetLinearisationSelectorItem;
           LinearisationSelector.DisableNotCheckedButtons;

           fMasterDataRec.PVUnits:=5; // set default selection to "Ohm"
           SetInputTypeSpecificProcessUnitSelectorItem;
           ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CLinUnits));
           SetSignalRangeEditUnits;
           
           fProtectedPath:=false;
           end;
        end;

     fMasterDataRec.TypeNumber:=lTypeNumber;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.CJCSelectorClick(Sender: TObject);
var lColdJunction: TOnOffSet;
begin
  if not(fProtectedPath) then
     begin
     lColdJunction:=TOnOffSet(CJCSelector.ItemIndex);

     // Evaluate Unusual Cold Junction Compensation Selection and show adequate warning
     // However when once ignored, don't show this warning again until dialog is closed
     // This path can be achieved only when "InputType=itTHC" and not "mV" is selected
     if (lColdJunction=tsOff) and
        not(fUnusualCJCConfirmed) then
        fUnusualCJCConfirmed:=(MessageDlg(LoadDllString(msWrongLinSet),mtWarning,[mbIgnore,mbCancel],0)=mrIgnore);

     if not(fUnusualCJCConfirmed) then
        begin
        fProtectedPath:=true;
        CJCSelector.ItemIndex:=integer(fMasterDataRec.ColdJunction);
        fProtectedPath:=false;
        end
     else
        begin
        fMasterDataRec.ColdJunction:=lColdJunction;

        fProtectedPath:=true;
        if (fMasterDataRec.ColdJunction=tsOff) then
           begin
           fMasterDataRec.ColdJunctionFailureAlarmLevel:=taOff;
           SetColdJunctionFailureAlarmLevelItems;
           CJC_Alarm_Selector.DisableNotCheckedButtons;
           fMasterDataRec.TripOutputSetting:=ttOff;
           SetEBDTripSelectorItems;
           EBDOnButton.Enabled:=false;
           end
        else
           begin
           CJC_Alarm_Selector.EnableAllButtons;
           EBDOnButton.Enabled:=true;
           end;

        fProtectedPath:=false;

        DecodeModuleData;
        UpdateAcceptButton;
        end;
     end;
end;

procedure TfmModuleData_73.RTDWiringSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.WiringNumber:=RTDWiringSelector.ItemIndex;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.LinearisationSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.Linearisation:=TOnOffSet(LinearisationSelector.ItemIndex);

     // Evaluate Unusual Linearisation Selection and show adequate warning
     if (fMasterDataRec.Linearisation=tsOff) and
        not(fUnusualLinConfirmed) and
        (((fMasterDataRec.InputType=itTHC) and (fMasterDataRec.TypeNumber<(InputNumberSelector.Items.Count-1))) or
         ((fMasterDataRec.InputType=itRTD) and (fMasterDataRec.TypeNumber<(InputNumberSelector.Items.Count-1)))) then
        fUnusualLinConfirmed:=(MessageDlg(LoadDllString(msWrongLinSet),mtWarning,[mbIgnore,mbCancel],0)=mrIgnore);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.ProcessUnitSelectorClick(Sender: TObject);
var lPVUnits: integer;
begin
  if not(fProtectedPath) then
     begin
     lPVUnits:=ProcessUnitSelector.ItemIndex;
     if not((fMasterDataRec.InputType=itRTD) and (lPVUnits=4)) then // not Ohm selection
        fMasterDataRec.PVUnits:=lPVUnits
     else fMasterDataRec.PVUnits:=5;

     SetSignalRangeEditUnits;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Point4mA_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.Point4mA:=Point4mA_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Point20mA_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.Point20mA:=Point20mA_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.FrequencySelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.FilterFrequencySetting:=TFrequency(FrequencySelector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.DampingConstant_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.DampingTimeConstant:=DampingConstant_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.SmoothingConstant_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.SmoothingTimeConstant:=SmoothingConstant_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.NE43OnButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.NamurOutput:=nsOn;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.NE43OffButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.NamurOutput:=nsOff;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.ResponseTimeSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.ResponseSpeed:=TSpeedSet(ResponseTimeSelector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.TX_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.TransmitterFailureAlarmLevel:=TAlarmSet(TX_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.OC_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.OpenCircuitAlarmLevel:=TAlarmSet(OC_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.CJC_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.ColdJunctionFailureAlarmLevel:=TAlarmSet(CJC_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.SC_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.ShortCircuitAlarmLevel:=TAlarmSet(SC_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.EBDOnButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.TripOutputSetting:=ttEBD;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.EBDOffButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fMasterDataRec.TripOutputSetting:=ttOff;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

end.
