unit ModuleData_73;

interface

uses Windows, ExtCtrls, StdCtrls, Controls, ComCtrls, Buttons, Classes, Gauges, Forms,
     Graphics, Dialogs, EditControls, GroupControls, DataTypes, HartModules;

type
  TModule1ChDataRec = record
    InputType     : TTempInputType;
    TypeNumber    : word;
    ColdJunction  : TOnOffSet;
    WiringNumber  : word;
    Linearisation : TOnOffSet;
    PVUnits       : word;

    Point4mA  : real;
    Point20mA : real;
    FilterFrequencySetting : TFilterFrequency;

    DampingTimeConstant    : real;
    SmoothingTimeConstant  : LongInt;

    NamurOption            : TNamurOptionSet;
    ResponseSpeed          : TResponseSpeed;

    TransmitterFailureAlarmLevel : TAlarmLevelSet;
    ColdJunctionFailureAlarmLevel: TAlarmLevelSet;
    OpenCircuitAlarmLevel        : TAlarmLevelSet;
    ShortCircuitAlarmLevel       : TAlarmLevelSet;

    TripOutputSetting     : Tx075TripType;
    TripThreshold         : real;
    TripHysteresis        : real;
    end;

  TfmModuleData_73 = class(TForm)
    PageControl1: TPageControl;

    // Input Tab Sheet
    InputTabSheet: TTabSheet;
    InputTypeSelector: TMyRadioGroup;
    InputNumberSelector: TMyRadioGroup;
    CJCSelector: TMyRadioGroup;
    RTDWiringSelector: TMyRadioGroup;
    LinearisationSelector: TMyRadioGroup;
    ProcessUnitSelector: TMyRadioGroup;
    SignalRangeBox: TGroupBox;
    Point4mA_Edit: TLabelRealEdit;
    Point20mA_Edit: TLabelRealEdit;
    FrequencySelector: TMyRadioGroup;

    // Output Tab Sheet
    OutputTabSheet: TTabSheet;
    OutputFilteringBox: TGroupBox;
    DampingConstant_Edit: TLabelRealEdit;
    SmoothingConstant_Edit: TLabelIntegerEdit;
    OutputModeBox: TGroupBox;
    NE43OnButton: TRadioButton;
    NE43OffButton: TRadioButton;
    ResponseTimeSelector: TMyRadioGroup;

    // Alarm Tab Sheet
    AlarmTabSheet: TTabSheet;

    AlarmLevelBox: TGroupBox;
    TX_Alarm_Selector: TMyRadioGroup;
    OC_Alarm_Selector: TMyRadioGroup;
    CJC_Alarm_Selector: TMyRadioGroup;
    SC_Alarm_Selector: TMyRadioGroup;
    TripEBDSelectorBox: TGroupBox;
    EBDOffButton: TRadioButton;
    EBDOnButton: TRadioButton;
    TripOnButton: TRadioButton;
    Threshold_Edit: TLabelRealEdit;
    Hysteresis_Edit: TLabelRealEdit;

    // Other components
    CommGauge: TGauge;
    OK_Button: TBitBtn;
    Close_Button: TBitBtn;
    TripParameterPanel: TPanel;

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
    procedure PageControl1DrawTab(Control: TCustomTabControl; TabIndex: Integer;
      const Rect: TRect; Active: Boolean);

    procedure InputTypeSelectorClick(Sender: TObject);
    procedure InputNumberSelectorClick(Sender: TObject);

    procedure CJCSelectorClick(Sender: TObject);

    procedure RTDWiringSelectorClick(Sender: TObject);

    procedure LinearisationSelectorClick(Sender: TObject);

    procedure ProcessUnitSelectorClick(Sender: TObject);

    procedure Point4mA_EditChange(Sender: TObject);
    procedure Point4mA_EditRangeValidationError(Sender: TObject);
    procedure Point20mA_EditChange(Sender: TObject);
    procedure Point20mA_EditRangeValidationError(Sender: TObject);

    procedure FrequencySelectorClick(Sender: TObject);

    procedure DampingConstant_EditChange(Sender: TObject);
    procedure DampingConstant_EditRangeValidationError(Sender: TObject);
    procedure SmoothingConstant_EditChange(Sender: TObject);
    procedure SmoothingConstant_EditRangeValidationError(Sender: TObject);

    procedure NE43OnButtonClick(Sender: TObject);
    procedure NE43OffButtonClick(Sender: TObject);
    procedure ResponseTimeSelectorClick(Sender: TObject);

    procedure TX_Alarm_SelectorClick(Sender: TObject);
    procedure OC_Alarm_SelectorClick(Sender: TObject);
    procedure CJC_Alarm_SelectorClick(Sender: TObject);
    procedure SC_Alarm_SelectorClick(Sender: TObject);

    procedure TripOnButtonClick(Sender: TObject);
    procedure EBDOnButtonClick(Sender: TObject);
    procedure EBDOffButtonClick(Sender: TObject);
    procedure Threshold_EditChange(Sender: TObject);
    procedure Threshold_EditRangeValidationError(Sender: TObject);
    procedure Hysteresis_EditChange(Sender: TObject);
    procedure Hysteresis_EditRangeValidationError(Sender: TObject);
  private
    { Private declarations }
    fDialogMode: TDialogMode;
    fSaveMode: TSaveMode;
    pHartModule: PBasicHartModule;
    fDeviceType: TDeviceType;
    fModule1ChDataRec: TModule1ChDataRec;
    fSignalSpan: real; // in mV or Ohm
    fProtectedPath,
    fUnusualLinConfirmed,
    fUnusualCJCConfirmed: boolean;

    procedure EncodeModuleData;
    procedure DecodeModuleData;

    procedure SetInputTypeSelectorLabels;
    procedure SetInputTypeItem;
    procedure SetInputTypeSpecificInputNumberSelectorLabels;
    procedure SetInputNumberSelectorItem;
    procedure SetCJCSelectorLabels;
    procedure SetRTDWiringSelectorLabels;
    procedure SetCJCorRTDWiringSelectorItems(DisableUnused: boolean);

    procedure SetLinearisationSelectorLabels;
    procedure SetLinearisationSelectorItems(DisableUnused: boolean);

    procedure SetInputTypeSpecificProcessUnitSelectorLabels;
    procedure SetInputTypeSpecificLastProcessUnitSelectorLabel;
    procedure SetInputTypeSpecificProcessUnitSelectorItem;

    procedure SetSignalRangeEditLabels;
    procedure SetSignalRangeEditLimits;
    procedure SetSignalRangeEditValues;
    procedure UpdateSignalRangeEditValues(LSignal, USignal: real);
    procedure SetSignalRangeEditUnits;

    procedure SetFilterFrequencyLabels;
    procedure SetFilterFrequencySelectorItems;

    procedure SetOutputFilteringEditLabels;
    procedure SetOutputFilteringEditLimits;
    procedure SetOutputFilteringEditValues;

    procedure SetOutputLimitationSelectorLabels;
    procedure SetOutputLimitationSelectorItems;
    procedure SetResponseTimeSelectorLabels;
    procedure SetResponseTimeSelectorItems;

    procedure SetAlarmLevelSelectorLabels;

    procedure SetTransmitterFailureAlarmLevelSelectorItems;
    procedure SetColdJunctionFailureAlarmLevelItems(DisableUnused: boolean);
    procedure SetOpenCircuitAlarmLevelItems;
    procedure SetShortCircuitAlarmLevelItems(DisableUnused: boolean);
    procedure SetAlarmLevelSelectorItems;

    procedure SetDeviceSpecificTripEBDSelectorLabels;
    procedure SetTripSelectorEditLimits;
    procedure SetEBDTripSelectorItems;

    procedure SetAllLabels;
    procedure SetRangeErrorMessages;
    procedure ArrangeDeviceTypeSpecificVisibleControls;
    procedure ArrangeDisplayModeLook;
    procedure ArrangeConfigurationModeLook;
    procedure SetAllDisplayValues;
    procedure IndicateReceivedChar;
    procedure UpdateAcceptButton;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
    procedure SetMode(DMode: TDialogMode; SMode: TSaveMode);
  end;

var
  fmModuleData_73: TfmModuleData_73;

implementation

{$R *.dfm}

uses ProcessUtils, StringUtils, TempUtils, DataInstances, DataConstants, LabelNumbers,
     HartDataTypes, HelpContextNumbers, ConversionProcs, MTL4575Sources;

function GetInputLimits(DeviceType: TDeviceType; InputType: TTempInputType; TypeNumber: word;
                        Linearisation: TOnOffSet; PVUnits: word): TRealLimits;
var lInputLimits: TRealLimits;
begin
  case InputType of
       itTHC: begin
              case TypeNumber of
                   0: lInputLimits:=C_Lim_B;
                   1: lInputLimits:=C_Lim_E;
                   2: lInputLimits:=C_Lim_J;
                   3: lInputLimits:=C_Lim_K;
                   4: lInputLimits:=C_Lim_N;
                   5: if not(DeviceType in [dt4075,dtx575,dtx573]) then lInputLimits:=C_Lim_R
                      else lInputLimits:=C_Lim_R75;
                   6: begin
                      if (DeviceType=dt4073XK) or (DeviceType=dt5074XK) then lInputLimits:=C_Lim_XK
                      else if not(DeviceType in [dt4075,dtx575,dtx573]) then lInputLimits:=C_Lim_S
                           else lInputLimits:=C_Lim_S75;
                      end;
                   7: lInputLimits:=C_Lim_T;
                   8: if not(DeviceType in [dt4075,dtx575,dtx573]) then lInputLimits:=C_Lim_mV
                      else lInputLimits:=C_Lim_XK;
                   9: if (DeviceType in [dt4075,dtx575,dtx573]) then lInputLimits:=C_Lim_mV;
                   end;
              end;
       itRTD: case DeviceType of
                   dt4075: case TypeNumber of
                                0: if Linearisation=tsOn then
                                      lInputLimits:=C_Lim_Pt
                                   else lInputLimits:=C_Lim_RLin;
                                1: lInputLimits:=C_Lim_Cu50;
                                2: lInputLimits:=C_Lim_Cu53;
                                3: lInputLimits:=C_Lim_RLin;
                                end;
                   dtx575,
                   dtx573: case TypeNumber of
                                0: if Linearisation=tsOn then
                                      lInputLimits:=C_Lim_Pt
                                   else lInputLimits:=C_Lim_RLin;
                                1: lInputLimits:=C_Lim_Pt;
                                2: lInputLimits:=C_Lim_Pt1T;
                                3,4,5: lInputLimits:=C_Lim_Ni;
                                6: lInputLimits:=C_Lim_Cu50;
                                7: lInputLimits:=C_Lim_Cu53;
                                8: lInputLimits:=C_Lim_RLin;
                                end;
                   else
                        begin
                        if TypeNumber=0 then lInputLimits:=C_Lim_Pt
                        else lInputLimits:=C_Lim_RLin;
                        end;
                   end;
       end;

  case PVUnits of
       1: begin             // Fahrenheit
          lInputLimits.LLim:=CelsiusToFahrenheit(lInputLimits.LLim);
          lInputLimits.ULim:=CelsiusToFahrenheit(lInputLimits.ULim);
          end;
       2: begin             // Rankine
          lInputLimits.LLim:=CelsiusToRankine(lInputLimits.LLim);
          lInputLimits.ULim:=CelsiusToRankine(lInputLimits.ULim);
          end;
       3: begin             // Kelvin
          lInputLimits.LLim:=CelsiusToKelvin(lInputLimits.LLim);
          lInputLimits.ULim:=CelsiusToKelvin(lInputLimits.ULim);
          end;
       4: begin             // mV
          // InputLimits.LLim:=InputLimits.LLim;               // ???
          // InputLimits.ULim:=InputLimits.ULim;
          end;
       5: begin             // Ohm
          // InputLimits.LLim:=InputLimits.LLim;               // ???
          // InputLimits.ULim:=InputLimits.ULim;
          end;
       end;
  Result:=lInputLimits;
end;

function GetSignalSpan (DT: TDeviceType; Source: TModule1ChDataRec): real;
var lPoint4mA, lPoint20mA: real;
begin
  Result:=0.0; // default return value

  if (Source.Point4mA<>Source.Point20mA) then
     begin
     case Source.PVUnits of
          1: begin
             lPoint4mA:=FahrenheitToCelsius(Source.Point4mA);
             lPoint20mA:=FahrenheitToCelsius(Source.Point20mA);
             end;
          2: begin
             lPoint4mA:=RankineToCelsius(Source.Point4mA);
             lPoint20mA:=RankineToCelsius(Source.Point20mA);
             end;
          3: begin
             lPoint4mA:=KelvinToCelsius(Source.Point4mA);
             lPoint20mA:=KelvinToCelsius(Source.Point20mA);
             end;
          // 0: ; // PVUnits in Celsius
          // 4: ; // PVUnits in mV
          // 5: ; // PVUnits in Ohm
          else
             begin
             lPoint4mA:=Source.Point4mA;
             lPoint20mA:=Source.Point20mA;
             end;
          end;

     case Source.InputType of
          itTHC: begin
                 case Source.TypeNumber of
                      0: begin
                         lPoint4mA:=GetITS90THCVoltage(ttB,lPoint4mA);
                         lPoint20mA:=GetITS90THCVoltage(ttB,lPoint20mA);
                         end;
                      1: begin
                         lPoint4mA:=GetITS90THCVoltage(ttE,lPoint4mA);
                         lPoint20mA:=GetITS90THCVoltage(ttE,lPoint20mA);
                         end;
                      2: begin
                         lPoint4mA:=GetITS90THCVoltage(ttJ,lPoint4mA);
                         lPoint20mA:=GetITS90THCVoltage(ttJ,lPoint20mA);
                         end;
                      3: begin
                         lPoint4mA:=GetITS90THCVoltage(ttK,lPoint4mA);
                         lPoint20mA:=GetITS90THCVoltage(ttK,lPoint20mA);
                         end;
                      4: begin
                         lPoint4mA:=GetITS90THCVoltage(ttN,lPoint4mA);
                         lPoint20mA:=GetITS90THCVoltage(ttN,lPoint20mA);
                         end;
                      5: begin
                         lPoint4mA:=GetITS90THCVoltage(ttR,lPoint4mA);
                         lPoint20mA:=GetITS90THCVoltage(ttR,lPoint20mA);
                         end;
                      6: // Note: On all MTL temperature modules the sensor number 6 is type S
                         // except of MTL4073XK and MTL5074XK, where the sensor number 6 was XK
                         if not(DT in [dt5074XK,dt4073XK]) then
                            begin
                            lPoint4mA:=GetITS90THCVoltage(ttS,lPoint4mA);
                            lPoint20mA:=GetITS90THCVoltage(ttS,lPoint20mA);
                            end
                         else
                            begin
                            // The formula for XK sensor calculation is based on the firmware sources for MTL4073XK
                            // The MTL4575 uses different approach with possibly minimally different results
                            lPoint4mA:=GetXKTHCVoltage(lPoint4mA);
                            lPoint20mA:=GetXKTHCVoltage(lPoint20mA);
                            end;
                      7: begin
                         lPoint4mA:=GetITS90THCVoltage(ttT,lPoint4mA);
                         lPoint20mA:=GetITS90THCVoltage(ttT,lPoint20mA);
                         end;
                      8: // Note: On MTL4075 and on new devices MTLx500 the sensor number 8 is XK
                         // otherwise (on all legacy modules) the sensor number 8 was mV
                         // and therefore no span calculation is required
                         if (DT in [dt4075,dtx575,dtx573]) then
                            begin
                            // The formula for XK sensor calculation is based on the firmware sources for MTL4073XK
                            // The MTL4575 uses different approach with possibly minimally different results
                            lPoint4mA:=GetXKTHCVoltage(lPoint4mA);
                            lPoint20mA:=GetXKTHCVoltage(lPoint20mA);
                            end;
                      9: // Note: On MTL4075 and on new devices MTLx500 the sensor number 9 is mV
                         // and therefore no span calculation is required
                         // otherwise the selection is wrong and span will be forced to 0.0
                         if not(DT in [dt4075,dtx575,dtx573]) then
                            lPoint4mA:=lPoint20mA; // force Result to 0.0
                      else lPoint4mA:=lPoint20mA; // force Result to 0.0
                      end;
                 Result:=lPoint20mA-lPoint4mA;
                 end;
          itRTD: begin
                 case Source.TypeNumber of
                      0: // Note: in some conditions the TypeNumber=0 (Pt100) will be used for linear resistors.
                         // This is fact a bug, but instead difficult debugging a work arround approach is used
                         // Therefore the below instruction will be performed only when linearisation is enabled
                         if not(Source.Linearisation=tsOff) then
                            begin
                            lPoint4mA:=GetRTDResistance(rtPlatin,100.0,lPoint4mA);
                            lPoint20mA:=GetRTDResistance(rtPlatin,100.0,lPoint20mA);
                            end;
                      1: case DT of
                              dt507x,
                              dtx073,
                              dt5074XK,
                              dt4073XK: if not(Source.Linearisation=tsOff) then // the linear resistor requires no linearisation
                                           lPoint4mA:=lPoint20mA;               // therefore wrong selection and force Result to 0.0
                              dt4075: begin
                                      // The formula for Cu50 sensor is based on the firmware sources for MTL4075/4575 etc.
                                      lPoint4mA:=T2OhmsCU50(lPoint4mA);
                                      lPoint20mA:=T2OhmsCU50(lPoint20mA);
                                      end;
                              dtx575,
                              dtx573: begin
                                      lPoint4mA:=GetRTDResistance(rtPlatin,500.0,lPoint4mA)/5.0;
                                      lPoint20mA:=GetRTDResistance(rtPlatin,500.0,lPoint20mA)/5.0;
                                      end;
                              end;
                      2: case DT of
                              dt4075: begin
                                      // The formula for Cu53 sensor is based on the firmware sources for MTL4075/4575 etc.
                                      lPoint4mA:=T2OhmsCU53(lPoint4mA);
                                      lPoint20mA:=T2OhmsCU53(lPoint20mA);
                                      end;
                              dtx575,
                              dtx573: begin
                                      lPoint4mA:=GetRTDResistance(rtPlatin,1000.0,lPoint4mA)/10.0;
                                      lPoint20mA:=GetRTDResistance(rtPlatin,1000.0,lPoint20mA)/10.0;
                                      end;
                              end;
                      3: case DT of
                              dt4075: if not(Source.Linearisation=tsOff) then // the linear resistor requires no linearisation
                                           lPoint4mA:=lPoint20mA;             // therefore wrong selection and force Result to 0.0
                              dtx575,
                              dtx573: begin
                                      // The formula for Ni100 sensor is based on the firmware sources for MTL4075/4575 etc.
                                      lPoint4mA:=T2OhmsNi100(lPoint4mA);
                                      lPoint20mA:=T2OhmsNi100(lPoint20mA);
                                      end;
                              end;
                      4: begin // Ni500 sensor
                         // The span calculation for Ni500 sensor will be substituted with "5 * Ni100 / 5 = Ni100"
                         // The formula for Ni100 sensor is based on the firmware sources for MTL4075/4575 etc.
                         lPoint4mA:=T2OhmsNi100(lPoint4mA);
                         lPoint20mA:=T2OhmsNi100(lPoint20mA);
                         end;
                      5: begin // Ni1000 sensor
                         // The span calculation for Ni1000 sensor will be substituted with "10 * Ni100 / 10 = Ni100"
                         // The formula for Ni100 sensor is based on the firmware sources for MTL4075/4575 etc.
                         lPoint4mA:=T2OhmsNi100(lPoint4mA);
                         lPoint20mA:=T2OhmsNi100(lPoint20mA);
                         end;
                      6: begin
                         // The formula for Cu50 sensor is based on the firmware sources for MTL4075/4575 etc.
                         lPoint4mA:=T2OhmsCU50(lPoint4mA);
                         lPoint20mA:=T2OhmsCU50(lPoint20mA);
                         end;
                      7: begin
                         // The formula for Cu53 sensor is based on the firmware sources for MTL4075/4575 etc.
                         lPoint4mA:=T2OhmsCU53(lPoint4mA);
                         lPoint20mA:=T2OhmsCU53(lPoint20mA);
                         end;
                      8: // This input type is reserved for linear resistors only (without linearisation)
                         // In case of use this number zogether with linearisation, set the 4mA=20mA
                         // This will cause span = 0.0 and suggest to terminate the configuration
                         if not(Source.Linearisation=tsOff) then
                            lPoint4mA:=lPoint20mA;
                      // For unexpected input configurations set 4mA=20mA causing span 0.0
                      else lPoint4mA:=lPoint20mA;
                      end;
                 Result:=lPoint20mA-lPoint4mA;
                 end;
          end;
     end;
end;

//****************************************************************************//

procedure TfmModuleData_73.EncodeModuleData;
var lPrimaryVariableOutputData: THPrimaryVariableOutput;
    lx073ParameterSet: THx073ParameterSet;
    lx075TripParameters: THx075TripParameters;
    lResponseSpeed,lNamurOption,lShortAlarm: byte;
begin
  if (pHartModule<>nil) then
     begin
     lPrimaryVariableOutputData:=TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData;
     lx073ParameterSet:=THartModule_x073(pHartModule^).x073ParameterSet;

     with fModule1ChDataRec do
          begin
          InputType:=InputTypeFromHART(fDeviceType,lx073ParameterSet.InputType);
          TypeNumber:=TypeNumberFromHART(fDeviceType,lx073ParameterSet.InputType);
          ColdJunction:=CJCompFromHART(lx073ParameterSet.ColdJunctionCompensationOnOff);
          WiringNumber:=WiringNumberFromHART(fDeviceType,lx073ParameterSet.InputType);
          Linearisation:=LinearFromHART(lPrimaryVariableOutputData.TransferFunction);

          PVUnits:=PVUnitsFromHART(lPrimaryVariableOutputData.PVUnits);
          Point4mA:=lPrimaryVariableOutputData.Point4mA;
          Point20mA:=lPrimaryVariableOutputData.Point20mA;
          FilterFrequencySetting:=FrequencyFromHART(lx073ParameterSet.FilterFrequencySelector);
          DampingTimeConstant:=lPrimaryVariableOutputData.DampingTimeConstant;
          SmoothingTimeConstant:=lx073ParameterSet.SmoothingTimeConstant;

          TransmitterFailureAlarmLevel:=AlarmLevelFromHART(lx073ParameterSet.TransmitterFailureAlarmLevel);
          ColdJunctionFailureAlarmLevel:=AlarmLevelFromHART(lx073ParameterSet.ColdJunctionFailureAlarmLevel);
          OpenCircuitAlarmLevel:=AlarmLevelFromHART(lx073ParameterSet.OpenCircuitAlarmLevel);

          if (fDeviceType in [dt4075,dtx575,dtx573]) then
             begin
             lx075TripParameters:=THartModule_x075(pHartModule^).x075TripParameters;

             TripOutputSetting:=x075TripTypeFromHART(lx075TripParameters.TripOutputFunction);
             TripThreshold:=lx075TripParameters.TripValue;
             TripHysteresis:=lx075TripParameters.Hysteresis;
             end;

          if (fDeviceType in [dtx575,dtx573]) then
             begin
             lResponseSpeed:=THartModule_x575(pHartModule^).ResponseTime;
             lNamurOption:=THartModule_x575(pHartModule^).NE43Mode;

             ResponseSpeed:=x575SpeedFromHART(lResponseSpeed);
             NamurOption:=x575NAMURFromHART(lNamurOption);
             end;

          if (fDeviceType=dtx573) then
             begin
             lShortAlarm:=THartModule_x573(pHartModule^).ShortCircuitAlarmLevel;
             ShortCircuitAlarmLevel:=AlarmLevelFromHART(lShortAlarm);
             end;
          end;
     end;
end;

procedure TfmModuleData_73.DecodeModuleData;
var lPrimaryVariableOutputData: THPrimaryVariableOutput;
    lx073ParameterSet: THx073ParameterSet;
    lx075TripParameters: THx075TripParameters;
    lResponseSpeed,lNamurOption,lShortAlarm: byte;
begin
  if (pHartModule<>nil) then
     begin
     lx073ParameterSet.InputType:=InputTypeAndNumberAndWiringToHART(fDeviceType,fModule1ChDataRec.InputType,fModule1ChDataRec.TypeNumber,fModule1ChDataRec.WiringNumber);
     lx073ParameterSet.ColdJunctionCompensationOnOff:=CJCompToHART(fModule1ChDataRec.ColdJunction);
     lx073ParameterSet.OpenCircuitAlarmLevel:=AlarmLevelToHART(fModule1ChDataRec.OpenCircuitAlarmLevel);
     lx073ParameterSet.TransmitterFailureAlarmLevel:=AlarmLevelToHART(fModule1ChDataRec.TransmitterFailureAlarmLevel);
     lx073ParameterSet.FilterFrequencySelector:=FrequencyToHART(fModule1ChDataRec.FilterFrequencySetting);
     lx073ParameterSet.SmoothingTimeConstant:=word(fModule1ChDataRec.SmoothingTimeConstant);
     lx073ParameterSet.ColdJunctionFailureAlarmLevel:=AlarmLevelToHART(fModule1ChDataRec.ColdJunctionFailureAlarmLevel);

     lPrimaryVariableOutputData.PVAlarm:=0; // not used in MTL products, set to 0
     lPrimaryVariableOutputData.TransferFunction:=LinearToHART(fModule1ChDataRec.Linearisation);
     lPrimaryVariableOutputData.PVUnits:=PVUnitsToHART(fModule1ChDataRec.PVUnits);
     lPrimaryVariableOutputData.Point4mA:=fModule1ChDataRec.Point4mA;
     lPrimaryVariableOutputData.Point20mA:=fModule1ChDataRec.Point20mA;
     lPrimaryVariableOutputData.DampingTimeConstant:=fModule1ChDataRec.DampingTimeConstant;

     TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData:=lPrimaryVariableOutputData;
     THartModule_x073(pHartModule^).x073ParameterSet:=lx073ParameterSet;

     if (fDeviceType in [dt4075,dtx575,dtx573]) then
        begin
        lx075TripParameters.TripOutputFunction:=x075TripTypeToHART(fModule1ChDataRec.TripOutputSetting);
        lx075TripParameters.TripValue:=fModule1ChDataRec.TripThreshold;
        lx075TripParameters.Hysteresis:=fModule1ChDataRec.TripHysteresis;

        THartModule_x075(pHartModule^).x075TripParameters:=lx075TripParameters;
        end;

     if (fDeviceType in [dtx575,dtx573]) then
        begin
        lResponseSpeed:=x575SpeedToHART(fModule1ChDataRec.ResponseSpeed);
        lNamurOption:=x575NAMURToHART(fModule1ChDataRec.NamurOption);

        THartModule_x575(pHartModule^).ResponseTime:=lResponseSpeed;
        THartModule_x575(pHartModule^).NE43Mode:=lNamurOption;
        end;

     if (fDeviceType=dtx573) then
        begin
        lShortAlarm:=AlarmLevelToHART(fModule1ChDataRec.ShortCircuitAlarmLevel);

        THartModule_x573(pHartModule^).ShortCircuitAlarmLevel:=lShortAlarm;
        end;

     fSignalSpan:=GetSignalSpan(fDeviceType,fModule1ChDataRec);
     end;
end;

//****************************************************************************//

procedure TfmModuleData_73.SetInputTypeSelectorLabels;
begin
  InputTypeSelector.Caption:=LanguageStrings[flInputType];
  InputTypeSelector.Items.Clear;
  InputTypeSelector.Items.Add(LanguageStrings[flTHC]);
  InputTypeSelector.Items.Add(LanguageStrings[flRTD]);
end;

procedure TfmModuleData_73.SetInputTypeItem;
begin
  case fModule1ChDataRec.InputType  of
       itTHC: InputTypeSelector.ItemIndex:=0;
       itRTD: InputTypeSelector.ItemIndex:=1;
       else InputTypeSelector.ItemIndex:=-1;
       end;
  if fDialogMode=dmDisplay then InputTypeSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetInputTypeSpecificInputNumberSelectorLabels;
var I: integer;
begin
  InputNumberSelector.Items.Clear;
  if fModule1ChDataRec.InputType=itTHC then
     begin
     InputNumberSelector.Caption:=DeleteAcceleratorKey(LanguageStrings[flTHCtypes]);

     InputNumberSelector.Height:=200;

     for I:=0 to 5 do InputNumberSelector.Items.Add(LanguageStrings[flTHC_B+I]);
     if (fDeviceType=dt5074XK) or (fDeviceType=dt4073XK) then
        InputNumberSelector.Items.Add(LanguageStrings[flTHC_XK])
     else InputNumberSelector.Items.Add(LanguageStrings[flTHC_B+6]);
     InputNumberSelector.Items.Add(LanguageStrings[flTHC_T]);
     if (fDeviceType in [dt4075,dtx575,dtx573]) then
        InputNumberSelector.Items.Add(LanguageStrings[flTHC_XK]);

     InputNumberSelector.Items.Add(LanguageStrings[flmV]);
     end
  else // fModule1ChDataRec.InputType=itRTD
     begin
     InputNumberSelector.Caption:=DeleteAcceleratorKey(LanguageStrings[flRTDtypes]);

     InputNumberSelector.Items.Add(LanguageStrings[flPt100]);
     if not(fDeviceType in [dt4075,dtx575,dtx573]) then
        begin
        InputNumberSelector.Height:=65;
        end
     else // fDeviceType in [dt4075,dtx575,dtx573]
        begin
        if (fDeviceType=dt4075) then
           begin
           InputNumberSelector.Height:=140;
           InputNumberSelector.Items.Add(LanguageStrings[flCu50]);
           InputNumberSelector.Items.Add(LanguageStrings[flCu53]);
           end
        else // fDeviceType=dtx575 or fDeviceType=dtx573
           begin
           InputNumberSelector.Height:=200;
           InputNumberSelector.Items.Add(LanguageStrings[flPt500]);
           InputNumberSelector.Items.Add(LanguageStrings[flPt1000]);
           InputNumberSelector.Items.Add(LanguageStrings[flNi100]);
           InputNumberSelector.Items.Add(LanguageStrings[flNi500]);
           InputNumberSelector.Items.Add(LanguageStrings[flNi1000]);
           InputNumberSelector.Items.Add(LanguageStrings[flCu50]);
           InputNumberSelector.Items.Add(LanguageStrings[flCu53]);
           end;
        end;
     InputNumberSelector.Items.Add(LanguageStrings[flResistor]);
     end;
end;

procedure TfmModuleData_73.SetInputNumberSelectorItem;
begin
  // detect resistor selection - Pt100 with linearisation off
  if ((fModule1ChDataRec.InputType=itRTD) and (fModule1ChDataRec.TypeNumber=0) and (fModule1ChDataRec.Linearisation=tsOff)) then
     InputNumberSelector.ItemIndex:=InputNumberSelector.Items.Count-1
  else // not resistor selection
     begin
     if fModule1ChDataRec.TypeNumber<InputNumberSelector.Items.Count then
        InputNumberSelector.ItemIndex:=fModule1ChDataRec.TypeNumber
     else InputNumberSelector.ItemIndex:=0;
     end;
  if fDialogMode=dmDisplay then InputNumberSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetCJCSelectorLabels;
begin
  CJCSelector.Caption:=LanguageStrings[flColdJnCompensation];

  CJCSelector.Items.Clear;
  CJCSelector.Items.Add(LanguageStrings[flOn]);
  CJCSelector.Items.Add(LanguageStrings[flOff]);
end;

procedure TfmModuleData_73.SetRTDWiringSelectorLabels;
begin
  RTDWiringSelector.Caption:=LanguageStrings[flWiring];

  RTDWiringSelector.Items.Clear;
  RTDWiringSelector.Items.Add(LanguageStrings[fl2Wire]);
  RTDWiringSelector.Items.Add(LanguageStrings[fl3Wire]);
  RTDWiringSelector.Items.Add(LanguageStrings[fl4Wire]);
end;

procedure TfmModuleData_73.SetCJCorRTDWiringSelectorItems(DisableUnused: boolean);
begin
  if fModule1ChDataRec.InputType=itTHC then
     begin
     CJCSelector.Visible:=true;
     RTDWiringSelector.Visible:=false;
     CJCSelector.EnableAllButtons;
     if fModule1ChDataRec.ColdJunction in [tsOn..tsOff] then
        CJCSelector.ItemIndex:=integer(fModule1ChDataRec.ColdJunction)
     else CJCSelector.ItemIndex:=1;
     if (fDialogMode=dmDisplay) or DisableUnused then CJCSelector.DisableNotCheckedButtons;
     end
  else
     begin
     CJCSelector.Visible:=false;
     RTDWiringSelector.Visible:=true;
     RTDWiringSelector.EnableAllButtons;
     if fModule1ChDataRec.WiringNumber in [0..3] then
        RTDWiringSelector.ItemIndex:=fModule1ChDataRec.WiringNumber
     else RTDWiringSelector.ItemIndex:=0;
     if fDialogMode=dmDisplay then RTDWiringSelector.DisableNotCheckedButtons;
     end;
end;

procedure TfmModuleData_73.SetLinearisationSelectorLabels;
begin
  LinearisationSelector.Caption:=LanguageStrings[flLinearisation];

  LinearisationSelector.Items.Clear;
  LinearisationSelector.Items.Add(LanguageStrings[flOn]);
  LinearisationSelector.Items.Add(LanguageStrings[flOff]);
end;

procedure TfmModuleData_73.SetLinearisationSelectorItems(DisableUnused: boolean);
begin
  LinearisationSelector.EnableAllButtons;
  if fModule1ChDataRec.Linearisation in [tsOn..tsOff] then
     LinearisationSelector.ItemIndex:=integer(fModule1ChDataRec.Linearisation)
  else LinearisationSelector.ItemIndex:=1;
  if (fDialogMode=dmDisplay) or DisableUnused then LinearisationSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetInputTypeSpecificProcessUnitSelectorLabels;
var I: integer;
begin
  ProcessUnitSelector.Caption:=LanguageStrings[flProcessUnits];

  ProcessUnitSelector.Items.Clear;
  for I:=0 to 3 do ProcessUnitSelector.Items.Add(LanguageStrings[flCelsius+I]);
  if fModule1ChDataRec.InputType=itTHC then
     ProcessUnitSelector.Items.Add(LanguageStrings[flCelsius+4])     // mV
  else ProcessUnitSelector.Items.Add(LanguageStrings[flCelsius+5]);  // Ohm
end;

procedure TfmModuleData_73.SetInputTypeSpecificLastProcessUnitSelectorLabel;
begin
  if (fModule1ChDataRec.InputType=itTHC) then
     ProcessUnitSelector.Items.Strings[4]:=LanguageStrings[flCelsius+4]    //mV
  else ProcessUnitSelector.Items.Strings[4]:=LanguageStrings[flCelsius+5]; // Ohm
end;

procedure TfmModuleData_73.SetInputTypeSpecificProcessUnitSelectorItem;
begin
  ProcessUnitSelector.EnableAllButtons;
  if (fModule1ChDataRec.PVUnits in [0..3]) or ((fModule1ChDataRec.PVUnits=4) and(fModule1ChDataRec.InputType=itTHC)) then
     ProcessUnitSelector.ItemIndex:=fModule1ChDataRec.PVUnits
  else
     if (fModule1ChDataRec.PVUnits=5) and (fModule1ChDataRec.InputType=itRTD) then
        ProcessUnitSelector.ItemIndex:=4
     else ProcessUnitSelector.ItemIndex:=0; // default units = C, however be careful here !!!
  if fDialogMode=dmDisplay then ProcessUnitSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetSignalRangeEditLabels;
begin
  SignalRangeBox.Caption:=LanguageStrings[flSignalRange];
  Point4mA_Edit.LabelCaption:=LanguageStrings[fl4mAPoint];
  Point20mA_Edit.LabelCaption:=LanguageStrings[fl20mAPoint];
end;

procedure TfmModuleData_73.SetSignalRangeEditLimits;
var lLimits: TRealLimits;
begin
  lLimits:=GetInputLimits(fDeviceType,fModule1ChDataRec.InputType, fModule1ChDataRec.TypeNumber, fModule1ChDataRec.Linearisation, fModule1ChDataRec.PVUnits);

  with Point4mA_Edit do
       begin
       Min:=lLimits.LLim;
       Max:=lLimits.Ulim;
       end;
  with Point20mA_Edit do
       begin
       Min:=lLimits.LLim;
       Max:=lLimits.Ulim;
       end;
end;

procedure  TfmModuleData_73.SetSignalRangeEditValues;
begin
  Point4mA_Edit.Value:=fModule1ChDataRec.Point4mA;
  Point20mA_Edit.Value:=fModule1ChDataRec.Point20mA;
end;

procedure TfmModuleData_73.UpdateSignalRangeEditValues(LSignal, USignal: real);
begin
  fModule1ChDataRec.Point4mA:=LSignal;
  fModule1ChDataRec.Point20mA:=USignal;
  SetSignalRangeEditValues;
end;

procedure TfmModuleData_73.SetSignalRangeEditUnits;
begin
  Point4mA_Edit.UnitText:=LanguageStrings[flCelsius+fModule1ChDataRec.PVUnits];
  Point20mA_Edit.UnitText:=LanguageStrings[flCelsius+fModule1ChDataRec.PVUnits];
end;

procedure TfmModuleData_73.SetFilterFrequencyLabels;
begin
  FrequencySelector.Caption:=LanguageStrings[flFilterFreq];

  FrequencySelector.Items.Clear;
  FrequencySelector.Items.Add(LanguageStrings[fl50Hz]);
  FrequencySelector.Items.Add(LanguageStrings[fl60Hz]);
end;

procedure TfmModuleData_73.SetFilterFrequencySelectorItems;
begin
  if fModule1ChDataRec.FilterFrequencySetting in [tf50..tf60] then
     FrequencySelector.ItemIndex:=integer(fModule1ChDataRec.FilterFrequencySetting)
  else FrequencySelector.ItemIndex:=0;
  if fDialogMode=dmDisplay then FrequencySelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetOutputFilteringEditLabels;
begin
  OutputFilteringBox.Caption:=LanguageStrings[flOutputFiltering];

  DampingConstant_Edit.LabelCaption:=LanguageStrings[flDampingConst];
  SmoothingConstant_Edit.LabelCaption:=LanguageStrings[flSmoothingConst];
end;

procedure TfmModuleData_73.SetOutputFilteringEditLimits;
begin
  with DampingConstant_Edit do
       begin
       Min:=C_Lim_Damp.LLim;
       Max:=C_Lim_Damp.Ulim;
       end;
  with SmoothingConstant_Edit do
       begin
       Min:=C_Lim_Smoo.LLim;
       Max:=C_Lim_Smoo.Ulim;
       end;
end;

procedure TfmModuleData_73.SetOutputFilteringEditValues;
begin
  DampingConstant_Edit.Value:=fModule1ChDataRec.DampingTimeConstant;
  SmoothingConstant_Edit.Value:=fModule1ChDataRec.SmoothingTimeConstant;
end;

procedure TfmModuleData_73.SetOutputLimitationSelectorLabels;
begin
  OutputModeBox.Caption:=LanguageStrings[flOutputCharacteristic];

  NE43OnButton.Caption:=LanguageStrings[flOutNE43];
  NE43OffButton.Caption:=LanguageStrings[flOutMTL];
end;

procedure TfmModuleData_73.SetOutputLimitationSelectorItems;
begin
  NE43OffButton.Enabled:=true;
  NE43OnButton.Enabled:=true;
  case fModule1ChDataRec.NamurOption of
       nsOff: NE43OffButton.Checked:=true;
       nsOn:  NE43OnButton.Checked:=true;
       nsNotAvailable: begin
                       // consider which action is required here ?
                       end;
       end;
  if fDialogMode=dmDisplay then
     begin
     NE43OnButton.Enabled:=NE43OnButton.Checked;
     NE43OffButton.Enabled:=NE43OffButton.Checked;
     end;

end;

procedure TfmModuleData_73.SetResponseTimeSelectorLabels;
begin
  ResponseTimeSelector.Caption:=LanguageStrings[flResponseTime];

  ResponseTimeSelector.Items.Clear;
  ResponseTimeSelector.Items.Add(LanguageStrings[flSlow]);
  ResponseTimeSelector.Items.Add(LanguageStrings[flMedium]);
  ResponseTimeSelector.Items.Add(LanguageStrings[flFast]);
end;

procedure TfmModuleData_73.SetResponseTimeSelectorItems;
begin
  ResponseTimeSelector.EnableAllButtons;
  if fModule1ChDataRec.ResponseSpeed in [tsMiddle..tsFast] then
     ResponseTimeSelector.ItemIndex:=integer(fModule1ChDataRec.ResponseSpeed)
  else ResponseTimeSelector.ItemIndex:=0;
  if fDialogMode=dmDisplay then ResponseTimeSelector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetAlarmLevelSelectorLabels;
begin
  AlarmLevelBox.Caption:=LanguageStrings[flAlarmLevels];

  TX_Alarm_Selector.Caption:=LanguageStrings[flAlarmTX];
  TX_Alarm_Selector.Items.Clear;
  TX_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  TX_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  TX_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);

  CJC_Alarm_Selector.Caption:=LanguageStrings[flAlarmCJC];
  CJC_Alarm_Selector.Items.Clear;
  CJC_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  CJC_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  CJC_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);

  OC_Alarm_Selector.Caption:=LanguageStrings[flAlarmOC];
  OC_Alarm_Selector.Items.Clear;
  OC_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  OC_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  OC_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);

  SC_Alarm_Selector.Caption:=LanguageStrings[flAlarmSC];
  SC_Alarm_Selector.Items.Clear;
  SC_Alarm_Selector.Items.Add(LanguageStrings[flLowAlarm]);
  SC_Alarm_Selector.Items.Add(LanguageStrings[flHighAlarm]);
  SC_Alarm_Selector.Items.Add(LanguageStrings[flAlarmOff]);
end;

procedure TfmModuleData_73.SetTransmitterFailureAlarmLevelSelectorItems;
begin
  TX_Alarm_Selector.EnableAllButtons;
  if fModule1ChDataRec.TransmitterFailureAlarmLevel in [taLow..taHigh] then
     TX_Alarm_Selector.ItemIndex:=integer(fModule1ChDataRec.TransmitterFailureAlarmLevel)
  else TX_Alarm_Selector.ItemIndex:=2;
  if fDialogMode=dmDisplay then TX_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetColdJunctionFailureAlarmLevelItems(DisableUnused: boolean);
begin
  CJC_Alarm_Selector.EnableAllButtons;
  if fModule1ChDataRec.ColdJunctionFailureAlarmLevel in [taLow..taHigh] then
     CJC_Alarm_Selector.ItemIndex:=integer(fModule1ChDataRec.ColdJunctionFailureAlarmLevel)
  else CJC_Alarm_Selector.ItemIndex:=2;
  if (fDialogMode=dmDisplay) or DisableUnused then CJC_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetOpenCircuitAlarmLevelItems;
begin
  OC_Alarm_Selector.EnableAllButtons;
  if fModule1ChDataRec.OpenCircuitAlarmLevel in [taLow..taHigh] then
     OC_Alarm_Selector.ItemIndex:=integer(fModule1ChDataRec.OpenCircuitAlarmLevel)
  else OC_Alarm_Selector.ItemIndex:=2;
  if fDialogMode=dmDisplay then OC_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetShortCircuitAlarmLevelItems(DisableUnused: boolean);
begin
  SC_Alarm_Selector.EnableAllButtons;
  if fModule1ChDataRec.ShortCircuitAlarmLevel in [taLow..taHigh] then
     SC_Alarm_Selector.ItemIndex:=integer(fModule1ChDataRec.ShortCircuitAlarmLevel)
  else SC_Alarm_Selector.ItemIndex:=2;
  if (fDialogMode=dmDisplay) or DisableUnused then SC_Alarm_Selector.DisableNotCheckedButtons;
end;

procedure TfmModuleData_73.SetAlarmLevelSelectorItems;
begin
  SetTransmitterFailureAlarmLevelSelectorItems;

  SetColdJunctionFailureAlarmLevelItems(false);

  SetOpenCircuitAlarmLevelItems;

  SetShortCircuitAlarmLevelItems(false);
end;

procedure TfmModuleData_73.SetDeviceSpecificTripEBDSelectorLabels;
begin
  if (fDeviceType in [dt4075,dtx575,dtx573]) then
     begin
     if (fDeviceType=dtx573) then
        begin
        TripEBDSelectorBox.Caption:=LanguageStrings[flEBDdetection];
        EBDOnButton.Caption:=LanguageStrings[flOn];
        end
     else // fDeviceType in [dt4075,dtx575]
        begin
        TripEBDSelectorBox.Caption:=LanguageStrings[flEBDdetectionTrip];
        TripOnButton.Caption:=LanguageStrings[flTrip];
        EBDOnButton.Caption:=LanguageStrings[flEBD];

        Threshold_Edit.LabelCaption:=LanguageStrings[flThreshold];
        Threshold_Edit.UnitText:=LanguageStrings[flPercent];
        Hysteresis_Edit.LabelCaption:=LanguageStrings[flHysteresis];
        Hysteresis_Edit.UnitText:=LanguageStrings[flPercent];
        end;
     EBDOffButton.Caption:=LanguageStrings[flOff];
     end;
end;

procedure TfmModuleData_73.SetTripSelectorEditLimits;
begin
  with Threshold_Edit do
       begin
       Min:=C_Lim_Trip.LLim;
       Max:=C_Lim_Trip.ULim;
       end;
  with Hysteresis_Edit do
       begin
       Min:=C_Lim_Hyst.LLim;
       Max:=C_Lim_Hyst.ULim;
       end;
end;

procedure TfmModuleData_73.SetEBDTripSelectorItems;
begin
  // Enable All Buttons
  TripOnButton.Enabled:=true;
  EBDOnButton.Enabled:=(fModule1ChDataRec.InputType=itTHC);
  EBDOffButton.Enabled:=true;

  TripParameterPanel.Enabled:=true;
  Threshold_Edit.Enabled:=true;
  Threshold_Edit.EditColor:=clWindow;
  Hysteresis_Edit.Enabled:=true;
  Hysteresis_Edit.EditColor:=clWindow;

  if (fModule1ChDataRec.TripOutputSetting=ttTrip) then
     begin
     TripOnButton.Checked:=true;

     Threshold_Edit.EditFont.Color:=Self.Font.Color;
     Hysteresis_Edit.EditFont.Color:=Self.Font.Color;

     Threshold_Edit.Value:=fModule1ChDataRec.TripThreshold;
     Hysteresis_Edit.Value:=fModule1ChDataRec.TripHysteresis;
     end
  else // not(fModule1ChDataRec.TripOutputSetting=ttTrip)
     begin
     if (fModule1ChDataRec.TripOutputSetting=ttEBD) and
        (fModule1ChDataRec.InputType=itTHC) then
        EBDOnButton.Checked:=true
     else // fModule1ChDataRec.TripOutputSetting=ttOff
        EBDOffButton.Checked:=true;

     TripParameterPanel.Enabled:=false;
     Threshold_Edit.Enabled:=false;
     Threshold_Edit.EditColor:=clSilver;
     Threshold_Edit.EditFont.Color:=clSilver;
     Hysteresis_Edit.Enabled:=false;
     Hysteresis_Edit.EditColor:=clSilver;
     Hysteresis_Edit.EditFont.Color:=clSilver;
     end;

  // Disable Not Checked Buttons and Edits and grey-out Edit Background
  if (fDialogMode=dmDisplay) then
     begin
     if not(TripOnButton.Checked) then
        TripOnButton.Enabled:=false;
     if EBDOnButton.Enabled and not(EBDOnButton.Checked) then
        EBDOnButton.Enabled:=false;
     if not(EBDOffButton.Checked) then
        EBDOffButton.Enabled:=false;

     TripParameterPanel.Enabled:=false;
     Threshold_Edit.Enabled:=false;
     Threshold_Edit.EditColor:=clSilver;
     Hysteresis_Edit.Enabled:=false;
     Hysteresis_Edit.EditColor:=clSilver;
     end;
end;

//****************************************************************************//

procedure TfmModuleData_73.SetAllLabels;
begin
  // Self.Caption will be set later according to the fDialogMode

  InputTabSheet.Caption:=LanguageStrings[flInputTab];

  SetInputTypeSelectorLabels;
  // InputNumber are module specific
  SetCJCSelectorLabels;
  SetRTDWiringSelectorLabels;

  SetLinearisationSelectorLabels;
  // Process Units are Input Type specific

  SetSignalRangeEditLabels;

  SetFilterFrequencyLabels;

  OutputTabSheet.Caption:=LanguageStrings[flOutputTab];

  SetOutputFilteringEditLabels;
  SetOutputFilteringEditLimits;

  SetOutputLimitationSelectorLabels;
  SetResponseTimeSelectorLabels;

  AlarmTabSheet.Caption:=LanguageStrings[flAlarmsTab];

  SetAlarmLevelSelectorLabels;

  SetTripSelectorEditLimits;

  OK_Button.Caption:=LanguageStrings[blSetting];
  Close_Button.Caption:=LanguageStrings[blClose];
end;

procedure TfmModuleData_73.SetRangeErrorMessages;
begin
  Point4mA_Edit.RangeErrorMessage:=LanguageStrings[msFieldBetween];
  Point20mA_Edit.RangeErrorMessage:=LanguageStrings[msFieldBetween];

  DampingConstant_Edit.RangeErrorMessage:=LanguageStrings[msFieldBetween];
  SmoothingConstant_Edit.RangeErrorMessage:=LanguageStrings[msFieldBetween];

  if fDeviceType in [dt4075,dtx575] then
     begin
     Threshold_Edit.RangeErrorMessage:=LanguageStrings[msFieldBetween];
     Hysteresis_Edit.RangeErrorMessage:=LanguageStrings[msFieldBetween];
     end;
end;

procedure TfmModuleData_73.ArrangeDeviceTypeSpecificVisibleControls;
begin
  if (fDeviceType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx575]) then
     SC_Alarm_Selector.Visible:=false;

  if (fDeviceType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075]) then
     OutputModeBox.Visible:=false;

  if (fDeviceType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx573]) then
     ResponseTimeSelector.Visible:=false;

  if (fDeviceType in [dt507x,dtx073,dt5074XK,dt4073XK]) then
     begin
     TripEBDSelectorBox.Visible:=false;
     end
  else
     begin
     TripEBDSelectorBox.Visible:=true;
     if (fDeviceType=dtx573) then
        begin
        TripEBDSelectorBox.Height:=45;
        TripOnButton.Visible:=false;
        EBDOffButton.Top:=18;
        EBDOffButton.Left:=96;
        EBDOnButton.Top:=18;
        EBDOnButton.Left:=184;
        TripParameterPanel.Visible:=false;
        end
     else // fDeviceType in [dt4075,dtx575]
        begin
        TripEBDSelectorBox.Height:=80;
        TripOnButton.Visible:=true;

        EBDOffButton.Top:=18;
        EBDOffButton.Left:=9;
        EBDOnButton.Top:=38;
        EBDOnButton.Left:=9;
        TripOnButton.Top:=58;
        TripOnButton.Left:=9;
        TripParameterPanel.Visible:=true;
        end;
     end;

  if fModule1ChDataRec.InputType=itTHC then
     begin
     CJCSelector.Visible:=true;
     RTDWiringSelector.Visible:=false;
     end
  else // fModule1ChDataRec.InputType=itRTD
     begin
     CJCSelector.Visible:=false;
     RTDWiringSelector.Visible:=true;
     end;
end;

procedure TfmModuleData_73.ArrangeDisplayModeLook;
begin
  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[flDisplayModule]);
  HelpContext:=hcDisplayModule_x573;

  // Input Configuration Tab Sheet

  SignalRangeBox.Enabled:=false;
  Point4mA_Edit.EditColor:=clSilver;
  Point20mA_Edit.EditColor:=clSilver;

  OutputFilteringBox.Enabled:=false;
  DampingConstant_Edit.EditColor:=clSilver;
  SmoothingConstant_Edit.EditColor:=clSilver;

  CommGauge.Visible:=false;
  OK_Button.Visible:=false;

  Close_Button.Top:=PageControl1.Top+PageControl1.Height+20;
  Close_Button.Left:=CommGauge.Left+(CommGauge.Width-Close_Button.Width) DIV 2;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmModuleData_73.ArrangeConfigurationModeLook;
var lButtonGap: integer;
begin
  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[flConfigModule]);
  HelpContext:=hcConfigureModule_x573;

  SetRangeErrorMessages;

  CommGauge.Visible:=true;
  CommGauge.Top:=PageControl1.Top+PageControl1.Height+15;

  OK_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Top:=OK_Button.Top;

  lButtonGap:=(CommGauge.Width-OK_Button.Width-Close_Button.Width) DIV 3;
  OK_Button.Left:=CommGauge.Left+lButtonGap;
  Close_Button.Left:=CommGauge.Left+OK_Button.Width+2*lButtonGap;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmModuleData_73.SetAllDisplayValues;
begin
  // Input Configuration Tab Sheet

  fProtectedPath:=true;

  SetInputTypeItem;

  SetInputNumberSelectorItem;

  // Disable unused CJC Buttons if mV (evaluated here as THC without Linearisation)
  SetCJCorRTDWiringSelectorItems((fModule1ChDataRec.InputType=itTHC) and (fModule1ChDataRec.Linearisation=tsOff));

  SetLinearisationSelectorItems(true);

  SetInputTypeSpecificProcessUnitSelectorItem;
  if fDialogMode=dmConfig then
     begin
     if (fModule1ChDataRec.PVUnits<4) and (fModule1ChDataRec.Linearisation=tsOn) then
        ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits))
     else if not(fModule1ChDataRec.PVUnits<4) and (fModule1ChDataRec.Linearisation=tsOff) then
             ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CLinUnits));
     end;

  SetSignalRangeEditValues;
  SetSignalRangeEditUnits;

  SetFilterFrequencySelectorItems;

  // Output Configuration Tab Sheet

  SetOutputFilteringEditValues;

  SetOutputLimitationSelectorItems;

  SetResponseTimeSelectorItems;

  // Alarm Configuration Tab Sheet

  SetAlarmLevelSelectorItems;

  if (fDialogMode=dmConfig) and
     (fModule1ChDataRec.InputType=itTHC) then
     SC_Alarm_Selector.DisableNotCheckedButtons
  else CJC_Alarm_Selector.DisableNotCheckedButtons;

  SetEBDTripSelectorItems;

  fProtectedPath:=false;
end;

procedure TfmModuleData_73.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(10);
     end
  else WaitMilliseconds(100);
end;

procedure TfmModuleData_73.UpdateAcceptButton;
begin
  if pHartModule<>nil then
     OK_Button.Enabled:=not(pHartModule^.VerifyDeviceData);
end;

//****************************************************************************//

procedure TfmModuleData_73.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmModuleData_73.SetMode(DMode: TDialogMode; SMode: TSaveMode);
begin
  fDialogMode:=DMode; fSaveMode:=SMode;
end;

//****************************************************************************//

procedure TfmModuleData_73.FormCreate(Sender: TObject);
begin
  fDialogMode:=dmDisplay;
  fSaveMode:=smToDevice;
  fProtectedPath:=false;
  fDeviceType:=dtNone;
  pHartModule:=nil;

  with fModule1ChDataRec do
       begin
       InputType:=itNone;
       TypeNumber:=0;
       ColdJunction:=tsOff;
       WiringNumber:=0;
       Linearisation:=tsOff;
       PVUnits:=0;
       Point4mA:=0.0;
       Point20mA:=100.0;
       FilterFrequencySetting:=tf50;
       DampingTimeConstant:=0.0;
       SmoothingTimeConstant:=0;
       NamurOption:=nsOff;
       ResponseSpeed:=tsSlow;
       TransmitterFailureAlarmLevel:=taLow;
       ColdJunctionFailureAlarmLevel:=taLow;
       OpenCircuitAlarmLevel:=taHigh;
       ShortCircuitAlarmLevel:=taOff;
       TripOutputSetting:=ttOff;
       TripThreshold:=50.0;
       TripHysteresis:=1.0;
       end;

  fSignalSpan:=0.0;

  fUnusualLinConfirmed:=false;
  fUnusualCJCConfirmed:=false;

  PageControl1.OwnerDraw:=true;

  SetAllLabels;
end;

procedure TfmModuleData_73.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin

     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);
     if (fDeviceType<>dtNone) then
        begin
        EncodeModuleData;

        SetInputTypeSpecificInputNumberSelectorLabels;
        SetInputTypeSpecificProcessUnitSelectorLabels;
        SetSignalRangeEditLimits;
        SetDeviceSpecificTripEBDSelectorLabels;

        ArrangeDeviceTypeSpecificVisibleControls;

        if fDialogMode=dmDisplay then ArrangeDisplayModeLook;
        if fDialogMode=dmConfig then ArrangeConfigurationModeLook;

        SetAllDisplayValues;
        if fDialogMode=dmConfig then OK_Button.Enabled:=false;
        end
     else
        begin
        MessageDlg('Unexpected error',mtError,[mbCancel],hcNoHelpTopic);
        Close;
        end;
     end
  else Close;
end;

procedure TfmModuleData_73.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var fExpectedTransferDataLength: integer;
    lUploadRequired,
    lTooSmallSpan,
    lIgnoreSmallSpan: boolean;
begin
  CanClose:=(ModalResult<>mrOK);
  if (pHartModule<>nil) and (fDialogMode=dmConfig) then
     begin
     // OriginalCursor:=Cursor;
     lUploadRequired:=false;
     lIgnoreSmallSpan:=false;
     PageControl1.Enabled:=false;
     OK_Button.Enabled:=false;
     Close_Button.Enabled:=false;
     if ((ModalResult<>mrOK) and not(pHartModule^.VerifyDeviceData)) then
        case (MessageDlg(LanguageStrings[mlChangedNotSaved],mtWarning,[mbYes,mbNo,mbCancel],hcNoHelpTopic)) of
             mrYes: lUploadRequired:=true;
             mrNo:  ModalResult:=mrIgnore;
             else CanClose:=false;
             end;

     if ((ModalResult=mrOK) or lUploadRequired) then
        begin

        lTooSmallSpan:=((fModule1ChDataRec.InputType=itTHC) and (fSignalSpan<3.0)) or
                       ((fModule1ChDataRec.InputType=itRTD) and (fSignalSpan<10.0));
        if lTooSmallSpan then
           case (MessageDlg(LanguageStrings[msSpanTooSmall],mtWarning,[mbIgnore,mbCancel],hcNoHelpTopic)) of
                mrIgnore: lIgnoreSmallSpan:=true;
                else CanClose:=false;
                end;

        if not(lTooSmallSpan) or lIgnoreSmallSpan then
           begin
           pHartModule^.EvaluateRequiredDeviceDataStatusFlags;
           fExpectedTransferDataLength:=pHartModule^.GetModuleTransferDataLength(tdUpload);
           if (fExpectedTransferDataLength>0) then
              begin
              if (fSaveMode=smToDevice) then
                 begin
                 CommGauge.MaxValue:=fExpectedTransferDataLength;

                 Cursor:=crHourGlass;
                 Application.ProcessMessages;
                 pHartModule^.UploadRequiredDeviceData;

                 if not(pHartModule^.VerifyDeviceData) then
                    begin
                    MessageDlg(LanguageStrings[msDifferentResponse],mtError,[mbOk],hcNoHelpTopic);

                    EncodeModuleData;
                    SetAllDisplayValues;
                    pHartModule^.SynchronizeRequiredDeviceData;
                    UpdateAcceptButton;

                    ModalResult:=mrCancel;
                    end;

                 CommGauge.Progress:=0;
                 end
              else  // (fSaveMode=smToMemory)
                 begin
                 pHartModule^.SynchronizeActualDeviceData;
                 MessageDlg(LanguageStrings[msDataSavedToMemory],mtConfirmation,[mbOk],hcNoHelpTopic);
                 end;
              end;
           end;
        end;

     if not(CanClose) then
        begin
        PageControl1.Enabled:=true;
        UpdateAcceptButton;
        Close_Button.Enabled:=true;
        end;
     Cursor:=OriginalCursor;
     end;
end;

procedure TfmModuleData_73.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmModuleData_73.PageControl1Change(Sender: TObject);
begin
  if (fDialogMode=dmDisplay) then
     begin
     if (PageControl1.ActivePage=InputTabSheet) then
        HelpContext:=hcDisplayInput73
     else if (PageControl1.ActivePage=OutputTabSheet) then
             HelpContext:=hcDisplayOutput73
     else // (PageControl1.ActivePage=AlarmTabSheet)
             HelpContext:=hcDisplayAlarm73;
     end
  else // fDialogMode=dmConfig
     begin
     if (PageControl1.ActivePage=InputTabSheet) then
        HelpContext:=hcConfigInput73
     else if (PageControl1.ActivePage=OutputTabSheet) then
             HelpContext:=hcConfigOutput73
     else // (PageControl1.ActivePage=AlarmTabSheet)
             HelpContext:=hcConfigAlarm73;
     end;
end;

procedure TfmModuleData_73.PageControl1DrawTab(Control: TCustomTabControl;
  TabIndex: Integer; const Rect: TRect; Active: Boolean);
begin
  if (Active) then
     Control.Canvas.Font.Style:=Control.Canvas.Font.Style+[fsBold]
  else Control.Canvas.Font.Style:=Control.Canvas.Font.Style-[fsBold];
  Control.Canvas.TextOut(Rect.Left+5,Rect.Top+3,PageControl1.Pages[TabIndex].Caption);
end;

//****************************************************************************//

procedure TfmModuleData_73.InputTypeSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     case InputTypeSelector.ItemIndex of
          0: fModule1ChDataRec.InputType:=itTHC;
          1: fModule1ChDataRec.InputType:=itRTD;
          else fModule1ChDataRec.InputType:=itNone;
          end;

     fModule1ChDataRec.Linearisation:=tsOn;   // linearisation on
     fModule1ChDataRec.PVUnits:=0;            // PVUnits C
     fModule1ChDataRec.TransmitterFailureAlarmLevel:=taLow;
     fModule1ChDataRec.OpenCircuitAlarmLevel:=taHigh;
     fModule1ChDataRec.ShortCircuitAlarmLevel:=taOff;
     fModule1ChDataRec.TripOutputSetting:=ttOff;

     if fModule1ChDataRec.InputType=itTHC then
        begin
        fModule1ChDataRec.TypeNumber:=3; // TypeNumberFromHART(fDeviceType,CDef_x073_THC_Set.InputType); // Type K
        fModule1ChDataRec.ColdJunction:=tsOn;
        // RTD wiring number is not relevant
        fModule1ChDataRec.Point4mA:=0.0;
        fModule1ChDataRec.Point20mA:=250.0;
        fModule1ChDataRec.ColdJunctionFailureAlarmLevel:=taLow;
        end
     else // fModule1ChDataRec.InputType=itRTD
        begin
       fModule1ChDataRec.TypeNumber:=0; // TypeNumberFromHART(fDeviceType,CDef_x073_RTD_Set.InputType); // Pt100, 2-wire
        // Cold Junction Selection is not relevant
        fModule1ChDataRec.WiringNumber:=0; // 2-wire
        fModule1ChDataRec.Point4mA:=0.0;
        fModule1ChDataRec.Point20mA:=100.0;
        fModule1ChDataRec.ColdJunctionFailureAlarmLevel:=taOff;
        end;

     fProtectedPath:=true;

     SetInputTypeSpecificInputNumberSelectorLabels;
     SetInputNumberSelectorItem;

     // Disable unused CJC Buttons is not required here (Input Number cannot be mV)
     SetCJCorRTDWiringSelectorItems(false);

     SetLinearisationSelectorItems(true);
     
     SetInputTypeSpecificLastProcessUnitSelectorLabel;
     SetInputTypeSpecificProcessUnitSelectorItem;
     ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits));

     SetSignalRangeEditValues;
     SetSignalRangeEditUnits;
     SetSignalRangeEditLimits;

     SetTransmitterFailureAlarmLevelSelectorItems;
     SetOpenCircuitAlarmLevelItems;

     if fModule1ChDataRec.InputType=itTHC then
        begin
        SetColdJunctionFailureAlarmLevelItems(false);
        SetShortCircuitAlarmLevelItems(true);
        end
     else // fModule1ChDataRec.InputType=itRTD
        begin
        SetColdJunctionFailureAlarmLevelItems(true);
        SetShortCircuitAlarmLevelItems(false);
        end;

     SetEBDTripSelectorItems;

     fProtectedPath:=false;
     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.InputNumberSelectorClick(Sender: TObject);
var lTypeNumber, lLinearTypeNumber: word;
    lUpdateRangeUnits: boolean;
begin
  if not(fProtectedPath) then
     begin
     lUpdateRangeUnits:=false;

     lTypeNumber:=InputNumberSelector.ItemIndex;
     // note: this is always the "mV" or "resistor" selection
     lLinearTypeNumber:=InputNumberSelector.Items.Count-1;

     if fModule1ChDataRec.InputType=itTHC then
        begin
        if (lTypeNumber<>lLinearTypeNumber) then
           begin
           // This is the first input type change from itmV to itTHC
           if (fModule1ChDataRec.TypeNumber=lLinearTypeNumber) then
              begin
              fProtectedPath:=true;

              fModule1ChDataRec.ColdJunction:=tsOn;
              SetCJCorRTDWiringSelectorItems(false);

              fModule1ChDataRec.Linearisation:=tsOn;
              SetLinearisationSelectorItems(true);

              fModule1ChDataRec.PVUnits:=0; // set default selection to "C"
              SetInputTypeSpecificProcessUnitSelectorItem;
              ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits));

              UpdateSignalRangeEditValues(0.0,250.0);

              lUpdateRangeUnits:=true;

              fModule1ChDataRec.ColdJunctionFailureAlarmLevel:=taLow;
              SetColdJunctionFailureAlarmLevelItems(false);

              // Note: RTD Short Circuit Alarm is already off

              fProtectedPath:=false;
              end;
           end
        else // the new selection is "mV"
           begin
           fProtectedPath:=true;

           fModule1ChDataRec.ColdJunction:=tsOff;
           SetCJCorRTDWiringSelectorItems(true);

           fModule1ChDataRec.Linearisation:=tsOff;
           SetLinearisationSelectorItems(true);

           fModule1ChDataRec.PVUnits:=4; // set default selection to "mV"
           SetInputTypeSpecificProcessUnitSelectorItem;
           ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CLinUnits));

           UpdateSignalRangeEditValues(0.0,50.0);

           lUpdateRangeUnits:=true;

           fModule1ChDataRec.ColdJunctionFailureAlarmLevel:=taOff;
           SetColdJunctionFailureAlarmLevelItems(true);

           fProtectedPath:=false;
           end;
        end
     else // fModule1ChDataRec.InputType=itRTD
        begin
        if (lTypeNumber<>lLinearTypeNumber) then
           begin
           // This is the first input type change from "resistor" to "true RTD"
           // Note: for RTD the LastTypeNumber applies only to the selector component
           // but not to the "fModule1ChDataRec.TypeNumber", therefore check linearisation
           if (fModule1ChDataRec.Linearisation=tsOff) then
              begin
              fProtectedPath:=true;

              // Note: RTD wiring remain unchanged

              fModule1ChDataRec.Linearisation:=tsOn;
              SetLinearisationSelectorItems(true);

              fModule1ChDataRec.PVUnits:=0; // set default selection to "C"
              SetInputTypeSpecificProcessUnitSelectorItem;
              ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CNonLinUnits));

              UpdateSignalRangeEditValues(0.0,100.0);

              lUpdateRangeUnits:=true;

              // Note: CJC Failure Alarm is already off (by InputType change to itRTD)

              SetShortCircuitAlarmLevelItems(false); // Enable Short circuit alarm selection

              fProtectedPath:=false;
              end;
           end
        else // the new selection is "resistor"
           begin
           fProtectedPath:=true;

           // Note: RTD wiring remain unchanged

           fModule1ChDataRec.Linearisation:=tsOff;
           SetLinearisationSelectorItems(true);

           fModule1ChDataRec.PVUnits:=5; // set default selection to "Ohm"
           SetInputTypeSpecificProcessUnitSelectorItem;
           ProcessUnitSelector.SetButtonEnableArray(GetDynArray(CLinUnits));

           UpdateSignalRangeEditValues(0.0,400.0);

           lUpdateRangeUnits:=true;

           fModule1ChDataRec.ShortCircuitAlarmLevel:=taOff;
           SetShortCircuitAlarmLevelItems(true);

           fProtectedPath:=false;
           end;
        end;

     // EBD is and Trip configuration remain unaffected during Input Number change

     fModule1ChDataRec.TypeNumber:=lTypeNumber;

     if lUpdateRangeUnits then
        SetSignalRangeEditUnits;
     SetSignalRangeEditLimits;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.CJCSelectorClick(Sender: TObject);
var lColdJunction: TOnOffSet;
begin
  if not(fProtectedPath) then
     begin
     lColdJunction:=TOnOffSet(CJCSelector.ItemIndex);

     // Evaluate Unusual Cold Junction Compensation Selection and show adequate warning
     // However when once ignored, don't show this warning again until dialog is closed
     // This path can be achieved only when "InputType=itTHC" and not "mV" is selected
     if (lColdJunction=tsOff) and
        not(fUnusualCJCConfirmed) then
        fUnusualCJCConfirmed:=(MessageDlg(LanguageStrings[msWrongCJCSetting],mtWarning,[mbIgnore,mbCancel],hcNoHelpTopic)=mrIgnore);

     if (lColdJunction=tsOn) or fUnusualCJCConfirmed then
        begin
        fModule1ChDataRec.ColdJunction:=lColdJunction;

        fProtectedPath:=true;
        if (fModule1ChDataRec.ColdJunction=tsOff) then
           begin
           fModule1ChDataRec.ColdJunctionFailureAlarmLevel:=taOff;
           SetColdJunctionFailureAlarmLevelItems(true);
           end
        else // fModule1ChDataRec.ColdJunction=tsOn
           begin
           fModule1ChDataRec.ColdJunctionFailureAlarmLevel:=taLow;
           SetColdJunctionFailureAlarmLevelItems(false);
           end;

        // EBD is and Trip configuration remain unaffected during CJC change
        fProtectedPath:=false;

        DecodeModuleData;
        UpdateAcceptButton;
        end
     else // lColdJunction=tsOff and not(fUnusualCJCConfirmed)
        begin
        fProtectedPath:=true;
        CJCSelector.ItemIndex:=integer(fModule1ChDataRec.ColdJunction);
        fProtectedPath:=false;
        end;
     end;
end;

procedure TfmModuleData_73.RTDWiringSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.WiringNumber:=RTDWiringSelector.ItemIndex;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.LinearisationSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.Linearisation:=TOnOffSet(LinearisationSelector.ItemIndex);

     // Evaluate Unusual Linearisation Selection and show adequate warning
     if (fModule1ChDataRec.Linearisation=tsOff) and
        not(fUnusualLinConfirmed) and
        (((fModule1ChDataRec.InputType=itTHC) and (fModule1ChDataRec.TypeNumber<(InputNumberSelector.Items.Count-1))) or
         ((fModule1ChDataRec.InputType=itRTD) and (fModule1ChDataRec.TypeNumber<(InputNumberSelector.Items.Count-1)))) then
        fUnusualLinConfirmed:=(MessageDlg(LanguageStrings[msWrongLinSetting],mtWarning,[mbIgnore,mbCancel],hcNoHelpTopic)=mrIgnore);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.ProcessUnitSelectorClick(Sender: TObject);
var lPVUnits: integer;
begin
  if not(fProtectedPath) then
     begin
     lPVUnits:=ProcessUnitSelector.ItemIndex;
     if not((fModule1ChDataRec.InputType=itRTD) and (lPVUnits=4)) then // not Ohm selection
        fModule1ChDataRec.PVUnits:=lPVUnits
     else fModule1ChDataRec.PVUnits:=5; //

     SetSignalRangeEditUnits;
     SetSignalRangeEditLimits;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Point4mA_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.Point4mA:=Point4mA_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Point4mA_EditRangeValidationError(Sender: TObject);
begin
  Point4mA_Edit.SetValueToValidRange;
end;

procedure TfmModuleData_73.Point20mA_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.Point20mA:=Point20mA_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Point20mA_EditRangeValidationError(Sender: TObject);
begin
  Point20mA_Edit.SetValueToValidRange;
end;

procedure TfmModuleData_73.FrequencySelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.FilterFrequencySetting:=TFilterFrequency(FrequencySelector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.DampingConstant_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.DampingTimeConstant:=DampingConstant_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.DampingConstant_EditRangeValidationError(Sender: TObject);
begin
  DampingConstant_Edit.SetValueToValidRange;
end;

procedure TfmModuleData_73.SmoothingConstant_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.SmoothingTimeConstant:=SmoothingConstant_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.SmoothingConstant_EditRangeValidationError(Sender: TObject);
begin
  SmoothingConstant_Edit.SetValueToValidRange;
end;

procedure TfmModuleData_73.NE43OnButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.NamurOption:=nsOn;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.NE43OffButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.NamurOption:=nsOff;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.ResponseTimeSelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.ResponseSpeed:=TResponseSpeed(ResponseTimeSelector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.TX_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.TransmitterFailureAlarmLevel:=TAlarmLevelSet(TX_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.OC_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.OpenCircuitAlarmLevel:=TAlarmLevelSet(OC_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.CJC_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.ColdJunctionFailureAlarmLevel:=TAlarmLevelSet(CJC_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.SC_Alarm_SelectorClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.ShortCircuitAlarmLevel:=TAlarmLevelSet(SC_Alarm_Selector.ItemIndex);

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.TripOnButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.TripOutputSetting:=ttTrip;

     fProtectedPath:=true;
     SetEBDTripSelectorItems;
     fProtectedPath:=false;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.EBDOnButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.TripOutputSetting:=ttEBD;

     fProtectedPath:=true;
     SetEBDTripSelectorItems;
     fProtectedPath:=false;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.EBDOffButtonClick(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.TripOutputSetting:=ttOff;

     fProtectedPath:=true;
     SetEBDTripSelectorItems;
     fProtectedPath:=false;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Threshold_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.TripThreshold:=Threshold_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Threshold_EditRangeValidationError(Sender: TObject);
begin
  Threshold_Edit.SetValueToValidRange;
end;

procedure TfmModuleData_73.Hysteresis_EditChange(Sender: TObject);
begin
  if not(fProtectedPath) then
     begin
     fModule1ChDataRec.TripHysteresis:=Hysteresis_Edit.Value;

     DecodeModuleData;
     UpdateAcceptButton;
     end;
end;

procedure TfmModuleData_73.Hysteresis_EditRangeValidationError(Sender: TObject);
begin
  Hysteresis_Edit.SetValueToValidRange;
end;

end.
