unit Monitor;

interface

uses Windows, ExtCtrls, Controls, StdCtrls, Buttons, Classes, Gauges, Forms,
     EditControls, DataTypes, HartDataTypes, HartModules;

type
  TfmMonitor = class(TForm)
    MonitorInfoBox: TGroupBox;
    CurrentGauge: TGauge;
    ProcessDisplay: TLabelDisplay;
    CurrentDisplay: TLabelDisplay;
    FirstAddDisplay: TLabelDisplay;
    SecondAddDisplay: TLabelDisplay;
    ChannelSelectionBox: TGroupBox;
    ChannelLabel: TLabel;
    ChannelSelection: TComboBox;
    CommGauge: TGauge;
    Close_Button: TBitBtn;
    MonitorTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure ChannelSelectionChange(Sender: TObject);
    procedure MonitorTimerTimer(Sender: TObject);
  private
    { Private declarations }
    fDeviceType: TDeviceType;
    pHartModule: PBasicHartModule;
    fTransferActive,
    fTHCSensor,
    fCJCPresent,
    fFullScale: boolean;
    fMonitorInterval: integer;
    procedure IndicateReceivedChar;
    procedure CommonPrefixMonitorLook;
    procedure CommonMonitorLook;
    procedure MonitorLook_x073;
    procedure MonitorLook_x573;
    procedure MonitorChannelLook_x576(Number: integer);
    procedure MonitorLook_x576;
    procedure MonitorLook_x532;
    procedure MonitorLook_1003;
    procedure CommonSuffixMonitorLook;

    procedure RefreshMonitor(Data: THDynamicVariables);
    procedure RefreshMonitor_x073(Data: THDynamicVariables);
    // procedure RefreshMonitor_x573;
    // procedure RefreshMonitor_x576;
    procedure RefreshMonitor_x532;
    procedure RefreshMonitor_1003;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
  end;

var
  fmMonitor: TfmMonitor;

implementation

{$R *.dfm}

uses Graphics, ProcessUtils, StringUtils, DataInstances, LabelNumbers, HartDataConstants,
     HelpContextNumbers, ConversionProcs;

const
  CMonitorInterval = 1000;
  CMonx576Interval = 2000;

procedure TfmMonitor.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(10);
     end
  else WaitMilliseconds(100);
end;

procedure TfmMonitor.CommonPrefixMonitorLook;
begin
  // Captions and Displays

  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlMonitor]);

  MonitorInfoBox.Caption:=LanguageStrings[flDynamicVariables];

  ProcessDisplay.Display:='';
  ProcessDisplay.UnitText:='';

  CurrentDisplay.Display:='';
  CurrentDisplay.UnitText:='';

  FirstAddDisplay.Display:='';
  FirstAddDisplay.UnitText:='';

  SecondAddDisplay.Display:='';
  SecondAddDisplay.UnitText:='';

  ChannelSelectionBox.Visible:=false;

  Close_Button.Caption:=LanguageStrings[blClose];

  // Dimensions

  Self.Width:=275;

  MonitorInfoBox.Top:=15;
  MonitorInfoBox.Width:=240;

  ChannelSelectionBox.Width:=240;
  ChannelSelectionBox.Height:=45;
  ChannelSelectionBox.Visible:=false;

  CommGauge.Width:=240;
end;

procedure TfmMonitor.CommonMonitorLook;
begin
  ProcessDisplay.LabelCaption:=LanguageStrings[flProcessValue];

  CurrentDisplay.LabelCaption:=LanguageStrings[flOutputCurrent];
  CurrentDisplay.UnitText:=LanguageStrings[flmiliAmp];
  CurrentDisplay.RealPrecision:=3;
  CurrentDisplay.SetDisplay(0.0);

  FirstAddDisplay.LabelCaption:=LanguageStrings[flColdJunctionComp];
end;

procedure TfmMonitor.MonitorLook_x073;
begin
  // DisplayCaptions

  ProcessDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
  ProcessDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));
  ProcessDisplay.SetDisplay(0.0);

  if (fTHCSensor and fCJCPresent) then
     begin
     FirstAddDisplay.UnitText:=ProcessDisplay.UnitText;
     FirstAddDisplay.RealPrecision:=ProcessDisplay.RealPrecision;
     FirstAddDisplay.SetDisplay(0.0);
     end
  else // not(fTHCSensor and fCJCPresent)
     begin
     FirstAddDisplay.UnitText:='';
     FirstAddDisplay.Enabled:=false;
     FirstAddDisplay.DisplayColor:=clBtnFace;
     FirstAddDisplay.DisplayFont.Color:=clBtnFace;
     end;

  SecondAddDisplay.Visible:=false;

  ChannelSelectionBox.Visible:=false;

  CommGauge.MaxValue:=22;

  // Dimensions

  MonitorInfoBox.Height:=60 + 3*30;
end;

procedure TfmMonitor.MonitorLook_x573;
begin
  // DisplayCaptions

  ProcessDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
  ProcessDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));
  ProcessDisplay.SetDisplay(0.0);

  if (fTHCSensor and fCJCPresent) then
     begin
     FirstAddDisplay.UnitText:=ProcessDisplay.UnitText;
     FirstAddDisplay.RealPrecision:=ProcessDisplay.RealPrecision;
     FirstAddDisplay.SetDisplay(0.0);
     end
  else // not(fTHCSensor and fCJCPresent)
     begin
     FirstAddDisplay.UnitText:='';
     FirstAddDisplay.Enabled:=false;
     FirstAddDisplay.DisplayColor:=clBtnFace;
     FirstAddDisplay.DisplayFont.Color:=clBtnFace;
     end;

  SecondAddDisplay.Visible:=false;

  ChannelSelectionBox.Visible:=false;

  CommGauge.MaxValue:=22;

  // Dimensions

  MonitorInfoBox.Height:=60 + 3*30;
end;

procedure TfmMonitor.MonitorChannelLook_x576(Number: integer);
begin
  // DisplayCaptions

  ProcessDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[Number].PVUnits)];
  ProcessDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(THartModule_x576(pHartModule^).InputParameters[Number].PVUnits));
  ProcessDisplay.SetDisplay(0.0);

  if (fTHCSensor and fCJCPresent) then
     begin
     FirstAddDisplay.UnitText:=ProcessDisplay.UnitText;
     FirstAddDisplay.RealPrecision:=ProcessDisplay.RealPrecision;
     FirstAddDisplay.SetDisplay(0.0);
     end
  else // not(fTHCSensor and fCJCPresent)
     begin
     FirstAddDisplay.UnitText:='';
     FirstAddDisplay.Enabled:=false;
     FirstAddDisplay.DisplayColor:=clBtnFace;
     FirstAddDisplay.DisplayFont.Color:=clBtnFace;
     end;
end;

procedure TfmMonitor.MonitorLook_x576;
var lx576Channel: integer;
begin
  // DisplayCaptions

  if (THartModule_x576(pHartModule^).ChannelNumber in [0,1]) then
     lx576Channel:=THartModule_x576(pHartModule^).ChannelNumber
  else lx576Channel:=0;

  MonitorChannelLook_x576(lx576Channel);

  SecondAddDisplay.Display:='';
  SecondAddDisplay.Visible:=false;

  ChannelLabel.Caption:=LanguageStrings[flChannel];
  ChannelSelectionBox.Visible:=true;

  ChannelSelection.ItemIndex:=lx576Channel;

  CommGauge.MaxValue:=33;

  // Dimensions

  MonitorInfoBox.Height:=60 + 3*30;
  ChannelSelectionBox.Top:=MonitorInfoBox.Top+MonitorInfoBox.Height+10;
end;

procedure TfmMonitor.MonitorLook_x532;
begin
  // DisplayCaptions

  ProcessDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
  ProcessDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));
  ProcessDisplay.SetDisplay(0.0);

  FirstAddDisplay.Visible:=false;
  SecondAddDisplay.Visible:=false;

  ChannelSelectionBox.Visible:=false;

  CommGauge.MaxValue:=22;

  // Dimensions
  MonitorInfoBox.Height:=60 + 2*30;
end;

procedure TfmMonitor.MonitorLook_1003;
begin
  // DisplayCaptions

  ProcessDisplay.UnitText:=LanguageStrings[flCelsius+PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits)];
  ProcessDisplay.RealPrecision:=GetPVUnitPrecision(PVUnitsFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.PVUnits));
  ProcessDisplay.SetDisplay(0.0);

  FirstAddDisplay.Visible:=false;
  SecondAddDisplay.Visible:=false;

  ChannelSelectionBox.Visible:=false;

  CommGauge.MaxValue:=22;

  // Dimensions
  MonitorInfoBox.Height:=60 + 2*30;
end;

procedure TfmMonitor.CommonSuffixMonitorLook;
begin
  if not(ChannelSelectionBox.Visible) then
     CommGauge.Top:=MonitorInfoBox.Top+MonitorInfoBox.Height+15
  else CommGauge.Top:=ChannelSelectionBox.Top+ChannelSelectionBox.Height+15;

  CommGauge.Progress:=0;

  Close_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Left:=(Self.Width-Close_Button.Width) DIV 2;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmMonitor.RefreshMonitor(Data: THDynamicVariables);
begin
  if not(fFullScale) then
     CurrentGauge.Progress:=round((Data.LoopCurrent-4.0)*6.25)
  else CurrentGauge.Progress:=round(Data.LoopCurrent*5.0);
  ProcessDisplay.SetDisplay(Data.PrimaryVariable);
  CurrentDisplay.SetDisplay(Data.LoopCurrent);
end;

procedure TfmMonitor.RefreshMonitor_x073(Data: THDynamicVariables);
begin
  if fTHCSensor and fCJCPresent then
     FirstAddDisplay.SetDisplay(Data.SecondaryVariable);
end;

{procedure TfmMonitor.RefreshMonitor_x573;
begin
  //
end;}

{procedure TfmMonitor.RefreshMonitor_x576;
begin
  //
end;}

procedure TfmMonitor.RefreshMonitor_x532;
begin
  //
end;

procedure TfmMonitor.RefreshMonitor_1003;
begin
  //
end;

procedure TfmMonitor.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmMonitor.FormCreate(Sender: TObject);
begin
  fDeviceType:=dtNone;
  pHartModule:=nil;

  fTransferActive:=false;
  fTHCSensor:=false;
  fCJCPresent:=false;
  fFullScale:=false;

  MonitorTimer.Enabled:=false;

  HelpContext:=hcMonitor;
end;

procedure TfmMonitor.FormShow(Sender: TObject);
var lActualChannel: byte;
begin
  CommonPrefixMonitorLook;

  if (pHartModule<>nil) then
     begin
     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);
     if (fDeviceType<>dtNone) then
        begin
        if (fDeviceType=dtx576) then
           lActualChannel:=THartModule_x576(pHartModule^).ChannelNumber
        else lActualChannel:=0;

        CommonMonitorLook;

        fTHCSensor:=(fDeviceType in CTemperatureConverter) and
                    ((fDeviceType in CSingleChannel) and
                     (InputTypeFromHART(fDeviceType,THartModule_x073(pHartModule^).x073ParameterSet.InputType)=itTHC)) or
                    ((fDeviceType in CDualChannel) and
                     (InputTypeFromHART(fDeviceType,THartModule_x576(pHartModule^).InputParameters[lActualChannel].SensorType)=itTHC));

        fCJCPresent:=(fDeviceType in CTemperatureConverter) and
                    ((fDeviceType in CSingleChannel) and
                     (THartModule_x073(pHartModule^).x073ParameterSet.ColdJunctionCompensationOnOff=1)) or
                    ((fDeviceType in CDualChannel) and
                     (THartModule_x576(pHartModule^).InputParameters[lActualChannel].CJC_Option=1));

        if (fDeviceType in CFrequencyConverter) and
           (THartModule_Freq(pHartModule^).FullScale>0) then
           fFullScale:=true
        else fFullScale:=false;


        case fDeviceType of
             dt507x,
             dtx073,
             dt5074XK,
             dt4073XK,
             dt4075,
             dtx575: MonitorLook_x073;
             dtx573: MonitorLook_x573;
             dtx576: MonitorLook_x576;
             dtx532: MonitorLook_x532;
             dt1003: MonitorLook_1003;
             end;

        MonitorTimer.Interval:=100;
        MonitorTimer.Enabled:=true;
        end;
     end;

  if (MonitorTimer.Enabled) then
     CommonSuffixMonitorLook
  else
     begin
     Close;
     end;
end;

procedure TfmMonitor.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not(fTransferActive);
end;

procedure TfmMonitor.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmMonitor.ChannelSelectionChange(Sender: TObject);
var lActualChannel: byte;
begin
  if (fDeviceType=dtx576) and
     (ChannelSelection.ItemIndex in [0,1]) then
     begin
     lActualChannel:=ChannelSelection.ItemIndex;
     THartModule_x576(pHartModule^).ChannelNumber:=lActualChannel;

     fTHCSensor:=(InputTypeFromHART(fDeviceType,THartModule_x576(pHartModule^).InputParameters[lActualChannel].SensorType)=itTHC);

     fCJCPresent:=(THartModule_x576(pHartModule^).InputParameters[lActualChannel].CJC_Option=1);

     MonitorChannelLook_x576(ChannelSelection.ItemIndex);
     ChannelSelectionBox.Top:=MonitorInfoBox.Top+MonitorInfoBox.Height+10;
     end;
end;

procedure TfmMonitor.MonitorTimerTimer(Sender: TObject);
var MonitorServiceDuration: dword;
    MonData: THDynamicVariables;
begin
  MonitorServiceDuration:=GetTickCount;
  fTransferActive:=true;
  MonitorTimer.Enabled:=false;

  if (pHartModule<>nil) and
     ((fDeviceType<>dtx576) and (pHartModule^.GeTHDynamicVariables(MonData)))  or
     ((fDeviceType=dtx576) and (THartModule_x576(pHartModule^).GeTHDynamicVariables(MonData))) then
     begin

     RefreshMonitor(MonData);

     case fDeviceType of
          dt507x,
          dtx073,
          dt5074XK,
          dt4073XK,
          dt4075,
          dtx575,
          dtx573,
          dtx576: RefreshMonitor_x073(MonData);
          dtx532: RefreshMonitor_x532;
          dt1003: RefreshMonitor_1003;
          end;

     WaitMilliseconds(50);
     CommGauge.Progress:=0;
     end;

  MonitorServiceDuration:=GetTickCount-MonitorServiceDuration;

  if (fDeviceType<>dtx576) then
     fMonitorInterval:=CMonitorInterval-integer(MonitorServiceDuration)
  else fMonitorInterval:=CMonx576Interval-integer(MonitorServiceDuration);
  while (fMonitorInterval<50) do
        fMonitorInterval:=fMonitorInterval+CMonitorInterval;

  MonitorTimer.Interval:=fMonitorInterval;
  MonitorTimer.Enabled:=true;
  fTransferActive:=false;
end;

end.
