unit Password;

interface

uses Windows, StdCtrls, Buttons, Controls, Classes, ExtCtrls, Forms, SysUtils;

type
  TPasswordMode = (pmCheck,pmChange);
  TChangePhase  = (cpOld,cpNew,cpRepeat);

  TfmPassword = class(TForm)
    Bevel: TPanel;
    PasswordLabel: TLabel;
    PasswordEdit: TEdit;
    OK_Button: TBitBtn;
    Close_Button: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure PasswordEditChange(Sender: TObject);
    procedure OK_ButtonClick(Sender: TObject);
    procedure Close_ButtonClick(Sender: TObject);
  private
    { Private declarations }
    PasswordMode: TPasswordMode;
    ChangePhase: TChangePhase;
    OriginalPassword,
    ChangedPassword: string;
  public
    { Public declarations }
    procedure SetMode(Value: TPasswordMode);
    procedure SetPassword(Value: string);
    function GetPassword: string;
  end;

const
  CDefaultPassword = 'MTL';

var
  fmPassword: TfmPassword;

implementation

{$R *.dfm}

uses StringUtils, DataInstances, LabelNumbers, HelpContextNumbers;

const
  CMasterPassword  = 'Immanuel';

procedure TfmPassword.SetMode(Value: TPasswordMode);
begin
  if Value in [pmCheck,pmChange] then PasswordMode:=Value;
end;

procedure TfmPassword.SetPassword(Value: string);
begin
  OriginalPassword:=UpperCase(Value);
end;

function TfmPassword.GetPassword: string;
begin
  if (PasswordMode=pmChange) then
     Result:=ChangedPassword
  else Result:=OriginalPassword;
end;

procedure TfmPassword.FormCreate(Sender: TObject);
begin
  PasswordMode:=pmCheck;
  OriginalPassword:=CDefaultPassword;

  HelpContext:=hcPassword;

  OK_Button.Caption:=LanguageStrings[blSetting];
  Close_Button.Caption:=LanguageStrings[blClose];

  OK_Button.Top:=Bevel.Top+Bevel.Height+20;
  OK_Button.Left:=Bevel.Left;
  Close_Button.Top:=OK_Button.Top;
  Close_Button.Left:=Bevel.Left+Bevel.Width-Close_Button.Width;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmPassword.FormShow(Sender: TObject);
begin
  case PasswordMode of
       pmCheck:  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[flVerifyPassword]);
       pmChange: begin
                 Caption:=' '+DeleteAcceleratorKey(LanguageStrings[flChangePassword]);
                 ChangePhase:=cpOld;
                 end;
       end;
  PasswordLabel.Caption:=LanguageStrings[flEnterPassword];
  PasswordEdit.Text:='';

  OK_Button.Enabled:=false;
end;

procedure TfmPassword.FormActivate(Sender: TObject);
begin
  // This curious sequence: first set focus to different control (button), then call ProcessMessages
  // and finally set focus to the required control was found by experience as necessary.
  // Otherwise the required control got the focus not at first attempt (but at second attempt yes ?).
  FocusControl(Close_Button);
  Application.ProcessMessages;
  FocusControl(PasswordEdit);
end;

procedure TfmPassword.PasswordEditChange(Sender: TObject);
begin
  OK_Button.Enabled:=(System.Length(PasswordEdit.Text)>0) or (PasswordMode=pmChange);
end;

procedure TfmPassword.OK_ButtonClick(Sender: TObject);
var NewInputPassword: string;
begin
  NewInputPassword:=UpperCase(PasswordEdit.Text);
  if PasswordMode=pmCheck then
     begin
     if (NewInputPassword=UpperCase(CMasterPassword)) then ModalResult:=mrIgnore
     else if (NewInputPassword<>OriginalPassword) then ModalResult:=mrCancel;
     end
  else
     begin
     case ChangePhase of
       cpOld: begin
              if (NewInputPassword<>UpperCase(CMasterPassword)) then
                 begin
                 if (NewInputPassword<>OriginalPassword) then ModalResult:=mrCancel
                 else
                    begin
                    PasswordLabel.Caption:=LanguageStrings[flEnterNewPassword];
                    PasswordEdit.Text:='';
                    ChangePhase:=cpNew;
                    ModalResult:=mrNone;
                    end;
                 end
              else ModalResult:=mrIgnore;
              end;
       cpNew: begin
              if (NewInputPassword<>'') then
                 begin
                 ChangedPassword:=NewInputPassword;
                 PasswordLabel.Caption:=LanguageStrings[flRepeatNewPassword];
                 PasswordEdit.Text:='';
                 ChangePhase:=cpRepeat;
                 ModalResult:=mrNone;
                 end
              else ModalResult:=mrYes;
              end;
       cpRepeat: begin
                 if (NewInputPassword<>ChangedPassword) then ModalResult:=mrNo;
                 end;
       end;
     end;
end;

procedure TfmPassword.Close_ButtonClick(Sender: TObject);
begin
  if (PasswordMode=pmChange) and not(ChangePhase<cpNew) then
     ModalResult:=mrYes
  else ModalResult:=mrCancel;
end;

end.
