unit Pcs45Main;

interface

uses Windows, Menus, ImgList, Controls, HartDevices, Dialogs, ComCtrls, Classes,
     ExtCtrls, Forms, SysUtils, HTMLHelpViewer, DataTypes, HartModules, StdCtrls;

type
  TfmPcs45Main = class(TForm)
    MainMenu1: TMainMenu;

    MFile: TMenuItem;
    MConnection: TMenuItem;
    MDisplay: TMenuItem;
    MDiagnostics: TMenuItem;
    MDeviceData: TMenuItem;
    MOptions: TMenuItem;
    MHelp: TMenuItem;

    FNew: TMenuItem;
    FOpen: TMenuItem;
    FSave: TMenuItem;
    FSaveAs: TMenuItem;
    F1: TMenuItem;
    FImport: TMenuItem;
    FExport: TMenuItem;
    FPrint: TMenuItem;
    FExit: TMenuItem;

    CEstablish: TMenuItem;
    CDisconnect: TMenuItem;

    DBasicData: TMenuItem;
    DModuleData: TMenuItem;
    GSetCurrentOutput: TMenuItem;
    GetSetDigitalOutputs: TMenuItem;
    G1: TMenuItem;
    GStatus: TMenuItem;
    GMonitor: TMenuItem;
    GDataLogger: TMenuItem;
    G2: TMenuItem;
    GReset: TMenuItem;
    G3: TMenuItem;
    GInspection: TMenuItem;

    CCalibration: TMenuItem;
    C1: TMenuItem;
    CBasicData: TMenuItem;
    CModuleData: TMenuItem;
    C2: TMenuItem;
    CChangeDevice: TMenuItem;
    CSaveToDevice: TMenuItem;
    CVerifyDevice: TMenuItem;
    C3: TMenuItem;
    CDefaultConfig: TMenuItem;
    C4: TMenuItem;
    CHardwareConfig: TMenuItem;

    OPassword: TMenuItem;
    OLanguage: TMenuItem;
    OPortSelection: TMenuItem;
    OErrorIndication: TMenuItem;
    OCalculator: TMenuItem;
    O1: TMenuItem;
    OResetPassword: TMenuItem;

    HContent: TMenuItem;
    HAbout: TMenuItem;

    StatusBar1: TStatusBar;
    Image1: TImage;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    PrintDialog1: TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    HartMaster1: THartMaster;
    ImageList1: TImageList;
    F3: TMenuItem;
    PopupMenu1: TPopupMenu;
    Hugo11: TMenuItem;
    Hugo21: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);

    procedure HartMaster1AnswerSimulationRequest(HartCommand: THartCommand; var HartAnswer: THartAnswer);

    procedure FNewClick(Sender: TObject);
    procedure FOpenClick(Sender: TObject);
    procedure FSaveClick(Sender: TObject);
    procedure FSaveAsClick(Sender: TObject);
    procedure FImportClick(Sender: TObject);
    procedure FExportClick(Sender: TObject);
    procedure FPrintClick(Sender: TObject);
    procedure FExitClick(Sender: TObject);

    procedure CEstablishClick(Sender: TObject);
    procedure CDisconnectClick(Sender: TObject);

    procedure DBasicDataClick(Sender: TObject);
    procedure DModuleDataClick(Sender: TObject);

    procedure GSetCurrentOutputClick(Sender: TObject);
    procedure GetSetDigitalOutputsClick(Sender: TObject);
    procedure GStatusClick(Sender: TObject);
    procedure GMonitorClick(Sender: TObject);
    procedure GDataLoggerClick(Sender: TObject);
    procedure GResetClick(Sender: TObject);
    procedure GInspectionClick(Sender: TObject);

    procedure CCalibrationClick(Sender: TObject);
    procedure CBasicDataClick(Sender: TObject);
    procedure CModuleDataClick(Sender: TObject);
    procedure CChangeDeviceClick(Sender: TObject);
    procedure CSaveToDeviceClick(Sender: TObject);
    procedure CVerifyDeviceClick(Sender: TObject);
    procedure CDefaultConfigClick(Sender: TObject);
    procedure CHardwareConfigClick(Sender: TObject);

    procedure OPasswordClick(Sender: TObject);
    procedure OLanguageClick(Sender: TObject);
    procedure OPortSelectionClick(Sender: TObject);
    procedure OErrorIndicationClick(Sender: TObject);
    procedure OCalculatorClick(Sender: TObject);
    procedure OResetPasswordClick(Sender: TObject);

    procedure HContentClick(Sender: TObject);
    procedure HAboutClick(Sender: TObject);
    procedure SaveDialog1TypeChange(Sender: TObject);
  private
    { Private declarations }
    fFormActivated: boolean;
    MTLHartModule: TBasicHartModule;

    ActualFileName: TFileName;
    ExcelAvailable: boolean;

    procedure TryToLokalize;
    procedure SelAllCaptions;
    procedure SetAllHelpContexts;
    procedure SetAllHints;

    procedure SetBasicMenu;

    procedure UpdateMenus;
    procedure UpdateStatusBar;

    function CheckReleasedPassword: boolean;

    procedure ArrangeCommunicationPort;
    procedure IndicateReceivedChar;
    procedure ShowDemoOperationWarning;
    function ShowAndAcceptISO9002Warning: boolean;

    function VerifyUnintentionalOverwriteAttempt: boolean;

    function VerifyDeviceDataAvailable: boolean;

    function GetActualConnectedDeviceType: TDeviceType;
    function VerifyDeviceTypeConnected: boolean;
    function VerifyDeviceDataAvailableAndDeviceType(var SM: TSaveMode): boolean;

    procedure SaveDeviceDataSet(Name: TFileName);
  public
    { Public declarations }
  end;

var
  fmPcs45Main: TfmPcs45Main;

implementation

{$R *.dfm}

uses ArraysAndGrids, ProcessUtils, StringUtils, LnCRCFileStream,
     DynLoadFTD2XX, SerialFTDevices, SerialComDevices,
     About, BasicData, ChangeDevice, DataLogger, DigitalDiagnostics, ErrorIndication, HardwareConfig,
     Inspection, Language, ModuleData_32, ModuleData_73, ModuleData_76, Monitor, NewDevice,
     Password, PortSelection, SetAndCalibrate, {SheetNames,} Status, Transfer, DataInstances,
     DataConstants, LabelNumbers, HelpContextNumbers, HartDataTypes, HartDataConstants,
     DemoOperation, ConversionProcs, ImportExportProcs, LanguageStringProcs, SystemProcs,
     Calculator;

// Private procedures and functions

procedure TfmPcs45Main.TryToLokalize;
var SFolder,LangSuffix,DefSuffix: string;
    I,LanguageCount: integer;
begin
  SFolder:=OwnPath+OwnName;
  LanguageCount:=System.Length(LanguageInfos);

  LangSuffix:='';
  for I:=0 to LanguageCount-1 do
      begin
      if (LanguageInfos[I].LangNumber=CFG45.Language) then
         begin
         LangSuffix:=GetLocaleFileSuffix(LanguageInfos[I].FileName);
         break;
         end;
      end;

  DefSuffix:='';
  for I:=0 to LanguageCount-1 do
      begin
      if (LanguageInfos[I].LangNumber=CDefaultLanguage) then
         begin
         DefSuffix:=GetLocaleFileSuffix(LanguageInfos[I].FileName);
         break;
         end;
      end;

  if (LangSuffix<>'') and FileExists(SFolder+LangSuffix+'.dll') then
     begin

     LoadNewResourceModule(SFolder+LangSuffix+'.dll');

     ReadLanguageStrings(LanguageStrings);
     ReadLanguageSystemStrings;

     if FileExists(SFolder+LangSuffix+'.chm') then
        Application.HelpFile:=SFolder+LangSuffix+'.chm'
     else if (DefSuffix<>'') and FileExists(SFolder+DefSuffix+'.chm') then
             Application.HelpFile:=SFolder+DefSuffix+'.chm';
     end
  else if (DefSuffix<>'') and FileExists(SFolder+DefSuffix+'.dll') then
          begin
          CFG45.Language:=CDefaultLanguage;
          SaveLanguageToRegistry(CFG45.Language);
          LoadNewResourceModule(SFolder+DefSuffix+'.dll');
          if FileExists(SFolder+DefSuffix+'.chm') then
             Application.HelpFile:=SFolder+DefSuffix+'.chm';
          end
       else
          begin
          MessageDlg('No any '+OwnName+' language module is available on your system!'+^M+
                     'The application will be closed now. Please check your system'+^M+
                     'and if necessary, install the application '+OwnName+' again.',mtError,[mbCancel],hcNoHelpTopic);
          Close;
          end;
end;

procedure TfmPcs45Main.SelAllCaptions;
begin
// Application caption
  Caption:=' '+LanguageStrings[flToolCaption]+' '+CPcs45Version+LanguageStrings[flCopyright];

// Main menu
  MFile.Caption:=LanguageStrings[mlFile];
  MConnection.Caption:=LanguageStrings[mlConnection];
  MDisplay.Caption:=LanguageStrings[mlDisplay];
  MDiagnostics.Caption:=LanguageStrings[mlDiagnostics];
  MDeviceData.Caption:=LanguageStrings[mlDeviceData];
  MOptions.Caption:=LanguageStrings[mlOptions];
  MHelp.Caption:=LanguageStrings[mlHelp];

// Submenu file

  FNew.Caption:=LanguageStrings[mlNewDataSet];
  FOpen.Caption:=LanguageStrings[mlOpenDataSet];
  FSave.Caption:=LanguageStrings[mlSaveDataSet];
  FSaveAs.Caption:=LanguageStrings[mlSaveDataSetAs];
  FImport.Caption:=LanguageStrings[mlImportFromFile];
  FExport.Caption:=LanguageStrings[mlExportToFile];
  FPrint.Caption:=LanguageStrings[mlPrint];
  FExit.Caption:=LanguageStrings[mlExit];

// Submenu connection

  CEstablish.Caption:=LanguageStrings[mlEstablish];
  CDisconnect.Caption:=LanguageStrings[mlDisconnect];

// Submenu display device data

  DBasicData.Caption:=LanguageStrings[mlBasicConfig];
  DModuleData.Caption:=LanguageStrings[mlModuleConfig];

// Submenu diagnostics

  GSetCurrentOutput.Caption:=LanguageStrings[mlSetCurrentOutput];
  GetSetDigitalOutputs.Caption:=LanguageStrings[mlSetDigitalOutputs];
  GStatus.Caption:=LanguageStrings[mlStatus];
  GMonitor.Caption:=LanguageStrings[mlMonitor];
  GDataLogger.Caption:=LanguageStrings[mlDataLogger];
  GReset.Caption:=LanguageStrings[mlResetDevice];
  GInspection.Caption:=LanguageStrings[mlInspection];

// Submenu device data configuration - completion to submenu device data

  CCalibration.Caption:=LanguageStrings[mlCalibrateOutput];
  CBasicData.Caption:=LanguageStrings[mlBasicConfig];
  CModuleData.Caption:=LanguageStrings[mlModuleConfig];
  CChangeDevice.Caption:=LanguageStrings[mlChangeDevice];
  CSaveToDevice.Caption:=LanguageStrings[mlSaveToDevice];
  CVerifyDevice.Caption:=LanguageStrings[mlVerifyDevice];
  CDefaultConfig.Caption:=LanguageStrings[mlDefaultConfig];
  CHardwareConfig.Caption:=LanguageStrings[mlHardwareConfig];

// Submenu options

  OPassword.Caption:=LanguageStrings[mlPassword];
  OLanguage.Caption:=LanguageStrings[mlLanguage];
  OPortSelection.Caption:=LanguageStrings[mlPortSelection];
  OErrorIndication.Caption:=LanguageStrings[mlErrorIndication];
  OResetPassword.Caption:=LanguageStrings[mlResetPassword];

// Submenu help
  HContent.Caption:=LanguageStrings[mlContents];
  HAbout.Caption:=LanguageStrings[mlAbout];

// Statusbar
  StatusBar1.Panels[0].Text:=LanguageStrings[slHelpF1];

  MOptions.HelpContext:=hcOptions;
end;

procedure TfmPcs45Main.SetAllHelpContexts;
begin
  FNew.HelpContext:=hcNewDataSet;
  FOpen.HelpContext:=hcOpen;
  FSave.HelpContext:=hcSave;
  FSaveAs.HelpContext:=hcSaveAs;
  FImport.HelpContext:=hcImport;
  FExport.HelpContext:=hcExport;
  FPrint.HelpContext:=hcPrint;
  FExit.HelpContext:=hcExitProgram;

  CEstablish.HelpContext:=hcEstablish;
  CDisconnect.HelpContext:=hcDisconnectq;

  DBasicData.HelpContext:=hcDisplayBasic;
  DModuleData.HelpContext:=hcDisplayModule;

  GSetCurrentOutput.HelpContext:=hcSetCurrent;
  GetSetDigitalOutputs.HelpContext:=hcSetDigital;
  GReset.HelpContext:=hcResetDevice;
  GStatus.HelpContext:=hcStatus;
  GMonitor.HelpContext:=hcMonitor;
  GDataLogger.HelpContext:=hcDataLogger;
  GInspection.HelpContext:=hcInspection;

  CCalibration.HelpContext:=hcCalibrate;
  CBasicData.HelpContext:=hcConfigBasic;
  CModuleData.HelpContext:=hcConfigModule;
  CChangeDevice.HelpContext:=hcChangeDevice;
  CSaveToDevice.HelpContext:=hcSaveToDevice;
  CVerifyDevice.HelpContext:=hcCompareToDevice;
  CDefaultConfig.HelpContext:=hcDefaultConfig;
  CHardwareConfig.HelpContext:=hcHardwareConfig;

  OPassword.HelpContext:=hcPassword;
  OLanguage.HelpContext:=hcLanguage;
  OPortSelection.HelpContext:=hcPortSelection;
  OErrorIndication.HelpContext:=hcErrorIndication;
  //OCalculator.HelpContext:=;
  //OResetPassword.HelpContext:=;

  //HContent.HelpContext:=;
  HAbout.HelpContext:=hcAbout;
end;

procedure TfmPcs45Main.SetAllHints;
begin
  //
end;


procedure TfmPcs45Main.SetBasicMenu;
begin
// Submenu file
  FSave.Enabled:=false;
  FSaveAs.Enabled:=false;
  FExport.Enabled:=false;
  FPrint.Enabled:=false;

// Submenu connection
  CDisconnect.Enabled:=false;

// Submenu diagnostics
  GetSetDigitalOutputs.Visible:=false;
  G3.Visible:=false;
  GInspection.Visible:=false;

// Submenu device data configuration - completion to submenu device data
  CChangeDevice.Enabled:=false;
  CChangeDevice.Visible:=true;
  CSaveToDevice.Enabled:=false;
  CVerifyDevice.Enabled:=false;
  C3.Visible:=false;
  CDefaultConfig.Visible:=false;
  C4.Visible:=false;
  CHardwareConfig.Visible:=false;

// Submenu options
  OCalculator.Visible:=false; // disable temporary visibility
  O1.Visible:=false;
  OResetPassword.Visible:=false;
end;

procedure TfmPcs45Main.UpdateMenus;
begin
  // Set Menu Item visibility

  G3.Visible:=false;
  GInspection.Visible:=false;

  C3.Visible:=false;
  CDefaultConfig.Visible:=false;
  C4.Visible:=false;
  CHardwareConfig.Visible:=false;

  O1.Visible:=false;
  OResetPassword.Visible:=false;

  // Set Menu Item Accessibility
  CDisconnect.Enabled:=MOD45.DeviceConnected;

  GSetCurrentOutput.Enabled:=(MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);
  GReset.Enabled:=(MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);
  CCalibration.Enabled:=(MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);

  CBasicData.Enabled:=(MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);
  CModuleData.Enabled:=(MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);

  if MOD45.DataAvailable then
     begin
     FSaveAs.Enabled:=true;
     FExport.Enabled:=true;
     FPrint.Enabled:=true;

     GetSetDigitalOutputs.Visible:=(MOD45.DevType in [dtx532,dt1003]);
     GetSetDigitalOutputs.Enabled:=GetSetDigitalOutputs.Visible and
                                   (MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);

     CChangeDevice.Enabled:=(MOD45.DevType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075]) and
                             (MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);
     CVerifyDevice.Enabled:=true;
     CSaveToDevice.Enabled:=(MOD45.PasswordReleased or MOD45.SupervisorAccess or MOD45.DemoMode);

     if (MOD45.SupervisorAccess) then
        begin
        O1.Visible:=true;
        OResetPassword.Visible:=true;
        OResetPassword.Enabled:=true;
        if (MOD45.DevType in [dtx532,dt1003]) then
           begin
           G3.Visible:=true;
           GInspection.Visible:=true;
           GInspection.Enabled:=true;
           end;
        if (MOD45.DevType=dt1003) then
           begin
           C4.Visible:=true;
           CHardwareConfig.Visible:=true;
           CHardwareConfig.Enabled:=true;
           end;
        end;
     end
  else
     begin
     FSaveAs.Enabled:=false;
     FExport.Enabled:=false;
     FPrint.Enabled:=false;

     GetSetDigitalOutputs.Visible:=false;

     CChangeDevice.Enabled:=false;
     CVerifyDevice.Enabled:=false;
     CSaveToDevice.Enabled:=false;
     end;
end;

procedure TfmPcs45Main.UpdateStatusBar;
begin
  if (MOD45.DevType<>dtNone) and
     (MTLHARTModule<>nil) then
     begin
     StatusBar1.Panels[1].Text:=CDeviceShortNames[MOD45.DevType];
     StatusBar1.Panels[2].Text:=MTLHARTModule.BasicDeviceData.TDD.TagName;
     end
  else
     begin
     StatusBar1.Panels[1].Text:=LanguageStrings[slNoDeviceSelected];
     StatusBar1.Panels[2].Text:=LanguageStrings[slNoDataAvailable];
     end;

  if MOD45.DeviceConnected then
     StatusBar1.Panels[3].Text:=LanguageStrings[slOnline]
  else StatusBar1.Panels[3].Text:=LanguageStrings[slOffline];

  if (MOD45.PasswordReleased) or (MOD45.SupervisorAccess) then
     StatusBar1.Panels[4].Text:=LanguageStrings[slFullAccess]
  else StatusBar1.Panels[4].Text:=LanguageStrings[slRestrictedAccess];

  MessageBeep(MB_ICONEXCLAMATION);
  MessageBeep($FFFF);
end;

function TfmPcs45Main.CheckReleasedPassword: boolean;
begin
  if not MOD45.PasswordReleased then
     begin
     Application.CreateForm(TfmPassword, fmPassword);
     with fmPassword do
          begin
          SetMode(pmCheck);
          SetPassword(CFG45.Password);
          ShowModal;
          MOD45.SupervisorAccess:=(ModalResult=mrIgnore);
          MOD45.PasswordReleased:=(ModalResult=mrOK) or MOD45.SupervisorAccess;
          Release;
          end;
     Result:=(MOD45.SupervisorAccess or MOD45.PasswordReleased);

     if MOD45.SupervisorAccess then
        MessageDlg(LanguageStrings[msEnabledBecauseValid],mtInformation,[mbOk],hcNoHelpTopic)
     else if MOD45.PasswordReleased then
        MessageDlg(LanguageStrings[msEnabledBecauseValid],mtInformation,[mbOk],hcNoHelpTopic)
     else MessageDlg(LanguageStrings[msDisabledBecauseWrong],mtError,[mbCancel],hcNoHelpTopic);

     UpdateMenus;
     UpdateStatusBar;
     end
  else Result:=true;
end;

procedure TfmPcs45Main.ArrangeCommunicationPort;
var Msg: string;
begin
  if CFG45.PCL45USBSelected then
     begin
     CFG45.HighSpeed:=true;
     HartMaster1.HartDeviceType:=dtFTDevice;
     if not(HartMaster1.Open(CPCL45USBName)) then
        begin
        MessageDlg(LanguageStrings[msCannotInitPCL45USB],mtError,[mbOk],hcNoHelpTopic);
        CFG45.PortNumber:=0;
        end;
     end
  else if (CFG45.PortNumber in [1..8]) then
     begin
     CFG45.HighSpeed:=false;
     HartMaster1.HartDeviceType:=dtSerialCom;
     if not(HartMaster1.Open(CFG45.PortNumber)) then
        begin
        FmtStr(Msg,LanguageStrings[msCannotOpenPort],[CFG45.PortNumber]);
        MessageDlg(Msg,mtError,[mbOk],hcNoHelpTopic);
        CFG45.PortNumber:=0;
        CFG45.HighSpeed:=true;
        end;
     end;

  MOD45.DemoMode:=(CFG45.PortNumber=0);

  if MOD45.DemoMode then
     begin
     HartMaster1.Open(0);
     // HartMaster1.OnReceiveChar:=nil;
     HartMaster1.OnAnswerSimulationRequest:=HartMaster1AnswerSimulationRequest;
     end;

  // if CFG45.HighSpeed then SetHartComm(FastComm) else SetHartComm(SlowComm);
  if not(MOD45.DemoMode or MOD45.PasswordReleased) then
     CheckReleasedPassword;
end;

procedure TfmPcs45Main.IndicateReceivedChar;
begin
  // Empty procedure required for correct operation
end;

procedure TfmPcs45Main.ShowDemoOperationWarning;
begin
  if MOD45.DemoMode then
     MessageDlg(LanguageStrings[msDemoAttention],mtInformation,[mbOk],hcNoHelpTopic);
end;

function TfmPcs45Main.ShowAndAcceptISO9002Warning: boolean;
begin
  if not(MOD45.DemoMode) then
     Result:=(MessageDlg(LanguageStrings[ms9002Warning],mtInformation,[mbOk,mbCancel],hcNoHelpTopic)=mrOk)
  else Result:=true;
end;

function TfmPcs45Main.VerifyUnintentionalOverwriteAttempt: boolean;
begin
  Result:=MOD45.DataAvailable and
         (MessageDlg(LanguageStrings[msDataPresentOverwrite],mtConfirmation,[mbYes,mbCancel],hcNoHelpTopic)<>mrYes);
end;

function TfmPcs45Main.VerifyDeviceDataAvailable: boolean;
begin
  if not(MOD45.DataAvailable) then
     begin
     if (MessageDlg(LanguageStrings[msNoDataShouldConnect],mtConfirmation,[mbYes,mbCancel],hcNoHelpTopic)=mrYes) then
        begin
        CEstablishClick(Self);
        Result:=(MOD45.DeviceConnected);
        end
     else Result:=false;
     end
  else Result:=true;
end;

function TfmPcs45Main.GetActualConnectedDeviceType: TDeviceType;
begin
  // OriginalCursor:=Cursor;
  Cursor:=crHourGlass;
  MTLHartModule.AssignReceivedCharEvent(IndicateReceivedChar);
  Result:=MTLHartModule.GetConnectedDeviceType;
  MTLHartModule.UnAssignReceivedCharEvent;
  Cursor:=OriginalCursor;
end;

function TfmPcs45Main.VerifyDeviceTypeConnected: boolean;
var DialogResult: integer;
begin
  Result:=false;
  DialogResult:=mrCancel;
  if MOD45.DeviceConnected then
     begin
     Result:=(MOD45.DevType=GetActualConnectedDeviceType);
     if not(Result) then
        DialogResult:=MessageDlg(LanguageStrings[msDifferentDeviceDetected]+^M+
                                 LanguageStrings[msShouldReconnectDevice],mtConfirmation,[mbYes,mbCancel],hcNoHelpTopic)
        else ShowDemoOperationWarning;
     end
  else // not(MOD45.DeviceConnected)
     DialogResult:=MessageDlg(LanguageStrings[msNoDeviceConnected],mtConfirmation,[mbYes,mbCancel],hcNoHelpTopic);

  if not(Result) and
     (DialogResult=mrYes)then
     begin
        CEstablishClick(Self);
        // Wait to allow object processes to be released 
        WaitMilliseconds(250);
        Result:=(MOD45.DeviceConnected);
     end;
end;

function TfmPcs45Main.VerifyDeviceDataAvailableAndDeviceType(var SM: TSaveMode): boolean;
var DialogResult: integer;
begin
  Result:=false;
  DialogResult:=mrCancel;
  SM:=smToDevice;
  if MOD45.DataAvailable then
     begin
     if MOD45.DeviceConnected then
        begin
        if (MOD45.DevType=GetActualConnectedDeviceType) then
           begin
           ShowDemoOperationWarning;
           SM:=smToDevice;
           Result:=true;
           end
        else
        DialogResult:=MessageDlg(LanguageStrings[msDifferentDeviceDetected]+^M+
                                 LanguageStrings[msShouldReconnectDevice],mtConfirmation,[mbYes,mbCancel],hcNoHelpTopic);
        end
     else // not(MOD45.DeviceConnected)
        begin
        SM:=smToMemory;
        Result:=true;
        end;
     end
  else // not(MOD45.DataAvailable)
     DialogResult:=MessageDlg(LanguageStrings[msNoDataShouldConnect],mtConfirmation,[mbYes,mbCancel],hcNoHelpTopic);

  if not(Result) and
     (DialogResult=mrYes)then
     begin
     CEstablishClick(Self);
     Result:=(MOD45.DeviceConnected);
     end;
end;

procedure TfmPcs45Main.SaveDeviceDataSet(Name: TFileName);
var SaveName: TFileName;
    IM: string;
begin
  if not(MOD45.PasswordReleased) then
     SaveName:=ExtractFilePath(Name)+CDef_PCSFile
  else Savename:=Name;

  MTLHartModule.GetActualDeviceDataStream.SaveToFile(SaveName);

  IM:=LanguageStrings[msSavedToFile]+StripPathString(SaveName,20);
  MessageDlg(IM,mtInformation,[mbOk],hcNoHelpTopic);
end;

procedure TfmPcs45Main.SaveDialog1TypeChange(Sender: TObject);
begin
  {
  if ExcelAvailable then
     begin
     if SaveDialog1.FilterIndex=1 then SaveDialog1.DefaultExt:=LowerCase(CDef_ExcelExt)
     else if SaveDialog1.FilterIndex=2 then SaveDialog1.DefaultExt:=LowerCase(CDef_CsvIExExt)
     else SaveDialog1.DefaultExt:=LowerCase(CDef_TxtIExExt);
     end
  else if SaveDialog1.FilterIndex=1 then SaveDialog1.DefaultExt:=LowerCase(CDef_CsvIExExt)
     else SaveDialog1.DefaultExt:=LowerCase(CDef_TxtIExExt);
  }
end;

// Form procedures and functions

procedure TfmPcs45Main.FormCreate(Sender: TObject);
var lApplicationFullFileName: TFileName;
begin
  OriginalCursor:=Cursor;

  lApplicationFullFileName:=GetApplicationFullFileName;
  OwnPath:=ExtractFilePath(lApplicationFullFileName);
  OwnName:=ExtractFileBaseName(lApplicationFullFileName);

  MOD45:=CDef_MOD;
  CFG45:=CDef_CFG;

  SetDefaultLanguageStrings;

  fFormActivated:=false;
  ActualFileName:='';
  ExcelAvailable:=false;

  ActiveLibrary:=0;
  SetLength(LanguageInfos,0);
end;

procedure TfmPcs45Main.FormShow(Sender: TObject);
begin
  LoadJPEGBitmapFile(OwnPath+OwnName+'CI.JPG', Image1);
  
  ReadAvailableLanguageInfos(LanguageInfos);
  ArrangeModeAndConfiguration;
  
  TryToLokalize;

  SelAllCaptions;
  SetAllHelpContexts;
  SetAllHints;

  // SetDeviceEnvironment(MOD45.DevType);

  SetBasicMenu;
  UpdateStatusBar;
end;

procedure TfmPcs45Main.FormActivate(Sender: TObject);
begin
  if not(fFormActivated) then
     begin
     fFormActivated:=true;
     
     LoadFTD2XXLibrary;

     MOD45.PCL45USBConnected:=(GetFTDeviceCount(CPCL45USBName)=1);
  
     if (MOD45.PCL45USBConnected and (CFG45.PCL45USBSelected)) or
        (not(MOD45.PCL45USBConnected) and
        ((CFG45.PortNumber=0) or (CFG45.PortNumber in [1..8]) and
        (SerialPortAvailable(CFG45.PortNumber)))) then
        ArrangeCommunicationPort
     else OPortSelectionClick(Sender);

     // Timer1.Enabled:=true;
     MTLHartModule:=TBasicHartModule.Create;
     MTLHartModule.AssignHartMaster(HartMaster1);
     end;
end;

procedure TfmPcs45Main.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  MTLHartModule.Free;
  if not(MOD45.DemoMode) then
     HartMaster1.Close;
  SaveDeviceTypeToRegistry(MOD45.DevType);
  // SaveLastPathsToRegistry(CLastPath,MOD45.LastPath);
end;

procedure TfmPcs45Main.FormDestroy(Sender: TObject);
begin
  HartMaster1.Free;
end;

procedure TfmPcs45Main.HartMaster1AnswerSimulationRequest(HartCommand: THartCommand; var HartAnswer: THartAnswer);
begin
  SimulateTransmitterHartAnswer(HartCommand, HartAnswer);
end;

// Menu procedures and functions

// Submenu File

procedure TfmPcs45Main.FNewClick(Sender: TObject);
begin
  if VerifyUnintentionalOverwriteAttempt then exit;

  Application.CreateForm(TfmNewDevice, fmNewDevice);
  with fmNewDevice do
       begin
       if (MOD45.DevType<>dtNone) then
          SetDevice(MOD45.DevType)
       else SetDevice(CDefaultNewDevice);
       ShowModal;
       if ModalResult=mrOK then
          begin
          MOD45.DevType:=GetDevice;
          GetSpecificHartModule(MOD45.DevType,MTLHartModule);

          MTLHartModule.AssignHartMaster(HartMaster1);
          MTLHartModule.SetModuleDeviceDataAvailableStatus;
          MTLHartModule.SynchronizeRequiredDeviceData;
          SetDefaultTransmitterParameters(MOD45.DevType);
          MOD45.DataAvailable:=true;
          MOD45.DeviceConnected:=false;
          UpdateMenus;
          UpdateStatusBar;

          end;
       Release;
       end;
end;

procedure TfmPcs45Main.FOpenClick(Sender: TObject);
  procedure ShowOpenSuccessMessage(FileName: TFileName);
  var InfoMessage: string;
  begin
    InfoMessage:=LanguageStrings[msLoadedFromFile]+StripPathString(FileName,30);
    MessageDlg(InfoMessage,mtInformation,[mbOk],hcNoHelpTopic);
  end;
  procedure ShowOpenFailMessage(FileName: TFileName);
  var InfoMessage: string;
  begin
    FmtStr(InfoMessage,LanguageStrings[msFileCorrupted],[ExtractFileName(FileName)]);
    MessageDlg(InfoMessage,mtError,[mbCancel],hcNoHelpTopic);
  end;
type TFileExtention = (feWrong,feD45,feD73,feMtl);
var NDI: THDeviceIdentifier;
    lFE: TFileExtention;
    SD: TMemoryStream;
    lDT: TDeviceType;
begin
  if VerifyUnintentionalOverwriteAttempt then exit;

  with OpenDialog1 do
       begin
       Title:=' '+LanguageStrings[dlLoadFile];
       DefaultExt:=CDef_Pcs45_Ext;
       Filter:=LanguageStrings[dsLoadFileFilter];
       if (MOD45.LastD45Path<>'') then
          InitialDir:=MOD45.LastD45Path;
       if Execute then
          begin
          if (UpperCase(ExtractFileExt(FileName))=CDef_Pcs45_Ext) then
             lFE:=feD45
          else if (UpperCase(ExtractFileExt(FileName))=CDef_Pcc73_Ext) then
                  lFE:=feD73
          else if (UpperCase(ExtractFileExt(FileName))=CDef_MTL_Ext) then
                  lFE:=feMtl
          else lFE:=feWrong;

          if (lFE<>feWrong) then
             begin
             if (UpperCase(ExtractFileExt(FileName))=CDef_Pcs45_Ext) then
                SD:=ImportConfigurationFile(FileName)
             else if (UpperCase(ExtractFileExt(FileName))=CDef_Pcc73_Ext) then
                     SD:=ImportOldConfigurationFile(FileName)
             else if (UpperCase(ExtractFileExt(FileName))=CDef_MTL_Ext) then
                     SD:=ImportVeryOldConfigurationFile(FileName)
             else SD:=TMemoryStream.Create;

             // common open and import code

             MOD45.DataAvailable:=false;
             ActualFileName:='';

             if (SD.Size>SizeOf(THDeviceIdentifier)) then
                begin
                SD.Seek(System.Length(CNew_Config_Header),soFromBeginning);
                SD.Read(NDI,SizeOf(THDeviceIdentifier));
                lDT:=IdentToType(NDI);

                if (lDT<>dtNone) then
                   begin
                   GetSpecificHartModule(lDT,MTLHartModule);
                   MTLHartModule.AssignHartMaster(HartMaster1);

                   if (MTLHartModule.SetActualDeviceData(SD)) then
                      begin
                      MTLHartModule.SynchronizeRequiredDeviceData;
                      // MTLHartModule.AssignHartMaster(HartMaster1);

                      MOD45.DevType:=lDT;
                      MOD45.DataAvailable:=true;
                      MOD45.DeviceConnected:=false;

                      ActualFileName:=FileName;
                      MOD45.LastD45Path:=ExtractFilePath(FileName);
                      ShowOpenSuccessMessage(FileName);

                      UpdateMenus;
                      UpdateStatusBar;
                      end
                   else MessageDlg(LanguageStrings[msUnknownDeviceData],mtError,[mbOk],hcNoHelpTopic);
                   end
                else ShowOpenFailMessage(FileName);
                end
             else ShowOpenFailMessage(FileName);

             // common open and import code

             SD.Free;
             end
          else MessageDlg(LanguageStrings[msWrongFileExtension],mtError,[mbOk],hcNoHelpTopic);
          end;
       end;
end;

procedure TfmPcs45Main.FSaveClick(Sender: TObject);
begin
  if (MOD45.DataAvailable) then
     SaveDeviceDataSet(ActualFilename)
  else MessageDlg(LanguageStrings[msNoDataNoFunction],mtError,[mbOk],hcNoHelpTopic);
end;

procedure TfmPcs45Main.FSaveAsClick(Sender: TObject);
begin
  if (MOD45.DataAvailable) then
     begin
     with SaveDialog1 do
          begin
          Title:=' '+LanguageStrings[dlSaveFile];
          DefaultExt:=CDef_Pcs45_Ext;
          Filter:=LanguageStrings[dsSaveFileFilter];

          if (FileName<>'') and (ExtractFileExt(FileName)<>DefaultExt) then
             FileName:=ChangeFileExt(FileName,DefaultExt);

          if (MOD45.LastD45Path<>'') then
             InitialDir:=MOD45.LastD45Path;
          if Execute then
             begin
             SaveDeviceDataSet(FileName);
             ActualFilename:=FileName;
             MOD45.LastD45Path:=ExtractFilePath(FileName);
             FSave.Enabled:=true;
             end;
          end;
     end
  else MessageDlg(LanguageStrings[msNoDataNoFunction],mtError,[mbOk],hcNoHelpTopic);
end;

procedure TfmPcs45Main.FImportClick(Sender: TObject);
  procedure ShowImportSuccessMessage(FileName: TFileName);
  var InfoMessage: string;
  begin
    InfoMessage:=LanguageStrings[msImportedFromFile]+StripPathString(FileName,30);
    MessageDlg(InfoMessage,mtInformation,[mbOk],hcNoHelpTopic);
  end;
  procedure ShowImportFailMessage(FileName: TFileName);
  var InfoMessage: string;
  begin
    FmtStr(InfoMessage,LanguageStrings[msFileCorrupted],[ExtractFileName(FileName)]);
    MessageDlg(InfoMessage,mtError,[mbCancel],hcNoHelpTopic);
  end;
type TFileExtention = (feWrong,feText,feCsv,feXls);
var NDI: THDeviceIdentifier;
    lFE: TFileExtention;
    SD: TMemoryStream;
    lDT: TDeviceType;
begin
 if VerifyUnintentionalOverwriteAttempt then exit;

  with OpenDialog1 do
       begin
       Title:=' '+LanguageStrings[dlImportFromFile];

       if ExcelAvailable then
          begin
          Filter:=LanguageStrings[dsXlsCsvTxtImpFilter];
          DefaultExt:=LowerCase(CDef_ExcelExt);
          end
       else
          begin
          Filter:=LanguageStrings[dsCsvTxtImportFilter];
          DefaultExt:=LowerCase(CDef_CsvIExExt);
          end;

       if (MOD45.LastImpexPath<>'') then
          InitialDir:=MOD45.LastImpexPath
       else InitialDir:=GetMyDocumentsFolder;
       if Execute then
          begin
          if (UpperCase(ExtractFileExt(FileName))=CDef_TxtIExExt) then
             lFE:=feText
          else if (UpperCase(ExtractFileExt(FileName))=CDef_CsvIExExt) then
                  lFE:=feCsv
          else if ExcelAvailable and (UpperCase(ExtractFileExt(FileName))=CDef_ExcelExt) then
                  lFE:=feXls
          else lFE:=feWrong;

          if (lFE<>feWrong) then
             begin
             if (UpperCase(ExtractFileExt(FileName))=CDef_TxtIExExt) then
                SD:=ImportTextConfigurationFile(FileName,CTAB)
             else if (UpperCase(ExtractFileExt(FileName))=CDef_CsvIExExt) then
                     SD:=ImportTextConfigurationFile(FileName,',')
             else if ExcelAvailable and (UpperCase(ExtractFileExt(FileName))=CDef_ExcelExt) then
                     SD:=ImportExcelConfigurationFile(FileName)
             else SD:=TMemoryStream.Create;

             // common open and import code

             MOD45.DataAvailable:=false;
             ActualFileName:='';

             if (SD.Size>SizeOf(THDeviceIdentifier)) then
                begin

                SD.Seek(System.Length(CNew_Config_Header),soFromBeginning);
                SD.Read(NDI,SizeOf(THDeviceIdentifier));
                lDT:=IdentToType(NDI);

                if (lDT<>dtNone) then
                   begin
                   GetSpecificHartModule(lDT,MTLHartModule);
                   MTLHartModule.AssignHartMaster(HartMaster1);

                   if (MTLHartModule.SetActualDeviceData(SD)) then
                      begin
                      MTLHartModule.SynchronizeRequiredDeviceData;
                      // MTLHartModule.AssignHartMaster(HartMaster1);

                      MOD45.DevType:=lDT;
                      MOD45.DataAvailable:=true;
                      MOD45.DeviceConnected:=false;

                      ActualFileName:=FileName;
                      MOD45.LastImpexPath:=ExtractFilePath(FileName);
                      ShowImportSuccessMessage(FileName);

                      UpdateMenus;
                      UpdateStatusBar;
                      end
                   else ShowImportFailMessage(FileName);
                   end
                else ShowImportFailMessage(FileName);

                end
             else ShowImportFailMessage(FileName);

             // common open and import code

             SD.Free;
             end
          else MessageDlg(LanguageStrings[msWrongFileExtension],mtError,[mbOk],hcNoHelpTopic);
          end;
       end;
end;

procedure TfmPcs45Main.FExportClick(Sender: TObject);
var ExportName: TFileName;
    lExportGrid: TStringDynGrid;
    Target: TLnCRCFileStream;
    IM: string;
begin
  if (MOD45.DataAvailable) then
     begin
     with SaveDialog1 do
          begin
          Title:=' '+LanguageStrings[dlExportToFile];

          if ExcelAvailable then
             begin
             Filter:=LanguageStrings[dsXlsCsvTxtExpFilter];
             DefaultExt:=LowerCase(CDef_ExcelExt);
             end
          else
             begin
             Filter:=LanguageStrings[dsCsvTxtImportFilter];
             DefaultExt:=LowerCase(CDef_CsvIExExt);
             end;

          if (FileName<>'') and (ExtractFileExt(FileName)<>DefaultExt) then
             FileName:=ChangeFileExt(FileName,DefaultExt);

          if (MOD45.LastImpexPath<>'') then
             InitialDir:=MOD45.LastImpexPath
          else InitialDir:=GetMyDocumentsFolder;

          if Execute then
             begin
             if not(MOD45.PasswordReleased) then
                ExportName:=ExtractFilePath(FileName)+ChangeFileExt(CDef_PCSFile,ExtractFileExt(FileName))
             else ExportName:=FileName;

             lExportGrid:=ExportConfigurationToGrid(TBasicHartModule(MTLHartModule));

             if (System.Length(lExportGrid)>0) then
                begin

                Target:=TLnCRCFileStream.Create(ExportName,fmCreate);

                if (UpperCase(ExtractFileExt(ExportName))=CDef_CsvIExExt) then
                   Target.WriteStringDynGridLn(lExportGrid,ListSeparator)
                else Target.WriteStringDynGridLn(lExportGrid,CTAB);

                Target.Free;

                IM:=LanguageStrings[msExportedToFile]+StripPathString(ExportName,20);
                MessageDlg(IM,mtInformation,[mbOk],hcNoHelpTopic);

                MOD45.LastImpexPath:=ExtractFilePath(FileName);
                end;
             end;
          end;
     end
  else MessageDlg(LanguageStrings[msNoDataNoFunction],mtError,[mbOk],hcNoHelpTopic);
end;

procedure TfmPcs45Main.FPrintClick(Sender: TObject);
var lExportGrid: TStringDynGrid;
begin
  if (MOD45.DataAvailable) then
     begin
     with PrintDialog1 do
          begin
          Options:=[poPrintToFile,poWarning];
          if Execute then
             begin
             lExportGrid:=ExportConfigurationToGrid(TBasicHartModule(MTLHartModule));
             PrintStringDynGrid(lExportGrid);
             end;
          end;
     end
  else MessageDlg(LanguageStrings[msNoDataNoFunction],mtError,[mbOk],hcNoHelpTopic);
end;

procedure TfmPcs45Main.FExitClick(Sender: TObject);
begin
  Close;
end;

// Submenu Connection

procedure TfmPcs45Main.CEstablishClick(Sender: TObject);
var lDevType: TDeviceType;
    Success: boolean;
begin
  ShowDemoOperationWarning;
  if VerifyUnintentionalOverwriteAttempt then exit;

  lDevType:=GetActualConnectedDeviceType;

  if (lDevType<>dtNone) then
    begin
    GetSpecificHartModule(lDevType,MTLHartModule);
    MTLHartModule.AssignHartMaster(HartMaster1);

    Application.CreateForm(TfmTransfer, fmTransfer);
    with fmTransfer do
         begin
         SetMode(MOD45.DevType,tjEstablish,smToDevice);
         AssignHartModule(@MTLHartModule);
         Success:=(ShowModal=mrOK);
         Release;
         end;
    end
  else
    begin
    Success:=false;
    MessageDlg(LanguageStrings[msDeviceNotResponding],mtInformation,[mbOk],hcNoHelpTopic);
    end;

  if Success then
     begin
     MOD45.DevType:=lDevType;
     MOD45.DeviceConnected:=true;
     MOD45.DataAvailable:=true;
     end
  else
     begin
     MOD45.DevType:=dtNone;
     MOD45.DeviceConnected:=false;
     MOD45.DataAvailable:=false;
     end;

  UpdateMenus;
  UpdateStatusBar;
end;

procedure TfmPcs45Main.CDisconnectClick(Sender: TObject);
begin
  MOD45.DevType:=dtNone;
  MOD45.DeviceConnected:=false;
  MOD45.DataAvailable:=false;

  UpdateMenus;
  UpdateStatusBar;
end;

// Submenu Display

procedure TfmPcs45Main.DBasicDataClick(Sender: TObject);
begin
  if not(VerifyDeviceDataAvailable) then exit;

  Application.CreateForm(TfmBasicData, fmBasicData);
  with fmBasicData do
       begin
       SetMode(dmDisplay, smToMemory);
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.DModuleDataClick(Sender: TObject);
begin
  if not(VerifyDeviceDataAvailable) then exit;

  if (MOD45.DevType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx575,dtx573]) then
     begin
     Application.CreateForm(TfmModuleData_73, fmModuleData_73);
     with fmModuleData_73 do
          begin
          SetMode(dmDisplay, smToMemory);
          AssignHartModule(@MTLHartModule);
          ShowModal;
          Release;
          end;
     end
  else
     if (MOD45.DevType=dtx576) then
        begin
        Application.CreateForm(TfmModuleData_76, fmModuleData_76);
        with fmModuleData_76 do
             begin
             SetMode(dmDisplay, smToMemory);
             AssignHartModule(@MTLHartModule);
             ShowModal;
             Release;
             end;
        end
     else
       if (MOD45.DevType in [dtx532,dt1003]) then
          begin
          Application.CreateForm(TfmModuleData_32, fmModuleData_32);
          with fmModuleData_32 do
               begin
               SetMode(dmDisplay, smToMemory);
               AssignHartModule(@MTLHartModule);
               ShowModal;
               Release;
               end;
          end;
end;

// Submenu Diagnostics

procedure TfmPcs45Main.GSetCurrentOutputClick(Sender: TObject);
begin
  if not(ShowAndAcceptISO9002Warning) then exit;
  if not(VerifyDeviceTypeConnected) then exit;

  Application.CreateForm(TfmSetAndCalibrate, fmSetAndCalibrate);
  with fmSetAndCalibrate do
       begin
       AssignHartModule(@MTLHartModule);
       SetMode(smSetCurrent);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.GetSetDigitalOutputsClick(Sender: TObject);
begin
  if not(ShowAndAcceptISO9002Warning) then exit;
  if not(VerifyDeviceTypeConnected) then exit;

  Application.CreateForm(TfmDigitalDiagnostics, fmDigitalDiagnostics);
  with fmDigitalDiagnostics do
       begin
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.GStatusClick(Sender: TObject);
begin
  if not(VerifyDeviceTypeConnected) then exit;

  Application.CreateForm(TfmStatus, fmStatus);
  with fmStatus do
       begin
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.GMonitorClick(Sender: TObject);
begin
  if not(VerifyDeviceTypeConnected) then exit;

  Application.CreateForm(TfmMonitor, fmMonitor);
  with fmMonitor do
       begin
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.GDataLoggerClick(Sender: TObject);
begin
  if not(VerifyDeviceTypeConnected) then exit;

  Application.CreateForm(TfmDataLogger, fmDataLogger);
  with fmDataLogger do
       begin
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.GResetClick(Sender: TObject);
begin
  if not(ShowAndAcceptISO9002Warning) then exit;

  MTLHartModule.ResetModule;
end;

procedure TfmPcs45Main.GInspectionClick(Sender: TObject);
begin
  Application.CreateForm(TfmInspection, fmInspection);
  with fmInspection do
       begin
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

// Submenu Configuration

procedure TfmPcs45Main.CCalibrationClick(Sender: TObject);
begin
  if not(ShowAndAcceptISO9002Warning) then exit;
  if not(VerifyDeviceTypeConnected) then exit;

  Application.CreateForm(TfmSetAndCalibrate, fmSetAndCalibrate);
  with fmSetAndCalibrate do
       begin
       AssignHartModule(@MTLHartModule);
       SetMode(smCalibrate);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.CBasicDataClick(Sender: TObject);
var lSaveMode: TSaveMode;
begin
  if not(VerifyDeviceDataAvailableAndDeviceType(lSaveMode)) then exit;

  Application.CreateForm(TfmBasicData, fmBasicData);
  with fmBasicData do
       begin
       SetMode(dmConfig,lSaveMode);
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.CModuleDataClick(Sender: TObject);
var lSaveMode: TSaveMode;
begin
  if not(VerifyDeviceDataAvailableAndDeviceType(lSaveMode)) then exit;

  if (MOD45.DevType in [dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx575,dtx573]) then
     begin
     Application.CreateForm(TfmModuleData_73, fmModuleData_73);
     with fmModuleData_73 do
          begin
          SetMode(dmConfig,lSaveMode);
          AssignHartModule(@MTLHartModule);
          ShowModal;
          Release;
          end;
     end
  else
     if (MOD45.DevType=dtx576) then
        begin
        Application.CreateForm(TfmModuleData_76, fmModuleData_76);
        with fmModuleData_76 do
             begin
             SetMode(dmConfig,lSaveMode);
             AssignHartModule(@MTLHartModule);
             ShowModal;
             Release;
             end;
        end
     else
       if (MOD45.DevType in [dtx532,dt1003]) then
          begin
          Application.CreateForm(TfmModuleData_32, fmModuleData_32);
          with fmModuleData_32 do
               begin
               SetMode(dmConfig,lSaveMode);
               AssignHartModule(@MTLHartModule);
               ShowModal;
               Release;
               end;
          end;
end;

procedure TfmPcs45Main.CChangeDeviceClick(Sender: TObject);
var lDeviceType: TDeviceType;
begin
  ShowDemoOperationWarning;

  Application.CreateForm(TfmChangeDevice, fmChangeDevice);
  with fmChangeDevice do
       begin
       ShowModal;
       if ModalResult=mrOK then
          begin
          lDeviceType:=GetDevice;
          MOD45.DevType:=lDeviceType;

          ChangeTypeHartModule(lDeviceType,MTLHartModule);

          MTLHartModule.AssignHartMaster(HartMaster1);
          MTLHartModule.SetModuleDeviceDataAvailableStatus;
          MTLHartModule.SynchronizeRequiredDeviceData;
          SetDefaultTransmitterParameters(MOD45.DevType);
          MOD45.DataAvailable:=true;
          MOD45.DeviceConnected:=false;
          UpdateMenus;
          UpdateStatusBar;

          end;
       Release;
       end;
end;

procedure TfmPcs45Main.CSaveToDeviceClick(Sender: TObject);
begin
  ShowDemoOperationWarning;

  Application.CreateForm(TfmTransfer, fmTransfer);
  with fmTransfer do
       begin

       SetMode(MOD45.DevType,tjSave,smToDevice);
       AssignHartModule(@MTLHartModule);

       ShowModal;

       if ModalResult=mrOK then
          MessageDlg(LanguageStrings[msDataSuccessfulSaved],mtInformation,[mbOK],hcNoHelpTopic)
       else MessageDlg(LanguageStrings[msDataSavingFailed]+^M+LanguageStrings[msDataDifferent],mtError,[mbOK],hcNoHelpTopic);

       Release;
       end;
end;

procedure TfmPcs45Main.CVerifyDeviceClick(Sender: TObject);
begin
  ShowDemoOperationWarning;

  Application.CreateForm(TfmTransfer, fmTransfer);
  with fmTransfer do
       begin

       SetMode(MOD45.DevType,tjCompare,smToDevice);
       AssignHartModule(@MTLHartModule);

       ShowModal;

       if ModalResult=mrOK then
          MessageDlg(LanguageStrings[msDataEqual],mtInformation,[mbOK],hcNoHelpTopic)
       else MessageDlg(LanguageStrings[msDataDifferent],mtError,[mbOK],hcNoHelpTopic);

       Release;
       end;
end;

procedure TfmPcs45Main.CDefaultConfigClick(Sender: TObject);
begin
  //
end;

procedure TfmPcs45Main.CHardwareConfigClick(Sender: TObject);
begin
  Application.CreateForm(TfmHardwareConfig, fmHardwareConfig);
  with fmHardwareConfig do
       begin
       AssignHartModule(@MTLHartModule);
       ShowModal;
       Release;
       end;
end;

// Submenu Options

procedure TfmPcs45Main.OPasswordClick(Sender: TObject);
var NewPassword: string;
begin
  Application.CreateForm(TfmPassword, fmPassword);
  with fmPassword do
       begin
       SetMode(pmChange);
       SetPassword(CFG45.Password);
       ShowModal;
       case ModalResult of
            mrOK:  begin
                   NewPassword:=fmPassword.GetPassword;
                   if (CFG45.Password<>NewPassword) then
                      begin
                      CFG45.Password:=NewPassword;
                      SavePasswordToRegistry(CFG45.Password);
                      MessageDlg(LanguageStrings[msPasswordSaved],mtInformation,[mbOk],hcNoHelpTopic);
                      end
                   else MessageDlg(LanguageStrings[msValidButNoChange],mtInformation,[mbOk],hcNoHelpTopic);
                   MOD45.PasswordReleased:=true;
                   end;
            mrYes: begin
                   MOD45.PasswordReleased:=true;
                   MessageDlg(LanguageStrings[msValidButNoChange],mtInformation,[mbOk],hcNoHelpTopic);
                   end;
            mrIgnore: begin
                      MOD45.SupervisorAccess:=true;  // Master password enabled
                      MOD45.PasswordReleased:=true;
                      MessageDlg(LanguageStrings[msValidButNoChange],mtInformation,[mbOk],hcNoHelpTopic);
                      end;

            mrCancel: begin
                      MOD45.PasswordReleased:=false;
                      MessageDlg(LanguageStrings[msDisabledBecauseWrong],mtError,[mbCancel],hcNoHelpTopic);
                      end;
            else
               begin
               MessageDlg(LanguageStrings[msNoChangeBecauseWrong],mtConfirmation,[mbOk],hcNoHelpTopic);
               end;
            end;
       Release;
       UpdateMenus;
       UpdateStatusBar;
       end;
end;

procedure TfmPcs45Main.OLanguageClick(Sender: TObject);
var UserLanguage, NewLanguage: LCID;
    I,UserLangNumber: integer;
    ChangeLanguage: boolean;
begin
  Application.CreateForm(TfmLanguage, fmLanguage);
  with fmLanguage do
       begin
       SetLanguage(CFG45.Language);
       ShowModal;
       if ModalResult=mrOK then
          begin
          NewLanguage:=GetLanguage;
          if (NewLanguage<>CFG45.Language) then
             begin

             UserLanguage:=GetUserDefaultLCID and $1FF;
             UserLangNumber:=-1;
             for I:=0 to System.Length(LanguageInfos)-1 do
                 if LanguageInfos[I].LangNumber=UserLanguage then UserLangNumber:=I;

             if (NewLanguage<>UserLanguage) and not(UserLangNumber<0) then
                ChangeLanguage:=(MessageDlg(LanguageInfos[UserLangNumber].ChangeWarning,mtConfirmation,[mbYes,mbNo],hcNoHelpTopic)=mrYes)
             else ChangeLanguage:=true;

             if ChangeLanguage then
                begin
                CFG45.Language:=NewLanguage;
                SaveLanguageToRegistry(CFG45.Language);
                TryToLokalize;
                SelAllCaptions;
                UpdateStatusBar;
                end;
             end;
          end;
       Release;
       end;
end;

procedure TfmPcs45Main.OPortSelectionClick(Sender: TObject);
var NewPortSetting: TPortSetting;
begin
  HartMaster1.Close;
  HartMaster1.OnReceiveChar:=nil;
  HartMaster1.OnAnswerSimulationRequest:=nil;

  NewPortSetting.USBSelection:=CFG45.PCL45USBSelected;
  NewPortSetting.CommNumber:=CFG45.PortNumber;

  Application.CreateForm(TfmPortSelection, fmPortSelection);
  with fmPortSelection do
       begin
       SetPortSelection(NewPortSetting);
       ShowModal;
       if ModalResult=mrOK then
          begin
          NewPortSetting:=GetPortSelection;
          if (CFG45.PCL45USBSelected<>NewPortSetting.USBSelection) or
             (CFG45.PortNumber<>NewPortSetting.CommNumber) then
             begin
             CFG45.PCL45USBSelected:=NewPortSetting.USBSelection;
             CFG45.PortNumber:=NewPortSetting.CommNumber;
             SavePortToRegistry(CFG45.PortNumber);
             end;
          end;
       Release;
       end;

  ArrangeCommunicationPort;
end;

procedure TfmPcs45Main.OErrorIndicationClick(Sender: TObject);
begin
  Application.CreateForm(TfmErrorIndication, fmErrorIndication);
  with fmErrorIndication do
       begin
       SetErrorIndication(CFG45.ErrorIndication);
       ShowModal;
       if ModalResult=mrOK then
          begin
          CFG45.ErrorIndication:=GetErrorIndication;
          with HartMaster1 do
               begin
               DefectErrorEnabled:=CFG45.ErrorIndication[0];
               DisruptionErrorEnabled:=CFG45.ErrorIndication[1];
               ExceptionErrorEnabled:=CFG45.ErrorIndication[2];
               end;
          SaveErrorIndicationToRegistry(CFG45.ErrorIndication);
          end;
       Release;
       end;
end;

procedure TfmPcs45Main.OCalculatorClick(Sender: TObject);
begin
  Application.CreateForm(TfmCalculator, fmCalculator);
  with fmCalculator do
       begin
       ShowModal;
       Release;
       end;
end;

procedure TfmPcs45Main.OResetPasswordClick(Sender: TObject);
var NewPassword: string;
begin
  if MOD45.SupervisorAccess then
     begin
     NewPassword:=CDefaultPassword;
     if (CFG45.Password<>NewPassword) then
        begin
        CFG45.Password:=NewPassword;
        SavePasswordToRegistry(CFG45.Password);
        MessageDlg(LanguageStrings[msPasswordSaved],mtInformation,[mbOk],hcNoHelpTopic);
        end
     else MessageDlg(LanguageStrings[msValidButNoChange],mtInformation,[mbOk],hcNoHelpTopic);
     MOD45.PasswordReleased:=true;
     end;
end;

// Submenu Help

procedure TfmPcs45Main.HContentClick(Sender: TObject);
begin
  if (Application.HelpFile<>'') and
     FileExists(Application.HelpFile) then
     Application.HelpShowTableOfContents;
end;

procedure TfmPcs45Main.HAboutClick(Sender: TObject);
begin
  Application.CreateForm(TfmAbout, fmAbout);
  with fmAbout do
       begin
       ShowModal;
       Release;
       end;
end;

end.
