unit PortSelection;

interface

uses Windows, ExtCtrls, StdCtrls, Buttons, Classes, Controls, Forms, GroupControls;

type
  TPortSetting = record
    USBSelection: boolean;
    CommNumber: integer;
    end;

  TfmPortSelection = class(TForm)
    USBSelection: TRadioGroup;
    SerialPortSelection: TMyRadioGroup;
    DemoSelection: TRadioGroup;
    OK_Button: TBitBtn;
    Close_Button: TBitBtn;
    PortTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PortTimerTimer(Sender: TObject);
    procedure USBSelectionClick(Sender: TObject);
    procedure SerialPortSelectionClick(Sender: TObject);
    procedure DemoSelectionClick(Sender: TObject);
  private
    { Private declarations }
    OriginalPortSetting,
    ActualPortSetting: TPortSetting;
    fPCL45USBDetection,
    PCL45USBConnected: boolean;
    procedure ArrangePortSelection;
    function CheckPortSettingChanged: boolean;
  public
    { Public declarations }
    procedure SetPortSelection(Value: TPortSetting);
    function GetPortSelection: TPortSetting;
  end;

var
  fmPortSelection: TfmPortSelection;

implementation

{$R *.dfm}

uses SysUtils, StringUtils, SerialComDevices, SerialFTDevices, DataInstances, DataConstants,
     LabelNumbers, HelpContextNumbers;

procedure TfmPortSelection.SetPortSelection(Value: TPortSetting);
begin
  OriginalPortSetting:=Value;
  ActualPortSetting:=OriginalPortSetting;
end;

function TfmPortSelection.GetPortSelection: TPortSetting;
begin
  Result:=ActualPortSetting;
end;

procedure TfmPortSelection.ArrangePortSelection;
var CommTab: array [1..8] of boolean;
    I, MaxComm: integer;
begin
  PCL45USBConnected:=(GetFTDeviceCount(CPCL45USBName)=1);

  MaxComm:=GetMaxSerialPortNumber;
  if (MaxComm>8) then MaxComm:=8;

  USBSelection.Top:=15;
  USBSelection.Visible:=PCL45USBConnected;

  if (MaxComm>0) then
     with SerialPortSelection do
          begin
          if not(PCL45USBConnected) then Top:=USBSelection.Top
          else Top:=USBSelection.Top+USBSelection.Height+5;

          if not(MaxComm>4) then
             begin
             Columns:=1;
             Height:=MaxComm*20 + 20; // SerialPortSelection.Height

             end
          else
             begin
             Columns:=2;
             Height:=((MaxComm+1) DIV 2)*20+20;  // SerialPortSelection.Height

             // Increase component widths for 2 columns
             Self.Width:=Self.Width+40;
             USBSelection.Width:=USBSelection.Width+40;
             SerialPortSelection.Width:=SerialPortSelection.Width+40;
             DemoSelection.Width:=DemoSelection.Width+40;
             end;

          Items.Clear;
          for I:=1 to MaxComm do
              begin
              Items.Add('COM&'+IntToStr(I));
              ButtonEnable[I]:=CommTab[I] and SerialPortAvailable(I);
              end;
        DemoSelection.Top:=SerialPortSelection.Top+SerialPortSelection.Height;
        end
     else
        begin
        SerialPortSelection.Visible:=false;
        if not(PCL45USBConnected) then DemoSelection.Top:=USBSelection.Top
        else DemoSelection.Top:=USBSelection.Top+USBSelection.Height+5;
        end;

  OK_Button.Top:=DemoSelection.Top+DemoSelection.Height+20;
  OK_Button.Left:=DemoSelection.Left+(DemoSelection.Width-OK_Button.Width) DIV 2;
  Close_Button.Top:=OK_Button.Top+OK_Button.Height+15;
  Close_Button.Left:=OK_Button.Left;
  Self.Height:=Close_Button.Top+80;

  if PCL45USBConnected then
     begin
     USBSelection.ItemIndex:=0;
     SerialPortSelection.ItemIndex:=-1;
     DemoSelection.ItemIndex:=-1;
     // overwrite original port setting to allow update
     OriginalPortSetting.CommNumber:=0;
     end
  else // not(PCL45USBConnected)
     begin
     USBSelection.ItemIndex:=-1;
     if ActualPortSetting.USBSelection or
        (ActualPortSetting.CommNumber=0) or
        (ActualPortSetting.CommNumber>MaxComm) or
        not(CommTab[ActualPortSetting.CommNumber]) then
        begin
        DemoSelection.ItemIndex:=0;
        SerialPortSelection.ItemIndex:=-1;
        end
     else
        begin
        SerialPortSelection.ItemIndex:=(ActualPortSetting.CommNumber-1);
        DemoSelection.ItemIndex:=-1;
        end;
     end;
end;

function TfmPortSelection.CheckPortSettingChanged: boolean;
begin
  Result:=(ActualPortSetting.USBSelection<>OriginalPortSetting.USBSelection) or
          (ActualPortSetting.CommNumber<>OriginalPortSetting.CommNumber);
end;

procedure TfmPortSelection.FormCreate(Sender: TObject);
begin
  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlPortSelection]);
  HelpContext:=hcPortSelection;

  Close_Button.Caption:=LanguageStrings[blClose];
  OK_Button.Caption:=LanguageStrings[blSetting];

  with OriginalPortSetting do
       begin
       USBSelection:=false;
       CommNumber:=0;
       end;
  ActualPortSetting:=OriginalPortSetting;

  PCL45USBConnected:=false;
  fPCL45USBDetection:=false;
end;

procedure TfmPortSelection.FormShow(Sender: TObject);
begin
  ArrangePortSelection;
  OK_Button.Enabled:=CheckPortSettingChanged;
  PortTimer.Enabled:=true;
end;

procedure TfmPortSelection.FormDestroy(Sender: TObject);
begin
  while fPCL45USBDetection do;
  PortTimer.Enabled:=false;
end;

procedure TfmPortSelection.PortTimerTimer(Sender: TObject);
var PCL45USB: integer;
begin
  fPCL45USBDetection:=true;
  PortTimer.Enabled:=false;
  PCL45USB:=GetFTDeviceCount(CPCL45USBName);
  if (PCL45USBConnected and (PCL45USB=0)) or
     (not(PCL45USBConnected) and (PCL45USB=1)) then ArrangePortSelection;
  PortTimer.Enabled:=true;
  fPCL45USBDetection:=false;
end;

procedure TfmPortSelection.USBSelectionClick(Sender: TObject);
begin
  if (USBSelection.ItemIndex=0) then
     begin
     SerialPortSelection.ItemIndex:=-1;
     DemoSelection.ItemIndex:=-1;

     ActualPortSetting.USBSelection:=true;
     ActualPortSetting.CommNumber:=-1;

     OK_Button.Enabled:=CheckPortSettingChanged;
     end;
end;

procedure TfmPortSelection.SerialPortSelectionClick(Sender: TObject);
begin
  if not(SerialPortSelection.ItemIndex<0) then
     begin
     USBSelection.ItemIndex:=-1;
     DemoSelection.ItemIndex:=-1;

     ActualPortSetting.USBSelection:=false;
     ActualPortSetting.CommNumber:=SerialPortSelection.ItemIndex+1;

     OK_Button.Enabled:=CheckPortSettingChanged;
     end;
end;

procedure TfmPortSelection.DemoSelectionClick(Sender: TObject);
begin
  if (DemoSelection.ItemIndex=0) then
     begin
     USBSelection.ItemIndex:=-1;
     SerialPortSelection.ItemIndex:=-1;

     ActualPortSetting.USBSelection:=false;
     ActualPortSetting.CommNumber:=0;

     OK_Button.Enabled:=CheckPortSettingChanged;
     end;
end;

end.
