unit SetAndCalibrate;

interface

uses Windows, StdCtrls, Buttons, Controls, Classes, Gauges, Forms,
     EditControls, DataTypes, HartModules;

type
  TSetCurrentOrCalibrateMode = (smSetCurrent, smCalibrate);
  TCalibrationPhase   = (cpNullPoint,cpEndPoint);

  TfmSetAndCalibrate = class(TForm)
    SetCurrentBox: TGroupBox;
    SetCurrentEdit: TLabelRealEdit;
    ChannelSelectionBox: TGroupBox;
    ChannelLabel: TLabel;
    ChannelSelection: TComboBox;
    Close_Button: TBitBtn;
    OK_Button: TBitBtn;
    CommGauge: TGauge;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure SetCurrentEditChange(Sender: TObject);
    procedure SetCurrentEditRangeValidationError(Sender: TObject);
    procedure ChannelSelectionChange(Sender: TObject);
    procedure OK_ButtonClick(Sender: TObject);
  private
    { Private declarations }
    fDeviceType: TDeviceType;
    pHartModule: PBasicHartModule;
    fTransferActive,
    fDisableTransfer: boolean;
    SetCurrentOrCalibrateMode: TSetCurrentOrCalibrateMode;
    CalibrationPhase: TCalibrationPhase;
    procedure IndicateReceivedChar;
    procedure ConfigureCurrentEdit(RequiredValue: real; Limits: TRealLimits);
    procedure CommonPrefixSetAndCalibrateLook;
    procedure SetAndCalibrateLook_x073;
    procedure SetAndCalibrateLook_x576;
    procedure SetAndCalibrateLook_x532;
    procedure SetAndCalibrateLook_1003;
    procedure CommonSuffixSetAndCalibrateLook;
    function SetRequiredCurrent(Value: single): boolean;
    function CalibratePoint4mA(Value: single): boolean;
    function CalibratePoint20mA(Value: single): boolean;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
    procedure SetMode(Value: TSetCurrentOrCalibrateMode);
  end;

var
  fmSetAndCalibrate: TfmSetAndCalibrate;

implementation

{$R *.dfm}

uses StringUtils, ProcessUtils, DataInstances, LabelNumbers, HelpContextNumbers,
     ConversionProcs;

const
  CReleaseOutput   =  0.0;
  CRangeBeginValue =  4.0;
  CRangeEndValue   = 20.0;

  C_Lim_Set  :TRealLimits=(LLim:  0.000; ULim: 22.250); {for current setting}
  C_Lim_Low  :TRealLimits=(LLim:  3.000; ULim:  5.000); {for low point - 4mA }
  C_Lim_High :TRealLimits=(LLim: 18.000; ULim: 25.000); {for high point - 20 mA}

procedure TfmSetAndCalibrate.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(25);
     end
  else WaitMilliseconds(100);
end;

procedure TfmSetAndCalibrate.ConfigureCurrentEdit(RequiredValue: real; Limits: TRealLimits);
begin
  with SetCurrentEdit do
       begin
       Value:=RequiredValue;
       Min:=C_Lim_Set.LLim;
       Max:=C_Lim_Set.ULim;
       end;
end;

procedure TfmSetAndCalibrate.CommonPrefixSetAndCalibrateLook;
begin
  // Captions and Edits

  SetCurrentBox.Caption:=LanguageStrings[flAnalogueOutput];

  if SetCurrentOrCalibrateMode=smSetCurrent then
     begin
     Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlSetCurrentOutput]);
     HelpContext:=hcSetCurrent;
     SetCurrentEdit.LabelCaption:=LanguageStrings[flRequiredValue];
     ConfigureCurrentEdit(CRangeBeginValue,C_Lim_Set);
     end;
  if SetCurrentOrCalibrateMode=smCalibrate then
     begin
     Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlCalibrateOutput]);
     HelpContext:=hcCalibrate;
     SetCurrentEdit.LabelCaption:=LanguageStrings[flMeasuredValue];
     ConfigureCurrentEdit(CRangeBeginValue,C_Lim_Low);
     CalibrationPhase:=cpNullPoint;
     end;

  SetCurrentEdit.RangeErrorMessage:=LanguageStrings[msFieldBetween];

  OK_Button.Caption:=LanguageStrings[blSetting];
  Close_Button.Caption:=LanguageStrings[blClose];

  // Dimensions

  Self.Width:=275;

  SetCurrentBox.Top:=15;
  SetCurrentBox.Width:=240;
  SetCurrentBox.Height:=25 + 1*30;

  ChannelSelectionBox.Top:=SetCurrentBox.Top+SetCurrentBox.Height+10;
  ChannelSelectionBox.Width:=240;
  ChannelSelectionBox.Height:=45;
  ChannelSelectionBox.Visible:=false;

  CommGauge.Width:=240;
end;

procedure TfmSetAndCalibrate.SetAndCalibrateLook_x073;
begin
  //
  CommGauge.MaxValue:=12;
end;

procedure TfmSetAndCalibrate.SetAndCalibrateLook_x576;
begin
  //
  ChannelSelectionBox.Visible:=true;

  ChannelLabel.Caption:=LanguageStrings[flChannel];

  if (THartModule_x576(pHartModule^).ChannelNumber in [0,1]) then
     ChannelSelection.ItemIndex:=THartModule_x576(pHartModule^).ChannelNumber
  else ChannelSelection.ItemIndex:=0;

  CommGauge.MaxValue:=13;
end;

procedure TfmSetAndCalibrate.SetAndCalibrateLook_x532;
begin
  //
  CommGauge.MaxValue:=12;
end;

procedure TfmSetAndCalibrate.SetAndCalibrateLook_1003;
begin
  //
  CommGauge.MaxValue:=12;
end;

procedure TfmSetAndCalibrate.CommonSuffixSetAndCalibrateLook;
var BT{,BB}: integer;
begin
  if not(ChannelSelectionBox.Visible) then
     CommGauge.Top:=SetCurrentBox.Top+SetCurrentBox.Height+15
  else CommGauge.Top:=ChannelSelectionBox.Top+ChannelSelectionBox.Height+15;

  CommGauge.Progress:=0;

  OK_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Top:=OK_Button.Top;

  BT:=(SetCurrentBox.Width-OK_Button.Width-Close_Button.Width) DIV 4;
  OK_Button.Left:=SetCurrentBox.Left+BT;
  Close_Button.Left:=SetCurrentBox.Left+OK_Button.Width+3*BT;

  OK_Button.Enabled:=false;
  Close_Button.Enabled:=false;

  Self.Height:=Close_Button.Top+80;
end;

function TfmSetAndCalibrate.SetRequiredCurrent(Value: single): boolean;
begin
  Result:=false;
  if (fDeviceType<>dtx576) then
     Result:=pHartModule^.SetOutputCurrent(Value)
  else
     if (fDeviceType=dtx576) then
        Result:=THartModule_x576(pHartModule^).SetOutputCurrent(Value);
end;

function TfmSetAndCalibrate.CalibratePoint4mA(Value: single): boolean;
begin
  Result:=false;
  if (fDeviceType<>dtx576) then
     Result:=pHartModule^.CalibratePoint4mA(Value)
  else
     if (fDeviceType=dtx576) then
        Result:=THartModule_x576(pHartModule^).CalibratePoint4mA(Value);
end;

function TfmSetAndCalibrate.CalibratePoint20mA(Value: single): boolean;
begin
  Result:=false;
  if (fDeviceType<>dtx576) then
     Result:=pHartModule^.CalibratePoint20mA(Value)
  else
     if (fDeviceType=dtx576) then
        Result:=THartModule_x576(pHartModule^).CalibratePoint20mA(Value);
end;

procedure TfmSetAndCalibrate.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmSetAndCalibrate.SetMode(Value: TSetCurrentOrCalibrateMode);
begin
  SetCurrentOrCalibrateMode:=Value;
end;

procedure TfmSetAndCalibrate.FormCreate(Sender: TObject);
begin
  fDeviceType:=dtNone;
  pHartModule:=nil;

  fTransferActive:=false;

  SetCurrentOrCalibrateMode:=smSetCurrent;
end;

procedure TfmSetAndCalibrate.FormShow(Sender: TObject);
begin
  if (pHartModule<>nil) then
     begin
     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);
     if (fDeviceType<>dtNone) then
        begin
        CommonPrefixSetAndCalibrateLook;

        case fDeviceType of
             dt507x,
             dtx073,
             dt5074XK,
             dt4073XK,
             dt4075,
             dtx573,
             dtx575: SetAndCalibrateLook_x073;
             dtx576: SetAndCalibrateLook_x576;
             dtx532: SetAndCalibrateLook_x532;
             dt1003: SetAndCalibrateLook_1003;
             end;

        CommonSuffixSetAndCalibrateLook;
        end
     else Close;
     end
  else Close;
end;

procedure TfmSetAndCalibrate.FormActivate(Sender: TObject);
begin
  fTransferActive:=true;
  if SetRequiredCurrent(CRangeBeginValue) then
     begin
     OK_Button.Enabled:=true;
     Close_Button.Enabled:=true;
     CommGauge.Progress:=0;
     end
  else Close;
  fTransferActive:=false;
end;

procedure TfmSetAndCalibrate.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not(fTransferActive);
end;

procedure TfmSetAndCalibrate.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  OK_Button.Enabled:=false;
  Close_Button.Enabled:=false;

  SetRequiredCurrent(CReleaseOutput);
end;

procedure TfmSetAndCalibrate.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmSetAndCalibrate.SetCurrentEditChange(Sender: TObject);
begin
  fDisableTransfer:=false;
end;

procedure TfmSetAndCalibrate.SetCurrentEditRangeValidationError(Sender: TObject);
begin
  if SetCurrentEdit<>nil then SetCurrentEdit.SetValueToValidRange;
  // Disable transfer after the value was limited to the range
  fDisableTransfer:=true;
end;

procedure TfmSetAndCalibrate.ChannelSelectionChange(Sender: TObject);
begin
  if (fDeviceType=dtx576) and
     (ChannelSelection.ItemIndex in [0,1]) then
     begin
     fTransferActive:=true;
     SetRequiredCurrent(CReleaseOutput);
     WaitMilliseconds(50);
     CommGauge.Progress:=0;
     THartModule_x576(pHartModule^).ChannelNumber:=(ChannelSelection.ItemIndex);
     SetRequiredCurrent(SetCurrentEdit.Value);
     WaitMilliseconds(50);
     CommGauge.Progress:=0;
     fTransferActive:=false;
     end;
end;

procedure TfmSetAndCalibrate.OK_ButtonClick(Sender: TObject);
var SetValue: real;
begin
  if (OK_Button<>nil) and not(OK_Button.Focused) then OK_Button.SetFocus
  else fDisableTransfer:=false;

  if not(fDisableTransfer) then
     begin
     OK_Button.Enabled:=false;
     Close_Button.Enabled:=false;
     if SetCurrentOrCalibrateMode=smSetCurrent then
        begin
        fTransferActive:=true;
        SetValue:=SetCurrentEdit.Value;
        if SetValue=0.0 then SetValue:=0.001;
        SetRequiredCurrent(SetValue);
        SetCurrentEdit.Value:=SetValue;
        WaitMilliseconds(50);
        CommGauge.Progress:=0;
        OK_Button.Enabled:=true;
        Close_Button.Enabled:=true;
        fTransferActive:=false;
        ModalResult:=mrNone;
        end
     else
        if SetCurrentOrCalibrateMode=smCalibrate then
           begin
           if CalibrationPhase=cpNullPoint then
              begin
              // Cannot change channel once the calibration procedure is started
              ChannelSelectionBox.Enabled:=false;

              fTransferActive:=true;

              SetValue:=SetCurrentEdit.Value;
              CalibratePoint4mA(SetValue);
              WaitMilliseconds(50);
              CommGauge.Progress:=0;

              SetValue:=CRangeEndValue;
              ConfigureCurrentEdit(SetValue,C_Lim_High);

              SetRequiredCurrent(SetValue);
              SetCurrentEdit.Value:=SetValue;
              WaitMilliseconds(50);
              CommGauge.Progress:=0;

              OK_Button.Enabled:=true;
              Close_Button.Enabled:=true;

              CalibrationPhase:=cpEndPoint;
              fTransferActive:=false;
              ModalResult:=mrNone;
              end
           else // cpEndPoint
              begin
              fTransferActive:=true;

              SetValue:=SetCurrentEdit.Value;
              CalibratePoint20mA(SetValue);
              WaitMilliseconds(50);
              CommGauge.Progress:=0;

              SetValue:=CReleaseOutput;
              SetRequiredCurrent(SetValue);
              WaitMilliseconds(50);
              CommGauge.Progress:=0;

              fTransferActive:=false;
              // Please note that the form will be closed now
              end;
           end
        else ModalResult:=mrNone;
     end
  else ModalResult:=mrNone;
end;

end.
