unit Status;

{$DEFINE GreyOutNotAvailable} // GreyOutNotAvailable or GreyOutNotEnabled

interface

uses Windows, ExtCtrls, StdCtrls, Buttons, Controls, Classes, Gauges,
     Forms, EditControls, DataTypes, HartDataTypes, HartModules;

type
  TfmStatus = class(TForm)
    StatusInfoBox: TGroupBox;
    StatusByte: TLabelDisplay;
    AlarmInfoBox: TGroupBox;
    FirstAlarmDisplay: TLabelDisplay;
    SecondAlarmDisplay: TLabelDisplay;
    ThirdAlarmDisplay: TLabelDisplay;
    FourthAlarmDisplay: TLabelDisplay;
    FifthAlarmDisplay: TLabelDisplay;
    ChannelSelectionBox: TGroupBox;
    ChannelLabel: TLabel;
    ChannelSelection: TComboBox;
    CommGauge: TGauge;
    Close_Button: TBitBtn;
    StatusTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure ChannelSelectionChange(Sender: TObject);
    procedure StatusTimerTimer(Sender: TObject);
  private
    { Private declarations }
    fDeviceType: TDeviceType;
    pHartModule: PBasicHartModule;
    fTransferActive,
    fTHCSensor,
    fCJCEnabled,
    fEBDEnabled,
    fSCEnabled,
    fLeakFlowEnabled,
    f1003LFDEnabled: boolean;
    fStatusInterval: integer;
    procedure IndicateReceivedChar;
    procedure CommonPrefixStatusLook;
    procedure StatusLook_x073;
    procedure StatusLook_x573;
    procedure StatusLook_x576_THCorRTD;
    procedure StatusLook_x576;
    procedure StatusLook_x532;
    procedure StatusLook_1003;
    procedure CommonSuffixStatusLook;
    procedure SetStatusDisplayState(var Target: TLabelDisplay; Value: boolean);
    procedure SetStatusDisplayInfo(var Target: TLabelDisplay; Value: boolean);
    procedure IndicateStatus_x073(Data: THTransmitterStatus);
    procedure IndicateStatus_x573(Data: THTransmitterStatus);
    procedure IndicateStatus_x576(Data: THTransmitterStatus);
    procedure IndicateStatus_x532(Data: THTransmitterStatus);
    procedure IndicateStatus_1003(Data: THTransmitterStatus);
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
  end;

var
  fmStatus: TfmStatus;

implementation

{$R *.dfm}

uses SysUtils, Graphics, ProcessUtils, StringUtils, DataInstances, LabelNumbers, HartDataConstants,
     HelpContextNumbers, ConversionProcs;

const
  CStatusInterval = 1000;
  CSts576Interval = 2000;

procedure TfmStatus.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     begin
     CommGauge.AddProgress(1);
     WaitMilliseconds(25);
     end
  else WaitMilliseconds(100);
end;

procedure TfmStatus.CommonPrefixStatusLook;
begin
  // Captions and Displays

  Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlStatus]);

  StatusInfoBox.Caption:=LanguageStrings[flStatusInformation];
  StatusByte.LabelCaption:=LanguageStrings[flStatusByte];
  StatusByte.Display:='';

  AlarmInfoBox.Caption:=LanguageStrings[flAlarmInformation];

  FirstAlarmDisplay.Display:='';
  SecondAlarmDisplay.Display:='';
  ThirdAlarmDisplay.Display:='';
  FourthAlarmDisplay.Display:='';
  FifthAlarmDisplay.Display:='';

  Close_Button.Caption:=LanguageStrings[blClose];

  // Dimensions

  Self.Width:=275;

  StatusInfoBox.Top:=15;
  StatusInfoBox.Width:=240;
  StatusInfoBox.Height:=25 + 1*30;

  AlarmInfoBox.Top:=StatusInfoBox.Top+StatusInfoBox.Height+10;
  AlarmInfoBox.Width:=240;

  ChannelSelectionBox.Width:=240;
  ChannelSelectionBox.Height:=45;
  ChannelSelectionBox.Visible:=false;

  CommGauge.Width:=240;
end;

procedure TfmStatus.StatusLook_x073;
begin
  // DisplayCaptions

  FirstAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmTX];
  SecondAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmOC];
  ThirdAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmCJC];

  SetStatusDisplayState(ThirdAlarmDisplay,fCJCEnabled);           // CJC alarm enabled only when configured

  FourthAlarmDisplay.Visible:=false;
  FifthAlarmDisplay.Visible:=false;

  ChannelSelectionBox.Visible:=false;

  CommGauge.MaxValue:=11;

  // Dimensions

  AlarmInfoBox.Height:=25 + 3*30;
end;

procedure TfmStatus.StatusLook_x573;
begin
  // DisplayCaptions

  FirstAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmTX];
  SecondAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmOC];

  ThirdAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmSC];
  SetStatusDisplayState(ThirdAlarmDisplay,fSCEnabled);             // SC alarm enabled only when configured

  FourthAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmCJC];
  SetStatusDisplayState(FourthAlarmDisplay,fCJCEnabled);           // CJC alarm enabled only when configured

  FifthAlarmDisplay.LabelCaption:=LanguageStrings[flEBDdetection];
  SetStatusDisplayState(FifthAlarmDisplay,fEBDEnabled);            // EBD alarm enabled only when configured

  ChannelSelectionBox.Visible:=false;

  CommGauge.MaxValue:=13;

  // Dimensions

  AlarmInfoBox.Height:=25 + 5*30;
end;

procedure TfmStatus.StatusLook_x576_THCorRTD;
begin
  SetStatusDisplayState(ThirdAlarmDisplay,fSCEnabled);     // SC alarm enabled only when configured
  SetStatusDisplayState(FourthAlarmDisplay,fCJCEnabled);   // CJC alarm enabled only when configured
  SetStatusDisplayState(FifthAlarmDisplay,fEBDEnabled);    // EBD alarm enabled only when configured
end;

procedure TfmStatus.StatusLook_x576;
begin
  // DisplayCaptions

  FirstAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmTX];
  SecondAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmOC];

  ThirdAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmSC];
  FourthAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmCJC];
  FifthAlarmDisplay.LabelCaption:=LanguageStrings[flEBDdetection];

  StatusLook_x576_THCorRTD;

  ChannelLabel.Caption:=LanguageStrings[flChannel];
  ChannelSelectionBox.Visible:=true;

  if (THartModule_x576(pHartModule^).ChannelNumber in [0,1]) then
     ChannelSelection.ItemIndex:=THartModule_x576(pHartModule^).ChannelNumber
  else ChannelSelection.ItemIndex:=0;

  CommGauge.MaxValue:=14;

  // Dimensions

  AlarmInfoBox.Height:=25 + 5*30;
  ChannelSelectionBox.Top:=AlarmInfoBox.Top+AlarmInfoBox.Height+10;
end;

procedure TfmStatus.StatusLook_x532;
begin
  // DisplayCaptions

  FirstAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmTX];
  SecondAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmLeakFlow];
  SetStatusDisplayState(SecondAlarmDisplay,fLeakFlowEnabled); // LeakFlow alarm enabled only when configured

  ThirdAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmTrip];
  FourthAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmLFD];

  FifthAlarmDisplay.Visible:=false;

  CommGauge.MaxValue:=13;

  // Dimensions

  AlarmInfoBox.Height:=25 + 4*30;
end;

procedure TfmStatus.StatusLook_1003;
begin
  // DisplayCaptions

  FirstAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmTX];
  SecondAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmLeakFlow];
  SetStatusDisplayState(SecondAlarmDisplay,fLeakFlowEnabled); // LeakFlow alarm enabled only when configured
  ThirdAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmTrip];
  FourthAlarmDisplay.LabelCaption:=LanguageStrings[flAlarmLFD];
  SetStatusDisplayState(FourthAlarmDisplay,f1003LFDEnabled);      // LFD alarm enabled only when configured

  FifthAlarmDisplay.Visible:=false;

  CommGauge.MaxValue:=12 + integer(f1003LFDEnabled);

  // Dimensions

  AlarmInfoBox.Height:=25 + 4*30;
end;

procedure TfmStatus.CommonSuffixStatusLook;
begin
  if not(ChannelSelectionBox.Visible) then
     CommGauge.Top:=AlarmInfoBox.Top+AlarmInfoBox.Height+15
  else CommGauge.Top:=ChannelSelectionBox.Top+ChannelSelectionBox.Height+15;

  CommGauge.Progress:=0;

  Close_Button.Top:=CommGauge.Top+CommGauge.Height+20;
  Close_Button.Left:=(Self.Width-Close_Button.Width) DIV 2;

  Self.Height:=Close_Button.Top+80;
end;

procedure TfmStatus.SetStatusDisplayState(var Target: TLabelDisplay; Value: boolean);
begin
  if Value then
     begin
     Target.DisplayColor:=clLime;
     Target.Enabled:=true;
     end
  else
     begin
     Target.DisplayColor:=clBtnFace;
     Target.Enabled:=false;
     Target.Display:='';
     end;
end;

procedure TfmStatus.SetStatusDisplayInfo(var Target: TLabelDisplay; Value: boolean);
begin
  if Value then
     begin
     Target.DisplayColor:=clRed;
     Target.Display:=LanguageStrings[flOn];
     end
  else
     begin
     Target.DisplayColor:=clLime;
     Target.Display:=LanguageStrings[flOff];
     end;
end;

procedure TfmStatus.IndicateStatus_x073(Data: THTransmitterStatus);
begin
  // Information order:
  // 0 - open circuit failure alarm
  // 1 - transmitter failure alarm
  // 2 - cold junction failure alarm

  // Display order
  // Alarm TX  - 1
  // Alarm OC  - 0
  // Alarm CJC - 2
  SetStatusDisplayInfo(FirstAlarmDisplay, Data.Status[1]='T');
  SetStatusDisplayInfo(SecondAlarmDisplay, Data.Status[0]='T');
  if fCJCEnabled then
     SetStatusDisplayInfo(ThirdAlarmDisplay, Data.Status[2]='T');
end;

procedure TfmStatus.IndicateStatus_x573(Data: THTransmitterStatus);
begin
  // Information order:
  // 0 - open circuit failure alarm
  // 1 - transmitter failure alarm
  // 2 - cold junction failure alarm
  // 3 - RTD short circuit failure alarm
  // 4 - EBD failure alarm

  // Display order
  // Alarm TX  - 1
  // Alarm OC  - 0
  // Alarm SC  - 3
  // Alarm CJC - 2
  // Alarm EBD - 4
  SetStatusDisplayInfo(FirstAlarmDisplay, Data.Status[1]='T');
  SetStatusDisplayInfo(SecondAlarmDisplay, Data.Status[0]='T');

  if fSCEnabled then
     SetStatusDisplayInfo(ThirdAlarmDisplay, Data.Status[3]='T');

  if fCJCEnabled then
     SetStatusDisplayInfo(FourthAlarmDisplay, Data.Status[2]='T');

  if fEBDEnabled then
     SetStatusDisplayInfo(FifthAlarmDisplay, Data.Status[4]='T');
end;

procedure TfmStatus.IndicateStatus_x576(Data: THTransmitterStatus);
begin
  // Information order:
  // 0 - channel number, shall be ignored
  // 1 - open circuit failure alarm
  // 2 - short circuit failure alarm - only RTD or incl. THC ?
  // 3 - transmitter failure alarm
  // 4 - cold junction failure alarm
  // 5 - EBD/Trip failure alarm

  // Display order
  // Alarm TX  - 2
  // Alarm OC  - 1
  // Alarm SC  - 4
  // Alarm CJC - 3
  // Alarm EBD - 5
  SetStatusDisplayInfo(FirstAlarmDisplay, Data.Status[2]='T');
  SetStatusDisplayInfo(SecondAlarmDisplay, Data.Status[1]='T');

  if fSCEnabled then
     SetStatusDisplayInfo(ThirdAlarmDisplay, Data.Status[4]='T');
  if fCJCEnabled then
     SetStatusDisplayInfo(FourthAlarmDisplay, Data.Status[3]='T');
  if fEBDEnabled then
     SetStatusDisplayInfo(FifthAlarmDisplay, Data.Status[5]='T');
end;

procedure TfmStatus.IndicateStatus_x532(Data: THTransmitterStatus);
begin
  // Information order:
  // 0 - transmitter failure alarm
  // 1 - Trip on Alarm Output
  // 2 - Trip on Pulse Output
  // 3 - LFD alarm
  // 4 - leak flow off alarm (only when enabled and present)

  // Display order
  // Alarm TX         - 0
  // Alarm LeakFlow   - 4 or none
  // Alarm Trip Alarm - 1 or 2
  // Alarm LFD Alarm  - 3
  SetStatusDisplayInfo(FirstAlarmDisplay, Data.Status[0]='T');
  if fLeakFlowEnabled then
     SetStatusDisplayInfo(SecondAlarmDisplay, Data.Status[4]='T');
  SetStatusDisplayInfo(ThirdAlarmDisplay, (Data.Status[1]='T') or (Data.Status[2]='T'));
  SetStatusDisplayInfo(FourthAlarmDisplay, Data.Status[3]='T');
end;

procedure TfmStatus.IndicateStatus_1003(Data: THTransmitterStatus);
begin
  // Information order:
  // 0 - transmitter failure alarm
  // 1 - leak flow off alarm
  // 2 - Trip A alarm
  // 3 - Trip B alarm
  // 4 - LFD alarm (only for the dt1003 when LFD enabled)

  // Display order
  // Alarm TX         - 0
  // Alarm LeakFlow   - 1
  // Alarm Trip Alarm - 2 or 3
  // Alarm LFD Alarm  - 4 or none
  SetStatusDisplayInfo(FirstAlarmDisplay, Data.Status[0]='T');
  if fLeakFlowEnabled then
     SetStatusDisplayInfo(SecondAlarmDisplay, Data.Status[1]='T');
  SetStatusDisplayInfo(ThirdAlarmDisplay, (Data.Status[2]='T') or (Data.Status[3]='T'));
  if f1003LFDEnabled then
     SetStatusDisplayInfo(FourthAlarmDisplay, Data.Status[4]='T');
end;

procedure TfmStatus.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmStatus.FormCreate(Sender: TObject);
begin
  fDeviceType:=dtNone;
  pHartModule:=nil;

  fTransferActive:=false;
  fTHCSensor:=false;
  fCJCEnabled:=false;
  fEBDEnabled:=false;
  fSCEnabled:=false;
  fLeakFlowEnabled:=false;
  f1003LFDEnabled:=false;

  StatusTimer.Enabled:=false;

  HelpContext:=hcStatus;
end;

procedure TfmStatus.FormShow(Sender: TObject);
var lActualChannel: byte;
begin
  CommonPrefixStatusLook;

  if (pHartModule<>nil) then
     begin
     fDeviceType:=IdentToType(pHartModule^.BasicDeviceData.UID);
     if (fDeviceType<>dtNone) then
        begin
        if (fDeviceType=dtx576) then
           lActualChannel:=THartModule_x576(pHartModule^).ChannelNumber
        else lActualChannel:=0;

        fTHCSensor:=(fDeviceType in CTemperatureConverter) and
                    (((fDeviceType in CSingleChannel) and
                      (InputTypeFromHART(fDeviceType,THartModule_x073(pHartModule^).x073ParameterSet.InputType)=itTHC)) or
                     ((fDeviceType in CDualChannel) and
                      (InputTypeFromHART(fDeviceType,THartModule_x576(pHartModule^).InputParameters[lActualChannel].SensorType)=itTHC)));

{$IFDEF GreyOutNotAvailable}
// Status indicator for alarm signals which are not available in the actual operation mode will be greyed out (shown inactive)
// For example CJC alarm will be greyed out with RTD sensor or with THC sensor but without activated CJC option.

        fCJCEnabled:=fTHCSensor and
                     (((fDeviceType in CSingleChannel) and
                       (CJCompFromHART(THartModule_x073(pHartModule^).x073ParameterSet.ColdJunctionCompensationOnOff)<>tsOff)) or
                      ((fDeviceType in CDualChannel) and
                       (CJCompx576FromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].CJC_Option)<>csOff)));

        fEBDEnabled:=fTHCSensor and
                     (fDeviceType in CWithEBDDetection);

        fSCEnabled:=not(fTHCSensor) and
                    (((fDeviceType=dtx573) and
                      (LinearFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.TransferFunction)=tsOn)) or
                     ((fDeviceType=dtx576) and
                      (LinearFromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].Linearisation)=tsOn)));

        fLeakFlowEnabled:=(fDeviceType=dtx532) or (fDeviceType=dt1003);
        
        f1003LFDEnabled:=(fDeviceType=dt1003);
{$ENDIF}

{$IFDEF GreyOutNotEnabled}
// Status indicator for alarm signals which are not enabled in the actual operation mode will be greyed out (shown inactive)
// For example CJC alarm will be greyed out with RTD sensor or with THC sensor and activated CJC option but disabled CJC alarm.
        fCJCEnabled:=fTHCSensor and
                     (((fDeviceType in CSingleChannel) and
                       (CJCompFromHART(THartModule_x073(pHartModule^).x073ParameterSet.ColdJunctionCompensationOnOff)<>tsOff)  and
                       (AlarmLevelFromHART(THartModule_x073(pHartModule^).x073ParameterSet.ColdJunctionFailureAlarmLevel)<>taOff)) or
                      ((fDeviceType in CDualChannel) and
                       (CJCompx576FromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].CJC_Option)<>csOff) and
                       (AlarmLevelFromHART(THartModule_x576(pHartModule^).OutoutParameters[lActualChannel].ColdJunctionFailureAlarmLevel)<>taOff)));

        fEBDEnabled:=fTHCSensor and
                     (fDeviceType in CWithEBDDetection) and
                     (((fDeviceType in CSingleChannel) and
                       (x075TripTypeFromHART(THartModule_x075(pHartModule^).x075TripParameters.TripOutputFunction)=ttEBD)) or
                      ((fDeviceType in CDualChannel) and
                       (x576EBDOptionFromHART(THartModule_x576(pHartModule^).OutputParameters[lActualChannel].EBD_Option)=tsOn)));

        fSCEnabled:=not(fTHCSensor) and
                    (((fDeviceType=dtx573) and
                      (LinearFromHART(TSingleChannelHartModule(pHartModule^).PrimaryVariableOutputData.TransferFunction)=tsOn) and
                      (AlarmLevelFromHART(THartModule_x573(pHartModule^).ShortCircuitAlarmLevel)<>taOff)) or
                     ((fDeviceType=dtx576) and
                      (LinearFromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].Linearisation)=tsOn) and
                      (AlarmLevelFromHART(THartModule_x576(pHartModule^).OutputParameters[lActualChannel].ShortCircuitAlarmLevel)<>taOff)));

        fLeakFlowEnabled:=((fDeviceType=dtx532) and
                               (LeakFlowModeFromHART(THartModule_x532(pHartModule^).LeakFlowOffParameters.FunctionSelect)=tsOn)) or
                              ((fDeviceType=dt1003) and
                               (LeakFlowModeFromHART(GetLeakFlowParameters_x532(THartModule_1003(pHartModule^).LeakFlowOffParameters))=tsOn));

        f1003LFDEnabled:=(fDeviceType=dt1003) and
                         (LFDMode1003FromHART(THartModule_1003(pHartModule^).InputAndLFDMode)=tsOn);
{$ENDIF}


        case fDeviceType of
             dt507x,
             dtx073,
             dt5074XK,
             dt4073XK,
             dt4075,
             dtx575: StatusLook_x073;
             dtx573: StatusLook_x573;
             dtx576: StatusLook_x576;
             dtx532: StatusLook_x532;
             dt1003: StatusLook_1003;
             end;

        StatusTimer.Interval:=100;
        StatusTimer.Enabled:=true;
        end;
     end;

  if (StatusTimer.Enabled) then
     CommonSuffixStatusLook
  else
     begin
     Close;
     end;
end;

procedure TfmStatus.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not(fTransferActive);
end;

procedure TfmStatus.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
end;

procedure TfmStatus.ChannelSelectionChange(Sender: TObject);
var lActualChannel: byte;
begin
  if (fDeviceType=dtx576) and
     (ChannelSelection.ItemIndex in [0,1]) then
     begin
     lActualChannel:=ChannelSelection.ItemIndex;
     THartModule_x576(pHartModule^).ChannelNumber:=lActualChannel;

     fTHCSensor:=(InputTypeFromHART(fDeviceType,THartModule_x576(pHartModule^).InputParameters[lActualChannel].SensorType)=itTHC);

{$IFDEF GreyOutNotAvailable}
// Status indicator for alarm signals which are not available in the actual operation mode will be greyed out (shown inactive)
// For example CJC alarm will be greyed out with RTD sensor or with THC sensor but without activated CJC option.

     fCJCEnabled:=fTHCSensor and
                  (CJCompx576FromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].CJC_Option)<>csOff);

     fEBDEnabled:=fTHCSensor;

     fSCEnabled:=not(fTHCSensor) and
                    (LinearFromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].Linearisation)=tsOn);
{$ENDIF}

{$IFDEF GreyOutNotEnabled}
// Status indicator for alarm signals which are not enabled in the actual operation mode will be greyed out (shown inactive)
// For example CJC alarm will be greyed out with RTD sensor or with THC sensor and activated CJC option but disabled CJC alarm.

     fCJCEnabled:=fTHCSensor and
                  (CJCompx576FromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].CJC_Option)<>csOff) and
                  (AlarmLevelFromHART(THartModule_x576(pHartModule^).OutoutParameters[lActualChannel].ColdJunctionFailureAlarmLevel)<>taOff)));

     fEBDEnabled:=fTHCSensor and
                  (x576EBDOptionFromHART(THartModule_x576(pHartModule^).OutputParameters[lActualChannel].EBD_Option)=tsOn);

     fSCEnabled:=not(fTHCSensor) and
                     (LinearFromHART(THartModule_x576(pHartModule^).InputParameters[lActualChannel].Linearisation)=tsOn) and
                     (AlarmLevelFromHART(THartModule_x576(pHartModule^).OutputParameters[lActualChannel].ShortCircuitAlarmLevel)<>taOff);

{$ENDIF}

     StatusLook_x576_THCorRTD;
     end;
end;

procedure TfmStatus.StatusTimerTimer(Sender: TObject);
var StatusServiceDuration: dword;
    StsData: THTransmitterStatus;
    StatusInfo: string;
begin
  StatusServiceDuration:=GetTickCount;
  fTransferActive:=true;
  StatusTimer.Enabled:=false;

  if (pHartModule<>nil) and
     ((fDeviceType<>dtx576) and (pHartModule^.GeTHTransmitterStatus(StsData)))  or
     ((fDeviceType=dtx576) and (THartModule_x576(pHartModule^).GeTHTransmitterStatus(StsData))) then
     begin
     if ((StsData.ComInfo and $80)>0) then            // Communication Error Summary
        StatusInfo:=' $'+IntToHex(StsData.ComInfo,2)
     else StatusInfo:='  '+IntToStr(StsData.ComInfo); // or Command Response Code
     StatusInfo:=StatusInfo+', $'+IntToHex(StsData.DevInfo,2);
     StatusByte.Display:=StatusInfo;

     case fDeviceType of
          dt507x,
          dtx073,
          dt5074XK,
          dt4073XK,
          dt4075,
          dtx575: IndicateStatus_x073(StsData);
          dtx573: IndicateStatus_x573(StsData);
          dtx576: IndicateStatus_x576(StsData);
          dtx532: IndicateStatus_x532(StsData);
          dt1003: IndicateStatus_1003(StsData);
          end;

     WaitMilliseconds(50);
     CommGauge.Progress:=0;
     end;

  StatusServiceDuration:=GetTickCount-StatusServiceDuration;

  if (fDeviceType<>dtx576) then
     fStatusInterval:=CStatusInterval-integer(StatusServiceDuration)
  else fStatusInterval:=CSts576Interval-integer(StatusServiceDuration);
  while (fStatusInterval<50) do
        fStatusInterval:=fStatusInterval+CStatusInterval;

  StatusTimer.Interval:=fStatusInterval;
  StatusTimer.Enabled:=true;
  fTransferActive:=false;
end;

end.
