unit Transfer;

interface

uses Windows, ExtCtrls, Gauges, Classes, Controls, StdCtrls, Forms,
     DataTypes, HartModules;

type
  TTransferJob = (tjNone,tjEstablish,tjCompare,tjSave);

  TfmTransfer = class(TForm)
    Label1: TLabel;
    CommGauge: TGauge;
    TransferTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TransferTimerTimer(Sender: TObject);
  private
    { Private declarations }
    DeviceType: TDeviceType;
    TransferJob: TTransferJob;
    SaveMode: TSaveMode;
    pHartModule: PBasicHartModule;
    procedure IndicateReceivedChar;
  public
    { Public declarations }
    procedure AssignHartModule(Value: PBasicHartModule);
    procedure SetMode(DType: TDeviceType; DJob: TTransferJob; SMode: TSaveMode);
  end;

var
  fmTransfer: TfmTransfer;

implementation

{$R *.dfm}

uses Dialogs, StringUtils, Graphics, ProcessUtils, DataInstances, LabelNumbers, HelpContextNumbers;

type       
  TJobPhase    = (jpStart,jpEnd);

var JobPhase: TJobPhase;

procedure TfmTransfer.IndicateReceivedChar;
begin
  if CommGauge.Progress < CommGauge.MaxValue then
     CommGauge.AddProgress(1)
  else WaitMilliseconds(100);
end;

procedure TfmTransfer.AssignHartModule(Value: PBasicHartModule);
begin
  pHartModule:=Value;
  if (pHartModule<>nil) then
     pHartModule^.AssignReceivedCharEvent(IndicateReceivedChar);
end;

procedure TfmTransfer.SetMode(DType: TDeviceType; DJob: TTransferJob; SMode: TSaveMode);
begin
  DeviceType:=DType; TransferJob:=DJob; SaveMode:=SMode;
end;

procedure TfmTransfer.FormCreate(Sender: TObject);
begin
  Caption:=' '+LanguageStrings[flDataTransfer];
  Label1.Caption:=LanguageStrings[flDataExchanging];

  HelpContext:=hcTransfer;

  TransferJob:=tjNone;
  CommGauge.Progress:=0;

  pHartModule:=nil;
end;

procedure TfmTransfer.FormShow(Sender: TObject);
begin
  Cursor:=crHourglass;
  if (pHartModule<>nil) then
     JobPhase:=jpStart
  else JobPhase:=jpEnd;
  TransferTimer.Enabled:=true;
end;

procedure TfmTransfer.FormDestroy(Sender: TObject);
begin
  if (pHartModule<>nil) then
     pHartModule^.UnAssignReceivedCharEvent;
  Cursor:=OriginalCursor;
end;

procedure TfmTransfer.TransferTimerTimer(Sender: TObject);
var fExpectedTransferDataLength: integer;
begin
  TransferTimer.Enabled:=false;
  if JobPhase=jpStart then
     begin
     case TransferJob of
          tjEstablish: begin
                       Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlEstablish]);
                       HelpContext:=hcEstablish;
                       Application.ProcessMessages;

                       fExpectedTransferDataLength:=pHartModule^.GetModuleTransferDataLength(tdDownload);
                       CommGauge.MaxValue:=fExpectedTransferDataLength;
                       pHartModule^.DownloadActualDeviceData;

                       WaitMilliseconds(500);

                       if (pHartModule^.DeviceDataAvailableStatus) then
                          begin
                          pHartModule^.SynchronizeRequiredDeviceData;
                          ModalResult:=mrOK;
                          end
                       else
                          begin
                          MessageDlg(LanguageStrings[msTransferError],mtError,[mbCancel],hcNoHelpTopic);
                          ModalResult:=mrCancel;
                          end;
                       end;
          tjCompare:   begin
                       Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlVerifyDevice]);
                       HelpContext:=hcCompareToDevice;
                       Application.ProcessMessages;

                       fExpectedTransferDataLength:=pHartModule^.GetModuleTransferDataLength(tdDownload);
                       CommGauge.MaxValue:=fExpectedTransferDataLength;
                       pHartModule^.DownloadActualDeviceData;

                       WaitMilliseconds(500);

                       if (pHartModule^.DeviceDataAvailableStatus) and
                          (pHartModule^.VerifyDeviceData) then
                          ModalResult:=mrOK
                       else ModalResult:=mrCancel;
                       end;
          tjSave:      begin
                       Caption:=' '+DeleteAcceleratorKey(LanguageStrings[mlSaveToDevice]);
                       HelpContext:=hcSaveToDevice;
                       Application.ProcessMessages;

                       fExpectedTransferDataLength:=pHartModule^.GetModuleTransferDataLength(tdDownload);
                       CommGauge.MaxValue:=fExpectedTransferDataLength;
                       pHartModule^.DownloadActualDeviceData;

                       WaitMilliseconds(250);
                       CommGauge.Progress:=0;
                       WaitMilliseconds(250);

                       pHartModule^.EvaluateRequiredDeviceDataStatusFlags;

                       fExpectedTransferDataLength:=pHartModule^.GetModuleTransferDataLength(tdUpload);
                       CommGauge.MaxValue:=fExpectedTransferDataLength;

                       if (fExpectedTransferDataLength>0) then
                          begin
                          pHartModule^.UploadRequiredDeviceData;

                          WaitMilliseconds(500);

                          if (pHartModule^.VerifyDeviceData) then
                             ModalResult:=mrOK
                          else ModalResult:=mrCancel;

                          end
                       else ModalResult:=mrOK;
                       end;
          end;

     JobPhase:=jpEnd;
     TransferTimer.Enabled:=true;
     end
  else Close;
end;

end.
