//******************************************************************************
//******************************************************************************
//******************************************************************************
//**
//**  DYNAMIC ARRAYS AND GRIDS (TWO DIMENSION ARRAYS)
//**  ARRAYSANDGRIDS.PAS (former GHCLASSES.PAS and GRIDUTILS)
//**
//**  Collection of helpful dynamic array and grid definitions and procedures
//**  It contains also some transfer procedures to and from Delphi String Grid
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.07.06 / 05.03.08 / 14.04.09 / 18.12.09 / 19.01.10 / 12.06.13
//**
//******************************************************************************
//******************************************************************************
//******************************************************************************
unit ArraysAndGrids;

interface

uses Types, Grids;

//*******************************************************************************
//***
//*** ATTENTION! In the Delphi Grid word, the cell item addressing sequence is:
//***
//*** ROW NUMBER, COLUMN NUMBER - All Delphi Grid procedures use this sequence
//***
//*** In contrary, the Delphi String Grid or Excel cell addressing sequence is:
//***
//*** COLUMN NUMBER, ROW NUMBER - All Excel object procedures use this sequence
//***
//*******************************************************************************

type

  TBooleanDynGrid   = array of TBooleanDynArray;
  TBooleanDyn3Dgrid = array of TBooleanDynGrid;
  
  TCharDynArray     = array of char;
  TCharDynGrid      = array of TCharDynArray;
  TCharDyn3DGrid    = array of TCharDynGrid;

  TByteDynGrid      = array of TByteDynArray;
  TByteDyn3DGrid    = array of TByteDynGrid;

  TIntegerDynGrid   = array of TIntegerDynArray;
  TIntegerDyn3DGrid = array of TIntegerDynGrid;

  TWordDynGrid      = array of TWordDynArray;
  TWordDyn3DGrid    = array of TWordDynGrid;

  TRealDynArray     = array of real;
  TRealDynGrid      = array of TRealDynArray;
  TRealDyn3DGrid    = array of TRealDynGrid;

  TSingleDynGrid    = array of TSingleDynArray;
  TSingleDyn3DGrid  = array of TSingleDynGrid;

  TDoubleDynGrid    = array of TDoubleDynArray;
  TDoubleDyn3DGrid  = array of TDoubleDynGrid;

  TStringDynGrid    = array of TStringDynArray;
  TStringDyn3DGrid  = array of TStringDynGrid;

  TGridSize = record
    ColCount,
    RowCount: integer;
    end;

// Array Procedures and Functions

  procedure ClearDynArray(var TargetArray: TBooleanDynArray); overload;
  procedure ClearDynArray(var TargetArray: TCharDynArray); overload;
  procedure ClearDynArray(var TargetArray: TByteDynArray); overload;
  procedure ClearDynArray(var TargetArray: TIntegerDynArray); overload;
  procedure ClearDynArray(var TargetArray: TWordDynArray); overload;
  procedure ClearDynArray(var TargetArray: TRealDynArray); overload;
  procedure ClearDynArray(var TargetArray: TDoubleDynArray); overload;
  procedure ClearDynArray(var TargetArray: TStringDynArray); overload;

  function AppendElementToArray(Element: boolean; var TargetArray: TBooleanDynArray): integer; overload;
  function AppendElementToArray(Element: char; var TargetArray: TCharDynArray): integer; overload;
  function AppendElementToArray(Element: byte; var TargetArray: TByteDynArray): integer; overload;
  function AppendElementToArray(Element: integer; var TargetArray: TIntegerDynArray): integer; overload;
  function AppendElementToArray(Element: word; var TargetArray: TWordDynArray): integer; overload;
  function AppendElementToArray(Element: real; var TargetArray: TRealDynArray): integer; overload;
  function AppendElementToArray(Element: double; var TargetArray: TDoubleDynArray): integer; overload;
  function AppendElementToArray(Element: string; var TargetArray: TStringDynArray): integer; overload;

  function CheckIfElementPresent(Element: boolean; DynArray: TBooleanDynArray): boolean; overload;
  function CheckIfElementPresent(Element: char; DynArray: TCharDynArray): boolean; overload;
  function CheckIfElementPresent(Element: byte; DynArray: TByteDynArray): boolean; overload;
  function CheckIfElementPresent(Element: integer; DynArray: TIntegerDynArray): boolean; overload;
  function CheckIfElementPresent(Element: word; DynArray: TWordDynArray): boolean; overload;
  function CheckIfElementPresent(Element: real; DynArray: TRealDynArray): boolean; overload;
  function CheckIfElementPresent(Element: double; DynArray: TDoubleDynArray): boolean; overload;
  function CheckIfElementPresent(Element: string; DynArray: TStringDynArray): boolean; overload;

  function AppendArrayToArray(SourceArray: TIntegerDynArray; var TargetArray: TIntegerDynArray): integer; overload;
  function AppendArrayToArray(SourceArray: TRealDynArray; var TargetArray: TRealDynArray): integer; overload;
  function AppendArrayToArray(SourceArray: TStringDynArray; var TargetArray: TStringDynArray): integer; overload;

  procedure DestroyDynArray(var TargetArray: TBooleanDynArray); overload;
  procedure DestroyDynArray(var TargetArray: TCharDynArray); overload;
  procedure DestroyDynArray(var TargetArray: TByteDynArray); overload;
  procedure DestroyDynArray(var TargetArray: TIntegerDynArray); overload;
  procedure DestroyDynArray(var TargetArray: TWordDynArray); overload;
  procedure DestroyDynArray(var TargetArray: TRealDynArray); overload;
  procedure DestroyDynArray(var TargetArray: TDoubleDynArray); overload;
  procedure DestroyDynArray(var TargetArray: TStringDynArray); overload;

  function ConvertConstToIntegerDynArray(Source: array of const): TIntegerDynArray;
  function GetIndexOfFirstStringInArray(Value: string; Source: TStringDynArray): integer;

// Grid Procedures and Functions

  function GetGridRowRect(GridSize: TGridSize; RowNumber: integer): TRect;
  function GetGridColumnRect(GridSize: TGridSize; ColNumber: integer): TRect;

  procedure ClearDynGrid(var TargetGrid: TBooleanDynGrid); overload;
  procedure ClearDynGrid(var TargetGrid: TCharDynGrid); overload;
  procedure ClearDynGrid(var TargetGrid: TByteDynGrid); overload;
  procedure ClearDynGrid(var TargetGrid: TIntegerDynGrid); overload;
  procedure ClearDynGrid(var TargetGrid: TWordDynGrid); overload;
  procedure ClearDynGrid(var TargetGrid: TRealDynGrid); overload;
  procedure ClearDynGrid(var TargetGrid: TDoubleDynGrid); overload;
  procedure ClearDynGrid(var TargetGrid: TStringDynGrid); overload;

  function AppendArrayToGrid(SourceArray: TBooleanDynArray; var TargetGrid: TBooleanDynGrid): integer; overload;
  function AppendArrayToGrid(SourceArray: TCharDynArray; var TargetGrid: TCharDynGrid): integer; overload;
  function AppendArrayToGrid(SourceArray: TByteDynArray; var TargetGrid: TByteDynGrid): integer; overload;
  function AppendArrayToGrid(SourceArray: TIntegerDynArray; var TargetGrid: TIntegerDynGrid): integer; overload;
  function AppendArrayToGrid(SourceArray: TWordDynArray; var TargetGrid: TWordDynGrid): integer; overload;
  function AppendArrayToGrid(SourceArray: TRealDynArray; var TargetGrid: TRealDynGrid): integer; overload;
  function AppendArrayToGrid(SourceArray: TDoubleDynArray; var TargetGrid: TDoubleDynGrid): integer; overload;
  function AppendArrayToGrid(SourceArray: TStringDynArray; var TargetGrid: TStringDynGrid): integer; overload;

  procedure DestroyDynGrid(var TargetGrid: TBooleanDynGrid); overload;
  procedure DestroyDynGrid(var TargetGrid: TCharDynGrid); overload;
  procedure DestroyDynGrid(var TargetGrid: TByteDynGrid); overload;
  procedure DestroyDynGrid(var TargetGrid: TIntegerDynGrid); overload;
  procedure DestroyDynGrid(var TargetGrid: TWordDynGrid); overload;
  procedure DestroyDynGrid(var TargetGrid: TRealDynGrid); overload;
  procedure DestroyDynGrid(var TargetGrid: TDoubleDynGrid); overload;
  procedure DestroyDynGrid(var TargetGrid: TStringDynGrid); overload;

  procedure SetDynGridSize(var TargetGrid: TBooleanDynGrid; GridSize: TGridSize); overload;
  procedure SetDynGridSize(var TargetGrid: TBooleanDynGrid; RowCount,ColCount: integer); overload;

  procedure SetDynGridSize(var TargetGrid: TRealDynGrid; GridSize: TGridSize); overload;
  procedure SetDynGridSize(var TargetGrid: TRealDynGrid; RowCount,ColCount: integer); overload;

  procedure SetDynGridSize(var TargetGrid: TStringDynGrid; GridSize: TGridSize); overload;
  procedure SetDynGridSize(var TargetGrid: TStringDynGrid; RowCount,ColCount: integer); overload;

  // 3D Grid Procedures and Functions

  procedure DestroyDyn3DGrid(var TargetGrid: TRealDyn3DGrid); overload;

  procedure SetDyn3DGridSize(var TargetGrid: TRealDyn3DGrid; RowCount,ColCount,SubColCount: integer); overload;


  // Delphi String Grid Procedures and Functions

  procedure CopyStringDynGridToDelphiStringGrid(Source: TStringDynGrid; var Target: TStringGrid);
  procedure CopyDelphiStringGridToStringDynGrid(Source: TStringGrid; var Target: TStringDynGrid);

  procedure ClearDelphiStringGrid(var Target: TStringGrid);
  procedure SelectDelphiStringGridCellRange(var Target: TStringGrid; T,L,B,R: integer);
  procedure ResetDelphiStringGrid(var Target: TStringGrid; DWidth: integer);

  procedure SetDelphiStringGridColWidths(var Target: TStringGrid; DefColWidth: integer; NewColWidths: TIntegerDynArray);

  procedure UnselectDelphiStringGridCell(var Target: TStringGrid);

implementation

const
  CEmptyRect: TRect = (Left: 0; Top: 0; Right: 0; Bottom: 0);

// Array Procedures and Functions

// Clear Array Procedures

procedure ClearDynArray(var TargetArray: TBooleanDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:=false;
end;

procedure ClearDynArray(var TargetArray: TCharDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:=#0;
end;

procedure ClearDynArray(var TargetArray: TByteDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:=0;
end;

procedure ClearDynArray(var TargetArray: TIntegerDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:=0;
end;

procedure ClearDynArray(var TargetArray: TWordDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:=0;
end;

procedure ClearDynArray(var TargetArray: TRealDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:=0.0;
end;

procedure ClearDynArray(var TargetArray: TDoubleDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:=0.0;
end;

procedure ClearDynArray(var TargetArray: TStringDynArray);
var I, L: integer;
begin
  L:=System.Length(TargetArray);
  for I:=0 to L-1 do
      TargetArray[I]:='';
end;

// Append Array Element Functions

function AppendElementToArray(Element: boolean; var TargetArray: TBooleanDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendElementToArray(Element: char; var TargetArray: TCharDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendElementToArray(Element: byte; var TargetArray: TByteDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendElementToArray(Element: integer; var TargetArray: TIntegerDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendElementToArray(Element: word; var TargetArray: TWordDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendElementToArray(Element: real; var TargetArray: TRealDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendElementToArray(Element: double; var TargetArray: TDoubleDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendElementToArray(Element: string; var TargetArray: TStringDynArray): integer;
var L: integer;
begin
  L:=System.Length(TargetArray);
  SetLength(TargetArray,L+1);
  TargetArray[L]:=Element;
  Result:=L+1;
end;

function AppendArrayToArray(SourceArray: TIntegerDynArray; var TargetArray: TIntegerDynArray): integer; overload;
var I, SL, TL: integer;
begin
  SL:=System.Length(SourceArray);
  TL:=System.Length(TargetArray);
  SetLength(TargetArray,TL+SL);
  for I:=0 to SL-1 do
      TargetArray[TL+I]:=SourceArray[I];
  Result:=TL+SL;
end;

function AppendArrayToArray(SourceArray: TRealDynArray; var TargetArray: TRealDynArray): integer;
var I, SL, TL: integer;
begin
  SL:=System.Length(SourceArray);
  TL:=System.Length(TargetArray);
  SetLength(TargetArray,TL+SL);
  for I:=0 to SL-1 do
      TargetArray[TL+I]:=SourceArray[I];
  Result:=TL+SL;
end;

function AppendArrayToArray(SourceArray: TStringDynArray; var TargetArray: TStringDynArray): integer;
var I, SL, TL: integer;
begin
  SL:=System.Length(SourceArray);
  TL:=System.Length(TargetArray);
  SetLength(TargetArray,TL+SL);
  for I:=0 to SL-1 do
      TargetArray[TL+I]:=SourceArray[I];
  Result:=TL+SL;
end;

// Check if Element already Present in Array Procedures

function CheckIfElementPresent(Element: boolean; DynArray: TBooleanDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

function CheckIfElementPresent(Element: char; DynArray: TCharDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

function CheckIfElementPresent(Element: byte; DynArray: TByteDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

function CheckIfElementPresent(Element: integer; DynArray: TIntegerDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

function CheckIfElementPresent(Element: word; DynArray: TWordDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

function CheckIfElementPresent(Element: real; DynArray: TRealDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

function CheckIfElementPresent(Element: double; DynArray: TDoubleDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

function CheckIfElementPresent(Element: string; DynArray: TStringDynArray): boolean;
var I, L: integer;
begin
  Result:=false;
  L:=System.Length(DynArray);
  for I:=0 to L-1 do
      if (DynArray[I]=Element) then
         begin
         Result:=true;
         break;
         end;
end;

// Destroy Array Procedures

procedure DestroyDynArray(var TargetArray: TBooleanDynArray);
begin
  SetLength(TargetArray,0);
end;

procedure DestroyDynArray(var TargetArray: TCharDynArray);
begin
  SetLength(TargetArray,0);
end;

procedure DestroyDynArray(var TargetArray: TByteDynArray);
begin
  SetLength(TargetArray,0);
end;

procedure DestroyDynArray(var TargetArray: TIntegerDynArray);
begin
  SetLength(TargetArray,0);
end;

procedure DestroyDynArray(var TargetArray: TWordDynArray);
begin
  SetLength(TargetArray,0);
end;

procedure DestroyDynArray(var TargetArray: TRealDynArray);
begin
  SetLength(TargetArray,0);
end;

procedure DestroyDynArray(var TargetArray: TDoubleDynArray);
begin
  SetLength(TargetArray,0);
end;

procedure DestroyDynArray(var TargetArray: TStringDynArray);
begin
  SetLength(TargetArray,0);
end;

function ConvertConstToIntegerDynArray(Source: array of const): TIntegerDynArray;
var I: integer;
begin
  if (System.Length(Source)=0) then SetLength(Result,0)
  else SetLength(Result,High(Source)+1);
  for I:=0 to High(Source) do
      with Source[I] do
           if VType=vtInteger then Result[I]:=VInteger;
end;


function GetIndexOfFirstStringInArray(Value: string; Source: TStringDynArray): integer;
var I: integer;
begin
  Result:=-1;
  for I:=0 to High(Source) do
      if (Value=Source[I]) then
         begin
         Result:=I;
         break;
         end;
end;

// Grid Procedures and Functions

function GetGridRowRect(GridSize: TGridSize; RowNumber: integer): TRect;
begin
  if RowNumber<GridSize.RowCount then
     begin
     Result.Left:=1;
     Result.Right:=GridSize.ColCount;
     Result.Top:=RowNumber;
     Result.Bottom:=RowNumber;
     end;
end;

function GetGridColumnRect(GridSize: TGridSize; ColNumber: integer): TRect;
begin
  if ColNumber<GridSize.ColCount then
     begin
     Result.Left:=ColNumber;
     Result.Right:=ColNumber;
     Result.Top:=1;
     Result.Bottom:=GridSize.ColCount;
     end;
end;

// Clear Grid Procedures

procedure ClearDynGrid(var TargetGrid: TBooleanDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:=false;
      end;
end;

procedure ClearDynGrid(var TargetGrid: TCharDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:=#0;
      end;
end;

procedure ClearDynGrid(var TargetGrid: TByteDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:=0;
      end;
end;

procedure ClearDynGrid(var TargetGrid: TIntegerDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:=0;
      end;
end;

procedure ClearDynGrid(var TargetGrid: TWordDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:=0;
      end;
end;

procedure ClearDynGrid(var TargetGrid: TRealDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:=0.0;
      end;
end;

procedure ClearDynGrid(var TargetGrid: TDoubleDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:=0.0;
      end;
end;

procedure ClearDynGrid(var TargetGrid: TStringDynGrid);
var IR, IC, LR, LC: integer;
begin
  LR:=System.Length(TargetGrid);
  for IR:=0 to LR-1 do
      begin
      LC:=System.Length(TargetGrid[IR]);
      for IC:=0 to LC-1 do
          TargetGrid[IR][IC]:='';
      end;
end;

// Append Grid Array Functions

function AppendArrayToGrid(SourceArray: TBooleanDynArray; var TargetGrid: TBooleanDynGrid): integer;
var I, L, N: integer;
    B: boolean;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      B:=SourceArray[I];
      TargetGrid[L,I]:=B;
      end;
  Result:=L+1;
end;

function AppendArrayToGrid(SourceArray: TCharDynArray; var TargetGrid: TCharDynGrid): integer;
var I, L, N: integer;
    C: char;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      C:=SourceArray[I];
      TargetGrid[L,I]:=C;
      end;
  Result:=L+1;
end;

function AppendArrayToGrid(SourceArray: TByteDynArray; var TargetGrid: TByteDynGrid): integer;
var I, L, N: integer;
    B: byte;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      B:=SourceArray[I];
      TargetGrid[L,I]:=B;
      end;
  Result:=L+1;
end;

function AppendArrayToGrid(SourceArray: TIntegerDynArray; var TargetGrid: TIntegerDynGrid): integer;
var I, L, N: integer;
    T: integer;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      T:=SourceArray[I];
      TargetGrid[L,I]:=T;
      end;
  Result:=L+1;
end;

function AppendArrayToGrid(SourceArray: TWordDynArray; var TargetGrid: TWordDynGrid): integer;
var I, L, N: integer;
    W: word;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      W:=SourceArray[I];
      TargetGrid[L,I]:=W;
      end;
  Result:=L+1;
end;

function AppendArrayToGrid(SourceArray: TRealDynArray; var TargetGrid: TRealDynGrid): integer;
var I, L, N: integer;
    R: real;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      R:=SourceArray[I];
      TargetGrid[L,I]:=R;
      end;
  Result:=L+1;
end;

function AppendArrayToGrid(SourceArray: TDoubleDynArray; var TargetGrid: TDoubleDynGrid): integer;
var I, L, N: integer;
    D: real;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      D:=SourceArray[I];
      TargetGrid[L,I]:=D;
      end;
  Result:=L+1;
end;

function AppendArrayToGrid(SourceArray: TStringDynArray; var TargetGrid: TStringDynGrid): integer;
var I, L, N: integer;
    S: string;
begin
  L:=System.Length(TargetGrid);
  SetLength(TargetGrid,L+1);
  N:=System.Length(SourceArray);
  SetLength(TargetGrid[L],N);
  for I:=0 to N-1 do
      begin
      S:=SourceArray[I];
      TargetGrid[L,I]:=S;
      end;
  Result:=L+1;
end;

// Destroy Grid Procedures

procedure DestroyDynGrid(var TargetGrid: TBooleanDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure DestroyDynGrid(var TargetGrid: TCharDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure DestroyDynGrid(var TargetGrid: TByteDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure DestroyDynGrid(var TargetGrid: TIntegerDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure DestroyDynGrid(var TargetGrid: TWordDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure DestroyDynGrid(var TargetGrid: TRealDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure DestroyDynGrid(var TargetGrid: TDoubleDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure DestroyDynGrid(var TargetGrid: TStringDynGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure SetDynGridSize(var TargetGrid: TBooleanDynGrid; GridSize: TGridSize);
var Row: integer;
begin
  SetLength(TargetGrid,GridSize.RowCount);
  for Row:=0 to GridSize.RowCount-1 do
      SetLength(TargetGrid[Row],GridSize.ColCount);
end;

// Attention! The RowCount,ColCount sequence was exchanged (swapped) on 12.06.2013
// and is now compliant with the Delphi Grid cell addressing sequence.
procedure SetDynGridSize(var TargetGrid: TBooleanDynGrid; RowCount,ColCount: integer);
var Row: integer;
begin
  SetLength(TargetGrid,RowCount);
  for Row:=0 to RowCount-1 do
      SetLength(TargetGrid[Row],ColCount);
end;

procedure SetDynGridSize(var TargetGrid: TRealDynGrid; GridSize: TGridSize);
var Row: integer;
begin
  SetLength(TargetGrid,GridSize.RowCount);
  for Row:=0 to GridSize.RowCount-1 do
      SetLength(TargetGrid[Row],GridSize.ColCount);
end;

// Attention! The RowCount,ColCount sequence was exchanged (swapped) on 12.06.2013
// and is now compliant with the Delphi Grid cell addressing sequence.
procedure SetDynGridSize(var TargetGrid: TRealDynGrid; RowCount,ColCount: integer);
var Row: integer;
begin
  SetLength(TargetGrid,RowCount);
  for Row:=0 to RowCount-1 do
      SetLength(TargetGrid[Row],ColCount);
end;

procedure SetDynGridSize(var TargetGrid: TStringDynGrid; GridSize: TGridSize);
var Row: integer;
begin
  SetLength(TargetGrid,GridSize.RowCount);
  for Row:=0 to GridSize.RowCount-1 do
      SetLength(TargetGrid[Row],GridSize.ColCount);
end;

// Attention! The RowCount,ColCount sequence was exchanged (swapped) on 12.06.2013
// and is now compliant with the Delphi Grid cell addressing sequence.
procedure SetDynGridSize(var TargetGrid: TStringDynGrid; RowCount,ColCount: integer);
var Row: integer;
begin
  SetLength(TargetGrid,RowCount);
  for Row:=0 to RowCount-1 do
      SetLength(TargetGrid[Row],ColCount);
end;

  // 3D Grid Procedures and Functions

procedure DestroyDyn3DGrid(var TargetGrid: TRealDyn3DGrid);
begin
  SetLength(TargetGrid,0);
end;

procedure SetDyn3DGridSize(var TargetGrid: TRealDyn3DGrid; RowCount,ColCount,SubColCount: integer);
var Row: integer;
begin
  SetLength(TargetGrid,RowCount);
  for Row:=0 to RowCount-1 do
      SetDynGridSize(TargetGrid[Row],ColCount,SubColCount);
end;

  // Delphi String Grid Procedures and Functions

procedure CopyStringDynGridToDelphiStringGrid(Source: TStringDynGrid; var Target: TStringGrid);
var GridSize: TGridSize;
    Col,Row: integer;
begin
  with GridSize do
       begin
       RowCount:=System.Length(Source);

       if RowCount=0 then ColCount:=0
       else ColCount:=System.Length(Source[0]);

       Target.RowCount:=RowCount;
       Target.ColCount:=ColCount;

       for Row:=0 to RowCount-1 do
           for Col:=0 to ColCount-1 do
               Target.Cells[Col,Row]:=Source[Row][Col];
       end;
end;

procedure CopyDelphiStringGridToStringDynGrid(Source: TStringGrid; var Target: TStringDynGrid);
var GridSize: TGridSize;
    Col,Row: integer;
begin
  with GridSize do
       begin
       RowCount:=Source.RowCount;
       ColCount:=Source.ColCount;

       SetDynGridSize(Target,GridSize);

       for Row:=0 to RowCount-1 do
           for Col:=0 to ColCount-1 do
               Target[Row][Col]:=Source.Cells[Col,Row];
       end;
end;

procedure ClearDelphiStringGrid(var Target: TStringGrid);
var I, J: integer;
begin
  for I:=0 to Target.RowCount-1 do
      for J:=0 to Target.ColCount-1 do Target.Cells[J,I]:='';
end;

procedure SelectDelphiStringGridCellRange(var Target: TStringGrid; T,L,B,R: integer);
var S: TGridRect;
begin
  S.Left:=L; S.Right:=R;
  S.Top:=T; S.Bottom:=B;

  Target.Selection:=S;
end;

procedure ResetDelphiStringGrid(var Target: TStringGrid; DWidth: integer);
begin
  with Target do
       begin
       ClearDelphiStringGrid(Target);

       ColCount:=1;
       RowCount:=1;
       DefaultColWidth:=DWidth;
       Options:=Options+[goRangeSelect,goDrawFocusSelected];
       SelectDelphiStringGridCellRange(Target,-1,-1,-1,-1);
       end;
end;

procedure SetDelphiStringGridColWidths(var Target: TStringGrid; DefColWidth: integer; NewColWidths: TIntegerDynArray);
var I: integer;
begin
  with Target do
       begin
       if DefColWidth>0 then DefaultColWidth:=DefColWidth;
       if (System.Length(NewColWidths)>0) and
          not(Target.ColCount<System.Length(NewColWidths)) then
          for I:=0 to High(NewColWidths) do
              if (NewColWidths[I]>0) then ColWidths[I]:=NewColWidths[I];
       end;
end;

procedure UnselectDelphiStringGridCell(var Target: TStringGrid);
var GR: TGridRect;
    // I: integer;
begin
  // Note: unselected StringGrid shows upper cell highlighted (in blue)
  // This awful behaviour can be removed with the below procedure
  with GR do
       begin
       Left:=-1;
       Top:=-1;
       Right:=-1;
       Bottom:=-1;
       end;
  Target.Selection:=GR;
end;

end.
