//******************************************************************************
//******************************************************************************
//******************************************************************************
//**
//**  CONVERSIONUTILS.PAS
//**
//**  Different helpful functions and procedures for string and number conversion
//**  Derived on 22.07.2008 from "HelpfulUtils.Pas"
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.08.96 / 22.07.08 / 21.10.08 / 08.06.09 / 22.10.09 / 29.04.10
//**           31.10.11 / 11.06.13 / 09.01.14
//******************************************************************************
//******************************************************************************
//******************************************************************************
unit ConversionUtils;

interface

uses Types, Classes;

const
  CASCIIIntegerDigits  = ['0'..'9'];
  CASCIIHexDigits  = ['0'..'9','A'..'F','a'..'f'];

  Crc_16_tab: array [0..255] of word = (
              $0000,  $1021,  $2042,  $3063,  $4084,  $50a5,  $60c6,  $70e7,
              $8108,  $9129,  $a14a,  $b16b,  $c18c,  $d1ad,  $e1ce,  $f1ef,
              $1231,  $0210,  $3273,  $2252,  $52b5,  $4294,  $72f7,  $62d6,
              $9339,  $8318,  $b37b,  $a35a,  $d3bd,  $c39c,  $f3ff,  $e3de,
              $2462,  $3443,  $0420,  $1401,  $64e6,  $74c7,  $44a4,  $5485,
              $a56a,  $b54b,  $8528,  $9509,  $e5ee,  $f5cf,  $c5ac,  $d58d,
              $3653,  $2672,  $1611,  $0630,  $76d7,  $66f6,  $5695,  $46b4,
              $b75b,  $a77a,  $9719,  $8738,  $f7df,  $e7fe,  $d79d,  $c7bc,
              $48c4,  $58e5,  $6886,  $78a7,  $0840,  $1861,  $2802,  $3823,
              $c9cc,  $d9ed,  $e98e,  $f9af,  $8948,  $9969,  $a90a,  $b92b,
              $5af5,  $4ad4,  $7ab7,  $6a96,  $1a71,  $0a50,  $3a33,  $2a12,
              $dbfd,  $cbdc,  $fbbf,  $eb9e,  $9b79,  $8b58,  $bb3b,  $ab1a,
              $6ca6,  $7c87,  $4ce4,  $5cc5,  $2c22,  $3c03,  $0c60,  $1c41,
              $edae,  $fd8f,  $cdec,  $ddcd,  $ad2a,  $bd0b,  $8d68,  $9d49,
              $7e97,  $6eb6,  $5ed5,  $4ef4,  $3e13,  $2e32,  $1e51,  $0e70,
              $ff9f,  $efbe,  $dfdd,  $cffc,  $bf1b,  $af3a,  $9f59,  $8f78,
              $9188,  $81a9,  $b1ca,  $a1eb,  $d10c,  $c12d,  $f14e,  $e16f,
              $1080,  $00a1,  $30c2,  $20e3,  $5004,  $4025,  $7046,  $6067,
              $83b9,  $9398,  $a3fb,  $b3da,  $c33d,  $d31c,  $e37f,  $f35e,
              $02b1,  $1290,  $22f3,  $32d2,  $4235,  $5214,  $6277,  $7256,
              $b5ea,  $a5cb,  $95a8,  $8589,  $f56e,  $e54f,  $d52c,  $c50d,
              $34e2,  $24c3,  $14a0,  $0481,  $7466,  $6447,  $5424,  $4405,
              $a7db,  $b7fa,  $8799,  $97b8,  $e75f,  $f77e,  $c71d,  $d73c,
              $26d3,  $36f2,  $0691,  $16b0,  $6657,  $7676,  $4615,  $5634,
              $d94c,  $c96d,  $f90e,  $e92f,  $99c8,  $89e9,  $b98a,  $a9ab,
              $5844,  $4865,  $7806,  $6827,  $18c0,  $08e1,  $3882,  $28a3,
              $cb7d,  $db5c,  $eb3f,  $fb1e,  $8bf9,  $9bd8,  $abbb,  $bb9a,
              $4a75,  $5a54,  $6a37,  $7a16,  $0af1,  $1ad0,  $2ab3,  $3a92,
              $fd2e,  $ed0f,  $dd6c,  $cd4d,  $bdaa,  $ad8b,  $9de8,  $8dc9,
              $7c26,  $6c07,  $5c64,  $4c45,  $3ca2,  $2c83,  $1ce0,  $0cc1,
              $ef1f,  $ff3e,  $cf5d,  $df7c,  $af9b,  $bfba,  $8fd9,  $9ff8,
              $6e17,  $7e36,  $4e55,  $5e74,  $2e93,  $3eb2,  $0ed1,  $1ef0);

type
  TSimpleTime = record
    H,M,S: word;
    end;

  TSimpleDate = record
    D,M,Y: word;
    end;

  function IsDecimalCharacter (C: char): boolean;
  function IsHexLetterCharacter (C: char): boolean;

  function DeleteHexPrefixOrSuffix(S: string): string;

  function IsDecimalIntegerString(S: string): boolean;
  function IsHexIntegerString(S: string): boolean;
  function IsFloatingPointString(S: string): boolean; overload;
  function IsFloatingPointString(S: string; DecSep: char): boolean; overload;

  function TryStrToInt (S: string; var Target: integer): boolean;
  function TryStrToFloat (S: string; var Target: Extended): boolean;

  function HexStringToByte (S: string): byte;
  function HexStringToWord (S: string): word;

//
  function GetIntegerFromString(S: string): integer; overload;
  function GetIntegerFromString(S: string; DefValue: integer): integer; overload;

  function GetRealFromString(S: string; DS: char): real; overload;
  function GetRealFromString(S: string; DS: char; DefValue: Extended): real; overload;

  function StrToIntDef (S: string; DefValue: integer): integer;
  function StrToFloatDef (S: string; DefValue: Extended): Extended;

  function Byte6StreamToReal48(Source: Pbyte): real;
  procedure Real48ToByte6Stream (Source: real; Target: Pbyte);

  function IntegerToGroupedString(Source: int64): string;

  function ByteToBinaryString (B: byte): string;
  function WordToBinaryString (W: word): string;

  function ByteToAsciiString (B: byte): string;
  function ByteStringToAsciiString (BS: string): string;

  function FormatShortDateString (SimpleDate: TSimpleDate): string;
  function FormatLongDateString (SimpleDate: TSimpleDate): string;

  function GetWeekOfYear(D : TDateTime) : byte;
  function GetDaysInMonth(Y,M: word): byte;

  function GetStringParityByte (Source: string): byte;
  function GetStringChecksumByte (Source: string): byte;
  function CalculateByteStringChecksumByte (Source: string; var Checksum: byte): boolean;

  function GetDataStreamParityByte (var Rec; L: word): byte;
  function GetFileStreamParityByte (Source: TFileStream): byte;

  function GetDataStreamCRC16Value (var Rec; L: word): word;
  function GetFileStreamCRC16Value (Source: TFileStream): word;

  function SmartRoundx(Source: string; DigitNumber: integer; Up: boolean): string; overload;
  function SmartRound(Source: string; DigitNumber: integer; Up: boolean): string; overload;
  function SmartRound(Source: string; DigitNumber: integer; Up: boolean; DS: char): string; overload;

implementation

uses SysUtils;

type
  PByteDynArray=^TByteDynArray;

function IsDecimalCharacter (C: char): boolean;
begin
  Result:=(C in CASCIIIntegerDigits);
end;

function IsHexLetterCharacter (C: char): boolean;
begin
  Result:=(C in CASCIIHexDigits) and not(C in CASCIIIntegerDigits);
end;

function DeleteHexPrefixOrSuffix(S: string): string;
var L: integer;
begin
  // Hexadecimal Integer Strings may have followed formats
  // C language prefix "0x"
  // Pascal prefix "$"
  // Suffix "h" or "H"
  // no any prefix or suffix
  L:=System.Length(S);
  Result:=S;
  if (L>0) then
     begin
     if (System.Pos('$',S)=1) then
        Result:=System.Copy(S,2,L-1)
     else if (System.Pos('h',LowerCase(S))=L) then
          Result:=System.Copy(S,1,L-1)
     else if (System.Pos('0x',LowerCase(S))=1) then
          Result:=System.Copy(S,3,L-2);
     end;
end;

function IsDecimalIntegerString(S: string): boolean;
var I, J, L: integer;
begin
  L:=System.Length(S);
  if (L>0) then
     begin
     Result:=true;
     if (S[1]<>'-') then J:=1
     else if (L>1) then J:=2
          else
             begin
             J:=L;
             Result:=false;
             end;
     for I:=J to L do
         if not(S[I] in CASCIIIntegerDigits) then
            begin
            Result:=false;
            break;
            end;
     end
  else Result:=false;
end;

function IsHexIntegerString(S: string): boolean;
var I, L: integer;
begin
  // Note, "S" must not contain any prefix or suffix
  L:=System.Length(S);
  if (L>0) then
     begin
     Result:=true;
     for I:=1 to L do
         if not(S[I] in CASCIIHexDigits) then
            begin
            Result:=false;
            break;
            end;
     end
  else Result:=false;
end;

function IsFloatingPointString(S: string): boolean;
var L: integer;
begin
  Result:=false;
  L:=System.Pos(DecimalSeparator,S);
  if (L<2) or IsDecimalIntegerString(System.Copy(S,1,L-1)) then
     begin
     System.Delete(S,1,L);
     L:=System.Pos('E',UpperCase(S));
     if (L=0) or IsDecimalIntegerString(System.Copy(S,1,L-1)) then
        begin
        System.Delete(S,1,L);
        Result:=IsDecimalIntegerString(S);
        end;
     end;
end;

function IsFloatingPointString(S: string; DecSep: char): boolean;
var L: integer;
begin
  Result:=false;
  L:=System.Pos(DecSep,S);
  if (L<2) or IsDecimalIntegerString(System.Copy(S,1,L-1)) then
     begin
     System.Delete(S,1,L);
     L:=System.Pos('E',UpperCase(S));
     if (L=0) or IsDecimalIntegerString(System.Copy(S,1,L-1)) then
        begin
        System.Delete(S,1,L);
        Result:=IsDecimalIntegerString(S);
        end;
     end;
end;

function TryStrToInt (S: string; var Target: integer): boolean;
begin
  try
    Target:=StrToInt(S);
    Result:=true;
  except
    on EConvertError do Result:=false;
  end;
end;

function TryStrToFloat (S: string; var Target: Extended): boolean;
begin
  Result:=false;
  if (S<>'') then
     try
       Target:=StrToFloat(S);
       Result:=true;
     except
       on EConvertError do Result:=false;
     end;
end;

function HexStringToByte (S: string): byte;
var I, B, C: byte;
begin
  // Note, "S" must not contain any prefix or suffix
  if (S='') or not(IsHexIntegerString(S)) then B:=0
  else
     begin
     if (System.Length(S)>2) then B:=$FF
     else
        begin
        B:=0;
        for I:=1 to 2 do
            begin
            if IsDecimalCharacter(S[I]) then
               C:=ord(S[I])-ord('0')
            else C:=ord(S[I])-ord('A')+10;
            if (I=1) then
               B:=B or C shl 4
            else B:=B or C;
            end;
        end;
     end;
  Result:=B;
end;

function HexStringToWord (S: string): word;
var W: word;
begin
  // Note, "S" must not contain any prefix or suffix
  if (S='') or not(IsHexIntegerString(S)) then W:=0
  else
     begin
     if (System.Length(S)>4) then W:=$FFFF
     else
        begin
        while System.Length(S)<4 do S:='0'+S;
        W:=HexStringToByte(System.Copy(S,1,2))*256+HexStringToByte(System.Copy(S,3,2));
        end;
     end;
  Result:=W;
end;

// the next three procedures should be in the future deleted and replaced with standard available procedures

function GetIntegerFromString(S: string): integer;
begin
  if (S<>'') then
     begin
     try
       Result:=StrToInt(S);
     except
       on EConvertError do Result:=0;
     end;
     end
  else Result:=0;
end;

function GetIntegerFromString(S: string; DefValue: integer): integer;
begin
  if (S<>'') then
     begin
     try
       Result:=StrToInt(S);
     except
       on EConvertError do Result:=DefValue;
     end;
     end
  else Result:=DefValue;
end;


function GetRealFromString(S: string; DS: char): real;
var OriginalSeparator: char;
begin
  if (S<>'') then
     begin
     OriginalSeparator:=DecimalSeparator;
     DecimalSeparator:=DS;
     try
       Result:=StrToFloat(S);
     except
       on EConvertError do Result:=0.0;
     end;
     DecimalSeparator:=OriginalSeparator;
     end
  else Result:=0.0;
end;


function GetRealFromString(S: string; DS: char; DefValue: Extended): real;
var OriginalSeparator: char;
begin
  if (S<>'') then
     begin
     OriginalSeparator:=DecimalSeparator;
     DecimalSeparator:=DS;
     try
       Result:=StrToFloat(S);
     except
       on EConvertError do Result:=DefValue;
     end;
     DecimalSeparator:=OriginalSeparator;
     end
  else Result:=DefValue;
end;

// end of the procedures to be replaced

function StrToIntDef (S: string; DefValue: integer): integer;
begin
  try
    Result:=StrToInt(S);
  except
    on EConvertError do Result:=DefValue;
  end;
end;

function StrToFloatDef (S: string; DefValue: Extended): Extended;
begin
  try
    Result:=StrToFloat(S);
  except
    on EConvertError do Result:=DefValue;
  end;
end;

function Byte6StreamToReal48(Source: Pbyte): real;
var InterResult: real48;
    LTarget: array [0..5] of byte absolute InterResult;
    I: byte;
begin
  InterResult:=0.0;
{$REALCOMPATIBILITY ON}
  if (Source<>nil) then
     begin
     for I:=0 to 5 do
         begin
         LTarget[I]:=Source^;
         if I<5 then Inc(Source);
         end;
     end;
{$REALCOMPATIBILITY OFF}
  Result:=InterResult;
end;

procedure Real48ToByte6Stream (Source: real; Target: Pbyte);
var InterResult: real48;
    LTarget: array [0..5] of byte absolute InterResult;
    I: byte;
begin
{$REALCOMPATIBILITY ON}
  InterResult:=Source;
{$REALCOMPATIBILITY OFF}
  if (Target<>nil) then
     begin
     for I:=0 to 5 do
         begin
         Target^:=LTarget[I];
         if I<5 then Inc(Target);
         end;
     end;
end;

function IntegerToGroupedString(Source: int64): string;
var S,R: string;
    L: integer;
begin
  S:=IntToStr(Source); R:='';
  repeat
    L:=System.Length(S);
    if (L>3) then
       begin
       R:=','+System.Copy(S,L-2,3)+R;
       System.Delete(S,L-2,3);
       L:=L-3;
       end;
  until not(L>3);

  Result:=S+R;
end;

function ByteToBinaryString (B: byte): string;
var I: byte;
    S: string;
begin
  S:='';
  for I:=0 to 7 do if ((($80) shr I) and B)>0 then S:=S+'1' else S:=S+'0';
  Result := S;
end;

function WordToBinaryString (W: word): string;
var I: byte;
    S: string;
begin
  S:='';
  for I:=0 to 15 do if ((($8000) shr I) and W)>0 then S:=S+'1' else S:=S+'0';
  Result := S;
end;

function ByteToAsciiString (B: byte): string;
begin
  Result:='$'+IntToHex(B DIV 16,1)+IntToHex(B MOD 16,1);
end;

function ByteStringToAsciiString (BS: string): string;
var I,L: integer;
begin
  L:=System.Length(BS);
  Result:='';
  for I:=1 to L do
      Result:=Result+ByteToAsciiString (ord(BS[I]));
end;

function GetZerosString (Length: word): string;
var S: string;
begin
  Str(Length:0, S);
  Result := System.Copy ('00', 1, 2 - System.Length(S)) + S;
end;

function FormatShortDateString (SimpleDate: TSimpleDate): string;
var SYear: string[4];
begin
  while (SimpleDate.Y > 99) do SimpleDate.Y:=SimpleDate.Y-100;
  Str (SimpleDate.Y:2, SYear); if SYear[1]=' ' then SYear[1]:='0';
  Result := GetZerosString (SimpleDate.D) + DateSeparator + GetZerosString (SimpleDate.M) + DateSeparator + SYear;
end;

function FormatLongDateString (SimpleDate: TSimpleDate): string;
var SYear: string[6];
begin
  Str (SimpleDate.Y:4, SYear);
  Result := GetZerosString (SimpleDate.D) + DateSeparator + GetZerosString (SimpleDate.M) + DateSeparator + SYear;
end;

function GetWeekOfYear(D : TDateTime) : byte;
var aDate        : Integer;
    MinDate      : tDateTime;
    MaxDate      : tDateTime;
    Day, Month, Year : Word;
    FirstDate    : Integer;
begin
    aDate := Trunc(D);
    MinDate := EncodeDate(1900, 1, 1);
    MaxDate := EncodeDate(3999, 12, 31);
    if (aDate < MinDate) or (aDate > MaxDate) then
      begin
      Result := 0;
      exit;
    end;

    aDate  := aDate  + 3 - ((5 + DayOfWeek(aDate)) MOD 7);
    DecodeDate(aDate, Year, Month, Day);
    FirstDate := Trunc(EncodeDate(Year, 1, 1));

    Result := 1 + (aDate - FirstDate) DIV 7;
end;

function GetDaysInMonth(Y,M: word): byte;
const
  CMonthDays: array [1..12] of byte = (31,28,31,30,31,30,31,31,30,31,30,31);
begin
  if ((Y MOD 4)=0) and (((Y MOD 100)>0) or ((Y MOD 400)=0)) and // If leap year
     (M=2) then                                                 // and February
     Result:=CMonthDays[M]+1
  else Result:=CMonthDays[M];
end;

function GetStringParityByte (Source: string): byte;
var Parity: byte;
    I,L: integer;
begin
  Parity:=0;
  L:=System.Length(Source);
  if (L>0) then
     for I:=1 to L do
         Parity:=Parity xor ord(Source[I]);
  Result:=Parity;
end;

function GetStringChecksumByte (Source: string): byte;
var Checksum: dword;
    I,L: integer;
begin
  Checksum:=0;
  L:=System.Length(Source);
  if (L>0) then
     for I:=1 to L do
         Checksum:=Checksum + ord(Source[I]);
  Result:=byte(Checksum);
end;

function CalculateByteStringChecksumByte (Source: string; var Checksum: byte): boolean;
var I,L: integer;
    CS: dword;
begin
  CS:=0;
  L:=System.Length(Source);
  if (L>0) and ((L MOD 2)=0) then
     begin
     L:=L DIV 2;
     for I:=0 to L-1 do
         CS:=CS + byte(StrToInt('$'+Source[I*2+1]+Source[I*2+2]));
     while (CS>$FF) do CS:=CS-$100;
     Checksum:=byte(CS);
     Result:=true;
     end
  else Result:=false;
end;

function GetDataStreamParityByte (var Rec; L: word): byte;
type
  PDataStream = ^TDataStream;
  TDataStream = array [0..65535] of byte;
var pBuffer: PDataStream; // PByteDynArray;
    lParity: byte;
    I: integer;
begin
  if (L>0) then
     begin
     pBuffer:=@Rec; lParity:=0;
     for I:=0 to L-1 do lParity := lParity xor (pBuffer^[I]);
     Result:=lParity;
     end
  else Result:=0;
end;

function GetFileStreamParityByte (Source: TFileStream): byte;
var DataBuffer: TByteDynArray;
    L: integer;
begin
  L:=Source.Size;
  SetLength(DataBuffer,L);
  Source.Seek(0,soFromBeginning);
  Source.Read(DataBuffer,L);
  Result:=GetDataStreamParityByte(DataBuffer,L-1);
end;

function GetDataStreamCRC16Value (var Rec; L: word): word;
var pBuffer: PByteDynArray;
    lCRC16: word;
    I: integer;
begin
  if (L>1) then
     begin
     pBuffer:=@Rec; lCRC16:=0;

     for I:=0 to L-1 do
         lCRC16 := Crc_16_tab[((lCRC16 shr 8) and $FF)] xor (lCRC16 shl 8) xor (pBuffer^[I]);

     lCRC16 := Crc_16_tab[((lCRC16 shr 8) and $FF)] xor (lCRC16 shl 8) xor 0;
     lCRC16 := Crc_16_tab[((lCRC16 shr 8) and $FF)] xor (lCRC16 shl 8) xor 0;

     Result:=lCRC16;
     end
  else Result:=0;
end;

function GetFileStreamCRC16Value (Source: TFileStream): word;
var DataBuffer: TByteDynArray;
    L: integer;
begin
  L:=Source.Size;
  SetLength(DataBuffer,L);
  Source.Seek(0,soFromBeginning);
  Source.Read(DataBuffer,L);
  Result:=GetDataStreamCRC16Value(DataBuffer,L);
end;

function SmartRoundx(Source: string; DigitNumber: integer; Up: boolean): string;
var I, L, N, P: integer;
    RS, SS: string;
    RR, SR: real;
    // M: boolean;
    C: char;
begin
  Result:=Source; // if rounding anyway not successful, then return the source string
  if IsFloatingPointString(Source) then
     begin
     L:=System.Length(Source);
     if (L>DigitNumber) then // break if source length too short
        begin
        P:=System.Pos(DecimalSeparator,Source);
        // M:=(Source[1]='-');
        N:=0; RS:=''; SS:='';
        for I:=0 to L-1 do   // now evaluate all string characters
            begin
            C:=Source[I+1];  // this is the evaluated character

            // create main rounded string
            if (N=0) then // no significant character (number digit) found until now
               begin
               if C in ['1'..'9'] then // search for the first significant character
                  N:=I+1;
               end
            else // first significan number was alread detectd
               begin
               if (P=0) then // no Decimal Separator in the source string
                  begin
                  if ((I-N+2)>DigitNumber) then
                     C:='0';
                  end
               else  // source string consists Decimal Separator
                  begin
                  if (C<>DecimalSeparator) then // actual character is not Decimal Separator
                     begin
                     if (N<P) then // significant character in left to Decimal Separator
                        begin
                        if (P>(N+DigitNumber-1)) then // Decimal Separator Outside
                           begin
                           if ((I-N+2)>DigitNumber) then
                              C:='0';
                           end
                        else                          // Decimal Separator Inside
                           begin
                           if ((I-N+1)>DigitNumber) then
                              C:='0';
                           end;
                        end
                     else          // significant character in right to Decimal Separator
                        begin
                        if ((I-N+2)>DigitNumber) then
                           C:='0';
                        end;
                     end;
                  end;
               end;
            RS:=RS+C;

            // create minor step string
            if (N=0) or (C=DecimalSeparator) then
               SS:=SS+C
            else
               begin
               if (P=0) or (N>P) then
                  begin
                  if ((I-N+2)=DigitNumber) then
                     SS:=SS+'1'
                  else SS:=SS+'0';
                  end
               else
                  begin
                  if (P>(N+DigitNumber-1)) then // Decimal Separator Outside
                     begin
                     if ((I-N+2)=DigitNumber) then
                        SS:=SS+'1'
                     else SS:=SS+'0';
                     end
                  else                          // Decimal Separator Inside
                     begin
                     if ((I-N+1)=DigitNumber) then
                        SS:=SS+'1'
                     else SS:=SS+'0';
                     end;
                  end;
               end;
            end;

        // evaluate minor step
        if (RS<>Source) then // when change required
           begin
           RR:=StrToFloat(RS);
           SR:=StrToFloat(SS);
           if Up then // round up
              begin
              if (RR>0.0) then
                 Result:=FloatToStr(RR+SR)
              else Result:=FloatToStr(RR);
              end
           else       // round down
              begin
              if (RR>0.0) then
                 Result:=FloatToStr(RR)
              else Result:=FloatToStr(RR+SR);
              end;
           end
        else Result:=RS;
        end;
     end;
end;

function SmartRound(Source: string; DigitNumber: integer; Up: boolean): string;
var I, L, N, P: integer;
    RS, SS: string;
    RR, SR: real;
    C: char;
begin
  Result:=Source; // if rounding anyway not successful, then return the source string

  if IsFloatingPointString(Source) then
     begin
     L:=System.Length(Source);
     if (L>DigitNumber) then // break if source length too short
        begin
        P:=System.Pos(DecimalSeparator,Source);
        N:=0; RS:=''; SS:='';
        for I:=0 to L-1 do   // now evaluate individually all string characters
            begin
            C:=Source[I+1];  // this is the actually evaluated character

            // create major rounded string
            if (C<>DecimalSeparator) then // evaluate only if actual character is not Decimal Separator
               begin
               if (N=0) then // if no significant character (number digit) found until now
                  begin
                  if C in ['1'..'9'] then // then search for the first significant character
                     N:=I+1;
                  end
               else // if first significan number was alread detected, then search for final sign. char.
                  begin
                  if ((I - N + 1 + integer((P=0) or (N>P) or (P>(N+DigitNumber-1))))>DigitNumber) then
                     C:='0';
                  end;
               end;
            RS:=RS+C;

            // create minor step string
            if (C=DecimalSeparator) or (N=0) then
               SS:=SS+C
            else
               begin
               if ((I - N + 1 + integer((P=0) or (N>P) or (P>(N+DigitNumber-1))))=DigitNumber) then
                  SS:=SS+'1'
               else SS:=SS+'0';
               end;
            end;

        // evaluate minor step
        if (RS<>Source) then // when change required
           begin
           RR:=StrToFloat(RS);
           SR:=StrToFloat(SS);
           if (Up and (RR>0.0)) or
              (not(Up) and not(RR>0.0)) then
              Result:=FloatToStr(RR+SR)
           else Result:=FloatToStr(RR);
           end
        else Result:=RS;
        end;
     end;
end;

function SmartRound(Source: string; DigitNumber: integer; Up: boolean; DS: char): string;
var I, L, N, P: integer;
    RS, SS: string;
    RR, SR: real;
    C, ODS: char;
begin
  Result:=Source; // if rounding anyway not successful, then return the source string
  ODS:=DecimalSeparator;
  DecimalSeparator:=DS;

  if IsFloatingPointString(Source) then
     begin
     L:=System.Length(Source);
     if (L>DigitNumber) then // break if source length too short
        begin
        P:=System.Pos(DecimalSeparator,Source);
        N:=0; RS:=''; SS:='';
        for I:=0 to L-1 do   // now evaluate individually all string characters
            begin
            C:=Source[I+1];  // this is the actually evaluated character

            // create major rounded string
            if (C<>DecimalSeparator) then // evaluate only if actual character is not Decimal Separator
               begin
               if (N=0) then // if no significant character (number digit) found until now
                  begin
                  if C in ['1'..'9'] then // then search for the first significant character
                     N:=I+1;
                  end
               else // if first significan number was alread detected, then search for final sign. char.
                  begin
                  if ((I - N + 1 + integer((P=0) or (N>P) or (P>(N+DigitNumber-1))))>DigitNumber) then
                     C:='0';
                  end;
               end;
            RS:=RS+C;

            // create minor step string
            if (C=DecimalSeparator) or (N=0) then
               SS:=SS+C
            else
               begin
               if ((I - N + 1 + integer((P=0) or (N>P) or (P>(N+DigitNumber-1))))=DigitNumber) then
                  SS:=SS+'1'
               else SS:=SS+'0';
               end;
            end;

        // evaluate minor step
        if (RS<>Source) then // when change required
           begin
           RR:=StrToFloat(RS);
           SR:=StrToFloat(SS);
           if (Up and (RR>0.0)) or
              (not(Up) and not(RR>0.0)) then
              Result:=FloatToStr(RR+SR)
           else Result:=FloatToStr(RR);
           end
        else Result:=RS;
        end;
     end;
  DecimalSeparator:=ODS;
end;

initialization
  //
end.
