//******************************************************************************
//******************************************************************************
//******************************************************************************
//**
//**  DynLoadFTD2XX.PAS
//**
//**  Dynamic Load and Interface to FTD2XX.DLL (USB) Module Procedures
//**
//**  Autor : Georg  Hahn
//**  Datum :  xx.xx.06 / 22.05.07 / 25.07.08 / 21.10.08 / 22.09.09 / 19.05.10
//**
//******************************************************************************
//******************************************************************************
//******************************************************************************
unit DynLoadFTD2XX;

{$DEFINE UnitFinished}                // DesignPhase or UnitFinished

interface

uses Windows;

type
  FT_HANDLE = DWORD;
  PFT_HANDLE = ^FT_HANDLE;

  FT_DEVICE_LIST_INFO_NODE = record
    Flags: DWORD;
    DevType: DWORD;
    ID: DWORD;
    LocId: DWORD;
    SerialNumber: array [0..15] of char;
    Description: array [0..63] of char;
    ftHandle: FT_HANDLE;
    end;
  PFT_DEVICE_LIST_INFO_NODE = ^FT_DEVICE_LIST_INFO_NODE;

  FT_PROGRAM_DATA = record
    Signature1: dword;        // Header - must be 0x00000000
    Signature2: dword;        // Header - must be 0xffffffff
    Version: dword;           // Header - FT_PROGRAM_DATA version
                              // 0 = original
                              // 1 = FT2232C extensions
                              // 2 = FT232R extensions
    VendorId: word;           // 0x0403
    ProductId: word;          // 0x6001 for FTDI or D020 to D027 for MTL
    Manufacturer: PChar;      // "FTDI"
    ManufacturerId: PChar;    // "FT"
    Description: PChar;       // "USB HS Serial Converter"
    SerialNumber: PChar;      // "FT000001" if fixed, or NULL
    MaxPower: word;           // 0 < MaxPower <= 500
    PnP: word;                // 0 = disabled, 1 = enabled
    SelfPowered: word;        // 0 = bus powered, 1 = self powered
    RemoteWakeup: word;       // 0 = not capable, 1 = capable
    //
    // Rev4 extensions
    //
    Rev4: UCHAR;              // non-zero if Rev4 chip, zero otherwise
    IsoIn: UCHAR;             // non-zero if in endpoint is isochronous
    IsoOut: UCHAR;            // non-zero if out endpoint is isochronous
    PullDownEnable: UCHAR;    // non-zero if pull down enabled
    SerNumEnable: UCHAR;      // non-zero if serial number to be used
    USBVersionEnable: UCHAR;  // non-zero if chip uses USBVersion
    USBVersion: word;         // BCD (0x0200 => USB2)
    //
    // FT2232C extensions
    //
    Rev5: UCHAR;              // non-zero if Rev5 chip, zero otherwise
    IsoInA: UCHAR;            // non-zero if in endpoint is isochronous
    IsoInB: UCHAR;            // non-zero if in endpoint is isochronous
    IsoOutA: UCHAR;           // non-zero if out endpoint is isochronous
    IsoOutB: UCHAR;           // non-zero if out endpoint is isochronous
    PullDownEnable5: UCHAR;   // non-zero if pull down enabled
    SerNumEnable5: UCHAR;     // non-zero if serial number to be used
    USBVersionEnable5: UCHAR; // non-zero if chip uses USBVersion
    USBVersion5: word;        // BCD (0x0200 => USB2)
    AIsHighCurrent: UCHAR;    // non-zero if interface is high current
    BIsHighCurrent: UCHAR;    // non-zero if interface is high current
    IFAIsFifo: UCHAR;         // non-zero if interface is 245 FIFO
    IFAIsFifoTar: UCHAR;      // non-zero if interface is 245 FIFO CPU target
    IFAIsFastSer: UCHAR;      // non-zero if interface is Fast serial
    AIsVCP: UCHAR;            // non-zero if interface is to use VCP drivers
    IFBIsFifo: UCHAR;         // non-zero if interface is 245 FIFO
    IFBIsFifoTar: UCHAR;      // non-zero if interface is 245 FIFO CPU target
    IFBIsFastSer: UCHAR;      // non-zero if interface is Fast serial
    BIsVCP: UCHAR;            // non-zero if interface is to use VCP drivers
    //
    // FT232R extensions
    //
    UseExtOsc: UCHAR;         // Use External Oscillator
    HighDriveIOs: UCHAR;      // High Drive I/Os
    EndpointSize: UCHAR;      // Endpoint size
    PullDownEnableR: UCHAR;   // non-zero if pull down enabled
    SerNumEnableR: UCHAR;     // non-zero if serial number to be used
    InvertTXD: UCHAR;         // non-zero if invert TXD
    InvertRXD: UCHAR;         // non-zero if invert RXD
    InvertRTS: UCHAR;         // non-zero if invert RTS
    InvertCTS: UCHAR;         // non-zero if invert CTS
    InvertDTR: UCHAR;         // non-zero if invert DTR
    InvertDSR: UCHAR;         // non-zero if invert DSR
    InvertDCD: UCHAR;         // non-zero if invert DCD
    InvertRI: UCHAR;          // non-zero if invert RI
    Cbus0: UCHAR;             // Cbus Mux control - Ignored for FT245R
    Cbus1: UCHAR;             // Cbus Mux control - Ignored for FT245R
    Cbus2: UCHAR;             // Cbus Mux control - Ignored for FT245R
    Cbus3: UCHAR;             // Cbus Mux control - Ignored for FT245R
    Cbus4: UCHAR;             // Cbus Mux control - Ignored for FT245R
    RIsVCP: UCHAR;            // non-zero if using VCP drivers
    end;
  PFT_PROGRAM_DATA = ^FT_PROGRAM_DATA;

  FTDCB = record
    DCBlength: DWORD;      // sizeof(FTDCB)
    BaudRate: DWORD;       // Baudrate at which running
    Flags: DWORD;
    wReserved: WORD;       // Not currently used
    XonLim: WORD;          // Transmit X-ON threshold
    XoffLim: WORD;         // Transmit X-OFF threshold
    ByteSize: BYTE;        // Number of bits/byte, 4-8
    Parity: BYTE;          // 0-4=None,Odd,Even,Mark,Space
    StopBits: BYTE;        // 0,1,2 = 1, 1.5, 2
    XonChar: char;         // Tx and Rx X-ON character
    XoffChar: char;        // Tx and Rx X-OFF character
    ErrorChar: char;       // Error replacement char
    EofChar: char;         // End of Input character
    EvtChar: char;         // Received Event character
    wReserved1: WORD;      // Fill for now.
    end;
  PFTDCB= ^FTDCB;

  FTCOMSTAT = record
    // TComStateFlags: dword; (7 bits used, 25 reserved, totally 32 bits = 4 bytes)
    // DWORD fCtsHold : 1;
    // DWORD fDsrHold : 1;
    // DWORD fRlsdHold : 1;
    // DWORD fXoffHold : 1;
    // DWORD fXoffSent : 1;
    // DWORD fEof : 1;
    // DWORD fTxim : 1;
    // DWORD fReserved : 25;
    Flags: byte;
    Reserved: array[0..2] of byte;
    cbInQue: DWORD;
    cbOutQue: DWORD;
    end;
  PFTCOMSTAT=^FTCOMSTAT;

  FTTIMEOUTS = record
    ReadIntervalTimeout: DWORD;         // Maximum time between read chars.
    ReadTotalTimeoutMultiplier: DWORD;  // Multiplier of characters.
    ReadTotalTimeoutConstant: DWORD;    // Constant in milliseconds.
    WriteTotalTimeoutMultiplier: DWORD; // Multiplier of characters.
    WriteTotalTimeoutConstant: DWORD;   // Constant in milliseconds.
    end;
  PFTTIMEOUTS = ^FTTIMEOUTS;

{$IFDEF DesignPhase}
  procedure CheckFTD2XXLibrary;
{$ENDIF}

  function LoadFTD2XXLibrary: boolean;
  procedure CloseFTD2XXLibrary;

// FTD2XX global  variables
var
    FTD2XXLibraryAvailable,
    ExtendedFTD2XXAPIAvailable: boolean;

    // General FTD2XX Procedures
    FT_GetLibraryVersion:       function (DLLVersion: pdword): integer; stdcall;
    FT_GetDriverVersion:        function (ftHandle: FT_HANDLE; DriverVersion: pdword): integer; stdcall;
    FT_CreateDeviceInfoList:    function (NumberDevices: pdword): integer; stdcall;
    FT_ListDevices:             function (pvArg1, pvArg2: pointer; dwFlags: dword): integer; stdcall;
    FT_GetDeviceInfoList:       function (DeviceList: PFT_DEVICE_LIST_INFO_NODE; NumberDevices: pdword): integer; stdcall;
    FT_GetDeviceInfoDetail:     function (Index: dword; Flags, DeviceType, ID, LocID: pdword; SerialNumber, Description: PChar; ftHandle: PFT_HANDLE): integer; stdcall;

    // Classic Interface Procedures
    FT_Open:                    function (PVDevice: integer; ptHandle: PFT_HANDLE): integer; stdcall;
    FT_Close:                   function (ftHandle: FT_HANDLE): integer; stdcall;
    FT_Read:                    function (ftHandle: FT_HANDLE; FTInBuf: pointer; BufferSize: dword; ResultPtr: pword): integer; stdcall;
    FT_Write:                   function (ftHandle: FT_HANDLE; FTOutBuf: pointer; BufferSize: dword; ResultPtr: pword): integer; stdcall;
    FT_SetBaudRate:             function (ftHandle: FT_HANDLE; BaudRate: dword): integer; stdcall;
    FT_SetDataCharacteristics:  function (ftHandle: FT_HANDLE; WordLength, StopBits, Parity: byte): integer; stdcall;
    FT_SetFlowControl:          function (ftHandle: FT_HANDLE; FlowControl: word; XonChar, XoffChar: byte): integer; stdcall;
    FT_ResetDevice:             function (ftHandle: FT_HANDLE): integer; stdcall;
    FT_SetDtr:                  function (ftHandle: FT_HANDLE): integer; stdcall;
    FT_ClrDtr:                  function (ftHandle: FT_HANDLE): integer; stdcall;
    FT_SetRts:                  function (ftHandle: FT_HANDLE): integer; stdcall;
    FT_ClrRts:                  function (ftHandle: FT_HANDLE): integer; stdcall;
    FT_GetModemStatus:          function (ftHandle: FT_HANDLE; ModemStatus: pword): integer; stdcall;
    FT_SetChars:                function (ftHandle: FT_HANDLE; EventChar, EventCharEnabled, ErrorChar, ErrorCharEnabled: byte): integer; stdcall;
    FT_Purge:                   function (ftHandle: FT_HANDLE; Mask: dword): integer; stdcall;
    FT_SetTimeouts:             function (ftHandle: FT_HANDLE; ReadTimeout, WriteTimeout: dword): integer; stdcall;
    FT_GetQueueStatus:          function (ftHandle: FT_HANDLE; RxBytes: pword): integer; stdcall;
    FT_OpenEx:                  function (pvArg1: pointer; dwFlags: dword; ptHandle: PFT_HANDLE): integer; stdcall;

    // EEPROM Interface Procedures
    FT_ReadEE:                  function (ftHandle: FT_HANDLE; WordOffset: dword; Value :pword): integer; stdcall;
    FT_WriteEE:                 function (ftHandle: FT_HANDLE; WordOffset: dword; Value :word): integer; stdcall;
    FT_EraseEE:                 function (ftHandle: FT_HANDLE): integer; stdcall;

    FT_EE_Read:                 function (ftHandle: FT_HANDLE; Data: PFT_PROGRAM_DATA): integer; stdcall;
    FT_EE_ReadEx:               function (ftHandle: FT_HANDLE; Data: PFT_PROGRAM_DATA; Manufacturer, ManufacturerId, Description, SerialNumber: PChar): integer; stdcall;

    FT_EE_Program:              function (ftHandle: FT_HANDLE; Data: PFT_PROGRAM_DATA): integer; stdcall;
    FT_EE_ProgramEx:            function (ftHandle: FT_HANDLE; Data: PFT_PROGRAM_DATA; Manufacturer, ManufacturerId, Description, SerialNumber: PChar): integer; stdcall;

    FT_EE_UARead:               function (ftHandle: FT_HANDLE; Data: pbyte; DataLen: dword; BytesRead: pdword): integer; stdcall;
    FT_EE_UAWrite:              function (ftHandle: FT_HANDLE; Data: pbyte; DataLen: dword): integer; stdcall;
    FT_EE_UASize:               function (ftHandle: FT_HANDLE; Size: pdword): integer; stdcall;

    // Extended API functions
    FT_SetLatencyTimer:         function (ftHandle: FT_HANDLE; ucTimer: byte): integer; stdcall;
    FT_GetLatencyTimer:         function (ftHandle: FT_HANDLE; pucTimer: pbyte): integer; stdcall;
    FT_SetBitMode:              function (ftHandle: FT_HANDLE; ucMask, ucMode: byte): integer; stdcall;
    FT_GetBitMode:              function (ftHandle: FT_HANDLE; pucMode: pbyte): integer; stdcall;
    FT_SetUSBParameters:        function (ftHandle: FT_HANDLE; InTransferSize, OutTransferSize: dword): integer; stdcall;

    // Win32 API Interface Procedures
    FT_W32_CreateFile:          function (lpFileName: PChar; dwDesiredAccess, dwShareMode: dword; lpSecurityAttributes: PSecurityAttributes; dwCreationDisposition, dwFlagsAndAttributes: dword; hTemplateFile: THandle): FT_HANDLE; stdcall;
    FT_W32_CloseHandle:         function (ftHandle: FT_HANDLE): BOOL; stdcall;
    FT_W32_ReadFile:            function (ftHandle: FT_HANDLE; var Buffer; nNumberOfBytesToRead: dword; lpNumberOfBytesRead: pdword; lpOverlapped: pOverlapped): BOOL; stdcall; // var lpNumberOfBytesRead: pdword;
    FT_W32_WriteFile:           function (ftHandle: FT_HANDLE; const Buffer; nNumberOfBytesToWrite: dword; lpNumberOfBytesWritten: pdword; lpOverlapped: pOverlapped): BOOL; stdcall; // var lpNumberOfBytesWritten: pdword;

    FT_W32_GetLastError:        function (ftHandle: FT_HANDLE): dword; stdcall;
    FT_W32_GetOverlappedResult: function (ftHandle: FT_HANDLE; const lpOverlapped: pOverlapped; lpNumberOfBytesTransferred: pdword; bWait: BOOL): BOOL; stdcall; // var lpNumberOfBytesTransferred: pdword;
    FT_W32_ClearCommBreak:      function (ftHandle: FT_HANDLE): BOOL; stdcall;
    FT_W32_ClearCommError:      function (ftHandle: FT_HANDLE; lpErrors: pdword; lpStat: PFTCOMSTAT): BOOL; stdcall;  // var lpErrors: pdword; 
    FT_W32_EscapeCommFunction:  function (ftHandle: FT_HANDLE; dwFunc: dword): BOOL; stdcall;
    FT_W32_GetCommModemStatus:  function (ftHandle: FT_HANDLE; lpModemStat: pdword): BOOL; stdcall; // var lpModemStat: pdword
    FT_W32_GetCommMask:         function (ftHandle: FT_HANDLE; dwEvtMask: pdword): BOOL; stdcall;  // var dwEvtMask: 
    FT_W32_GetCommState:        function (ftHandle: FT_HANDLE; lpDCB: PFTDCB): BOOL; stdcall;  // var lpDCB: PFTDCB
    FT_W32_GetCommTimeouts:     function (ftHandle: FT_HANDLE; var lpCommTimeouts: PFTTIMEOUTS): BOOL; stdcall;
    FT_W32_PurgeComm:           function (ftHandle: FT_HANDLE; dwFlags: dword): BOOL; stdcall;
    FT_W32_SetCommBreak:        function (ftHandle: FT_HANDLE): BOOL; stdcall;
    FT_W32_SetCommMask:         function (ftHandle: FT_HANDLE; dwEvtMask: dword): BOOL; stdcall;
    FT_W32_SetCommState:        function (ftHandle: FT_HANDLE; const lpDCB: PFTDCB): BOOL; stdcall;
    FT_W32_SetCommTimeouts:     function (ftHandle: FT_HANDLE; const lpCommTimeouts: PFTTIMEOUTS): BOOL; stdcall;
    FT_W32_SetupComm:           function (ftHandle: FT_HANDLE; dwInQueue, dwOutQueue: dword): BOOL; stdcall;
    FT_W32_WaitCommEvent:       function (ftHandle: FT_HANDLE; var lpEvtMask: pdword; lpOverlapped: pOverlapped): BOOL; stdcall;

const
// FT_Result Values
    FT_OK                           = 0;
    FT_SUCCESS                      = FT_OK;
    FT_INVALID_HANDLE               = 1;
    FT_DEVICE_NOT_FOUND             = 2;
    FT_DEVICE_NOT_OPENED            = 3;
    FT_IO_ERROR                     = 4;
    FT_INSUFFICIENT_RESOURCES       = 5;
    FT_INVALID_PARAMETER            = 6;
    FT_INVALID_BAUD_RATE            = 7;

    FT_DEVICE_NOT_OPENED_FOR_ERASE  = 8;
    FT_DEVICE_NOT_OPENED_FOR_WRITE  = 9;
    FT_FAILED_TO_WRITE_DEVICE       = 10;
    FT_EEPROM_READ_FAILED           = 11;
    FT_EEPROM_WRITE_FAILED          = 12;
    FT_EEPROM_ERASE_FAILED          = 13;
	  FT_EEPROM_NOT_PRESENT           = 14;
	  FT_EEPROM_NOT_PROGRAMMED        = 15;
	  FT_INVALID_ARGS                 = 16;
	  FT_NOT_SUPPORTED                = 17;
	  FT_OTHER_ERROR                  = 18;
  	FT_DEVICE_LIST_NOT_READY        = 19;

// FT_Open_Ex Flags
    FT_OPEN_BY_SERIAL_NUMBER        = 1;
    FT_OPEN_BY_DESCRIPTION          = 2;
    FT_OPEN_BY_LOCATION	            = 4;

// FT_List_Devices Flags
    FT_LIST_NUMBER_ONLY             = $80000000;
    FT_LIST_BY_INDEX                = $40000000;
    FT_LIST_ALL                     = $20000000;

// Baud Rate Selection
    FT_BAUD_300 = 300;
    FT_BAUD_600 = 600;
    FT_BAUD_1200 = 1200;
    FT_BAUD_2400 = 2400;
    FT_BAUD_4800 = 4800;
    FT_BAUD_9600 = 9600;
    FT_BAUD_14400 = 14400;
    FT_BAUD_19200 = 19200;
    FT_BAUD_38400 = 38400;
    FT_BAUD_57600 = 57600;
    FT_BAUD_115200 = 115200;
    FT_BAUD_230400 = 230400;
    FT_BAUD_460800 = 460800;
    FT_BAUD_921600 = 921600;

// Data Bits Selection
    FT_DATA_BITS_5 = 5;
    FT_DATA_BITS_6 = 6;
    FT_DATA_BITS_7 = 7;
    FT_DATA_BITS_8 = 8;

// Stop Bits Selection
    FT_STOP_BITS_1    = 0;
    FT_STOP_BITS_1_5  = 1;
    FT_STOP_BITS_2    = 2;

// Parity Selection
    FT_PARITY_NONE    = 0;
    FT_PARITY_ODD     = 1;
    FT_PARITY_EVEN    = 2;
    FT_PARITY_MARK    = 3;
    FT_PARITY_SPACE   = 4;

// FTDCB Flags

    FT_dcb_Binary                = $00000001;   // Binary Mode (skip EOF check)
    FT_dcb_Parity                = $00000002;   // Enable parity checking
    FT_dcb_OutxCtsFlow           = $00000004;   // CTS handshaking on output
    FT_dcb_OutxDsrFlow           = $00000008;   // DSR handshaking on output
    FT_dcb_DtrControlEnable      = $00000010;   // DTR Flow control 1
    FT_dcb_DtrControlHandshake   = $00000020;   // DTR Flow control 2
    FT_dcb_DsrSensitivity        = $00000040;   // DSR Sensitivity
    FT_dcb_TXContinueOnXoff      = $00000080;   // Continue TX when Xoff sent
    FT_dcb_OutX                  = $00000100;   // Enable output X-ON/X-OFF
    FT_dcb_InX                   = $00000200;   // Enable input X-ON/X-OFF
    FT_dcb_ErrorChar             = $00000400;   // Enable Err Replacement
    FT_dcb_Null                  = $00000800;   // Enable Null stripping
    FT_dcb_RtsControlEnable      = $00001000;   // Rts Flow control 1
    FT_dcb_RtsControlHandshake   = $00002000;   // Rts Flow control 2
    FT_dcb_RtsControlToggle      = $00003000;   // Rts Flow control 1+2
    FT_dcb_AbortOnError          = $00004000;   // Abort all reads and writes on Error

// Flow Control Selection
    FT_FLOW_NONE      = $0000;
    FT_FLOW_RTS_CTS   = $0100;
    FT_FLOW_DTR_DSR   = $0200;
    FT_FLOW_XON_XOFF  = $0400;

// Purge Commands
    FT_PURGE_RX       = 1;
    FT_PURGE_TX       = 2;

//  Event Handler
    FT_EVENT_RXCHAR       = 1;
    FT_EVENT_MODEM_STATUS = 2;
    FT_EVENT_TXEMPTY      = 4;

// Timouts
    FT_DEFAULT_RX_TIMEOUT = 300;
    FT_DEFAULT_TX_TIMEOUT = 300;

// Device Types
    FT_DEVICE_BM          = 0;
    FT_DEVICE_AM          = 1;
    FT_DEVICE_100AX       = 2;
    FT_DEVICE_UNKNOWN     = 3;
    FT_DEVICE_2232C       = 4;
    FT_DEVICE_232R        = 5;

// IO Buffer Sizes
    FT_In_Buffer_Size = $8000;    // 32k
    FT_In_Buffer_Index = FT_In_Buffer_Size - 1;
    FT_Out_Buffer_Size = $8000;    // 32k
    FT_Out_Buffer_Index = FT_Out_Buffer_Size - 1;

implementation

uses SysUtils;

const
    CLibFile   = 'FTD2XX.DLL';

{$IFDEF DesignPhase}

    // There are two known library version: 2.01.04 = "Old" and 3.01.18 = "New"

    // General FTD2XX Procedures
    CFTD2XXGeneralProcFuncNamesOld: string =
                       'FT_ListDevices';

    CFTD2XXGeneralProcFuncNamesNew: array [1..5] of string =
                      ('FT_GetLibraryVersion',                //
                       'FT_GetDriverVersion',                 //
                       'FT_CreateDeviceInfoList',             //
                       'FT_GetDeviceInfoList',                //
                       'FT_GetDeviceInfoDetail');             //

    // Classic Interface Procedures
    CFTD2XXClassicProcFuncNames: array [1..18] of string =
                      ('FT_Open',
                       'FT_Close',
                       'FT_Read',
                       'FT_Write',
                       'FT_SetBaudRate',
                       'FT_SetDataCharacteristics',
                       'FT_SetFlowControl',
                       'FT_ResetDevice',
                       'FT_SetDtr',
                       'FT_ClrDtr',
                       'FT_SetRts',
                       'FT_ClrRts',
                       'FT_GetModemStatus',
                       'FT_SetChars',
                       'FT_Purge',
                       'FT_SetTimeouts',
                       'FT_GetQueueStatus',
                       'FT_OpenEx');

    // EEPROM Interface Procedures
    CFTD2XXEEPROMProcFuncNames: array [1..10] of string =
                      ('FT_ReadEE',
                       'FT_WriteEE',
                       'FT_EraseEE',
                       'FT_EE_Read',
                       'FT_EE_ReadEx',
                       'FT_EE_Program',
                       'FT_EE_ProgramEx',
                       'FT_EE_UARead',
                       'FT_EE_UAWrite',
                       'FT_EE_UASize');

    // Extended API functions
    CFTD2XXExtendedAPIFunctions: array [1..5] of string =
                      ('FT_SetLatencyTimer',
                       'FT_GetLatencyTimer',
                       'FT_SetBitMode',
                       'FT_GetBitMode',
                       'FT_SetUSBParameters');

    // Win32 API Interface Procedures
    CFTD2XXWin32APIProcFuncNamesOld: array [1..19] of string =
                      ('FT_W32_CreateFile',
                       'FT_W32_CloseHandle',
                       'FT_W32_ReadFile',
                       'FT_W32_WriteFile',
                       'FT_W32_GetLastError',
                       'FT_W32_GetOverlappedResult',
                       'FT_W32_ClearCommBreak',
                       'FT_W32_ClearCommError',
                       'FT_W32_EscapeCommFunction',
                       'FT_W32_GetCommModemStatus',
                       'FT_W32_GetCommState',
                       'FT_W32_GetCommTimeouts',
                       'FT_W32_PurgeComm',
                       'FT_W32_SetCommBreak',
                       'FT_W32_SetCommMask',
                       'FT_W32_SetCommState',
                       'FT_W32_SetCommTimeouts',
                       'FT_W32_SetupComm',
                       'FT_W32_WaitCommEvent');

    CFTD2XXWin32APIProcFuncNamesNew: string =
                       'FT_W32_GetCommMask';

{$ENDIF}

var LibraryHandle: THandle;
    NewFTD2XXLibrary: boolean;

{$IFDEF DesignPhase}
procedure CheckFTD2XXLibrary;
var LibCharName: array [0..MAX_PATH] of char;
    CBProc: procedure; stdcall;
    LibName: string;
    Len,I: word;
begin
   Len:=GetCurrentDirectory(MAX_PATH,LibCharName);
   if (Len>2) and (Len<240) then
      begin
      StrCat(LibCharName,'\'+CLibFile);
      LibName:=StrPas(LibCharName);
      end;
   if not(FileExists(LibName)) then
      begin
      Len:=GetSystemDirectory(LibCharName,MAX_PATH);
      if (Len>2) and (Len<240) then
         begin
         StrCat(LibCharName,'\'+CLibFile);
         LibName:=StrPas(LibCharName);
         end;
      end;
  if FileExists(LibName) then
     begin
     StrPCopy(LibCharName,LibName);
     LibraryHandle:=LoadLibrary(LibCharName);
     // General FTD2XX Procedures - Old
     StrPCopy(LibCharName,CFTD2XXGeneralProcFuncNamesOld);
     @CBProc:=GetProcAddress(LibraryHandle,LibCharName);
     if (@CBProc<>nil) then
        MessageDlg(CFTD2XXGeneralProcFuncNamesOld,mtInformation,[mbOK],0)
     else MessageDlg(CFTD2XXGeneralProcFuncNamesOld,mtError,[mbAbort],0);
     // General FTD2XX Procedures - New
     for I:=1 to 5 do
         begin
         StrPCopy(LibCharName,CFTD2XXGeneralProcFuncNamesNew[I]);
         @CBProc:=GetProcAddress(LibraryHandle,LibCharName);
         if (@CBProc<>nil) then
            MessageDlg(CFTD2XXGeneralProcFuncNamesNew[I],mtInformation,[mbOK],0)
         else MessageDlg(CFTD2XXGeneralProcFuncNamesNew[I],mtError,[mbAbort],0);
         end;
     // Classic Interface Procedures
     for I:=1 to 18 do
         begin
         StrPCopy(LibCharName,CFTD2XXClassicProcFuncNames[I]);
         @CBProc:=GetProcAddress(LibraryHandle,LibCharName);
         if (@CBProc<>nil) then
            MessageDlg(CFTD2XXClassicProcFuncNames[I],mtInformation,[mbOK],0)
         else MessageDlg(CFTD2XXClassicProcFuncNames[I],mtError,[mbAbort],0);
         end;
    // EEPROM Interface Procedures
     for I:=1 to 10 do
         begin
         StrPCopy(LibCharName,CFTD2XXEEPROMProcFuncNames[I]);
         @CBProc:=GetProcAddress(LibraryHandle,LibCharName);
         if (@CBProc<>nil) then
            MessageDlg(CFTD2XXEEPROMProcFuncNames[I],mtInformation,[mbOK],0)
         else MessageDlg(CFTD2XXEEPROMProcFuncNames[I],mtError,[mbAbort],0);
         end;
    // Extended API functions
     for I:=1 to 5 do
         begin
         StrPCopy(LibCharName,CFTD2XXExtendedAPIFunctions[I]);
         @CBProc:=GetProcAddress(LibraryHandle,LibCharName);
         if (@CBProc<>nil) then
            MessageDlg(CFTD2XXExtendedAPIFunctions[I],mtInformation,[mbOK],0)
         else MessageDlg(CFTD2XXExtendedAPIFunctions[I],mtError,[mbAbort],0);
         end;
     // Win32 API Interface Procedures - Old
     for I:=1 to 19 do
         begin
         StrPCopy(LibCharName,CFTD2XXWin32APIProcFuncNamesOld[I]);
         @CBProc:=GetProcAddress(LibraryHandle,LibCharName);
         if (@CBProc<>nil) then
            MessageDlg(CFTD2XXWin32APIProcFuncNamesOld[I],mtInformation,[mbOK],0)
         else MessageDlg(CFTD2XXWin32APIProcFuncNamesOld[I],mtError,[mbAbort],0);
         end;
     // Win32 API Interface Procedures - New
     StrPCopy(LibCharName,CFTD2XXWin32APIProcFuncNamesNew);
     @CBProc:=GetProcAddress(LibraryHandle,LibCharName);
     if (@CBProc<>nil) then
        MessageDlg(CFTD2XXWin32APIProcFuncNamesNew,mtInformation,[mbOK],0)
     else MessageDlg(CFTD2XXWin32APIProcFuncNamesNew,mtError,[mbAbort],0);
     FreeLibrary(LibraryHandle);
     end
  else LibraryHandle:=0;
end;
{$ENDIF}

procedure ResetPointer;
begin
  FTD2XXLibraryAvailable:=false;
  NewFTD2XXLibrary:=false;
end;

function LoadFTD2XXLibrary: boolean;
var FTD2XXGeneralProceduresOld, FTD2XXGeneralProceduresNew,
    FTD2XXClassicProcedures, FTD2XXEEPROMProcedures, FTD2XXExtendedAPIProcedures,
    FTD2XXWin32APIProceduresOld,FTD2XXWin32APIProceduresNew: boolean;
    LibCharName: array [0..MAX_PATH] of char;
    LibName: string;
    Len: word;
begin
   Len:=GetCurrentDirectory(MAX_PATH,LibCharName);
   if (Len>2) and (Len<240) then
      begin
      StrCat(LibCharName,'\'+CLibFile);
      LibName:=StrPas(LibCharName);
      end;
   if not(FileExists(LibName)) then
      begin
      Len:=GetSystemDirectory(LibCharName,MAX_PATH);
      if (Len>2) and (Len<240) then
         begin
         StrCat(LibCharName,'\'+CLibFile);
         LibName:=StrPas(LibCharName);
         end;
      end;
  if FileExists(LibName) then
     begin
     StrPCopy(LibCharName,LibName);
     LibraryHandle:=LoadLibrary(LibCharName);

     // General FTD2XX Procedures - Old
     @FT_ListDevices:=GetProcAddress(LibraryHandle,'FT_ListDevices');

     FTD2XXGeneralProceduresOld:=(@FT_ListDevices<>nil);

     // General FTD2XX Procedures - New
     @FT_GetLibraryVersion:=GetProcAddress(LibraryHandle,'FT_GetLibraryVersion');
     @FT_GetDriverVersion:=GetProcAddress(LibraryHandle,'FT_GetDriverVersion');
     @FT_CreateDeviceInfoList:=GetProcAddress(LibraryHandle,'FT_CreateDeviceInfoList');
     @FT_GetDeviceInfoList:=GetProcAddress(LibraryHandle,'FT_GetDeviceInfoList');
     @FT_GetDeviceInfoDetail:=GetProcAddress(LibraryHandle,'FT_GetDeviceInfoDetail');

     FTD2XXGeneralProceduresNew:=(@FT_GetLibraryVersion<>nil) and
                                 (@FT_GetDriverVersion<>nil) and
                                 (@FT_CreateDeviceInfoList<>nil) and
                                 (@FT_GetDeviceInfoList<>nil) and
                                 (@FT_GetDeviceInfoDetail<>nil);

     // Classic Interface Procedures
     @FT_Open:=GetProcAddress(LibraryHandle,'FT_Open');
     @FT_Close:=GetProcAddress(LibraryHandle,'FT_Close');

     @FT_Read:=GetProcAddress(LibraryHandle,'FT_Read');
     @FT_Write:=GetProcAddress(LibraryHandle,'FT_Write');

     @FT_SetBaudRate:=GetProcAddress(LibraryHandle,'FT_SetBaudRate');
     @FT_SetDataCharacteristics:=GetProcAddress(LibraryHandle,'FT_SetDataCharacteristics');
     @FT_SetFlowControl:=GetProcAddress(LibraryHandle,'FT_SetFlowControl');
     @FT_ResetDevice:=GetProcAddress(LibraryHandle,'FT_ResetDevice');
     @FT_SetDtr:=GetProcAddress(LibraryHandle,'FT_SetDtr');
     @FT_ClrDtr:=GetProcAddress(LibraryHandle,'FT_ClrDtr');
     @FT_SetRts:=GetProcAddress(LibraryHandle,'FT_SetRts');
     @FT_ClrRts:=GetProcAddress(LibraryHandle,'FT_ClrRts');

     @FT_GetModemStatus:=GetProcAddress(LibraryHandle,'FT_GetModemStatus');
     @FT_SetChars:=GetProcAddress(LibraryHandle,'FT_SetChars');
     @FT_Purge:=GetProcAddress(LibraryHandle,'FT_Purge');
     @FT_SetTimeouts:=GetProcAddress(LibraryHandle,'FT_SetTimeouts');
     @FT_GetQueueStatus:=GetProcAddress(LibraryHandle,'FT_GetQueueStatus');
     @FT_OpenEx:=GetProcAddress(LibraryHandle,'FT_OpenEx');

     FTD2XXClassicProcedures:=(@FT_Open<>nil) and
                              (@FT_Close<>nil) and
                              (@FT_Read<>nil) and
                              (@FT_Write<>nil) and
                              (@FT_SetBaudRate<>nil) and
                              (@FT_SetDataCharacteristics<>nil) and
                              (@FT_SetFlowControl<>nil) and
                              (@FT_ResetDevice<>nil) and
                              (@FT_SetDtr<>nil) and
                              (@FT_ClrDtr<>nil) and
                              (@FT_SetRts<>nil) and
                              (@FT_ClrRts<>nil) and

                              (@FT_GetModemStatus<>nil) and
                              (@FT_SetChars<>nil) and
                              (@FT_Purge<>nil) and
                              (@FT_SetTimeouts<>nil) and
                              (@FT_GetQueueStatus<>nil) and
                              (@FT_OpenEx<>nil);

     // EEPROM Interface Procedures
     @FT_ReadEE:=GetProcAddress(LibraryHandle,'FT_ReadEE');
     @FT_WriteEE:=GetProcAddress(LibraryHandle,'FT_WriteEE');
     @FT_EraseEE:=GetProcAddress(LibraryHandle,'FT_EraseEE');
     @FT_EE_Read:=GetProcAddress(LibraryHandle,'FT_EE_Read');
     @FT_EE_ReadEx:=GetProcAddress(LibraryHandle,'FT_EE_ReadEx');
     @FT_EE_Program:=GetProcAddress(LibraryHandle,'FT_EE_Program');
     @FT_EE_ProgramEx:=GetProcAddress(LibraryHandle,'FT_EE_ProgramEx');
     @FT_EE_UARead:=GetProcAddress(LibraryHandle,'FT_EE_UARead');
     @FT_EE_UAWrite:=GetProcAddress(LibraryHandle,'FT_EE_UAWrite');
     @FT_EE_UASize:=GetProcAddress(LibraryHandle,'FT_EE_UASize');

     FTD2XXEEPROMProcedures:=(@FT_ReadEE<>nil) and
                             (@FT_WriteEE<>nil) and
                             (@FT_EraseEE<>nil) and
                             (@FT_EE_Read<>nil) and
                             (@FT_EE_ReadEx<>nil) and
                             (@FT_EE_Program<>nil) and
                             (@FT_EE_ProgramEx<>nil) and
                             (@FT_EE_UARead<>nil) and
                             (@FT_EE_UAWrite<>nil) and
                             (@FT_EE_UASize<>nil);

    // Extended API functions
     @FT_SetLatencyTimer:=GetProcAddress(LibraryHandle,'FT_SetLatencyTimer');
     @FT_GetLatencyTimer:=GetProcAddress(LibraryHandle,'FT_GetLatencyTimer');
     @FT_SetBitMode:=GetProcAddress(LibraryHandle,'FT_SetBitMode');
     @FT_GetBitMode:=GetProcAddress(LibraryHandle,'FT_GetBitMode');
     @FT_SetUSBParameters:=GetProcAddress(LibraryHandle,'FT_SetUSBParameters');

     FTD2XXExtendedAPIProcedures:=(@FT_SetLatencyTimer<>nil) and
                                  (@FT_GetLatencyTimer<>nil) and
                                  (@FT_SetBitMode<>nil) and
                                  (@FT_GetBitMode<>nil) and
                                  (@FT_SetUSBParameters<>nil);

     // Win32 API Interface Procedures
     @FT_W32_CreateFile:=GetProcAddress(LibraryHandle,'FT_W32_CreateFile');
     @FT_W32_CloseHandle:=GetProcAddress(LibraryHandle,'FT_W32_CloseHandle');
     @FT_W32_ReadFile:=GetProcAddress(LibraryHandle,'FT_W32_ReadFile');
     @FT_W32_WriteFile:=GetProcAddress(LibraryHandle,'FT_W32_WriteFile');
     @FT_W32_GetLastError:=GetProcAddress(LibraryHandle,'FT_W32_GetLastError');
     @FT_W32_GetOverlappedResult:=GetProcAddress(LibraryHandle,'FT_W32_GetOverlappedResult');
     @FT_W32_ClearCommBreak:=GetProcAddress(LibraryHandle,'FT_W32_ClearCommBreak');
     @FT_W32_ClearCommError:=GetProcAddress(LibraryHandle,'FT_W32_ClearCommError');
     @FT_W32_EscapeCommFunction:=GetProcAddress(LibraryHandle,'FT_W32_EscapeCommFunction');
     @FT_W32_GetCommModemStatus:=GetProcAddress(LibraryHandle,'FT_W32_GetCommModemStatus');
     @FT_W32_GetCommState:=GetProcAddress(LibraryHandle,'FT_W32_GetCommState');
     @FT_W32_GetCommTimeouts:=GetProcAddress(LibraryHandle,'FT_W32_GetCommTimeouts');
     @FT_W32_PurgeComm:=GetProcAddress(LibraryHandle,'FT_W32_PurgeComm');
     @FT_W32_SetCommBreak:=GetProcAddress(LibraryHandle,'FT_W32_SetCommBreak');
     @FT_W32_SetCommMask:=GetProcAddress(LibraryHandle,'FT_W32_SetCommMask');
     @FT_W32_SetCommState:=GetProcAddress(LibraryHandle,'FT_W32_SetCommState');
     @FT_W32_SetCommTimeouts:=GetProcAddress(LibraryHandle,'FT_W32_SetCommTimeouts');
     @FT_W32_SetupComm:=GetProcAddress(LibraryHandle,'FT_W32_SetupComm');
     @FT_W32_WaitCommEvent:=GetProcAddress(LibraryHandle,'FT_W32_WaitCommEvent');

     FTD2XXWin32APIProceduresOld:=(@FT_W32_CreateFile<>nil) and
                                  (@FT_W32_CloseHandle<>nil) and
                                  (@FT_W32_ReadFile<>nil) and
                                  (@FT_W32_WriteFile<>nil) and
                                  (@FT_W32_GetLastError<>nil) and
                                  (@FT_W32_GetOverlappedResult<>nil) and
                                  (@FT_W32_ClearCommBreak<>nil) and
                                  (@FT_W32_ClearCommError<>nil) and
                                  (@FT_W32_EscapeCommFunction<>nil) and
                                  (@FT_W32_GetCommModemStatus<>nil) and
                                  (@FT_W32_GetCommState<>nil) and
                                  (@FT_W32_GetCommTimeouts<>nil) and
                                  (@FT_W32_PurgeComm<>nil) and
                                  (@FT_W32_SetCommBreak<>nil) and
                                  (@FT_W32_SetCommMask<>nil) and
                                  (@FT_W32_SetCommState<>nil) and
                                  (@FT_W32_SetCommTimeouts<>nil) and
                                  (@FT_W32_SetupComm<>nil) and
                                  (@FT_W32_WaitCommEvent<>nil);

     // Win32 API Interface Procedures
     @FT_W32_GetCommMask:=GetProcAddress(LibraryHandle,'FT_W32_GetCommMask');

     FTD2XXWin32APIProceduresNew:=(@FT_W32_GetCommMask<>nil);


     if FTD2XXGeneralProceduresOld and
        FTD2XXClassicProcedures and
        FTD2XXEEPROMProcedures and
        FTD2XXWin32APIProceduresOld then
        begin
        FTD2XXLibraryAvailable:=true;
        NewFTD2XXLibrary:=FTD2XXGeneralProceduresNew and FTD2XXWin32APIProceduresNew;
        ExtendedFTD2XXAPIAvailable:=FTD2XXExtendedAPIProcedures;
        Result:=true;
        end
     else
        begin
        FreeLibrary(LibraryHandle);
        FTD2XXLibraryAvailable:=false;
        LibraryHandle:=0;
        Result:=false;
        end;
     end
  else
     begin
     FTD2XXLibraryAvailable:=false;
     LibraryHandle:=0;
     Result:=false;
     end;
end;

procedure CloseFTD2XXLibrary;
begin
  if (LibraryHandle<>0) then
     begin
     FreeLibrary(LibraryHandle);
     FTD2XXLibraryAvailable:=false;
     LibraryHandle:=0;
     end;
end;

initialization
  ResetPointer;
end.
