{******************************************************************************
*******************************************************************************
*******************************************************************************
***
***	OWN DELPHI COMPONENTS
***	EDITCONTROLS.PAS
***
***	Collection of useful Edit and Display components
***
***	Autor : Georg  Hahn
***	Datum :  02.11.99 / 06.05.00 / 06.01.03 / 13.08.04 / 11.05.05 / 15.09.08
***
*******************************************************************************
*******************************************************************************
******************************************************************************}

unit EditControls;
{$D+}

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, StdCtrls;

const
  RegisterPage = 'EditControls';

  CWidthMargin     = 2;
  CHeightMargin    = 4;
  CDefaultLCWidth  = 120;
  CDefaultLCHeight = 21;
  CScrollBarWidth  = 16;
  CScrollBarLength = 128;

type
  TGHScrollBar = class(TWinControl)
  private
    fKind: TScrollBarKind;
    fPosition: integer;
    fMin: integer;
    fMax: integer;
    fPageSize: integer;
    fRTLFactor: integer;
    fReverseScrolling,
    fPreventScrollJitter: boolean;
    fSmallChange: TScrollBarInc;
    fLargeChange: TScrollBarInc;
    fOnChange: TNotifyEvent;
    fOnScroll: TScrollEvent;
    procedure DoScroll(var Message: TWMScroll);
    function NotRightToLeft: Boolean;
    procedure SetKind(Value: TScrollBarKind);
    procedure SetMax(Value: integer);
    procedure SetMin(Value: integer);
    function GetPosition: integer;
    procedure SetPosition(Value: integer);
    procedure SetPageSize(Value: integer);
    procedure SetReverseScrolling(Value: boolean);
    procedure CNHScroll(var Message: TWMHScroll); message CN_HSCROLL;
    procedure CNVScroll(var Message: TWMVScroll); message CN_VSCROLL;
    procedure CNCtlColorScrollBar(var Message: TMessage); message CN_CTLCOLORSCROLLBAR;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure Change; dynamic;
    procedure Scroll(ScrollCode: TScrollCode; var ScrollPos: integer); dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetParams(APosition, AMin, AMax: integer);
  published
    property Align;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Kind: TScrollBarKind read fKind write SetKind default sbHorizontal;
    property LargeChange: TScrollBarInc read fLargeChange write fLargeChange default 1;
    property Max: integer read fMax write SetMax default 100;
    property Min: integer read fMin write SetMin default 0;
    property PageSize: integer read fPageSize write SetPageSize;
    property ParentBiDiMode;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property Position: integer read GetPosition write SetPosition default 0;
    property ReverseScrolling: boolean read fReverseScrolling write SetReverseScrolling default false;
    property ShowHint;
    property SmallChange: TScrollBarInc read fSmallChange write fSmallChange default 1;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnContextPopup;
    property OnChange: TNotifyEvent read fOnChange write fOnChange;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnScroll: TScrollEvent read fOnScroll write fOnScroll;
    property OnStartDock;
    property OnStartDrag;
  end;

  TMyCustomEdit = class(TCustomEdit)
  private
    fAlignment: TAlignment;
    procedure SetAlignment(Value: TAlignment);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Alignment: TAlignment read  fAlignment write SetAlignment default taRightJustify;
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TCharValidation = (cvNone, cvEnabled, cvDisabled);
  TRangeValidation = (rvNone, rvOnEnter, rvOnChange, rvOnExit);

  TCustomTypeEdit = class(TMyCustomEdit)
  private
    fAutoLimit: boolean;
    fCharValidation: TCharValidation;
    {fRangeValidation: TRangeValidation;}
    fOnWrongChar,
    fOnDataTypeError,
    fOnRangeValidationError: TNotifyEvent;
    procedure ConvertValueToText; virtual; abstract;
    function IsValidChar(var Key: char): boolean; virtual; abstract;
    function TextSuccessfulConverted: boolean; virtual; abstract;
    procedure ValidateRanges; virtual; abstract;
    procedure SetValueToValidRange; virtual; abstract;
    procedure SetAutoLimit(Value: boolean);
    procedure CMChanged(var Message: TMessage); message CM_CHANGED;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    fRangeValidation: TRangeValidation;
    property AutoLimit: boolean read fAutoLimit write SetAutoLimit default false;
    property CharValidation: TCharValidation read fCharValidation write fCharValidation default cvNone;
    property RangeValidation: TRangeValidation read fRangeValidation write fRangeValidation default rvNone;
    procedure KeyPress(var Key: char); override;
    property OnWrongChar: TNotifyEvent read fOnWrongChar write fOnWrongChar;
    property OnDataTypeError: TNotifyEvent read fOnDataTypeError write fOnDataTypeError;
    property OnRangeValidationError: TNotifyEvent read fOnRangeValidationError write fOnRangeValidationError;
  public
    procedure ClearEdit;
  published
  end;

  TLCStyle = (lcHorizontal,lcVertical);
  TLCDimensions = record
    LCWidth,LCHeight,LabelWidth,LabelHeight: integer;
    end;

  TUnitPosition = (upLeft,upRight);

  TCustomLabelControl = class(TWinControl)
  private
    fLabel,
    fUnit: TStaticText;
    {fAutoSize,}
    fNoLCStyleInfo: boolean;
    fLCStyle: TLCStyle;
    fLCDimensions: TLCDimensions;
    fUnitWidth: integer;
    fLabelCaption,
    fUnitText: string;
    fLabelAlignment: TAlignment;
    fUnitPosition: TUnitPosition;
    {procedure SetAutoSize(Value: boolean); override;}
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure SetLCStyle(Value: TLCStyle);
    procedure SetLabelAlignment(Value: TAlignment);
    procedure SetLabelCaption (Value: string);
    function  GetLabelFont: TFont;
    procedure SetLabelFont (Value: TFont);
    procedure SetLabelHeight(Value: integer);
    procedure SetLabelWidth(Value: integer);
    procedure SetUnitPosition(Value: TUnitPosition);
    procedure SetUnitText (Value: string);
    procedure SetUnitWidth(Value: integer);
    procedure SetAutoHeight(NonAutoValue: integer); virtual; abstract;
    procedure AdjustHeight;
    procedure SwapLCWidthHeight; virtual; abstract;
    procedure AdjustLCWidthHeight;
    procedure AssortHorizontal; virtual;
    procedure AssortVertical; virtual;
    procedure AssortComponents;
  protected
    fAutoSize: boolean;
    procedure SetAutoSize(Value: boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Free;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: integer); override;
    procedure Invalidate; override;
  published
    property Anchors;
    property AutoSize: boolean read fAutoSize write SetAutoSize default false;
    property BiDiMode;
    {property BorderStyle;}
    property Constraints;
    property LCStyle: TLCStyle read fLCStyle write SetLCStyle;
    property LabelAlignment: TAlignment read fLabelAlignment write SetLabelAlignment default taLeftJustify;
    property LabelCaption: string read fLabelCaption write SetLabelCaption;
    property LabelFont: TFont read GetLabelFont write SetLabelFont;
    property LabelHeight: integer read fLCDimensions.LabelHeight write SetLabelHeight default CDefaultLCHeight-2*CHeightMargin;
    property LabelWidth: integer read fLCDimensions.LabelWidth write SetLabelWidth default CDefaultLCWidth DIV 2;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property UnitText: string read fUnitText write SetUnitText;
    property UnitWidth: integer read fUnitWidth write SetUnitWidth default 0;
    property UnitPosition: TUnitPosition read fUnitPosition write SetUnitPosition default upLeft;
    property Visible;
  end;

  TCustomLabelEdit = class(TCustomLabelControl)
  private
    fEdit: TMyCustomEdit;
    fEditString: string;
    fEditAlignment: TAlignment;
    fCharCase: TEditCharCase;
    fEditWidth,
    fEditHeight,
    fMaxEditLength: integer;
    fPasswordChar: char;
    fAutoSelect,
    fHideSelection,
    fDisableEdition,
    fOEMConvert,
    fReadOnly: boolean;
    fOnEnter,
    fOnChange,
    fOnExit: TNotifyEvent;
    fOnKeyPress: TKeyPressEvent;
    fOnKeyDown,
    fOnKeyUp: TKeyEvent;
    procedure InsertEditField; virtual; abstract;
    procedure SetAutoSelect(Value: boolean);
    procedure SetEditAlignment(Value: TAlignment);
    function  GetEditColor: TColor;
    procedure SetEditColor (Value: TColor);
    function  GetEditFont: TFont;
    procedure SetEditFont (Value: TFont);
    procedure SetMaxEditLength(Value: integer);
    function GetEditText: string;
    procedure SetEditText(Value: string);
    procedure SetHideSelection(Value: boolean);
    procedure SetDisableEdition(Value: boolean);
    procedure SetOEMConvert(Value: boolean);
    procedure SetPasswordChar(Value: char);
    procedure SetReadOnly(Value: boolean);
    procedure SetAutoHeight(NonAutoValue: integer); override;
    procedure SwapLCWidthHeight; override;
    procedure AssortHorizontal; override;
    procedure AssortVertical; override;

    procedure EditChanged(Sender: TObject);
    procedure EditEnter(Sender: TObject);
    procedure EditExit(Sender: TObject); virtual;
    procedure EditKeyDown (Sender: TObject; var Key: word; Shift: TShiftState);
    procedure EditKeyPressed(Sender: TObject; var Key: char);
    procedure EditKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);
  protected

    property EditText: string read GetEditText write SetEditText;
    property CharCase: TEditCharCase read fCharCase write fCharCase default ecNormal;
    property PasswordChar: char read fPasswordChar write SetPasswordChar default #0;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Free;
    procedure SelectAll;
  published
    property AutoSelect: boolean read fAutoSelect write SetAutoSelect default true;
    property AutoSize: boolean read fAutoSize write SetAutoSize default false;
    {property BorderStyle;}
    property Color;
    property Ctl3D;

    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property EditColor: TColor read GetEditColor write SetEditColor;
    property EditAlignment: TAlignment read fEditAlignment write SetEditAlignment
             default taLeftJustify;
    property EditFont: TFont read GetEditFont write SetEditFont;
    property HideSelection: boolean read fHideSelection write SetHideSelection default true;
    property DisableEdition: boolean read fDisableEdition write SetDisableEdition default false;
    property ImeMode;
    property ImeName;
    property MaxEditLength: integer read fMaxEditLength write SetMaxEditLength default 0;
    property OEMConvert: boolean read fOEMConvert write SetOEMConvert default false;
    property ParentBiDiMode;
    property ParentCtl3D;
    property PopupMenu;
    property ReadOnly: boolean read fReadOnly write SetReadOnly default false;
    property TabOrder;
    property TabStop;

    property OnChange: TNotifyEvent read fOnChange write fOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter: TNotifyEvent read fOnEnter write fOnEnter;
    property OnExit: TNotifyEvent read fOnExit write fOnExit;
    property OnKeyDown: TKeyEvent read fOnKeyDown write fOnKeyDown;
    property OnKeyPress: TKeyPressEvent read fOnKeyPress write fOnKeyPress;
    property OnKeyUp: TKeyEvent read fOnKeyUp write fOnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TCustomLabelTypeEdit = class(TCustomLabelEdit)
  private
    fAutoLimit: boolean;
    fCharValidation: TCharValidation;
    {fRangeValidation: TRangeValidation;}
    fOnWrongChar,
    fOnDataTypeError,
    fOnRangeValidationError: TNotifyEvent;
    procedure SetAutoLimit(Value: boolean);
    procedure SetCharValidation(Value: TCharValidation);
    procedure SetRangeValidation(Value: TRangeValidation);
  protected
    fRangeValidation: TRangeValidation;
    property AutoLimit: boolean read fAutoLimit write SetAutoLimit default false;
    property CharValidation: TCharValidation read fCharValidation write SetCharValidation default cvNone;
    property RangeValidation: TRangeValidation read fRangeValidation write SetRangeValidation default rvNone;
    property OnWrongChar: TNotifyEvent read fOnWrongChar write fOnWrongChar;
    property OnDataTypeError: TNotifyEvent read fOnDataTypeError write fOnDataTypeError;
    property OnRangeValidationError: TNotifyEvent read fOnRangeValidationError write fOnRangeValidationError;
  public
    constructor Create(AOwner: TComponent); override;
    procedure ClearEdit;
  published
  end;

  TMyEdit = class(TMyCustomEdit)
  private
  protected
  public
  published
    property CharCase;
    property PasswordChar;
    property Text;
  end;

  TCaseTypeValidation = (ctvNone, ctvChangeToLower, ctvChangeToUpper, ctvLowerOnly, ctvUpperOnly);

  TStringEdit = class(TCustomTypeEdit)
  private
    fControlChars: string;
    fCaseTypeValidation: TCaseTypeValidation;
    procedure ConvertValueToText;  override;
    function IsValidChar(var Key: char): boolean; override;
    function TextSuccessfulConverted: boolean;  override;
    procedure ValidateRanges; override;
    procedure SetValueToValidRange; override;
    procedure SetCaseTypeValidation(Value: TCaseTypeValidation);
  protected
  public
    constructor Create(AOwner: TComponent); override;
  published
    property CaseTypeValidation: TCaseTypeValidation read fCaseTypeValidation write SetCaseTypeValidation default ctvNone;
    property CharValidation;
    property ControlChars: string read fControlChars write fControlChars;
    property PasswordChar;
    property Text;
    property OnWrongChar;
  end;

  TIntegerFormat = (ifDecimal,ifHex);

  TIntegerEdit = class(TCustomTypeEdit)
  private
    fValue,
    fMin,
    fMax: integer;
    fIntegerFormat: TIntegerFormat;
    procedure ConvertValueToText;  override;
    function IsValidChar(var Key: char): boolean; override;
    function TextSuccessfulConverted: boolean;  override;
    procedure ValidateRanges; override;
    procedure SetMin(Value: integer);
    procedure SetMax(Value: integer);
    procedure SetIntegerFormat(Value: TIntegerFormat);
    procedure SetValue(Value: integer);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    {procedure Invalidate; override;}
    function GetFormatedText(Value: integer): string;
    procedure SetValueToValidRange; override;
  published
    property AutoLimit;
    property IntegerFormat: TIntegerFormat read fIntegerFormat write SetIntegerFormat default ifDecimal;
    property Min: integer read fMin write SetMin default 0;
    property Max: integer read fMax write SetMax default 100;
    property Value: integer read fValue write SetValue default 0;
    property RangeValidation;
    property OnWrongChar;
    property OnDataTypeError;
    property OnRangeValidationError;
  end;

  TRealFormat = (rfNormal,rfScientific);

  TRealEdit = class(TCustomTypeEdit)
  private
    fValue,
    fMin,
    fMax: real;
    fRealFormat: TRealFormat;
    fRealPrecision: byte;
    procedure ConvertValueToText;  override;
    function IsValidChar(var Key: char): boolean; override;
    function TextSuccessfulConverted: boolean;  override;
    procedure ValidateRanges; override;
    procedure SetRealFormat(Value: TRealFormat);
    procedure SetRealPrecision(Value: byte);
    procedure SetValue(Value: real);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    {procedure Invalidate; override;}
    function GetFormatedText(Value: real): string;
    procedure SetValueToValidRange; override;
  published
    property AutoLimit;
    property RealFormat: TRealFormat read fRealFormat write SetRealFormat default rfNormal;
    property RealPrecision: byte read fRealPrecision write SetRealPrecision default 3;
    property Min: real read fMin write fMin;
    property Max: real read fMax write fMax;
    property Value: real read fValue write SetValue;
    property RangeValidation;
    property OnWrongChar;
    property OnDataTypeError;
    property OnRangeValidationError;
  end;

  TDateTimeFormat = (dtfDate,dtfTime);

  TDateTimeEdit = class(TCustomTypeEdit)
  private
    fValue,
    fMin,
    fMax: TDateTime;
    fDateTimeFormat: TDateTimeFormat;
    fValidSeparator: char;
    fFirst,fSecond,fThird: byte;
    procedure ConvertValueToText;  override;
    function IsValidChar(var Key: char): boolean; override;
    function TextSuccessfulConverted: boolean;  override;
    procedure ValidateRanges; override;
    procedure SetDateTimeFormat(Value: TDateTimeFormat);
    procedure SetValue(Value: TDateTime);
    procedure SetRangeValidation(Value: TRangeValidation);
    procedure CMChanged(var Message: TMessage); message CM_CHANGED;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    procedure EvaluateDateTimeFormat;
  public
    constructor Create(AOwner: TComponent); override;
    {procedure Invalidate; override;}
    function GetFormatedText(Value: TDateTime): string;
    procedure SetValueToValidRange; override;
  published
    property AutoLimit;
    property DateTimeFormat: TDateTimeFormat read fDateTimeFormat write SetDateTimeFormat default dtfDate;
    property Min: TDateTime read fMin write fMin;
    property Max: TDateTime read fMax write fMax;
    property Value: TDateTime read fValue write SetValue;
    property RangeValidation: TRangeValidation read fRangeValidation write SetRangeValidation default rvNone;
    property OnWrongChar;
    property OnDataTypeError;
    property OnRangeValidationError;
  end;

  TTypeFormat = (tfString,tfDecimal,tfHex,tfFloat,tfScientific,tfDate,tfTime);

  TDisplay = class(TWinControl)
  private
    fDisplay,
    fBackground: TStaticText;
    fAlignment: TAlignment;
    {fAutoSize: boolean;}
    fDisplayString: string;
    fDisplayFormat: TTypeFormat;
    fDisplayHeight: word;
    fHexPositions,
    fRealPrecision: byte;
    procedure AdjustHeight;
    procedure SetAlignment(Value: TAlignment);
    {procedure SetAutoSize(Value: boolean);}
    function  GetColor: TColor;
    procedure SetColor (Value: TColor);
    function  GetFont: TFont;
    procedure SetFont (Value: TFont);
    procedure SetDisplayFormat(Value: TTypeFormat);
    procedure SetHexPositions(Value: byte);
    procedure SetRealPrecision(Value: byte);
    procedure Set_Display (Value: string);
  protected
    fAutoSize: boolean;
    procedure SetAutoSize(Value: boolean); override;
    property Caption;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Free;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: integer); override;
    procedure Invalidate; override;
    procedure Reset;

    function SetDisplay(Value: string): boolean; overload;
    function SetDisplay(Value: integer): boolean; overload;
    function SetDisplay(Value: extended): boolean; overload;
  published
    property Alignment: TAlignment read  fAlignment write SetAlignment default taRightJustify;
    property Anchors;
    property AutoSize: boolean read fAutoSize write SetAutoSize default false;
    property BiDiMode;
    {property BorderStyle;}
    property Constraints;
    property Color: TColor read GetColor write SetColor;
    property Display: string read fDisplayString write Set_Display;
    property DisplayFormat: TTypeFormat read fDisplayFormat write SetDisplayFormat default tfString;
    property Font: TFont read GetFont write SetFont;
    property HexPositions: byte read fHexPositions write SetHexPositions default 0;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property RealPrecision: byte read fRealPrecision write SetRealPrecision default 3;
    property ShowHint;
    property Visible;
  end;

  TLabelComponent = class(TCustomLabelControl)
  private
    fDefaultWidth,
    fDefaultHeight: integer;
    procedure SetAutoHeight(NonAutoValue: integer); override;
    procedure SwapLCWidthHeight; override;
    procedure AssortHorizontal; override;
    procedure AssortVertical; override;
  protected
  public
  published
    property Color;
  end;

  TLabelDisplay = class(TCustomLabelControl)
  private
    fDisplay,
    fBackground: TStaticText;
    fDisplayString: string;
    fDisplayAlignment: TAlignment;
    fDisplayFormat: TTypeFormat;
    fDisplayWidth,
    fDisplayHeight: integer;
    fHexPositions,
    fRealPrecision: byte;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    function  GetDisplayColor: TColor;
    procedure SetDisplayColor (Value: TColor);
    function  GetDisplayFont: TFont;
    procedure SetDisplayFont (Value: TFont);
    procedure SetDisplayAlignment(Value: TAlignment);
    procedure SetDisplayFormat(Value: TTypeFormat);
    procedure SetHexPositions(Value: byte);
    procedure SetRealPrecision(Value: byte);
    procedure Set_Display (Value: string);
    procedure SetAutoHeight(NonAutoValue: integer); override;
    procedure SwapLCWidthHeight; override;
    procedure AssortHorizontal; override;
    procedure AssortVertical; override;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Free;
    procedure Invalidate; override;
    procedure Reset;

    function SetDisplay(Value: string): boolean; overload;
    function SetDisplay(Value: integer): boolean; overload;
    function SetDisplay(Value: extended): boolean; overload;
  published
    property Color;
    property Display: string read fDisplayString write Set_Display;
    property DisplayAlignment: TAlignment read fDisplayAlignment write SetDisplayAlignment
             default taLeftJustify;
    property DisplayColor: TColor read GetDisplayColor write SetDisplayColor;
    property DisplayFont: TFont read GetDisplayFont write SetDisplayFont;
    property DisplayFormat: TTypeFormat read fDisplayFormat write SetDisplayFormat default tfString;
    property HexPositions: byte read fHexPositions write SetHexPositions default 0;
    property RealPrecision: byte read fRealPrecision write SetRealPrecision default 3;
  end;

  TLabelEdit = class(TCustomLabelEdit)
  private
    procedure InsertEditField; override;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Invalidate; override;
  published
    property CharCase;
    property PasswordChar;
    property EditText;
  end;

  TLabelStringEdit = class(TCustomLabelTypeEdit)
  private
    fControlChars: string;
    fCaseTypeValidation: TCaseTypeValidation;
    procedure InsertEditField; override;
    procedure SetCaseTypeValidation(Value: TCaseTypeValidation);
    procedure SetControlChars(Value: string);
    procedure EditWrongChar(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Invalidate; override;
  published
    property CaseTypeValidation: TCaseTypeValidation read fCaseTypeValidation write SetCaseTypeValidation default ctvNone;
    property CharValidation;
    property ControlChars: string read fControlChars write SetControlChars;
    property PasswordChar;
    property EditText;
    property OnWrongChar;
  end;

  TLabelIntegerEdit = class(TCustomLabelTypeEdit)
  private
    fMin,
    fMax: integer;
    fIntegerFormat: TIntegerFormat;
    fRangeErrorMessage: string;
    procedure InsertEditField; override;
    procedure SetMin(Value: integer); virtual;
    procedure SetMax(Value: integer); virtual;
    procedure SetIntegerFormat(Value: TIntegerFormat);
    function GetValue: integer;
    procedure SetValue(Value: integer); virtual;
    procedure SetRangeValidation(Value: TRangeValidation);
    procedure EditDataTypeError(Sender: TObject);
    procedure EditRangeValidationError(Sender: TObject); virtual;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Invalidate; override;
    function GetFormatedText(Value: integer): string;
    procedure SetValueToValidRange;
    property RangeErrorMessage: string write fRangeErrorMessage;
  published
    property AutoLimit;
    property IntegerFormat: TIntegerFormat read fIntegerFormat write SetIntegerFormat default ifDecimal;
    property Min: integer read fMin write SetMin default 0;
    property Max: integer read fMax write SetMax default 100;
    property Value: integer read GetValue write SetValue default 0;
    property RangeValidation: TRangeValidation read fRangeValidation write SetRangeValidation default rvNone;
    property OnWrongChar;
    property OnDataTypeError;
    property OnRangeValidationError;
  end;

  TLabelRealEdit = class(TCustomLabelTypeEdit)
  private
    fMin,
    fMax: real;
    fRealFormat: TRealFormat;
    fRealPrecision: byte;
    fRangeErrorMessage: string;
    procedure InsertEditField; override;
    procedure SetMin(Value: real); virtual;
    procedure SetMax(Value: real); virtual;
    function GetValue: real;
    procedure SetValue(Value: real); virtual;
    procedure SetRangeValidation(Value: TRangeValidation);
    procedure SetRealFormat(Value: TRealFormat);
    procedure SetRealPrecision(Value: byte); virtual;
    procedure EditDataTypeError(Sender: TObject);
    procedure EditRangeValidationError(Sender: TObject); virtual;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Invalidate; override;
    function GetFormatedText(Value: real): string;
    procedure SetValueToValidRange;
    property RangeErrorMessage: string write fRangeErrorMessage;
  published
    property AutoLimit;
    property RealFormat: TRealFormat read fRealFormat write SetRealFormat default rfNormal;
    property RealPrecision: byte read fRealPrecision write SetRealPrecision default 3;
    property Min: real read fMin write SetMin;
    property Max: real read fMax write SetMax;
    property Value: real read GetValue write SetValue;
    property RangeValidation: TRangeValidation read fRangeValidation write SetRangeValidation default rvNone;
    property OnWrongChar;
    property OnDataTypeError;
    property OnRangeValidationError;
  end;


  TLabelDateTimeEdit = class(TCustomLabelTypeEdit)
  private
    fMin,
    fMax: TDateTime;
    fDateTimeFormat: TDateTimeFormat;
    fRangeErrorMessage: string;
    procedure InsertEditField; override;
    procedure SetDateTimeFormat(Value: TDateTimeFormat);
    procedure SetMin(Value: TDateTime);
    procedure SetMax(Value: TDateTime);
    function GetValue: TDateTime;
    procedure SetValue(Value: TDateTime);
    procedure SetRangeValidation(Value: TRangeValidation);
    procedure EditDataTypeError(Sender: TObject);
    procedure EditRangeValidationError(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Invalidate; override;
    function GetFormatedText(Value: TDateTime): string;
    procedure SetValueToValidRange;
    property RangeErrorMessage: string write fRangeErrorMessage;
  published
    property AutoLimit;
    property DateTimeFormat: TDateTimeFormat read fDateTimeFormat write SetDateTimeFormat default dtfDate;
    property Min: TDateTime read fMin write SetMin;
    property Max: TDateTime read fMax write SetMax;
    property Value: TDateTime read GetValue write SetValue;
    property RangeValidation: TRangeValidation read fRangeValidation write SetRangeValidation default rvNone;
    property OnWrongChar;
    property OnDataTypeError;
    property OnRangeValidationError;
  end;

  TLabelScrollIntegerEdit = class(TLabelIntegerEdit)
  private
    fScrollBar: TScrollBar;
    fScrollBarLength: integer;
    fArrangingScrollBarPosition,
    fReverseScrolling: boolean;
    fOnEndScroll: TNotifyEvent;
    fOnScroll: TScrollEvent;
    procedure ArrangeAcrollBarPosition;
    procedure ArrangeAllScrollBarValues;
    procedure SetMin(Value: integer); override;
    procedure SetMax(Value: integer); override;
    procedure SetValue(Value: integer); override;
    procedure SetReverseScrolling(Value: boolean);
    procedure SetAutoHeight(NonAutoValue: integer); override;
    procedure SwapLCWidthHeight; override;
    procedure AssortHorizontal; override;
    procedure AssortVertical; override;
    procedure EditExit(Sender: TObject); override;
    procedure EditRangeValidationError(Sender: TObject); override;
    procedure EditScrollBarChanged(Sender: TObject);
    procedure EditScrollBarEndScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: integer);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Free;
  published
    property ReverseScrolling: boolean read fReverseScrolling write SetReverseScrolling;
    property OnEndScroll: TNotifyEvent read fOnEndScroll write fOnEndScroll;
    property OnScroll: TScrollEvent read fOnScroll write fOnScroll;
  end;

  // TScrollType = (stLinear,stLogarithmic);

  TLabelScrollRealEdit = class(TLabelRealEdit)
  private
    fScrollBar: TScrollBar;
    fScrollBarLength: integer;
    fArrangingScrollBarPosition,
    fReverseScrolling: boolean;
    fScrollMultiplier: integer;
    fOnEndScroll: TNotifyEvent;
    fOnScroll: TScrollEvent;
    procedure ArrangeAcrollBarPosition;
    procedure ArrangeAllScrollBarValues;
    procedure SetMin(Value: real); override;
    procedure SetMax(Value: real); override;
    procedure SetValue(Value: real); override;
    procedure SetReverseScrolling(Value: boolean);
    procedure SetAutoHeight(NonAutoValue: integer); override;
    procedure SwapLCWidthHeight; override;
    procedure SetRealPrecision(Value: byte); override;
    procedure AssortHorizontal; override;
    procedure AssortVertical; override;
    procedure EditExit(Sender: TObject); override;
    procedure EditRangeValidationError(Sender: TObject); override;
    procedure EditScrollBarChanged(Sender: TObject);
    procedure EditScrollBarEndScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: integer);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Free;
  published
    property ReverseScrolling: boolean read fReverseScrolling write SetReverseScrolling;
    property OnEndScroll: TNotifyEvent read fOnEndScroll write fOnEndScroll;
    property OnScroll: TScrollEvent read fOnScroll write fOnScroll;
  end;

  TCustomLabelComboBox = class(TCustomLabelControl)
  private
    fComboBox: TComboBox;
    {fEditString: string;
    fComboBoxAlignment: TAlignment;
    fCharCase: TEditCharCase;}
    fComboBoxWidth,
    fComboBoxHeight{,
    fMaxEditLength}: integer;
    {fPasswordChar: char;
    fAutoSelect,
    fHideSelection,
    fOEMConvert,
    fReadOnly: boolean;}
    fOnEnter,
    fOnChange,
    fOnExit: TNotifyEvent;
    fOnKeyPress: TKeyPressEvent;
    fOnKeyDown,
    fOnKeyUp: TKeyEvent;
    procedure InsertComboBox; virtual; abstract;
    {procedure SetAutoSelect(Value: boolean);
    procedure SetEditAlignment(Value: TAlignment);
    function  GetEditColor: TColor;
    procedure SetEditColor (Value: TColor);
    function  GetEditFont: TFont;
    procedure SetEditFont (Value: TFont);
    procedure SetMaxEditLength(Value: integer);
    function GetEditText: string;
    procedure SetEditText(Value: string);
    procedure SetHideSelection(Value: boolean);
    procedure SetOEMConvert(Value: boolean);
    procedure SetPasswordChar(Value: char);
    procedure SetReadOnly(Value: boolean);}
    procedure SetAutoHeight(NonAutoValue: integer); override;
    procedure SwapLCWidthHeight; override;
    procedure AssortHorizontal; override;
    procedure AssortVertical; override;

    {procedure EditChanged(Sender: TObject);
    procedure EditEnter(Sender: TObject);
    procedure EditExit(Sender: TObject); virtual;
    procedure EditKeyDown (Sender: TObject; var Key: word; Shift: TShiftState);
    procedure EditKeyPressed(Sender: TObject; var Key: char);
    procedure EditKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);}
  protected
    {property EditText: string read GetEditText write SetEditText;
    property CharCase: TEditCharCase read fCharCase write fCharCase default ecNormal;
    property PasswordChar: char read fPasswordChar write SetPasswordChar default #0;}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Free;
    {procedure SelectAll;}
  published
    {property AutoSelect: boolean read fAutoSelect write SetAutoSelect default true;
    property AutoSize: boolean read fAutoSize write SetAutoSize default false;}
    {property BorderStyle;}
    {property Color;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property EditColor: TColor read GetEditColor write SetEditColor;
    property EditAlignment: TAlignment read fEditAlignment write SetEditAlignment
             default taLeftJustify;
    property EditFont: TFont read GetEditFont write SetEditFont;
    property HideSelection: boolean read fHideSelection write SetHideSelection default true;
    property ImeMode;
    property ImeName;
    property MaxEditLength: integer read fMaxEditLength write SetMaxEditLength default 0;
    property OEMConvert: boolean read fOEMConvert write SetOEMConvert default false;
    property ParentBiDiMode;
    property ParentCtl3D;
    property PopupMenu;
    property ReadOnly: boolean read fReadOnly write SetReadOnly default false;
    property TabOrder;
    property TabStop;}

    property OnChange: TNotifyEvent read fOnChange write fOnChange;
    {property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;}
    property OnEnter: TNotifyEvent read fOnEnter write fOnEnter;
    property OnExit: TNotifyEvent read fOnExit write fOnExit;
    property OnKeyDown: TKeyEvent read fOnKeyDown write fOnKeyDown;
    property OnKeyPress: TKeyPressEvent read fOnKeyPress write fOnKeyPress;
    property OnKeyUp: TKeyEvent read fOnKeyUp write fOnKeyUp;
    {property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;}
  end;

  TLabelComboBox = class(TCustomLabelComboBox)
  private
    procedure InsertComboBox; override;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure Invalidate; override;
  published
    {property CharCase;
    property PasswordChar;
    property EditText;}
  end;

  procedure Register;

  function GetEditHeightFromFont(Source: TFont; AOwner: TComponent): word;
  function GetStaticTextHeightFromFont(Source: TFont; AOwner: TComponent): word;

  function GetTextWidthInPixel(Source: string; Font: TFont): integer;

implementation

uses SysUtils, Dialogs, Consts, Math, ConversionUtils;

const
  CLCDimensions: TLCDimensions = (LCWidth: CDefaultLCWidth; LCHeight: CDefaultLCHeight; LabelWidth: CDefaultLCWidth DIV 2; LabelHeight: CDefaultLCHeight-2*CHeightMargin);

  CDisplay = 'Display';
  CEdit    = 'Edit';
  CLabel   = 'Label';
  CString  = 'String';

type
  TDateTimeStringArray = array [0..2] of string;
  TDateTimeValueArray = array [0..2] of word;

procedure Register;
begin
  RegisterComponents(RegisterPage, [TMyEdit]);

  RegisterComponents(RegisterPage, [TStringEdit]);
  RegisterComponents(RegisterPage, [TIntegerEdit]);
  RegisterComponents(RegisterPage, [TRealEdit]);
  RegisterComponents(RegisterPage, [TDateTimeEdit]);

  RegisterComponents(RegisterPage, [TDisplay]);

  { TLabelComponent is a dummy, which allow to check
    the method and properties of TCustomLabelComponent }
  {RegisterComponents(RegisterPage, [TLabelComponent]);}

  RegisterComponents(RegisterPage, [TLabelDisplay]);

  RegisterComponents(RegisterPage, [TLabelEdit]);

  RegisterComponents(RegisterPage, [TLabelStringEdit]);
  RegisterComponents(RegisterPage, [TLabelIntegerEdit]);
  RegisterComponents(RegisterPage, [TLabelRealEdit]);
  RegisterComponents(RegisterPage, [TLabelDateTimeEdit]);

  RegisterComponents(RegisterPage, [TLabelScrollIntegerEdit]);
  RegisterComponents(RegisterPage, [TLabelScrollRealEdit]);

  RegisterComponents(RegisterPage, [TLabelComboBox]);

  RegisterComponents(RegisterPage, [TGHScrollBar]);
end;

function        LowCase( ch : Char ) : Char;
{ the is unfortunately no LowCase function in the System unit }
asm
        CMP     AL,'A'
        JB      @@exit
        CMP     AL,'Z'
        JA      @@exit
        SUB     AL,'A' - 'a'
@@exit:
end;

function        IsLowCase( ch : Char ) : boolean;
asm
        CMP     AL,'a'
        JB      @@NotLowCase
        CMP     AL,'z'
        JA      @@NotLowCase
@@LowCase:
        MOV     AL,1
        JMP     @@exit
@@NotLowCase:
        MOV     AL,0
@@exit:
end;

function        IsUpCase( ch : Char ) : boolean;
asm
        CMP     AL,'A'
        JB      @@NotUpCase
        CMP     AL,'Z'
        JA      @@NotUpCase
@@UpCase:
        MOV     AL,1
        JMP     @@exit
@@NotUpCase:
        MOV     AL,0
@@exit:
end;

{procedure ShareDateTimeString (Source: string; Separator: char; var Target: TDateTimeStringArray);
var SeparatorPosition: byte;
begin
  SeparatorPosition:=System.Pos(Separator,Source);
  if SeparatorPosition=0 then Target[0]:=Source
  else
     begin
     Target[0]:=System.Copy(Source,1,SeparatorPosition-1);
     System.Delete(Source,1,SeparatorPosition);
     SeparatorPosition:=System.Pos(Separator,Source);
     if SeparatorPosition=0 then Target[1]:=Source
     else
        begin
        Target[1]:=System.Copy(Source,1,SeparatorPosition-1);
        System.Delete(Source,1,SeparatorPosition);
        Target[2]:=Source;
        end;
     end;
end;}

{procedure DateTimeStringArrayToValue(Source: TDateTimeStringArray; var Target: TDateTimeValueArray);
var I: byte;
begin
  for I:=0 to 2 do if Source[I]<>'' then Target[I]:=StrToInt(Source[I]) else Target[I]:=0;
end;}

function GetObjectNumber(Source: string): word;
begin
  while (Source<>'') and not(Source[1] in ['0'..'9']) do System.Delete(Source,1,1);
  if (Source<>'') and IsDecimalIntegerString(Source)
     then Result:=StrToInt(Source)
  else Result:=0;
end;

function GetEditHeightFromFont(Source: TFont; AOwner: TComponent): word;
var GEdit: TEdit;
begin
  GEdit:=TEdit.Create(AOwner);
  with GEdit do
       begin
       Font:=Source;
       Text:='Aj';
       Result:=Height+2;
       Free;
       end;
end;

function GetStaticTextHeightFromFont(Source: TFont; AOwner: TComponent): word;
var GLabel: TStaticText;
begin
  GLabel:=TStaticText.Create(AOwner);
  with GLabel do
       begin
       Font:=Source;
       Caption:='Aj';
       Result:=Height;
       Free;
       end;
end;

function GetTextWidthInPixel(Source: string; Font: TFont): integer;
var // Canvas: TCanvas;
    BM: TBitmap;
begin
  Result:=0;

  // Canvas := TCanvas.Create;
  BM:=TBitmap.Create;
  try
    BM.Canvas.Font := Font;
    Result:=BM.Canvas.TextWidth(Source);
  finally
    BM.Canvas.Free;
  end;
end;

    { custom components - not registered }

    { TGHScrollBar }

constructor TGHScrollBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 121;
  Height := GetSystemMetrics(SM_CYHSCROLL);
  TabStop := True;
  ControlStyle := [csFramed, csDoubleClicks, csOpaque];
  fKind := sbHorizontal;
  fPosition := 0;
  fMin := 0;
  fMax := 100;
  fSmallChange := 1;
  fLargeChange := 1;
  fReverseScrolling := false;
  if SysLocale.FarEast and (Win32Platform = VER_PLATFORM_WIN32_NT) then
    ImeMode := imDisable;
end;

procedure TGHScrollBar.CreateParams(var Params: TCreateParams);
const
  Kinds: array[TScrollBarKind] of DWORD = (SBS_HORZ, SBS_VERT);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'SCROLLBAR');
  Params.Style := Params.Style or Kinds[fKind];
  if fKind = sbVertical then
    if not UseRightToLeftAlignment then
      Params.Style := Params.Style or SBS_RIGHTALIGN
    else
      Params.Style := Params.Style or SBS_LEFTALIGN;
  if NotRightToLeft then
    fRTLFactor := 1
  else
    fRTLFactor := -1;
end;

procedure TGHScrollBar.CreateWnd;
var
  ScrollInfo: TScrollInfo;
  LBounds: TRect;
begin
  //  Windows' does not always create the window size we ask for, so we have
  //  insist sometimes.  Setting BoundsRect will only adjust the size if needed.
  LBounds := BoundsRect;
  inherited CreateWnd;
  BoundsRect := LBounds;

  SetScrollRange(Handle, SB_CTL, fMin, fMax, False);
  ScrollInfo.cbSize := SizeOf(ScrollInfo);
  ScrollInfo.nPage := fPageSize;
  ScrollInfo.fMask := SIF_PAGE;
  SetScrollInfo(Handle, SB_CTL, ScrollInfo, False);
  if NotRightToLeft then
    SetScrollPos(Handle, SB_CTL, fPosition, True)
  else
    SetScrollPos(Handle, SB_CTL, fMin + fMax - fPosition, True);  // "fMin + " inserted to remove the vcl bug
end;

function TGHScrollBar.NotRightToLeft: Boolean;
begin
  Result := (not IsRightToLeft) or (fKind = sbVertical);
end;

procedure TGHScrollBar.SetKind(Value: TScrollBarKind);
begin
  if fKind <> Value then
     begin
     fKind := Value;
     if not (csLoading in ComponentState) then
        SetBounds(Left, Top, Height, Width);
     RecreateWnd;
     end;
end;

procedure TGHScrollBar.SetParams(APosition, AMin, AMax: integer);
begin
  if not (csLoading in ComponentState) and              // condition inserted to prevent spurious exception for negative limits
     ((AMax < AMin) or ((AMax-AMin) < FPageSize)) then  // "-AMin" inserted to remove vcl bug
     raise EInvalidOperation.Create(SScrollBarRange);
  if APosition < AMin then APosition := AMin;
  if APosition > AMax then APosition := AMax;
  if (fMin <> AMin) or (fMax <> AMax) then
     begin
     fMin := AMin;
     fMax := AMax;
     if HandleAllocated then
        SetScrollRange(Handle, SB_CTL, AMin, AMax, fPosition = APosition);
     end;
  if fPosition <> APosition then
     begin
     fPosition := APosition;
     if HandleAllocated then
        if NotRightToLeft then
           SetScrollPos(Handle, SB_CTL, fPosition, True)
        else
           SetScrollPos(Handle, SB_CTL, fMin + fMax - fPosition, True);  // "fMin + " inserted to remove vcl bug
    Enabled := True;
    Change;
    end;
end;

function TGHScrollBar.GetPosition: integer;
begin
  // inserted function to allow reverse scrolling behaviour
  if not(fReverseScrolling) then
     Result:=fPosition
  else Result:=fMin + fMax - fPosition;
end;

procedure TGHScrollBar.SetPosition(Value: integer);
begin
  if not(fPreventScrollJitter) then
     begin
     fPreventScrollJitter:=true;
     if not(fReverseScrolling) then
        SetParams(Value, fMin, fMax)
     else SetParams(fMin + fMax - Value, fMin, fMax);
     // SetParams(Value, fMin, fMax);
     fPreventScrollJitter:=false;
     end;
end;

procedure TGHScrollBar.SetPageSize(Value: integer);
var
  ScrollInfo: TScrollInfo;
begin
  if (fPageSize = Value) or (Value > (fMax-fMin)) then exit; // "fMax" replaced with "(fMax-fMin)" to remove vcl bug
  fPageSize := Value;
  ScrollInfo.cbSize := SizeOf(ScrollInfo);
  ScrollInfo.nPage := Value;
  ScrollInfo.fMask := SIF_PAGE;
  if HandleAllocated then
    SetScrollInfo(Handle, SB_CTL, ScrollInfo, True);
end;

procedure TGHScrollBar.SetReverseScrolling(Value: boolean);
begin
  // inserted function to allow reverse scrolling behaviour
  if (fReverseScrolling<>Value) then
     begin
     fReverseScrolling:=Value;
     Position:=fMin + fMax - fPosition;
     end;
end;

procedure TGHScrollBar.SetMin(Value: integer);
begin
  SetParams(fPosition, Value, fMax);
end;

procedure TGHScrollBar.SetMax(Value: integer);
begin
  SetParams(fPosition, fMin, Value);
end;

procedure TGHScrollBar.Change;
begin
  inherited Changed;
  if Assigned(fOnChange) then fOnChange(Self);
end;

procedure TGHScrollBar.Scroll(ScrollCode: TScrollCode; var ScrollPos: integer);
begin
  if Assigned(fOnScroll) then fOnScroll(Self, ScrollCode, ScrollPos);
end;

procedure TGHScrollBar.DoScroll(var Message: TWMScroll);
var
  ScrollPos: integer;
  NewPos: Longint;
  ScrollInfo: TScrollInfo;
begin
  with Message do
  begin
    NewPos := fPosition;
    case TScrollCode(ScrollCode) of
      scLineUp:
        Dec(NewPos, fSmallChange * fRTLFactor);
      scLineDown:
        Inc(NewPos, fSmallChange * fRTLFactor);
      scPageUp:
        Dec(NewPos, fLargeChange * fRTLFactor);
      scPageDown:
        Inc(NewPos, fLargeChange * fRTLFactor);
      scPosition, scTrack:
        with ScrollInfo do
        begin
          cbSize := SizeOf(ScrollInfo);
          fMask := SIF_ALL;
          GetScrollInfo(Handle, SB_CTL, ScrollInfo);
          NewPos := nTrackPos;
          { We need to reverse the positioning because SetPosition below
            calls SetParams that reverses the position. This acts as a
            double negative. }
          if not NotRightToLeft then NewPos := fMin + fMax - NewPos;   // "fMin + " inserted to remove vcl bug
        end;
      scTop:
        NewPos := fMin;
      scBottom:
        NewPos := fMax;
    end;
    if NewPos < fMin then NewPos := fMin;
    if NewPos > fMax then NewPos := fMax;
    ScrollPos := NewPos;
    Scroll(TScrollCode(ScrollCode), ScrollPos);
    SetPosition(ScrollPos);
  end;
end;

procedure TGHScrollBar.CNHScroll(var Message: TWMHScroll);
begin
  DoScroll(Message);
end;

procedure TGHScrollBar.CNVScroll(var Message: TWMVScroll);
begin
  DoScroll(Message);
end;

procedure TGHScrollBar.CNCtlColorScrollBar(var Message: TMessage);
begin
  with Message do
    CallWindowProc(DefWndProc, Handle, Msg, WParam, LParam);
end;

procedure TGHScrollBar.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  DefaultHandler(Message);
end;
    { TMyCustomEdit }

constructor TMyCustomEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fAlignment:=taRightJustify;
end;

procedure TMyCustomEdit.CreateParams(var Params: TCreateParams);
const Alignments: array[TAlignment] of word = (ES_LEFT, ES_RIGHT, ES_CENTER);
begin
  inherited CreateParams(Params);
  Params.Style:=Params.Style or Alignments[Alignment];
end;

procedure TMyCustomEdit.SetAlignment(Value: TAlignment);
begin
  if fAlignment<>Value then begin
     fAlignment:=Value;
     RecreateWnd;
     end;
end;

    { TCustomTypeEdit }

procedure TCustomTypeEdit.SetAutoLimit(Value: boolean);
begin
  if (fAutoLimit<>Value) then begin
     fAutoLimit:=Value;
     if fAutoLimit then SetValueToValidRange;
     end;
end;

procedure TCustomTypeEdit.CMChanged(var Message: TMessage);
begin
  inherited;
  if not(TextSuccessfulConverted) and (Assigned(fOnDataTypeError)) then
     fOnDataTypeError(Self);
  if fRangeValidation=rvOnChange then ValidateRanges;
end;

procedure TCustomTypeEdit.CMEnter(var Message: TCMEnter);
begin
  ConvertValueToText;
  inherited;
  if fRangeValidation=rvOnEnter then ValidateRanges;
end;

procedure TCustomTypeEdit.CMExit(var Message: TCMExit);
begin
  ConvertValueToText;
  inherited;
  if fRangeValidation=rvOnExit then ValidateRanges;
end;

procedure TCustomTypeEdit.KeyPress(var Key: char);
begin
  if (Key<' ') or IsValidChar(Key) then inherited KeyPress(Key)
  else
     begin
     if Assigned(fOnWrongChar) then fOnWrongChar(Self);
     Key:=#0;
     end;
end;

procedure TCustomTypeEdit.ClearEdit;
begin
  Text:='';
end;

    { TCustomLabelControl }

procedure TCustomLabelControl.SetAutoSize(Value: boolean);
begin
  if fAutoSize<>Value then begin
     fAutoSize:=Value;
     if fAutoSize then begin
        AdjustHeight;
        AssortComponents;
        end;
     end;
end;

procedure TCustomLabelControl.CMEnabledChanged(var Message: TMessage);
begin
  inherited;

  if fLabel<>nil then
     fLabel.Enabled:=Enabled;
  if fUnit<>nil then
     fUnit.Enabled:=Enabled;
end;

procedure TCustomLabelControl.SetLCStyle (Value: TLCStyle);
begin
  if (fLCStyle<>Value) or fNoLCStyleInfo or ((csDesigning in ComponentState) and (fLCStyle=lcVertical)) then begin
     fLCStyle:=Value;
     if {((csDesigning in ComponentState) and (fLCStyle=lcVertical)) or }
        not(fNoLCStyleInfo) then AdjustLCWidthHeight;
     fNoLCStyleInfo:=false;
     if fAutoSize then AdjustHeight;
     AssortComponents;
     end;
end;

procedure TCustomLabelControl.SetLabelAlignment (Value: TAlignment);
begin
  if fLabelAlignment<>Value then begin
     fLabelAlignment:=Value;
     if fLabel<>nil then fLabel.Alignment:=fLabelAlignment;
     end;
end;

procedure TCustomLabelControl.SetLabelCaption (Value: string);
begin
  if fLabelCaption<>Value then begin
     fLabelCaption:=Value;
     if fLabel<>nil then fLabel.Caption:=fLabelCaption;
     end;
end;

function TCustomLabelControl.GetLabelFont: TFont;
begin
  if fLabel<>nil then begin
     if fAutoSize then AdjustHeight;
     AssortComponents;
     Result:=fLabel.Font;
     end
  else Result:=TFont(nil);
end;

procedure TCustomLabelControl.SetLabelFont (Value: TFont);
begin
  if (fLabel<>nil) and (fLabel.Font<>Value) then begin
     fLabel.Font:=Value;
     if (fUnit<>nil) then fUnit.Font:=Value;
     if fAutoSize then AdjustHeight;
     AssortComponents;
     end;
end;

procedure TCustomLabelControl.SetLabelHeight (Value: integer);
begin
  if (fLCDimensions.LabelHeight<>Value) and not(fAutoSize)  then begin
     if Value>fLCDimensions.LCHeight then Value:=fLCDimensions.LCHeight;
     fLCDimensions.LabelHeight:=Value;
     AssortComponents;
     end;
end;

procedure TCustomLabelControl.SetLabelWidth (Value: integer);
begin
  if fLCDimensions.LabelWidth<>Value then begin
     if Value>(fLCDimensions.LCWidth-2*CWidthMargin) then
        Value:=fLCDimensions.LCHeight-2*CWidthMargin;
     fLCDimensions.LabelWidth:=Value;
     AssortComponents;
     end;
end;

procedure TCustomLabelControl.SetUnitPosition(Value: TUnitPosition);
begin
  if fUnitPosition<>Value then begin
     fUnitPosition:=Value;
     AssortComponents;
     end;
end;

procedure TCustomLabelControl.SetUnitText(Value: string);
begin
  if fUnitText<>Value then begin
     if (csDesigning in ComponentState) and (fUnitWidth=0) and (fUnitText='') then SetUnitWidth(40);
     fUnitText:=Value;
     if fUnit<>nil then fUnit.Caption:=fUnitText;
     end;
end;

procedure TCustomLabelControl.SetUnitWidth(Value: integer);
begin
  if fUnitWidth<>Value then begin
     if Value>(fLCDimensions.LCWidth-2*CWidthMargin) then
        Value:=fLCDimensions.LCHeight-2*CWidthMargin;
     fUnitWidth:=Value;
     if (fUnitWidth=0) and (fUnit<>nil) then begin
        fUnit.Free; fUnit:=nil;
        end
     else if (fUnit=nil) then begin
             fUnit:=TStaticText.Create(Self);
             with fUnit do begin
                  Parent:=Self;
                  Alignment:=taCenter;
                  AutoSize:=false;
                  Caption:=fUnitText;
                  Font:=fLabel.Font;
                  if (csDesigning in ComponentState) then Color:=clTeal;
                  end;
             end;
     AssortComponents;
     if fUnit<>nil then fUnit.Invalidate;
     end;
end;

procedure TCustomLabelControl.AdjustHeight;
begin
  if fLabel<>nil then fLCDimensions.LabelHeight:=GetStaticTextHeightFromFont(fLabel.Font,Self);
  SetAutoHeight(Self.Height);
  if (fLCDimensions.LCHeight<>Self.Height) then
     inherited SetBounds(Self.Left,Self.Top,Self.Width,fLCDimensions.LCHeight);
end;

procedure TCustomLabelControl.AdjustLCWidthHeight;
begin
  SwapLCWidthHeight;
  if (fLCDimensions.LCWidth<>Self.Width) or (fLCDimensions.LCHeight<>Self.Height) then
     inherited SetBounds(Self.Left,Self.Top,fLCDimensions.LCWidth,fLCDimensions.LCHeight);
end;

procedure TCustomLabelControl.AssortHorizontal;
begin
  with fLCDimensions do begin
       if fLabel<>nil then begin
          fLabel.Top:=((LCHeight-LabelHeight) DIV 2);
          fLabel.Left:=CWidthMargin;
          fLabel.Height:=LabelHeight;
          fLabel.Width:=LabelWidth;
          end;

       if (fUnit<>nil) then begin
          fUnit.Top:=fLabel.Top;
          if fUnitPosition=upLeft then fUnit.Left:=LabelWidth+2*CWidthMargin
          else fUnit.Left:=LCWidth-fUnitWidth-CWidthMargin;
          fUnit.Height:=LabelHeight;
          fUnit.Width:=fUnitWidth;
          end;
       end;
end;

procedure TCustomLabelControl.AssortVertical;
begin
  with fLCDimensions do begin
       if fLabel<>nil then begin
          fLabel.Top:=CHeightMargin;
          fLabel.Left:=CWidthMargin;
          fLabel.Height:=LabelHeight;
          fLabel.Width:=LabelWidth;
          end;

       if (fUnit<>nil) then begin
          fUnit.Top:=fLabel.Top;
          if fUnitPosition=upLeft then fUnit.Left:=LabelWidth+2*CWidthMargin
          else fUnit.Left:=LCWidth-fUnitWidth-CWidthMargin;
          fUnit.Height:=LabelHeight;
          fUnit.Width:=fUnitWidth;
          end;
       end;
end;

procedure TCustomLabelControl.AssortComponents;
begin
  fLCDimensions.LCWidth:=Self.Width; fLCDimensions.LCHeight:=Self.Height;
  case fLCStyle of
       lcHorizontal: AssortHorizontal;
       lcVertical:   AssortVertical;
      end;
end;

constructor TCustomLabelControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fLabel:=TStaticText.Create(Self);
  with fLabel do begin
       Parent:=Self;
       AutoSize:=false;
       if (csDesigning in ComponentState) then Color:=clTeal;
       end;

  fAutoSize:=false;
  fUnitWidth:=0; fUnitPosition:=upLeft;
  fLCStyle:=lcHorizontal;
  fNoLCStyleInfo:=true;

  fLCDimensions:=CLCDimensions;

  with fLCDimensions do begin
       LabelHeight:=GetStaticTextHeightFromFont(fLabel.Font,Self);
       LCHeight:=LabelHeight+2*CHeightMargin;
       inherited SetBounds(0,0,LCWidth,LCHeight);
       end;

  SetLabelCaption(CLabel);
end;

destructor TCustomLabelControl.Free;
begin
  if fLabel<>nil then fLabel.Free;
  if fUnit<>nil then fUnit.Free;
  inherited Free;
end;

procedure TCustomLabelControl.SetBounds(ALeft, ATop, AWidth, AHeight: integer);
begin
  SetAutoHeight(AHeight);
  inherited SetBounds(ALeft, ATop, AWidth,fLCDimensions.LCHeight);
  AssortComponents;
end;

procedure TCustomLabelControl.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fLabelCaption=CLabel) then
     SetLabelCaption(CLabel+IntToStr(GetObjectNumber(Name)));

  inherited;
end;

    { TCustomLabelEdit }

procedure TCustomLabelEdit.SetAutoSelect(Value: boolean);
begin
  if fAutoSelect<>Value then begin
     fAutoSelect:=Value;
     if fEdit<>nil then fEdit.AutoSelect:=fAutoSelect;
     end;
end;


procedure TCustomLabelEdit.SetEditAlignment(Value: TAlignment);
begin
  if fEditAlignment<>Value then begin
     fEditAlignment:=Value;
     if fEdit<>nil then fEdit.Alignment:=fEditAlignment;
     end;
end;

function TCustomLabelEdit.GetEditColor: TColor;
begin
  if fEdit<>nil then Result:=fEdit.Color else Result:=TColor(0);
end;

procedure TCustomLabelEdit.SetEditColor(Value: TColor);
begin
  if (fEdit<>nil) and (fEdit.Color<>Value) then fEdit.Color:=Value;
end;

function TCustomLabelEdit.GetEditFont: TFont;
begin
  if fEdit<>nil then begin
     if fAutoSize then AdjustHeight;
     AssortComponents;
     Result:=fEdit.Font;
     end
  else Result:=TFont(nil);
end;

procedure TCustomLabelEdit.SetEditFont(Value: TFont);
begin
  if (fEdit<>nil) and (fEdit.Font<>Value) then begin
     fEdit.Font:=Value;
     if fAutoSize then AdjustHeight;
     AssortComponents;
     end;
end;

function TCustomLabelEdit.GetEditText: string;
begin
  if fEdit<>nil then Result:=fEdit.Text else Result:='';
end;

procedure TCustomLabelEdit.SetEditText(Value: string);
begin
  if fEditString<>Value then begin
     fEditString:=Value;
     if fEdit<>nil then fEdit.Text:=fEditString;
     end;
end;

procedure TCustomLabelEdit.SetHideSelection(Value: boolean);
begin
  if fHideSelection<>Value then begin
     fHideSelection:=Value;
     if fEdit<>nil then fEdit.HideSelection:=fHideSelection;
     end;
end;

procedure TCustomLabelEdit.SetDisableEdition(Value: boolean);
begin
  if fDisableEdition<>Value then begin
     fDisableEdition:=Value;
     if fEdit<>nil then fEdit.ReadOnly:=fDisableEdition;
     end;
end;

procedure TCustomLabelEdit.SetMaxEditLength (Value: integer);
begin
  if fMaxEditLength<>Value then begin
     fMaxEditLength:=Value;
     if fEdit<>nil then fEdit.MaxLength:=fMaxEditLength;
     end;
end;

procedure TCustomLabelEdit.SetOEMConvert(Value: boolean);
begin
  if fOEMConvert<>Value then begin
     fOEMConvert:=Value;
     if fEdit<>nil then fEdit.OEMConvert:=fOEMConvert;
     end;
end;

procedure TCustomLabelEdit.SetPasswordChar(Value: char);
begin
  if fPasswordChar<>Value then begin
     fPasswordChar:=Value;
     if fEdit<>nil then fEdit.PasswordChar:=fPasswordChar;
     end;
end;

procedure TCustomLabelEdit.SetReadOnly(Value: boolean);
begin
  if fReadOnly<>Value then begin
     fReadOnly:=Value;
     if fEdit<>nil then fEdit.ReadOnly:=fReadOnly;
     end;
end;

procedure TCustomLabelEdit.SetAutoHeight(NonAutoValue: integer);
var EffectiveLabelHeight: integer;
begin
  if fAutoSize then begin
     fEditHeight:=GetEditHeightFromFont(fEdit.Font,Self);
     with fLCDimensions do begin
          EffectiveLabelHeight:=LabelHeight+2*CHeightMargin;
          case fLCStyle of
               lcHorizontal: if EffectiveLabelHeight>fEditHeight then LCHeight:=EffectiveLabelHeight
                          else LCHeight:=fEditHeight;
               lcVertical:   LCHeight:=EffectiveLabelHeight+fEditHeight;
               end;
          end;
     end
  else fLCDimensions.LCHeight:=NonAutoValue;
end;

procedure TCustomLabelEdit.SwapLCWidthHeight;
var LabelUnitWidth,EffectiveLabelHeight: integer;
begin
  with fLCDimensions do begin
       EffectiveLabelHeight:=LabelHeight+2*CHeightMargin;
       case fLCStyle of
            lcHorizontal: begin
                          LCWidth:=LabelWidth+fUnitWidth+fEditWidth+2*CWidthMargin;
                          if fUnitWidth>0 then begin
                             if fUnitPosition=upLeft then LCWidth:=LCWidth+CWidthMargin
                             else LCWidth:=LCWidth+2*CWidthMargin;
                             end;
                          if EffectiveLabelHeight>fEditHeight then LCHeight:=EffectiveLabelHeight
                          else LCHeight:=fEditHeight;
                          end;
            lcVertical:   begin
                          LabelUnitWidth:=LabelWidth+fUnitWidth+2*CWidthMargin;
                          if fUnitWidth>0 then LabelUnitWidth:=LabelUnitWidth+CWidthMargin;
                          if LabelUnitWidth>fEditWidth then LCWidth:=LabelUnitWidth
                          else LCWidth:=fEditWidth;
                          LCHeight:=LabelHeight+fEditHeight+2*CHeightMargin;
                          end;
           end;
      end;
end;

procedure TCustomLabelEdit.AssortHorizontal;
begin
  inherited AssortHorizontal;

  with fLCDimensions do begin
       if fEdit<>nil then begin
          fEdit.Top:=(LCHeight - fEditHeight) DIV 2;
          if (fUnitPosition=upLeft) and (fUnitWidth>0) then fEdit.Left:=LabelWidth+fUnitWidth+3*CWidthMargin
          else fEdit.Left:=LabelWidth+2*CWidthMargin;
          fEdit.Height:=fEditHeight;
          if (fUnitWidth>0) then begin
             if (fUnitPosition=upLeft) then fEditWidth:=LCWidth-LabelWidth-fUnitWidth-3*CWidthMargin
             else fEditWidth:=LCWidth-LabelWidth-fUnitWidth-4*CWidthMargin;
             end
          else fEditWidth:=LCWidth-LabelWidth-2*CWidthMargin;
          fEdit.Width:=fEditWidth;
          end;
       end;
end;

procedure TCustomLabelEdit.AssortVertical;
var HeightMargin: integer;
begin
  inherited AssortVertical;

  with fLCDimensions do begin
       HeightMargin:=(LCHeight-LabelHeight-fEditHeight) DIV 2;
       if HeightMargin<0 then HeightMargin:=0;
       fEditWidth:=LCWidth;

       if fLabel<>nil then fLabel.Top:=HeightMargin;
       if fUnit<>nil then fUnit.Top:=fLabel.Top;

       if fEdit<>nil then begin
          fEdit.Top:=LabelHeight+2*HeightMargin;
          fEdit.Left:=0;
          fEdit.Height:=fEditHeight;
          fEdit.Width:=fEditWidth;
          end;
     end;
end;

procedure TCustomLabelEdit.EditChanged(Sender: TObject);
begin
  if Assigned(fOnChange) then fOnChange(Self);
end;

procedure TCustomLabelEdit.EditEnter(Sender: TObject);
begin
  if Assigned(fOnEnter) then fOnEnter(Self);
end;

procedure TCustomLabelEdit.EditExit(Sender: TObject);
begin
  if Assigned(fOnExit) then fOnExit(Self);
end;

procedure TCustomLabelEdit.EditKeyDown(Sender: TObject; var Key: word; Shift: TShiftState);
begin
  if Assigned(fOnKeyDown) then fOnKeyDown(Self,Key,Shift);
end;

procedure TCustomLabelEdit.EditKeyPressed(Sender: TObject; var Key: char);
begin
  if Assigned(fOnKeyPress) then fOnKeyPress(Self,Key);
end;

procedure TCustomLabelEdit.EditKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);
begin
  if Assigned(fOnKeyUp) then fOnKeyUp(Self,Key,Shift);
end;

constructor TCustomLabelEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fAutoSelect:=true;
  fCharCase:=ecNormal;
  fEditAlignment:=taRightJustify;
  fMaxEditLength:=0;
  fHideSelection:=true;
  fDisableEdition:=false;
  fOEMConvert:=false;
  fPasswordChar:=#0;
  fReadOnly:=false;

  InsertEditField;
  if fEdit<>nil then
     with fEdit do begin
          Parent:=Self;
          Alignment:=fEditAlignment;
          AutoSelect:=fAutoSelect;
          CharCase:=fCharCase;
          HideSelection:=fHideSelection;
          OEMConvert:=fOEMConvert;
          PasswordChar:=fPasswordChar;
          ReadOnly:=fReadOnly;
          OnChange:=EditChanged;
          OnEnter:=EditEnter;
          OnExit:=EditExit;
          OnKeyDown:=EditKeyDown;
          OnKeyPress:=EditKeyPressed;
          OnKeyUp:=EditKeyUp;
          end;

  fLabel.FocusControl:=fEdit;

  fEditHeight:=GetEditHeightFromFont(fEdit.Font,Self);
  if fEditHeight>(fLCDimensions.LabelHeight+2*CHeightMargin) then fLCDimensions.LCHeight:=fEditHeight;
end;

destructor TCustomLabelEdit.Free;
begin
  if fEdit<>nil then fEdit.Free;

  inherited Free;
end;

procedure TCustomLabelEdit.SelectAll;
begin
  if fEdit<>nil then fEdit.SelectAll;
end;

    { TCustomLabelTypeEdit }

procedure TCustomLabelTypeEdit.SetAutoLimit(Value: boolean);
begin
  if fAutoLimit<>Value then begin
     fAutoLimit:=Value;
     if fEdit<>nil then (fEdit as TCustomTypeEdit).AutoLimit:=fAutoLimit;
     end;
end;

procedure TCustomLabelTypeEdit.SetCharValidation(Value: TCharValidation);
begin
  if fCharValidation<>Value then begin
     fCharValidation:=Value;
     if fEdit<>nil then (fEdit as TCustomTypeEdit).CharValidation:=fCharValidation;
     end;
end;

procedure TCustomLabelTypeEdit.SetRangeValidation(Value: TRangeValidation);
begin
  if fRangeValidation<>Value then begin
     fRangeValidation:=Value;
     if fEdit<>nil then (fEdit as TCustomTypeEdit).RangeValidation:=fRangeValidation;
     end;
end;

constructor TCustomLabelTypeEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fAutoLimit:=false;
  fCharValidation:=cvNone;
  fRangeValidation:=rvNone;

  if fEdit<>nil then begin
     with (fEdit as TCustomTypeEdit) do begin
          AutoLimit:=fAutoLimit;
          CharValidation:=fCharValidation;
          RangeValidation:=fRangeValidation;
          end;
     OnWrongChar:=(fEdit as TCustomTypeEdit).OnWrongChar;
     end;
end;

procedure TCustomLabelTypeEdit.ClearEdit;
begin
  if fEdit<>nil then (fEdit as TCustomTypeEdit).ClearEdit;
end;

    { registered components }

    { TMyEdit }

    { no source declaration for TMyEdit, only type declaration }


    { TStringEdit }

procedure TStringEdit.ConvertValueToText;
var NewText: string;
    IX: word;
begin
  { CharCase:=ecLowerCase}
  { CharCase:=ecUpperCase}
  case fCaseTypeValidation of
       ctvChangeToLower: Text:=LowerCase(Text);
       ctvChangeToUpper: Text:=UpperCase(Text);
       ctvLowerOnly: begin
                     NewText:=Text; IX:=1;
                     while not(IX>System.Length(NewText)) do begin
                           if (NewText[IX] in ['A'..'Z']) then System.Delete(NewText,IX,1)
                           else System.Inc(IX);
                           end;
                     Text:=NewText;
                     end;
       ctvUpperOnly: begin
                     NewText:=Text; IX:=1;
                     while not(IX>System.Length(NewText)) do begin
                           if (NewText[IX] in ['a'..'z']) then System.Delete(NewText,IX,1)
                           else System.Inc(IX);
                           end;
                     Text:=NewText;
                     end;
       end;
end;

function TStringEdit.IsValidChar(var Key: char): boolean;
var CharPresent: boolean;
begin
  if ((fCaseTypeValidation=ctvLowerOnly) and IsUpCase(Key)) or
     ((fCaseTypeValidation=ctvUpperOnly) and IsLowCase(Key)) then Result:=false
  else
     begin
     case fCaseTypeValidation of
          ctvChangeToLower: Key:=LowCase(Key);
          ctvChangeToUpper: Key:=UpCase(Key);
          end;
     CharPresent:=System.Pos(Key,fControlChars)<>0;
     Result:=(fCharValidation=cvNone) or
             ((fCharValidation=cvEnabled) and (CharPresent)) or
             ((fCharValidation=cvDisabled) and not(CharPresent));
     end;
end;

function TStringEdit.TextSuccessfulConverted: boolean;
begin
  { Text can be always converted to text }
  { procedure required only because of abstract declaration in TCustomTypeEdit }
  Result:=true;
end;

procedure TStringEdit.ValidateRanges;
begin
  { the is no Range Validation for TStringEdit }
  { procedure required only because of abstract declaration in TCustomTypeEdit }
end;

procedure TStringEdit.SetValueToValidRange;
begin
  { the is no Valid Range for TStringEdit }
  { procedure required only because of abstract declaration in TCustomTypeEdit }
end;

procedure TStringEdit.SetCaseTypeValidation(Value: TCaseTypeValidation);
begin
  if fCaseTypeValidation<>Value then begin
     fCaseTypeValidation:=Value;
     ConvertValueToText;
     end;
end;

constructor TStringEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fCaseTypeValidation:=ctvNone;
  fControlChars:='';
  Text:='';
end;

    { TIntegerEdit }

function TIntegerEdit.GetFormatedText(Value: integer): string;
var FmtText: string;
begin
  if fIntegerFormat=ifDecimal then FmtStr(FmtText, '%d', [Value])
  else FmtStr(FmtText, '%x', [Value]);
  Result:=FmtText;
end;

procedure TIntegerEdit.ConvertValueToText;
var NewText: string;
begin
  NewText:=GetFormatedText(fValue);
  if NewText<>Text then Text:=NewText;
end;

function TIntegerEdit.IsValidChar(var Key: char): boolean;
var NumberChar,ZeroChar,HexChar,MinusChar: boolean;
begin
  Key:=UpCase(Key);
  NumberChar:=(Key in ['1'..'9']);
  ZeroChar:=(Key='0');
  HexChar:=((Key in ['A'..'F']) and (fIntegerFormat=ifHex));
  MinusChar:=((Key ='-') and (fIntegerFormat=ifDecimal));
  if ((Text='') or ((Text<>'') and (Text=SelText))) then Result:=ZeroChar or NumberChar or MinusChar or HexChar
  else
     begin
     if NumberChar or HexChar then begin
        if ((SelStart=1) and (System.Copy(Text,1,1)='0')) or
           ((SelStart=2) and (System.Copy(Text,1,2)='-0')) then begin
           SelStart:=SelStart-1; SelLength:=1;
            end;
        Result:=true;
        end
     else
        begin
        Result:=(MinusChar and ((SelStart=0) and (System.Pos('-',Text)=0))) or
                (ZeroChar and (((SelStart>0) and not(Text[1] in ['-','0'])) or
                               ((SelStart>1) and (Text[1]='-') and (Text[2] in ['1'..'9']))));
        end;
     end;
end;

function TIntegerEdit.TextSuccessfulConverted: boolean;
begin
  Result:=true;
  try
    if fIntegerFormat=ifDecimal then begin
       if (Text='') or (Text='-') or (Text='0') then fValue:=0
       else fValue:=StrToInt(Text);
       end
    else
       begin
       if (Text='') then fValue:=0
       else fValue:=StrToInt('$'+Text);
       end;
  except
    on EConvertError do Result:=false;
  end;
end;

procedure TIntegerEdit.ValidateRanges;
begin
  if (fValue<fMin) or (fValue>fMax) then begin
     if Assigned(fOnRangeValidationError) then fOnRangeValidationError(Self);
     if fAutoLimit then SetValueToValidRange;
     end;
end;

procedure TIntegerEdit.SetValueToValidRange;
begin
  if fValue<fMin then begin fValue:=fMin; ConvertValueToText; end
  else if fValue>fMax then begin fValue:=fMax; ConvertValueToText; end;
end;

procedure TIntegerEdit.SetMin(Value: integer);
begin
  if (fMin<>Value) then begin
     if not((fIntegerFormat=ifHex) and (Value<0)) then fMin:=Value
     else fMin:=0;
     end;
end;

procedure TIntegerEdit.SetMax(Value: integer);
begin
  if (fMax<>Value) then begin
     if not((fIntegerFormat=ifHex) and (Value>100)) then fMax:=Value
     else fMax:=100;
     end;
end;

procedure TIntegerEdit.SetIntegerFormat(Value: TIntegerFormat);
begin
  if fIntegerFormat<>Value then begin
     fIntegerFormat:=Value;
     if (fIntegerFormat=ifHex) then begin
        if (fMin<0) then fMin:=0;
        if (fMax<0) then Max:=100;
        end;
     ConvertValueToText;
     end;
end;

procedure TIntegerEdit.SetValue(Value: integer);
begin
  if fValue<>Value then begin
     fValue:=Value;
     ConvertValueToText;
     end;
end;

constructor TIntegerEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fAutoLimit:=false;
  fIntegerFormat:=ifDecimal;
  fMin:=0; fMax:=100;
  fValue:=0;
  Text:='0';
end;

{ the job of the below procedure should be to place the component suffix number
 (i.e. 3 for IntegerEdit3 to the Text window - but unfortunately, the is no expected effect  }
{procedure TIntegerEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fValue=0) then Text:=IntToStr(GetObjectNumber(Name));
  inherited;
end;}

    { TRealEdit }

function TRealEdit.GetFormatedText(Value: real): string;
var FmtText: string;
begin
  if fRealFormat=rfNormal then FmtStr(FmtText, '%0.'+IntToStr(fRealPrecision)+'f', [Value])
  else FmtStr(FmtText, '%.'+IntToStr(fRealPrecision)+'E', [Value]);
  Result:=FmtText;
end;

procedure TRealEdit.ConvertValueToText;
var NewText: string;
begin
  NewText:=GetFormatedText(fValue);
  if NewText<>Text then Text:=NewText;
end;

function TRealEdit.IsValidChar(var Key: char): boolean;
var SeparatorPosition,EPosition: byte;
begin
  Key:=UpCase(Key);
  if ((Text='') or ((Text<>'') and (Text=SelText))) then Result:=(Key in ['-','0'..'9']) or ((Key=DecimalSeparator) and (fRealPrecision>0))
  else
     begin
     if (fRealFormat=rfScientific) then EPosition:=System.Pos('E',Text) else EPosition:=0;
     if ((Key in ['1'..'9']) and not(SelStart<System.Pos('-',Text))) then begin
        if ((SelStart=EPosition+1) and (System.Copy(Text,EPosition+1,1)='0')) or
           ((SelStart=EPosition+2) and (System.Copy(Text,EPosition+1,2)='-0')) then begin
           SelStart:=SelStart-1; SelLength:=1;
           end;
        Result:=true;
        end
     else
        begin
        SeparatorPosition:=System.Pos(DecimalSeparator,Text);
        Result:=((Key=DecimalSeparator) and (SeparatorPosition=0) and (fRealPrecision>0)) or
                ((Key='E') and (fRealFormat=rfScientific) and (EPosition=0)) or
                ((Key ='-') and ((SelStart=0) and (System.Pos('-',Text)=0)) or ((EPosition>0) and (SelStart=EPosition))) or
                ((Key='0') and (((SeparatorPosition>0) and not(SelStart<SeparatorPosition)) or
                               ((SelStart>0) and (Text[1] in ['1'..'9'])) or
                               ((SelStart>1) and (Text[1]='-') and (Text[2] in ['1'..'9'])) or
                               ((SelStart=0) and (SeparatorPosition=1)) or
                               ((SelStart=1) and (Text='-')) or
                               ((SelStart=1) and (SeparatorPosition=2) and (Text[1]='-'))));
        end;
     end;
end;

function TRealEdit.TextSuccessfulConverted: boolean;
begin
  Result:=true;
  try
    if (Text='') or (Text='-') or ((Text<>'') and (Text[1]='E')) then fValue:=0.0
    else fValue:=StrToFloat(Text);
  except
    on EConvertError do Result:=false;
  end;
end;

procedure TRealEdit.SetValue(Value: real);
begin
  if fValue<>Value then begin
     fValue:=Value;
     ConvertValueToText;
     end;
end;

procedure TRealEdit.SetRealPrecision(Value: byte);
begin
  if fRealPrecision<>Value then begin
     if not(Value>8) then fRealPrecision:=Value else fRealPrecision:=8;
     ConvertValueToText;
     end;
end;

procedure TRealEdit.SetRealFormat(Value: TRealFormat);
begin
  if fRealFormat<>Value then begin
     fRealFormat:=Value;
     ConvertValueToText;
     end;
end;

procedure TRealEdit.ValidateRanges;
begin
  if (fValue<fMin) or (fValue>fMax) then begin
     if Assigned(fOnRangeValidationError) then fOnRangeValidationError(Self);
     if fAutoLimit then SetValueToValidRange;
     end;
end;

procedure TRealEdit.SetValueToValidRange;
begin
  if fValue<fMin then begin fValue:=fMin; ConvertValueToText; end
  else if fValue>fMax then begin fValue:=fMax; ConvertValueToText; end;
end;

constructor TRealEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fAutoLimit:=false;
  fRealFormat:=rfNormal;
  fRealPrecision:=3;
  fMin:=0.0; fMax:=100.0;
  fValue:=0.0;

  ConvertValueToText;
end;

{ the job of the below procedure should be to place the component suffix number
 (i.e. 3.000 for RealEdit3 to the Text window - but unfortunately, the is no expected effect  }
{procedure TRealEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fValue=0.0) then begin
     fValue:=GetObjectNumber(Name); ConvertValueToText; end;
  inherited;
end;}

    { TDateTimeEdit }

procedure TDateTimeEdit.EvaluateDateTimeFormat;
{var DateTimeStringArray: TDateTimeStringArray;}
begin
     case fDateTimeFormat of
          dtfTime: begin
                   fValidSeparator:=TimeSeparator;
                   fFirst:=2; fSecond:=2; fThird:=2;
                   end;
          dtfDate: begin
                   fValidSeparator:=DateSeparator;
                   {ShareDateTimeString(ShortDateFormat,DateSeparator,DateTimeStringArray);}
                   { place to change the year position for different systems
                    - as result of ShortDateFormat string analysis: dd.MM.yyyy }
                   fFirst:=2; fSecond:=2; fThird:=4;
                   end;
          end;
end;

function TDateTimeEdit.GetFormatedText(Value: TDateTime): string;
var FmtText: string;
begin
  if fDateTimeFormat=dtfDate then FmtText:=(DateToStr(fValue))
  else FmtText:=(TimeToStr(fValue));
  Result:=FmtText;
end;

procedure TDateTimeEdit.ConvertValueToText;
var NewText: string;
begin
  NewText:=GetFormatedText(fValue);
  if NewText<>Text then Text:=NewText;
end;

function TDateTimeEdit.IsValidChar(var Key: char): boolean;
  function CountCharsInString(SP: char; Target: string): byte;
  var I,L,C: byte;
  begin
    L:=System.Length(Target); C:=0;
    for I:=1 to L do if Target[I]=SP then Inc(C);
    Result:=C;
  end;
var SeparatorCount,FirstSep,SecondSep,TextLength: byte;
begin
  if ((Text='') or ((Text<>'') and (Text=SelText))) then Result:=(Key in ['0'..'9'])
  else
     begin
     if not((Key in ['0'..'9']) or (Key = fValidSeparator)) then Result:=false
     else
        begin
        SeparatorCount:=CountCharsInString(fValidSeparator,Text);
        if (Key=fValidSeparator) then Result:=(SelStart>0) and
                                              (SeparatorCount<2) and
                                              (Text[SelStart]<>fValidSeparator) and
                                              ((SelStart>=System.Length(Text)) or (Text[SelStart+1]<>fValidSeparator))
        else
           begin
           FirstSep:=0; SecondSep:=0; TextLength:=System.Length(Text);
           case SeparatorCount of
                1: FirstSep:=System.Pos(fValidSeparator,Text);
                2: begin
                   FirstSep:=System.Pos(fValidSeparator,Text);
                   SecondSep:=System.Pos(fValidSeparator,System.Copy(Text,FirstSep+1,255))+FirstSep;
                   end;
                end;
           Result:=((SeparatorCount=0) and (SelStart<fFirst)) or
                   ((SeparatorCount in [1,2]) and (FirstSep<(fFirst+1)) and (SelStart<fFirst)) or
                   ((SeparatorCount=1) and not(FirstSep>SelStart) and ((SelStart-FirstSep)<fSecond) and (TextLength<(FirstSep+fSecond))) or
                   ((SeparatorCount=2) and not(FirstSep>SelStart) and ((SelStart-SecondSep)<(fSecond+1)) and ((SecondSep-FirstSep)<(fSecond+1))) or
                   ((SeparatorCount=2) and not(SecondSep>SelStart) and (TextLength<(SecondSep+fThird)));
           end;
        end;
     end;
end;

function TDateTimeEdit.TextSuccessfulConverted: boolean;
begin
  Result:=true;
  try
    if DateTimeFormat=dtfDate then fValue:=StrToDate(Text)
    else fValue:=StrToTime(Text);
  except
    on EConvertError do Result:=false;
  end;
end;

procedure TDateTimeEdit.SetValue(Value: TDateTime);
begin
  if fValue<>Value then begin
     fValue:=Value;
     ConvertValueToText;
     end;
end;

procedure TDateTimeEdit.SetRangeValidation(Value: TRangeValidation);
begin
  if Value=rvOnChange then Value:=rvNone;
  if fRangeValidation<>Value then fRangeValidation:=Value;
end;

procedure TDateTimeEdit.SetValueToValidRange;
begin
  if fValue<fMin then begin fValue:=fMin; ConvertValueToText; end
  else if fValue>fMax then begin fValue:=fMax; ConvertValueToText; end;
end;

procedure TDateTimeEdit.SetDateTimeFormat(Value: TDateTimeFormat);
begin
  if fDateTimeFormat<>Value then begin
     fDateTimeFormat:=Value;
     EvaluateDateTimeFormat;
     ConvertValueToText;
     end;
end;

procedure TDateTimeEdit.ValidateRanges;
begin
  if (fValue<fMin) or (fValue>fMax) then begin
     if Assigned(fOnRangeValidationError) then fOnRangeValidationError(Self);
     if fAutoLimit then SetValueToValidRange;
     end;

  if (fValue<fMin) or (fValue>fMax) then
     if Assigned(fOnRangeValidationError) then fOnRangeValidationError(Self);
end;

procedure TDateTimeEdit.CMChanged(var Message: TMessage);
begin
  if Parent <> nil then Parent.WindowProc(Message);
end;

procedure TDateTimeEdit.CMExit(var Message: TCMExit);
begin
  if not(TextSuccessfulConverted) and (Assigned(fOnDataTypeError)) then
     fOnDataTypeError(Self);
  ConvertValueToText;
  inherited;
  if fRangeValidation=rvOnExit then ValidateRanges;
end;

constructor TDateTimeEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fAutoLimit:=false;
  fDateTimeFormat:=dtfDate;
  fMin:=0.0; fMax:=365.25*250;
  fValue:=Date;

  EvaluateDateTimeFormat;
  ConvertValueToText;
end;

{ the job of the below procedure should be to place the component suffix number
 (i.e. Today+3 for DateTimeEdit3 to the Text window - but unfortunately, the is no expected effect  }
{procedure TDateTimeEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fValue=0.0) then begin
     fValue:=Date+GetObjectNumber(Name); ConvertValueToText; end;
  inherited;
end;}

    { TDisplay }

procedure TDisplay.AdjustHeight;
begin
  if fAutoSize then begin
     fDisplayHeight:=GetStaticTextHeightFromFont(fDisplay.Font,Self);
     SetBounds(Left,Top,Width,Height);
     end;
end;

procedure TDisplay.SetAutoSize(Value: boolean);
begin
  if fAutoSize<>Value then begin
     fAutoSize:=Value;
     AdjustHeight;
     end;
end;

function TDisplay.GetColor: TColor;
begin
  if fDisplay<>nil then Result:=fDisplay.Color else Result:=TColor(0);
end;

procedure TDisplay.SetColor(Value: TColor);
begin
  if (fDisplay<>nil) and (fDisplay.Color<>Value) then begin
     fDisplay.Color:=Value;
     if fBackground<>nil then fBackground.Color:=Value;
     end;
end;

function TDisplay.GetFont: TFont;
begin
  if fDisplay<>nil then Result:=fDisplay.Font else Result:=TFont(nil);
end;

procedure TDisplay.SetFont(Value: TFont);
begin
  if (fDisplay<>nil) and (fDisplay.Font<>Value) then begin
     fDisplay.Font:=Value;
     AdjustHeight;
     end;
end;

procedure TDisplay.SetAlignment(Value: TAlignment);
begin
  if fAlignment<>Value then begin
     fAlignment:=Value;
     fDisplay.Alignment:=fAlignment;
     end;
end;

procedure TDisplay.Set_Display(Value: string);
begin
  if fDisplayString<>Value then begin
     fDisplayString:=Value;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

procedure TDisplay.SetDisplayFormat(Value: TTypeFormat);
begin
  if fDisplayFormat<>Value then begin
     fDisplayFormat:=Value;
     end;
end;

procedure TDisplay.SetHexPositions(Value: byte);
begin
  if fHexPositions<>Value then begin
     if not(Value>8) then fHexPositions:=Value else fHexPositions:=8;
     end;
end;

procedure TDisplay.SetRealPrecision(Value: byte);
begin
  if fRealPrecision<>Value then begin
     if not(Value>7) then fRealPrecision:=Value else fRealPrecision:=7;
     end;
end;

constructor TDisplay.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fAlignment:=taRightJustify;

  fBackground:=TStaticText.Create(Self);
  with fBackground do begin
       Parent:=Self;
       AutoSize:=false;
       Color:=clBlack;
       BorderStyle:=sbsSunken;
       end;
  fDisplay:=TStaticText.Create(Self);
  with fDisplay do begin
       Parent:=Self;
       Alignment:=taRightJustify;
       AutoSize:=false;
       Color:=clBlack;
       Font.Color:=clLime;
       end;

  fAutoSize:=false;
  fDisplayHeight:=GetStaticTextHeightFromFont(fDisplay.Font,Self);
  if (csDesigning in ComponentState) then
     SetBounds(0, 0, (CDefaultLCWidth DIV 2)+2*CWidthMargin,fDisplayHeight+2*CHeightMargin);

  fDisplayFormat:=tfString;
  fHexPositions:=8;
  fRealPrecision:=3;

  Set_Display(CDisplay);
end;

destructor TDisplay.Free;
begin
  if fDisplay<>nil then fDisplay.Free;
  if fBackground<>nil then fBackground.Free;

  inherited Free;
end;

procedure TDisplay.SetBounds(ALeft, ATop, AWidth, AHeight: integer);
begin
  if AutoSize then AHeight:=fDisplayHeight+2*CHeightMargin;
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  if fBackground<>nil then fBackground.SetBounds(0, 0, AWidth, AHeight);
  if fDisplay<>nil then fDisplay.SetBounds(CWidthMargin, CHeightMargin, AWidth-2*CWidthMargin, AHeight-2*CHeightMargin);
end;

procedure TDisplay.Invalidate;
var N: integer;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fDisplay<>nil) then
     begin
     N:=GetObjectNumber(Name);
     if (N>0) then
        Set_Display(CDisplay+IntToStr(N));
     end;
  inherited;
end;

procedure TDisplay.Reset;
begin
  if fDisplayFormat in [tfDecimal,tfHex] then SetDisplay(0)
  else if fDisplayFormat in [tfFloat,tfScientific,tfDate,tfTime] then SetDisplay(0.0)
  else if (fDisplayFormat=tfString) then SetDisplay('');
end;

function TDisplay.SetDisplay(Value: string): boolean;
begin
  if (fDisplayFormat<>tfString) then begin
     Value:='#####'; Result:=false;
     end
  else Result:=true;
  if fDisplayString<>Value then begin
     fDisplayString:=Value;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

function TDisplay.SetDisplay(Value: integer): boolean;
var NewDisplay: string;
begin
  Result:=true;
  case fDisplayFormat of
       tfDecimal: FmtStr(NewDisplay, '%d', [Value]);
       tfHex: if (fHexPositions=0) then FmtStr(NewDisplay, '%x', [Value])
              else FmtStr(NewDisplay, '%.'+IntToStr(fHexPositions)+'x', [Value]);
       else
          begin
          NewDisplay:='#####';
          Result:=false;
          end;
       end;
  if fDisplayString<>NewDisplay then begin
     fDisplayString:=NewDisplay;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

function TDisplay.SetDisplay(Value: extended): boolean;
var NewDisplay: string;
begin
  Result:=true;
  case fDisplayFormat of
       tfFloat: FmtStr(NewDisplay, '%0.'+IntToStr(fRealPrecision)+'f', [Value]);
       tfScientific: FmtStr(NewDisplay, '%.'+IntToStr(fRealPrecision)+'E', [Value]);
       tfDate: NewDisplay:=(DateToStr(Value));
       tfTime: NewDisplay:=(TimeToStr(Value));
       else
          begin
          NewDisplay:='#####';
          Result:=false;
          end;
       end;
  if fDisplayString<>NewDisplay then begin
     fDisplayString:=NewDisplay;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

    { TLabelComponent }

procedure TLabelComponent.SetAutoHeight(NonAutoValue: integer);
var EffectiveLabelHeight: integer;
begin
  if fAutoSize then begin
     with fLCDimensions do begin
          EffectiveLabelHeight:=LabelHeight+2*CHeightMargin;
          case fLCStyle of
               lcHorizontal: LCHeight:=EffectiveLabelHeight;
               lcVertical:   LCHeight:=EffectiveLabelHeight+fDefaultHeight;
               end;
          end;
     end
  else fLCDimensions.LCHeight:=NonAutoValue;
end;

procedure TLabelComponent.SwapLCWidthHeight;
var LabelUnitWidth,EffectiveLabelHeight: integer;
begin
  with fLCDimensions do begin
       EffectiveLabelHeight:=LabelHeight+2*CHeightMargin;
       case fLCStyle of
            lcHorizontal: begin
                          LCWidth:=LabelWidth+fUnitWidth+fDefaultWidth+3*CWidthMargin;
                          if fUnitWidth>0 then begin
                             if (fUnitPosition=upLeft) then LCWidth:=LCWidth+CWidthMargin
                             else LCWidth:=LCWidth+2*CWidthMargin;
                             end;
                          if EffectiveLabelHeight>fDefaultHeight then LCHeight:=EffectiveLabelHeight
                          else LCHeight:=fDefaultHeight;
                          end;
            lcVertical:   begin
                          LabelUnitWidth:=LabelWidth+fUnitWidth+2*CWidthMargin;
                          if fUnitWidth>0 then LabelUnitWidth:=LabelUnitWidth+CWidthMargin;
                          if LabelUnitWidth>fDefaultWidth then LCWidth:=LabelUnitWidth
                          else LCWidth:=fDefaultWidth;
                          LCHeight:=LabelHeight+fDefaultHeight+2*CHeightMargin;
                          end;
           end;
      end;
end;

procedure TLabelComponent.AssortHorizontal;
begin
  inherited AssortHorizontal;

  with fLCDimensions do begin
       if fUnitWidth>0 then begin
          if (fUnitPosition=upLeft) then fDefaultWidth:=fDefaultWidth-3*CWidthMargin
          else fDefaultWidth:=fDefaultWidth-4*CWidthMargin;
          end
       else fDefaultWidth:=LCWidth-LabelWidth-fUnitWidth-2*CWidthMargin;
       fDefaultHeight:=LabelHeight;
       end;
end;

procedure TLabelComponent.AssortVertical;
var HeightMargin: integer;
begin
  inherited AssortVertical;

  with fLCDimensions do begin
       HeightMargin:=(LCHeight-LabelHeight-fDefaultHeight) DIV 2;
       if HeightMargin<0 then HeightMargin:=0;

       if fLabel<>nil then fLabel.Top:=HeightMargin;
       if fUnit<>nil then fUnit.Top:=HeightMargin;

       fDefaultWidth:=LCWidth;
       fDefaultHeight:=LabelHeight;
       end;
end;

    { TLabelDisplay }

procedure TLabelDisplay.CMEnabledChanged(var Message: TMessage);
begin
  inherited;

  //
end;

function TLabelDisplay.GetDisplayColor: TColor;
begin
  if fDisplay<>nil then Result:=fDisplay.Color else Result:=TColor(0);
end;

procedure TLabelDisplay.SetDisplayColor(Value: TColor);
begin
  if (fDisplay<>nil) and (fDisplay.Color<>Value) then begin
     if fDisplay.ParentColor then fDisplay.ParentColor:=false;
     fDisplay.Color:=Value;
     if (fBackground<>nil) and not(csDesigning in ComponentState) then begin
        if fBackground.ParentColor then fBackground.ParentColor:=false;
        fBackground.Color:=Value;
        end;
     end;
end;

function TLabelDisplay.GetDisplayFont: TFont;
begin
  if fDisplay<>nil then begin
     if fAutoSize then AdjustHeight;
     AssortComponents;
     Result:=fDisplay.Font;
     end
  else Result:=TFont(nil);
end;

procedure TLabelDisplay.SetDisplayFont(Value: TFont);
begin
  if (fDisplay<>nil) and (fDisplay.Font<>Value) then begin
     fDisplay.Font:=Value;
     if fAutoSize then AdjustHeight;
     AssortComponents;
     end;
end;

procedure TLabelDisplay.Set_Display(Value: string);
begin
  if fDisplayString<>Value then begin
     fDisplayString:=Value;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

procedure TLabelDisplay.SetAutoHeight(NonAutoValue: integer);
begin
  if fAutoSize then begin
     fDisplayHeight:=GetStaticTextHeightFromFont(fDisplay.Font,Self);
     with fLCDimensions do
          case fLCStyle of
               lcHorizontal: if LabelHeight>fDisplayHeight then LCHeight:=LabelHeight+2*CHeightMargin
                             else LCHeight:=fDisplayHeight+2*CHeightMargin;
               lcVertical:   fLCDimensions.LCHeight:=fLCDimensions.LabelHeight+fDisplayHeight+4*CHeightMargin;
               end;
     end
  else fLCDimensions.LCHeight:=NonAutoValue;
end;

procedure TLabelDisplay.SwapLCWidthHeight;
var LabelUnitWidth,BackgroundWidth: integer;
begin
  with fLCDimensions do begin
       case fLCStyle of
            lcHorizontal: begin
                          LCWidth:=LabelWidth+fUnitWidth+fDisplayWidth+4*CWidthMargin;
                          if fUnitWidth>0 then begin
                             if fUnitPosition=upLeft then LCWidth:=LCWidth+CWidthMargin
                             else LCWidth:=LCWidth+2*CWidthMargin;
                             end;
                          if LabelHeight>fDisplayHeight then LCHeight:=LabelHeight+2*CHeightMargin
                          else LCHeight:=fDisplayHeight+2*CHeightMargin;
                          end;
            lcVertical:   begin
                          LabelUnitWidth:=LabelWidth+fUnitWidth+2*CWidthMargin;
                          if fUnitWidth>0 then LabelUnitWidth:=LabelUnitWidth+CWidthMargin;
                          BackgroundWidth:=fDisplayWidth+2*CWidthMargin;
                          if LabelUnitWidth>BackgroundWidth then LCWidth:=LabelUnitWidth
                          else LCWidth:=BackgroundWidth;
                          LCHeight:=LabelHeight+fDisplayHeight+4*CHeightMargin;
                          end;
           end;
      end;
end;

procedure TLabelDisplay.AssortHorizontal;
begin
  inherited AssortHorizontal;

  with fLCDimensions do begin
       if fBackground<>nil then begin
          fBackground.Top:=0;
          if (fUnitPosition=upLeft) and (fUnitWidth>0) then fBackground.Left:=LabelWidth+fUnitWidth+3*CWidthMargin
          else fBackground.Left:=LabelWidth+2*CWidthMargin;
          fBackground.Height:=LCHeight;
          if (fUnitWidth>0) then begin
             if (fUnitPosition=upLeft) then fBackground.Width:=LCWidth-LabelWidth-fUnitWidth-3*CWidthMargin
             else fBackground.Width:=LCWidth-LabelWidth-fUnitWidth-4*CWidthMargin;
             end
          else fBackground.Width:=LCWidth-LabelWidth-2*CWidthMargin;

          fDisplayWidth:=fBackground.Width-2*CWidthMargin;

          if fDisplay<>nil then begin
             fDisplay.Top:=((LCHeight-fDisplayHeight) DIV 2)-1;
             fDisplay.Left:=CWidthMargin;
             fDisplay.Height:=fDisplayHeight;
             fDisplay.Width:=fDisplayWidth-2;
             fDisplay.Invalidate;
             end;
          end;
     end;
end;

procedure TLabelDisplay.AssortVertical;
var HeightMargin,BackgroundHeight: integer;
begin
  inherited AssortVertical;

  with fLCDimensions do begin
       HeightMargin:=(LCHeight-LabelHeight-fDisplayHeight) DIV 4;
       if HeightMargin<0 then HeightMargin:=0;

       if fLabel<>nil then fLabel.Top:=HeightMargin;
       if fUnit<>nil then fUnit.Top:=fLabel.Top;

       BackgroundHeight:=fDisplayHeight+2*HeightMargin;

       if fBackground<>nil then begin
          fBackground.Top:=LCHeight-BackgroundHeight;
          fBackground.Left:=0;
          fBackground.Height:=BackgroundHeight;
          fBackground.Width:=LCWidth;

          fDisplayWidth:=fBackground.Width-2*CWidthMargin;

          if fDisplay<>nil then begin
             fDisplay.Top:=HeightMargin-1;
             fDisplay.Left:=CWidthMargin;
             fDisplay.Height:=fDisplayHeight;
             fDisplay.Width:=fDisplayWidth-2;
             end;
          end;

     end;
end;

procedure TLabelDisplay.SetDisplayAlignment(Value: TAlignment);
begin
  if fDisplayAlignment<>Value then begin
     fDisplayAlignment:=Value;
     if fDisplay<>nil then fDisplay.Alignment:=fDisplayAlignment;
     end;
end;

procedure TLabelDisplay.SetDisplayFormat(Value: TTypeFormat);
begin
  if fDisplayFormat<>Value then begin
     fDisplayFormat:=Value;
     end;
end;

procedure TLabelDisplay.SetHexPositions(Value: byte);
begin
  if fHexPositions<>Value then begin
     if not(Value>8) then fHexPositions:=Value else fHexPositions:=8;
     end;
end;

procedure TLabelDisplay.SetRealPrecision(Value: byte);
begin
  if fRealPrecision<>Value then begin
     if not(Value>7) then fRealPrecision:=Value else fRealPrecision:=7;
     end;
end;

constructor TLabelDisplay.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fBackground:=TStaticText.Create(Self);
  with fBackground do begin
       Parent:=Self;
       AutoSize:=false;
       if (csDesigning in ComponentState) then Color:=clFuchsia else Color:=clBlack;
       BorderStyle:=sbsSunken;
       end;
  fDisplay:=TStaticText.Create(fBackground);
  with fDisplay do begin
       Parent:=fBackground;
       Alignment:=taRightJustify;
       AutoSize:=false;
       Color:=clBlack;
       Font.Color:=clLime;
       end;

  fDisplayHeight:=GetStaticTextHeightFromFont(fDisplay.Font,Self);
  if fDisplayHeight>fLCDimensions.LabelHeight then fLCDimensions.LCHeight:=fDisplayHeight+2*CHeightMargin;

  Set_Display(CDisplay);
end;

destructor TLabelDisplay.Free;
begin
  if fDisplay<>nil then fDisplay.Free;
  if fBackground<>nil then fBackground.Free;

  inherited Free;
end;

procedure TLabelDisplay.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fDisplay<>nil) and (fDisplay.Caption=CDisplay) then Set_Display(CDisplay+IntToStr(GetObjectNumber(Name)));
  inherited;
end;

procedure TLabelDisplay.Reset;
begin
  if fDisplayFormat in [tfDecimal,tfHex] then SetDisplay(0)
  else if fDisplayFormat in [tfFloat,tfScientific,tfDate,tfTime] then SetDisplay(0.0)
  else if (fDisplayFormat=tfString) then SetDisplay('');
end;

function TLabelDisplay.SetDisplay(Value: string): boolean;
begin
  if (fDisplayFormat<>tfString) then begin
     Value:='#####'; Result:=false;
     end
  else Result:=true;
  if fDisplayString<>Value then begin
     fDisplayString:=Value;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

function TLabelDisplay.SetDisplay(Value: integer): boolean;
var NewDisplay: string;
begin
  Result:=true;
  case fDisplayFormat of
       tfDecimal: FmtStr(NewDisplay, '%d', [Value]);
       tfHex: if (fHexPositions=0) then FmtStr(NewDisplay, '%x', [Value])
              else FmtStr(NewDisplay, '%.'+IntToStr(fHexPositions)+'x', [Value]);
       else
          begin
          NewDisplay:='#####';
          Result:=false;
          end;
       end;
  if fDisplayString<>NewDisplay then begin
     fDisplayString:=NewDisplay;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

function TLabelDisplay.SetDisplay(Value: extended): boolean;
var NewDisplay: string;
begin
  Result:=true;
  case fDisplayFormat of
       tfFloat: FmtStr(NewDisplay, '%0.'+IntToStr(fRealPrecision)+'f', [Value]);
       tfScientific: FmtStr(NewDisplay, '%.'+IntToStr(fRealPrecision)+'E', [Value]);
       tfDate: NewDisplay:=(DateToStr(Value));
       tfTime: NewDisplay:=(TimeToStr(Value));
       else
          begin
          NewDisplay:='#####';
          Result:=false;
          end;
       end;
  if fDisplayString<>NewDisplay then begin
     fDisplayString:=NewDisplay;
     if fDisplay<>nil then fDisplay.Caption:=fDisplayString;
     end;
end;

    { TLabelEdit }

procedure TLabelEdit.InsertEditField;
begin
  fEdit:=TMyEdit.Create(Self);
end;

constructor TLabelEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  SetEditText(CEdit);
end;

procedure TLabelEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fEdit<>nil) and (fEditString=CEdit) then SetEditText(CEdit+IntToStr(GetObjectNumber(Name)));
  inherited;
end;

    { TLabelStringEdit }

procedure TLabelStringEdit.InsertEditField;
begin
  fEdit:=TStringEdit.Create(Self);
end;

procedure TLabelStringEdit.SetCaseTypeValidation(Value: TCaseTypeValidation);
begin
  if fCaseTypeValidation<>Value then begin
     fCaseTypeValidation:=Value;
     if fEdit<>nil then (fEdit as TStringEdit).CaseTypeValidation:=fCaseTypeValidation;
     end;
end;

procedure TLabelStringEdit.SetControlChars(Value: string);
begin
  if fControlChars<>Value then begin
     fControlChars:=Value;
     if fEdit<>nil then (fEdit as TStringEdit).ControlChars:=fControlChars;
     end;
end;

procedure TLabelStringEdit.EditWrongChar(Sender: TObject);
begin
  if Assigned(fOnWrongChar) then fOnWrongChar(Self);
end;

constructor TLabelStringEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fCaseTypeValidation:=ctvNone;
  fControlChars:='';

  if fEdit<>nil then
     with (fEdit as TStringEdit) do begin
          CaseTypeValidation:=fCaseTypeValidation;
          ControlChars:=fControlChars;
          OnWrongChar:=EditWrongChar;
          end;

  EditText:=CString;
end;

procedure TLabelStringEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fEdit<>nil) and (fEditString=CString) then EditText:=(CString+IntToStr(GetObjectNumber(Name)));
  inherited;
end;

    { TLabelIntegerEdit }

procedure TLabelIntegerEdit.InsertEditField;
begin
  fEdit:=TIntegerEdit.Create(Self);
end;

procedure TLabelIntegerEdit.SetMin(Value: integer);
begin
  if fMin<>Value then begin
     fMin:=Value;
     if fEdit<>nil then (fEdit as TIntegerEdit).Min:=fMin;
     end;
end;

procedure TLabelIntegerEdit.SetMax(Value: integer);
begin
  if fMax<>Value then begin
     fMax:=Value;
     if fEdit<>nil then (fEdit as TIntegerEdit).Max:=fMax;
     end;
end;

procedure TLabelIntegerEdit.SetIntegerFormat(Value: TIntegerFormat);
begin
  if fIntegerFormat<>Value then begin
     fIntegerFormat:=Value;
     if fEdit<>nil then (fEdit as TIntegerEdit).IntegerFormat:=fIntegerFormat;
     end;
end;

function TLabelIntegerEdit.GetValue: integer;
begin
  if fEdit<>nil then Result:=(fEdit as TIntegerEdit).Value else Result:=0;
end;

procedure TLabelIntegerEdit.SetValue(Value: integer);
begin
  if (fEdit<>nil) and ((fEdit as TIntegerEdit).Value<>Value) then (fEdit as TIntegerEdit).Value:=Value;
end;

procedure TLabelIntegerEdit.SetRangeValidation(Value: TRangeValidation);
begin
  if fRangeValidation<>Value then begin
     fRangeValidation:=Value;
     if fEdit<>nil then (fEdit as TIntegerEdit).RangeValidation:=fRangeValidation;
     end;
end;

procedure TLabelIntegerEdit.EditDataTypeError(Sender: TObject);
begin
  if Assigned(fOnDataTypeError) then fOnDataTypeError(Self);
end;

procedure TLabelIntegerEdit.EditRangeValidationError(Sender: TObject);
var ErrorMessage: string;
begin
  if fRangeErrorMessage<>'' then begin
     FmtStr(ErrorMessage,fRangeErrorMessage,[fLabelCaption,GetFormatedText(fMin),GetFormatedText(fMax)]);
     MessageDlg(ErrorMessage, mtError, [mbCancel], 0);
     end;
  if Assigned(fOnRangeValidationError) then fOnRangeValidationError(Self);
end;

constructor TLabelIntegerEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fIntegerFormat:=ifDecimal;
  fMin:=0; fMax:=100;
  fRangeErrorMessage:='';

  if fEdit<>nil then
     with (fEdit as TIntegerEdit) do begin
          IntegerFormat:=fIntegerFormat;
          Max:=fMax; Min:=fMin; 
          Value:=0;
          OnDataTypeError:=EditDataTypeError;
          OnRangeValidationError:=EditRangeValidationError;
          end;
end;

procedure TLabelIntegerEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fEdit<>nil) and ((fEdit as TIntegerEdit).Value=0) then (fEdit as TIntegerEdit).Value:=GetObjectNumber(Name);
  inherited;
end;

function TLabelIntegerEdit.GetFormatedText(Value: integer): string;
begin
  if fEdit<>nil then Result:=(fEdit as TIntegerEdit).GetFormatedText(Value)
  else Result:='#####';
end;

procedure TLabelIntegerEdit.SetValueToValidRange;
begin
  if fEdit<>nil then (fEdit as TIntegerEdit).SetValueToValidRange;
end;

    { TLabelRealEdit }

procedure TLabelRealEdit.InsertEditField;
begin
  fEdit:=TRealEdit.Create(Self);
end;

procedure TLabelRealEdit.SetMin(Value: real);
begin
  if fMin<>Value then begin
     fMin:=Value;
     if fEdit<>nil then (fEdit as TRealEdit).Min:=fMin;
     end;
end;

procedure TLabelRealEdit.SetMax(Value: real);
begin
  if fMax<>Value then begin
     fMax:=Value;
     if fEdit<>nil then (fEdit as TRealEdit).Max:=fMax;
     end;
end;

function TLabelRealEdit.GetValue: real;
begin
  if fEdit<>nil then Result:=(fEdit as TRealEdit).Value else Result:=0;
end;

procedure TLabelRealEdit.SetValue(Value: real);
begin
  if (fEdit<>nil) and ((fEdit as TRealEdit).Value<>Value) then (fEdit as TRealEdit).Value:=Value;
end;

procedure TLabelRealEdit.SetRangeValidation(Value: TRangeValidation);
begin
  if fRangeValidation<>Value then begin
     fRangeValidation:=Value;
     if fEdit<>nil then (fEdit as TRealEdit).RangeValidation:=fRangeValidation;
     end;
end;

procedure TLabelRealEdit.SetRealFormat(Value: TRealFormat);
begin
  if fRealFormat<>Value then begin
     fRealFormat:=Value;
     if fEdit<>nil then (fEdit as TRealEdit).RealFormat:=fRealFormat;
     end;
end;

procedure TLabelRealEdit.SetRealPrecision(Value: byte);
begin
  if fRealPrecision<>Value then begin
     fRealPrecision:=Value;
     if fEdit<>nil then (fEdit as TRealEdit).RealPrecision:=fRealPrecision;
     end;
end;

procedure TLabelRealEdit.EditDataTypeError(Sender: TObject);
begin
  if Assigned(fOnDataTypeError) then fOnDataTypeError(Self);
end;

procedure TLabelRealEdit.EditRangeValidationError(Sender: TObject);
var ErrorMessage: string;
begin
  if fRangeErrorMessage<>'' then begin
     FmtStr(ErrorMessage,fRangeErrorMessage,[fLabelCaption,GetFormatedText(fMin),GetFormatedText(fMax)]);
     MessageDlg(ErrorMessage, mtError, [mbCancel], 0);
     end;
  if Assigned(fOnRangeValidationError) then fOnRangeValidationError(Self);
end;

constructor TLabelRealEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fRealFormat:=rfNormal;
  fRealPrecision:=3;
  fMin:=0.0; fMax:=100.0;
  fRangeErrorMessage:='';

  if fEdit<>nil then
     with (fEdit as TRealEdit) do begin
          RealFormat:=fRealFormat;
          Max:=fMax; Min:=fMin;
          RealPrecision:=fRealPrecision;
          Value:=0.0;
          OnDataTypeError:=EditDataTypeError;
          OnRangeValidationError:=EditRangeValidationError;
          end;

end;

procedure TLabelRealEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fEdit<>nil) and ((fEdit as TRealEdit).Value=0.0) then (fEdit as TRealEdit).Value:=GetObjectNumber(Name);
  inherited;
end;

function TLabelRealEdit.GetFormatedText(Value: real): string;
begin
  if fEdit<>nil then Result:=(fEdit as TRealEdit).GetFormatedText(Value)
  else Result:='#####';
end;

procedure TLabelRealEdit.SetValueToValidRange;
begin
  if fEdit<>nil then (fEdit as TRealEdit).SetValueToValidRange;
end;

    { TLabelDateTimeEdit }

procedure TLabelDateTimeEdit.InsertEditField;
begin
  fEdit:=TDateTimeEdit.Create(Self);
end;

procedure TLabelDateTimeEdit.SetDateTimeFormat(Value: TDateTimeFormat);
begin
  if fDateTimeFormat<>Value then begin
     fDateTimeFormat:=Value;
     if fEdit<>nil then (fEdit as TDateTimeEdit).DateTimeFormat:=fDateTimeFormat;
     end;
end;

procedure TLabelDateTimeEdit.SetMin(Value: TDateTime);
begin
  if fMin<>Value then begin
     fMin:=Value;
     if fEdit<>nil then (fEdit as TDateTimeEdit).Min:=fMin;
     end;
end;

procedure TLabelDateTimeEdit.SetMax(Value: TDateTime);
begin
  if fMax<>Value then begin
     fMax:=Value;
     if fEdit<>nil then (fEdit as TDateTimeEdit).Max:=fMax;
     end;
end;

function TLabelDateTimeEdit.GetValue: TDateTime;
begin
  if fEdit<>nil then Result:=(fEdit as TDateTimeEdit).Value else Result:=0;
end;

procedure TLabelDateTimeEdit.SetValue(Value: TDateTime);
begin
  if (fEdit<>nil) and ((fEdit as TDateTimeEdit).Value<>Value) then (fEdit as TDateTimeEdit).Value:=Value;
end;

procedure TLabelDateTimeEdit.SetRangeValidation(Value: TRangeValidation);
begin
  if fRangeValidation<>Value then begin
     fRangeValidation:=Value;
     if fEdit<>nil then (fEdit as TDateTimeEdit).RangeValidation:=fRangeValidation;
     end;
end;

procedure TLabelDateTimeEdit.EditDataTypeError(Sender: TObject);
begin
  if Assigned(fOnDataTypeError) then fOnDataTypeError(Self);
end;

procedure TLabelDateTimeEdit.EditRangeValidationError(Sender: TObject);
var ErrorMessage: string;
begin
  if fRangeErrorMessage<>'' then begin
     FmtStr(ErrorMessage,fRangeErrorMessage,[fLabelCaption,GetFormatedText(fMin),GetFormatedText(fMax)]);
     MessageDlg(ErrorMessage, mtError, [mbCancel], 0);
     end;
  if Assigned(fOnRangeValidationError) then fOnRangeValidationError(Self);
end;

constructor TLabelDateTimeEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fDateTimeFormat:=dtfDate;
  fMin:=0.0; fMax:=365.25*250;
  fRangeErrorMessage:='';

  if fEdit<>nil then
     with (fEdit as TDateTimeEdit) do begin
          DateTimeFormat:=fDateTimeFormat;
          Min:=fMin; Max:=fMax;
          Value:=Date;
          OnDataTypeError:=EditDataTypeError;
          OnRangeValidationError:=EditRangeValidationError;
          end;

end;

procedure TLabelDateTimeEdit.Invalidate;
begin
  if (csDesigning in ComponentState) and (Name<>'') and (fEdit<>nil) and ((fEdit as TDateTimeEdit).Value=Date) then (fEdit as TDateTimeEdit).Value:=Date+GetObjectNumber(Name);
  inherited;
end;

function TLabelDateTimeEdit.GetFormatedText(Value: TDateTime): string;
begin
  if fEdit<>nil then Result:=(fEdit as TDateTimeEdit).GetFormatedText(Value)
  else Result:='#####';
end;

procedure TLabelDateTimeEdit.SetValueToValidRange;
begin
  if fEdit<>nil then (fEdit as TDateTimeEdit).SetValueToValidRange;
end;

    { TLabelScrollIntegerEdit }

procedure TLabelScrollIntegerEdit.ArrangeAcrollBarPosition;
begin
  fArrangingScrollBarPosition:=true;
  if not(fReverseScrolling) then
     fScrollBar.Position:=Self.Value-Self.Min
  else fScrollBar.Position:=Self.Max-Self.Value;
  fArrangingScrollBarPosition:=false;
end;

procedure TLabelScrollIntegerEdit.ArrangeAllScrollBarValues;
begin
  if fScrollBar<>nil then
     begin
     fScrollBar.Min:=0;
     fScrollBar.Max:=Self.Max-Self.Min;
     ArrangeAcrollBarPosition;
     end;
end;

procedure TLabelScrollIntegerEdit.SetMin(Value: integer);
begin
  inherited SetMin(Value);
  ArrangeAllScrollBarValues;
end;

procedure TLabelScrollIntegerEdit.SetMax(Value: integer);
begin
  inherited SetMax(Value);
  ArrangeAllScrollBarValues;
end;

procedure TLabelScrollIntegerEdit.SetValue(Value: integer);
begin
  inherited SetValue(Value);
  ArrangeAllScrollBarValues;
end;

procedure TLabelScrollIntegerEdit.SetReverseScrolling(Value: boolean);
begin
  if fReverseScrolling<>Value then
     begin
     fReverseScrolling:=Value;
     ArrangeAllScrollBarValues;
     end;
end;

procedure TLabelScrollIntegerEdit.SetAutoHeight(NonAutoValue: integer);
begin
  if fAutoSize and (not(fNoLCStyleInfo) or (csDesigning in ComponentState)) then begin
     with fLCDimensions do
          case fLCStyle of
               lcHorizontal: begin
                             inherited SetAutoHeight(LCHeight-CHeightMargin -CScrollBarWidth);
                             LCHeight:=LCHeight+CHeightMargin+CScrollBarWidth;
                             end;
               lcVertical:   begin
                             inherited SetAutoHeight(NonAutoValue-CHeightMargin -fScrollBarLength);
                             LCHeight:=NonAutoValue;
                             end;
               end;
     end
  else fLCDimensions.LCHeight:=NonAutoValue;
end;

procedure TLabelScrollIntegerEdit.SwapLCWidthHeight;
begin
  if fNoLCStyleInfo or (csReading in ComponentState) then exit;

  inherited SwapLCWidthHeight;
  with fLCDimensions do
       case fLCStyle of
            lcHorizontal: begin
                          LCHeight:=LCHeight+CHeightMargin+CScrollBarWidth;
                          if fScrollBarLength>LCWidth then LCWidth:=fScrollBarLength;
                          end;
            lcVertical:   begin
                          LCHeight:=LCHeight+CHeightMargin+fScrollBarLength;
                          end;
            end;
end;

procedure TLabelScrollIntegerEdit.AssortHorizontal;
var HeightOffset: integer;
begin
  if fNoLCStyleInfo and not(csDesigning in ComponentState) then exit;

  inherited AssortHorizontal;

  with fLCDimensions do begin
       HeightOffset:=(CHeightMargin+CScrollBarWidth) DIV 2;
       if HeightOffset<0 then HeightOffset:=0;
       if fLabel<>nil then fLabel.Top:=fLabel.Top-HeightOffset;
       if fUnit<>nil then fUnit.Top:=fLabel.Top;
       if fEdit<>nil then fEdit.Top:=fEdit.Top-HeightOffset;

       fScrollBarLength:=LCWidth;

       if fScrollBar<>nil then
          with fScrollBar do begin
               Kind:=sbHorizontal;
               Top:=LCHeight-CScrollBarWidth;
               Width:=fScrollBarLength;
               Height:=CScrollBarWidth;
               end;
       end;
end;

procedure TLabelScrollIntegerEdit.AssortVertical;
begin
  if fNoLCStyleInfo and not(csDesigning in ComponentState) then exit;

  inherited AssortVertical;

  with fLCDimensions do begin
       if fLabel<>nil then fLabel.Top:=CHeightMargin;
       if fUnit<>nil then fUnit.Top:=CHeightMargin;
       if fEdit<>nil then fEdit.Top:=LabelHeight+2*CHeightMargin;

       fScrollBarLength:=LCHeight-LabelHeight-fEditHeight-3*CHeightMargin;
       if fScrollBarLength<0 then fScrollBarLength:=0;

       if fScrollBar<>nil then
          with fScrollBar do begin
               Kind:=sbVertical;
               Left:=0;
               Top:=LCHeight-fScrollBarLength;
               Width:=CScrollBarWidth;
               Height:=fScrollBarLength;
               end;
       end;
end;

procedure TLabelScrollIntegerEdit.EditExit(Sender: TObject);
begin
  inherited;
  if fScrollBar<>nil then
     ArrangeAcrollBarPosition;
end;

procedure TLabelScrollIntegerEdit.EditRangeValidationError(Sender: TObject);
begin
  inherited EditRangeValidationError(Sender);
  if fScrollBar<>nil then
     ArrangeAcrollBarPosition;
end;

procedure TLabelScrollIntegerEdit.EditScrollBarChanged(Sender: TObject);
begin
  if (fScrollBar<>nil) and
     not(fArrangingScrollBarPosition) then
     begin
     if not(fReverseScrolling) then
        SetValue(fScrollBar.Position+Self.Min)
     else SetValue(Self.Max-fScrollBar.Position);
     end;
  if Assigned(fOnChange) then fOnChange(Self);
end;

procedure TLabelScrollIntegerEdit.EditScrollBarEndScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: integer);
begin
  // if not(fAdjustingScrollBar) then begin
  if Assigned(fOnScroll) then fOnScroll(Self,ScrollCode,ScrollPos);
  if (ScrollCode=scEndScroll) and Assigned(fOnEndScroll) then fOnEndScroll(Self);
  // end
end;

constructor TLabelScrollIntegerEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fArrangingScrollBarPosition:=false;
  fReverseScrolling:=false;

  fScrollBar:=TScrollBar.Create(Self);
  with fScrollBar do begin
       Parent:=Self;
       Height:=CScrollBarWidth;
       ArrangeAllScrollBarValues;
       OnChange:=EditScrollBarChanged;
       OnScroll:=EditScrollBarEndScroll;
       end;

  fScrollBarLength:=CScrollBarLength;

  with fLCDimensions do begin
       LCWidth:=fScrollBarLength;
       LCHeight:=LCHeight+CHeightMargin+CScrollBarWidth;
       inherited SetBounds(0, 0, LCWidth, LCHeight);
       end;

  AutoLimit:=true;
  RangeValidation:=rvOnExit;
  LCStyle:=lcHorizontal;
end;

destructor TLabelScrollIntegerEdit.Free;
begin
  if fScrollBar<>nil then fScrollBar.Free;

  inherited Free;
end;

{ TLabelScrollRealEdit }

procedure TLabelScrollRealEdit.ArrangeAcrollBarPosition;
begin
  fArrangingScrollBarPosition:=true;
  if not(fReverseScrolling) then
     fScrollBar.Position:=System.Round((Self.Value-Self.Min)*fScrollMultiplier)
  else fScrollBar.Position:=System.Round((Self.Max-Self.Value)*fScrollMultiplier);
  fArrangingScrollBarPosition:=false;
end;

procedure TLabelScrollRealEdit.ArrangeAllScrollBarValues;
begin
  if fScrollBar<>nil then
     begin
     fScrollBar.Min:=0;
     fScrollBar.Max:=System.Round((Self.Max-Self.Min)*fScrollMultiplier);
     ArrangeAcrollBarPosition;
     end;
end;

procedure TLabelScrollRealEdit.SetMin(Value: real);
begin
  inherited SetMin(Value);
  ArrangeAllScrollBarValues;
end;

procedure TLabelScrollRealEdit.SetMax(Value: real);
begin
  inherited SetMax(Value);
  ArrangeAllScrollBarValues;
end;

procedure TLabelScrollRealEdit.SetValue(Value: real);
begin
  inherited SetValue(Value);
  ArrangeAcrollBarPosition;
end;

procedure TLabelScrollRealEdit.SetReverseScrolling(Value: boolean);
begin
  if fReverseScrolling<>Value then
     begin
     fReverseScrolling:=Value;
     ArrangeAllScrollBarValues;
     end;
end;

procedure TLabelScrollRealEdit.SetAutoHeight(NonAutoValue: integer);
begin
  if fAutoSize and (not(fNoLCStyleInfo) or (csDesigning in ComponentState)) then begin
     with fLCDimensions do
          case fLCStyle of
               lcHorizontal: begin
                             inherited SetAutoHeight(LCHeight-CHeightMargin -CScrollBarWidth);
                             LCHeight:=LCHeight+CHeightMargin+CScrollBarWidth;
                             end;
               lcVertical:   begin
                             inherited SetAutoHeight(NonAutoValue-CHeightMargin -fScrollBarLength);
                             LCHeight:=NonAutoValue;
                             end;
               end;
     end
  else fLCDimensions.LCHeight:=NonAutoValue;
end;

procedure TLabelScrollRealEdit.SwapLCWidthHeight;
begin
  if fNoLCStyleInfo or (csReading in ComponentState) then exit;

  inherited SwapLCWidthHeight;
  with fLCDimensions do
       case fLCStyle of
            lcHorizontal: begin
                          LCHeight:=LCHeight+CHeightMargin+CScrollBarWidth;
                          if fScrollBarLength>LCWidth then LCWidth:=fScrollBarLength;
                          end;
            lcVertical:   begin
                          LCHeight:=LCHeight+CHeightMargin+fScrollBarLength;
                          end;
            end;
end;

procedure TLabelScrollRealEdit.SetRealPrecision(Value: byte);
begin
  inherited SetRealPrecision(Value);
  fScrollMultiplier:=System.Round(Power(10,Value));
end;

procedure TLabelScrollRealEdit.AssortHorizontal;
var HeightOffset: integer;
begin
  if fNoLCStyleInfo and not(csDesigning in ComponentState) then exit;

  inherited AssortHorizontal;

  with fLCDimensions do begin
       HeightOffset:=(CHeightMargin+CScrollBarWidth) DIV 2;
       if HeightOffset<0 then HeightOffset:=0;
       if fLabel<>nil then fLabel.Top:=fLabel.Top-HeightOffset;
       if fUnit<>nil then fUnit.Top:=fLabel.Top;
       if fEdit<>nil then fEdit.Top:=fEdit.Top-HeightOffset;

       fScrollBarLength:=LCWidth;

       if fScrollBar<>nil then
          with fScrollBar do begin
               Kind:=sbHorizontal;
               Top:=LCHeight-CScrollBarWidth;
               Width:=fScrollBarLength;
               Height:=CScrollBarWidth;
               end;
       end;
end;

procedure TLabelScrollRealEdit.AssortVertical;
begin
  if fNoLCStyleInfo and not(csDesigning in ComponentState) then exit;

  inherited AssortVertical;

  with fLCDimensions do begin
       if fLabel<>nil then fLabel.Top:=CHeightMargin;
       if fUnit<>nil then fUnit.Top:=CHeightMargin;
       if fEdit<>nil then fEdit.Top:=LabelHeight+2*CHeightMargin;

       fScrollBarLength:=LCHeight-LabelHeight-fEditHeight-3*CHeightMargin;
       if fScrollBarLength<0 then fScrollBarLength:=0;

       if fScrollBar<>nil then
          with fScrollBar do begin
               Kind:=sbVertical;
               Left:=0;
               Top:=LCHeight-fScrollBarLength;
               Width:=CScrollBarWidth;
               Height:=fScrollBarLength;
               end;
       end;
end;

procedure TLabelScrollRealEdit.EditExit(Sender: TObject);
begin
  inherited;
  if fScrollBar<>nil then
     ArrangeAcrollBarPosition;
end;

procedure TLabelScrollRealEdit.EditRangeValidationError(Sender: TObject);
begin
  inherited EditRangeValidationError(Sender);
  if fScrollBar<>nil then
     ArrangeAcrollBarPosition;
end;

procedure TLabelScrollRealEdit.EditScrollBarChanged(Sender: TObject);
begin
  if (fScrollBar<>nil) and
     not(fArrangingScrollBarPosition) then
     begin
     if not(fReverseScrolling) then
        SetValue(fScrollBar.Position/fScrollMultiplier+Self.Min)
     else SetValue(Self.Max-fScrollBar.Position/fScrollMultiplier);
     end;
  if Assigned(fOnChange) then fOnChange(Self);
end;

procedure TLabelScrollRealEdit.EditScrollBarEndScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: integer);
begin
  // if not(fAdjustingScrollBar) then begin
  if Assigned(fOnScroll) then fOnScroll(Self,ScrollCode,ScrollPos);
  if (ScrollCode=scEndScroll) and Assigned(fOnEndScroll) then fOnEndScroll(Self);
  // end
end;

constructor TLabelScrollRealEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  fArrangingScrollBarPosition:=false;
  fReverseScrolling:=false;
  fScrollMultiplier:=System.Round(Power(10,Self.RealPrecision));

  fScrollBar:=TScrollBar.Create(Self);
  with fScrollBar do begin
       Parent:=Self;
       Height:=CScrollBarWidth;
       ArrangeAllScrollBarValues;
       OnChange:=EditScrollBarChanged;
       OnScroll:=EditScrollBarEndScroll;
       end;

  fScrollBarLength:=CScrollBarLength;

  with fLCDimensions do begin
       LCWidth:=fScrollBarLength;
       LCHeight:=LCHeight+CHeightMargin+CScrollBarWidth;
       inherited SetBounds(0, 0, LCWidth, LCHeight);
       end;

  AutoLimit:=true;
  RangeValidation:=rvOnExit;
  LCStyle:=lcHorizontal;
end;

destructor TLabelScrollRealEdit.Free;
begin
  if fScrollBar<>nil then fScrollBar.Free;

  inherited Free;
end;

    { TCustomLabelComboBox }

procedure TCustomLabelComboBox.SetAutoHeight(NonAutoValue: integer);
var EffectiveLabelHeight: integer;
begin
  if fAutoSize then begin
     fComboBoxHeight:=GetEditHeightFromFont(fComboBox.Font,Self);
     with fLCDimensions do begin
          EffectiveLabelHeight:=LabelHeight+2*CHeightMargin;
          case fLCStyle of
               lcHorizontal: if EffectiveLabelHeight>fComboBoxHeight then LCHeight:=EffectiveLabelHeight
                          else LCHeight:=fComboBoxHeight;
               lcVertical:   LCHeight:=EffectiveLabelHeight+fComboBoxHeight;
               end;
          end;
     end
  else fLCDimensions.LCHeight:=NonAutoValue;
end;

procedure TCustomLabelComboBox.SwapLCWidthHeight;
var LabelUnitWidth,EffectiveLabelHeight: integer;
begin
  with fLCDimensions do begin
       EffectiveLabelHeight:=LabelHeight+2*CHeightMargin;
       case fLCStyle of
            lcHorizontal: begin
                          LCWidth:=LabelWidth+fUnitWidth+fComboBoxWidth+2*CWidthMargin;
                          if fUnitWidth>0 then begin
                             if fUnitPosition=upLeft then LCWidth:=LCWidth+CWidthMargin
                             else LCWidth:=LCWidth+2*CWidthMargin;
                             end;
                          if EffectiveLabelHeight>fComboBoxHeight then LCHeight:=EffectiveLabelHeight
                          else LCHeight:=fComboBoxHeight;
                          end;
            lcVertical:   begin
                          LabelUnitWidth:=LabelWidth+fUnitWidth+2*CWidthMargin;
                          if fUnitWidth>0 then LabelUnitWidth:=LabelUnitWidth+CWidthMargin;
                          if LabelUnitWidth>fComboBoxWidth then LCWidth:=LabelUnitWidth
                          else LCWidth:=fComboBoxWidth;
                          LCHeight:=LabelHeight+fComboBoxHeight+2*CHeightMargin;
                          end;
           end;
      end;
end;

procedure TCustomLabelComboBox.AssortHorizontal;
begin
  inherited AssortHorizontal;

  with fLCDimensions do begin
       if fComboBox<>nil then begin
          fComboBox.Top:=(LCHeight - fComboBoxHeight) DIV 2;
          if (fUnitPosition=upLeft) and (fUnitWidth>0) then fComboBox.Left:=LabelWidth+fUnitWidth+3*CWidthMargin
          else fComboBox.Left:=LabelWidth+2*CWidthMargin;
          fComboBox.Height:=fComboBoxHeight;
          if (fUnitWidth>0) then begin
             if (fUnitPosition=upLeft) then fComboBoxWidth:=LCWidth-LabelWidth-fUnitWidth-3*CWidthMargin
             else fComboBoxWidth:=LCWidth-LabelWidth-fUnitWidth-4*CWidthMargin;
             end
          else fComboBoxWidth:=LCWidth-LabelWidth-2*CWidthMargin;
          fComboBox.Width:=fComboBoxWidth;
          end;
       end;
end;

procedure TCustomLabelComboBox.AssortVertical;
var HeightMargin: integer;
begin
  inherited AssortVertical;

  with fLCDimensions do begin
       HeightMargin:=(LCHeight-LabelHeight-fComboBoxHeight) DIV 2;
       if HeightMargin<0 then HeightMargin:=0;
       fComboBoxWidth:=LCWidth;

       if fLabel<>nil then fLabel.Top:=HeightMargin;
       if fUnit<>nil then fUnit.Top:=fLabel.Top;

       if fComboBox<>nil then begin
          fComboBox.Top:=LabelHeight+2*HeightMargin;
          fComboBox.Left:=0;
          fComboBox.Height:=fComboBoxHeight;
          fComboBox.Width:=fComboBoxWidth;
          end;
     end;
end;

constructor TCustomLabelComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

{  fAutoSelect:=true;
  fCharCase:=ecNormal;
  fEditAlignment:=taRightJustify;
  fMaxEditLength:=0;
  fHideSelection:=true;
  fOEMConvert:=false;
  fPasswordChar:=#0;
  fReadOnly:=false;}

  InsertComboBox;
  if fComboBox<>nil then
     with fComboBox do begin
          Parent:=Self;
          {Alignment:=fEditAlignment;
          AutoSelect:=fAutoSelect;
          CharCase:=fCharCase;
          HideSelection:=fHideSelection;
          OEMConvert:=fOEMConvert;
          PasswordChar:=fPasswordChar;
          ReadOnly:=fReadOnly;
          OnChange:=EditChanged;
          OnEnter:=EditEnter;
          OnExit:=EditExit;
          OnKeyDown:=EditKeyDown;
          OnKeyPress:=EditKeyPressed;
          OnKeyUp:=EditKeyUp;}
          end;

  fLabel.FocusControl:=fComboBox;

  fComboBoxHeight:=GetEditHeightFromFont(fComboBox.Font,Self);
  if fComboBoxHeight>(fLCDimensions.LabelHeight+2*CHeightMargin) then fLCDimensions.LCHeight:=fComboBoxHeight;
end;

destructor TCustomLabelComboBox.Free;
begin
  if fComboBox<>nil then fComboBox.Free;

  inherited Free;
end;

    { TLabelComboBox }

procedure TLabelComboBox.InsertComboBox;
begin
  fComboBox:=TComboBox.Create(Self);
end;

constructor TLabelComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  {SetEditText(CEdit);}
end;

procedure TLabelComboBox.Invalidate;
begin
  {if (csDesigning in ComponentState) and (Name<>'') and (fEdit<>nil) and (fEditString=CEdit) then SetEditText(CEdit+IntToStr(GetObjectNumber(Name)));}
  inherited;
end;

end.

