//*****************************************************************************
//*****************************************************************************
//*****************************************************************************
//**
//**  GROUPCONTROL.PAS // DERIVED FROM "MYRADIOGROUP.PAS" //
//**
//**  Collection of useful Group components
//**
//**  Autor : Georg  Hahn
//**  Datum :  xx.yy.zz / 11.03.04 / 12.01.10
//**
//*****************************************************************************
//*****************************************************************************
//*****************************************************************************
unit GroupControls;

interface

uses
  Windows, Messages, Types, Classes, Controls, StdCtrls;

type
  TCustomCheckGroup = class(TCustomGroupBox)
  private
    { Private declarations }
    fCheckers: TList;
    fItems: TStrings;
    fColumns: integer;
    fGreyDisabled: boolean;
    fReading: boolean;
    fUpdating: boolean;
    fValue: integer;
    procedure ArrangeCheckers;
    procedure CheckerClick(Sender: TObject);
    // procedure ItemsChange(Sender: TObject);
    procedure SetCheckerCount(Value: integer);
    procedure SetColumns(Value: integer);
    procedure SetGreyDisabled(Value: boolean);
    procedure SetValue(Value: integer);
    procedure SetItems(Value: TStrings);
    procedure UpdateCheckers;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    { Protected declarations }
    procedure ReadState(Reader: TReader); override;
    function CanModify: boolean; virtual;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    property Columns: integer read fColumns write SetColumns default 1;
    property GreyDisabled: boolean read fGreyDisabled write SetGreyDisabled default false;
    property Value: integer read fValue write SetValue default 0;
    property Items: TStrings read fItems write SetItems;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure FlipChildren(AllLevels: boolean); override;
  published
    { Published declarations }
  end;

  TGroupChecker = class(TCheckBox)
  private
    fInClick: boolean;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
  public
    constructor InternalCreate(CheckGroup: TCustomCheckGroup);
    destructor Destroy; override;
  end;

  TCheckGroup = class(TCustomCheckGroup)
  published
    { Published declarations }
    property Align;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property GreyDisabled;
    property Items;
    property Constraints;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Value;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDock;
    property OnStartDrag;
  end;

  TMyCustomRadioGroup = class(TCustomGroupBox)
  private
    FButtons: TList;
    FItems: TStrings;
    FItemIndex: Integer;
    FColumns: Integer;
    FReading: Boolean;
    FUpdating: Boolean;
    procedure ArrangeButtons;
    procedure ButtonClick(Sender: TObject);
    procedure ItemsChange(Sender: TObject);
    procedure SetButtonCount(Value: Integer); virtual;
    procedure SetColumns(Value: Integer);
    procedure SetItemIndex(Value: Integer);
    procedure SetItems(Value: TStrings);
    procedure UpdateButtons;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    procedure ReadState(Reader: TReader); override;
    function CanModify: Boolean; virtual;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    property Columns: Integer read FColumns write SetColumns default 1;
    property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
    property Items: TStrings read FItems write SetItems;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure FlipChildren(AllLevels: Boolean); override;
  end;

  TBoolArray = array [0..255] of boolean;

  TMyRadioGroup = class(TMyCustomRadioGroup)
  private
    //FButtonEnable: TBoolArray;
    fButtonEnable: TBooleanDynArray;
    procedure SetAllButtonsEnable(Value: boolean);
    procedure SetButtonCount(Value: Integer); override;
    procedure SetButtonEnable(Index: byte; Value: boolean);
  public
    constructor Create(AOwner: TComponent); override;

    procedure EnableAllButtons;
    procedure DisableNotCheckedButtons;
    procedure SetButtonEnableArray(Source: TBooleanDynArray);
    property ButtonEnable[Index: byte]: boolean write SetButtonEnable;
  published
    property Align;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ItemIndex;
    property Items;
    property Constraints;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

uses Forms;

procedure Register;
begin
  RegisterComponents('EditControls', [TCheckGroup]);
  RegisterComponents('EditControls', [TMyRadioGroup]);
end;

// TGroupChecker

constructor TGroupChecker.InternalCreate(CheckGroup: TCustomCheckGroup);
begin
  inherited Create(CheckGroup);
  CheckGroup.fCheckers.Add(Self);
  Visible := False;
  Enabled := CheckGroup.Enabled;
  ParentShowHint := False;
  OnClick := CheckGroup.CheckerClick;
  Parent := CheckGroup;
end;

destructor TGroupChecker.Destroy;
begin
  TCustomCheckGroup(Owner).fCheckers.Remove(Self);
  inherited Destroy;
end;

procedure TGroupChecker.CNCommand(var Message: TWMCommand);
begin
  if not fInClick then
     begin
     fInClick := true;
     try
       if ((Message.NotifyCode = BN_CLICKED) or
          (Message.NotifyCode = BN_DOUBLECLICKED)) and
          TCustomCheckGroup(Parent).CanModify then inherited;
     except
       Application.HandleException(Self);
     end;
     fInClick := False;
     end;
end;

procedure TGroupChecker.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  TCustomCheckGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then
     begin
     if not TCustomCheckGroup(Parent).CanModify then Key := #0;
     end;
end;

procedure TGroupChecker.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  TCustomCheckGroup(Parent).KeyDown(Key, Shift);
end;

// TCustomCheckGroup

constructor TCustomCheckGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csSetCaption, csDoubleClicks];
  fCheckers := TList.Create;
  fItems := TStringList.Create;
  // TStringList(fItems).OnChange := ItemsChange;
  fColumns := 1;
  fGreyDisabled := false;
  fValue := 0;
end;

destructor TCustomCheckGroup.Destroy;
begin
  SetCheckerCount(0);
  TStringList(fItems).OnChange := nil;
  fItems.Free;
  fCheckers.Free;
  inherited Destroy;
end;

procedure TCustomCheckGroup.FlipChildren(AllLevels: boolean); 
begin
  // The radio buttons are flipped using BiDiMode
end;

procedure TCustomCheckGroup.ArrangeCheckers;
var ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: integer;
    DC: HDC;
    SaveFont: HFont;
    Metrics: TTextMetric;
    DeferHandle: THandle;
    ALeft: integer;
begin
  if (fCheckers.Count <> 0) and not fReading then
     begin
     DC := GetDC(0);
     SaveFont := SelectObject(DC, Font.Handle);
     GetTextMetrics(DC, Metrics);
     SelectObject(DC, SaveFont);
     ReleaseDC(0, DC);
     ButtonsPerCol := (fCheckers.Count + fColumns - 1) div fColumns;
     ButtonWidth := (Width - 10) div fColumns;
     I := Height - Metrics.tmHeight - 5;
     ButtonHeight := I div ButtonsPerCol;
     TopMargin := Metrics.tmHeight + 1 + (I mod ButtonsPerCol) div 2;
     DeferHandle := BeginDeferWindowPos(fCheckers.Count);
     try
       for I := 0 to fCheckers.Count - 1 do
           with TGroupChecker(fCheckers[I]) do
                begin
                BiDiMode := Self.BiDiMode;
                ALeft := (I div ButtonsPerCol) * ButtonWidth + 8;
                if UseRightToLeftAlignment then
                   ALeft := Self.ClientWidth - ALeft - ButtonWidth;
                DeferHandle := DeferWindowPos(DeferHandle, Handle, 0,
                               ALeft,
                               (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
                               ButtonWidth, ButtonHeight,
                               SWP_NOZORDER or SWP_NOACTIVATE);
                Visible := true;
           end;
     finally
       EndDeferWindowPos(DeferHandle);
     end;
     end;
end;

procedure TCustomCheckGroup.CheckerClick(Sender: TObject);
var I: integer;
begin
  if not fUpdating then
     begin
     I := fCheckers.IndexOf(Sender);
     if TGroupChecker(fCheckers[I]).Checked then fValue := fValue or (1 shl I)
     else fValue := fValue and not(1 shl I);
     Changed;
     Click;
     end;
end;

{procedure TCustomCheckGroup.ItemsChange(Sender: TObject);
begin
  if not fReading then
  begin
    if fItemIndex >= fItems.Count then fItemIndex := fItems.Count - 1;
    UpdateCheckers;
  end;
end;}

procedure TCustomCheckGroup.ReadState(Reader: TReader);
begin
  fReading := true;
  inherited ReadState(Reader);
  fReading := False;
  UpdateCheckers;
end;

procedure TCustomCheckGroup.SetCheckerCount(Value: integer);
begin
  while fCheckers.Count < Value do TGroupChecker.InternalCreate(Self);
  while fCheckers.Count > Value do TGroupChecker(fCheckers.Last).Free;
end;

procedure TCustomCheckGroup.SetColumns(Value: integer);
begin
  if Value < 1 then Value := 1;
  if Value > 16 then Value := 16;
  if fColumns <> Value then
     begin
     fColumns := Value;
     ArrangeCheckers;
     Invalidate;
     end;
end;

procedure TCustomCheckGroup.SetGreyDisabled(Value: boolean);
var I: integer;
begin
  if (fGreyDisabled<>Value) then
     begin
     fGreyDisabled:=Value;
     for I := 0 to fCheckers.Count - 1 do
         if not(fGreyDisabled) then
            TGroupChecker(fCheckers[I]).Enabled := Enabled
         else TGroupChecker(fCheckers[I]).Enabled :=true;
     end;
end;

procedure TCustomCheckGroup.SetValue(Value: integer);
var I: integer;
begin
  if fReading then fValue := Value else
  begin
    if Value < -1 then Value := -1;
    if Value > (1 shl fCheckers.Count) - 1 then Value := (1 shl fCheckers.Count) - 1;

    for I := 0 to fCheckers.Count-1 do
        begin
        fUpdating := true;
        TGroupChecker(fCheckers[I]).Checked := (Value and (1 shl I) > 0);
        fUpdating := False;
        end;
        
    fValue:=Value;
  end;
end;

procedure TCustomCheckGroup.SetItems(Value: TStrings);
begin
  fItems.Assign(Value);
end;

procedure TCustomCheckGroup.UpdateCheckers;
var I: integer;
begin
  SetCheckerCount(fItems.Count);
  for I := 0 to fCheckers.Count - 1 do
      begin
      TGroupChecker(fCheckers[I]).Caption := fItems[I];
      fUpdating := true;
      TGroupChecker(fCheckers[I]).Checked := (Value and (1 shl I) > 0);
      fUpdating := False;
      end;
  ArrangeCheckers;
  Invalidate;
end;

procedure TCustomCheckGroup.CMEnabledChanged(var Message: TMessage);
var I: integer;
begin
  inherited;
  for I := 0 to fCheckers.Count - 1 do
      if not(fGreyDisabled) then
         TGroupChecker(fCheckers[I]).Enabled := Enabled
      else TGroupChecker(fCheckers[I]).Enabled :=true;
end;

procedure TCustomCheckGroup.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ArrangeCheckers;
end;

procedure TCustomCheckGroup.WMSize(var Message: TWMSize);
begin
  inherited;
  ArrangeCheckers;
end;

function TCustomCheckGroup.CanModify: boolean;
begin
  Result := true;
end;

procedure TCustomCheckGroup.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

// TMyGroupButton

type
  TMyGroupButton = class(TRadioButton)
  private
    FInClick: Boolean;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
  public
    constructor InternalCreate(RadioGroup: TMyCustomRadioGroup);
    destructor Destroy; override;
  end;

constructor TMyGroupButton.InternalCreate(RadioGroup: TMyCustomRadioGroup);
begin
  inherited Create(RadioGroup);
  RadioGroup.FButtons.Add(Self);
  Visible := False;
  Enabled := RadioGroup.Enabled;
  ParentShowHint := False;
  OnClick := RadioGroup.ButtonClick;
  Parent := RadioGroup;
end;

destructor TMyGroupButton.Destroy;
begin
  TMyCustomRadioGroup(Owner).FButtons.Remove(Self);
  inherited Destroy;
end;

procedure TMyGroupButton.CNCommand(var Message: TWMCommand);
begin
  if not FInClick then
     begin
       FInClick := True;
       try
         if ((Message.NotifyCode = BN_CLICKED) or
           (Message.NotifyCode = BN_DOUBLECLICKED)) and
           TMyCustomRadioGroup(Parent).CanModify then
           inherited;
       except
         Application.HandleException(Self);
       end;
     FInClick := False;
     end;
end;

procedure TMyGroupButton.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  TMyCustomRadioGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then
     begin
     if not TMyCustomRadioGroup(Parent).CanModify then Key := #0;
     end;
end;

procedure TMyGroupButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  TMyCustomRadioGroup(Parent).KeyDown(Key, Shift);
end;

// TMyCustomRadioGroup

constructor TMyCustomRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csSetCaption, csDoubleClicks];
  FButtons := TList.Create;
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChange;
  FItemIndex := -1;
  FColumns := 1;
end;

destructor TMyCustomRadioGroup.Destroy;
begin
  SetButtonCount(0);
  TStringList(FItems).OnChange := nil;
  FItems.Free;
  FButtons.Free;
  inherited Destroy;
end;

procedure TMyCustomRadioGroup.FlipChildren(AllLevels: Boolean);
begin
  // The radio buttons are flipped using BiDiMode
end;

procedure TMyCustomRadioGroup.ArrangeButtons;
var
  ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: Integer;
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
  DeferHandle: THandle;
  ALeft: Integer;
begin
  if (FButtons.Count <> 0) and not FReading then
     begin
     DC := GetDC(0);
     SaveFont := SelectObject(DC, Font.Handle);
     GetTextMetrics(DC, Metrics);
     SelectObject(DC, SaveFont);
     ReleaseDC(0, DC);
     ButtonsPerCol := (FButtons.Count + FColumns - 1) div FColumns;
     ButtonWidth := (Width - 10) div FColumns;
     I := Height - Metrics.tmHeight - 5;
     ButtonHeight := I div ButtonsPerCol;
     TopMargin := Metrics.tmHeight + 1 + (I mod ButtonsPerCol) div 2;
     DeferHandle := BeginDeferWindowPos(FButtons.Count);
     try
       for I := 0 to FButtons.Count - 1 do
           with TMyGroupButton(FButtons[I]) do
           begin
             BiDiMode := Self.BiDiMode;
             ALeft := (I div ButtonsPerCol) * ButtonWidth + 8;
             if UseRightToLeftAlignment then
               ALeft := Self.ClientWidth - ALeft - ButtonWidth;
             DeferHandle := DeferWindowPos(DeferHandle, Handle, 0,
                            ALeft,
                            (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
                            ButtonWidth, ButtonHeight,
                            SWP_NOZORDER or SWP_NOACTIVATE);
             Visible := True;
             end;
     finally
       EndDeferWindowPos(DeferHandle);
       end;
     end;
end;

procedure TMyCustomRadioGroup.ButtonClick(Sender: TObject);
begin
  if not FUpdating then
     begin
     FItemIndex := FButtons.IndexOf(Sender);
     Changed;
     Click;
     end;
end;

procedure TMyCustomRadioGroup.ItemsChange(Sender: TObject);
begin
  if not FReading then
     begin
     if FItemIndex >= FItems.Count then FItemIndex := FItems.Count - 1;
     UpdateButtons;
     end;
end;

procedure TMyCustomRadioGroup.ReadState(Reader: TReader);
begin
  FReading := True;
  inherited ReadState(Reader);
  FReading := False;
  UpdateButtons;
end;

procedure TMyCustomRadioGroup.SetButtonCount(Value: Integer);
begin
  while FButtons.Count < Value do TMyGroupButton.InternalCreate(Self);
  while FButtons.Count > Value do TMyGroupButton(FButtons.Last).Free;
end;

procedure TMyCustomRadioGroup.SetColumns(Value: Integer);
begin
  if Value < 1 then Value := 1;
  if Value > 16 then Value := 16;
  if FColumns <> Value then
     begin
     FColumns := Value;
     ArrangeButtons;
     Invalidate;
     end;
end;

procedure TMyCustomRadioGroup.SetItemIndex(Value: Integer);
begin
  if FReading then FItemIndex := Value else
     begin
     if Value < -1 then Value := -1;
     if Value >= FButtons.Count then Value := FButtons.Count - 1;
     if FItemIndex <> Value then
        begin
        if FItemIndex >= 0 then
           TMyGroupButton(FButtons[FItemIndex]).Checked := False;
        FItemIndex := Value;
        if FItemIndex >= 0 then
           TMyGroupButton(FButtons[FItemIndex]).Checked := True;
        end;
     end;
end;

procedure TMyCustomRadioGroup.SetItems(Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TMyCustomRadioGroup.UpdateButtons;
var
  I: Integer;
begin
  SetButtonCount(FItems.Count);
  for I := 0 to FButtons.Count - 1 do
      TMyGroupButton(FButtons[I]).Caption := FItems[I];
  if FItemIndex >= 0 then
     begin
     FUpdating := True;
     TMyGroupButton(FButtons[FItemIndex]).Checked := True;
     FUpdating := False;
     end;
  ArrangeButtons;
  Invalidate;
end;

procedure TMyCustomRadioGroup.CMEnabledChanged(var Message: TMessage);
var
  I: Integer;
begin
  inherited;
  for I := 0 to FButtons.Count - 1 do
      TMyGroupButton(FButtons[I]).Enabled := Enabled;
end;

procedure TMyCustomRadioGroup.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ArrangeButtons;
end;

procedure TMyCustomRadioGroup.WMSize(var Message: TWMSize);
begin
  inherited;
  ArrangeButtons;
end;

function TMyCustomRadioGroup.CanModify: Boolean;
begin
  Result := True;
end;

procedure TMyCustomRadioGroup.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

// TMyRadioGroup

procedure TMyRadioGroup.SetAllButtonsEnable(Value: boolean);
var I: integer;
begin
  for I:=0 to FButtons.Count-1 do
      begin
      fButtonEnable[I]:=Value;
      TMyGroupButton(FButtons[I]).Enabled:=Value;
      end;
end;

procedure TMyRadioGroup.SetButtonCount(Value: Integer);
begin
  inherited SetButtonCount(Value);
  SetLength(fButtonEnable,Value);
  SetAllButtonsEnable(true);
end;

procedure TMyRadioGroup.SetButtonEnable(Index: byte; Value: boolean);
begin
  if (Index<FButtons.Count) then
     begin
     fButtonEnable[Index]:=Value;
     TMyGroupButton(FButtons[Index]).Enabled := Value;
     end;
end;

constructor TMyRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetLength(fButtonEnable,0);
end;

procedure TMyRadioGroup.EnableAllButtons;
begin
  SetAllButtonsEnable(true);
end;

procedure TMyRadioGroup.DisableNotCheckedButtons;
var I: integer;
begin
  for I:=0 to FButtons.Count-1 do
      begin
      if not(TMyGroupButton(FButtons[I]).Checked) then
         begin
         fButtonEnable[I]:=false;
         TMyGroupButton(FButtons[I]).Enabled:=false;
         end;
      end;
end;

procedure TMyRadioGroup.SetButtonEnableArray(Source: TBooleanDynArray);
var I: integer;
begin
  for I:=0 to FButtons.Count-1 do
      if not(System.Length(Source)<FButtons.Count) then
         SetButtonEnable(I,Source[I]);
end;

end.
