//******************************************************************************
//******************************************************************************
//******************************************************************************
//**
//**  LnCRCFileStream.PAS
//**
//**  TFileStream with additional helpful propertes and methodes
//**
//**  Autor : Georg  Hahn
//**  Datum :  xx.yy.96 / 30.11.06 / 30.05.07 / 23.10.08 / 22.10.09 / 19.01.10
//**           25.06.13
//******************************************************************************
//******************************************************************************
//******************************************************************************
unit LnCRCFileStream;

interface

uses Classes, SysUtils, Types, ArraysAndGrids;

const
  CMaxStringBufferLength   = 100;
  CMaxCRCalcBufferLength   = 1000;

type

// TLnCRCFileStream class

  TLnCRCFileStream = class(TFileStream)
  private
    fBuffer: array [0..CMaxStringBufferLength] of char;
    fAString: AnsiString;
    fLnPosition: Longint;
    fOnReadStringArray: TNotifyEvent;
    function CalculateCRC16: word;
    function CalculateParityByte: byte;

    function ReadStreamFileToStringDynArray: TStringDynArray;

    function ReadStreamFileLineToStringDynArray(SepChar: char): TStringDynArray; overload;
    function ReadStreamFileLineToStringDynArray(SepChar: char; SuppressBlanks: boolean): TStringDynArray; overload;

    function ReadStreamFileLineToStringDynArray(SepChars: TSysCharSet): TStringDynArray; overload;
    function ReadStreamFileLineToStringDynArray(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynArray; overload;

    function ReadStreamFileToStringDynGrid(SepChar: char): TStringDynGrid; overload;
    function ReadStreamFileToStringDynGrid(SepChar: char; SuppressBlanks: boolean): TStringDynGrid; overload;

    function ReadStreamFileToStringDynGrid(SepChars: TSysCharSet): TStringDynGrid; overload;
    function ReadStreamFileToStringDynGrid(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynGrid; overload;

    procedure WriteStringDynArrayToStreamFile(Value: TStringDynArray; SepChar: char);
    procedure WriteStringDynGridToStreamFile(Value: TStringDynGrid; SepChar: char);
  public
    constructor Create(const FileName: string; Mode: Word);

    function Read(var Buffer; Count: Longint): Longint; override;
    function ReadStringCount(Count: Longint): AnsiString;
    function ReadStringLn: AnsiString;
    function ReadStringUntilChar(Value: char): AnsiString;
    function ReadStringUntilChars(Value: TSysCharSet): AnsiString;

    function ReadStringDynArrayLn: TStringDynArray; overload;
    function ReadStringDynArrayLn(SepChar: char): TStringDynArray; overload;
    function ReadStringDynArrayLn(SepChars: TSysCharSet): TStringDynArray; overload;
    function ReadStringDynArrayLn(SepChar: char; SuppressBlanks: boolean): TStringDynArray; overload;
    function ReadStringDynArrayLn(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynArray; overload;

    function ReadStringDynGridLn: TStringDynGrid; overload;
    function ReadStringDynGridLn(SepChar: char): TStringDynGrid; overload;
    function ReadStringDynGridLn(SepChars: TSysCharSet): TStringDynGrid; overload;
    function ReadStringDynGridLn(SepChar: char; SuppressBlanks: boolean): TStringDynGrid; overload;
    function ReadStringDynGridLn(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynGrid; overload;

    function SeekUntilString(Value: string): boolean;
    procedure WriteString(Value: AnsiString);
    procedure WriteStringLn;  overload;
    procedure WriteStringLn(Value: AnsiString);  overload;
    procedure WriteStringDynArrayLn(Value: TStringDynArray); overload;
    procedure WriteStringDynArrayLn(Value: TStringDynArray; SepChar: char); overload;
    procedure WriteStringDynGridLn(Value: TStringDynGrid); overload;
    procedure WriteStringDynGridLn(Value: TStringDynGrid; SepChar: char); overload;
    procedure ClearBuffer;

    function ReadFileToStringDynArrayLn: TStringDynArray;
    procedure WriteStringDynArrayLnToFile(Source: TStringDynArray);

    function Seek(Offset: Longint; Origin: Word): Longint; override;
    function Eof: boolean;

    function GetCRC16: word;
    procedure AppendCRC16;

    function GetParityByte: byte;
    procedure AppendParityByte;

    property LnPosition: Longint read fLnPosition;
    property OnReadStringArray: TNotifyEvent read fOnReadStringArray write fOnReadStringArray;
  end;

  function ReadStreamFileToStringDynArray(Source: TFileName): TStringDynArray;

  function ReadStreamFileToStringDynGrid(Source: TFileName): TStringDynGrid; overload;
  function ReadStreamFileToStringDynGrid(Source: TFileName; SepChar: char): TStringDynGrid; overload;
  function ReadStreamFileToStringDynGrid(Source: TFileName; SepChars: TSysCharSet): TStringDynGrid; overload;

  procedure WriteStringDynArrayToFile(Source: TStringDynArray; Target: TFileName);
  procedure WriteStringDynGridToFile(Source: TStringDynGrid; Target: TFileName);

  function GetStreamCRC16Integrity(Source: TFileName): boolean;

implementation


uses StringUtils, ConversionUtils;

function ReadStreamFileToStringDynArray(Source: TFileName): TStringDynArray;
var SourceStream: TLnCRCFileStream;
begin
  SetLength(Result,0);
  if FileExists(Source) then
     begin
     SourceStream:=TLnCRCFileStream.Create(Source,fmOpenRead);
     Result:=SourceStream.ReadFileToStringDynArrayLn;
     SourceStream.Free;
     end;
end;

function ReadStreamFileToStringDynGrid(Source: TFileName): TStringDynGrid;
var SourceStream: TLnCRCFileStream;
begin
  SetLength(Result,0);
  if FileExists(Source) then
     begin
     SourceStream:=TLnCRCFileStream.Create(Source,fmOpenRead);
     Result:=SourceStream.ReadStringDynGridLn;
     SourceStream.Free;
     end;
end;

function ReadStreamFileToStringDynGrid(Source: TFileName; SepChar: char): TStringDynGrid;
var SourceStream: TLnCRCFileStream;
begin
  SetLength(Result,0);
  if FileExists(Source) then
     begin
     SourceStream:=TLnCRCFileStream.Create(Source,fmOpenRead);
     Result:=SourceStream.ReadStringDynGridLn(SepChar);
     SourceStream.Free;
     end;
end;

function ReadStreamFileToStringDynGrid(Source: TFileName; SepChars: TSysCharSet): TStringDynGrid;
var SourceStream: TLnCRCFileStream;
begin
  SetLength(Result,0);
  if FileExists(Source) then
     begin
     SourceStream:=TLnCRCFileStream.Create(Source,fmOpenRead);
     Result:=SourceStream.ReadStringDynGridLn(SepChars);
     SourceStream.Free;
     end;
end;

procedure WriteStringDynArrayToFile(Source: TStringDynArray; Target: TFileName);
var TargetStream: TLnCRCFileStream;
begin
  if (System.Length(Source)>0) then
     begin
     TargetStream:=TLnCRCFileStream.Create(Target,fmCreate);
     TargetStream.WriteStringDynArrayLnToFile(Source);
     TargetStream.Free;
     end;
end;

procedure WriteStringDynGridToFile(Source: TStringDynGrid; Target: TFileName);
var TargetStream: TLnCRCFileStream;
    {I,} L: integer;
begin
  L:=System.Length(Source);
  if (L>0) then
     begin
     TargetStream:=TLnCRCFileStream.Create(Target,fmCreate);
     TargetStream.WriteStringDynGridLn(Source,',');
     {for I:=0 to L-1 do
         TargetStream.WriteStringDynArrayLnToFile(Source[I]);}
     TargetStream.Free;
     end;
end;

function GetStreamCRC16Integrity(Source: TFileName): boolean;
var SourceStream: TLnCRCFileStream;
begin
  if FileExists(Source) then
     begin
     SourceStream:=TLnCRCFileStream.Create(Source,fmOpenRead);
     Result:=(SourceStream.GetCRC16=0);
     SourceStream.Free;
     end
  else Result:=false;
end;

// TLnCRCFileStream

function TLnCRCFileStream.CalculateCRC16: word;
var fFileBuffer: array [0..CMaxCRCalcBufferLength] of byte;
    lCRC16: word;
    I,N: integer;
begin
  inherited Seek(0,soFromBeginning);
  lCRC16:=0; N:=CMaxCRCalcBufferLength;

  while (Position<Size) do
        begin
        if (Size-Position)<CMaxCRCalcBufferLength then
           N:=Size-Position;
        inherited Read(fFileBuffer,N);
        for I:=0 to N-1 do
            lCRC16 := Crc_16_tab[((lCRC16 shr 8) and $FF)] xor (lCRC16 shl 8) xor fFileBuffer[I];
        end;

  lCRC16 := Crc_16_tab[((lCRC16 shr 8) and $FF)] xor (lCRC16 shl 8) xor 0;
  lCRC16 := Crc_16_tab[((lCRC16 shr 8) and $FF)] xor (lCRC16 shl 8) xor 0;

  Result:=lCRC16;
end;

function TLnCRCFileStream.CalculateParityByte: byte;
var fFileBuffer: array [0..CMaxCRCalcBufferLength] of byte;
    lParity: byte;
    I,N: integer;
begin
  inherited Seek(0,soFromBeginning);
  lParity:=0; N:=CMaxCRCalcBufferLength;

  while (Position<Size) do
        begin
        if (Size-Position)<CMaxCRCalcBufferLength then
           N:=Size-Position;
        inherited Read(fFileBuffer,N);
        for I:=0 to N-1 do
            lParity := lParity xor fFileBuffer[I];
        end;

  Result:=lParity;
end;

function TLnCRCFileStream.ReadStreamFileToStringDynArray: TStringDynArray;
var Target: TStringDynArray;
    InSA: AnsiString;
begin
  if not(Eof) then
     begin
     SetLength(Target,0);
     repeat
       InSA:=ReadStringLn;
       AppendElementToArray(InSA,Target);
     until (Eof);
     Result:=Target;
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileLineToStringDynArray(SepChar: char): TStringDynArray;
var Target: TStringDynArray;
    InSA: AnsiString;
    I,P: integer;
begin
  InSA:=ReadStringLn;
  if (InSA<>'') then
     begin
     SetLength(Target,1); I:=0;
     repeat
       P:=System.Pos(SepChar,InSA);
       if (P>0) then
          begin
          Target[I]:=System.Copy(InSA,1,P-1);
          System.Inc(I);
          SetLength(Target,I+1);
          end
       else Target[I]:=InSA;
       System.Delete(InSA,1,P);
     until (P=0);
     Result:=Target;
     if Assigned(fOnReadStringArray) then fOnReadStringArray(Self);
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileLineToStringDynArray(SepChar: char; SuppressBlanks: boolean): TStringDynArray;
var Target: TStringDynArray;
    InSA, TAS: AnsiString;
    I,P: integer;
  procedure SortStringToArray;
  begin
    if (TAS<>'') or not(SuppressBlanks) then
       begin
       System.Inc(I);
       SetLength(Target,I);
       Target[I-1]:=TAS;
       end;
  end;
begin
  InSA:=ReadStringLn;
  if (InSA<>'') then
     begin
     I:=0; SetLength(Target,I);
     repeat
       P:=System.Pos(SepChar,InSA);
       if (P>0) then
          begin
          TAS:=System.Copy(InSA,1,P-1);
          SortStringToArray;
          end
       else
          begin
          TAS:=InSA;
          SortStringToArray;
          end;
       System.Delete(InSA,1,P);
     until (P=0);
     Result:=Target;
     if Assigned(fOnReadStringArray) then fOnReadStringArray(Self);
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileLineToStringDynArray(SepChars: TSysCharSet): TStringDynArray;
var Target: TStringDynArray;
    InSA: AnsiString;
    I,P: integer;
begin
  InSA:=ReadStringLn;
  if (InSA<>'') then
     begin
     SetLength(Target,1); I:=0;
     repeat
       P:=GetSysCharSetPosition(SepChars,InSA);
       if (P>0) then
          begin
          Target[I]:=System.Copy(InSA,1,P-1);
          System.Inc(I);
          SetLength(Target,I+1);
          end
       else Target[I]:=InSA;
       System.Delete(InSA,1,P);
     until (P=0);
     Result:=Target;
     if Assigned(fOnReadStringArray) then fOnReadStringArray(Self);
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileLineToStringDynArray(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynArray;
var Target: TStringDynArray;
    InSA, TAS: AnsiString;
    I,P: integer;
  procedure SortStringToArray;
  begin
    if (TAS<>'') or not(SuppressBlanks) then
       begin
       System.Inc(I);
       SetLength(Target,I);
       Target[I-1]:=TAS;
       end;
  end;
begin
  InSA:=ReadStringLn;
  if (InSA<>'') then
      begin
      I:=0; SetLength(Target,I);
     repeat
       P:=GetSysCharSetPosition(SepChars,InSA);
       if (P>0) then
          begin
          TAS:=System.Copy(InSA,1,P-1);
          SortStringToArray;
          end
       else
          begin
          TAS:=InSA;
          SortStringToArray;
          end;
       System.Delete(InSA,1,P);
     until (P=0);
     Result:=Target;
     if Assigned(fOnReadStringArray) then fOnReadStringArray(Self);
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileToStringDynGrid(SepChar: char): TStringDynGrid;
var Target: TStringDynGrid;
    SA: TStringDynArray;
    I: integer;
begin
  if not(Eof) then
     begin
     I:=0; SetLength(SA,0);
     repeat
       SA:=ReadStreamFileLineToStringDynArray(SepChar);
       if (SA<>nil) then
          begin
          SetLength(Target,I+1);
          Target[I]:=SA;
          System.Inc(I);
          end;
     until (Eof);
     Result:=Target;
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileToStringDynGrid(SepChar: char; SuppressBlanks: boolean): TStringDynGrid;
var Target: TStringDynGrid;
    SA: TStringDynArray;
    I: integer;
begin
  if not(Eof) then
     begin
     I:=0; SetLength(SA,0);
     repeat
       SA:=ReadStreamFileLineToStringDynArray(SepChar, SuppressBlanks);
       if (SA<>nil) then
          begin
          SetLength(Target,I+1);
          Target[I]:=SA;
          System.Inc(I);
          end;
     until (Eof);
     Result:=Target;
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileToStringDynGrid(SepChars: TSysCharSet): TStringDynGrid;
var Target: TStringDynGrid;
    SA: TStringDynArray;
    I: integer;
begin
  if not(Eof) then
     begin
     I:=0; SetLength(SA,0);
     repeat
       SA:=ReadStreamFileLineToStringDynArray(SepChars);
       if (SA<>nil) then
          begin
          SetLength(Target,I+1);
          Target[I]:=SA;
          System.Inc(I);
          end;
     until (Eof);
     Result:=Target;
     end
  else Result:=nil;
end;

function TLnCRCFileStream.ReadStreamFileToStringDynGrid(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynGrid;
var Target: TStringDynGrid;
    SA: TStringDynArray;
    I: integer;
begin
  if not(Eof) then
     begin
     I:=0; SetLength(SA,0);
     repeat
       SA:=ReadStreamFileLineToStringDynArray(SepChars, SuppressBlanks);
       if (SA<>nil) then
          begin
          SetLength(Target,I+1);
          Target[I]:=SA;
          System.Inc(I);
          end;
     until (Eof);
     Result:=Target;
     end
  else Result:=nil;
end;

procedure TLnCRCFileStream.WriteStringDynArrayToStreamFile(Value: TStringDynArray; SepChar: char);
var L, I: integer;
    S: AnsiString;
begin
  L:=System.Length(Value);
  if (L>0) then
     begin
     S:=Value[0];
     for I:=1 to L-1 do S:=S+SepChar+Value[I];
     WriteString(S+CCR+CLF);
     end;
end;

procedure TLnCRCFileStream.WriteStringDynGridToStreamFile(Value: TStringDynGrid; SepChar: char);
var R, C, I, J: integer;
    S: AnsiString;
begin
  R:=System.Length(Value);
  if (R>0) then
     begin
     for I:=0 to R-1 do
         begin
         C:=System.Length(Value[I]);
         if C>0 then S:=Value[I,0]
         else S:='';
         if C>1 then for J:=1 to C-1 do S:=S+SepChar+Value[I,J];
         WriteString(S+CCR+CLF);
         end;

     {C:=System.Length(Value[0]);
     for J:=0 to C-1 do
         begin
         S:=Value[J,0];
         for I:=1 to R-1 do S:=S+SepChar+Value[J,I];
         WriteString(S+sCR+sLF);
         end;}

     end;
end;

constructor TLnCRCFileStream.Create(const FileName: string; Mode: Word);
begin
  inherited Create(FileName,Mode);
  fAString:=''; fLnPosition:=0;
end;

function TLnCRCFileStream.Read(var Buffer; Count: Longint): Longint;
begin
  Result := inherited Read(Buffer,Count);
  fLnPosition:=0;
end;

function TLnCRCFileStream.ReadStringCount(Count: Longint): AnsiString;
var EmptyOldRead: boolean;
    InSA: AnsiString;
    I,L: Longint;
begin
  EmptyOldRead:=(fAString='');
  if (Position<Size) or not(EmptyOldRead) then
     begin
     InSA:=''; fLnPosition:=0;
     L:=0; I:=System.Length(fAString);
     if (EmptyOldRead) or (I<Count) then
        begin
        repeat
          if fAString<>'' then InSA:=InSA+fAString;
          L:=System.Length(InSA);
          I:=inherited Read(fBuffer,CMaxStringBufferLength);
          fAString:=AnsiString(fBuffer);
        until not((L+I)<Count) or Eof;
        end;
     InSA:=InSA+System.Copy(fAString,1,Count-L);
     System.Delete(fAString,1,Count-L);
     Result:=InSA;
     end
  else Result:='';
end;                      

function TLnCRCFileStream.ReadStringLn: AnsiString;
var EmptyOldRead: boolean;
    InSA: AnsiString;
    I: Longint;
begin
  EmptyOldRead:=(fAString='');
  if (Position<Size) or not(EmptyOldRead) then
      begin
      InSA:='';
      if (EmptyOldRead) or (System.Pos(CLF,fAString)=0) then
         begin
         repeat
           fLnPosition:=Position;
           if fAString<>'' then InSA:=InSA+fAString;
           I:=inherited Read(fBuffer,CMaxStringBufferLength);
           if (I<CMaxStringBufferLength) then fBuffer[I]:=#0;
           fAString:=AnsiString(fBuffer);
         until (System.Pos(CLF,fAstring)>0) or Eof;
         end;
     I:=System.Pos(CLF,fAString);
     fLnPosition:=fLnPosition+I;
     if I=1 then System.Delete(InSA,System.Length(InSA),1)
     else InSA:=InSA+System.Copy(fAString,1,I-2);
     System.Delete(fAString,1,I);
     Result:=InSA;
     end
  else Result:='';
end;

function TLnCRCFileStream.ReadStringUntilChar(Value: char): string;
var EmptyOldRead: boolean;
    InSA: AnsiString;
    I, P: Longint;
begin
  EmptyOldRead:=(fAString='');
  if (Position<Size) or not(EmptyOldRead) then
     begin
     InSA:=''; fLnPosition:=0;
     I:=System.Length(fAString);
     P:=System.Pos(Value,fAString);
      if (EmptyOldRead) or (P=0) then  // P=System.Pos(Value,fAString)
         begin
         repeat
           if fAString<>'' then InSA:=InSA+fAString;
           I:=inherited Read(fBuffer,CMaxStringBufferLength);
           fAString:=AnsiString(fBuffer);
           P:=System.Pos(Value,fAstring);
         until (P>0) or Eof;
         end;
     if (P=0) and (I<CMaxStringBufferLength) then
        begin
        InSA:=InSA+fAString;
        fAString:='';
        end
     else
        begin
        if P=1 then System.Delete(InSA,System.Length(InSA),1)
        else InSA:=InSA+System.Copy(fAString,1,P-1);
        System.Delete(fAString,1,P+1);
        end;
        
     Result:=InSA;
     end
  else Result:='';
end;

function TLnCRCFileStream.ReadStringUntilChars(Value: TSysCharSet): AnsiString;
var EmptyOldRead: boolean;
    InSA: AnsiString;
    I, P: Longint;
begin
  EmptyOldRead:=(fAString='');
  if (Position<Size) or not(EmptyOldRead) then
     begin
     InSA:=''; fLnPosition:=0;
     I:=System.Length(fAString);
     // P:=System.Pos(Value,fAString);
     P:=GetSysCharSetPosition(Value,fAString);
      if (EmptyOldRead) or (P=0) then  // P=System.Pos(Value,fAString)
         begin
         repeat
           if fAString<>'' then InSA:=InSA+fAString;
           I:=inherited Read(fBuffer,CMaxStringBufferLength);
           fAString:=AnsiString(fBuffer);
           // P:=System.Pos(Value,fAstring);
           P:=GetSysCharSetPosition(Value,fAString);
         until (P>0) or Eof;
         end;
     if (P=0) and (I<CMaxStringBufferLength) then
        begin
        InSA:=InSA+fAString;
        fAString:='';
        end
     else
        begin
        if P=1 then System.Delete(InSA,System.Length(InSA),1)
        else InSA:=InSA+System.Copy(fAString,1,P-1);
        System.Delete(fAString,1,P+1);
        end;
        
     Result:=InSA;
     end
  else Result:='';
end;

function TLnCRCFileStream.ReadStringDynArrayLn: TStringDynArray;
begin
  Result:=ReadStreamFileLineToStringDynArray(ListSeparator);
end;

function TLnCRCFileStream.ReadStringDynArrayLn(SepChar: char): TStringDynArray;
begin
  Result:=ReadStreamFileLineToStringDynArray(SepChar);
end;

function TLnCRCFileStream.ReadStringDynArrayLn(SepChars: TSysCharSet): TStringDynArray;
begin
  Result:=ReadStreamFileLineToStringDynArray(SepChars);
end;

function TLnCRCFileStream.ReadStringDynArrayLn(SepChar: char; SuppressBlanks: boolean): TStringDynArray;
begin
  Result:=ReadStreamFileLineToStringDynArray(SepChar,SuppressBlanks);
end;

function TLnCRCFileStream.ReadStringDynArrayLn(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynArray;
begin
  Result:=ReadStreamFileLineToStringDynArray(SepChars,SuppressBlanks);
end;

function TLnCRCFileStream.ReadStringDynGridLn: TStringDynGrid;
begin
  Result:=ReadStreamFileToStringDynGrid(ListSeparator);
end;

function TLnCRCFileStream.ReadStringDynGridLn(SepChar: char): TStringDynGrid;
begin
  Result:=ReadStreamFileToStringDynGrid(SepChar);
end;

function TLnCRCFileStream.ReadStringDynGridLn(SepChars: TSysCharSet): TStringDynGrid;
begin
  Result:=ReadStreamFileToStringDynGrid(SepChars);
end;

function TLnCRCFileStream.ReadStringDynGridLn(SepChar: char; SuppressBlanks: boolean): TStringDynGrid;
begin
  Result:=ReadStreamFileToStringDynGrid(SepChar,SuppressBlanks);
end;

function TLnCRCFileStream.ReadStringDynGridLn(SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynGrid;
begin
  Result:=ReadStreamFileToStringDynGrid(SepChars,SuppressBlanks);
end;

function TLnCRCFileStream.SeekUntilString(Value: string): boolean;
var InS: string;
begin
  InS:=ReadStringLn;
  while ((Position<Size) and (InS<>Value)) do InS:=ReadStringLn;
  Result:=((Position<Size) and (InS=Value));
end;

procedure TLnCRCFileStream.WriteString(Value: AnsiString);
begin
  while (System.Length(Value)>CMaxStringBufferLength) do
        begin
        StrPCopy(fBuffer,System.Copy(Value,1,CMaxStringBufferLength));
        System.Delete(Value,1,CMaxStringBufferLength);
        inherited Write(fBuffer,CMaxStringBufferLength);
        end;
  StrPCopy(fBuffer,Value);
  inherited Write(fBuffer,System.Length(Value));
end;

procedure TLnCRCFileStream.WriteStringLn;
begin
  WriteString(CCR+CLF);
end;

procedure TLnCRCFileStream.WriteStringLn(Value: AnsiString);
begin
  WriteString(Value+CCR+CLF);
end;

procedure TLnCRCFileStream.WriteStringDynArrayLn(Value: TStringDynArray);
begin
  WriteStringDynArrayToStreamFile(Value,ListSeparator);
end;

procedure TLnCRCFileStream.WriteStringDynArrayLn(Value: TStringDynArray; SepChar: char);
begin
  WriteStringDynArrayToStreamFile(Value,SepChar);
end;

procedure TLnCRCFileStream.WriteStringDynGridLn(Value: TStringDynGrid);
begin
  WriteStringDynGridToStreamFile(Value,ListSeparator);
end;

procedure TLnCRCFileStream.WriteStringDynGridLn(Value: TStringDynGrid; SepChar: char);
begin
  WriteStringDynGridToStreamFile(Value,SepChar);
end;

procedure TLnCRCFileStream.ClearBuffer;
begin
  fAString:=''; fLnPosition:=0;
end;

function TLnCRCFileStream.ReadFileToStringDynArrayLn: TStringDynArray;
begin
  Result:=ReadStreamFileToStringDynArray;
end;

procedure TLnCRCFileStream.WriteStringDynArrayLnToFile(Source: TStringDynArray);
var I, L: integer;
begin
   L:=System.Length(Source);
   if (L>0) then
      for I:=0 to L-1 do
          WriteStringLn(Source[I]);
end;

function TLnCRCFileStream.Seek(Offset: Longint; Origin: Word): Longint;
begin
  Result := inherited Seek(Offset,Origin);
  // if not((Offset=0) and (Origin=soFromCurrent)) then ClearBuffer;
end;

function TLnCRCFileStream.Eof: boolean;
begin
  Result:=(Position=Size) and (fAString='');
end;

function TLnCRCFileStream.GetCRC16: word;
var OldPosition: Longint;
begin
  OldPosition:=Position;
  Result:=CalculateCRC16;
  inherited Seek(OldPosition,soFromBeginning);
end;

procedure TLnCRCFileStream.AppendCRC16;
var CR: array [0..1] of byte;
    fCRC16: word;
begin
  fCRC16:=CalculateCRC16;
  CR[0]:=(fCRC16 shr 8) and $FF;
  CR[1]:=fCRC16 and $FF;
  inherited Write(CR,2);
end;

function TLnCRCFileStream.GetParityByte: byte;
var OldPosition: Longint;
begin
  OldPosition:=Position;
  Result:=CalculateParityByte;
  inherited Seek(OldPosition,soFromBeginning);
end;

procedure TLnCRCFileStream.AppendParityByte;
var fParityByte: word;
begin
  fParityByte:=CalculateParityByte;
  inherited Write(fParityByte,1);
end;

end.
