//******************************************************************************
//******************************************************************************
//******************************************************************************
//**
//**  RegUtils.PAS
//**
//**  Useful Registry Access Procedures
//**
//**  Autor : Georg  Hahn
//**  Datum :  xx.yy.96 / 23.10.06 / 22.10.09 / 13.01.10 / 02.02.10
//**
//******************************************************************************
//******************************************************************************
//******************************************************************************
unit RegUtils;

interface

uses Windows, Classes;

function RegistryKeyExists(MainKey: HKEY; PathKey: string): boolean;
function RegistryValueExists(MainKey: HKEY; PathKey, ValueName: string): boolean;

function GetSubKeySetFromRegistry(MainKey: HKEY; PathKey: string; var KeyStrings: TStringList): boolean;
function GetSubKeyNamesFromRegistry(MainKey: HKEY; PathKey: string; var NameStrings: TStringList): boolean;

function DeleteRegistrySubKey(MainKey: HKEY; PathKey,SubKey: string): boolean;
function DeleteRegistryValue(MainKey: HKEY; PathKey,ValueName: string): boolean;

function CleanRegistryKey(MainKey: HKEY; PathKey: string): boolean;
function CleanRegistryValues(MainKey: HKEY; PathKey: string): boolean;

function LoadStringFromRegistry(MainKey: HKEY; PathKey,ValueName,DefaultValue: string): string;
function LoadBooleanFromRegistry(MainKey: HKEY; PathKey,ValueName: string; DefaultValue: boolean): boolean;
function LoadIntegerFromRegistry(MainKey: HKEY; PathKey,ValueName: string; DefaultValue: integer): integer;
function LoadRealFromRegistry(MainKey: HKEY; PathKey,ValueName: string; DefaultValue: double): double;
function LoadBinaryDataToRegistry(MainKey: HKEY; PathKey,ValueName: string; BufSize: integer; var Buffer): boolean;

procedure SaveStringToRegistry(MainKey: HKEY; PathKey,ValueName,Value: string);
procedure SaveBooleanToRegistry(MainKey: HKEY; PathKey,ValueName: string; Value: boolean);
procedure SaveIntegerToRegistry(MainKey: HKEY; PathKey,ValueName: string; Value: integer);
procedure SaveRealToRegistry(MainKey: HKEY; PathKey,ValueName: string; Value: double);
procedure SaveBinaryDataToRegistry(MainKey: HKEY; PathKey,ValueName: string; BufSize: integer; var Buffer);

procedure CreateRegistryEntryForAutoStart(FileExtension,AutoStartInfo,AutoFileDescription,ApplicationName: string);

function GetWindowsVersionInfo(txBuild: string): string;
function RegistryEntryForSerialCommExists(Number: integer): boolean;

implementation

uses SysUtils, Registry;

function RegistryKeyExists(MainKey: HKEY; PathKey: string): boolean;
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         Result:=KeyExists(PathKey);
         end;
  finally
    MyRegistry.Free;
  end;
end;

function RegistryValueExists(MainKey: HKEY; PathKey, ValueName: string): boolean;
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         Result:=KeyExists(PathKey) and OpenKey(PathKey,false) and ValueExists(ValueName);
         end;
  finally
    MyRegistry.Free;
  end;
end;

function GetSubKeySetFromRegistry(MainKey: HKEY; PathKey: string; var KeyStrings: TStringList): boolean;
var MyRegistry: TRegistry;
begin
  Result:=false;
  KeyStrings.Clear;
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) then
              begin
              GetKeyNames(KeyStrings);
              Result:=true;
              end;
         except
           on ERegistryException do KeyStrings.Clear;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function GetSubKeyNamesFromRegistry(MainKey: HKEY; PathKey: string; var NameStrings: TStringList): boolean;
var MyRegistry: TRegistry;
begin
  Result:=false;
  NameStrings.Clear;
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) then
              begin
              GetValueNames(NameStrings);
              Result:=true;
              end;
         except
           on ERegistryException do NameStrings.Clear;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function DeleteRegistrySubKey(MainKey: HKEY; PathKey,SubKey: string): boolean;
var MyRegistry: TRegistry;
begin
  Result:=false;
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         if KeyExists(PathKey) and OpenKey(PathKey,false) and KeyExists(SubKey) then
            Result:=DeleteKey(SubKey);
         end;
  finally
    MyRegistry.Free;
  end;
end;

function DeleteRegistryValue(MainKey: HKEY; PathKey,ValueName: string): boolean;
var MyRegistry: TRegistry;
begin
  Result:=false;
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         if KeyExists(PathKey) and OpenKey(PathKey,false) and ValueExists(ValueName) then
            Result:=DeleteValue(ValueName);
         end;
  finally
    MyRegistry.Free;
  end;
end;

function CleanRegistryKey(MainKey: HKEY; PathKey: string): boolean;
var MyRegistry: TRegistry;
    KS: TStringList;
    I: integer;
begin
  Result:=false;
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         KS:=TStringList.Create;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) then
              begin
              GetKeyNames(KS);
              if KS.Count>0 then
                 begin
                 for I:=0 to KS.Count-1 do
                     DeleteKey(PathKey+'\'+KS.Strings[I]);
                 end;
              Result:=true;
              end;
         finally
           KS.Free;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function CleanRegistryValues(MainKey: HKEY; PathKey: string): boolean;
var MyRegistry: TRegistry;
    VS: TStringList;
    I: integer;
begin
  Result:=false;
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         VS:=TStringList.Create;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) then
              begin
              GetKeyNames(VS);
              if VS.Count>0 then
                 begin
                 for I:=0 to VS.Count-1 do
                     DeleteValue(VS.Strings[I]);
                 end;
              Result:=true;
              end;
         finally
           VS.Free;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function LoadStringFromRegistry(MainKey: HKEY; PathKey,ValueName,DefaultValue: string): string;
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) and ValueExists(ValueName) then
              Result:=ReadString(ValueName)
           else Result:=DefaultValue;
         except
           on ERegistryException do Result:=DefaultValue;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function LoadBooleanFromRegistry(MainKey: HKEY; PathKey,ValueName: string; DefaultValue: boolean): boolean;
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) and ValueExists(ValueName) then
              Result:=ReadBool(ValueName)
           else Result:=DefaultValue;
         except
           on ERegistryException do Result:=DefaultValue;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function LoadIntegerFromRegistry(MainKey: HKEY; PathKey,ValueName: string; DefaultValue: integer): integer;
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) and ValueExists(ValueName) then
              Result:=ReadInteger(ValueName)
           else Result:=DefaultValue;
         except
           on ERegistryException do Result:=DefaultValue;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function LoadRealFromRegistry(MainKey: HKEY; PathKey,ValueName: string; DefaultValue: double): double;
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) and ValueExists(ValueName) then
              Result:=ReadFloat(ValueName)
           else Result:=DefaultValue;
         except
           on ERegistryException do Result:=DefaultValue;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function LoadBinaryDataToRegistry(MainKey: HKEY; PathKey,ValueName: string; BufSize: integer; var Buffer): boolean;
var MyRegistry: TRegistry;
    X: integer;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         Access:=KEY_READ;
         try
           if KeyExists(PathKey) and OpenKey(PathKey,false) and ValueExists(ValueName) then
              begin
              X:=ReadBinaryData(ValueName,Buffer,BufSize);
              Result:=(X>0);
              end
           else Result:=false;
         except
           on ERegistryException do Result:=false;
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

procedure SaveStringToRegistry(MainKey: HKEY; PathKey,ValueName,Value: string);
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         if OpenKey(PathKey, true) then WriteString(ValueName,Value);
         end;
  finally
    MyRegistry.Free;
  end;
end;

procedure SaveBooleanToRegistry(MainKey: HKEY; PathKey,ValueName: string; Value: boolean);
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         if OpenKey(PathKey, true) then WriteBool(ValueName,Value);
         end;
  finally
    MyRegistry.Free;
  end;
end;

procedure SaveIntegerToRegistry(MainKey: HKEY; PathKey,ValueName: string; Value: integer);
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         if OpenKey(PathKey, true) then WriteInteger(ValueName,Value);
         end;
  finally
    MyRegistry.Free;
  end;
end;

procedure SaveRealToRegistry(MainKey: HKEY; PathKey,ValueName: string; Value: double);
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         if OpenKey(PathKey, true) then WriteFloat(ValueName,Value);
         end;
  finally
    MyRegistry.Free;
  end;
end;

procedure SaveBinaryDataToRegistry(MainKey: HKEY; PathKey,ValueName: string; BufSize: integer; var Buffer);
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey:=MainKey;
         if OpenKey(PathKey, true) then WriteBinaryData(ValueName,Buffer,BufSize);
         end;
  finally
    MyRegistry.Free;
  end;
end;

procedure CreateRegistryEntryForAutoStart(FileExtension,AutoStartInfo,AutoFileDescription,ApplicationName: string);
var MyRegistry: TRegistry;
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey := HKEY_CLASSES_ROOT;
         if not(KeyExists('\.'+FileExtension)) then
            begin
            if OpenKey('\.'+FileExtension, true) then
               begin
               WriteString('',AutoStartInfo);
               CloseKey;
               if OpenKey(AutoStartInfo, true) then
                  begin
                  WriteString('',AutoFileDescription);
                  CloseKey;
                  if OpenKey(AutoStartInfo+'\DefaultIcon',true) then
                     begin
                     WriteString('',ApplicationName+',1');
                     CloseKey;
                     if OpenKey(AutoStartInfo+'\shell\open\command', true) then
                        WriteString('',ApplicationName+' %1');
                     end;
                  end;
               end;
            end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function GetWindowsVersionInfo(txBuild: string): string;
  function CutMicrosoft (Source: string): string;
  begin
    if (System.Pos('Microsoft ',Source)=1) then
       begin
       System.Delete(Source,1,System.Length('Microsoft '));
       Result:=Source;
       end
    else Result:=Source;
  end;
var MyRegistry: TRegistry;
const CNTKey = '\Software\Microsoft\Windows NT\CurrentVersion\';
      CNotNTKey = '\Software\Microsoft\Windows\CurrentVersion\';
begin
  MyRegistry:=TRegistry.Create;
  try
    with MyRegistry do
         begin
         RootKey := HKEY_LOCAL_MACHINE;
         Access:=KEY_READ;
         try
           if OpenKey(CNotNTKey, false) and
              ValueExists('ProductName') and
              ValueExists('VersionNumber') then
              Result:=CutMicrosoft (ReadString('ProductName')) + ',   '+txBuild+'  '+ReadString('VersionNumber')
           else if OpenKey(CNTKey, false) and
                   ValueExists('ProductName') and
                   ValueExists('CurrentVersion') and
                   ValueExists('CurrentBuildNumber') then
                   Result:=CutMicrosoft (ReadString('ProductName')) + ',   '+txBuild+'  '+ReadString('CurrentVersion')+'.'+ReadString('CurrentBuildNumber')
                else Result:='';
         except
           on ERegistryException do Result:='';
           end;
         end;
  finally
    MyRegistry.Free;
  end;
end;

function RegistryEntryForSerialCommExists(Number: integer): boolean;
var MyRegistry: TRegistry;
    ValueName: string;
begin
  Result:=false;
  case Win32Platform of
       VER_PLATFORM_WIN32_WINDOWS: ValueName:='COM'+IntToStr(Number);
       VER_PLATFORM_WIN32_NT: ValueName:='\Device\Serial'+IntToStr(Number-1)+'\';
       else ValueName:='';
       end;
  if ValueName<>'' then
     begin
     MyRegistry:=TRegistry.Create;
     try
       with MyRegistry do
            begin
            RootKey := HKEY_LOCAL_MACHINE;
            try
              if OpenKey('\Hardware\DeviceMap\SerialComm\', false) then
                 Result:=(ReadString(ValueName)='COM'+IntToStr(Number));
            except
              on ERegistryException do Result:=false;
              end;
            end;
     finally
       MyRegistry.Free;
       end;
     end;
end;

end.
