//*****************************************************************************
//*****************************************************************************
//*****************************************************************************
//**
//**  SERIALFTDEVICES.PAS // BASED PARTIALLY ON PCLTEST.PAS FROM NOVEMBER 2005 //
//**
//**  Helpful constants and functions for access to USB serial FTD2XX devices
//**
//**  Autor : Georg  Hahn
//**  Datum :  22.11.05 / 30.06.08 / 24.10.08 / 02.12.09 / 12.01.10 / 19.05.10
//**
//*****************************************************************************
//*****************************************************************************
//*****************************************************************************
unit SerialFTDevices;
{$M+}
{$UNSAFECODE ON}

{$DEFINE WithThread}

interface

uses
  Windows, Classes, Types, StringUtils, DynLoadFTD2XX, SerialComDevices, UsefulObjects;

const
  CDefaultFTDBaudRate  = FT_BAUD_9600;
  CMaxFTDeviceCount    = 99;

  CMaxEEPROMSize       = 1024;

  CDescriptionSize     = 64;
  CSerialNumberSize    = 16;

type
  TFTDInfoType = (itSerialNumber,itDescription);
  TFTDOpenMode = (omByNumber,omByDescription,omBySerialNumber);

  TFTDeviceNameString   = string[CDescriptionSize];
  TFTSerialNumberString = string[CSerialNumberSize];

  TFTDeviceNameArray   = array [0..CDescriptionSize-1] of char;
  TFTSerialNumberArray = array [0..CSerialNumberSize] of char;

  TEEPROMArray = array [0..CMaxEEPROMSize-1] of byte;

  TFTDSettings = record    // refer to SerialComDevices
    FTDOpenMode: TFTDOpenMode;
    DeviceName: TFTDeviceNameString;
    SerialNumber: TFTSerialNumberString;
    DeviceNumber,
    BaudRate: dword;
    Flags: dword;
    ByteSize,
    StopBits,
    Parity: byte;
    EOSChar,
    XonChar,
    XoffChar: char;
    end;

  function GetNextFTDBaudRateValue(Value: dword; Ascending: boolean): dword;
  function GetFTDCharTransmissionTime(FTDSettings: TFTDSettings): word;

  function GetFTDeviceCount: dword; overload;
  function GetFTDeviceInfo(Number: dword; InfoType: TFTDInfoType): string;
  function GetFTDeviceCount(DeviceName: string): dword; overload;
  function GetFTDeviceCount(DeviceName, SerialNumber: string): dword; overload;

  function GetFTDeviceInfos(Number: dword; var DeviceName, SerialNumber: string): boolean;

  function GetFTDeviceNumber(DeviceName: string; var Number: dword): boolean; overload;
  function GetFTDeviceNumber(DeviceName, SerialNumber: string; var Number: dword): boolean; overload;

  function ReadFTDIUserEEPROM(Number: dword): TByteDynArray;
  function WriteFTDIUserEEPROM(Number: dword; Data: TByteDynArray): boolean;

  function FTDIDeviceExists(Number: dword): boolean; overload;
  function FTDIDeviceExists(DeviceName: string): boolean; overload;
  function FTDIDeviceExists(DeviceName, SerialNumber: string): boolean; overload;

  function FTDIDeviceAvailable(Number: dword): boolean; overload;
  function FTDIDeviceAvailable(DeviceName: string): boolean; overload;
  function FTDIDeviceAvailable(DeviceName, SerialNumber: string): boolean; overload;

  function OpenFTDIDevice(FTDSettings: TFTDSettings): FT_HANDLE;
  function FTDIDeviceOpened(fFTDHandle: FT_HANDLE): boolean;
  procedure CloseFTDIDevice(var fFTDHandle: FT_HANDLE);

  function SetFTD_TXDLine(fFTDHandle: FT_HANDLE; TXD: boolean): boolean;
  function SetFTD_DTRLine(fFTDHandle: FT_HANDLE; DTR: boolean): boolean;
  function SetFTD_RTSLine(fFTDHandle: FT_HANDLE; RTS: boolean): boolean;
  function SetFTD_DTRandRTSlines(fFTDHandle: FT_HANDLE; DTR,RTS: boolean): boolean;
  function GetFTD_CTSLine(fFTDHandle: FT_HANDLE): boolean;
  function GetFTD_DSRLine(fFTDHandle: FT_HANDLE): boolean;
  function GetFTD_CTSandDSRLines(fFTDHandle: FT_HANDLE): TModemStatus;

  procedure PurgeFTDReadBuffer(fFTDHandle: FT_HANDLE);
  procedure PurgeFTDWriteBuffer(fFTDHandle: FT_HANDLE);
  procedure PurgeFTDReadWriteBuffer(fFTDHandle: FT_HANDLE);

  function GetFTDCharsInQue(fFTDHandle: FT_HANDLE): word;
  function GetFTDCharsOutQue(fFTDHandle: FT_HANDLE): word;

  function SetFTDMonitoredEvents(fFTDHandle: FT_HANDLE; EvtMask: dword): boolean;
  function GetFTDMonitoredEventsStatus(fFTDHandle: FT_HANDLE; EvtMask: dword): boolean;

  function PutStringToFTD(fFTDHandle: FT_HANDLE; Str: string): boolean; overload;
  function PutStringToFTD(fFTDHandle: FT_HANDLE; Str: string; TOC: TTimeOutController; TimeOut: dword): boolean; overload;

  function GetCharFromFTD(fFTDHandle: FT_HANDLE; var Ch: char): boolean; overload;
  function GetCharFromFTD(fFTDHandle: FT_HANDLE; var Ch: char; TOC: TTimeOutController; TimeOut: dword): boolean; overload;

  function GetStringFromFTD(fFTDHandle: FT_HANDLE; CharCount: byte; BreakReadOnEOS: boolean; EOSChar: char): string; overload;
  function GetStringFromFTD(fFTDHandle: FT_HANDLE; CharCount: byte; BreakReadOnEOS: boolean; EOSChar: char; TOC: TTimeOutController; TimeOut: dword): string; overload;

type

{$IFDEF WithThread}
  PFTDReceiver = ^TFTDReceiver;
  TFTDReceiver = class(TThread)
  private
    { Private declarations }
    pFTDHandle: PFT_HANDLE;
    pBreakReadOnEOS,
    pRemoveEOSOnRead,
    pSimulationMode: pboolean;
    pEOSchar: pchar;
    pReadTimeOut,
    pMaxStringLength: pword;
    pReceptionString,
    pSimulationString: pstring;

    fEnabled,
    fDisabled,
    fReceptionFinished,
    fReceptionTimeOut: boolean;
    fReceptionBegin,
    fReceptionTimeOutLimit: dword;
    fThreadReceptionString: string;

    fOnThreadReceiveChar,
    fOnThreadReceptionFinished: TSimpleEvent;

    fOnThreadReadChar: TRxCharRead;
    fOnThreadReadString,
    fOnThreadReceptionTimeOut: TRxStringRead;
    fOnThreadReadTermination: TRxReadTermination;
  protected
    { Protected-Deklarationen }
    function GetSimulatedChar(var Ch: char): boolean;
    procedure Execute; override;
  public
    { Public-Deklarationen}
    constructor Create;
    destructor Destroy; override;
    procedure Resume;
    procedure Suspend;
  published
    { Published-Deklarationen }
    property OnThreadReceiveChar: TSimpleEvent read fOnThreadReceiveChar write fOnThreadReceiveChar;
    property OnThreadReceptionFinished: TSimpleEvent read fOnThreadReceptionFinished write fOnThreadReceptionFinished;

    property OnThreadReadChar: TRxCharRead read fOnThreadReadChar write fOnThreadReadChar;
    property OnThreadReadString: TRxStringRead read fOnThreadReadString write fOnThreadReadString;
    property OnThreadReceptionTimeOut: TRxStringRead read fOnThreadReceptionTimeOut write fOnThreadReceptionTimeOut;
    property OnThreadReadTermination: TRxReadTermination read fOnThreadReadTermination write fOnThreadReadTermination;
  end;
{$ENDIF}

  TCustomFTDevice = class(TComponent)
  private
    { Private declarations }
    fFTDHandle: FT_HANDLE;
    fSmallTimeOut: TTimeOutController;
    fLargeTimeOut: TTimeOutController;
{$IFDEF WithThread}
    fFTDReceiver: TFTDReceiver;
{$ENDIF}
    fOpenMode: TFTDOpenMode;
    fInputDSRSensivity,
    fAppendEOSonWrite,
    fBreakReadOnEOS,
    fRemoveEOSOnRead,
    fValidateBaudRate,
    fSimulationMode: boolean;
    fFTDNumber: dword;
    fDeviceName: TFTDeviceNameString;
    fSerialNumber: TFTSerialNumberString;
    fBaudRate: dword;
    fByteSize,
    fStopBits,
    fParityMode,
    fDTRControl,
    fRTSControl: byte;
    fEOSChar,
    fXonChar,
    fXoffChar: char;
    fOutputControlMode: TOutputControlMode;
    fXonXoffControlMode: TXonXoffControlMode;
    fCharTransmitInterval,
    fCharTransmitDuration,
    fWriteTimeOut,
    fReadTimeOut,
    fMaxStringLength: word;
{$IFDEF WithThread}
    fFTDReceptionString,
{$ENDIF}
    fFTDSimulationString: string;
    fOnTransmitChar,
    fOnTransmissionFinished,
    fOnReceiveChar,
    fOnReceptionFinished: TSimpleEvent;
{$IFDEF WithThread}
    fOnReadChar: TRxCharRead;
    fOnReadString,
    fOnReceptionTimeOut: TRxStringRead;
    fOnReadTermination: TRxReadTermination;
{$ENDIF}
    fOnReadStringSimulation: TRxStringRequest;
  protected
    { Protected declarations }
    function GetFTDOpenStatus: boolean;
    procedure SetOpenMode(Value: TFTDOpenMode);
    procedure SetDeviceNumber(Value: dword);
    procedure SetDeviceName(Value: TFTDeviceNameString);
    procedure SetSerialNumber(Value: TFTSerialNumberString);
    procedure SetBaudRate(Value: dword);
    procedure SetByteSize(Value: byte);
    procedure SetStopBits(Value: byte);
    procedure SetParityMode(Value: byte);
    procedure SetEOSChar(Value: char);
    procedure SetXonChar(Value: char);
    procedure SetXoffChar(Value: char);
    procedure SetDTRControl(Value: byte);
    procedure SetRTSControl(Value: byte);
    procedure SetOutputControlMode(Value: TOutputControlMode);
    procedure SetXonXoffControlMode(Value: TXonXoffControlMode);
    procedure SetInputDSRSensivity(Value: boolean);
    procedure SetMaxStringLength (Value: word);
    procedure SetValidateBaudRate(Value: boolean);
    procedure SetCharTransmitInterval(Value: word);
    procedure SetWriteTimeOut(Value: word);
    procedure SetReadTimeOut(Value: word);
    function GetCTSLineStatus: boolean;
    function GetDSRLineStatus: boolean;

    function PutString(Source: string): boolean;
    function TransmitString(Source: string): boolean;
    function SimulateTransmission (Source: string): boolean;

    function CharsInQue: word;
    function CharsOutQue: word;
    function GetChar(var Target: char): boolean;
    function GetString(CharCount: byte): string;
    function EvaluateFTDSettings: TFTDSettings;

    property FTDOpen: boolean read GetFTDOpenStatus;
    property OpenMode: TFTDOpenMode read fOpenMode write SetOpenMode default omByNumber;
    property DeviceNumber: dword read fFTDNumber write SetDeviceNumber default CMaxFTDeviceCount+1;
    property DeviceName: TFTDeviceNameString read fDeviceName write SetDeviceName;
    property SerialNumber: TFTSerialNumberString read fSerialNumber write SetSerialNumber;
    property BaudRate: dword read fBaudRate write SetBaudRate default CDefaultFTDBaudRate;
    property ByteSize: byte read fByteSize write SetByteSize default FT_DATA_BITS_8;
    property StopBit: byte read fStopBits write SetStopBits default FT_STOP_BITS_1;
    property ParityMode: byte read fParityMode write SetParityMode default NOPARITY;
    property EOSChar: char read fEOSChar write SetEOSChar default CLF;
    property XonChar: char read fXonChar write SetXonChar default CXon;
    property XoffChar: char read fXoffChar write SetXoffChar default CXoff;
    property DTRControl: byte read fDTRControl write SetDTRControl default DTR_CONTROL_DISABLE;
    property RTSControl: byte read fRTSControl write SetRTSControl default RTS_CONTROL_DISABLE;
    property OutputControlMode: TOutputControlMode read fOutputControlMode write SetOutputControlMode default ocNone;
    property XonXoffControlMode: TXonXoffControlMode read fXonXoffControlMode write SetXonXoffControlMode default xcNone;
    property InputDSRSensivity: boolean read fInputDSRSensivity write SetInputDSRSensivity default false;
    property AppendEOSonWrite: boolean read fAppendEOSonWrite write fAppendEOSonWrite default false;
    property BreadReadOnEOS: boolean read fBreakReadOnEOS write fBreakReadOnEOS default false;
    property RemoveEOSonRead: boolean read fRemoveEOSonRead write fRemoveEOSonRead default false;
    property MaxStringLength: word read fMaxStringLength write SetMaxStringLength;
    property ValidateBaudRate: boolean read fValidateBaudRate write SetValidateBaudRate default false;
    property CharTransmitInterval: word read fCharTransmitInterval write SetCharTransmitInterval default 0;
    property WriteTimeOut: word read fWriteTimeOut write SetWriteTimeOut default 0;
    property ReadTimeOut: word read fReadTimeOut write SetReadTimeOut default 0;

    property CTSLineStatus: boolean read GetCTSLineStatus;
    property DSRLineStatus: boolean read GetDSRLineStatus;

    property OnTransmitChar: TSimpleEvent read fOnTransmitChar write fOnTransmitChar;
    property OnTransmissionFinished: TSimpleEvent read fOnTransmissionFinished write fOnTransmissionFinished;
    property OnReceiveChar: TSimpleEvent read fOnReceiveChar write fOnReceiveChar;
    property OnReceptionFinished: TSimpleEvent read fOnReceptionFinished write fOnReceptionFinished;
{$IFDEF WithThread}
    property OnReadChar: TRxCharRead read fOnReadChar write fOnReadChar;
    property OnReadString: TRxStringRead read fOnReadString write fOnReadString;
    property OnReceptionTimeOut: TRxStringRead read fOnReceptionTimeOut write fOnReceptionTimeOut;
    property OnReadTermination: TRxReadTermination read fOnReadTermination write fOnReadTermination;
{$ENDIF}
    property OnReadStringSimulation: TRxStringRequest read fOnReadStringSimulation write fOnReadStringSimulation;
  public
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;

    function Open: boolean; virtual;
    procedure Close; virtual;

    procedure PurgeReadBuffer;
    procedure PurgeWriteBuffer;
    procedure PurgeReadWriteBuffer;

    function GetCTSandDSRLineStatus: TModemStatus;
    function WriteString(Source: string): boolean; virtual;
    function ReadString(var Target: string; CharCount: byte): boolean; virtual;
    function QueryString(Question: string; BreakTime: integer; var Answer: string; CharCount: byte): boolean; virtual;
    function ForceReceiverSimulation(Value: string): boolean;
  published
    { Published declarations }
  end;

  TSimpleFTDevice = class(TCustomFTDevice)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    { Published declarations }
    property FTDOpen;
    property OpenMode;
    property DeviceNumber;
    property DeviceName;
    property SerialNumber;
    property BaudRate;
    property ByteSize;
    property StopBit;
    property ParityMode;
    property EOSChar;
    property XonChar;
    property XoffChar;
    property DTRControl;
    property RTSControl;
    property OutputControlMode;
    property XonXoffControlMode;
    property InputDSRSensivity;
    property AppendEOSonWrite;
    property BreadReadOnEOS;
    property RemoveEOSonRead;
    property MaxStringLength;
    property ValidateBaudRate;
    property CharTransmitInterval;
    property WriteTimeOut;
    property ReadTimeOut;
    property CTSLineStatus;
    property DSRLineStatus;

    property OnTransmitChar;
    property OnTransmissionFinished;
    property OnReceiveChar;
    property OnReceptionFinished;
{$IFDEF WithThread}
    property OnReadChar;
    property OnReadString;
    property OnReceptionTimeOut;
    property OnReadTermination;
{$ENDIF}
    property OnReadStringSimulation;
  end;

  procedure Register;

var
  TerminateWaitFTDeviceLoop: boolean;

implementation

uses SysUtils, ProcessUtils;

const
  CVIDNumber = 'VID_0403'; // This is the vendor ID number for FTDI
  CPIDNumber = 'PID_D020'; // MTL PID range number is from PID_D020 to PID_D027

  CBaudRatesTable: array [0..13] of dword = (FT_BAUD_300, FT_BAUD_600, FT_BAUD_1200, FT_BAUD_2400,FT_BAUD_4800, FT_BAUD_9600, FT_BAUD_14400, FT_BAUD_19200,
                                             FT_BAUD_38400, FT_BAUD_57600, FT_BAUD_115200, FT_BAUD_230400,FT_BAUD_460800, FT_BAUD_921600);

procedure Register;
begin
  RegisterComponents('Hardware', [TSimpleFTDevice]);
end;

function AcceptableFTDBaudRate(Value: dword): boolean;
var I: byte;
begin
  Result:=false;
  for I:=0 to System.Length(CBaudRatesTable)-1 do
      if (Value=CBaudRatesTable[I]) then
         begin
         Result:=true;
         break;
         end;
end;

function GetNextFTDBaudRateValue(Value: dword; Ascending: boolean): dword;
var I,L: byte;
begin
  L:=System.Length(CBaudRatesTable);
  for I:=L-1 downto 0 do
      if (Value=CBaudRatesTable[I]) then break;
  if Ascending then
     begin
     if (I<L-2) then System.Inc(I);
     end
  else  // not(Ascending)
     begin
     if (I>1) then System.Dec(I);
     end;
  Result:=CBaudRatesTable[I];
end;

// The below character transmission time is in micro seconds and gives acceptable
// result accuracy when the Baud Rate is between 100 to 100k
// Note: The result is calculated sligthly higher for StopBits > 1.
function GetFTDCharTransmissionTime(FTDSettings: TFTDSettings): word;
begin
  Result:=(1000000*(FTDSettings.ByteSize+FTDSettings.StopBits+1)) DIV FTDSettings.BaudRate;
end;

// SerialFTDUtils functions and procedures

// Attention ! There are two different FTD2XX library versions exists simultanuously
// The "Old" library version doesn't supply the followed functions and procedures:
// "FT_GetLibraryVersion", "FT_GetDriverVersion", "FT_CreateDeviceInfoList",
// "FT_GetDeviceInfoList", "FT_GetDeviceInfoDetail" and "FT_W32_GetCommMask"
// Therefore several below functions uses different (library version dependent) approach

// Note: The "old" fashion library function seems to be faster, therefore prefer
// these version (and keep the new version solutions as comments)

function GetFTDeviceCount: dword;
var FT_Device_Count : dword;
begin
  Result:=0;
  if FTD2XXLibraryAvailable then
     if (FT_ListDevices(@FT_Device_Count,nil,FT_LIST_NUMBER_ONLY)=FT_OK) then
        Result:=FT_Device_Count;
     // if (NewFTD2XXLibrary and (FT_CreateDeviceInfoList(@FT_Device_Count)=FT_OK)) or
     //    (not(NewFTD2XXLibrary) and (FT_ListDevices(@FT_Device_Count,nil,FT_LIST_NUMBER_ONLY)=FT_OK)) then
     //    Result:=FT_Device_Count;
end;

function GetFTDeviceInfo(Number: dword; InfoType: TFTDInfoType): string;
var FT_Device_String_Buffer : TFTDeviceNameArray;
    Flags : dword;
begin
  Result:='';
  if FTD2XXLibraryAvailable and
     (Number<GetFTDeviceCount) then // note that the first device number is 0
     begin
     case InfoType of
          itDescription:  Flags:=(FT_LIST_BY_INDEX or FT_OPEN_BY_DESCRIPTION);
          itSerialNumber: Flags:=(FT_LIST_BY_INDEX or FT_OPEN_BY_SERIAL_NUMBER);
          else Flags:=0;
          end;
     if (FT_ListDevices(pointer(Number),@FT_Device_String_Buffer,Flags)=FT_OK) then
        Result:=CopyPChar(@FT_Device_String_Buffer,CDescriptionSize);
     end;
end;

function GetFTDeviceCount(DeviceName: string): dword;
var I, K, FT_Device_Count: dword;
    // Flags, DeviceType, ID, LocID : dword;
    // ftSerialNumber: TFTSerialNumberArray;
    // ftDescription: TFTDeviceNameArray;
    // ftHandle: FT_HANDLE;
begin
  Result:=0;
  if FTD2XXLibraryAvailable then
     begin
     FT_Device_Count:=GetFTDeviceCount;
     if (FT_Device_Count>0) then
        begin
        K:=0;
        for I:=0 to FT_Device_Count-1 do
            begin
            if (GetFTDeviceInfo(I,itDescription)=DeviceName) then
               System.Inc(K);
            // if (NewFTD2XXLibrary and (FT_GetDeviceInfoDetail(I,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) and (DeviceName=CopyPChar(@ftDescription,CDescriptionSize))) or
            //    (not(NewFTD2XXLibrary) and (DeviceName=GetFTDeviceInfo(I,itDescription))) then
            //    System.Inc(K);
            end;
        Result:=K;
        end;
     end;
end;

function GetFTDeviceCount(DeviceName, SerialNumber: string): dword; overload;
var I, K, FT_Device_Count: dword;
    // Flags, DeviceType, ID, LocID : dword;
    // ftSerialNumber: TFTSerialNumberArray;
    // ftDescription: TFTDeviceNameArray;
    // ftHandle: FT_HANDLE;
begin
  Result:=0;
  if FTD2XXLibraryAvailable then
     begin
     FT_Device_Count:=GetFTDeviceCount;
     if (FT_Device_Count>0) then
        begin
        K:=0;
        for I:=0 to FT_Device_Count-1 do
            begin
            if ((GetFTDeviceInfo(I,itDescription)=DeviceName) and (GetFTDeviceInfo(I,itSerialNumber)=SerialNumber)) then
               System.Inc(K);
            // if (NewFTD2XXLibrary and (FT_GetDeviceInfoDetail(I,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) and (DeviceName=CopyPChar(@ftDescription,CDescriptionSize))) or
            //    (not(NewFTD2XXLibrary) and (DeviceName=GetFTDeviceInfo(I,itDescription)) and (SerialNumber=GetFTDeviceInfo(I,itSerialNumber))) then
            //    System.Inc(K);
            end;
        Result:=K;
        end;
     end;
end;

function GetFTDeviceInfos(Number: dword; var DeviceName, SerialNumber: string): boolean;
// var Flags, DeviceType, ID, LocID : dword;
//     ftSerialNumber: TFTSerialNumberArray;
//     ftDescription: TFTDeviceNameArray;
//     ftHandle: FT_HANDLE;
begin
  Result:=false;
  if FTD2XXLibraryAvailable and
     (Number<GetFTDeviceCount) then // note that the first device number is 0
     begin
     DeviceName:=GetFTDeviceInfo(Number,itDescription);
     SerialNumber:=GetFTDeviceInfo(Number,itSerialNumber);
     Result:=true;
     // if NewFTD2XXLibrary then
     //    begin
     //    if (FT_GetDeviceInfoDetail(Number,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) then
     //       begin
     //       DeviceName:=CopyPChar(@ftDescription,CDescriptionSize);
     //       SerialNumber:=CopyPChar(@ftSerialNumber,CSerialNumberSize);
     //       Result:=true;
     //       end;
     //    end
     // else  // OldFTD2XXLibrary
     //    begin
     //    DeviceName:=GetFTDeviceInfo(Number,itDescription);
     //    SerialNumber:=GetFTDeviceInfo(Number,itSerialNumber);
     //    Result:=true;
     //    end;
     end;
end;

function GetFTDeviceNumber(DeviceName: string; var Number: dword): boolean;
var I, FT_Device_Count: dword;
    // Flags, DeviceType, ID, LocID : dword;
    // ftDescription: TFTDeviceNameArray;
    // ftSerialNumber: TFTSerialNumberArray;
    // ftHandle: FT_HANDLE;
begin
  Result:=false;
  if FTD2XXLibraryAvailable then
     begin
     FT_Device_Count:=GetFTDeviceCount;
     if (FT_Device_Count>0) then
        begin
        for I:=0 to FT_Device_Count-1 do
            begin
            if (GetFTDeviceInfo(I,itDescription)=DeviceName) then
               begin
               Number:=I;
               Result:=true;
               break;
               end;
            // if (NewFTD2XXLibrary and (FT_GetDeviceInfoDetail(I,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) and (DeviceName=CopyPChar(@ftDescription,CDescriptionSize))) or
            //    (not(NewFTD2XXLibrary) and (DeviceName=GetFTDeviceInfo(I,itDescription))) then
            //    begin
            //    Number:=I;
            //    Result:=true;
            //    break;
            //    end;
            end;
        end;
     end;
end;

function GetFTDeviceNumber(DeviceName, SerialNumber: string; var Number: dword): boolean;
var I, FT_Device_Count: dword;
    // Flags, DeviceType, ID, LocID : dword;
    // ftDescription: TFTDeviceNameArray;
    // ftSerialNumber: TFTSerialNumberArray;
    // ftHandle: FT_HANDLE;
begin
  Result:=false;
  if FTD2XXLibraryAvailable then
     begin
     FT_Device_Count:=GetFTDeviceCount;
     if (FT_Device_Count>0) then
        begin
        for I:=0 to FT_Device_Count-1 do
            begin
            if ((GetFTDeviceInfo(I,itDescription)=DeviceName) and (GetFTDeviceInfo(I,itSerialNumber)=SerialNumber)) then
               begin
               Number:=I;
               Result:=true;
               break;
               end;
            // if (NewFTD2XXLibrary and (FT_GetDeviceInfoDetail(I,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) and (DeviceName=CopyPChar(@ftDescription,CDescriptionSize)) and (SerialNumber=CopyPChar(@ftSerialNumber,CSerialNumberSize))) or
            //    (not(NewFTD2XXLibrary) and (DeviceName=GetFTDeviceInfo(I,itDescription)) and (SerialNumber=GetFTDeviceInfo(I,itSerialNumber))) then
            //    begin
            //    Number:=I;
            //    Result:=true;
            //    break;
            //    end;
            end;
        end;
     end;
end;

function ReadFTDIUserEEPROM(Number: dword): TByteDynArray;
var FT_Device_Count, I, L, R: dword;
    ED: TEEPROMArray;
    EEData: TByteDynArray;
    ftHandle: FT_HANDLE;
begin
  SetLength(EEData,0);
  if FTD2XXLibraryAvailable then
     begin
     FT_Device_Count:=GetFTDeviceCount;
     if (FT_Device_Count>0) and
        not(Number>FT_Device_Count) then
        begin
        if (FT_Open(Number,@ftHandle)=FT_OK) then
           begin
           FT_EE_UASize(ftHandle,@L);
           if (L>0) then
              begin
              if (L>CMaxEEPROMSize) then L:=CMaxEEPROMSize;
              FT_EE_UARead(ftHandle,@ED,L,@R);
              if (R>0) then
                 begin
                 SetLength(EEData,R);
                 for I:=0 to R-1 do
                     EEData[I]:=ED[I];
                 end;
              end;
           FT_Close(ftHandle);
           end;
        end;
     end;
  Result:=EEData;
end;

function WriteFTDIUserEEPROM(Number: dword; Data: TByteDynArray): boolean;
var FT_Device_Count, I, L, TW, WR: dword;
    ED: TEEPROMArray;
    ftHandle: FT_HANDLE;
begin
  Result:=false;
  TW:=System.Length(Data);
  if (TW>0) and
     FTD2XXLibraryAvailable then
     begin
     FT_Device_Count:=GetFTDeviceCount;
     if (FT_Device_Count>0) and
        not(Number>FT_Device_Count) then
        begin
        if (FT_Open(Number,@ftHandle)=FT_OK) then
           begin
           FT_EE_UASize(ftHandle,@L);
           if (L>CMaxEEPROMSize) then L:=CMaxEEPROMSize;
           if (TW<L) then L:=TW;
           for I:=0 to L-1 do
               ED[I]:=Data[I];
           WR:=FT_EE_UAWrite(Number,@ED,L);
           Result:=(WR=TW);
           FT_Close(ftHandle);
           end;
        end;
     end;
end;

function FTDIDeviceExists(Number: dword): boolean;
var FT_Device_Count: dword;
begin
  FT_Device_Count:=GetFTDeviceCount;

  Result:=(FT_Device_Count>0) and not(Number>FT_Device_Count);
end;

function FTDIDeviceExists(DeviceName: string): boolean;
begin
  Result:=(GetFTDeviceCount(DeviceName)>0);
end;

function FTDIDeviceExists(DeviceName, SerialNumber: string): boolean;
begin
  Result:=(GetFTDeviceCount(DeviceName, SerialNumber)>0);
end;

function FTDIDeviceAvailable(Number: dword): boolean;
var // FT_Device_Count, Flags, DeviceType, ID, LocID : dword;
    // ftDescription: TFTDeviceNameArray;
    // ftSerialNumber: TFTSerialNumberArray;
    ftHandle: FT_HANDLE;
begin
  Result:=false;
  if FTD2XXLibraryAvailable then
     begin
     Result:=(FT_Open(Number,@ftHandle)=FT_OK) and
             (FT_Close(ftHandle)=FT_OK);
     // if NewFTD2XXLibrary then
     //    begin
     //    Result:=(FT_CreateDeviceInfoList(@FT_Device_Count)=FT_OK) and
     //            (FT_Device_Count>0) and
     //            not(Number>FT_Device_Count) and
     //            (FT_GetDeviceInfoDetail(Number,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) and
     //            ((Flags and $1)=0);
     //    end
     // else  // OldFTD2XXLibrary
     //    begin
     //    Result:=(FT_Open(Number,@ftHandle)=FT_OK) and
     //            (FT_Close(ftHandle)=FT_OK);
     //    end;
     end;
end;

function FTDIDeviceAvailable(DeviceName: string): boolean;
var I: dword;
    // FT_Device_Count, Flags, DeviceType, ID, LocID : dword;
    // ftDescription: TFTDeviceNameArray;
    // ftSerialNumber: TFTSerialNumberArray;
    ftHandle: FT_HANDLE;
begin
  Result:=false;
  if FTD2XXLibraryAvailable then
     begin
     // FT_Device_Count:=GetFTDeviceCount;
     if (GetFTDeviceCount(DeviceName)>0) then // note that the first device number is 0
        begin
        if GetFTDeviceNumber(DeviceName,I) then
           Result:=(FT_Open(I,@ftHandle)=FT_OK) and
                   (FT_Close(ftHandle)=FT_OK);
        // if NewFTD2XXLibrary then
        //    begin
        //    for I:=0 to FT_Device_Count-1 do
        //        begin
        //        if (FT_GetDeviceInfoDetail(I,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) then
        //           begin
        //           if ((Flags and $1)=0) and // device is not open
        //              (DeviceName=CopyPChar(@ftDescription,CDescriptionSize)) then
        //              begin
        //              Result:=true;
        //              break;
        //              end;
        //           end;
        //        end;
        //    end
        // else  // OldFTD2XXLibrary
        //    begin
        //    if GetFTDeviceNumber(DeviceName,I) then
        //       Result:=(FT_Open(I,@ftHandle)=FT_OK) and
        //               (FT_Close(ftHandle)=FT_OK);
        //    end;
        end;
     end;
end;

function FTDIDeviceAvailable(DeviceName, SerialNumber: string): boolean;
var I: dword;
    // FT_Device_Count, Flags, DeviceType, ID, LocID : dword;
    // ftDescription: TFTDeviceNameArray;
    // ftSerialNumber: TFTSerialNumberArray;
    ftHandle: FT_HANDLE;
begin
  Result:=false;
  if FTD2XXLibraryAvailable then
     begin
     // FT_Device_Count:=GetFTDeviceCount;
     if (GetFTDeviceCount(DeviceName)>0) then // note that the first device number is 0
        begin
        if GetFTDeviceNumber(DeviceName,SerialNumber,I) then
           Result:=(FT_Open(I,@ftHandle)=FT_OK) and
                   (FT_Close(ftHandle)=FT_OK);
        // if NewFTD2XXLibrary then
        //    begin
        //    for I:=0 to FT_Device_Count-1 do
        //        begin
        //        if (FT_GetDeviceInfoDetail(I,@Flags,@DeviceType,@ID,@LocID,@ftSerialNumber,@ftDescription,@ftHandle)=FT_OK) then
        //           begin
        //           if ((Flags and $1)=0) and // device is not open
        //              (DeviceName=CopyPChar(@ftDescription,CDescriptionSize)) and
        //              (SerialNumber=CopyPChar(@ftSerialNumber,CSerialNumberSize)) then
        //              begin
        //              Result:=true;
        //              break;
        //              end;
        //           end;
        //        end;
        //    end
        // else  // OldFTD2XXLibrary
        //    begin
        //    if GetFTDeviceNumber(DeviceName,SerialNumber,I) then
        //       Result:=(FT_Open(I,@ftHandle)=FT_OK) and
        //               (FT_Close(ftHandle)=FT_OK);
        //    end;
        end;
     end;
end;

function OpenFTDIDevice(FTDSettings: TFTDSettings): FT_HANDLE;
var fFTDTimeouts: FTTIMEOUTS;
    fFTDHandle: FT_HANDLE;
    fFTDOpen: boolean;
    fPString: PChar;
    fFlags: dword;
    lFTDCB: FTDCB;
begin
  if FTDIDeviceExists(FTDSettings.DeviceName) and
     FTDIDeviceAvailable(FTDSettings.DeviceName) then
     begin

     fFTDOpen:=false;
     fPString:=StrAlloc(SizeOf(TFTDeviceNameString));

     fFlags:=FILE_ATTRIBUTE_NORMAL or FILE_FLAG_OVERLAPPED;
     if (FTDSettings.FTDOpenMode=omByNumber) then
        begin
        FTDSettings.DeviceName:=GetFTDeviceInfo(FTDSettings.DeviceNumber,itDescription);
        StrPCopy(fPString,FTDSettings.DeviceName);
        fFlags:=fFlags or FT_OPEN_BY_DESCRIPTION;
        end
     else if (FTDSettings.FTDOpenMode=omByDescription) then
             begin
             StrPCopy(fPString,FTDSettings.DeviceName);
             fFlags:=fFlags or FT_OPEN_BY_DESCRIPTION;
             end
          else
             begin
             StrPCopy(fPString,FTDSettings.SerialNumber);
             fFlags:=fFlags or FT_OPEN_BY_SERIAL_NUMBER;
             end;

     fFTDHandle:=FT_W32_CreateFile(PChar(fPString),
                                   GENERIC_READ or GENERIC_WRITE, 0, nil,
                                   OPEN_EXISTING, fFlags, 0);

     if (fFTDHandle <> FT_INVALID_HANDLE) then
        begin
           if (ExtendedFTD2XXAPIAvailable) then
              FT_SetLatencyTimer(fFTDHandle,16);
           FillChar(fFTDTimeouts, Sizeof(fFTDTimeouts), 0);
           fFTDTimeouts.ReadIntervalTimeout := MAXDWORD;
           if (FT_W32_SetCommTimeouts(fFTDHandle, @fFTDTimeouts)) then
              begin
              if FT_W32_GetCommState(fFTDHandle, @lFTDCB) then
                 begin
                 // insert FTDSettings configuration to the DCB structure
                 lFTDCB.BaudRate:=FTDSettings.BaudRate;
                 lFTDCB.ByteSize:=FTDSettings.ByteSize;
                 lFTDCB.StopBits:=FTDSettings.StopBits;
                 lFTDCB.Parity:=FTDSettings.Parity;
                 lFTDCB.Flags:=FT_dcb_Binary;  // Win32 Platform will accept only binary mode

                 lFTDCB.Flags:=lFTDCB.Flags or FTDSettings.Flags;

                 if ((lFTDCB.Flags and (FT_dcb_InX or FT_dcb_OutX))>0) then
                    begin
                    lFTDCB.XonChar:=FTDSettings.XonChar;
                    lFTDCB.XoffChar:=FTDSettings.XoffChar;
                    end;
                 lFTDCB.EofChar:=FTDSettings.EOSChar;
                 // and configure FTDI to the Settings
                 if FT_W32_SetCommState(fFTDHandle, @lFTDCB) then
                    // condition "and FT_W32_SetupComm(fFTDHandle, 255, 255)" was removed
                    // because it will not work on computers with Windows NT
                    begin
                    fFTDOpen:=true;
                    FT_W32_PurgeComm(fFTDHandle, FT_PURGE_RX + FT_PURGE_TX);
                    end;
                 end;
              end;
        end;

     StrDispose(fPString);

     if not(fFTDOpen) and (fFTDHandle <> FT_INVALID_HANDLE) then
        begin
        FT_W32_CloseHandle(fFTDHandle);
        fFTDHandle:=FT_INVALID_HANDLE;
        end;
     Result:=fFTDHandle;
     end
  else Result:=FT_INVALID_HANDLE;
end;

function FTDIDeviceOpened(fFTDHandle: FT_HANDLE): boolean;
begin
  Result:=(fFTDHandle <> FT_INVALID_HANDLE);
end;

procedure CloseFTDIDevice(var fFTDHandle: FT_HANDLE);
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     FT_W32_CloseHandle(fFTDHandle);
     fFTDHandle:=FT_INVALID_HANDLE;
     end;
end;

function SetFTD_TXDLine(fFTDHandle: FT_HANDLE; TXD: boolean): boolean;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     if TXD then FT_W32_EscapeCommFunction(fFTDHandle,SETBREAK)
     else FT_W32_EscapeCommFunction(fFTDHandle,CLRBREAK);
     Result:=(FT_W32_GetLastError(fFTDHandle)=0);
     end
  else Result:=false;
end;

function SetFTD_DTRLine(fFTDHandle: FT_HANDLE; DTR: boolean): boolean;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     if DTR then FT_W32_EscapeCommFunction(fFTDHandle,SETDTR)
     else FT_W32_EscapeCommFunction(fFTDHandle,CLRDTR);
     Result:=(FT_W32_GetLastError(fFTDHandle)=0);
     end
  else Result:=false;
end;

function SetFTD_RTSLine(fFTDHandle: FT_HANDLE; RTS: boolean): boolean;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     if RTS then FT_W32_EscapeCommFunction(fFTDHandle,SETRTS)
     else FT_W32_EscapeCommFunction(fFTDHandle,CLRRTS);
     Result:=(FT_W32_GetLastError(fFTDHandle)=0);
     end
  else Result:=false;
end;

function SetFTD_DTRandRTSlines(fFTDHandle: FT_HANDLE; DTR,RTS: boolean): boolean;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     if DTR then FT_W32_EscapeCommFunction(fFTDHandle,SETDTR)
     else FT_W32_EscapeCommFunction(fFTDHandle,CLRDTR);
     if RTS then FT_W32_EscapeCommFunction(fFTDHandle,SETRTS)
     else FT_W32_EscapeCommFunction(fFTDHandle,CLRRTS);
     Result:=(FT_W32_GetLastError(fFTDHandle)=0);
     end
  else Result:=false;
end;

function GetFTD_CTSLine(fFTDHandle: FT_HANDLE): boolean;
var fModemStat: dword;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) and
     (FT_W32_GetCommModemStatus(fFTDHandle,@fModemStat)) then
     begin
     Result:=((fModemStat and MS_CTS_ON)>0) and (FT_W32_GetLastError(fFTDHandle)=0);
     end
  else Result:=false;
end;

function GetFTD_DSRLine(fFTDHandle: FT_HANDLE): boolean;
var fModemStat: dword;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) and
     (FT_W32_GetCommModemStatus(fFTDHandle,@fModemStat)) then
     begin
     Result:=((fModemStat and MS_DSR_ON)>0) and (FT_W32_GetLastError(fFTDHandle)=0);
     end
  else Result:=false;
end;

function GetFTD_CTSandDSRLines(fFTDHandle: FT_HANDLE): TModemStatus;
var fModemStat: dword;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) and
     (FT_W32_GetCommModemStatus(fFTDHandle,@fModemStat)) and
     (FT_W32_GetLastError(fFTDHandle)=0) then
     begin
     Result.CTSLine:=((fModemStat and MS_CTS_ON)>0);
     Result.DSRLine:=((fModemStat and MS_DSR_ON)>0);
     end
  else
     begin
     Result.CTSLine:=false;
     Result.DSRLine:=false;
     end;
end;

procedure PurgeFTDReadBuffer(fFTDHandle: FT_HANDLE);
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     FT_W32_PurgeComm(fFTDHandle, FT_PURGE_RX);
     end;
end;

procedure PurgeFTDWriteBuffer(fFTDHandle: FT_HANDLE);
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     FT_W32_PurgeComm(fFTDHandle, FT_PURGE_TX);
     end;
end;

procedure PurgeFTDReadWriteBuffer(fFTDHandle: FT_HANDLE);
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     FT_W32_PurgeComm(fFTDHandle, FT_PURGE_RX + FT_PURGE_TX);
     end;
end;

function GetFTDCharsInQue(fFTDHandle: FT_HANDLE): word;
var fStat: FTCOMSTAT;
    fErrors: dword;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     FT_W32_ClearCommError(fFTDHandle, @fErrors, @fStat);
     Result:=fStat.cbInQue;
     end
  else Result:=0;
end;

function GetFTDCharsOutQue(fFTDHandle: FT_HANDLE): word;
var fStat: FTCOMSTAT;
    fErrors: dword;
begin
  // Attention! Please note, that for non-overlapped write access
  // this function will always return zero (and not the que size)

  // Furthermore, please note that for overlapped write access
  // the result means the software interface buffer
  // and not the hardware communication transmitter !!!
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     FT_W32_ClearCommError(fFTDHandle, @fErrors, @fStat);
     Result:=fStat.cbOutQue;
     end
  else Result:=$FFFF;
end;

function SetFTDMonitoredEvents(fFTDHandle: FT_HANDLE; EvtMask: dword): boolean;
begin
  if (fFTDHandle <> FT_INVALID_HANDLE) and
     FT_W32_SetCommMask(fFTDHandle,EvtMask) then
     Result:=true
  else Result:=false;
end;

function GetFTDMonitoredEventsStatus(fFTDHandle: FT_HANDLE; EvtMask: dword): boolean;
// var fEvtMask: dword;
begin
  // Attention! Please note, that the to be monitored event must be enabled before
  // (see the above function). Please note also, that for multiple event mask
  // (more than one event) this function will return ambiguous result.
  // Furthermore, please note that e.g. the FT_W32_EVENT_TXEMPTY event status means software
  // interface buffer and not the hardware communication transmitter !!!
  Result:=false;
  // if NewFTD2XXLibrary and
  //   (fFTDHandle <> FT_INVALID_HANDLE) and
  //    FT_W32_GetCommMask(fFTDHandle,@fEvtMask) then
  //    begin
  //    Result:=(fEvtMask and EvtMask)>0;
  //    end
  // else Result:=false;
end;

function PutStringToFTD(fFTDHandle: FT_HANDLE; Str: string): boolean;
var fOverlapped: TOverlapped;
    fPString: TCharBuffer;
    fErrorCode: dword;
    fLength: integer;
    fStat: FTCOMSTAT;
begin
  // This function may return with success before the IO write operation is completed
  // The success result means that the operation was correctly passed to the API interface
  // and when the system indication is ERROR_IO_PENDING, then success is pressumed.
  Result:=false;
  if (fFTDHandle <> FT_INVALID_HANDLE) and                       // FTDI device is open
     FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat) and  // function call succeeds
     ((fErrorCode and CE_TXFULL) = 0) then                       // output buffer is not full
     begin

     fLength:=System.Length(Str);
     StrCopyBuff(fPString,Str);

     FillChar(fOverlapped,SizeOf(fOverlapped),0);

     if not(FT_W32_WriteFile(fFTDHandle, fPString, fLength, @fErrorCode, @fOverlapped)) then
        begin                                         // function call fail (but don't worry)
        fErrorCode:=FT_W32_GetLastError(fFTDHandle);  // possible only because of
        Result:=(fErrorCode=ERROR_IO_PENDING);        // the IO write is pending
        end
     else Result:=(fErrorCode=dword(fLength));
     end;
end;

function PutStringToFTD(fFTDHandle: FT_HANDLE; Str: string; TOC: TTimeOutController; TimeOut: dword): boolean;
var fOverlapped: TOverlapped;
    fPString: TCharBuffer;
    fErrorCode: dword;
    fLength: integer;
    fStat: FTCOMSTAT;
begin
  // This function will return with success when the IO write operation is correctly completed,
  // however it means that the last byte was successful passed to the hardware interface.
  // It requires in the interface parameters pointer to specialised Event object and TimeOut value
  // When TimeOut is elapsed before the IO operation is completed, then the return value is false
  Result:=false;
  if (fFTDHandle <> FT_INVALID_HANDLE) and                       // FTDI device is open
     FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat) and  // function call succeeds
     ((fErrorCode and CE_TXFULL) = 0) then                       // output buffer is not full
     begin

     fLength:=System.Length(Str);
     StrCopyBuff(fPString,Str);

     FillChar(fOverlapped,SizeOf(fOverlapped),0);
     TOC.Initialize(TimeOut);

     if not(FT_W32_WriteFile(fFTDHandle, fPString, fLength, @fErrorCode, @fOverlapped)) then
        begin
        if (FT_W32_GetLastError(fFTDHandle)=ERROR_IO_PENDING) then
           begin
           while not(TOC.Elapsed) and not(FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false)) do;
           Result:=(fErrorCode=dword(fLength));
           end;
        end
     else Result:=(fErrorCode=dword(fLength));
     end;
end;

function WaitUntilFTDCharsAvailableOrTimeOutElapsed(fFTDHandle: FT_HANDLE; CharCount: byte; TOC: TTimeOutController; TimeOut: dword): boolean;
var fErrorCode: dword;
    fStat: FTCOMSTAT;
    //fInQue: word;
begin
  if (FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat)) then
     begin
     if (fStat.cbInQue<CharCount) then
        begin
        TerminateWaitFTDeviceLoop:=false;
        if (TimeOut>0) then // wait until received char pending or timeout elapsed
           begin
           TOC.Initialize(TimeOut);
           // while not(TerminateWaitFTDeviceLoop) and not(TOC.Elapsed) and
           //      (FT_GetQueueStatus(fFTDHandle, @fInQue)=FT_OK) and (fInQue<CharCount) do;
           while not(TerminateWaitFTDeviceLoop) and not(TOC.Elapsed) and
                 FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat) and (fStat.cbInQue<CharCount) do;
           end;
        end;
     // Result:=not(TerminateWaitFTDeviceLoop) and not(fInQue<CharCount);
     Result:=not(TerminateWaitFTDeviceLoop) and not(fStat.cbInQue<CharCount);
     end
  else Result:=false;
end;

function GetCharFromFTD(fFTDHandle: FT_HANDLE; var Ch: char): boolean;
var fOverlapped: TOverlapped;
    fPString: TTwoChars;
    fErrorCode: dword;
begin
  // If at least one character is available, this function will perform the IO read operation
  // and will return with true (and set Ch), otherwise it will return immediately with false.
  Result:=false;
  // Note: The function "GetFTDCharsInQue" will also check if the fFTDHandle value is valid
  if (GetFTDCharsInQue(fFTDHandle)>0) then
     begin
     FillChar(fOverlapped,SizeOf(fOverlapped),0);
     if FT_W32_ReadFile(fFTDHandle, fPString, 1, @fErrorCode, @fOverlapped) and
        FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false) and
        (fErrorCode=1) then
        begin
        Ch:=fPString[0];
        Result:=true;
        end;
     end;
end;


function GetCharFromFTD(fFTDHandle: FT_HANDLE; var Ch: char; TOC: TTimeOutController; TimeOut: dword): boolean;
var fOverlapped: TOverlapped;
    fPString: TTwoChars;
    fErrorCode: dword;
begin
  // This function will wait until at least one character is available (and perform IO read operation
  // and set Ch value) or until the TimeOut is elapsed (and return in this case with false).
  // Used with parameter TimeOut=0 it is identical as the previous overloaded version.
  Result:=false;
  if (fFTDHandle <> FT_INVALID_HANDLE) then
     begin
     if WaitUntilFTDCharsAvailableOrTimeOutElapsed(fFTDHandle,1,TOC,TimeOut) then
        begin
        FillChar(fOverlapped,SizeOf(fOverlapped),0);
        if FT_W32_ReadFile(fFTDHandle, fPString, 1, @fErrorCode, @fOverlapped) and
           FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false) and
           (fErrorCode=1) then
           begin
           Ch:=fPString[0];
           Result:=true;
           end;
        end;
     end;
end;

function GetStringFromFTD(fFTDHandle: FT_HANDLE; CharCount: byte; BreakReadOnEOS: boolean; EOSChar: char): string; overload;
var fOverlapped: TOverlapped;
    fPString: TCharBuffer;
    fTempReceive: string;
    fLength,I: integer;
    fErrorCode: dword;
    fStat: FTCOMSTAT;
begin
  fTempReceive:='';
  if (CharCount>0) and
     (fFTDHandle <> FT_INVALID_HANDLE) and
     (FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat)) then
     begin
     if not(fStat.cbInQue<CharCount) then fLength:=CharCount else fLength:=fStat.cbInQue;
     if (fLength>CMaxBufferLength) then fLength:=CMaxBufferLength;
     if (fLength>0) then
        begin
        FillChar(fOverlapped,SizeOf(fOverlapped),0);
        if not(BreakReadOnEOS) then
           begin
           if FT_W32_ReadFile(fFTDHandle, fPString, fLength, @fErrorCode, @fOverlapped) and
              FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false) and
              (fErrorCode>0) then
              for I:=0 to fErrorCode-1 do
                  fTempReceive:=fTempReceive+fPString[I];
           end
        else  // BreakReadOnEOS
           begin
           for I:=1 to fLength do
               begin
               if FT_W32_ReadFile(fFTDHandle, fPString, 1, @fErrorCode, @fOverlapped) and
                  FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false) and
                  (fErrorCode=1) then
                  begin
                  fTempReceive:=fTempReceive+fPString[0];
                  if (fPString[0]=EOSChar) then break;
                  end
               else break;
               end;
           end;
        end;
     end;
  Result:=fTempReceive;
end;

function GetStringFromFTD(fFTDHandle: FT_HANDLE; CharCount: byte; BreakReadOnEOS: boolean; EOSChar: char; TOC: TTimeOutController; TimeOut: dword): string; overload;
var fOverlapped: TOverlapped;
    fPString: TCharBuffer;
    fTempReceive: string;
    fLength,I: integer;
    fErrorCode: dword;
    fStat: FTCOMSTAT;
    // fInQue: word;
begin
  fTempReceive:='';
  if (CharCount>0) and
     (fFTDHandle <> FT_INVALID_HANDLE) and
     (FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat)) then
     begin
     if not(BreakReadOnEOS) then
        begin
        if WaitUntilFTDCharsAvailableOrTimeOutElapsed(fFTDHandle,CharCount,TOC,TimeOut) then
           fLength:=CharCount
        // else if (FT_GetQueueStatus(fFTDHandle, @fInQue)=FT_OK) then
        //        fLength:=fInQue
        else if FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat) then
                fLength:=fStat.cbInQue
             else fLength:=0;
        if (fLength>0) then
           begin
           FillChar(fOverlapped,SizeOf(fOverlapped),0);
           if FT_W32_ReadFile(fFTDHandle, fPString, fLength, @fErrorCode,@fOverlapped) and
              FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false) and
              (fErrorCode>0) then
              for I:=0 to fErrorCode-1 do
                  fTempReceive:=fTempReceive+fPString[I];
           end;
        end
     else // BreakReadOnEOS
        begin
        if (TimeOut>0) then
           begin
           FillChar(fOverlapped,SizeOf(fOverlapped),0);
           TOC.Initialize(TimeOut);
           TerminateWaitFTDeviceLoop:=false;
           while not(TerminateWaitFTDeviceLoop) and not(TOC.Elapsed) do
                 begin
                 if FT_W32_ClearCommError(fFTDHandle, @fErrorCode, @fStat) and
                    (fStat.cbInQue>0) then
                    begin
                    fErrorCode:=FT_W32_GetLastError(fFTDHandle);
                    if FT_W32_ReadFile(fFTDHandle, fPString, 1, @fErrorCode, @fOverlapped) and
                       FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false) and
                       (fErrorCode=1) then
                       begin
                       fTempReceive:=fTempReceive+fPString[0];
                       if (fPString[0]=EOSChar) or
                          not(System.Length(fTempReceive)<CharCount) then break;
                       end
                    else break;
                    end;
                 end;
           end
        else  // not(TimeOut>0)
           begin
           if not(fStat.cbInQue<CharCount) then fLength:=CharCount else fLength:=fStat.cbInQue;
           if (fLength>CMaxBufferLength) then fLength:=CMaxBufferLength;
           if (fLength>0) then
              begin
              FillChar(fOverlapped,SizeOf(fOverlapped),0);
              for I:=1 to fLength do
                  begin
                  if FT_W32_ReadFile(fFTDHandle, fPString, 1, @fErrorCode, @fOverlapped) and
                     FT_W32_GetOverlappedResult(fFTDHandle, @fOverlapped, @fErrorCode, false) and
                     (fErrorCode=1) then
                     begin
                     fTempReceive:=fTempReceive+fPString[0];
                     if (fPString[0]=EOSChar) then break;
                     end
                  else break;
                  end;
              end;
           end;
        end;
     end;

  Result:=fTempReceive;
end;

// TFTDReceiver

{$IFDEF WithThread}
constructor TFTDReceiver.Create;
begin
  inherited Create(true);

  pFTDHandle:=nil;
  pBreakReadOnEOS:=nil;
  pRemoveEOSOnRead:=nil;
  pSimulationMode:=nil;

  pEOSchar:=nil;
  pReadTimeOut:=nil;
  pMaxStringLength:=nil;
  pReceptionString:=nil;
  pSimulationString:=nil;

  fOnThreadReceiveChar:=nil;
  fOnThreadReceptionFinished:=nil;

  fOnThreadReadChar:=nil;
  fOnThreadReadString:=nil;
  fOnThreadReceptionTimeOut:=nil;
  fOnThreadReadTermination:=nil;
end;

destructor TFTDReceiver.Destroy;
begin

  inherited;
end;

procedure TFTDReceiver.Resume;
begin
  if not(fEnabled) then
     begin
     fReceptionFinished:=false;
     fReceptionTimeOut:=false;
     fThreadReceptionString:='';

     inherited Resume;
     fEnabled:=true;
     end;
end;

procedure TFTDReceiver.Suspend;
begin
  if fEnabled then
     begin
     fEnabled:=false;
     while not(fDisabled) do;
     inherited Suspend;
     end;
end;

function TFTDReceiver.GetSimulatedChar(var Ch: char): boolean;
begin
  if (pSimulationString<>nil) and
     (pSimulationString^<>'') then
     begin
     Ch:=pSimulationString^[1];
     System.Delete(pSimulationString^,1,1);
     Result:=true;
     end
  else Result:=false;
end;

procedure TFTDReceiver.Execute;
var StringLength: word;
    NewChar: char;
begin
  StringLength:=0;

  while True do
        begin
        if fEnabled then
           begin
           if fDisabled then fDisabled:=false;

           if (pFTDHandle<>nil) and
              (pSimulationMode<>nil) then
              begin

              if (not(pSimulationMode^) and
                 (GetFTDCharsInQue(pFTDHandle^)>0) and
                 GetCharFromFTD(pFTDHandle^,NewChar)) or
                 (pSimulationMode^ and (GetSimulatedChar(NewChar))) then
                 begin

                 if Assigned(fOnThreadReceiveChar) then fOnThreadReceiveChar;

                 if Assigned(fOnThreadReadChar) then fOnThreadReadChar(NewChar)
                 else // not(Assigned(fOnThreadReadChar))
                    begin
                    if (fThreadReceptionString='') then
                       begin
                       fReceptionBegin:=GetTickCount;
                       if (pReadTimeOut<>nil) then
                          fReceptionTimeOutLimit:=fReceptionBegin+pReadTimeOut^
                       else fReceptionTimeOutLimit:=fReceptionBegin+CDefaultReadTimeOut;
                       end;

                    fThreadReceptionString:=fThreadReceptionString+NewChar;
                    StringLength:=System.Length(fThreadReceptionString);

                    if (StringLength=10) then // wait more patiently for long strings in reception
                       begin
                       if (pReadTimeOut<>nil) then
                          fReceptionTimeOutLimit:=fReceptionTimeOutLimit+pReadTimeOut^
                       else fReceptionTimeOutLimit:=fReceptionTimeOutLimit+CDefaultReadTimeOut;
                       end;

                    fReceptionFinished:=((pBreakReadOnEOS<>nil) and (pBreakReadOnEOS^) and (pEOSchar<>nil) and (NewChar=pEOSChar^)) or
                                         ((pMaxStringLength<>nil) and not(StringLength<pMaxStringLength^));
                    end;
                 if not(fReceptionFinished) and
                    Assigned(fOnThreadReadTermination) then
                    fOnThreadReadTermination(fThreadReceptionString,GetTickCount-fReceptionBegin,fReceptionFinished);
                 end
              else // no any char available, so control the time out
                 begin
                 if (fThreadReceptionString<>'') and not(fReceptionTimeOut) then
                    fReceptionTimeOut:=not(fReceptionTimeOutLimit>GetTickCount);
                 end;

              if fReceptionFinished or fReceptionTimeOut then
                 begin
                 if ((pRemoveEOSOnRead<>nil) and (pRemoveEOSOnRead^) and (pEOSchar<>nil) and (NewChar=pEOSChar^)) then
                    System.Delete(fThreadReceptionString,StringLength,1);

                 if Assigned(fOnThreadReceptionFinished) then fOnThreadReceptionFinished;

                 if not(fReceptionTimeOut) then
                    begin
                    if Assigned(fOnThreadReadString) then
                       fOnThreadReadString(fThreadReceptionString)
                    else if (pReceptionString<>nil) then
                            begin
                            while (pReceptionString^<>'') do; // wait ages until serviced
                            pReceptionString^:=fThreadReceptionString;
                            end;
                    end
                 else // ReceptionTimeOut
                    begin
                    if Assigned(fOnThreadReceptionTimeOut) then
                       fOnThreadReceptionTimeOut(fThreadReceptionString)
                    else if (pReceptionString<>nil) then
                            begin
                            while (pReceptionString^<>'') do; // wait ages until serviced
                            pReceptionString^:=fThreadReceptionString;
                            end;
                    end;

                 fReceptionFinished:=false;
                 fReceptionTimeOut:=false;
                 fThreadReceptionString:='';
                 end;
              end;
           end
        else // not(fEnabled)
           if not(fDisabled) then fDisabled:=true;
        end;
end;
{$ENDIF}

// TCustomFTDevice

function TCustomFTDevice.GetFTDOpenStatus: boolean;
begin
  Result:=(fFTDHandle <> FT_INVALID_HANDLE);
end;

procedure TCustomFTDevice.SetOpenMode(Value: TFTDOpenMode);
begin
  if not(GetFTDOpenStatus) and (fOpenMode<>Value) then fOpenMode:=Value;
end;

procedure TCustomFTDevice.SetDeviceNumber(Value: dword);
begin
  if not(GetFTDOpenStatus) and (fFTDNumber<>Value) and (Value in [0..CMaxFTDeviceCount]) then fFTDNumber:=Value;
end;

procedure TCustomFTDevice.SetDeviceName(Value: TFTDeviceNameString);
begin
  if not(GetFTDOpenStatus) and (fDeviceName<>Value) then fDeviceName:=Value;
end;

procedure TCustomFTDevice.SetSerialNumber(Value: TFTSerialNumberString);
begin
  if not(GetFTDOpenStatus) and (fSerialNumber<>Value) then fSerialNumber:=Value;
end;

procedure TCustomFTDevice.SetBaudRate(Value: dword);
begin
  if not(GetFTDOpenStatus) and (fBaudRate<>Value) and
     (not(fValidateBaudRate) or AcceptableFTDBaudRate(Value)) then
     begin
     fBaudRate:=Value;
     fCharTransmitDuration:=(1000000*(fByteSize+fStopBits+1)) DIV fBaudRate;
     end;
end;

procedure TCustomFTDevice.SetByteSize(Value: byte);
begin
  if not(GetFTDOpenStatus) and (fByteSize<>Value) and (Value in [4..8]) then
     begin
     fByteSize:=Value;
     fCharTransmitDuration:=(1000000*(fByteSize+fStopBits+1)) DIV fBaudRate;
     end;
end;

procedure TCustomFTDevice.SetStopBits(Value: byte);
begin
  if not(GetFTDOpenStatus) and (fStopBits<>Value) and (Value in [ONESTOPBIT..TWOSTOPBITS]) then
     begin
     fStopBits:=Value;
     fCharTransmitDuration:=(1000000*(fByteSize+fStopBits+1)) DIV fBaudRate;
     end;
end;

procedure TCustomFTDevice.SetParityMode(Value: byte);
begin
  if not(GetFTDOpenStatus) and (fParityMode<>Value) and (Value in [NOPARITY..SPACEPARITY]) then fParityMode:=Value;
end;

procedure TCustomFTDevice.SetEOSChar(Value: char);
begin
  if not(GetFTDOpenStatus) and (fEOSChar<>Value) then fEOSChar:=Value;
end;

procedure TCustomFTDevice.SetXonChar(Value: char);
begin
  if not(GetFTDOpenStatus) and (fXonChar<>Value) then fXonChar:=Value;
end;

procedure TCustomFTDevice.SetXoffChar(Value: char);
begin
  if not(GetFTDOpenStatus) and (fXoffChar<>Value) then fXoffChar:=Value;
end;

procedure TCustomFTDevice.SetDTRControl(Value: byte);
begin
  if not(GetFTDOpenStatus) and (fDTRControl<>Value) and (Value in [DTR_CONTROL_DISABLE..DTR_CONTROL_HANDSHAKE]) then fDTRControl:=Value;
end;

procedure TCustomFTDevice.SetRTSControl(Value: byte);
begin
  if not(GetFTDOpenStatus) and (fRTSControl<>Value) and (Value in [RTS_CONTROL_DISABLE..RTS_CONTROL_TOGGLE]) then fRTSControl:=Value;
end;

procedure TCustomFTDevice.SetOutputControlMode(Value: TOutputControlMode);
begin
  if not(GetFTDOpenStatus) and (fOutputControlMode<>Value) and (Value in [ocNone..ocCtsDsr]) then fOutputControlMode:=Value;
end;

procedure TCustomFTDevice.SetXonXoffControlMode(Value: TXonXoffControlMode);
begin
  if not(GetFTDOpenStatus) and (fXonXoffControlMode<>Value) and (Value in [xcNone..xcInputOutput]) then fXonXoffControlMode:=Value;
end;

procedure TCustomFTDevice.SetInputDSRSensivity(Value: boolean);
begin
  if not(GetFTDOpenStatus) and (fInputDSRSensivity<>Value) then fInputDSRSensivity:=Value;
end;

procedure TCustomFTDevice.SetMaxStringLength(Value: word);
begin
  if (Value<>fMaxStringLength) and not(Value>1000) then
     fMaxStringLength:=Value;
end;

procedure TCustomFTDevice.SetValidateBaudRate(Value: boolean);
begin
  if not(GetFTDOpenStatus) and (fValidateBaudRate<>Value) then
     begin
     fValidateBaudRate:=Value;
     if fValidateBaudRate and not(AcceptableFTDBaudRate(fBaudRate)) then
        begin
        fBaudRate:= CDefaultFTDBaudRate;
        fCharTransmitDuration:=(1000000*(fByteSize+fStopBits+1)) DIV fBaudRate;
        end;
     end;
end;

procedure TCustomFTDevice.SetCharTransmitInterval(Value: word);
begin
  // Don't accept Char Transmit Interval > 1000
  if (fCharTransmitInterval<>Value) and not(fCharTransmitInterval>1000) then
     fCharTransmitInterval:=Value;
end;

procedure TCustomFTDevice.SetWriteTimeOut(Value: word);
begin
  // Don't accept Time Out > 10000 (which is 10s)
  if (fWriteTimeOut<>Value) and not(fWriteTimeOut>10000) then
     fWriteTimeOut:=Value;
end;

procedure TCustomFTDevice.SetReadTimeOut(Value: word);
begin
  // Don't accept Time Out > 60000 (which is 60s = 1min)
  if (fReadTimeOut<>Value) and not(fReadTimeOut>60000) then
     fReadTimeOut:=Value;
end;

function TCustomFTDevice.GetCTSLineStatus: boolean;
begin
  Result:=GetFTD_CTSLine(fFTDHandle);
end;

function TCustomFTDevice.GetDSRLineStatus: boolean;
begin
  Result:=GetFTD_DSRLine(fFTDHandle);
end;

function TCustomFTDevice.PutString(Source: string): boolean;
var fMessageTransmissionTime: word;
begin
  if (fWriteTimeOut=0) then
     Result:=PutStringToFTD(fFTDHandle,Source)
  else
     begin
     fMessageTransmissionTime:=System.Round(System.Length(Source)*fCharTransmitDuration/1000);
     Result:=PutStringToFTD(fFTDHandle,Source,fSmallTimeOut,fMessageTransmissionTime+fWriteTimeOut);
     end;
end;

function TCustomFTDevice.TransmitString(Source: string): boolean;
var fLength,I: integer;
begin
  if (fCharTransmitInterval=0) then
     begin
     if not(Assigned(fOnTransmitChar)) then
        Result:=PutString(Source)
     else // Assigned(fOnTransmitChar)
        begin
        Result:=true;
        fLength:=System.Length(Source);
        for I:=1 to fLength do
            begin
            if PutString(Source[I]) then fOnTransmitChar
            else
               begin
               Result:=false;
               break;
               end;
            end;
        end;
     end
  else  // fCharTransmitInterval > 0
     begin
     Result:=true;
     fLength:=System.Length(Source);
     if (fLength>CMaxBufferLength) then fLength:=CMaxBufferLength;
     for I:=1 to fLength do
         begin
         if PutString(Source[I]) then
            begin
            WaitMicroseconds(fCharTransmitDuration+fCharTransmitInterval*1000);
            if Assigned(fOnTransmitChar) then fOnTransmitChar;
            end
         else
            begin
            Result:=false;
            break;
            end;
         end;
     end;
end;

function TCustomFTDevice.SimulateTransmission (Source: string): boolean;
var fLength,I: integer;
begin
  fLength:=System.Length(Source);

  if Assigned(fOnTransmitChar) then
     begin
     for I:=1 to fLength do
         begin
         WaitMicroseconds(fCharTransmitDuration+fCharTransmitInterval*1000);
         fOnTransmitChar;
         end;
     end
  else WaitMilliseconds((fCharTransmitDuration*fLength) DIV 1000);
  Result:=true;
end;


function TCustomFTDevice.CharsInQue: word;
begin
  Result:=GetFTDCharsInQue(fFTDHandle);
end;

function TCustomFTDevice.CharsOutQue: word;
begin
  Result:=GetFTDCharsOutQue(fFTDHandle);
end;

function TCustomFTDevice.GetChar(var Target: char): boolean;
var C: char;
begin
  if (fReadTimeOut=0) then
     Result:=GetCharFromFTD(fFTDHandle,C)
  else Result:=GetCharFromFTD(fFTDHandle,C,fSmallTimeOut,fReadTimeOut);
  if Result then Target:=C;
end;

function TCustomFTDevice.GetString(CharCount: byte): string;
begin
  if (fReadTimeOut=0) then
     Result:=GetStringFromFTD(fFTDHandle,CharCount,fBreakReadOnEOS,fEOSChar)
  else Result:=GetStringFromFTD(fFTDHandle,CharCount,fBreakReadOnEOS,fEOSChar,fSmallTimeOut,fReadTimeOut);
end;

function TCustomFTDevice.EvaluateFTDSettings: TFTDSettings;
var lFTDSettings: TFTDSettings;
begin
  with lFTDSettings do
       begin
       FTDOpenMode:=fOpenMode;
       DeviceName:=fDeviceName;
       SerialNumber:=fSerialNumber;
       DeviceNumber:=fFTDNumber;

       BaudRate:=fBaudRate;
       ByteSize:=fByteSize;
       StopBits:=fStopBits;
       Parity:=fParityMode;
       Flags:=0;
       // create DCB Flag items
       case fDTRControl of
            DTR_CONTROL_ENABLE:    Flags:=Flags or FT_dcb_DtrControlEnable;
            DTR_CONTROL_HANDSHAKE: Flags:=Flags or FT_dcb_DtrControlHandshake;
            end;
       case fRTSControl of
            RTS_CONTROL_ENABLE:    Flags:=Flags or FT_dcb_RtsControlEnable;
            RTS_CONTROL_HANDSHAKE: Flags:=Flags or FT_dcb_RtsControlHandshake;
            RTS_CONTROL_TOGGLE:    Flags:=Flags or FT_dcb_RtsControlToggle;
            end;
       case fOutputControlMode of
            ocCtsOnly: Flags:=Flags or FT_dcb_OutxCtsFlow;
            ocDsrOnly: Flags:=Flags or FT_dcb_OutxDsrFlow;
            ocCtsDsr:  Flags:=Flags or FT_dcb_OutxCtsFlow or FT_dcb_OutxDsrFlow;
            end;
       if fInputDSRSensivity then Flags:=Flags or FT_dcb_DsrSensitivity;
       case fXonXoffControlMode of
            xcInputOnly:   Flags:=Flags or FT_dcb_InX;
            xcOutputOnly:  Flags:=Flags or FT_dcb_OutX;
            xcInputOutput: Flags:=Flags or FT_dcb_InX or FT_dcb_OutX;
            end;
       if (fXonXoffControlMode<>xcNone) then
          begin
          XonChar:=fXonChar;
          XoffChar:=fXoffChar;
          end;
       EOSChar:=fEOSChar;
       end;
  Result:=lFTDSettings;
end;

constructor TCustomFTDevice.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);

  fFTDNumber:=CMaxFTDeviceCount+1;

  fFTDHandle:=FT_INVALID_HANDLE;
  fSmallTimeOut := TTimeOutController.Create;
  fLargeTimeOut := TTimeOutController.Create;

  fOpenMode:=omByNumber;

  fBaudRate:=CDefaultFTDBaudRate;
  fByteSize:=FT_DATA_BITS_8;  // 8
  fStopBits:=FT_STOP_BITS_1;  // 1
  fCharTransmitDuration:=(1000000*(fByteSize+fStopBits+1)) DIV fBaudRate;

  fParityMode:=NOPARITY;

  fEOSChar:=CLF;
  fXonChar:=CXon;
  fXoffChar:=CXoff;

  fDTRControl:=DTR_CONTROL_DISABLE;
  fRTSControl:=RTS_CONTROL_DISABLE;
  fOutputControlMode:=ocNone;
  fXonXoffControlMode:=xcNone;

  fInputDSRSensivity:=false;

  fAppendEOSonWrite:=false;
  fBreakReadOnEOS:=false;
  fRemoveEOSonRead:=false;

  fValidateBaudRate:=false;

  fSimulationMode:=false;
  fFTDSimulationString:='';

  fCharTransmitInterval:=0;
  fCharTransmitDuration:=0;

  fWriteTimeOut:=0;
  fReadTimeOut:=0;
  fMaxStringLength:=CDefaultMaxStringLength;

{$IFDEF WithThread}
  fFTDReceiver:=TFTDReceiver.Create;
  with fFTDReceiver do
       begin
       FreeOnTerminate:=true;
       Priority:=tpNormal;

       pFTDHandle:=@fFTDHandle;
       pBreakReadOnEOS:=@fBreakReadOnEOS;
       pRemoveEOSOnRead:=@pRemoveEOSOnRead;
       pSimulationMode:=@fSimulationMode;

       pEOSchar:=@fEOSchar;

       pReadTimeOut:=@fReadTimeOut;
       pMaxStringLength:=@fMaxStringLength;
       pReceptionString:=@fFTDReceptionString;
       pSimulationString:=@fFTDSimulationString;

       OnThreadReceiveChar:=nil;
       OnThreadReceptionFinished:=nil;

       OnThreadReadChar:=nil;
       OnThreadReadString:=nil;
       OnThreadReceptionTimeOut:=nil;
       OnThreadReadTermination:=nil;
       end;
  fFTDReceptionString:='';
{$ENDIF}
end;

destructor TCustomFTDevice.Destroy;
begin
  if (GetFTDOpenStatus) then CloseHandle(fFTDHandle);
  fSmallTimeOut.Free;
  fLargeTimeOut.Free;
  inherited Destroy;
end;

function TCustomFTDevice.Open: boolean;
begin
  if ((fOpenMode=omByNumber) and (fFTDNumber>0)) or
     ((fOpenMode=omByDescription) and (fDeviceName<>'')) or
     ((fOpenMode=omBySerialNumber) or (fSerialNumber<>'')) then
     begin
     fFTDHandle:=OpenFTDIDevice(EvaluateFTDSettings);
     fSimulationMode:=false;
     Result:=GetFTDOpenStatus;
     end
  else
     begin
     fSimulationMode:=true;
     Result:=true;
     end;

{$IFDEF WithThread}
  if Result then
     begin
     with fFTDReceiver do
          begin
          if Assigned(fOnReceiveChar) then
             OnThreadReceiveChar:=OnReceiveChar;
          if Assigned(fOnReceptionFinished) then
             OnThreadReceptionFinished:=OnReceptionFinished;

          if Assigned(fOnReadChar) then
             OnThreadReadChar:=OnReadChar;
          if Assigned(fOnReadString) then
             OnThreadReadString:=OnReadString;
          if Assigned(fOnReceptionTimeOut) then
             OnThreadReceptionTimeOut:=OnReceptionTimeOut;
          if Assigned(fOnReadTermination) then
             OnThreadReadTermination:=OnReadTermination;

          Resume;
          end;
     end;
{$ENDIF}
end;

procedure TCustomFTDevice.Close;
begin
{$IFDEF WithThread}
  if not(fFTDReceiver.Suspended) then
     fFTDReceiver.Suspend;
{$ENDIF}
  fSmallTimeOut.Terminate;
  fLargeTimeOut.Terminate;
  CloseFTDIDevice(fFTDHandle);
  fSimulationMode:=false;
end;

procedure TCustomFTDevice.PurgeReadBuffer;
begin
  if not(csDesigning in ComponentState) then PurgeFTDReadBuffer(fFTDHandle);
end;

procedure TCustomFTDevice.PurgeWriteBuffer;
begin
  if not(csDesigning in ComponentState) then PurgeFTDWriteBuffer(fFTDHandle);
end;

procedure TCustomFTDevice.PurgeReadWriteBuffer;
begin
  if not(csDesigning in ComponentState) then PurgeFTDReadWriteBuffer(fFTDHandle);
end;

function TCustomFTDevice.GetCTSandDSRLineStatus: TModemStatus;
begin
  Result:=GetFTD_CTSandDSRLines(fFTDHandle);
end;

function TCustomFTDevice.WriteString(Source: string): boolean;
begin
  if fAppendEOSonWrite then Source:=Source+fEOSChar;

  if (not(fSimulationMode) and TransmitString(Source)) or
     (fSimulationMode and SimulateTransmission(Source)) then
     begin
     if (Assigned(fOnTransmissionFinished)) then fOnTransmissionFinished;
     Result:=true;
     end
  else Result:=false;
end;

function TCustomFTDevice.ReadString(var Target: string; CharCount: byte): boolean;
var fTarget: string;
{$IFNDEF WithThread}
    fLength,I: integer;
    fC: char;
{$ENDIF}
begin
{$IFDEF WithThread}
  if (CharCount>0) and
     not(Assigned(fOnReadChar)) and
     not(Assigned(fOnReadString)) and
     not(Assigned(fOnReceptionTimeOut)) then
     begin

     fLargeTimeOut.Initialize(fReadTimeOut);
     while (fFTDReceptionString='') and not(fLargeTimeOut.Elapsed) do;

     fTarget:=fFTDReceptionString;
     fFTDReceptionString:='';
     Result:=true;
{$ELSE}
  if (CharCount>0) then
     begin
     if not(fSimulationMode) then
        begin
        if not(Assigned(fOnReceiveChar)) then
           begin
           fTarget:=GetString(CharCount);
           fLength:=System.Length(fTarget);
           end
        else // Assigned(fOnReceiveChar)
           begin
           fTarget:=''; fLength:=0;
           fLargeTimeOut.Initialize(fReadTimeOut);
           while not(fLargeTimeOut.Elapsed) do
                 begin
                 if GetChar(fC) then
                    begin
                    if Assigned(fOnReceiveChar) then fOnReceiveChar;
                    fTarget:=fTarget+fC;
                    fLength:=System.Length(fTarget);
                    if fBreakReadOnEOS and (fC=fEOSChar) then break;
                    end;
                 end;
            end;
        end
     else  // fSimulationMode
        begin
        if Assigned(fOnReadStringSimulation) then fOnReadStringSimulation(fTarget)
        else
           begin
           fTarget:=fFTDSimulationString;
           fFTDSimulationString:='';
           end;
        fLength:=System.Length(fTarget);
        if (Assigned(fOnReceiveChar)) then
           begin
           for I:=1 to fLength do
               begin
               WaitMilliseconds((fCharTransmitDuration DIV 1000)+1);
               if Assigned(fOnReceiveChar) then fOnReceiveChar;
               end;
           end;
        end;

     if fRemoveEOSonRead and (fLength>0) and (fTarget[fLength]=fEOSChar) then System.Delete(fTarget,fLength,1);
     if (fLength>0) and ((fLength=CharCount) or (fBreakReadOnEOS and (fTarget[fLength]=fEOSChar))) then
        begin
        if Assigned(fOnReceptionFinished) then fOnReceptionFinished;
        Result:=true;
        end
     else Result:=false;
{$ENDIF}
     end
  else
     begin
     fTarget:='';
     Result:=false;
     end;
  Target:=fTarget;
end;

function TCustomFTDevice.QueryString(Question: string; BreakTime: integer; var Answer: string; CharCount: byte): boolean;
begin
  if WriteString(Question) then
     begin
     WaitMilliseconds(BreakTime);
     Result:=ReadString(Answer,CharCount);
     end
  else Result:=false;
end;

function TCustomFTDevice.ForceReceiverSimulation(Value: string): boolean;
begin
  // Attention! don't use the function "ForceReceiverSimulation"
  // and assign Event "OnReadStringSimulation" in the same compilation
  // just as a simple rule: use "ForceReceiverSimulation" for slave
  // and "OnReadStringSimulation" for master operation.
  if fSimulationMode and
     not(Assigned(fOnReadStringSimulation)) then
     begin
     fFTDSimulationString:=Value;
     Result:=true;
     end
  else Result:=false;
end;

end.
