//******************************************************************************
//******************************************************************************
//******************************************************************************
//**
//**  STRINGUTILS.PAS
//**
//**  Different helpful functions and procedures for string handling
//**  Derived on 22.07.2008 from "HelpfulUtils.Pas"
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.08.96 / 22.07.08 / ... / 12.01.10 / 04.12.13
//**
//******************************************************************************
//******************************************************************************
//******************************************************************************
unit StringUtils;

interface

uses Types, SysUtils, Classes, ArraysAndGrids;

const
  CMaxStringLength = 65536;
  CMaxBufferLength = 1023;

  CEOT  = ^D;  // #4
  CTAB  = ^I;  // #9
  CLF   = ^J;  // #10
  CCR   = ^M;  // #13
  CXon  = ^Q;  // #17
  CXOff = ^S;  // #19
  CHide = ^Z;  // #26

  CHideRest = ^M+^J+^Z;

  CUpperLetters = ['A'..'Z'];
  CLowerLetters = ['a'..'z'];
  CAllLetters   = ['A'..'Z','a'..'z'];

type

  TTwoChars   = array [0..1] of char;
  TCharBuffer = array [0..CMaxBufferLength] of char;

  function CopyPChar(Source: PChar): string; overload;
  function CopyPChar(Source: PChar; MaxLength: integer): string; overload;

  function StrCopyBuff(var Target: TCharBuffer; const Source: string): boolean;
  function BuffCopyStr(Source: TCharBuffer; MaxLength: integer): string;

  function GetSysCharSetPosition(Value: TSysCharSet; Source: AnsiString): integer;

  function DeleteLeadingChars (S: string; Ch: char): string; overload;
  function DeleteLeadingChars (S: string; CharSet: TSysCharSet): string; overload;
  function DeleteLeadingCharsUntil (S: string; Ch: char): string; overload;
  function DeleteLeadingCharsUntil (S: string; CharSet: TSysCharSet): string; overload;

  function DeleteTrailingChars (S: string; Ch: char): string; overload;
  function DeleteTrailingChars (S: string; CharSet: TSysCharSet): string; overload;
  function DeleteTrailingCharsUntil (S: string; Ch: char): string; overload;
  function DeleteTrailingCharsUntil (S: string; CharSet: TSysCharSet): string; overload;

  function DeleteAllChars (S: string; Ch: char): string; overload;
  function DeleteAllChars (S: string; CharSet: TSysCharSet): string; overload;

  function DeleteAcceleratorKey(S: string): string;

  function CopyStringUntil(S: string; Ch: char): string; overload;
  function CopyStringUntil(S: string; CharSet: TSysCharSet): string; overload;

  function ReplaceSubstrings(Source,Before,After: string): string;

  function SplitStringToStringDynArray(Source: string; SepChar: char): TStringDynArray; overload;
  function SplitStringToStringDynArray(Source: string; SepChars: TSysCharSet): TStringDynArray; overload;

  function SplitStringToStringDynArray(Source: string; SepChar: char; SuppressBlanks: boolean): TStringDynArray; overload;
  function SplitStringToStringDynArray(Source: string; SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynArray; overload;

  function SplitStringToStringList(Source: string; SepChar: char; var Target: TStringList): integer; overload;
  function SplitStringToStringList(Source: string; SepChar: char; SuppressBlanks: boolean; var Target: TStringList): integer; overload;

  function SplitStringToStringList(Source: string; SepChars: TSysCharSet; var Target: TStringList): integer; overload;
  function SplitStringToStringList(Source: string; SepChars: TSysCharSet; SuppressBlanks: boolean; var Target: TStringList): integer; overload;

  function GetRowFromStringDynGrid(Source: TStringDynGrid; RowNumber: integer): TStringDynArray;
  function GetColumnFromStringDynGrid(Source: TStringDynGrid; ColumnNumber: integer): TStringDynArray;

  function CompareStringDynArrays(A,B: TStringDynArray): boolean;

  function GetMaxStringListLength(Source: TStringList): integer;

  function RestoreRessourceStringControls (Source: string): string;

  procedure HookResourceString(ResStringRec: pResStringRec; NewStr: pChar) ;

implementation

uses Windows;

function CopyPChar(Source: PChar): string;
var I: integer;
begin
  Result:=''; I:=0;
  while (Source[I] <> #0) and (I<CMaxStringLength-1) do
        begin
        Result:=Result+Source[I];
        Inc(I);
        end;
end;

function CopyPChar(Source: PChar; MaxLength: integer): string;
var I: integer;
begin
  Result:=''; I:=0;
  while (Source[I] <> Chr(0)) and (I<MaxLength) do
        begin
        Result:=Result+Source[I];
        Inc(I);
        end;
end;

function StrCopyBuff(var Target: TCharBuffer; const Source: string): boolean;
var I, L: integer;
begin
  L:=System.Length(Source);
  if (L<CMaxBufferLength) then
     begin
     for I:=0 to L-1 do
         Target[I]:=Source[I+1];
     Result:=true;
     end
  else Result:=false;
end;

function BuffCopyStr(Source: TCharBuffer; MaxLength: integer): string;
begin

end;

function GetSysCharSetPosition(Value: TSysCharSet; Source: AnsiString): integer;
var IP,IL: integer;
begin
  IP:=0;
  for IL:=1 to System.Length(Source) do
      if (Source[IL] in Value) then
         begin
         IP:=IL; break;
         end;
  Result:=IP;
end;

function DeleteLeadingChars (S: string; Ch: char): string;
begin
  while (S<>'') and (S[1]=Ch) do System.Delete (S,1,1);
  Result:=S;
end;

function DeleteLeadingChars (S: string; CharSet: TSysCharSet): string;
begin
  while (S<>'') and (S[1] in CharSet) do System.Delete (S,1,1);
  Result:=S;
end;

function DeleteLeadingCharsUntil (S: string; Ch: char): string;
begin
  while (S<>'') and (S[1] <> Ch) do System.Delete (S,1,1);
  Result:=S;
end;

function DeleteLeadingCharsUntil (S: string; CharSet: TSysCharSet): string;
begin
  while (S<>'') and not(S[1] in CharSet) do System.Delete (S,1,1);
  Result:=S;
end;

function DeleteTrailingChars (S: string; Ch: char): string;
begin
  while (S<>'') and (S[System.Length(S)]=Ch) do System.Delete (S,System.Length(S),1);
  Result:=S;
end;

function DeleteTrailingChars (S: string; CharSet: TSysCharSet): string;
begin
  while (S<>'') and (S[System.Length(S)] in CharSet) do System.Delete (S,System.Length(S),1);
  Result:=S;
end;

function DeleteTrailingCharsUntil (S: string; Ch: char): string;
begin
  while (S<>'') and (S[System.Length(S)] <> Ch) do System.Delete (S,System.Length(S),1);
  Result:=S;
end;

function DeleteTrailingCharsUntil (S: string; CharSet: TSysCharSet): string;
begin
  while (S<>'') and not(S[System.Length(S)] in CharSet) do System.Delete (S,System.Length(S),1);
  Result:=S;
end;

function DeleteAllChars (S: string; Ch: char): string;
begin
  while (System.Pos(Ch,S) <> 0) do System.Delete (S,System.Pos(Ch,S),1);
  Result:=S;
end;

function DeleteAllChars (S: string; CharSet: TSysCharSet): string;
var I: integer;
begin
  if (S<>'') and (CharSet<>[]) then
     begin
     for I:=1 to System.Length(S) do
         while (S<>'') and not(I>System.Length(S)) and (S[I] in CharSet) do System.Delete (S,I,1);
     end;
  Result:=S;
end;

function DeleteAcceleratorKey(S: string): string;
begin
  Result:=DeleteAllChars(S,'&');
end;

function CopyStringUntil(S: string; Ch: char): string;
var I: integer;
begin
  Result:='';
  if (S<>'') then
     for I:=1 to System.Length(S) do
         if not(S[I]=Ch) then Result:=Result+S[I]
         else break;
end;

function CopyStringUntil(S: string; CharSet: TSysCharSet): string;
var I: integer;
begin
  Result:='';
  if (S<>'') then
     for I:=1 to System.Length(S) do
         if not(S[I] in CharSet) then Result:=Result+S[I]
         else break;
end;

function ReplaceSubstrings(Source,Before,After: string): string;
var P,L: integer;
    S: string;
begin
  P:=System.Pos(Before,Source);
  L:=System.Length(Before);
  if (P>0) and (L>0) then
     begin
     S:='';
     repeat
       S:=System.Copy(Source,1,P-1)+After;
       System.Delete(Source,1,P+L-1);
       Source:=S+Source;
       P:=System.Pos(Before,Source);
     until (P=0);
     end;
  Result:=Source;
end;

function SplitStringToStringDynArray(Source: string; SepChar: char): TStringDynArray;
var Target: TStringDynArray;
    I,P: integer;
begin
  if (Source<>'') then
     begin
     SetLength(Target,1); I:=0;
     repeat
       P:=System.Pos(SepChar,Source);
       if (P>0) then
          begin
          Target[I]:=System.Copy(Source,1,P-1);
          System.Inc(I);
          SetLength(Target,I+1);
          end
       else Target[I]:=Source;
       System.Delete(Source,1,P);
     until (P=0);
     Result:=Target;
     end
  else Result:=nil;
end;

function SplitStringToStringDynArray(Source: string; SepChars: TSysCharSet): TStringDynArray;
var Target: TStringDynArray;
    I,P: integer;
begin
  if (Source<>'') then
     begin
     SetLength(Target,1); I:=0;
     repeat
       P:=GetSysCharSetPosition(SepChars,Source);
       if (P>0) then
          begin
          Target[I]:=System.Copy(Source,1,P-1);
          System.Inc(I);
          SetLength(Target,I+1);
          end
       else Target[I]:=Source;
       System.Delete(Source,1,P);
     until (P=0);
     Result:=Target;
     end
  else Result:=nil;
end;

function SplitStringToStringDynArray(Source: string; SepChar: char; SuppressBlanks: boolean): TStringDynArray;
var Target: TStringDynArray;
    I,P: integer;
    S: string;
begin
  if (Source<>'') then
     begin
     I:=0; SetLength(Target,I);
     repeat
       P:=System.Pos(SepChar,Source);
       if (P>0) then
          begin
          S:=System.Copy(Source,1,P-1);
          if (S<>'') or not(SuppressBlanks) then
             AppendElementToArray(S,Target);
          end
       else
          begin
          S:=Source;
          if (S<>'') or not(SuppressBlanks) then
             AppendElementToArray(S,Target);
          end;
       System.Delete(Source,1,P);
     until (P=0);
     Result:=Target;
     end
  else Result:=nil;
end;

function SplitStringToStringDynArray(Source: string; SepChars: TSysCharSet; SuppressBlanks: boolean): TStringDynArray;
var Target: TStringDynArray;
    I,P: integer;
    S: string;
begin
  if (Source<>'') then
     begin
     I:=0; SetLength(Target,I);
     repeat
       P:=GetSysCharSetPosition(SepChars,Source);
       if (P>0) then
          begin
          S:=System.Copy(Source,1,P-1);
          if (S<>'') or not(SuppressBlanks) then
             AppendElementToArray(S,Target);
          end
       else
          begin
          S:=Source;
          if (S<>'') or not(SuppressBlanks) then
             AppendElementToArray(S,Target);
          end;
       System.Delete(Source,1,P);
     until (P=0);
     Result:=Target;
     end
  else Result:=nil;
end;

function SplitStringToStringList(Source: string; SepChar: char; var Target: TStringList): integer;
var P: integer;
begin
  Target.Clear;
  while (Source<>'') do
        begin
        P:=System.Pos(SepChar,Source);
        if (P>0) then
           begin
           Target.Add(System.Copy(Source,1,P-1));
           System.Delete(Source,1,P);
           end
        else  // Source=''
           begin
           Target.Add(Source);
           Source:='';
           end;
        end;
  Result:=Target.Count;
end;

function SplitStringToStringList(Source: string; SepChar: char; SuppressBlanks: boolean; var Target: TStringList): integer;
var P: integer;
    S: string;
begin
  Target.Clear;
  while (Source<>'') do
        begin
        P:=System.Pos(SepChar,Source);
        if (P>0) then
           begin
           S:=System.Copy(Source,1,P-1);
           if (S<>'') or not(SuppressBlanks) then Target.Add(S);
           System.Delete(Source,1,P);
           end
        else  // Source=''
           begin
           Target.Add(Source);
           Source:='';
           end;
        end;
  Result:=Target.Count;
end;

function SplitStringToStringList(Source: string; SepChars: TSysCharSet; var Target: TStringList): integer;
var P: integer;
begin
  Target.Clear;
  while (Source<>'') do
        begin
        P:=GetSysCharSetPosition(SepChars,Source);
        if (P>0) then
           begin
           Target.Add(System.Copy(Source,1,P-1));
           System.Delete(Source,1,P);
           end
        else  // Source=''
           begin
           Target.Add(Source);
           Source:='';
           end;
        end;
  Result:=Target.Count;
end;

function SplitStringToStringList(Source: string; SepChars: TSysCharSet; SuppressBlanks: boolean; var Target: TStringList): integer;
var P: integer;
    S: string;
begin
  Target.Clear;
  while (Source<>'') do
        begin
        P:=GetSysCharSetPosition(SepChars,Source);
        if (P>0) then
           begin
           S:=System.Copy(Source,1,P-1);
           if (S<>'') or not(SuppressBlanks) then Target.Add(S);
           System.Delete(Source,1,P);
           end
        else  // Source=''
           begin
           Target.Add(Source);
           Source:='';
           end;
        end;
  Result:=Target.Count;
end;

function GetRowFromStringDynGrid(Source: TStringDynGrid; RowNumber: integer): TStringDynArray;
var NewArray: TStringDynArray;
begin
  if (RowNumber<System.Length(Source)) then
     Result:=Source[RowNumber]
  else
     begin
     SetLength(NewArray,0);
     Result:=NewArray;
     end;
end;

function GetColumnFromStringDynGrid(Source: TStringDynGrid; ColumnNumber: integer): TStringDynArray;
var NewArray: TStringDynArray;
    I, L: integer;
begin
  L:=System.Length(Source);
  SetLength(NewArray,L);
  for I:=0 to L-1 do
      if (ColumnNumber<System.Length(Source[I])) then
         NewArray[I]:=Source[I,ColumnNumber]
      else NewArray[I]:='';
  Result:=NewArray;  
end;

function GetMaxStringListLength(Source: TStringList): integer;
var I, Max: integer;
begin
  if (Source.Count>0) then
     begin
     Max:=0;
     for I:=0 to Source.Count-1 do
         if (Length(Source.Strings[I])>Max) then Max:=Length(Source.Strings[I]);
     Result:=Max;
     end
  else Result:=0;
end;

function CompareStringDynArrays(A,B: TStringDynArray): boolean;
var I, LA: integer;
begin
  LA:=System.Length(A);
  if (LA=System.Length(B)) then
     begin
     Result:=true;
     for I:=0 to LA-1 do
         if (A[I]<>B[I]) then
            begin
            Result:=false;
            break;
            end;
     end
  else Result:=false;
end;

// Unfortunately, control characters (#0 to #31) will be not supported by Ressource Strings
// This is a serious handicap, when messages for multilingual applications shall be created
// including LF, TAB or other control characters for proper appearance in the massage window.
// The below function, used intermediary, will restore the control characters.
function RestoreRessourceStringControls (Source: string): string;
var I : integer;
    S: string;
    Ch: char;
begin
  S:=''; I:=System.Pos('^',Source);
  while (I>0) do
     begin
     Ch:=System.Upcase(Source[I+1]);
     if Ch in CUpperLetters then Ch:=char(ord(Ch)-ord('@')) else Ch:=' ';
     S:=S+System.Copy(Source,1,I-1)+Ch;
     System.Delete(Source,1,I+1);
     I:=System.Pos('^',Source);
     end;
  Result:=S+Source;
end;

// approach designed by Jens Borrisholt
procedure HookResourceString(ResStringRec: pResStringRec; NewStr: pChar) ;
var OldProtect: dword;
begin
   VirtualProtect(ResStringRec, SizeOf(ResStringRec^), PAGE_EXECUTE_READWRITE, @OldProtect) ;
   ResStringRec^.Identifier := Integer(NewStr) ;
   VirtualProtect(ResStringRec, SizeOf(ResStringRec^), OldProtect, @OldProtect) ;
 end;

initialization
  //
end.
