//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**
//**  ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
//**  COMPARISONPROCS.PAS
//**
//**  PCS45 data structure comparison procedures for PCS45
//**
//**  Autor : Georg  Hahn
//**  Datum :  xx.yy.1996 / 23.08.05 / 20.10.06 / 29.03.07 / 25.10.08 / 22.04.14
//**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit ComparisonProcs;

interface

uses HartDataTypes;

  function CompareUIDs(A,B: THDeviceIdentifier): boolean;
  function CompareTDDs(A,B: THTagDescriptorDate): boolean;
  function ComparePVOs(A,B: THPrimaryVariableOutput): boolean;
  function CompareRNGs(A,B: THPrimaryVariableOutput): boolean;
  function CompareDMPs(A,B: THPrimaryVariableOutput): boolean;

  function Compare073s(A,B: THx073ParameterSet): boolean;
  function Compare075s(A,B: THx075TripParameters): boolean;

  function Comparex576Inp(A,B: THInputParameters_x576): boolean;
  function Comparex576Out(A,B: THOutputParameters_x576): boolean;
  function Comparex576Com(A,B: THCommonParameters_x576): boolean;

  function CompareLFPs(A,B: THLeakFlowOffParameters_x532): boolean; overload;
  function CompareLFPs(A,B: THLeakFlowOffParameters_1003): boolean; overload;
  function CompareAOPs(A,B: THAdjustOutputParameters): boolean;
  function CompareDVFs(A,B: THDividerParameters): boolean;
  function DetectClearDVFs(Required: byte): boolean;
  function CompareTPXs(A,B: THTripParameters_x532): boolean; overload;
  function CompareTPXs(A,B: THTripParameters_1003): boolean; overload;
  function DetectClearTPXs(Required: byte): boolean;
  function CompareInpLFDs(A,B: byte): boolean;
  function DetectClearInpLFDs(Required: byte): boolean;

implementation

uses HartDataConstants;

function CheckNegligibleRealDifference (A,B,MaxDiff: real): boolean;
begin
  if ((A+B)<>0.0) then
     begin
     if (A=0.0) or (B=0.0) then Result:=(System.Abs(A-B)<(5.0*MaxDiff))
     else Result:=(System.Abs((A-B)/(A+B))<MaxDiff);
     end
  else Result:=(A=B);
end;

function CompareUIDs(A,B: THDeviceIdentifier): boolean;
begin
  Result:=((A.DeviceTypeCode254=B.DeviceTypeCode254) and
           (A.ManufacturerIdentCode=B.ManufacturerIdentCode) and
           (A.ManufacturersDeviceType=B.ManufacturersDeviceType) and
           (A.NumberOfPreambles=B.NumberOfPreambles) and
           (A.RevisionLevelOfUCD=B.RevisionLevelOfUCD) and
           (A.RevisionLevelOfTSD=B.RevisionLevelOfTSD) and
           (A.SoftwareRevision=B.SoftwareRevision) and
           (A.HardwareRevision=B.HardwareRevision) and
           (A.Flags=B.Flags) and
           (A.DeviceIdentNumber[1]=B.DeviceIdentNumber[1]) and
           (A.DeviceIdentNumber[2]=B.DeviceIdentNumber[2]) and
           (A.DeviceIdentNumber[3]=B.DeviceIdentNumber[3]));
end;

function CompareTDDs(A,B: THTagDescriptorDate): boolean;
begin
  Result:=((A.TagName=B.TagName) and
           (A.Descriptor=B.Descriptor) and
           (A.Day=B.Day) and
           (A.Month=B.Month) and
           (A.Year=B.Year));
end;

function ComparePVOs(A,B: THPrimaryVariableOutput): boolean;
begin
  Result:=((A.PVAlarm=B.PVAlarm) and
           (A.TransferFunction=B.TransferFunction) and
           (A.PVUnits=B.PVUnits) and
           (CheckNegligibleRealDifference(A.Point20mA,B.Point20mA,0.0001)) and
           (CheckNegligibleRealDifference(A.Point4mA,B.Point4mA,0.0001)) and
           (CheckNegligibleRealDifference(A.DampingTimeConstant,B.DampingTimeConstant,0.0001)));
end;

function CompareRNGs(A,B: THPrimaryVariableOutput): boolean;
begin
  Result:=(// (A.PVAlarm=B.PVAlarm) and
           // (A.TransferFunction=B.TransferFunction) and
           (A.PVUnits=B.PVUnits) and
           (CheckNegligibleRealDifference(A.Point20mA,B.Point20mA,0.0001)) and
           (CheckNegligibleRealDifference(A.Point4mA,B.Point4mA,0.0001)));
end;

function CompareDMPs(A,B: THPrimaryVariableOutput): boolean;
begin
  Result:=(CheckNegligibleRealDifference(A.DampingTimeConstant,B.DampingTimeConstant,0.0001));
end;

function Compare073s(A,B: THx073ParameterSet): boolean;
begin
  Result:=((A.OpenCircuitAlarmLevel=B.OpenCircuitAlarmLevel) and
           (A.TransmitterFailureAlarmLevel=B.TransmitterFailureAlarmLevel) and
           (A.FilterFrequencySelector=B.FilterFrequencySelector) and
           (A.InputType=B.InputType) and
           (A.SmoothingTimeConstant=B.SmoothingTimeConstant) and
           (A.ColdJunctionCompensationOnOff=B.ColdJunctionCompensationOnOff) and
           (A.ColdJunctionFailureAlarmLevel=B.ColdJunctionFailureAlarmLevel));
end;

function Compare075s(A,B: THx075TripParameters): boolean;
begin
  Result:=(A.TripOutputFunction=B.TripOutputFunction) and
          ((A.TripOutputFunction=CHart_Option_Off) or
          ((A.TripValue=B.TripValue) and
           (A.Hysteresis=B.Hysteresis)));
end;

function Comparex576Inp(A,B: THInputParameters_x576): boolean;
begin
  Result:=(A.Channel=B.Channel) and
          (A.SensorType=B.SensorType) and
          (A.PVUnits=B.PVUnits) and
          (A.Linearisation=B.Linearisation) and
          (A.CJC_Option=B.CJC_Option) and
          CheckNegligibleRealDifference(A.Point4mA,B.Point4mA,0.0001) and
          CheckNegligibleRealDifference(A.Point20mA,B.Point20mA,0.0001) and
          CheckNegligibleRealDifference(A.CJC_UserValue,B.CJC_UserValue,0.0001);
end;

function Comparex576Out(A,B: THOutputParameters_x576): boolean;
begin
  Result:=(A.Channel=B.Channel) and
          (A.OpenCircuitAlarmLevel=B.OpenCircuitAlarmLevel) and
          (A.ShortCircuitAlarmLevel=B.ShortCircuitAlarmLevel) and
          (A.TransmitterFailureAlarmLevel=B.TransmitterFailureAlarmLevel) and
          (A.ColdJunctionFailureAlarmLevel=B.ColdJunctionFailureAlarmLevel) and
          (A.NAMUR_MTL=B.NAMUR_MTL) and
          (CheckNegligibleRealDifference(A.DampingTimeConstant,B.DampingTimeConstant,0.0001)) and
          (CheckNegligibleRealDifference(A.SmoothingTimeConstant,B.SmoothingTimeConstant,0.0001)) and
          (A.EBD_Option=B.EBD_Option) and
          (CheckNegligibleRealDifference(A.TripLimit,B.TripLimit,0.0001)) and
          (CheckNegligibleRealDifference(A.TripHysteresis,B.TripHysteresis,0.0001));
end;

function Comparex576Com(A,B: THCommonParameters_x576): boolean;
begin
  Result:=(A.response_time=B.response_time) and
          (A.FilterFrequencySelector=B.FilterFrequencySelector) and
          (A.InputOption=B.InputOption);
end;

function CompareLFPs(A,B: THLeakFlowOffParameters_x532): boolean;
begin
  Result:=((A.FunctionSelect=B.FunctionSelect) and
           ((A.FunctionSelect=CHart_Option_Off) or
            ((A.LFUnits=B.LFUnits) and
             CheckNegligibleRealDifference(A.LeakFlowOffValue,B.LeakFlowOffValue,0.0001) and
             CheckNegligibleRealDifference(A.LeakFlowOffCurrent,B.LeakFlowOffCurrent,0.0001))));
end;

function CompareLFPs(A,B: THLeakFlowOffParameters_1003): boolean;
begin
  Result:=((A.FunctionSelect=B.FunctionSelect) and
           ((A.FunctionSelect=CHart_Option_Off) or
            (CheckNegligibleRealDifference(A.LeakFlowOffValue,B.LeakFlowOffValue,0.0001) and
             CheckNegligibleRealDifference(A.LeakFlowOffCurrent,B.LeakFlowOffCurrent,0.0001))));
end;

function CompareAOPs(A,B: THAdjustOutputParameters): boolean;
begin
  Result:=((A.AdjMode=B.AdjMode) and
           ((A.AdjMode=CHart_Adjust_Off) or
            ((A.AdjUnits=B.AdjUnits) and CheckNegligibleRealDifference(A.OffsetValue,B.OffsetValue,0.0001) and CheckNegligibleRealDifference(A.GainValue,B.GainValue,0.0001))));
end;

function CompareDVFs(A,B: THDividerParameters): boolean;
begin
  Result:=((A.FunctionSelect=B.FunctionSelect) and
           ((A.FunctionSelect=CHart_Option_Off) or
            (CheckNegligibleRealDifference(A.Divider,B.Divider,0.0001)) and (CheckNegligibleRealDifference(A.Duration,B.Duration,0.0001))));
end;

function DetectClearDVFs(Required: byte): boolean;
begin
  Result:=(Required=0);
end;

function CompareTPXs(A,B: THTripParameters_x532): boolean;
begin
  Result:=((A.FunctionSelect=B.FunctionSelect) and
           ((A.FunctionSelect=CP_Blank_Pattern) or
            (((A.TPAUnits=A.TPAUnits) and CheckNegligibleRealDifference(A.TripAValue,B.TripAValue,0.0001) and CheckNegligibleRealDifference(A.AHysteresis,B.AHysteresis,0.0001)) and
             ((A.TPBUnits=A.TPBUnits) and CheckNegligibleRealDifference(A.TripBValue,B.TripBValue,0.0001) and CheckNegligibleRealDifference(A.BHysteresis,B.BHysteresis,0.0001)))));
end;

function CompareTPXs(A,B: THTripParameters_1003): boolean;
begin
  Result:=((A.FunctionSelect and (CP3_TripModeInfo or CP3_Trip_OnRelay)) = (B.FunctionSelect and (CP3_TripModeInfo or CP3_Trip_OnRelay))) and
          (((A.FunctionSelect and CP3_TripModeInfo) = CP_Blank_Pattern) or
           ((A.TPUnits=B.TPUnits) and CheckNegligibleRealDifference(A.TripValue,B.TripValue,0.0001) and CheckNegligibleRealDifference(A.Hysteresis,B.Hysteresis,0.0001)));
end;

function DetectClearTPXs(Required: byte): boolean;
begin
  Result:=((Required and CP3_TripModeInfo) = CP_Blank_Pattern);
end;

function CompareInpLFDs(A,B: byte): boolean;
begin
  Result:=((A and CP3_LFD_Enabled) = (B and CP3_LFD_Enabled)) and
          (((A and CP3_LFD_Enabled) = CP_Blank_Pattern) or
           ((A and CP3_LFD_OnRelay) = (B and CP3_LFD_OnRelay)));
end;

function DetectClearInpLFDs(Required: byte): boolean;
begin
  Result:=((Required and CP3_LFD_Enabled) = CP_Blank_Pattern)
end;

end.
