//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**
//**  ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
//**  CONVERSIONPROCS.PAS
//**
//**  Data conversion procedures (Pascal <--> Hart and other)
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.11.1996 / 11.05.03 / 23.08.06 / ... / 24.10.08 / ... / 22.02.11 / 22.04.14 / 04.08.15
//**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit ConversionProcs;

interface

uses Types, Controls, ConversionUtils, HartDataTypes, DataTypes;

const
  //        1234567890123456789012345678901234567890123456789012345678901234
  CHARTAB = '@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_ !"#$%&'')*+,-./0123456789:;<=>?';

  function CodeString (S: string): string;
  function Point2Comma (S: string): string;

  function CelsiusToFahrenheit (Celsius: real): real;
  function FahrenheitToCelsius (Fahrenheit: real): real;
  function CelsiusToRankine (Celsius: real): real;
  function RankineToCelsius (Rankine: real): real;
  function CelsiusToKelvin (Celsius: real): real;
  function KelvinToCelsius (Kelvin: real): real;

  function Ascii2Hart (S: string; L: word): string;
  function Hart2Ascii (S: string): string;

  function Integer2Hart (I: integer; L: word): string;
  function Hart2Integer (S: string): integer;

  function Single2Hart (R: single): string;
  function Hart2Single (S: string): single;

  function IdentToType(Data: THDeviceIdentifier): TDeviceType;
  function DeviceTypeToIdent(Value: TDeviceType): byte;

  function HARTHardwareRevisionToString(Value: byte): string;

  function PVUnitsFromHART (Value: byte): word;
  function PVUnitsToHART (Value: word): byte;

  function GetPVUnitPrecision (PVUnits: word): byte;

  function LinearFromHART (Value: byte): TOnOffSet;
  function LinearToHART (Value: TOnOffSet): byte;
  function CJCompFromHART (Value: byte): TOnOffSet;
  function CJCompToHART (Value: TOnOffSet): byte;

  function InputTypeFromHART (DeviceType: TDeviceType; InputType: byte): TTempInputType;
  function TypeNumberFromHART (DeviceType: TDeviceType; TypeNumber: byte): word;
  function WiringNumberFromHART (DeviceType: TDeviceType; TypeNumber: byte): word;
  function InputTypeAndNumberAndWiringToHART (DeviceType: TDeviceType; InputType: TTempInputType; TypeNumber,WiringNumber: word): byte;

  function FrequencyFromHART (Value: byte): TFilterFrequency;
  function FrequencyToHART (Value: TFilterFrequency): byte;

  function AlarmLevelFromHART (Value: byte): TAlarmLevelSet;
  function AlarmLevelToHART (Value: TAlarmLevelSet): byte;

  function x075TripTypeFromHART(Value: byte): Tx075TripType;
  function x075TripTypeToHART(Value: Tx075TripType): byte;

  function TripFunctionNameFromHART(TripFunction: byte; TripValue: real): string;
  function TripFunctionNameFromInfo(Value: Tx075TripType): string;

  function x575SpeedFromHART(Value: byte): TResponseSpeed;
  function x575SpeedToHART(Value: TResponseSpeed): byte;

  function x575NAMURFromHART(Value: byte): TNamurOptionSet;
  function x575NAMURToHART(Value: TNamurOptionSet): byte;

  // Temperature converter MTL4576

  function InputOptionx576FromHART(Value: byte): Tx576InputOptionSet;
  function InputOptionx576ToHART(Value: Tx576InputOptionSet): byte;

  function CJCompx576FromHART (Value: byte): Tx576CJCSet;
  function CJCompx576ToHART (Value: Tx576CJCSet): byte;

  function x576NAMURFromHART(Value: byte): TNamurOptionSet;
  function x576NAMURToHART(Value: TNamurOptionSet): byte;

  function x576EBDOptionFromHART(Value: byte): TOnOffSet;
  function x576EBDOptionToHART(Value: TOnOffSet): byte;

  function UserSensorTypeFromHART(Value: byte): TTempInputType;
  function UserSensorTypeToHART(Value: TTempInputType): byte;

  // Frequency converter MTL4532

  function FreqInputSensorFromHART(Value: byte): TFreqInputType;
  function FreqInputSensorToHART(Value: TFreqInputType): byte;

  function FreqOutputModeFromHART(FSValue,NE43Value: byte): TOutputModeSet;
  procedure FreqOutputModeToHART(Value: TOutputModeSet; var FSValue,NE43Value: byte);

  function LeakFlowModeFromHART (Value: byte): TOnOffSet;
  function LeakFlowModeToHART (Value: TOnOffSet): byte;

  function AdjustModeFromHART (Value: byte): TAdjustModeSet;
  function AdjustModeToHART (Value: TAdjustModeSet): byte;

  function PulseOutputModex532FromHART(PulseFunc,Input: byte): TPulseOutputMode;
  procedure PulseOutputModex532ToHART(Value: TPulseOutputMode; var PulseFunc,Input: byte);

  function AlarmOutputModeFromHART(AlarmFunc,Input: byte): TAlarmOutputMode;
  function InhibitModeFromHART(AlarmFunc,Invert: byte): TInhibitMode;
  function AlarmOutputInvertFromHART(AlarmFunc,Input,Invert: byte): boolean;

  function AlarmOutputModeToHART(Value: TAlarmOutputMode; Invert: boolean; Inhibit: TInhibitMode): byte;
  procedure AlarmOutputModeToInputHART(Value: TAlarmOutputMode; var Input: byte);
  procedure InhibitModeToHART(Source: TInhibitMode; var Invert: byte);

  // Frequency converter E01003

  function GetLeakFlowParameters_x532 (Value: THLeakFlowOffParameters_1003): THLeakFlowOffParameters_x532;
  function GetLeakFlowParameters_1003 (Value: THLeakFlowOffParameters_x532): THLeakFlowOffParameters_1003;

  function PulseOutputMode1003FromHART (PulseFunc,InpLFD,TripAFunc,TripBFunc: byte): TPulseOutputMode;
  procedure PulseOutputMode1003ToHART (Value: TPulseOutputMode; var PulseFunc,InpLFD,TripAFunc,TripBFunc: byte);

  function TripOutputInverted (TripFunc: byte): boolean;

  function AlarmOutputMode1003FromHART(InpLFD,TripAFunc,TripBFunc: byte): TAlarmOutputMode;
  procedure AlarmOutputMode1003ToHART(Value: TAlarmOutputMode; var TripAFunc,TripBFunc,InpLFD: byte);
  procedure InvertTripOutput(Value: boolean; var TripFunc: byte);

  function LFDMode1003FromHART(Value: byte): TOnOffSet;

  function ValueUnitsToHART (Value: TTripUnits): byte;
  function ValueUnitsFromHART (Value: byte): TTripUnits;

  function PercentValueFromRange(Value, Min, Max: single): single;

  // Text file import export procedures

  function EncodeByteValueFromString (Value: string; var Target: byte): boolean;
  function EncodeWordValueFromString (Value: string; var Target: word): boolean;
  function EncodeSingleValueFromString (Value: string; var Target: single): boolean;

  function EncodeShortDateString (Value: string): TSimpleDate;
  function EncodeLongDateString (Value: string): TSimpleDate;
  function Check_SimpleDate(var SimpleDate: TSimpleDate): boolean;

  function EncodeHARTDate(Day,Month,Year: byte): TDate;
  procedure DecodeHARTDate(Source: TDate; var Day,Month,Year: byte);

  function ManufacturersIdentNameFromHART(Value: byte): string;
  function CheckManufacturersIdentCodeFromString (Value: string): boolean;

  function GetDevTypeFromString (Value: string): TDeviceType;
  function EncodeHardwareRevisionFromString (Value: string; var Target: byte): boolean;

  function EncodePVUnitsFromString (Value: string; var Target: byte): boolean;
  function EncodeAlarmFromString (Value: string; var Target: byte): boolean;

  function TempInputTypeNameFromDeviceInfo(DeviceType: TDeviceType; InputType: TTempInputType; Linearisation: TOnOffSet; TypeNumber,WiringNumber: word): string;
  function TempInputTypeNameFromHARTInfo(DeviceType: TDeviceType; InputTypeInfo,LinearisationInfo: byte): string;
  function Encodex073InputTypeFromString (Value,Aux,PVU,LIN: string; DeviceType: TDeviceType; var Target: byte): boolean;
  function Encodex576InputTypeFromString (Value,Aux,PVU,LIN: string; DeviceType: TDeviceType; var Target: byte): boolean;

  function Encodex073LinearisationFromString (Value: string; var Target: byte): boolean;
  function Encodex073CJCFromString (Value: string; var Target: byte): boolean;
  function Encodex576CJCFromString (Value: string; var Target: byte): boolean;
  function Encodex073FilterFreqFromString (Value: string; var Target: byte): boolean;

  function Encodex075TripFunctionFromString (Value: string; var Target: byte): boolean;
  function Encodex075TripParameterFromString (Value: string; var Target: THx075TripParameters): boolean;
  function Encodex573EBDFromString (Value: string; var Target: byte): boolean;
  function Encodex575ResponseTimeFromString(Value: string; var Target: byte): boolean;

  function Encodex575OutputCharacteristicFromString(Value: string; var Target: byte): boolean;
  function Encodex532OutputCharacteristicFromString(Value,Aux: string; var Target: byte): boolean;
  function Encodex576OutputCharacteristicFromString(Value: string; var Target: byte): boolean;
  function Encodex576InputOptionFromString(Value: string; var Target: byte): boolean;

  function Encodex532InputSignalSourceFromString (Value: string; var Target: byte): boolean;
  function Encodex532AdjustModeFromString (Value: string; var Target: byte): boolean;
  function Encodex532LeakFlowOffFromString (Value: string; var Target: byte): boolean;
  function Encodex532PulseOutputModeFromString (Value: string; var Target,Input: byte): boolean;
  function Encodex532AlarmOutputModeFromString (Value: string; var Target,Input: byte): boolean;
  function Encodex532InhibitModeFromString(Value: string; var Target: byte): boolean;

  // Encode functions for Pcs-E01003 text files
  function Encode1003OutputModeFromString(Value: string; var Target: T1003OutputMode): boolean;
  function Encode1003InvertedTrip(Value: string; var Target: byte): boolean;

  function Encode1003PulseOutputModeFromString (Value,Aux: string; var DVPFS,TPAFS,TPBFS,Input: byte): boolean;
  function Encode1003AlarmOutputModeFromString (Value,Aux: string; var TPAFS,TPBFS,Input: byte): boolean;

  function GetDynArray(Source: TUnitEnableArray): TBooleanDynArray;

implementation

uses SysUtils, StringUtils, DataConstants, DataInstances, LabelNumbers, HartDataConstants;

const
 SingleError = 19999.8888;

function CodeChar (I: char): char;
begin
  Result := chr(ord(I) XOR 45);
end;

function CodeString (S: string): string;
var I, L: integer;
begin
  L := System.Length (S);
  for I := 1 to L do S[I] := CodeChar(S[I]);
  Result:=S;
end;

function Point2Comma (S: string): string;
begin
  if System.Pos('.', S) > 0 then S[System.Pos('.',S)] := ',';
  Result:=S;
end;

function CelsiusToFahrenheit (Celsius: real): real;
begin
  Result:=9*Celsius/5+32;
end;

function FahrenheitToCelsius (Fahrenheit: real): real;
begin
  Result:=5*(Fahrenheit-32)/9;
end;

function CelsiusToRankine (Celsius: real): real;
begin
  Result:=(9*(Celsius+273.1555))/5;
  // up to the version 2.04b wrong formula was used here: Result:=(5*(Celsius+273.1555))/9;
end;

function RankineToCelsius (Rankine: real): real;
begin
  Result:=(5*Rankine/9)-273.1555;
end;

function CelsiusToKelvin (Celsius: real): real;
begin
  Result:=Celsius+273.1555;
end;

function KelvinToCelsius (Kelvin: real): real;
begin
  Result:=Kelvin-273.1555;
end;

function Ascii2Hart (S: string; L: word): string;
var NewHart : string;
    O1,O2,O3,O4 : LongInt;
    K,N : LongInt;
    I : word;
begin
  while (System.Length(S)<>L) do S := S + ' ';
  I := 1;
  NewHart := '';
  repeat
    O1 := System.Pos(S[I+0],CHARTAB)-1;
    O2 := System.Pos(S[I+1],CHARTAB)-1;
    O3 := System.Pos(S[I+2],CHARTAB)-1;
    O4 := System.Pos(S[I+3],CHARTAB)-1;

    N  := ((O1*64+O2)*64+O3)*64+O4;
    K  := System.Trunc(N/256)*256;
    O3 := N-K;
    N  := System.Trunc(K/256);
    K  := System.Trunc(N/256)*256;
    O2 := N-K;
    O1 := System.Trunc(N/256);
    NewHart  := NewHart + chr(O1)+chr(O2)+chr(O3);
    I := I + 4;
  until (I>System.Length(S));
  Result := NewHart;
end;

function Hart2Ascii (S: string): string;
var NewAscii : string;
    I, L : word;
    O : byte;
    N : byte;
begin
  NewAscii := '';
  L := System.Length(S);
  I := 1;
  if (FRAC(L/3)<>0) or (L<3)
  then NewAscii := 'Error'
  else
     while I < L do
           begin
           O := ord(S[I]) SHR 2;
           NewAscii := NewAscii + copy(CHARTAB,O+1,1);

           O := (ord(S[I]) AND $03) SHL 4;
           System.Inc(I);
           N := ord(S[I]) SHR 4;
           N := O + N;
           NewAscii := NewAscii + copy(CHARTAB,n+1,1);

           O := (ord(S[I]) AND $0F) SHL 2;
           System.Inc(I);
           N := ord(S[I]) SHR 6;
           N := O + N;
           NewAscii := NewAscii + copy(CHARTAB,n+1,1);

           N := ord(S[I]) AND $3F;
           NewAscii := NewAscii + copy(CHARTAB,n+1,1);
           System.Inc(I);
           end;
  Result := NewAscii;
end;

function Integer2Hart (I: integer; L: word): string;
begin
  case L of
       1: Result:=chr(I MOD 256);
       2: Result:=chr(I DIV 256)+chr(I MOD 256);
       3: Result:=chr(I DIV (256*256))+chr((I DIV 256) MOD 256)+chr(I MOD 256);
       else Result:='';
       end;
end;

function Hart2Integer (S: string): integer;
var L: integer;
begin
  L:=System.Length(S);
  case L of
       1:  Result:=ord(S[1]);
       2:  Result:=ord(S[1])*256+ord(S[2]);
       3:  Result:=ord(S[1])*256*256+ord(S[2])*256+ord(S[3]);
       else Result:=0;
       end;
end;

function Single2Hart (R: single): string;
var S : single;
    C : array[1..4] of char absolute S;
    NewHart : string[4];
begin
   S := R;
   NewHart[0] := #4;
   NewHart[1] := C[4];
   NewHart[2] := C[3];
   NewHart[3] := C[2];
   NewHart[4] := C[1];
   Result := NewHart;
end;

function Hart2Single (S: string): single;
var NewSingle : single;
    C : array[1..4] of char absolute NewSingle;
begin
  if (((ord(S[1]) and $7F)=$7F) and (ord(S[2])>$7E)) then NewSingle := SingleError
  else
     begin
     C[1] := S[4];
     C[2] := S[3];
     C[3] := S[2];
     C[4] := S[1];
     end;
  Result := NewSingle;
end;

function IdentToType(Data: THDeviceIdentifier): TDeviceType;
begin
  case Data.ManufacturersDeviceType of
       2:  Result:=dt507x;
       3:  Result:=dtx073;
       6:  Result:=dt5074XK;
       7:  if not(Data.SoftwareRevision<40) and not(Data.HardwareRevision<31) then Result:=dt4073XK
           else if (Data.HardwareRevision<31) then
                   begin
                   if (Data.SoftwareRevision<4) then Result:=dtNone
                   else Result:=dt1003;
                   end
           else Result:=dtNone;
           // Note: the MTL4073XK firmware sources shows:
           // SoftwareRevision=41 and HardwareRevision=64
           // Note: the E01003 firmware sources shows:
           // SoftwareRevision<4 for "older versions" (not longer supported)
           // SoftwareRevision=4 for "new version"
           // and HardwareRevision<31
       8:  Result:=dt4075;
       9:  Result:=dtx575;
       11: Result:=dtx576;
       12: // Note: the Manufacturer Device Type was assigned by mistake
           // to two different products: MTL4532 and MTL5582.
           // Because of another mistake in the Hardware Revision assignment
           // for MTL5582 set to 1, which means 1.0 in according to HART_SPEC-127
           // both products can be reliebly differed as shown below:
           if (Data.HardwareRevision<8) then
              Result:=dtx582B
           else Result:=dtx532;
       13: Result:=dtx573;
       14: Result:=dtx582B;
       else Result:=dtNone;
       end;
end;

function DeviceTypeToIdent(Value: TDeviceType): byte;
begin
  case Value of
       dt507x:   Result:=2;
       dtx073:   Result:=3;
       dt5074XK: Result:=6;
       dt4073XK: Result:=7;
       dt4075:   Result:=8;
       dtx575:   Result:=9;
       dtx576:   Result:=11;
       dtx532:   Result:=12;
       dtx573:   Result:=13;
       dtx582B:  Result:=14;
       dt1003:   Result:=7;
    else Result:=0;
    end;
end;

function HARTHardwareRevisionToString(Value: byte): string;
var SR1,SR2: string;
begin
  SR1:=IntToStr(Value shr 3);
  SR2:=IntToStr(Value and $07);
  Result:=SR1+'.'+SR2;
end;

function PVUnitsFromHART (Value: byte): word;
begin
  // HART command 44 -> 32=C, 33=F, 34=R, 35=K, 36=mV, 37=Ohm, 38=Herz, 39=mA, 51=sec, 57=%
  if Value in [CHart_Celsius..CHart_milliamper] then Result:=Value-CHart_Celsius
  else if Value=CHart_Second then Result:=8
  else if Value=CHart_Percent then Result:=9
  else Result:=10;
end;

function PVUnitsToHART (Value: word): byte;
begin
  // HART command 44 -> 32=C, 33=F, 34=R, 35=K, 36=mV, 37=Ohm
  if Value in [0..5] then Result:=Value+CHart_Celsius else Result:=CHart_Celsius;
end;

function GetPVUnitPrecision (PVUnits: word): byte;
begin
  case PVUnits of
       4: Result:=3;      // for mV
       5: Result:=2;      // for Ohm
       6: Result:=1;      // for Herz
       7: Result:=3;      // for mA
       else Result:=1;    // for C,F,R, K and for all other units
       end;
end;

function LinearFromHART (Value: byte): TOnOffSet;
begin
  // HART command 15 means 1 = linearisation on and 0 = linearisation off
  // Dave Coulson confirmed on 26.02.98 that actually valid is: 0 = on and 1 = off
  if Value=0 then Result:=tsOn else Result:=tsOff;
end;

function LinearToHART (Value: TOnOffSet): byte;
begin
  // HART command 47 means 0 = linearisation on and 1 = linearisation off
  if Value=tsOn then Result:=0 else Result:=1;
end;

function CJCompFromHART (Value: byte): TOnOffSet;
begin
  // HART command 128/129, parameter 5 means 1 = enabled and 2 = disabled
  if Value=1 then Result:=tsOn else Result:=tsOff;
end;

function CJCompToHART (Value: TOnOffSet): byte;
begin
  // HART command 128/129, parameter 5 means 1 = enabled and 2 = disabled
  if Value=tsOn then Result:=1 else Result:=2;
end;

function InputTypeFromHART (DeviceType: TDeviceType; InputType: byte): TTempInputType;
begin
  // refer to HART command 128 Input Type specification for parameter relations
  case DeviceType of
       dt4075: case InputType of
                    0,1,2,3,4,5,6,7,17,20 : Result := itTHC;
                    8,11,14,32,35,38,64,67,70 : Result := itRTD;
                    else Result := itNone;
                    end;
       dtx575,
       dtx573: case InputType of
                    0,1,2,3,4,5,6,7,17,20 : Result := itTHC;
                    8,9,10,11,12,13,14,15,16,
                    32,35,38,64,67,70,
                    128,129,130,131,132,133,134,135,136 : Result := itRTD;
                    else Result := itNone;
                    end;
       dtx576: case InputType of
                    0,1,2,3,4,5,6,7,8,9,$0F : Result := itTHC;
                    $10,$20,$30,$40,$50,$60,$70,$80,$90,
                    $11,$21,$31,$41,$51,$61,$71,$81,$91,
                    $12,$22,$32,$42,$52,$62,$72,$82,$92,
                    $F0,$F1,$F2 : Result:=itRTD;
                    else Result:=itNone;
                    end;
       else    case InputType of  // dtx073..dt4073XK
                    0,1,2,3,4,5,6,7,17 : Result := itTHC;
                    8,11,14 : Result := itRTD;
                    else Result := itNone;
                    end;
       end;
end;

function TypeNumberFromHART (DeviceType: TDeviceType; TypeNumber: byte): word;
begin
  // refer to HART command 128 Input Type specification for parameter relations
  case DeviceType of
       dt4075: case TypeNumber of
                    0,1,2,3,4,5,6,7 : Result := TypeNumber;   // B, E, J, K, N, R, S, T
                    20 : Result:=8;                           // XK
                    17 : Result:=9;                           // mV
                    8,11,14 :  Result := 0;                   // Pt100: 2-wire, 3-wire, 4-wire
                    32,35,38 : Result := 1;                   // Cu50: 2-wire, 3-wire, 4-wire
                    64,67,70 : Result := 2;                   // Cu53: 2-wire, 3-wire, 4-wire
                    else Result := 0;
                    end;
       dtx575,
       dtx573: case TypeNumber of
                    0,1,2,3,4,5,6,7 : Result := TypeNumber;   // B, E, J, K, N, R, S, T
                    20 : Result:=8;                           // XK
                    17 : Result:=9;                           // mV
                    8,11,14 :    Result := 0;                 // Pt100:  2-wire, 3-wire, 4-wire
                    9,12,15 :    Result := 1;                 // Pt500:  2-wire, 3-wire, 4-wire
                    10,13,16 :   Result := 2;                 // Pt1000: 2-wire, 3-wire, 4-wire
                    128,131,134: Result := 3;                 // Ni100:  2-wire, 3-wire, 4-wire
                    129,132,135: Result := 4;                 // Ni500:  2-wire, 3-wire, 4-wire
                    130,133,136: Result := 5;                 // Ni1000: 2-wire, 3-wire, 4-wire
                    32,35,38 :   Result := 6;                 // Cu50:   2-wire, 3-wire, 4-wire
                    64,67,70 :   Result := 7;                 // Cu53:   2-wire, 3-wire, 4-wire
                    else Result := 0;
                    end;
       dtx576: case TypeNumber of
                    0,1,2,3,4,5,6,7 : Result := TypeNumber;   // B, E, J, K, N, R, S, T
                    8 : Result:=8;                            // XK
                    9 : Result:=9;                            // User THC
                    $0F : Result:=10;                         // mV
                    $10,$11,$12: Result := 0;                 // Pt100:    2-wire, 3-wire, 4-wire
                    $20,$21,$22: Result := 1;                 // Pt500:    2-wire, 3-wire, 4-wire
                    $30,$31,$32: Result := 2;                 // Pt1000:   2-wire, 3-wire, 4-wire
                    $40,$41,$42: Result := 3;                 // Ni100:    2-wire, 3-wire, 4-wire
                    $50,$51,$52: Result := 4;                 // Ni500:    2-wire, 3-wire, 4-wire
                    $60,$61,$62: Result := 5;                 // Ni1000:   2-wire, 3-wire, 4-wire
                    $70,$71,$72: Result := 6;                 // Cu50:     2-wire, 3-wire, 4-wire
                    $80,$81,$82: Result := 7;                 // Cu53:     2-wire, 3-wire, 4-wire
                    $90,$91,$92: Result := 8;                 // User RTD: 2-wire, 3-wire, 4-wire
                    $A0,$A1,$A2: Result := 8;                 // User RTD > 400 Ohm
                    $F0,$F1,$F2: Result := 9;                 // Resistor (not Pt100 without linearisation!)
                    else Result := 0;
                    end;
       else    case TypeNumber of  // dtx073..dt4073XK
                    0,1,2,3,4,5,6,7 : Result := TypeNumber;   // B, E, J, K, N, R, S(XK), T
                    17 : Result:=8;                           // mV
                    8,11,14 : Result := 0;                    // Pt100: 2-wire, 3-wire, 4-wire
                    else Result := 0;
                    end;
       end;
end;

function WiringNumberFromHART (DeviceType: TDeviceType; TypeNumber: byte): word;
begin
  // refer to HART command 128 Input Type specification for parameter relations
  case DeviceType of
       dt4075: case TypeNumber of
                    8,11,14 :  Result := (TypeNumber-8) DIV 3;     // Pt100: 2-wire, 3-wire, 4-wire
                    32,35,38 : Result := (TypeNumber-32) DIV 3;    // Cu50: 2-wire, 3-wire, 4-wire
                    64,67,70 : Result := (TypeNumber-64) DIV 3;    // Cu53: 2-wire, 3-wire, 4-wire
                    else Result := 0;
                    end;
       dtx575,
       dtx573: case TypeNumber of
                    8,11,14 :   Result := (TypeNumber-8) DIV 3;    // Pt100:  2-wire, 3-wire, 4-wire
                    9,12,15  :  Result := (TypeNumber-9) DIV 3;    // Pt500:  2-wire, 3-wire, 4-wire
                    10,13,16 :  Result := (TypeNumber-10) DIV 3;   // Pt1000: 2-wire, 3-wire, 4-wire
                    128,131,134:Result := (TypeNumber-128) DIV 3;  // Ni100:  2-wire, 3-wire, 4-wire
                    129,132,135:Result := (TypeNumber-129) DIV 3;  // Ni500:  2-wire, 3-wire, 4-wire
                    130,133,136:Result := (TypeNumber-130) DIV 3;  // Ni1000: 2-wire, 3-wire, 4-wire
                    32,35,38 :  Result := (TypeNumber-32) DIV 3;   // Cu50:   2-wire, 3-wire, 4-wire
                    64,67,70 :  Result := (TypeNumber-64) DIV 3;   // Cu53:   2-wire, 3-wire, 4-wire
                     else Result := 0;
                    end;
       dtx576: case TypeNumber of
                    $11,$21,$31,$41,$51,$61,$71,$81,$91,$F2: Result:=1;
                    $12,$22,$32,$42,$52,$62,$72,$82,$92,$F3: Result:=2;
                    else Result:=0;
                    end;
       else    case TypeNumber of
                    8,11,14 : Result := (TypeNumber-8) DIV 3;      // Pt100: 2-wire, 3-wire, 4-wire
                    else Result := 0;
                    end;
       end;
end;

function InputTypeAndNumberAndWiringToHART (DeviceType: TDeviceType; InputType: TTempInputType; TypeNumber,WiringNumber: word): byte;
var lITx576: byte;
begin
  // refer to HART command 128 Input Type specification for parameter relations
  case DeviceType of
       dt4075,
       dtx575,
       dtx573: case InputType of
                    itTHC: if TypeNumber in [0..7] then Result:=TypeNumber
                           else if TypeNumber=8 then Result:=20
                           else if TypeNumber=9 then Result:=17
                           else Result:=0;
                    itRTD: if (WiringNumber in [0..2]) then
                              begin
                              if (DeviceType=dt4075) then
                                 begin
                                 if (TypeNumber in [0,3]) then Result:=8+(WiringNumber)*3
                                 else if (TypeNumber=1) then Result:=32+(WiringNumber)*3
                                 else if (TypeNumber=2) then Result:=64+(WiringNumber)*3
                                 else Result:=0;
                                 end
                              else // DeviceType=dtx575 or dtx573
                                 begin
                                 if (TypeNumber in [0,8]) then Result:=8+(WiringNumber)*3
                                 else if (TypeNumber=1) then Result:=9+(WiringNumber)*3
                                 else if (TypeNumber=2) then Result:=10+(WiringNumber)*3
                                 else if (TypeNumber=3) then Result:=128+(WiringNumber)*3
                                 else if (TypeNumber=4) then Result:=129+(WiringNumber)*3
                                 else if (TypeNumber=5) then Result:=130+(WiringNumber)*3
                                 else if (TypeNumber=6) then Result:=32+(WiringNumber)*3
                                 else if (TypeNumber=7) then Result:=64+(WiringNumber)*3
                                 else Result:=0;
                                 end;
                              end
                           else Result:=0; // not(WiringNumber in [0..2])
                    else Result:= 0;
                    end;
       dtx576: case InputType of
                    itTHC: if TypeNumber in [0..9] then Result:=TypeNumber
                           else if TypeNumber=10 then Result:=$0F
                           else Result:=0;
                    itRTD: begin
                           if TypeNumber in [0..8] then lITx576:=(TypeNumber+1)*$10
                           else if TypeNumber=9 then lITx576:=$F0
                           else lITx576:=$10;
                           case WiringNumber of
                                1: Result:=lITx576+1;
                                2: Result:=lITx576+2;
                                else Result:=lITx576;
                                end;
                           end;
                    end;
       else    case InputType of // dtx073..dt4073XK
                    itTHC: if TypeNumber in [0..7] then Result:=TypeNumber
                           else if TypeNumber=8 then Result:=17
                           else Result:=0;
                    itRTD: begin
                           if (WiringNumber in [0..2]) and (TypeNumber in [0..1]) then Result:=8+(WiringNumber)*3
                           else Result:=0;
                           end
                    else Result:= 0;
                    end;
       end;
end;

function FrequencyFromHART (Value: byte): TFilterFrequency;
begin
  // HART command 128, parameter 2 means 1 = 50Hz  2 = 60Hz
  if Value=1 then Result:=tf50 else Result:=tf60;
end;

function FrequencyToHART (Value: TFilterFrequency): byte;
begin
  // HART command 128, parameter 2 means 1 = 50Hz  2 = 60Hz
  if Value=tf50 then Result:=1 else Result:=2;
end;

function AlarmLevelFromHART (Value: byte): TAlarmLevelSet;
begin
  // HART command 128, parameter 0, 1 and 70 means:
  //  1 = Low, 2 = High and 3 = Off
  case Value of
       1:   Result:=taLow;
       2:   Result:=taHigh;
       else Result:=taOff;
       end;
end;

function AlarmLevelToHART (Value: TAlarmLevelSet): byte;
begin
  // HART command 128, parameter 0, 1 and 70 means:
  //  1 = Low, 2 = High and 3 = Off
  case Value of
       taLow:  Result:=1;
       taHigh: Result:=2;
       else    Result:=3;
       end;
end;

function x075TripTypeFromHART(Value: byte): Tx075TripType;
begin
  // HART command 204/205, parameter 0, 1 and 2 means:
  //  0 = Off, 1 = EBD and 2 = Trip
  case Value of
       1:   Result:=ttEBD;
       2:   Result:=ttTrip;
       else Result:=ttOff;
       end;
end;

function x075TripTypeToHART(Value: Tx075TripType): byte;
begin
  case Value of
       ttEBD:   Result:=1;
       ttTrip:   Result:=2;
       else Result:=0;
       end;
end;

function TripFunctionNameFromHART(TripFunction: byte; TripValue: real): string;
begin
  case TripFunction of
       2: Result:=FloatToStrF(TripValue,ffFixed,3,1);
       1: Result:=LanguageStrings[flEBD];
       else Result:= LanguageStrings[flOff];
       end;
end;

function TripFunctionNameFromInfo(Value: Tx075TripType): string;
begin
  case Value of
       ttTrip: Result:=LanguageStrings[flTrip];
       ttEBD: Result:=LanguageStrings[flEBD];
       else Result:= LanguageStrings[flOff];
       end;
end;

function x575SpeedFromHART(Value: byte): TResponseSpeed;
begin
  case Value of
       2: Result:=tsFast;
       1: Result:=tsMiddle;
       else Result:=tsSlow;
       end;
end;

function x575SpeedToHART(Value: TResponseSpeed): byte;
begin
  case Value of
       tsFast: Result:=2;
       tsMiddle: Result:=1;
       else Result:=0;
       end;
end;

function x575NAMURFromHART(Value: byte): TNamurOptionSet;
begin
  case Value of
       0: Result:=nsOff;
       1: Result:=nsOn;
       else Result:=nsNotAvailable;
       end;
end;

function x575NAMURToHART(Value: TNamurOptionSet): byte;
begin
  case Value of
       nsOff: Result:=0;
       nsOn: Result:=1;
       else Result:=2;
       end;
end;

function InputOptionx576FromHART(Value: byte): Tx576InputOptionSet;
begin
  // HART command 207/152, parameter 3 means 0 = 2in-2out, 1 = 1in-2out and 2 to 5 = 2in-1out
  case Value of
       //5: Result:=is2In1OutMax;   // deactivate actually not used options
       //4: Result:=is2In1OutMin;
       //3: Result:=is2In1OutDiff;
       //2: Result:=is2In1OutAdd;
       1: Result:=is1In2Out;
       else Result:=is2In2Out;
       end;
end;

function InputOptionx576ToHART(Value: Tx576InputOptionSet): byte;
begin
  case Value of
       //is2In1OutMax: Result:=5;  // deactivate actually not used options
       //is2In1OutMin: Result:=4;
       //is2In1OutDiff: Result:=3;
       //is2In1OutAdd: Result:=2;
       is1In2Out: Result:=1;
       else Result:=0;
       end;
end;

function CJCompx576ToHART (Value: Tx576CJCSet): byte;
begin
  // HART command 151/152, parameter 5 means 0 = disabled, 1 = enabled and 3 = user defined
  case Value of
       csOn: Result:=1;
       csUser: Result:=2;
       else Result:=0;
       end;
end;

function CJCompx576FromHART (Value: byte): Tx576CJCSet;
begin
  // HART command 151/152, parameter 5 means 0 = disabled, 1 = enabled and 3 = user defined
  if Value=1 then Result:=csOn
  else if Value=2 then Result:=csUser
  else Result:=csOff;
end;

function x576NAMURFromHART(Value: byte): TNamurOptionSet;
begin
  // HART command 161/162, parameter 6 means 0 = NAMUR NE 43, 1 = MTL standard
  case Value of
       0: Result:=nsOn;
       1: Result:=nsOff;
       else Result:=nsNotAvailable;
       end;
end;

function x576NAMURToHART(Value: TNamurOptionSet): byte;
begin
  case Value of
       nsOff: Result:=1;
       nsOn: Result:=0;
       else Result:=2;
       end;
end;

function x576EBDOptionFromHART(Value: byte): TOnOffSet;
begin
  // HART command 204/205, parameter 0, 1 and 2 means:
  //  0 = Off, 1 = EBD and 2 = Trip (however there is no Trip on MTLx576)
  case Value of
       1:   Result:=tsOn; // EBD
       else Result:=tsOff;
       end;
end;

function x576EBDOptionToHART(Value: TOnOffSet): byte;
begin
  case Value of
       tsOn:   Result:=1; // EBD
       else Result:=0;
       end;
end;

function UserSensorTypeFromHART(Value: byte): TTempInputType;
begin
  case Value of
       $09: Result:=itTHC;
       $90,
       $A0: Result:=itRTD;
       else Result:=itNone;
       end;
end;

function UserSensorTypeToHART(Value: TTempInputType): byte;
begin
  case Value of
       itTHC: Result:=$09;
       itRTD: Result:=$90;
       // or $A0 ???
       end;
end;

function FreqInputSensorFromHART(Value: byte): TFreqInputType;
begin
  if ((Value and CP_InputSelector)=CP_InputProxy1) then
     Result:=fiProxy
  else Result:=fiVoltCurrent;
  // The below specified values are reserved for future use:
  // if CP_InputProxy1=0 and CP_InputProxy2=1 then fiProxy2
  // if CP_InputProxy1=1 and CP_InputProxy2=1 then fiProxy12 (Proxy1 - Proxy2)
end;

function FreqInputSensorToHART(Value: TFreqInputType): byte;
begin
  if (Value=fiProxy) then
     Result:=CP_InputProxy1
  else Result:=CP_Blank_Pattern;
end;

function FreqOutputModeFromHART(FSValue,NE43Value: byte): TOutputModeSet;
begin
  if (FSValue=CHart_FullScale_On) then
     Result:=omDead
  else
     if (NE43Value=CHart_x575_NE43_On) then
        Result:=omNE43
     else Result:=omLive;
end;

procedure FreqOutputModeToHART(Value: TOutputModeSet; var FSValue,NE43Value: byte);
begin
  case Value of
       omDead: begin
               FSValue:=CHart_FullScale_On;
               NE43Value:=CP_Blank_Pattern;
               end;
       omNE43: begin
               FSValue:=CP_Blank_Pattern;
               NE43Value:=CHart_x575_NE43_On;
               end;
       omLive: begin
               FSValue:=CP_Blank_Pattern;
               NE43Value:=CP_Blank_Pattern;
               end;
       end;
end;

function LeakFlowModeFromHART (Value: byte): TOnOffSet;
begin
  // HART command 133/134, parameter means 0 = Off  1 = On
  if ((Value and CHart_Option_On)>0) then
     Result:=tsOn
  else Result:=tsOff;
end;

function LeakFlowModeToHART (Value: TOnOffSet): byte;
begin
  // HART command 133/134, parameter means 0 = Off  1 = On
  if Value=tsOn then Result:=CHart_Option_On else Result:=CHart_Option_Off;
end;

function AdjustModeFromHART (Value: byte): TAdjustModeSet;
begin
  // HART command 140/141 means:
  //  0 = Off, 4 = Soft and 9 = Hard
  case Value of
       CHart_Adjust_Soft: Result:=tmSoft;
       CHart_Adjust_Hard: Result:=tmHard;  // only on E01003
       else Result:=tmOff;
       end;
end;

function AdjustModeToHART (Value: TAdjustModeSet): byte;
begin
  // HART command 140/141 means:
  //  0 = Off, 4 = Soft and 9 = Hard
  case Value of
       tmSoft:  Result:=CHart_Adjust_Soft;
       tmHard:  Result:=CHart_Adjust_Hard; // only on E01003
       else     Result:=CHart_Adjust_Off;
       end;
end;

function PulseOutputModex532FromHART(PulseFunc,Input: byte): TPulseOutputMode;
begin
  if ((PulseFunc and CP_Divider_Enabled)>0) then
     begin
     if ((PulseFunc and CP_Divider_Toggle)>0) then
        Result:=pmToggle
     else Result:=pmPulses;
     end
  else // (PulseFunc and CP_Divider_Enabled)=0
     begin
     if ((Input and CP_PulseToInput)>0) then
        Result:=pm11
     else Result:=pmOff;
     end;
  // The below specified values are reserved for future use:
  // pmTrip1, pmTrip2
end;

procedure PulseOutputModex532ToHART(Value: TPulseOutputMode; var PulseFunc,Input: byte);
begin
  Input:=Input and not(CP_PulseToInput or CP_PulseToLFD or CP_PulseToInhibit);
  case Value of
       pm11:  Input:=Input+CP_PulseToInput;
       pmToggle: PulseFunc:=CP_Divider_Enabled or CP_Divider_Toggle;
       pmPulses: PulseFunc:=CP_Divider_Enabled;
       end;
end;

function AlarmOutputModeFromHART(AlarmFunc,Input: byte): TAlarmOutputMode;
begin
  if ((AlarmFunc and CP_TripAOnALarm)>0) or
     ((AlarmFunc and CP_TripABOnAlarm)>0) then
     begin
     if ((AlarmFunc and CP_TripAOnALarm)>0) then
        Result:=amTrip1
     else Result:=amTrip2;
     end
  else // there is no any Trip function on Alarm Output
     begin
      if ((Input and CP_AlarmToLFD)>0) then
         Result:=amLFD
      else
         if ((Input and CP_AlarmToInput)>0) then
            Result:=am11
         else Result:=amOff;
     end;
end;

function InhibitModeFromHART(AlarmFunc,Invert: byte): TInhibitMode;
begin
  if (((AlarmFunc and CP_InhibitEnabled)>0) and
      (((AlarmFunc and CP_TripAOnALarm)>0) or
       ((AlarmFunc and CP_TripABOnAlarm)>0))) then
     begin
     if ((Invert and CP_StatusProxy2)=0) then // Inhibit input is on Proxy2
        Result:=imNormal
     else Result:=imInverted;
     end
  else
     Result:=imOff;
end;

function AlarmOutputInvertFromHART(AlarmFunc,Input,Invert: byte): boolean;
begin
  Result:=(((AlarmFunc and CP_TripABOnAlarm)>0) and ((AlarmFunc and CP_TripABInverted)>0)) or
          (((AlarmFunc and CP_TripAOnALarm)>0) and ((AlarmFunc and CP_TripAInverted)>0)); // or
          //(((AlarmFunc and (CP_TripAOnALarm or CP_TripABOnALarm))=0) and ((Input and CP_AlarmToLFD)>0) and ((Invert and CP_StatusLFD1)>0));
end;

function AlarmOutputModeToHART(Value: TAlarmOutputMode; Invert: boolean; Inhibit: TInhibitMode): byte;
begin
  case Value of
       amOff,
       am11,
       amLFD: Result:=0;
       amTrip1: begin
                if Invert then
                   Result:=CP_TripAOnALarm or CP_TripAInverted
                else Result:=CP_TripAOnALarm;
                if (Inhibit<>imOff) then Result:=Result or CP_InhibitEnabled;
                end;
       amTrip2: begin
                if Invert then
                   Result:=CP_TripABOnALarm or CP_TripABInverted
                else Result:=CP_TripABOnALarm;
                if (Inhibit<>imOff) then Result:=Result or CP_InhibitEnabled;
                end;
       end;
end;

procedure AlarmOutputModeToInputHART(Value: TAlarmOutputMode; var Input: byte);
begin
  Input:=Input and not(CP_AlarmToLFD or CP_AlarmToInput or CP_AlarmToInhibit);
  if (Value=amLFD) then
     Input:=Input or CP_AlarmToLFD
  else
     if (Value=am11) then
         Input:=Input or CP_AlarmToInput;
end;

procedure InhibitModeToHART(Source: TInhibitMode; var Invert: byte);
begin
  if (Source=imInverted) then
     Invert:=Invert or CP_StatusProxy2; // Inhibit input is on Proxy2
end;

function GetLeakFlowParameters_x532 (Value: THLeakFlowOffParameters_1003): THLeakFlowOffParameters_x532;
begin
  if (Value.FunctionSelect=CP3_LeakFlowEnabled) then
     Result.FunctionSelect:=CHart_Option_On
  else Result.FunctionSelect:=CHart_Option_Off;
  Result.LFUnits:=CHart_Percent;
  Result.LeakFlowOffValue:=Value.LeakFlowOffValue;
  Result.LeakFlowOffCurrent:=Value.LeakFlowOffCurrent;
end;

function GetLeakFlowParameters_1003 (Value: THLeakFlowOffParameters_x532): THLeakFlowOffParameters_1003;
begin
  if (Value.FunctionSelect=CHart_Option_On) then
     Result.FunctionSelect:=CP3_LeakFlowEnabled
  else Result.FunctionSelect:=CP3_Off_Pattern;
  Result.LeakFlowOffValue:=Value.LeakFlowOffValue;
  Result.LeakFlowOffCurrent:=Value.LeakFlowOffCurrent;
end;

function PulseOutputMode1003FromHART(PulseFunc,InpLFD,TripAFunc,TripBFunc: byte): TPulseOutputMode;
var TripAonPulse, TripBonPulse: boolean;
begin
  if ((PulseFunc and CP3_Divider_Enabled)>0) then
     Result:=pmPulses
  else // (PulseFunc and CP3_On_Pattern)=0
     begin
     if ((InpLFD and CP3_LFD_Enabled)>0) and
        ((InpLFD and CP3_LFD_OnRelay)=0) then
        Result:=pmLFD
     else // (InpLFD and CP3_LFD_Enabled)=0
        begin
        TripAonPulse:=((TripAFunc and CP3_Trip_Enabled)>0) and ((TripAFunc and CP3_Trip_OnRelay)=0);
        TripBonPulse:=((TripBFunc and CP3_Trip_Enabled)>0) and ((TripBFunc and CP3_Trip_OnRelay)=0);
        if TripAonPulse and TripBonPulse then
           Result:=pmTrip2
        else
           if TripAonPulse then // or TripBonPulse (no, only TripA is available on Pulse Output
              Result:=pmTrip1
           else Result:=pmOff;
        end;
     end;
end;

procedure PulseOutputMode1003ToHART(Value: TPulseOutputMode; var PulseFunc,InpLFD,TripAFunc,TripBFunc: byte);
begin
  case Value of
       pmPulses: PulseFunc:=CP3_Divider_Enabled;
       pmLFD: InpLFD:=InpLFD or CP3_LFD_Enabled;
       pmTrip1: TripAFunc:=CP3_Trip_Enabled;
       pmTrip2: begin
                TripAFunc:=CP3_Trip_Enabled;
                TripBFunc:=CP3_Trip_Enabled or CP3_TripB_Pattern;
                end;
       end;
end;

function TripOutputInverted (TripFunc: byte): boolean;
begin
  Result:=((TripFunc and CP3_Trip_Inverted)>0);
end;

function AlarmOutputMode1003FromHART(InpLFD,TripAFunc,TripBFunc: byte): TAlarmOutputMode;
var TripAonAlarm, TripBonAlarm: boolean;
begin
  TripAonAlarm:=((TripAFunc and CP3_Trip_Enabled)>0) and ((TripAFunc and CP3_Trip_OnRelay)>0);
  TripBonAlarm:=((TripBFunc and CP3_Trip_Enabled)>0) and ((TripBFunc and CP3_Trip_OnRelay)>0);
  if TripAonAlarm and TripBonAlarm then
     Result:=amTrip2
  else
     if TripBonAlarm then // or TripAonPulse (no, only TripB is available on Alarm Output
        Result:=amTrip1
     else
        if ((InpLFD and CP3_LFD_Enabled)>0) and
           ((InpLFD and CP3_LFD_OnRelay)>0) then
           Result:=amLFD
        else Result:=amOff;
end;

procedure AlarmOutputMode1003ToHART(Value: TAlarmOutputMode; var TripAFunc,TripBFunc,InpLFD: byte);
begin
  case Value of
       amLFD: InpLFD:=InpLFD or CP3_LFD_Enabled or CP3_LFD_OnRelay;
       amTrip1: TripBFunc:=CP3_Trip_Enabled or CP3_Trip_OnRelay  or CP3_TripB_Pattern;
       amTrip2: begin
                TripAFunc:=CP3_Trip_Enabled or CP3_Trip_OnRelay ;
                TripBFunc:=CP3_Trip_Enabled or CP3_Trip_OnRelay  or CP3_TripB_Pattern;
                end;
       end;
end;

procedure InvertTripOutput(Value: boolean; var TripFunc: byte);
begin
  if Value then
     TripFunc:=TripFunc or  CP3_Trip_Inverted;
end;

function LFDMode1003FromHART(Value: byte): TOnOffSet;
begin
  if ((Value and CP3_LFD_OnRelay)<>0) then
     Result:=tsOn
  else Result:=tsOff;
end;

function ValueUnitsFromHART (Value: byte): TTripUnits;
begin
  // HART command 133, means 57 = Percent  38 = Process Unit (Hz)
  if (Value = CHart_Percent) then Result:=tuPercent else Result:=tuProcess;
end;

function ValueUnitsToHART (Value: TTripUnits): byte;
begin
  // HART command 134, means 57 = Percent  38 = Process Unit (Hz)
  if (Value = tuPercent) then Result:=CHart_Percent else Result:=CHart_Herz;
end;

function PercentValueFromRange(Value, Min, Max: single): single;
var lSpan,lValue: real;
begin
  lSpan:=System.Abs(Max-Min);
  lValue:=System.Abs(Value-Min);

  Result:=100.0*lValue/lSpan;
end;


function EncodeByteValueFromString (Value: string; var Target: byte): boolean;
begin
  Result:=true;
  try Target:=StrToInt(Value);
  except
    on EConvertError do Result:=false;
    end;
end;

function EncodeWordValueFromString (Value: string; var Target: word): boolean;
begin
  Result:=true;
  try Target:=StrToInt(Value);
  except
    on EConvertError do Result:=false;
    end;
end;

function EncodeSingleValueFromString (Value: string; var Target: single): boolean;
var OD: char;
begin
  Value:=DeleteLeadingChars(Value,' ');
  OD:=DecimalSeparator;
  if (System.Pos('.',Value)>0) then DecimalSeparator:='.'
  else if (System.Pos(',',Value)>0) then DecimalSeparator:=',';
  Result:=true;
  try Target:=StrToFloat(Value);
  except
    on EConvertError do Result:=false;
    end;
  DecimalSeparator:=OD;
end;

function EncodeShortDateString (Value: string): TSimpleDate;
var InterResult: TSimpleDate;
begin
  if (Value[1] in CASCIIIntegerDigits) and (Value[2] in CASCIIIntegerDigits) and
     (Value[4] in CASCIIIntegerDigits) and (Value[5] in CASCIIIntegerDigits) and
     (Value[7] in CASCIIIntegerDigits) and (Value[8] in CASCIIIntegerDigits) then
     begin
     InterResult.D:=StrToInt(System.Copy(Value,1,2));
     InterResult.M:=StrToInt(System.Copy(Value,4,2));
     InterResult.Y:=StrToInt(System.Copy(Value,7,2));
     if (InterResult.Y<50) then InterResult.Y:=InterResult.Y+100;
     Result:=InterResult;
     end
  else Result:=CDefDate;
end;

function EncodeLongDateString (Value: string): TSimpleDate;
var InterResult: TSimpleDate;
begin
  if (Value[1] in CASCIIIntegerDigits) and (Value[2] in CASCIIIntegerDigits) and
     (Value[4] in CASCIIIntegerDigits) and (Value[5] in CASCIIIntegerDigits) and
     (Value[7] in CASCIIIntegerDigits) and (Value[8] in CASCIIIntegerDigits) and (Value[9] in CASCIIIntegerDigits) and (Value[10] in CASCIIIntegerDigits) then
     begin
     InterResult.D:=StrToInt(System.Copy(Value,1,2));
     InterResult.M:=StrToInt(System.Copy(Value,4,2));
     InterResult.Y:=StrToInt(System.Copy(Value,7,4))-1900;
     Result:=InterResult;
     end
  else Result:=CDefDate;
end;

function Check_SimpleDate(var SimpleDate: TSimpleDate): boolean;
var WrongDate: boolean;
begin
  if not(SimpleDate.Y<1900) and (SimpleDate.M in [1..12]) then
      WrongDate:=not((SimpleDate.D>0) and not(SimpleDate.D>GetDaysInMonth(SimpleDate.Y,SimpleDate.M)))
  else WrongDate:=true;
  if WrongDate then DecodeDate(Date,SimpleDate.Y,SimpleDate.M,SimpleDate.D);
  Result:=not(WrongDate);
end;

function EncodeHARTDate(Day,Month,Year: byte): TDate;
var FullYear: word;
    DateCorrect: boolean;
begin
  if (Day>0) and (Month in [1..12]) then
     begin
     FullYear:=Year+1900;
     DateCorrect:=not(Day>GetDaysInMonth(FullYear,Month));
     end
  else DateCorrect:=false;
  if DateCorrect then
     Result:=EncodeDate(FullYear,Month,Day)
  else Result:=CMinimalHARTDate; // 01/01/1900
end;

procedure DecodeHARTDate(Source: TDate; var Day,Month,Year: byte);
var FullYear,D,M: word;
begin
  DecodeDate(Source,FullYear,M,D);
  Year:=FullYear-1900;
  Month:=M;
  Day:=D;
end;

function ManufacturersIdentNameFromHART(Value: byte): string;
begin
  if (Value=CHart_Manufacturer_MTL) then Result:=CMtlManufacturer
  else Result:='Manufacturer number '+IntToStr(Value);
end;

function CheckManufacturersIdentCodeFromString (Value: string): boolean;
begin
  Result:=(UpperCase(Value)=UpperCase(CMtlManufacturer));
end;

function GetDevTypeFromString (Value: string): TDeviceType;
begin
  if (UpperCase(Value)='MTL4073XK') then Result:=dt4073XK
  else if (UpperCase(Value)='MTL5074XK') then Result:=dt5074XK
  else if (UpperCase(Value)='MTL4075') then Result:=dt4075
  else if (System.Pos('MTL4576',UpperCase(Value))>0) then Result:=dtx576
  else if (System.Pos('MTL5576',UpperCase(Value))>0) then Result:=dtx576
  else if (System.Pos('MTL4575',UpperCase(Value))>0) then Result:=dtx575
  else if (System.Pos('MTL5575',UpperCase(Value))>0) then Result:=dtx575
  else if (System.Pos('MTL4573',UpperCase(Value))>0) then Result:=dtx573
  else if (System.Pos('MTL5573',UpperCase(Value))>0) then Result:=dtx573
  else if (System.Pos('MTL4532',UpperCase(Value))>0) then Result:=dtx532
  else if (System.Pos('MTL5532',UpperCase(Value))>0) then Result:=dtx532
  else if (System.Pos('MTL4582',UpperCase(Value))>0) then Result:=dtx582B
  else if (System.Pos('MTL5582',UpperCase(Value))>0) then Result:=dtx582B
  else if (System.Pos('MTL5074',UpperCase(Value))>0) then Result:=dt507x
  else if (System.Pos('MTL5073',UpperCase(Value))>0) then Result:=dt507x
  else if (System.Pos('MTL4073',UpperCase(Value))>0) then Result:=dtx073
  else if (System.Pos('MTL3073',UpperCase(Value))>0) then Result:=dtx073
  else if (UpperCase(Value)='E01003') then Result:=dt1003
  else Result:=dtNone;
end;

function EncodeHardwareRevisionFromString (Value: string; var Target: byte): boolean;
var SM,SL: byte;
    P: word;
begin
  Result:=true;
  P:=System.Pos('.',Value);
  if P>0 then
     if (EncodeByteValueFromString(System.Copy(Value,1,P-1),SM)) and
        (EncodeByteValueFromString(System.Copy(Value,P+1,System.Length(Value)-P),SL)) then Target:=(SM shl 3) + SL
     else Result:=false;
end;

function EncodePVUnitsFromString (Value: string; var Target: byte): boolean;
var InterValue: string;
begin
  // HART command 44 -> 32=C, 33=F, 34=R, 35=K, 36=mV, 37=Ohm, 38=Herz, 39=mA, 57=%
  Result:=true;
  InterValue:=UpperCase(Value);
  if (System.Pos('C',InterValue)>0) then Target:=CHart_Celsius
  else if (System.Pos('F',InterValue)>0) then Target:=CHart_Fahrenheit
  else if (System.Pos('R',InterValue)>0) then Target:=CHart_Rankine
  else if (System.Pos('K',InterValue)>0) then Target:=CHart_Kelvin
  else if (System.Pos(UpperCase('mV'),InterValue)>0) then Target:=CHart_millivolt
  else if (System.Pos('O',InterValue)>0) then Target:=CHart_Ohm
  else if (System.Pos('H',InterValue)>0) then Target:=CHart_Herz
  else if (System.Pos(UpperCase('mA'),InterValue)>0) then Target:=CHart_milliamper
  else if (System.Pos('%',InterValue)>0) then Target:=CHart_Percent
  else Result:=false;
end;

function EncodeAlarmFromString (Value: string; var Target: byte): boolean;
var I,LN: integer;
begin
  // HART command 128, parameter 0, 1 and 70 means:
  //  1 = Low, 2 = High and 3 = Off
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.AlarmLow)) then
         begin
         Target:=1; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.AlarmHigh)) then
              begin
              Target:=2; Result:=true; break;
              end
            else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.AlarmOff)) then
                    begin
                    Target:=3; Result:=true; break;
                    end;
end;

function TempInputTypeNameFromDeviceInfo(DeviceType: TDeviceType; InputType: TTempInputType; Linearisation: TOnOffSet; TypeNumber,WiringNumber: word): string;
begin
  Result:='error';
  if InputType=itTHC then
     case DeviceType of
          dt507x,
          dtx073: if TypeNumber in [0..7] then Result:=LanguageStrings[flTHC_B+TypeNumber]+' '+LanguageStrings[flTType]
                  else if TypeNumber=8 then Result:=LanguageStrings[flmV];
          dt4073XK,
          dt5074XK:  if TypeNumber in [0..5,7] then Result:=LanguageStrings[flTHC_B+TypeNumber]+' '+LanguageStrings[flTType]
                     else if TypeNumber=6 then Result:=LanguageStrings[flTHC_XK]+' '+LanguageStrings[flTType]
                     else if TypeNumber=8 then Result:=LanguageStrings[flmV];

          dt4075,
          dtx575,
          dtx573: if TypeNumber in [0..8] then Result:=LanguageStrings[flTHC_B+TypeNumber]+' '+LanguageStrings[flTType]
                  else if TypeNumber=9 then Result:=LanguageStrings[flmV];
          dtx576: if TypeNumber in [0..8] then Result:=LanguageStrings[flTHC_B+TypeNumber]+' '+LanguageStrings[flTType]
                  else if TypeNumber=9 then Result:=LanguageStrings[flUser] // user THC sensor
                  else if TypeNumber=10 then Result:=LanguageStrings[flmV];
          end
  else if InputType=itRTD then
          case DeviceType of
               dt507x,
               dtx073,
               dt4073XK,
               dt5074XK: if (WiringNumber in [0..2]) and (TypeNumber=0) then
                            begin
                            if Linearisation=tsOn then
                               Result:=LanguageStrings[flPt100]+', '+LanguageStrings[fl2Wire+WiringNumber]
                            else Result:=LanguageStrings[flResistor]+', '+LanguageStrings[fl2Wire+WiringNumber];
                            end;
               dt4075: if (WiringNumber in [0..2]) and (TypeNumber in [0..2]) then
                          begin
                          if Linearisation=tsOn then
                             Result:=LanguageStrings[flPt100+TypeNumber]+', '+LanguageStrings[fl2Wire+WiringNumber]
                          else Result:=LanguageStrings[flResistor]+', '+LanguageStrings[fl2Wire+WiringNumber];
                          end;
               dtx575,
               dtx573: if (WiringNumber in [0..2]) then
                          begin
                          if (TypeNumber=0) then
                             begin
                             if Linearisation=tsOn then
                                Result:=LanguageStrings[flPt100+TypeNumber]+', '+LanguageStrings[fl2Wire+WiringNumber]
                             else Result:=LanguageStrings[flResistor]+', '+LanguageStrings[fl2Wire+WiringNumber];
                             end
                          else if (TypeNumber in [1..5]) then
                             Result:=LanguageStrings[flPt500+TypeNumber-1]+', '+LanguageStrings[fl2Wire+WiringNumber]
                          else if (TypeNumber in [6..7]) then
                             Result:=LanguageStrings[flPt100+TypeNumber-5]+', '+LanguageStrings[fl2Wire+WiringNumber];
                       end;
               dtx576: if (WiringNumber in [0..2]) then
                          begin
                          if (TypeNumber in [0..7]) then
                             Result:=LanguageStrings[flPt100+TypeNumber]+', '+LanguageStrings[fl2Wire+WiringNumber]
                          else
                             if (TypeNumber=8) then
                                Result:=LanguageStrings[flUser]+', '+LanguageStrings[fl2Wire+WiringNumber]
                             else
                                if (TypeNumber=9) then
                                   Result:=LanguageStrings[flResistor]+', '+LanguageStrings[fl2Wire+WiringNumber];
                          end;
               end;
end;

function TempInputTypeNameFromHARTInfo(DeviceType: TDeviceType; InputTypeInfo,LinearisationInfo: byte): string;
var sTypeNumber,sWiringNumber: word;
    sLinearisation: TOnOffSet;
    sInputType: TTempInputType;
begin
  sInputType:=InputTypeFromHART(DeviceType,InputTypeInfo);
  sTypeNumber:=TypeNumberFromHART(DeviceType,InputTypeInfo);
  sWiringNumber:=WiringNumberFromHART(DeviceType,InputTypeInfo);
  sLinearisation:=LinearFromHART(LinearisationInfo);

  Result:=TempInputTypeNameFromDeviceInfo(DeviceType,sInputType,sLinearisation,sTypeNumber,sWiringNumber);
end;

function DetectSpecifiedTHC(Source,THC: string): boolean;
begin
  // This should clearly detect the THC type in English, French and German language
  Result:=(System.Pos(UpperCase(THC+' typ'),Source)>0) or
          (System.Pos(UpperCase('typ '+THC),Source)>0) or
          (System.Pos(UpperCase('type '+THC),Source)>0);
end;

function Encodex073InputTypeFromString (Value,Aux,PVU,LIN: string; DeviceType: TDeviceType; var Target: byte): boolean;
var L, N: integer;
    lPVUnits, lLin: byte;
begin
  // Valid for all temperature converters except MTL4576
  Result:=true;
  Value:=UpperCase(Value);
  if (Value<>'') then
     begin
     if (System.Pos(UpperCase('typ'),Value)>0) then // detect any THC input type
        begin
        if DetectSpecifiedTHC(Value,'XK') then
           begin
           if (DeviceType in [dt4073XK,dt5074XK]) then Target:=6
           else if (DeviceType in [dt4075,dtx573,dtx575]) then Target:=20
           else Result:=false;
           end
        else if DetectSpecifiedTHC(Value,'B') then
                Target:=0
        else if DetectSpecifiedTHC(Value,'E') then
                Target:=1
        else if DetectSpecifiedTHC(Value,'J') then
                Target:=2
        else if DetectSpecifiedTHC(Value,'K') then
                Target:=3
        else if DetectSpecifiedTHC(Value,'N') then
                Target:=4
        else if DetectSpecifiedTHC(Value,'R') then
                Target:=5
        else if DetectSpecifiedTHC(Value,'S') then
                Target:=6
        else if DetectSpecifiedTHC(Value,'T') then
                Target:=7
        else Result:=false;
        end
     else
        if (System.Pos(UpperCase('mV'),Value)>0) then // mV input
           Target:=17
        else // if must be any RTD (or wrong) input
           begin
           if (System.Pos(UpperCase('Pt1000'),Value)=1) then Target:=10
           else if (System.Pos(UpperCase('Pt500'),Value)=1) then Target:=9
           else if (System.Pos(UpperCase('Pt100'),Value)=1) or
                   (System.Pos(UpperCase('Pt100'),Value)=8) or
                   (System.Pos(UpperCase('RTD'),Value)=8) then Target:=8
           else if (System.Pos(UpperCase('Cu50'),Value)=1) then Target:=32
           else if (System.Pos(UpperCase('Cu53'),Value)=1) then Target:=64
           else if (System.Pos(UpperCase('Ni1000'),Value)=1) then Target:=130
           else if (System.Pos(UpperCase('Ni500'),Value)=1) then Target:=129
           else if (System.Pos(UpperCase('Ni100'),Value)=1) then Target:=128
           else
              // check if the input type is eventually resistor (PVUnits=Ohm and linearisation off)
              // Note: in some legacy PCS45 version this input type was not named as Pt100
              begin
              EncodePVUnitsFromString(PVU,lPVUnits);
              EncodeX073LinearisationFromString(LIN,lLin);
              if (lPVUnits=37) and  // Ohm
                 (lLin=1) then      // Linearisation off
                 begin
                 Target:=8;
                 end
              else Result:=false;
              end;
           if Result and
              (((DeviceType in [dt507x..dt4073XK]) and (Target=8)) or
               ((DeviceType=dt4075) and (Target in[8,32,64])) or
               ((DeviceType in[dtx575,dtx573]) and (Target in[8,9,10,32,64,128,129,130]))) then
              begin
              L:=System.Length(Value);
              N:=System.Pos(',',Value);
              if (Aux='') and
                 (N>0) and
                 (L>N)then
                 Aux:=System.Copy(Value,N+1,L-N);
              if (System.Pos(UpperCase('4'),Aux)>0) then Target:=Target+6
              else if (System.Pos(UpperCase('3'),Aux)>0) then Target:=Target+3
              else if (System.Pos(UpperCase('2'),Aux)>0) then Target:=Target
              else Result:=false;
              end;
           end;
     end
  else Result:=false;
end;

function DetectUserSensorFromString (Value: string): boolean;
var I,LN: integer;
begin
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.User)) then
         begin
         Result:=true; break;
         end;
end;

function Encodex576InputTypeFromString (Value,Aux,PVU,LIN: string; DeviceType: TDeviceType; var Target: byte): boolean;
var L, N: integer;
    lPVUnits, lLin: byte;
begin
  // Valid only for the temperature converters MTL4576
  Result:=true; // LN:=System.Length(LanguageInfos);
  Value:=UpperCase(Value);
  if (Value<>'') then
     begin
     // detect first THC User or RTD User
     // and manipulate Value string for further detection
     if DetectUserSensorFromString(Value) then
        begin
        if (Aux='') then
           Value:=UpperCase('User type') // assume THC User
        else Value:=UpperCase('User')    // assume RTD User
        end;
     if (System.Pos(UpperCase('typ'),Value)>0) then // detect any THC input type
        begin
        if DetectSpecifiedTHC(Value,'User') then
           Target:=9
        else if DetectSpecifiedTHC(Value,'XK') then
                Target:=8
        else if DetectSpecifiedTHC(Value,'B') then
                Target:=0
        else if DetectSpecifiedTHC(Value,'E') then
                Target:=1
        else if DetectSpecifiedTHC(Value,'J') then
                Target:=2
        else if DetectSpecifiedTHC(Value,'K') then
                Target:=3
        else if DetectSpecifiedTHC(Value,'N') then
                Target:=4
        else if DetectSpecifiedTHC(Value,'R') then
                Target:=5
        else if DetectSpecifiedTHC(Value,'S') then
                Target:=6
        else if DetectSpecifiedTHC(Value,'T') then
                Target:=7
        else Result:=false;
        end
     else
        if (System.Pos(UpperCase('mV'),Value)>0) then // mV input
           Target:=$0F
        else // if must be any RTD (or wrong) input
           begin
           if (System.Pos(UpperCase('User'),Value)=1) then Target:=$90
           else if (System.Pos(UpperCase('Pt1000'),Value)=1) then Target:=$30
           else if (System.Pos(UpperCase('Pt500'),Value)=1) then Target:=$20
           else if (System.Pos(UpperCase('Pt100'),Value)=1) then Target:=$10
           else if (System.Pos(UpperCase('Ni1000'),Value)=1) then Target:=$60
           else if (System.Pos(UpperCase('Ni500'),Value)=1) then Target:=$50
           else if (System.Pos(UpperCase('Ni100'),Value)=1) then Target:=$40
           else if (System.Pos(UpperCase('Cu50'),Value)=1) then Target:=$70
           else if (System.Pos(UpperCase('Cu53'),Value)=1) then Target:=$80
           else if (System.Pos(UpperCase('Ohm'),Value)=1) then Target:=$F0 // eventually different coding ?
           else // check if the input type is eventually resistor (PVUnits=Ohm and linearisation off)
              begin
              EncodePVUnitsFromString(PVU,lPVUnits);
              Encodex073LinearisationFromString(LIN,lLin);
              if (lPVUnits=37) and  // Ohm
                 (lLin=1) then      // Linearisation off
                 begin
                 Target:=$F0;
                 end
              else Result:=false;
              end;
           if Result and (Target in [$10,$20,$30,$40,$50,$60,$70,$80,$90,$F0]) then
              begin
              L:=System.Length(Value);
              N:=System.Pos(',',Value);
              if (Aux='') and
                 (N>0) and
                 (L>N)then
                 Aux:=System.Copy(Value,N+1,L-N);
              if (System.Pos(UpperCase('4'),Aux)>0) then Target:=Target+2
              else if (System.Pos(UpperCase('3'),Aux)>0) then Target:=Target+1
              else if (System.Pos(UpperCase('2'),Aux)>0) then Target:=Target
              else Result:=false;
              end;
           end;
     end
  else Result:=false;
end;

function Encodex073LinearisationFromString (Value: string; var Target: byte): boolean;
var I,LN: integer;
begin
  // HART command 47 means 0 = linearisation on and 1 = linearisation off
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=1; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then
              begin
              Target:=0; Result:=true; break;
              end;
end;

function Encodex073CJCFromString (Value: string; var Target: byte): boolean;
var I,LN: integer;
begin
  // HART command 128, parameter 5 means 1 = enabled and 2 = disabled
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=2; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then
              begin
              Target:=1; Result:=true; break;
              end;
end;

function Encodex576CJCFromString (Value: string; var Target: byte): boolean;
var I,LN: integer;
begin
  // HART command 151/152, parameter 5 means 0 = disabled, 1 = enabled and 2 = User
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=0; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then
              begin
              Target:=1; Result:=true; break;
              end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.User)) then
              begin
              Target:=2; Result:=true; break;
              end;
end;

function Encodex073FilterFreqFromString (Value: string; var Target: byte): boolean;
begin
  // HART command 128, parameter 2 means 1 = 50Hz  2 = 60Hz
  Result:=true;
  if (System.Pos('50',Value)>0) then Target:=1
  else if (System.Pos('60',Value)>0) then Target:=2
  else Result:=false;
end;

function Encodex075TripFunctionFromString (Value: string; var Target: byte): boolean;
var I,LN: integer;
begin
  // HART command 204 and 205 for MTL4075 means 0 = disabled, 2 = enabled and 1 = EBD
  if (IsFloatingPointString(Value,'.')) or
     (IsFloatingPointString(Value,',')) then
     begin
     Target:=2;
     Result:=true;
     end
  else
     begin
     Result:=false; LN:=System.Length(LanguageInfos);
     for I:=0 to LN-1 do
         if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
            begin
            Target:=0; Result:=true; break;
            end
         else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.EBD)) then
                 begin
                 Target:=1; Result:=true; break;
                 end
         else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.Trip)) then
                 begin
                 Target:=2; Result:=true; break;
                 end;
     end;
end;

function Encodex075TripParameterFromString (Value: string; var Target: THx075TripParameters): boolean;
var lTripValue: Extended;
    lDC: char;
begin
  // HART command 204 and 205 for MTL4075 means 0 = disabled, 2 = enabled and 1 = EBD
  Result:=Encodex075TripFunctionFromString (Value, Target.TripOutputFunction);
  if Result and
     (Target.TripOutputFunction=2) then
      begin
      if (IsFloatingPointString(Value,'.')) then
         lDC:='.'
      else if (IsFloatingPointString(Value,',')) then
              lDC:=','
           else lDC:=#0;
      if (lDC<>#0) then
          begin
          lTripValue:=GetRealFromString(Value,lDC,0.0);
          if (lTripValue>0.0) then
             begin
             Target.TripValue:=lTripValue;
             Target.TripOutputFunction:=2;
             Result:=true;
             end;
          end;
     end;
end;

function Encodex573EBDFromString (Value: string; var Target: byte): boolean;
var I, LN: integer;
begin
  // HART command 204 and 205 for MTL4573 means 0 = disabled, 1 = EBD
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=0; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then
              begin
              Target:=1; Result:=true; break;
              end;
end;

function Encodex575ResponseTimeFromString(Value: string; var Target: byte): boolean;
var lResponseTime: integer;
    I,LN: integer;
begin
  // HART command 207 and 208 for MTL4575 means 0 = Slow, 1 = Medium and 2 = Fast
  // Note: in case of wrong information set response to slow (instead to raise fault)
  Value:=DeleteLeadingChars(Value,' ');
  if not(IsDecimalIntegerString(Value)) then
     begin
     LN:=System.Length(LanguageInfos);
     for I:=0 to LN-1 do
         if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.Slow)) then
            begin
            Target:=0; Result:=true; break;
            end
         else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.Medium)) then
                 begin
                 Target:=1; Result:=true; break;
                 end
               else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.Fast)) then
                       begin
                       Target:=2; Result:=true; break;
                       end;
     end
  else // some legacy units have used numeric values
     begin
     TryStrToInt(Value,lResponseTime);
     case lResponseTime of
          100: Target:=2;
          200: Target:=1;
          else Target:=0;
          end;
     Result:=true;
     end
end;

function Encodex575OutputCharacteristicFromString(Value: string; var Target: byte): boolean;
begin
  // HART command 209/210 for MTL4575 means:
  //  0 = Off (MTL), 1 = On (NAMUR)
  Result:=true;
  Target:=0;
  if (System.Pos(UpperCase('NAMUR'),UpperCase(Value))>0) or
     (System.Pos(UpperCase('NE43'),UpperCase(Value))>0) then
     Target:=1
  else if (System.Pos(UpperCase('MTL'),UpperCase(Value))>0) then
          Target:=0
       else Result:=false;
end;

function Encodex532OutputCharacteristicFromString(Value,Aux: string; var Target: byte): boolean;
begin
  Result:=true;
  Target:=0; // default value - 4-20mA, no NE43
  if (Aux<>'') then
     begin
     if (System.Pos('4-20',Value)>0) then
        begin
        if (UpperCase(Value)=UpperCase('NAMUR')) then
           Target:=2;
        end
     else Result:=false; // only 4-20 allowed
     end
  else // Aux='' (empty)
     begin
     if (System.Pos('0-20',Value)>0) then
        Target:=1
     else
        if (System.Pos('4-20',Value)>0) then
           begin
           if (UpperCase(Value)=UpperCase('NE43')) then
              Target:=2;
           end
        else Result:=false;
     end;
end;

function Encodex576OutputCharacteristicFromString(Value: string; var Target: byte): boolean;
begin
  // HART command 161/162, parameter 6 for MTL4576 means:
  //  0 = On (NAMUR), 1 = Off (MTL)
  Result:=true;
  Target:=1;
  if (System.Pos(UpperCase('NAMUR'),UpperCase(Value))>0) or
     (System.Pos(UpperCase('NE43'),UpperCase(Value))>0) then
     Target:=0
  else if (System.Pos(UpperCase('MTL'),UpperCase(Value))>0) then
          Target:=1
       else Result:=false;
end;

function Encodex576InputOptionFromString(Value: string; var Target: byte): boolean;
var I, LN, lInputOption: integer;
begin
  // HART command 207 and 208, parameter 3 for MTL4576 means 0 = 2in-2out, 1 = 1in-2out etc
  Value:=DeleteLeadingChars(Value,' ');
  if not(IsDecimalIntegerString(Value)) then
     begin
     LN:=System.Length(LanguageInfos);
     for I:=0 to LN-1 do
         if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.in2out2)) then
            begin
            Target:=0; Result:=true; break;
            end
         else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.in1out2)) then
                 begin
                 Target:=1; Result:=true; break;
                 end;
     end
  else // legacy PCS45 versions have used numeric values
     begin
     TryStrToInt(Value,lInputOption);
     if not(lInputOption<0) and
        not(lInputOption>1) then
        begin
        Target:=lInputOption;
        Result:=true;
        end;
     end
end;

// The below functions and procedure for E01003 and MTL4532 are possible incorrect
// and shall be reviewed and updated

function Encodex532InputSignalSourceFromString (Value: string; var Target: byte): boolean;
var I, LN: integer;
begin
  // HART command 131 and 132 means:
  //  0 = Voltage or Current Pulses, 1 - Switch/Proximity detector
  // Result:=false; LN:=System.Length(LanguageInfos);
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.CurrentVoltage)) then
         begin
         Target:=0; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.SwitchProxy)) then
              begin
              Target:=1; Result:=true; break;
              end;
end;

function Encodex532AdjustModeFromString (Value: string; var Target: byte): boolean;
var I, LN: integer;
begin
  // HART command 140 and 141 means:
  //  0 = Off, 4 = enabled (Software) and only for E01003: 9 = Hardware
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=0; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) or      // MTL4532
              (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.SoftAdjust)) then // E01003
              begin
              Target:=4; Result:=true; break;
              end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.HardAdjust)) then // only E01003
              begin
              Target:=9; Result:=true; break;
              end;
  if not(Result) then
     begin
     if (UpperCase(Value)=UpperCase('Software')) then // only E01003
        begin
        Target:=4; Result:=true;
        end
     else if (UpperCase(Value)=UpperCase('Hardware')) then // only E01003
             begin
             Target:=9; Result:=true;
             end;
     end;
end;

function Encodex532LeakFlowOffFromString (Value: string; var Target: byte): boolean;
var I, LN: integer;
begin
  // HART command 133/134, parameter means 0 = Off  1 = On
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=0; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then
              begin
              Target:=1; Result:=true; break;
              end;
end;

function Encodex532PulseOutputModeFromString (Value: string; var Target,Input: byte): boolean;
var I, LN: integer;
begin
  // HART command 138/139, parameter means 0 = Off, 1 = On (fixed pulses),  3 = toggle output
  // However also command 131/132 parameter means $04 = 1:1 direct
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=0; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then
              begin
              Target:=1; Result:=true; break;
              end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.Toggle)) then
              begin
              Target:=3; Result:=true; break;
              end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.FixedPulses)) then
              begin
              Target:=1; Result:=true; break;
              end;
  if not(Result) and
     (System.Pos('1:1',Value)>0) then
     begin
     Target:=0; Input:=Input or CP_PulseToInput;
     Result:=true;
     end;
end;

function Encodex532AlarmOutputModeFromString (Value: string; var Target,Input: byte): boolean;
var I, LN: integer;
begin
  // HART command 136/137, parameter means $=00 = Off
  // $01 - Trip A on Alarm output, $04 - Trip B on Pulse Output,
  // $10 - Trip AB on Alarm Output, $20 - Trip AB on Pulse Output
  // $02 - Trip A inverted, $08 - Trip B inverted, $40 - Trip AB inverted
  // $80 - Inhibit enabled (not evaluated here)
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=CP_Blank_Pattern; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LFD)) or
              (UpperCase(Value)='LFD') then
              begin
              Target:=CP_Blank_Pattern; Input:=Input or CP_AlarmToLFD; Result:=true; break;
              end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.SimpleTrip)) then
              begin
              Target:=CP_TripAOnALarm; Result:=true; break;
              end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.WindowTrip)) then
              begin
              Target:=CP_TripABOnAlarm; Result:=true; break;
              end;
end;

function Encodex532InhibitModeFromString(Value: string; var Target: byte): boolean;
var I, LN: integer;
begin
  // HART command 133/134, parameter means $=00 = Off  $80 = On
  Result:=false; LN:=System.Length(LanguageInfos);
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Target:=0; Result:=true; break;
         end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then   // normal
              begin
              Target:=Target or CP_AlarmToInhibit; Result:=true; break;
              end
      else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOn)) then   // inverted
              begin
              Target:=Target or CP_AlarmToInhibit; Result:=true; break;
              end;
end;

// Encode functions only for compatibility with text files from Pcs-E01003, Version 1.04
function Encode1003OutputModeFromString(Value: string; var Target: T1003OutputMode): boolean;
begin
  Result:=false;
  if (UpperCase(Value)=UpperCase('aus')) then
     begin
     Target:=omOff; Result:=true;
     end
  else if (UpperCase(Value)=UpperCase('Ch1')) then
          begin
          Target:=omTransistor; Result:=true;
          end
  else if (UpperCase(Value)=UpperCase('Ch2')) then
          begin
          Target:=omRelay; Result:=true;
          end;
end;

function Encode1003InvertedTrip(Value: string; var Target: byte): boolean;
begin
  Result:=false;
  if (UpperCase(Value)=UpperCase('tief')) or
     (UpperCase(Value)=UpperCase('hoch')) then
     begin
     Result:=true;
     if (UpperCase(Value)=UpperCase('hoch')) then
        Target:=Target or CP3_Trip_Inverted;
     end;
end;

function Encode1003PulseOutputModeFromString (Value,Aux: string; var DVPFS,TPAFS,TPBFS,Input: byte): boolean;
var lFound, lInvert: boolean;
    I, LN: integer;
begin
  Result:=false; LN:=System.Length(LanguageInfos);
  lFound:=false; lInvert:=false;
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Result:=true; break;
         end
  else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.FixedPulses)) then
          begin
          DVPFS:=CP3_Divider_Enabled; Result:=true; break;
          end
  else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LFD)) then
          begin
          Input:=Input or CP3_LFD_Enabled; Result:=true; break;
          end
       else
          begin
          if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.normal)) then
             begin
             lFound:=true;
             end;
          if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.inverted)) then
             begin
             lFound:=true; lInvert:=true;
             end;
          if lFound then
             begin
             if (System.Pos('1',Aux)=0) then
                begin
                TPAFS:=CP3_Trip_Enabled;
                if lInvert then
                   TPAFS:=TPAFS or CP3_Trip_Inverted;
                end
             else // the trip text consists 1, so its must be a window trip
                begin
                TPAFS:=CP3_Trip_Enabled; TPBFS:=CP3_Trip_Enabled or CP3_TripB_Pattern;
                if lInvert then
                   begin
                   TPAFS:=TPAFS or CP3_Trip_Inverted;
                   TPBFS:=TPBFS or CP3_Trip_Inverted;
                   end;
                end;
             Result:=true; break;
             end;
          end;
end;

function Encode1003AlarmOutputModeFromString (Value,Aux: string; var TPAFS,TPBFS,Input: byte): boolean;
var lFound, lInvert: boolean;
    I, LN: integer;
begin
  Result:=false; LN:=System.Length(LanguageInfos);
  lFound:=false; lInvert:=false;
  for I:=0 to LN-1 do
      if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LabelOff)) then
         begin
         Result:=true; break;
         end
  else if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.LFD)) then
          begin
          Input:=Input or CP3_LFD_Enabled or CP3_LFD_OnRelay; Result:=true; break;
          end
       else
          begin
          if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.normal)) then
             begin
             lFound:=true;
             end;
          if (UpperCase(Value)=UpperCase(LanguageInfos[I].IELabels.inverted)) then
             begin
             lFound:=true; lInvert:=true;
             end;
          if lFound then
             begin
             if (System.Pos('1',Aux)=0) then
                begin
                TPBFS:=CP3_Trip_Enabled or CP3_Trip_OnRelay;
                if lInvert then
                   TPBFS:=TPBFS or CP3_Trip_Inverted;
                end
             else // the trip text consists 1, so its must be a window trip
                begin
                TPAFS:=CP3_Trip_Enabled or CP3_Trip_OnRelay; TPBFS:=CP3_Trip_Enabled or CP3_TripB_Pattern or CP3_Trip_OnRelay;
                if lInvert then
                   begin
                   TPAFS:=TPAFS or CP3_Trip_Inverted;
                   TPBFS:=TPBFS or CP3_Trip_Inverted;
                   end;
                end;
             Result:=true; break;
             end;
          end;
end;


function GetDynArray(Source: TUnitEnableArray): TBooleanDynArray;
var NewBoolArray: TBooleanDynArray;
    I, L: integer;
begin
  L:=SizeOf(TUnitEnableArray);
  SetLength(NewBoolArray,L);
  for I:=0 to L-1 do
      NewBoolArray[I]:=Source[I];
  Result:=NewBoolArray;
end;

end.




