//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**
//**  ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
//**  DATACONSTANTS.PAS
//**
//**  Declaration of global constants for PCS455
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.11.96 / 10.07.01 / 01.04.03 / ... / 29.03.07 / ... / 08.01.10 / 11.04.14 / 25.04.17
//**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit DataConstants;

interface

uses Windows, ConversionUtils, DataTypes;

const

  CApplicationName   = 'PCS45';

  CPcs45Version      = '4.02'; // final version will be be 4.02

  CDefaultLanguage   = LANG_ENGLISH;

  CPCL45USBName      = 'PCL45USB PC link';

  CMtlManufacturer   = 'MTL Group';

  CDef_CFG: TCFG45   = (PortNumber: 0;
                        HighSpeed: false;
                        PCL45USBSelected: false;
                        Language: CDefaultLanguage;
                        Password: 'MTL';
                        ErrorIndication: (true,true,false));

  CDef_MOD: TMOD45   = (DemoMode: true; DeviceConnected: false;
                        DataAvailable: false; PasswordReleased: false;
                        SupervisorAccess: false;
                        DevType: dtNone;
                        LastD45Path: '';
                        LastImpexPath: '';
                        LastLoggerPath: '';
                        PCL45USBConnected: false);

  CFastComm: TCommTimes = (Recovery :   5; ResponseDelay:  50; TimeOut: 1000; DelayParity: false);
  CSlowComm: TCommTimes = (Recovery :  50; ResponseDelay: 350; TimeOut: 2000; DelayParity: true);
  CVerySlow: TCommTimes = (Recovery : 100; ResponseDelay: 500; TimeOut: 10000; DelayParity: true);

  CDefaultNewDevice     = dtx573;

  CDef_FileName    = 'DEMO';

  CDef_Pcs45_Ext   = '.D45';
  CDef_PcsFile     = CDef_FileName+CDef_Pcs45_Ext;

  CDef_Pcc73_Ext   = '.D73';
  CDef_MTL_Ext     = '.MTL';

  CDef_TxtIExExt   = '.TXT';
  CDef_TxtIExFile  = CDef_FileName+CDef_TxtIExExt;

  CDef_CsvIExExt   = '.CSV';
  CDef_CsvIExFile  = CDef_FileName+CDef_CsvIExExt;

  CDef_ExcelExt    = '.XLS';
  Def_ExcelFile    = CDef_FileName+CDef_ExcelExt;

  CDef_LOGExt      = '.DAT';
  CDef_LOGFile     = CDef_FileName+CDef_LOGExt;

  COld_Config_Header = 'PCC73 configuration data file'+^J+^M+^Z;
  CNew_Config_Header = 'PCS45 configuration data file'+^J+^M+^Z;

  CDefDate: TSimpleDate = (D: 1; M: 1; Y: 80);

  CMinimalHARTDate = 2;     // 01/01/1900
  CMaximalHARTDate = 93503; // 31.12.2155 / This is end of the HART age

  // All temperature limits are specified in C
  C_Lim_B  : TRealLimits=(LLim:   0.0;  ULim:1820.0);
  C_Lim_E  : TRealLimits=(LLim: -270.0; ULim:1000.0);
  C_Lim_J  : TRealLimits=(LLim: -210.0; ULim:1200.0);
  C_Lim_K  : TRealLimits=(LLim: -270.0; ULim:1372.0);
  C_Lim_N  : TRealLimits=(LLim: -270.0; ULim:1300.0);
  C_Lim_R  : TRealLimits=(LLim:  -50.0; ULim:1767.6);
  C_Lim_S  : TRealLimits=(LLim:  -50.0; ULim:1767.6);
  C_Lim_R75: TRealLimits=(LLim:  -50.0; ULim:1768.1);
  C_Lim_S75: TRealLimits=(LLim:  -50.0; ULim:1768.1);
  C_Lim_XK : TRealLimits=(LLim: -200.0; ULim: 800.0);
  C_Lim_T  : TRealLimits=(LLim: -270.0; ULim: 400.0);

  C_Lim_Pt   : TRealLimits=(LLim:-200.0; ULim: 850.0);
  C_Lim_RLin : TRealLimits=(LLim:   0.0; ULim: 400.0);
  C_Lim_Cu50 : TRealLimits=(LLim:-200.0; ULim: 200.0);
  C_Lim_Cu53 : TRealLimits=(LLim: -50.0; ULim: 200.0);
  C_Lim_mV   : TRealLimits=(LLim: -77.0; ULim:  77.0);

  C_Lim_Pt1T : TRealLimits=(LLim:-200.0; ULim: 350.0);
  C_Lim_Ni   : TRealLimits=(LLim: -60.0; ULim: 180.0);

  C_Lim_Damp : TRealLimits=(LLim:   0.0; ULim:  60.0);
  C_Lim_Smoo : TLongLimits=(LLim:     0; ULim:    60);

  C_Lim_Trip : TRealLimits=(LLim:   5.0; ULim:  95.0); // in %
  C_Lim_Hyst : TRealLimits=(LLim:   0.3; ULim:   5.0);

  C_Lim_CJC  : TRealLimits=(LLim:   0.0; ULim:  60.0); // in C

  // Used for PVUnit selector on all temperature converters MTL3073...MTLx576
                                   // C    F    R     K   mV/Ohm
  CAllUnitsOn:  TUnitEnableArray = (true, true, true, true, true);
  CNonLinUnits: TUnitEnableArray = (true, true, true, true, false); // non linear sensor, means that linearisation is on
  CLinUnits:    TUnitEnableArray = (false,false,false,false,true);  // linear sensor, means that linearisation is off

  // The absolute low level for the frequency input signal is 0.00047 Hz - for Point 4mA (or for Point 20mA when inverted)
  C_LowLim_Freq = 0.00047;
  // Any frequency input signal specified for the Point 4mA and for the Point 20mA must be between 0.00047 and 10000.1Hz
  C_Lim_Freq:  TRealLimits=(LLim: 0.00047; ULim: 10000.1);
  // Assuming worse acceptable resolution of 1%, the minimal upper level is 0.06 Hz - for Point 20mA (or for Point 4mA)
  // This is simultaneously the maximal frequency input signal span (absolute difference between Point 20mA and Point 4mA)
  C_SpanLim_Freq = 0.06;
  // The minimal frequency input signal span ratio is 1/3.  This is also caused by worse acceptable resolution of 1%
  C_MinSpanRatio = 1.0/3.0;

implementation

end.
