//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**  ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
//**  DATATYPES.PAS
//**
//**  Description of datatypes for PCS45
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.11.96 / 12.09.99 / ... / 01.04.03 / ... / 23.08.06 / ... / 08.01.10 / 22.04.14 / 29.07.15
///**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit DataTypes;

interface

uses
  Windows, SysUtils, LabelNumbers;

type

// The below definition with enumerated Device type numbers are very similar but not identical
// with the "ManufacturersDeviceType" as specified in the Hart specification document HCF-SPEC-127,
// command number 0. While different device versions with e.g. modified hardware or firmware
// or simple because of unconsequencies in the number allocation, may share the ID number,
// the below defined numbers are unique to the device and configuration interface behaviour.
// This numbers will be assigned by PCS45 after the connection is established and the MDT and other
// reply data were evaluated and the "true" device type was identified.
// Please note, that this number is never saved and it is alive only during PCS45 is working.
// Therefore there is no problem when in new PCS45 version the sequence will be changed.
// For better overview, the different product types and MTL Luton and MTL Bochum products
// are listed in different lines and adequately commented.

  TDeviceType = (dtNone,    // unknown or not identified product
                 dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx575,dtx573, // single channel temperature converter
                 dtx576,    // dual channel temperature converter
                 dtx532,    // single channel frequency converter
                 dtx582B,   // single channel RTD isolator
                 dt1003);   // MTL Instruments GmbH product. Don't put any Luton product to this line!

  TDevicesSet = set of TDeviceType;

  TLabelInfos    = array [flFirstLabelNumber..flLastLabelNumber] of string;

  TMOD45 = record
    DemoMode, DeviceConnected, DataAvailable,
    PasswordReleased, SupervisorAccess : boolean;
    DevType: TDeviceType;
    LastD45Path,
    LastImpexPath,
    LastLoggerPath: TFileName;
    PCL45USBConnected: boolean;
    end;

  TTrippleBoolArray = array [0..2] of boolean;

  TCFG45 = record
    PortNumber: integer;
    HighSpeed,
    PCL45USBSelected: boolean;
    Language: LANGID;
    Password: string[16];
    ErrorIndication: TTrippleBoolArray;
    end;

  TImportExportLabels = record
    LabelOn, LabelOff,
    AlarmLow, AlarmHigh, AlarmOff,
    Typ, Resist,
    Slow, Medium, Fast,
    EBD, Trip, User,
    in2out2,in1out2,
    SwitchProxy,CurrentVoltage,
    Toggle,FixedPulses,
    LFD,SimpleTrip,WindowTrip,
    SoftAdjust,HardAdjust,
    normal,inverted: string;
    end;

  TLanguageInfo = record
    LangNumber: LANGID;
    LangName: string;
    FileName: TFileName;
    ChangeWarning: string;
    IELabels: TImportExportLabels;
    end;

  TLanguageInfos = array of TLanguageInfo;

  TCommTimes = record
    Recovery,
    ResponseDelay,
    TimeOut : word;
    DelayParity: boolean;
    end;

  TByteSet = set of byte;

  TDialogMode = (dmDisplay, dmConfig);
  TSaveMode   = (smToMemory, smToDevice);

  TOnOffSet  = (tsOn, tsOff);
  TFilterFrequency = (tf50, tf60);
  TAlarmLevelSet  = (taLow, taHigh, taOff);

  TTempInputType = (itNone,itTHC,itRTD);
  TResponseSpeed  = (tsSlow,tsMiddle,tsFast);
  TNamurOptionSet  = (nsOff,nsOn,nsNotAvailable);

  Tx075TripType = (ttTrip,ttEBD,ttOff);

  Tx576InputOptionSet = (is2In2Out, is1In2Out, is2In1OutAdd, is2In1OutDiff, is2In1OutMin, is2In1OutMax);
  Tx576CJCSet = (csOn, csOff, csUser);

  TFreqInputType = (fiVoltCurrent,fiProxy);
  TOutputModeSet = (omDead,omLive,omNE43);

  TAdjustModeSet  = (tmOff, tmSoft, tmHard);
  TInhibitMode = (imOff,imNormal,imInverted);

  TPulseOutputMode = (pmOff, pm11, pmToggle, pmPulses, pmLFD, pmTrip1, pmTrip2);
  TAlarmOutputMode = (amOff, am11, amLFD, amTrip1, amTrip2);

  T1003OutputMode = (omOff, omTransistor, omRelay);
  TTripUnits = (tuPercent, tuProcess);

  PLongLimits = ^TLongLimits;
  TLongLimits = record
    LLim, ULim: LongInt;
    end;

  PRealLimits = ^TRealLimits;
  TRealLimits = record
    LLim, ULim: Real;
    end;

  PDataStream = ^TDataStream;
  TDataStream = array [0..65535] of byte;

  TExtendedDataStream = record
    MarkValue: word;
    Bytes: TDataStream;
    end;

  // Used for PVUnit selector on all temperature converters MTL3073...MTLx576
  TUnitEnableArray = array [0..4] of boolean;

implementation

end.

