//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**
//**  ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
//**  DEMOOPERATION.PAS
//**
//**  Demo operation for PCS45, derived from "(Hart)Commands.pas"
//**
//**  Autor : Georg  Hahn
//**  Datum :  xx.yy.09 / 10.11.09 / 16.04.14 / 29.07.15
//**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit DemoOperation;

interface

uses HartDevices, HartDataTypes, DataTypes;

  procedure SetDefaultTransmitterParameters (Value: TDeviceType);

  procedure SimulateTransmitterHartAnswer (HartCommand: THartCommand; var HartAnswer: THartAnswer);

type
  TRandomVariation = record
    Enabled: boolean;
    Ratio: real;
    end;

var
  RandomVariation: TRandomVariation;

  Demo_TDD      : THTagDescriptorDate;
  Demo_PVO      : THPrimaryVariableOutput;
  Demo_x073     : THx073ParameterSet;
  Demo_x075TPA  : THx075TripParameters;
  Demo_DV       : array [0..1] of THDynamicVariables;
  Demo_SV       : THSecondaryVariables;
  Demo_PV       : THPrimaryVariables;

  Demo_x576_In: array [0..1] of THInputParameters_x576;
  Demo_x576_Out: array [0..1] of THOutputParameters_x576;
  Demo_x576_Comm: THCommonParameters_x576;
  Demo_x576_DVO: array [0..1] of THPrimaryVariableOutput;

  // DemoRAW: TRAWInputOutputValues;
  // DemoCal: TCalibrationData;

implementation

uses ProcessUtils, ConversionProcs, HartDataConstants;

var
  DemoDevice: TDeviceType;

  Demo_UID      : THDeviceIdentifier;
  Demo_AssNum   : integer;
  Demo_NE43     : byte;
  Demo_Speed    : byte;
  Demo_020mA    : byte;
  Demo_Input    : byte;
  Demo_Invert   : byte;
  Demo_HWConf   : byte;
  Demo_SCAlarm  : byte;

  Demo_LeakOff  : THLeakFlowOffParameters_x532;
  Demo_Adjust   : THAdjustOutputParameters;
  Demo_Divider  : THDividerParameters;
  Demo_Trip_32  : THTripParameters_x532;
  Demo_TripA,
  Demo_TRipB    : THTripParameters_1003;

  // Demo_Trimmer: array [1..4] of THTrimmerVariables_1003;

  Demo_P4Cal,
  Demo_P20Cal   : single;

  Demo_RAW      : THRAWInputOutputValues;
  Demo_Cal      : THCalibrationData;

procedure SeTHDynamicVariables;
var S: single;
begin
  if (DemoDevice<>dtx576) then
     begin
     if ((DemoDevice=dtx532) or (DemoDevice=dt1003)) and
        (Demo_020mA>0) then // 0-20mA = Full Scale
        Demo_DV[0].LoopCurrent:=10.0
     else Demo_DV[0].LoopCurrent:=12.0;
     Demo_DV[0].PrimaryUnit:=Demo_PVO.PVUnits;
     Demo_DV[0].PrimaryVariable:=(Demo_PVO.Point20mA+Demo_PVO.Point4mA)/2.0;
     Demo_DV[0].SecondaryUnit:=CHart_Celsius;
     Demo_DV[0].SecondaryVariable:=20.0;
     end
  else
     begin
     Demo_DV[0].LoopCurrent:=4.0+1.0*(16.0/3.0);
     Demo_DV[1].LoopCurrent:=4.0+2.0*(16.0/3.0);
     Demo_DV[0].PrimaryUnit:=Demo_x576_In[0].PVUnits;
     S:=(Demo_x576_In[0].Point20mA+Demo_x576_In[0].Point4mA)/3.0;
     Demo_DV[0].PrimaryVariable:=Demo_x576_In[0].Point4mA+S;
     Demo_DV[1].PrimaryUnit:=Demo_x576_In[1].PVUnits;
     S:=(Demo_x576_In[1].Point20mA+Demo_x576_In[1].Point4mA)/3.0;
     Demo_DV[1].PrimaryVariable:=Demo_x576_In[1].Point4mA+2.0*S;
     Demo_DV[0].SecondaryUnit:=CHart_Celsius;
     Demo_DV[0].SecondaryVariable:=20.0;
     Demo_DV[1].SecondaryUnit:=CHart_Celsius;
     Demo_DV[1].SecondaryVariable:=20.0;
     end;
end;

procedure SetDefaultTransmitterParameters (Value: TDeviceType);
begin
  DemoDevice:=Value;

  case DemoDevice of
       dt507x: Demo_UID:=CDef_UID_x073;
       dtx073: Demo_UID:=CDef_UID_x073;
       dt5074XK: Demo_UID:=CDef_UID_x073XK;
       dt4073XK: Demo_UID:=CDef_UID_x073XK;
       dt4075: Demo_UID:=CDef_UID_x075;
       dtx575: Demo_UID:=CDef_UID_x575;
       dtx576: Demo_UID:=CDef_UID_x576;
       dtx573: Demo_UID:=CDef_UID_x573;
       dtx532: Demo_UID:=CDef_UID_x532;
       dtx582B: Demo_UID:=CDef_UID_x582B;
       dt1003: Demo_UID:=CDef_UID_1003;
       else Demo_UID:=CDef_UID_x073;
       end;

  Demo_UID.ManufacturersDeviceType:=CDeviceIdentifiers[DemoDevice];
  Demo_TDD     :=CDef_TDD;
  Demo_AssNum  :=0;

  if not(DemoDevice in [dtx532,dt1003]) then
     Demo_PVO:=CDef_PVO_x073
  else Demo_PVO:=CDef_PVO_x532;

  Demo_x073    :=CDef_x073_THC_Set;

  Demo_NE43    :=0;
  Demo_020mA   :=0;
  Demo_Input   :=0;
  Demo_Invert  :=0;
  Demo_HWConf  :=0;
  Demo_SCAlarm :=CHart_Alarm_Off;

  Demo_LeakOff :=CDef_x532_LeakOff;
  Demo_Adjust  :=CDef_AdjustOutput;
  Demo_Divider :=CDef_Divider;
  Demo_Trip_32 :=CDef_x532_Trip;
  Demo_TripA   :=CDef_1003_TripA;
  Demo_TripB   :=CDef_1003_TripB;

  Demo_x576_In[0]:=CDef_x576_InPar;
  Demo_x576_In[1]:=CDef_x576_InPar;
  Demo_x576_In[1].Channel:=1;
  Demo_x576_Out[0]:=CDef_x576_OutPar;
  Demo_x576_Out[1]:=CDef_x576_OutPar;
  Demo_x576_Out[1].Channel:=1;
  Demo_x576_Comm:=CDef_x576_Common;

  Demo_P4Cal :=4.0;
  Demo_P20Cal:=20.0;

  SeTHDynamicVariables;
end;

procedure SimulateTransmitterHartAnswer (HartCommand: THartCommand; var HartAnswer: THartAnswer);
const StatusSet: array [0..1] of char = ('T','F');
var I, N, Sel: byte;
    S, LC, PV, SV: single;
begin
  HartAnswer:=CDemoAnswer;
  HartAnswer.CommandNumber:=HartCommand.CommandNumber;

  case HartCommand.CommandNumber of
       0:   HartAnswer.Data:=chr(Demo_UID.DeviceTypeCode254)+
                             chr(Demo_UID.ManufacturerIdentCode)+
                             chr(Demo_UID.ManufacturersDeviceType)+
                             chr(Demo_UID.NumberOfPreambles)+
                             chr(Demo_UID.RevisionLevelOfUCD)+
                             chr(Demo_UID.RevisionLevelOfTSD)+
                             chr(Demo_UID.SoftwareRevision)+
                             chr(Demo_UID.HardwareRevision)+
                             chr(Demo_UID.Flags)+
                             chr(Demo_UID.DeviceIdentNumber[1])+
                             chr(Demo_UID.DeviceIdentNumber[2])+
                             chr(Demo_UID.DeviceIdentNumber[3]);
       1:   begin
            if not(RandomVariation.Enabled) then
               PV:=Demo_DV[0].PrimaryVariable
            else PV:=RandomRealValue(Demo_DV[0].PrimaryVariable,RandomVariation.Ratio);
            HartAnswer.Data:=chr(Demo_DV[0].PrimaryUnit)+
                             Single2Hart(PV);
            end;
       2:   begin
            if ((DemoDevice=dtx532) or (DemoDevice=dt1003)) and
               (Demo_020mA>0) then // 0-20mA = Full Scale
               S:=100.0*(Demo_DV[0].LoopCurrent)/20.0
            else S:=100.0*(Demo_DV[0].LoopCurrent-4.0)/16.0;
            if not(RandomVariation.Enabled) then
               begin
               LC:=Demo_DV[0].LoopCurrent;
               end
            else
               begin
               LC:=RandomRealValue(Demo_DV[0].LoopCurrent,RandomVariation.Ratio);
               S:=RandomRealValue(S,RandomVariation.Ratio);
               end;
            HartAnswer.Data:=Single2Hart(LC)+
                             Single2Hart(S);
            end;
       3:   begin
            if not(RandomVariation.Enabled) then
               begin
               LC:=Demo_DV[0].LoopCurrent;
               PV:=Demo_DV[0].PrimaryVariable;
               SV:=Demo_DV[0].SecondaryVariable;
               end
            else
               begin
               LC:=RandomRealValue(Demo_DV[0].LoopCurrent,RandomVariation.Ratio);
               PV:=RandomRealValue(Demo_DV[0].PrimaryVariable,RandomVariation.Ratio);
               SV:=RandomRealValue(Demo_DV[0].SecondaryVariable,0.2*RandomVariation.Ratio);
               end;
            HartAnswer.Data:=Single2Hart(LC)+
                             chr(Demo_DV[0].PrimaryUnit)+
                             Single2Hart(PV)+
                             chr(Demo_DV[0].SecondaryUnit)+
                             Single2Hart(SV);
            end;
       13:  HartAnswer.Data:=Ascii2Hart(Demo_TDD.TagName,8)+
                             Ascii2Hart(Demo_TDD.Descriptor,16)+
                             chr(Demo_TDD.Day)+
                             chr(Demo_TDD.Month)+
                             chr(Demo_TDD.Year);
       15:  HartAnswer.Data:=chr(Demo_PVO.PVAlarm)+
                             chr(Demo_PVO.TransferFunction)+
                             chr(Demo_PVO.PVUnits)+
                             Single2Hart(Demo_PVO.Point20mA)+
                             Single2Hart(Demo_PVO.Point4mA)+
                             Single2Hart(Demo_PVO.DampingTimeConstant)+
                             Single2Hart(0.0);
       16:  HartAnswer.Data:=Integer2Hart(Demo_AssNum,3);
       18:  if (System.Length(HartCommand.Data)=21) then
               begin
               Demo_TDD.TagName:=Hart2Ascii(System.Copy(HartCommand.Data,1,6));
               Demo_TDD.Descriptor := Hart2Ascii(System.Copy(HartCommand.Data,7,12));
               Demo_TDD.Day        := ord(HartCommand.Data[19]);
               Demo_TDD.Month      := ord(HartCommand.Data[20]);
               Demo_TDD.Year       := ord(HartCommand.Data[21]);
               HartAnswer.Data:=Ascii2Hart(Demo_TDD.TagName,8)+
                                Ascii2Hart(Demo_TDD.Descriptor,16)+
                                chr(Demo_TDD.Day)+
                                chr(Demo_TDD.Month)+
                                chr(Demo_TDD.Year);
               end;
       19:  if (System.Length(HartCommand.Data)=3) then
               begin
               Demo_AssNum:=Hart2Integer(HartCommand.Data);
               HartAnswer.Data:=Integer2Hart(Demo_AssNum,3);
               end;
       34:  if (System.Length(HartCommand.Data)=4) then
               begin
               Demo_PVO.DampingTimeConstant:=Hart2Single(System.Copy(HartCommand.Data,1,4));
               HartAnswer.Data:=Single2Hart(Demo_PVO.DampingTimeConstant);
               end;
       35:  if (System.Length(HartCommand.Data)=9) then
               begin
               Demo_PVO.PVUnits:=ord(HartCommand.Data[1]);
               Demo_PVO.Point20mA:=Hart2Single(System.Copy(HartCommand.Data,2,4));
               Demo_PVO.Point4mA:=Hart2Single(System.Copy(HartCommand.Data,6,4));
               HartAnswer.Data:=chr(Demo_PVO.PVUnits)+
                                Single2Hart(Demo_PVO.Point20mA)+
                                Single2Hart(Demo_PVO.Point4mA);
               SeTHDynamicVariables;
               end;
       38:  if (System.Length(HartCommand.Data)=0) then
               begin
               // reset configuration changed flag (bit 6 of the status byte)
               HartAnswer.Data:='';
               end;
       40:  if (System.Length(HartCommand.Data)=4) then
               begin
               S:=Hart2Single(System.Copy(HartCommand.Data,1,4));
               if not(S<0.001) then
                  Demo_DV[0].LoopCurrent:=S;
               HartAnswer.Data:=Single2Hart(S);
               end;
       42:  ; // Reset (cold start)
       44:  if (System.Length(HartCommand.Data)>0) then
               begin
               Demo_PVO.PVUnits:=ord(HartCommand.Data[1]);
               HartAnswer.Data:=HartCommand.Data;
               end;
       45:  if (System.Length(HartCommand.Data)=4) then
               begin
               Demo_P4Cal:=Hart2Single(HartCommand.Data);
               HartAnswer.Data:=Single2Hart(Demo_P4Cal);
               end;
       46:  if (System.Length(HartCommand.Data)=4) then
               begin
               Demo_P20Cal:=Hart2Single(HartCommand.Data);
               HartAnswer.Data:=Single2Hart(Demo_P20Cal);
               end;
       47:  if (System.Length(HartCommand.Data)>0) then
               begin
               Demo_PVO.TransferFunction:=ord(HartCommand.Data[1]);
               HartAnswer.Data:=HartCommand.Data;
               end;
       48:  begin
            case DemoDevice of
                 dtx573,
                 dtx532,
                 dt1003: N:=5;
                 else N:=3;
            end;
            HartAnswer.Data:='';
            for I:=1 to N do
                HartAnswer.Data:=HartAnswer.Data+StatusSet[System.Round(System.Random)];
            end;
       128: if (System.Length(HartCommand.Data)>0) then
               begin
               Sel:=ord(HartCommand.Data[1]);
               if (Sel=4) and (System.Length(HartCommand.Data)>1) then
                  HartAnswer.Data:=Integer2Hart(Demo_x073.SmoothingTimeConstant,2)
               else
                  begin
                  case Sel of
                       0:  HartAnswer.Data:=chr(Demo_x073.OpenCircuitAlarmLevel);
                       1:  HartAnswer.Data:=chr(Demo_x073.TransmitterFailureAlarmLevel);
                       2:  HartAnswer.Data:=chr(Demo_x073.FilterFrequencySelector);
                       3:  HartAnswer.Data:=chr(Demo_x073.InputType);
                       5:  HartAnswer.Data:=chr(Demo_x073.ColdJunctionCompensationOnOff);
                       6:  if (DemoDevice=dtx532) then
                              HartAnswer.Data:=chr(Demo_NE43)
                           else if (DemoDevice=dtx573) then
                                   HartAnswer.Data:=chr(Demo_SCAlarm)
                           else HartAnswer.CommStatus:=64;
                       33: if (DemoDevice=dtx532) then
                              HartAnswer.Data:=chr(Demo_020mA)
                           else HartAnswer.CommStatus:=64;
                       70: HartAnswer.Data:=chr(Demo_x073.ColdJunctionFailureAlarmLevel);
                       end;
                   end;
               end;
       129: if (System.Length(HartCommand.Data)>1) then
               begin
               Sel:=ord(HartCommand.Data[1]);
               if (Sel=4) and (System.Length(HartCommand.Data)>2) then
                  begin
                  Demo_x073.SmoothingTimeConstant:=Hart2Integer(System.Copy(HartCommand.Data,2,2));
                  HartAnswer.Data:='';
                  end
               else
                  begin
                  I:=ord(HartCommand.Data[2]);
                  case Sel of
                       0:  Demo_x073.OpenCircuitAlarmLevel:=I;
                       1:  Demo_x073.TransmitterFailureAlarmLevel:=I;
                       2:  Demo_x073.FilterFrequencySelector:=I;
                       3:  Demo_x073.InputType:=I;
                       5:  Demo_x073.ColdJunctionCompensationOnOff:=I;
                       6:  if (DemoDevice=dtx532) then
                              Demo_NE43:=I
                           else if (DemoDevice=dtx573) then
                                   Demo_SCAlarm:=I
                           else HartAnswer.CommStatus:=64;
                       33: if (DemoDevice=dtx532) then
                              Demo_020mA:=I
                           else HartAnswer.CommStatus:=64;
                       70: Demo_x073.ColdJunctionFailureAlarmLevel:=I;
                       else HartAnswer.CommStatus:=2;
                       end;
                  HartAnswer.Data:='';
                  end;
               end;

       130: begin // only E01003 and MTL4532 - digital output status
            N:=System.Length(HartCommand.Data);
            if (DemoDevice=dtx532) and (N=0) then
               HartAnswer.Data:=chr(0)
            else if (DemoDevice=dt1003) and (N=1) then
               HartAnswer.Data:=chr(0)
            else HartAnswer.CommStatus:=64;
            end;
       131: begin
            N:=System.Length(HartCommand.Data);
            if (DemoDevice=dtx532) and (N=2) then
               begin
               Demo_Input:=ord(HartCommand.Data[1]);
               Demo_Invert:=ord(HartCommand.Data[2]);
               HartAnswer.Data:='';
               end
            else if (DemoDevice=dt1003) and (N=1) then
               begin
               Demo_Input:=ord(HartCommand.Data[1]);
               HartAnswer.Data:='';
               end
            else HartAnswer.CommStatus:=64;
            end;
       132: case DemoDevice of  // only MTL4532 and E01003 - InputandLFDConfig or Input
                 dtx532: HartAnswer.Data:=chr(Demo_Input)+
                                          chr(Demo_Invert);
                 dt1003: HartAnswer.Data:=chr(Demo_Input);
                 else HartAnswer.CommStatus:=64;
                 end;
       133: begin
            N:=System.Length(HartCommand.Data);
            if (DemoDevice=dtx532) and (N=10) then
               begin
               Demo_LeakOff.FunctionSelect:=ord(HartCommand.Data[1]);
               Demo_LeakOff.LFUnits:=CHart_Percent;
               Demo_LeakOff.LeakFlowOffValue:=Hart2Single(System.Copy(HartCommand.Data,3,4));
               Demo_LeakOff.LeakFlowOffCurrent:=Hart2Single(System.Copy(HartCommand.Data,7,2));
               HartAnswer.Data:='';
               end
            else if (DemoDevice=dt1003) and (N=9) then
                 begin
                 Demo_LeakOff.FunctionSelect:=ord(HartCommand.Data[1]);
                 Demo_LeakOff.LeakFlowOffValue:=Hart2Single(System.Copy(HartCommand.Data,2,4));
                 Demo_LeakOff.LeakFlowOffCurrent:=Hart2Single(System.Copy(HartCommand.Data,6,2));
                 HartAnswer.Data:='';
                 end
            else HartAnswer.CommStatus:=64;
            end;
       134: if (DemoDevice=dtx532) then
               HartAnswer.Data:=chr(Demo_LeakOff.FunctionSelect)+
                                chr(CHart_Percent)+ // Demo_LeakOff.LFUnits
                                Single2Hart(Demo_LeakOff.LeakFlowOffValue)+
                                Single2Hart(Demo_LeakOff.LeakFlowOffCurrent)
            else if (DemoDevice=dt1003) then
               HartAnswer.Data:=chr(Demo_LeakOff.FunctionSelect)+
                                Single2Hart(Demo_LeakOff.LeakFlowOffValue)+
                                Single2Hart(Demo_LeakOff.LeakFlowOffCurrent)
            else HartAnswer.CommStatus:=64;
       135: if (System.Length(HartCommand.Data)=1) then
               begin
               case (ord(HartCommand.Data[1])) of
                    13: ;
                    15: ;
                    23: ;
                    25: ;
                    27: ;
                    33: ;
                    35: ;
                    37: ;
                    43: ;
                    45: ;
                    47: ;
                    else HartAnswer.CommStatus:=2;
                    end;
               HartAnswer.Data:='';
               end;
       136: begin
            N:=System.Length(HartCommand.Data);
            if (DemoDevice=dtx532) and ((N=1) or (N=10) or (N=19)) then
               begin
               Demo_Trip_32.FunctionSelect:=ord(HartCommand.Data[1]);
               if (N=10) or (N=19) then
                  begin
                  Demo_Trip_32.TPAUnits:=ord(HartCommand.Data[2]);
                  Demo_Trip_32.TripAValue:=Hart2Single(System.Copy(HartCommand.Data,3,4));
                  Demo_Trip_32.AHysteresis:=Hart2Single(System.Copy(HartCommand.Data,7,4));
                  end;
               if (N=19) then
                  begin
                  Demo_Trip_32.TPAUnits:=ord(HartCommand.Data[11]);
                  Demo_Trip_32.TripAValue:=Hart2Single(System.Copy(HartCommand.Data,12,4));
                  Demo_Trip_32.AHysteresis:=Hart2Single(System.Copy(HartCommand.Data,16,4));
                  end;
               HartAnswer.Data:='';
               end
            else if (DemoDevice=dt1003) and (N=10) then
                    begin
                    I:=ord(HartCommand.Data[1]);
                    if not((I and CP3_TripB_Pattern)>0) then
                       begin
                       Demo_TripA.FunctionSelect:=I;
                       Demo_TripA.TPUnits:=CHart_Herz;
                       Demo_TripA.TripValue:=Hart2Single(System.Copy(HartCommand.Data,3,4));
                       Demo_TripA.Hysteresis:=Hart2Single(System.Copy(HartCommand.Data,7,4));
                       end
                    else
                       begin
                       Demo_TripB.FunctionSelect:=I;
                       Demo_TripB.TPUnits:=CHart_Herz;
                       Demo_TripB.TripValue:=Hart2Single(System.Copy(HartCommand.Data,3,4));
                       Demo_TripB.Hysteresis:=Hart2Single(System.Copy(HartCommand.Data,7,4));
                       end;
                    HartAnswer.Data:='';
                    end
            else HartAnswer.CommStatus:=64;
            end;
       137: if (DemoDevice=dtx532) then
               begin
               HartAnswer.Data:=chr(Demo_Trip_32.FunctionSelect);
               if ((Demo_Trip_32.FunctionSelect and $01) > 0) or
                  ((Demo_Trip_32.FunctionSelect and $04) > 0) then
                  HartAnswer.Data:=HartAnswer.Data+
                                   chr(CHart_Percent)+ // Demo_Trip_32.TPAUnits
                                   Single2Hart(Demo_Trip_32.TripAValue)+
                                   Single2Hart(Demo_Trip_32.AHysteresis);
               if ((Demo_Trip_32.FunctionSelect and $10) > 0) or
                  ((Demo_Trip_32.FunctionSelect and $20) > 0) then
                  HartAnswer.Data:=HartAnswer.Data+
                                   chr(CHart_Percent)+ // Demo_Trip_32.TPBUnits
                                   Single2Hart(Demo_Trip_32.TripBValue)+
                                   Single2Hart(Demo_Trip_32.BHysteresis);
               end
            else if (DemoDevice=dt1003) then
                    begin
                    I:=ord(HartCommand.Data[1]);
                    if not((I and CP3_TripB_Pattern)>0) then
                       begin
                       HartAnswer.Data:=chr(Demo_TripA.FunctionSelect)+
                                        chr(CHart_Percent)+ // Demo_TripA.TPUnits
                                        Single2Hart(Demo_TripA.TripValue)+
                                        Single2Hart(Demo_TripA.Hysteresis);
                       end
                    else
                       begin
                       HartAnswer.Data:=chr(Demo_TripB.FunctionSelect)+
                                        chr(CHart_Percent)+ // Demo_TripB.TPUnits
                                        Single2Hart(Demo_TripB.TripValue)+
                                        Single2Hart(Demo_TripB.Hysteresis);
                       end;
                    end
            else HartAnswer.CommStatus:=64;
       138: if ((DemoDevice=dtx532) or (DemoDevice=dt1003)) and
               (System.Length(HartCommand.Data)=9) then
               begin
               Demo_Divider.FunctionSelect:=ord(HartCommand.Data[1]);
               Demo_Divider.Divider:=Hart2Single(System.Copy(HartCommand.Data,2,4));
               Demo_Divider.Duration:=Hart2Single(System.Copy(HartCommand.Data,6,4));
               end;
       139: if (DemoDevice=dtx532) or (DemoDevice=dt1003) then
               HartAnswer.Data:=chr(Demo_Divider.FunctionSelect)+
                                Single2Hart(Demo_Divider.Divider)+
                                Single2Hart(Demo_Divider.Duration)
            else HartAnswer.CommStatus:=64;
       140: if ((DemoDevice=dtx532) or (DemoDevice=dt1003)) and
               (System.Length(HartCommand.Data)=10) then
               begin
               Demo_Adjust.AdjMode:=ord(HartCommand.Data[1]);
               Demo_Adjust.AdjUnits:=CHart_Percent; // ord(HartCommand.Data[2])
               Demo_Adjust.OffsetValue:=Hart2Single(System.Copy(HartCommand.Data,3,4));
               Demo_Adjust.GainValue:=Hart2Single(System.Copy(HartCommand.Data,7,4));
               HartAnswer.Data:='';
               end
            else if (DemoDevice=dtx576) and
                    (System.Length(HartCommand.Data)=5) then
                    begin
                    S:=Hart2Single(System.Copy(HartCommand.Data,2,4));
                    if not(S<0.001) then
                       Demo_DV[0].LoopCurrent:=S;
                    HartAnswer.Data:=HartCommand.Data[1]+Single2Hart(S);
                    end
                 else HartAnswer.CommStatus:=64;
       141: if (DemoDevice=dtx532) or (DemoDevice=dt1003) then
               HartAnswer.Data:=chr(Demo_Adjust.AdjMode)+
                                chr(CHart_Percent)+ // Demo_Adjust.AdjUnits
                                Single2Hart(Demo_Adjust.OffsetValue)+
                                Single2Hart(Demo_Adjust.GainValue)
            else HartAnswer.CommStatus:=64;
       142: if (System.Length(HartCommand.Data)=4) then // only E01003 - Trimmer Values
               begin
               // TrimmerZeroValue:=Hart2Integer(System.Copy(HartCommand.Data,1,2));
               // TrimmerSpanValue:=Hart2Integer(System.Copy(HartCommand.Data,2,2));
               HartAnswer.Data:=Integer2Hart(0,2)+
                                Integer2Hart(0,2);
               end;
       143: if (DemoDevice=dt1003) then
               begin
       {HartAnswer.Data:=Integer2Hart(0,2)+  // only E01003 - Trimmer Values
                             Integer2Hart(0,2);}
          {end
       else
          begin
          if (Sel in [2,3]) then Data:=DemoData
          else
             begin
             Data.ZeroValue := 500 + random(1400);
             Data.SpanValue := 500 + random(1400);
             end;
          Result:=true;
          end;}
               end
            else HartAnswer.CommStatus:=64;
       144: if (DemoDevice=dt1003) and // only E01003 - Save Hardware configuration
               (System.Length(HartCommand.Data)=1) then
               Demo_HWConf:=ord(HartCommand.Data[1])
            else HartAnswer.CommStatus:=64;
       145: if (DemoDevice=dt1003) then
               HartAnswer.Data:=chr(Demo_HWConf)
            else if (DemoDevice=dtx576) and
                    (System.Length(HartCommand.Data)=6) then
                    begin
                    if (ord(HartCommand.Data[2])=4) then
                       begin
                       Demo_P4Cal:=Hart2Single(System.Copy(HartCommand.Data,2,4));
                       HartAnswer.Data:=HartCommand.Data[1]+
                                        HartCommand.Data[2]+
                                        Single2Hart(Demo_P4Cal);
                       end
                    else if (ord(HartCommand.Data[2])=20) then
                            begin
                            Demo_P20Cal:=Hart2Single(System.Copy(HartCommand.Data,2,4));
                            HartAnswer.Data:=HartCommand.Data[1]+
                                             HartCommand.Data[2]+
                                             Single2Hart(Demo_P20Cal);
                            end
                         else HartAnswer.CommStatus:=64;
                    end
                 else HartAnswer.CommStatus:=64;
       146: if ((DemoDevice=dtx532) or (DemoDevice=dt1003)) and
               (System.Length(HartCommand.Data)=1) then
               begin
               Demo_020mA:=ord(HartCommand.Data[1]);
               HartAnswer.Data:='';
               end
            else HartAnswer.CommStatus:=64;
       147: if ((DemoDevice=dtx532) or (DemoDevice=dt1003)) then
               HartAnswer.Data:=chr(Demo_020mA)
            else HartAnswer.CommStatus:=64;
       148: if (DemoDevice=dtx576) then
               begin
               HartAnswer.Data:=HartCommand.Data[1];
               for I:=1 to 5 do
                   HartAnswer.Data:=HartAnswer.Data+StatusSet[System.Round(System.Random)];
               end
            else HartAnswer.CommStatus:=64;
       150: if (HartCommand.Data='Herder') then // only E01003 - Trimmer Values
               begin
               // Restore default configuration
               // don't reply
               end;
       151: if (System.Length(HartCommand.Data)>0) then
               begin
               Sel:=ord(HartCommand.Data[1]);
               HartAnswer.Data:=chr(Sel)+
                                chr(Demo_x576_In[Sel].SensorType)+
                                chr(Demo_x576_In[Sel].PVUnits)+
                                chr(Demo_x576_In[Sel].Linearisation)+
                                chr(Demo_x576_In[Sel].CJC_Option)+
                                Single2Hart(Demo_x576_In[Sel].Point4mA)+
                                Single2Hart(Demo_x576_In[Sel].Point20mA)+
                                Single2Hart(Demo_x576_In[Sel].CJC_UserValue);
               end;
       152: if (System.Length(HartCommand.Data)=17) then
               begin
               Sel:=ord(HartCommand.Data[1]);
               if (Sel in [0,1]) then
                  begin
                  Demo_x576_In[Sel].Channel:=Sel;
                  Demo_x576_In[Sel].SensorType:=ord(HartCommand.Data[2]);
                  Demo_x576_In[Sel].PVUnits:=ord(HartCommand.Data[3]);
                  Demo_x576_In[Sel].Linearisation:=ord(HartCommand.Data[5]);
                  Demo_x576_In[Sel].CJC_Option:=ord(HartCommand.Data[5]);
                  Demo_x576_In[Sel].Point4mA:=Hart2Single(System.Copy(HartCommand.Data,6,4));
                  Demo_x576_In[Sel].Point20mA:=Hart2Single(System.Copy(HartCommand.Data,10,4));
                  Demo_x576_In[Sel].CJC_UserValue:=Hart2Single(System.Copy(HartCommand.Data,14,4));
                  HartAnswer.Data:=chr(Sel)+
                                   chr(Demo_x576_In[Sel].SensorType)+
                                   chr(Demo_x576_In[Sel].PVUnits)+
                                   chr(Demo_x576_In[Sel].Linearisation)+
                                   chr(Demo_x576_In[Sel].CJC_Option)+
                                   Single2Hart(Demo_x576_In[Sel].Point4mA)+
                                   Single2Hart(Demo_x576_In[Sel].Point20mA)+
                                   Single2Hart(Demo_x576_In[Sel].CJC_UserValue);
                  SeTHDynamicVariables;
                  end
               else HartAnswer.CommStatus:=64;
               end
            else HartAnswer.CommStatus:=64;
       155: if (System.Length(HartCommand.Data)=1) then // only E01003_new - reply internal values
               begin
               case ord(HartCommand.Data[1]) of
                    0: HartAnswer.Data:=chr(0)+chr(0)+chr(0)+chr(0)+chr(0)+chr(0);
                    1: HartAnswer.Data:=chr(0)+chr(0)+chr(0)+chr(0)+chr(0)+chr(0);
                    2: HartAnswer.Data:=chr(0)+chr(0)+chr(0)+chr(0)+chr(0)+chr(0);
                    3: HartAnswer.Data:=chr(0)+chr(0)+chr(0)+chr(0)+chr(0)+chr(0);
                    else HartAnswer.CommStatus:=2;
                    end;
               end;
          {end
       else
          begin
          for I:=0 to 5 do Data^[I]:=7+random(25);
          Result:=true;
          end;}
       161: if (System.Length(HartCommand.Data)>0) then
               begin
               Sel:=ord(HartCommand.Data[1]);
               HartAnswer.Data:=chr(Sel)+
                                chr(Demo_x576_Out[Sel].OpenCircuitAlarmLevel)+
                                chr(Demo_x576_Out[Sel].ShortCircuitAlarmLevel)+
                                chr(Demo_x576_Out[Sel].TransmitterFailureAlarmLevel)+
                                chr(Demo_x576_Out[Sel].ColdJunctionFailureAlarmLevel)+
                                chr(Demo_x576_Out[Sel].NAMUR_MTL)+
                                Single2Hart(Demo_x576_Out[Sel].DampingTimeConstant)+
                                Single2Hart(Demo_x576_Out[Sel].SmoothingTimeConstant)+
                                chr(Demo_x576_Out[Sel].EBD_Option)+
                                Single2Hart(Demo_x576_Out[Sel].TripLimit)+
                                Single2Hart(Demo_x576_Out[Sel].TripHysteresis);
               end;
       162: if (System.Length(HartCommand.Data)=23) then
               begin
               Sel:=ord(HartCommand.Data[1]);
               if (Sel in [0,1]) then
                  begin
                  Demo_x576_Out[Sel].Channel:=Sel;
                  Demo_x576_Out[Sel].OpenCircuitAlarmLevel:=ord(HartCommand.Data[2]);
                  Demo_x576_Out[Sel].ShortCircuitAlarmLevel:=ord(HartCommand.Data[3]);
                  Demo_x576_Out[Sel].TransmitterFailureAlarmLevel:=ord(HartCommand.Data[4]);
                  Demo_x576_Out[Sel].ColdJunctionFailureAlarmLevel:=ord(HartCommand.Data[5]);
                  Demo_x576_Out[Sel].NAMUR_MTL:=ord(HartCommand.Data[6]);
                  Demo_x576_Out[Sel].DampingTimeConstant:=Hart2Single(System.Copy(HartCommand.Data,7,4));
                  Demo_x576_Out[Sel].SmoothingTimeConstant:=Hart2Single(System.Copy(HartCommand.Data,11,4));
                  Demo_x576_Out[Sel].EBD_Option:=ord(HartCommand.Data[15]);
                  Demo_x576_Out[Sel].TripLimit:=Hart2Single(System.Copy(HartCommand.Data,16,4));
                  Demo_x576_Out[Sel].TripHysteresis:=Hart2Single(System.Copy(HartCommand.Data,20,4));
                  HartAnswer.Data:=chr(Sel)+
                                   chr(Demo_x576_Out[Sel].OpenCircuitAlarmLevel)+
                                   chr(Demo_x576_Out[Sel].ShortCircuitAlarmLevel)+
                                   chr(Demo_x576_Out[Sel].TransmitterFailureAlarmLevel)+
                                   chr(Demo_x576_Out[Sel].ColdJunctionFailureAlarmLevel)+
                                   chr(Demo_x576_Out[Sel].NAMUR_MTL)+
                                   Single2Hart(Demo_x576_Out[Sel].DampingTimeConstant)+
                                   Single2Hart(Demo_x576_Out[Sel].SmoothingTimeConstant)+
                                   chr(Demo_x576_Out[Sel].EBD_Option)+
                                   Single2Hart(Demo_x576_Out[Sel].TripLimit)+
                                   Single2Hart(Demo_x576_Out[Sel].TripHysteresis);
                  end
               else HartAnswer.CommStatus:=64;
               end
            else HartAnswer.CommStatus:=64;
       200: if (DemoDevice<>dtx582B) then
               begin
               if not(RandomVariation.Enabled) then
                  HartAnswer.Data:=Single2Hart(Demo_PV.ScaledInput)+
                                   Single2Hart(Demo_PV.SmoothedInput)+
                                   Single2Hart(Demo_PV.ColdJunctionResistance)+
                                   Single2Hart(Demo_PV.ColdJunctionTemperature)+
                                   Single2Hart(Demo_PV.ColdJunctionCompensation)+
                                   Single2Hart(Demo_PV.ProcessVariable)+
                                   chr(Demo_SV.SensorFlags)
               else HartAnswer.Data:=Single2Hart(RandomRealValue(Demo_PV.ScaledInput,RandomVariation.Ratio))+
                                     Single2Hart(RandomRealValue(Demo_PV.SmoothedInput,RandomVariation.Ratio))+
                                     Single2Hart(RandomRealValue(Demo_PV.ColdJunctionResistance,RandomVariation.Ratio))+
                                     Single2Hart(RandomRealValue(Demo_PV.ColdJunctionTemperature,RandomVariation.Ratio))+
                                     Single2Hart(RandomRealValue(Demo_PV.ColdJunctionCompensation,RandomVariation.Ratio))+
                                     Single2Hart(RandomRealValue(Demo_PV.ProcessVariable,RandomVariation.Ratio))+
                                     chr(Demo_SV.SensorFlags);
               end
            else
               begin
               if not(RandomVariation.Enabled) then
                  HartAnswer.Data:=Single2Hart(Demo_PV.ScaledInput)
               else HartAnswer.Data:=Single2Hart(RandomRealValue(Demo_PV.ScaledInput,RandomVariation.Ratio));
               end;
       201: if not(RandomVariation.Enabled) then
               HartAnswer.Data:=Single2Hart(Demo_SV.ProcessVariableScaledInput)+
                                Single2Hart(Demo_SV.DampedProcessVariable)+
                                Single2Hart(Demo_SV.LoopCurrent)+
                                Single2Hart(Demo_SV.NoiseFloor)+
                                // ignore not significant 8 data bytes from 17 to 24
                                #17#18#19#20#21#22#23#24+
                                chr(Demo_SV.SensorFlags)
            else HartAnswer.Data:=Single2Hart(RandomRealValue(Demo_SV.ProcessVariableScaledInput,RandomVariation.Ratio))+
                                  Single2Hart(RandomRealValue(Demo_SV.DampedProcessVariable,RandomVariation.Ratio))+
                                  Single2Hart(RandomRealValue(Demo_SV.LoopCurrent,RandomVariation.Ratio))+
                                  Single2Hart(RandomRealValue(Demo_SV.NoiseFloor,RandomVariation.Ratio))+
                                  // ignore not significant 8 data bytes from 17 to 24
                                  #17#18#19#20#21#22#23#24+
                                  chr(Demo_SV.SensorFlags);

       202: if not(RandomVariation.Enabled) then
               HartAnswer.Data:=Integer2Hart(Demo_RAW.DACValue,2)+
                                Integer2Hart(Demo_RAW.MainInput,2)+
                                Integer2Hart(Demo_RAW.AuxInput,2)+
                                Integer2Hart(Demo_RAW.MainExcitation,2)+
                                Integer2Hart(Demo_RAW.AuxExcitation,2)+
                                Integer2Hart(Demo_RAW.LeadOffTest1,2)+
                                Integer2Hart(Demo_RAW.LeadOffTest2,2)+
                                Integer2Hart(Demo_RAW.LeadOffTest5,2)+
                                Integer2Hart(Demo_RAW.LeadOffTest6,2)+
                                // ignore not significant 6 data bytes from 19 to 24
                                #19#20#21#22#23#24+
                                chr(Demo_RAW.SensorFlags)
            else HartAnswer.Data:=Integer2Hart(RandomIntegerValue(Demo_RAW.DACValue,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.MainInput,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.AuxInput,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.MainExcitation,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.AuxExcitation,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.LeadOffTest1,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.LeadOffTest2,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.LeadOffTest5,RandomVariation.Ratio),2)+
                             Integer2Hart(RandomIntegerValue(Demo_RAW.LeadOffTest6,RandomVariation.Ratio),2)+
                             // ignore not significant 6 data bytes from 19 to 24
                             #19#20#21#22#23#24+
                             chr(Demo_RAW.SensorFlags);
       203: if (DemoDevice<>dtx582B) then
               begin
               HartAnswer.Data:=Single2Hart(Demo_Cal.RrefCal)+
                                Single2Hart(Demo_Cal.Rin5Cal)+
                                Single2Hart(Demo_Cal.Rin6Cal)+
                                Single2Hart(Demo_Cal.VrefCal)+
                                // ignore not significant 7 data bytes from 18 to 24
                                #18#19#20#21#22#23#24+
                                chr(Demo_Cal.SensorFlags);
               end
            else // DemoDevice=dtx582B
               begin
               case HartCommand.Data[1] of
                    #0: HartAnswer.Data:=HartCommand.Data+
                                         Single2Hart(14000.0);
                    #1: HartAnswer.Data:=HartCommand.Data+
                                         Single2Hart(160.0);
                    #2: HartAnswer.Data:=HartCommand.Data+
                                         Single2Hart(160.0);
                    end;
               end;
       204: if (DemoDevice<>dtx582B) then
               begin
               if (System.Length(HartCommand.Data)=9) then // MTL4075 only
                  begin
                  Demo_x075TPA.TripValue:=Hart2Single(System.Copy(HartCommand.Data,1,4));
                  Demo_x075TPA.Hysteresis:=Hart2Single(System.Copy(HartCommand.Data,5,4));
                  Demo_x075TPA.TripOutputFunction:=ord(HartCommand.Data[9]);
                  HartAnswer.Data:=Single2Hart(Demo_x075TPA.TripValue)+
                                   Single2Hart(Demo_x075TPA.Hysteresis)+
                                   chr(Demo_x075TPA.TripOutputFunction);
                  end;
               end
            else
               begin
               HartAnswer.Data:=HartCommand.Data;
               end;
       205: // MTL4075 only
            HartAnswer.Data:=Single2Hart(Demo_x075TPA.TripValue)+
                             Single2Hart(Demo_x075TPA.Hysteresis)+
                             chr(Demo_x075TPA.TripOutputFunction);
       207: begin
            N:=System.Length(HartCommand.Data);
            case DemoDevice of
                 dtx575,
                 dtx573: if (N=1) then
                            begin
                            Demo_Speed:=ord(HartCommand.Data[1]);
                            HartAnswer.Data:=HartCommand.Data;
                            end
                         else HartAnswer.CommStatus:=64;
                 dtx576: if (N=3) then
                            begin
                            Demo_x576_Comm.response_time:=ord(HartCommand.Data[1]);
                            Demo_x576_Comm.FilterFrequencySelector:=ord(HartCommand.Data[2]);
                            Demo_x576_Comm.InputOption:=ord(HartCommand.Data[3]);
                            HartAnswer.Data:=chr(Demo_x576_Comm.response_time)+
                                             chr(Demo_x576_Comm.FilterFrequencySelector)+
                                             chr(Demo_x576_Comm.InputOption);
                            end
                         else HartAnswer.CommStatus:=64;
                 else HartAnswer.CommStatus:=64;
                 end;
             end;
       208:  case DemoDevice of
                 dtx575,
                 dtx573: HartAnswer.Data:=chr(Demo_Speed);
                 dtx576: HartAnswer.Data:=chr(Demo_x576_Comm.response_time)+
                                          chr(Demo_x576_Comm.FilterFrequencySelector)+
                                          chr(Demo_x576_Comm.InputOption);
                 else HartAnswer.CommStatus:=64;
                 end;
       209: case DemoDevice of
                 dtx575,
                 dtx573: if (System.Length(HartCommand.Data)=1) then
                            begin
                            Demo_NE43:=ord(HartCommand.Data[1]);
                            HartAnswer.Data:=HartCommand.Data;
                            end;
                 dtx576: begin
                         HartAnswer.Data:=chr(0)+  // always zero
                                          chr($09)+
                                          Single2Hart(-200.0)+
                                          Single2Hart(850.0)+
                                          Single2Hart(850.0)+
                                          chr(0);
                         end;
                 end;
       210: case DemoDevice of
                 dtx575: HartAnswer.Data:=chr(Demo_NE43); // reply NAMUR option for MTL4575
                 dtx576: begin
                         HartAnswer.Data:=HartCommand.Data;
                         end;
                 end;

       213: if (DemoDevice=dtx576) and
               (System.Length(HartCommand.Data)=1) then
               begin
               Sel:=ord(HartCommand.Data[1]);
               if (Sel in [0,1]) then
                  begin
                  if not(RandomVariation.Enabled) then
                     begin
                     LC:=Demo_DV[Sel].LoopCurrent;
                     PV:=Demo_DV[Sel].PrimaryVariable;
                     SV:=Demo_DV[Sel].SecondaryVariable;
                     end
                  else
                     begin
                     LC:=RandomRealValue(Demo_DV[Sel].LoopCurrent,RandomVariation.Ratio);
                     PV:=RandomRealValue(Demo_DV[Sel].PrimaryVariable,RandomVariation.Ratio);
                     SV:=RandomRealValue(Demo_DV[Sel].SecondaryVariable,0.2*RandomVariation.Ratio);
                     end;
                  HartAnswer.Data:=HartCommand.Data[1]+
                                   Single2Hart(LC)+
                                   chr(Demo_DV[Sel].PrimaryUnit)+
                                   Single2Hart(PV)+
                                   chr(Demo_DV[Sel].SecondaryUnit)+
                                   Single2Hart(SV)+
                                   chr(0)+
                                   Single2Hart(0.0)+
                                   chr(0)+
                                   Single2Hart(0.0);
                     end
                  else HartAnswer.CommStatus:=64;
               end
               else HartAnswer.CommStatus:=64;
       220: if (System.Length(HartCommand.Data)=1) and
               (HartCommand.Data[1]=#85) then
               begin
               // restore autotest defaults
               end;
       221: if (System.Length(HartCommand.Data)=4) then
               begin
               //R7:=Hart2Single(System.Copy(HartCommand.Data,1,4));
               HartAnswer.Data:='';
               end;
       222: if (System.Length(HartCommand.Data)=4) then
               begin
               //R2FIL2:=Hart2Single(System.Copy(HartCommand.Data,1,4));
               HartAnswer.Data:='';
               end;
       223: if (System.Length(HartCommand.Data)=4) then
               begin
               //R3FIL3:=Hart2Single(System.Copy(HartCommand.Data,1,4));
               HartAnswer.Data:='';
               end;
       224: if (System.Length(HartCommand.Data)=4) then
               begin
               //IC2mV:=Hart2Single(System.Copy(HartCommand.Data,1,4));
               HartAnswer.Data:='';
               end;
       225: if (System.Length(HartCommand.Data)=1) and
               (HartCommand.Data[1]=#255) then
               begin
               // write calibration flag
               end;
       226: if (System.Length(HartCommand.Data)=4) then
               begin
               //CJC:=Hart2Single(System.Copy(HartCommand.Data,1,4));
               HartAnswer.Data:='';
               end;
       227: if (DemoDevice=dtx582B) then
               begin
               HartAnswer.Data:=chr(CHart_Ohm)+
                                Single2Hart(400.0)+
                                Single2Hart(10.0);
               end;
       else
          HartAnswer.CommStatus:=64;
       end;
end;

initialization
  SetDefaultTransmitterParameters(dtx073);
  with RandomVariation do
       begin
       Enabled:=true;
       Ratio:=0.025;
       end;
end.
