//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**
//**  ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
//**  HARDCOMMANDS.PAS
//**
//**  HART command set with Read and Write procedures for PCS45
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.11.96 / ... / 09.07.02 / ... / 03.10.08 / ... / 10.11.09 / 08.01/10 / 13.04.14 / 10.08.15
//**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit HartCommands;

interface

uses Types, HartDevices, HartDataTypes, DataTypes;

  function ExamineAvailableCommandSet(var HartComm: THartMaster; First,Last: byte): TByteSet;

// General purpose read and write functions, not assigned to specific command numbers or device types

  function WriteRead_EmptyCommand (var HartComm: THartMaster; CommandNumber: byte; SuppressErrorMessages: boolean): boolean;

  function Read_One_Byte (var HartComm: THartMaster; CommandNumber: byte; var Target: byte; SuppressErrorMessages: boolean): boolean;
  function Write_One_Byte (var HartComm: THartMaster; CommandNumber: byte; Source: byte; SuppressErrorMessages: boolean): boolean;
  function WriteRead_One_Byte (var HartComm: THartMaster; CommandNumber: byte; var SourceAndTarget: byte; SuppressErrorMessages: boolean): boolean;

  function Read_One_Word (var HartComm: THartMaster; CommandNumber: byte; var Target: word; SuppressErrorMessages: boolean): boolean;
  function Write_One_Word (var HartComm: THartMaster; CommandNumber: byte; Source: word; SuppressErrorMessages: boolean): boolean;

  function Read_Selected_Byte (var HartComm: THartMaster; CommandNumber, Selector: byte; var Target: byte; SuppressErrorMessages: boolean): boolean;
  function Write_Selected_Byte (var HartComm: THartMaster; CommandNumber, Selector: byte; Source: byte; SuppressErrorMessages: boolean): boolean;
  function WriteRead_One_Selected_Byte (var HartComm: THartMaster; CommandNumber, Selector: byte; var SourceAndTarget: byte; SuppressErrorMessages: boolean): boolean;

  function Read_Selected_Word (var HartComm: THartMaster; CommandNumber, Selector: byte; var Target: word; SuppressErrorMessages: boolean): boolean;
  function Write_Selected_Word (var HartComm: THartMaster; CommandNumber, Selector: byte; Source: word; SuppressErrorMessages: boolean): boolean;

  function Read_One_Single (var HartComm: THartMaster; CommandNumber: byte; var Target: single; SuppressErrorMessages: boolean): boolean;
  function Write_One_Single (var HartComm: THartMaster; CommandNumber: byte; Source: single; SuppressErrorMessages: boolean): boolean;
  function WriteRead_One_Single (var HartComm: THartMaster; CommandNumber: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;

  function Read_Selected_Single (var HartComm: THartMaster; CommandNumber: byte; var Selector: byte; var Target: single; SuppressErrorMessages: boolean): boolean;
  function WriteRead_Selected_Single(var HartComm: THartMaster; CommandNumber: byte; var Selector: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;

  function Read_Two_Singles (var HartComm: THartMaster; CommandNumber: byte; var Target1, Target2: single; SuppressErrorMessages: boolean): boolean;
  function Read_Data_String (var HartComm: THartMaster; CommandNumber: byte; Request: string; var Target: string; SuppressErrorMessages: boolean): boolean;
  function Write_Data_String (var HartComm: THartMaster; CommandNumber: byte; Source: string; SuppressErrorMessages: boolean): boolean;

  function Write_Byte_And_Single (var HartComm: THartMaster; CommandNumber: byte; Source1: byte; Source2: single; SuppressErrorMessages: boolean): boolean;

  // Read and write functions assigned to universal command numbers, valid for all device types

  function Read_Identifier (var HartComm: THartMaster; var Target: THDeviceIdentifier; SuppressErrorMessages: boolean): boolean;
  function Read_PrimaryVariable(var HartComm: THartMaster; var Target: THPrimaryVariable; var SSS: string; SuppressErrorMessages: boolean): boolean;
  function Read_DynamicVariables (var HartComm: THartMaster; var Target: THDynamicVariables; BreakTime: word; SuppressErrorMessages: boolean): boolean;

  function Read_TagDescriptorDate (var HartComm: THartMaster; var Target: THTagDescriptorDate; BreakTime: word; SuppressErrorMessages: boolean): boolean;
  function WriteRead_TagDescriptorDate_Block (var HartComm: THartMaster; var SourceAndTarget: THTagDescriptorDate; SuppressErrorMessages: boolean): boolean;

  function Read_PrimaryVariableOutput (var HartComm: THartMaster; var Target: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;

  function Read_AssemblyNumber (var HartComm: THartMaster; var Target: THFinalAssemblyNumber; SuppressErrorMessages: boolean): boolean;
  function WriteRead_AssemblyNumber (var HartComm: THartMaster; var SourceAndTarget: THFinalAssemblyNumber; SuppressErrorMessages: boolean): boolean;

// Read and write functions assigned to common practice command numbers, valid mostly for all MTL device types (with possible variations)

  function WriteRead_Damping_Value (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;
  function WriteRead_Primary_Variables_Block (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;

  function WriteRead_Reset_Configuration_Changed (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;

  function WriteRead_Current_Value (var HartComm: THartMaster; CommandNumber: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;

  function Write_Master_Reset (var HartComm: THartMaster; ExpectAnswer: boolean; SuppressErrorMessages: boolean): boolean;
  function WriteRead_Process_Units (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;
  function WriteRead_TransferFunction_Information (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;

  function Read_TransmitterStatus (var HartComm: THartMaster; var Target: THTransmitterStatus; SuppressErrorMessages: boolean): boolean;

// Read and write functions assigned to transmitter specific command numbers, valid only for all MTL device types

// Read and write functions assigned to all MTL temperature converters MTLxy7z

  function Read_PrimaryVariables(var HartComm: THartMaster; var Target: THPrimaryVariables; SuppressErrorMessages: boolean): boolean;
  function Read_SecondaryVariables(var HartComm: THartMaster; var Target: THSecondaryVariables; SuppressErrorMessages: boolean): boolean;
  function Read_RAWInputOutputValues(var HartComm: THartMaster; var Target: THRAWInputOutputValues; SuppressErrorMessages: boolean): boolean;
  function Read_CalibrationData(var HartComm: THartMaster; var Target: THCalibrationData; SuppressErrorMessages: boolean): boolean;

  function Write_Restore_AllAutotestDefaults (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;

  function Write_Rref_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
  function Write_Rin5_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
  function Write_Rin6_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
  function Write_Vref_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
  function Write_CalibrationFlag (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;
  function Write_CJC_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;

 // Read and write functions assigned to the MTL4075 (and posibly to the successors)

  function Read_TripParameters_x075 (var HartComm: THartMaster; var Target: THx075TripParameters; SuppressErrorMessages: boolean): boolean;
  function WriteRead_TripParameters_x075 (var HartComm: THartMaster; var SourceAndTarget: THx075TripParameters; SuppressErrorMessages: boolean): boolean;

 // Read and write functions assigned to the MTL4576 (and posibly to the successors)

  function Read_Channel_DynamicVariables (var HartComm: THartMaster; ChannelNumber: byte; var Target: THDynamicVariables; BreakTime: word; SuppressErrorMessages: boolean): boolean;

  function WriteRead_Channel_Current_Value (var HartComm: THartMaster; ChannelNumber: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
  function WriteRead_Channel_TrimmDAC (var HartComm: THartMaster; ChannelNumber, TrimmMode: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;

  function Read_Channel_TransmitterStatus (var HartComm: THartMaster; ChannelNumber: byte; var Target: THTransmitterStatus; SuppressErrorMessages: boolean): boolean;

  function Read_InputParameters_x576 (var HartComm: THartMaster; Channel: byte; var Target: THInputParameters_x576; SuppressErrorMessages: boolean): boolean;
  function WriteRead_InputParameters_x576 (var HartComm: THartMaster; Channel: byte; var SourceAndTarget: THInputParameters_x576; SuppressErrorMessages: boolean): boolean;

  function Read_OutputParameters_x576 (var HartComm: THartMaster; Channel: byte; var Target: THOutputParameters_x576; SuppressErrorMessages: boolean): boolean;
  function WriteRead_OutputParameters_x576 (var HartComm: THartMaster; Channel: byte; var SourceAndTarget: THOutputParameters_x576; SuppressErrorMessages: boolean): boolean;

  function Read_CommonParameters_x576 (var HartComm: THartMaster; var Target: THCommonParameters_x576; SuppressErrorMessages: boolean): boolean;
  function WriteRead_CommonParameters_x576 (var HartComm: THartMaster; var SourceAndTarget: THCommonParameters_x576; SuppressErrorMessages: boolean): boolean;

  function Read_UserSensorHeader_x576 (var HartComm: THartMaster; var Target: THUserSensorHeader_x576; SuppressErrorMessages: boolean): boolean;

  function Read_UserSensorTable_x576 (var HartComm: THartMaster; var Target: THUserSensorTable; SuppressErrorMessages: boolean): boolean;
  function WriteVerify_UserSensorData_x576 (var HartComm: THartMaster; Source: THUserSensorData_x576; SuppressErrorMessages: boolean): boolean;

// Read and write functions assigned to all MTL frequency converters

  function Write_Digital_Diagnostics (var HartComm: THartMaster; Selector: byte; SuppressErrorMessages: boolean): boolean;

  function Write_DividerParameters (var HartComm: THartMaster; Source: THDividerParameters; SuppressErrorMessages: boolean): boolean;
  function Read_DividerParameters (var HartComm: THartMaster; var Target: THDividerParameters; SuppressErrorMessages: boolean): boolean;

  function Write_DividerParameters_1003 (var HartComm: THartMaster; Source: THDividerParameters; SuppressErrorMessages: boolean): boolean;
  function Read_DividerParameters_1003 (var HartComm: THartMaster; var Target: THDividerParameters; SuppressErrorMessages: boolean): boolean;

  function Write_AdjustOutputParameters (var HartComm: THartMaster; Source: THAdjustOutputParameters; SuppressErrorMessages: boolean): boolean;
  function Read_AdjustOutputParameters (var HartComm: THartMaster; var Target: THAdjustOutputParameters; SuppressErrorMessages: boolean): boolean;

  function Write_SetDeviceToDefault (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;

// Read and write functions assigned to the eurocard E01003 (and possibly to the MTL4532)

  function Write_InputSourceSelection_1003 (var HartComm: THartMaster; Source: THInputAndLFDParameters_1003; SuppressErrorMessages: boolean): boolean;
  function Read_InputSourceSelection_1003 (var HartComm: THartMaster; var Target: THInputAndLFDParameters_1003; SuppressErrorMessages: boolean): boolean;

  function Read_LeakFlowOffParameters_1003 (var HartComm:THartMaster; var Target:THLeakFlowOffParameters_1003; SuppressErrorMessages: boolean): boolean;
  function Write_LeakFlowOffParameters_1003 (var HartComm: THartMaster; Source: THLeakFlowOffParameters_1003; SuppressErrorMessages: boolean): boolean;

  function Write_TripParameters_1003 (var HartComm: THartMaster; Source: THTripParameters_1003; SuppressErrorMessages: boolean): boolean;
  function Read_TripParameters_1003 (var HartComm: THartMaster;  var Target: THTripParameters_1003; Selector: byte; SuppressErrorMessages: boolean): boolean;

  function Write_TrimmerVariables (var HartComm: THartMaster; Source: THTrimmerVariables_1003; Selector: byte; SuppressErrorMessages: boolean): boolean;
  function Read_TrimmerVariables (var HartComm: THartMaster; var Target: THTrimmerVariables_1003; Selector: byte; SuppressErrorMessages: boolean): boolean;

  function Read_Internal_Value (var HartComm: THartMaster; var Target: TByteDynArray; Selector: byte; SuppressErrorMessages: boolean): boolean;

  // Read and write functions assigned to the MTL4532

  function Read_LeakFlowOffParameters_x532 (var HartComm:THartMaster; var Target: THLeakFlowOffParameters_x532; SuppressErrorMessages: boolean): boolean;
  function Write_LeakFlowOffParameters_x532 (var HartComm: THartMaster; Source: THLeakFlowOffParameters_x532; SuppressErrorMessages: boolean): boolean;

  function Write_TripParameters_x532 (var HartComm: THartMaster; Source: THTripParameters_x532; SuppressErrorMessages: boolean): boolean;
  function Read_TripParameters_x532 (var HartComm: THartMaster; var Target: THTripParameters_x532; SuppressErrorMessages: boolean): boolean;

  // Read and write functions assigned to the MTL4582B

  function Read_ScaledIPT_x582B (var HartComm: THartMaster; var Target: single; SuppressErrorMessages: boolean): boolean;
  function Read_CalibrationParameters_x582B (var HartComm: THartMaster; Selector: byte; var Target: single; SuppressErrorMessages: boolean): boolean;
  function WriteRead_CalibrationParameters_x582B (var HartComm: THartMaster; Selector: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
  function Write_HardwareRevision_x582B (var HartComm: THartMaster; HardwareRevision: byte; SuppressErrorMessages: boolean): boolean;
  function Read_CalibrationFlag_x582B (var HartComm: THartMaster; var Target: byte; SuppressErrorMessages: boolean): boolean;
  function Read_PrimaryVariableOutput_x582B (var HartComm: THartMaster; var Target: THPrimaryVariableRange; SuppressErrorMessages: boolean): boolean;

var
  HartAnswer: THartAnswer;

implementation

uses SysUtils, Classes, Dialogs, DataInstances, HartDataConstants, LabelNumbers, HelpContextNumbers,
     ConversionProcs;

const
  CShortWaitTime   =  200;
  CDefaultWaitTime =  500;
  CLongWaitTime    = 2500;

function ExamineAvailableCommandSet(var HartComm: THartMaster; First,Last: byte): TByteSet;
var NewCommandSet: TByteSet;
    I: byte;
const CHardwareResetCommand: byte = 42;
begin
  NewCommandSet:=[];
  for I:=First to Last do
      begin
      if (I<>CHardwareResetCommand) then   // Don't examine Hardware Reset command yet
         begin
         HartAnswer:=HartComm.MasterDialog(I,'',CDefaultWaitTime);
         if ((HartAnswer.CommandNumber=I) and
            (HartAnswer.CommStatus<>64)) or
            (HartAnswer.Data='TimeOut') then
            NewCommandSet:=NewCommandSet+[I];
         end;
      end;
  if not((First>CHardwareResetCommand) or (Last<CHardwareResetCommand)) then
     begin
     HartAnswer:=HartComm.MasterDialog(CHardwareResetCommand,'',CDefaultWaitTime);
     if (HartAnswer.CommStatus<>64) then
        NewCommandSet:=NewCommandSet+[CHardwareResetCommand];
     end;
  
  Result:=NewCommandSet;
end;

procedure DisplayCommErrors(Errors: THartCommErrors; Number,CommStatus,DevStatus: byte);
var CommandInfo,HartStatusInfo,ConfigStatusInfo,DeviceStatusInfo: string[48];
    ErrorMessage: string;
    HelpInfo: THelpContext;
begin
  CommandInfo:=LanguageStrings[msCommandNumber]+IntToStr(Number);
  HartStatusInfo:=LanguageStrings[msStatusInfo]+'$'+IntToHex(CommStatus,2);
  ConfigStatusInfo:=LanguageStrings[msStatusInfo]+IntToStr(CommStatus);
  DeviceStatusInfo:=LanguageStrings[msStatusInfo]+'$'+IntToHex(DevStatus,2);

  HelpInfo:=hcInterfaceError;

  if (eeTimeOut in Errors.ExecutionErrors) then ErrorMessage:=LanguageStrings[msTimeOutError]+^M+CommandInfo
  else if  (teHartFormat in Errors.TransferErrors) then
           ErrorMessage:=LanguageStrings[msCommunicationDataError]+^M+CommandInfo+^M+HartStatusInfo
       else if (teParity in Errors.TransferErrors) or (teHartFormat in Errors.TransferErrors) or (teDifferentCommand in Errors.TransferErrors) then
               ErrorMessage:=LanguageStrings[msCommunicationDataError]+^M+CommandInfo
            else if (deWrongData in Errors.DeviceErrors) then
                    begin
                    HelpInfo:=hcConfigError;
                    ErrorMessage:=LanguageStrings[msConfigDataError]+^M+CommandInfo+^M+ConfigStatusInfo;
                    end
                 else
                    begin
                    if (deDeviceException in Errors.DeviceErrors) then
                       begin
                       ErrorMessage:=LanguageStrings[msUnitException]+^M+DeviceStatusInfo;
                       HelpInfo:=hcDeviceProblem;
                       end
                    else if (deDeviceProblems in Errors.DeviceErrors) then
                            begin
                            ErrorMessage:=LanguageStrings[msUnitCorrupted]+^M+DeviceStatusInfo;
                            HelpInfo:=hcDeviceProblem;
                            end
                         else if (deDeviceFailure in Errors.DeviceErrors) then
                                 begin
                                 ErrorMessage:=LanguageStrings[msUnitFailure];
                                 HelpInfo:=hcDeviceError;
                                 end
                              else
                                 begin
                                 ErrorMessage:=LanguageStrings[msUnexpectedCommError];
                                 HelpInfo:=hcNoHelpTopic;
                                 end;
                    end;
  MessageDlg(ErrorMessage, mtError,[mbCancel,mbHelp],HelpInfo);
end;

//*************************************************************************************************
//**
//**  General purpose read and write functions (for simple data items),
//**  not assigned to certain command numbers or device types
//**
//*************************************************************************************************

function WriteRead_EmptyCommand (var HartComm: THartMaster; CommandNumber: byte; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,'',CDefaultWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_One_Byte (var HartComm: THartMaster; CommandNumber: byte; var Target: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=1) then
          begin
          Target := ord(HartAnswer.Data[1]);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Write_One_Byte (var HartComm: THartMaster; CommandNumber: byte; Source: byte; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Source),CDefaultWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function WriteRead_One_Byte (var HartComm: THartMaster; CommandNumber: byte; var SourceAndTarget: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(SourceAndTarget),CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=1) then
          begin
          SourceAndTarget := ord(HartAnswer.Data[1]);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_One_Word (var HartComm: THartMaster; CommandNumber: byte; var Target: word; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=2) then
          begin
          Target := Hart2Integer(HartAnswer.Data);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Write_One_Word (var HartComm: THartMaster; CommandNumber: byte; Source: word; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,Integer2Hart(Source,2),CDefaultWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_Selected_Byte (var HartComm: THartMaster; CommandNumber, Selector: byte; var Target: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Selector)+#0,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=1) then
          begin
          Target := ord(HartAnswer.Data[1]);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Write_Selected_Byte (var HartComm: THartMaster; CommandNumber, Selector: byte; Source: byte; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Selector)+chr(Source),CDefaultWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function WriteRead_One_Selected_Byte (var HartComm: THartMaster; CommandNumber, Selector: byte; var SourceAndTarget: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Selector)+chr(SourceAndTarget),CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=1) then // 2
          begin
          SourceAndTarget := ord(HartAnswer.Data[1]);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_Selected_Word (var HartComm: THartMaster; CommandNumber, Selector: byte; var Target: word; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Selector)+#0,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=2) then
          begin
          Target := Hart2Integer(HartAnswer.Data);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Write_Selected_Word (var HartComm: THartMaster; CommandNumber, Selector: byte; Source: word; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Selector)+Integer2Hart(Source,2),CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_One_Single (var HartComm: THartMaster; CommandNumber: byte; var Target: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=4) then
          begin
          Target := Hart2Single(HartAnswer.Data);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Write_One_Single (var HartComm: THartMaster; CommandNumber: byte; Source: single; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := Single2Hart(Source);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,S,CShortWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function WriteRead_One_Single (var HartComm: THartMaster; CommandNumber: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := Single2Hart(SourceAndTarget);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=4) then
          begin
          SourceAndTarget := Hart2Single(HartAnswer.Data);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_Selected_Single (var HartComm: THartMaster; CommandNumber: byte; var Selector: byte; var Target: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Selector),CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=5) then
          begin
          Selector := ord(HartAnswer.Data[1]);
          Target := Hart2Single(System.Copy(HartAnswer.Data,2,4));
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function WriteRead_Selected_Single (var HartComm: THartMaster; CommandNumber: byte; var Selector: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Selector)+Single2Hart(SourceAndTarget),2*CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=5) then
          begin
          Selector := ord(HartAnswer.Data[1]);
          SourceAndTarget := Hart2Single(System.Copy(HartAnswer.Data,2,4));
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_Two_Singles (var HartComm: THartMaster; CommandNumber: byte; var Target1, Target2: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=8) then
          begin
          Target1 := Hart2Single(System.Copy(HartAnswer.Data,1,4));
          Target2 := Hart2Single(System.Copy(HartAnswer.Data,5,4));
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_Data_String (var HartComm: THartMaster; CommandNumber: byte; Request: string; var Target: string; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) then
          begin
          Target:=HartAnswer.Data;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Write_Data_String (var HartComm: THartMaster; CommandNumber: byte; Source: string; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,Source,CShortWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Write_Byte_And_Single (var HartComm: THartMaster; CommandNumber: byte; Source1: byte; Source2: single; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(CommandNumber,chr(Source1)+Single2Hart(Source2),CDefaultWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,CommandNumber,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

//*************************************************************************************************
//**
//**  Read and write functions assigned to universal command numbers, valid for all device types
//**
//*************************************************************************************************

// Command 0 - Read Identifier
function Read_Identifier (var HartComm: THartMaster; var Target: THDeviceIdentifier; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(0,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=12) then
          begin
          with Target do
               begin
               DeviceTypeCode254       := ord(HartAnswer.Data[1]);
               ManufacturerIdentCode   := ord(HartAnswer.Data[2]);
               ManufacturersDeviceType := ord(HartAnswer.Data[3]);
               NumberOfPreambles       := ord(HartAnswer.Data[4]);
               RevisionLevelOfUCD      := ord(HartAnswer.Data[5]);
               RevisionLevelOfTSD      := ord(HartAnswer.Data[6]);
               SoftwareRevision        := ord(HartAnswer.Data[7]);
               HardwareRevision        := ord(HartAnswer.Data[8]);
               Flags                   := ord(HartAnswer.Data[9]);
               DeviceIdentNumber[1]    := ord(HartAnswer.Data[10]);
               DeviceIdentNumber[2]    := ord(HartAnswer.Data[11]);
               DeviceIdentNumber[3]    := ord(HartAnswer.Data[12]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,0,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 1 - Read Primary Variable
function Read_PrimaryVariable(var HartComm: THartMaster; var Target: THPrimaryVariable; var SSS: string; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  SSS:='';
  with HartComm do
       begin
       HartAnswer:=MasterDialog(1,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=5) then
          begin
          with Target do
               begin
               PrimaryUnit:=ord(HartAnswer.Data[1]);
               PrimaryVariable:=Hart2Single(System.Copy(HartAnswer.Data,2,4));
               SSS:=HartAnswer.Data;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,1,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 2 - Read Primary Variable Current and Percent of Range
// use for this operation the function "Read_Two_Singles"

// Command 3 - Read Dynamic Variables
function Read_DynamicVariables (var HartComm: THartMaster; var Target: THDynamicVariables; BreakTime: word; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(3,'',BreakTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(System.Length(HartAnswer.Data)<14) then   // note that the reply may consist more data
          begin
          with Target do
               begin
               LoopCurrent         := Hart2Single(System.Copy(HartAnswer.Data,1,4));
               PrimaryUnit         := ord(HartAnswer.Data[5]);
               PrimaryVariable     := Hart2Single(System.Copy(HartAnswer.Data,6,4));
               SecondaryUnit       := ord(HartAnswer.Data[10]);
               SecondaryVariable   := Hart2Single(System.Copy(HartAnswer.Data,11,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and (AnyErrorOccur or // why is ComStatus evalauation required here ?
          ((HartAnswer.CommStatus and CCommandResponseStatus)>0) or ((HartAnswer.DeviceStatus and CFieldDeviceMalfunction)>0)) then
          DisplayCommErrors(HartCommErrors,3,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 13 - Read Tag, Descriptior and Date
function Read_TagDescriptorDate (var HartComm: THartMaster; var Target: THTagDescriptorDate; BreakTime: word; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(13,'',BreakTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=21) then
          begin
          with Target do
               begin
               TagName    := Hart2Ascii(System.Copy(HartAnswer.Data,1,6));
               Descriptor := Hart2Ascii(System.Copy(HartAnswer.Data,7,12));
               Day        := ord(HartAnswer.Data[19]);
               Month      := ord(HartAnswer.Data[20]);
               Year       := ord(HartAnswer.Data[21]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,13,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 18 - Write (and Read) Tag, Descriptior and Date
function WriteRead_TagDescriptorDate_Block (var HartComm: THartMaster; var SourceAndTarget: THTagDescriptorDate; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S:=Ascii2Hart(SourceAndTarget.TagName,8) +
     Ascii2Hart(SourceAndTarget.Descriptor,16) +
     chr(SourceAndTarget.Day) + chr(SourceAndTarget.Month) + chr(SourceAndTarget.Year);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(18,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=21) then
          begin
          with SourceAndTarget do
               begin
               TagName    := Hart2Ascii(System.Copy(HartAnswer.Data,1,6));
               Descriptor := Hart2Ascii(System.Copy(HartAnswer.Data,7,12));
               Day        := ord(HartAnswer.Data[19]);
               Month      := ord(HartAnswer.Data[20]);
               Year       := ord(HartAnswer.Data[21]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,18,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 15 - Read Primary Variable Output Information
function Read_PrimaryVariableOutput (var HartComm: THartMaster; var Target: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(15,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(System.Length(HartAnswer.Data)<15) then
          begin
          with Target do
               begin
               PVAlarm             := ord(HartAnswer.Data[1]);
               TransferFunction    := ord(HartAnswer.Data[2]);
               PVUnits             := ord(HartAnswer.Data[3]);
               Point20mA           := Hart2Single(System.Copy(HartAnswer.Data,4,4));
               Point4mA            := Hart2Single(System.Copy(HartAnswer.Data,8,4));
               DampingTimeConstant := Hart2Single(System.Copy(HartAnswer.Data,12,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,15,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 16 - Read Assembly Number
function Read_AssemblyNumber (var HartComm: THartMaster; var Target: THFinalAssemblyNumber; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(16,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=3) then
          begin
          with Target do
               begin
               Number[1] := ord(HartAnswer.Data[1]);
               Number[2] := ord(HartAnswer.Data[2]);
               Number[3] := ord(HartAnswer.Data[3]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,16,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 19 - Write (and Read) Assembly Number
function WriteRead_AssemblyNumber (var HartComm: THartMaster; var SourceAndTarget: THFinalAssemblyNumber; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S:=chr(SourceAndTarget.Number[1])+chr(SourceAndTarget.Number[2])+chr(SourceAndTarget.Number[3]);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(19,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=3) then
          begin
          with SourceAndTarget do
               begin
               Number[1] := ord(HartAnswer.Data[1]);
               Number[2] := ord(HartAnswer.Data[2]);
               Number[3] := ord(HartAnswer.Data[3]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,19,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

//*************************************************************************************************
//**
//**  Read and write functions assigned to common practice command numbers
//**  valid mostly for all MTL device types (with possible variations)
//**
//*************************************************************************************************

// Command 34 - Write (and Read) Damping Value
function WriteRead_Damping_Value (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;
begin
  Result:=WriteRead_One_Single(HartComm,34,SourceAndTarget.DampingTimeConstant,SuppressErrorMessages);
end;

// Command 35 - Write (and Read) Primary Variable Range Values
function WriteRead_Primary_Variables_Block (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := chr(SourceAndTarget.PVUnits)+Single2Hart(SourceAndTarget.Point20mA)+Single2Hart(SourceAndTarget.Point4mA);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(35,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (not(ConfigurationErrorOccur) or (HartAnswer.CommStatus=6)) and
          (System.Length(HartAnswer.Data)=9) then
          begin
          with SourceAndTarget do
               begin
               PVUnits   := ord(HartAnswer.Data[1]);
               Point20mA := Hart2Single(System.Copy(HartAnswer.Data,2,4));
               Point4mA  := Hart2Single(System.Copy(HartAnswer.Data,6,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,35,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 38 - Write (and Read) Reset Configuration Changed Flag - valid only for E01003 and MTL4532
function WriteRead_Reset_Configuration_Changed (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;
begin
  Result:=WriteRead_EmptyCommand(HartComm,38,SuppressErrorMessages);
end;

// Commands 40, 45 and 46 - Write (and Read) Primary Variable Current, Trim DAC Zero and DAC Gain
function WriteRead_Current_Value (var HartComm: THartMaster; CommandNumber: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
begin
  if (CommandNumber in [40,45,46]) then
     Result:=WriteRead_One_Single(HartComm,CommandNumber,SourceAndTarget,SuppressErrorMessages)
  else Result:=false;
end;

// Command 42 - Write (force) Master Reset
function Write_Master_Reset (var HartComm: THartMaster; ExpectAnswer: boolean; SuppressErrorMessages: boolean): boolean;
begin
  if ExpectAnswer then
     Result:=WriteRead_EmptyCommand(HartComm,42,SuppressErrorMessages)
  else Result:=false; // Write_EmptyCommand(HartComm,42,SuppressErrorMessages);
end;

// Command 44 - Write (and Read) Process Units
function WriteRead_Process_Units (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;
begin
  Result:=WriteRead_One_Byte(HartComm,44,SourceAndTarget.PVUnits,SuppressErrorMessages);
end;

// Command 47 - Write (and Read) Transfer Function
function WriteRead_TransferFunction_Information (var HartComm: THartMaster; var SourceAndTarget: THPrimaryVariableOutput; SuppressErrorMessages: boolean): boolean;
begin
  // the description of command 47 about negation of linearisation byte must be wrong !
  Result:=WriteRead_One_Byte(HartComm,47,SourceAndTarget.TransferFunction,SuppressErrorMessages);
end;

// Command 48 - Read Transmitter Status Information
function Read_TransmitterStatus (var HartComm: THartMaster; var Target: THTransmitterStatus; SuppressErrorMessages: boolean): boolean;
var I,L: byte;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(48,'',CDefaultWaitTime);
       Target.ComInfo:=HartAnswer.CommStatus;
       Target.DevInfo:=HartAnswer.DeviceStatus;
       L:=System.Length(HartAnswer.Data);
       if not(ExecutionErrorOccur) and
          not (TransferErrorOccur) and
          (L>0) then
          begin
          SetLength(Target.Status,L);
          for I:=0 to L-1 do
              Target.Status[I]:=HartAnswer.Data[I+1];
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,48,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

//*************************************************************************************************
//**
//**  Read and write functions assigned to transmitter specific command numbers,
//**  valid only for all MTL device types
//**
//*************************************************************************************************

//*************************************************************************************************
//**  Read and write functions assigned to all MTL temperature converters MTLxy7z
//*************************************************************************************************

// Command 200 - Read Primary Variables
function Read_PrimaryVariables(var HartComm: THartMaster; var Target: THPrimaryVariables; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(200,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(System.Length(HartAnswer.Data)<25) then
          begin
          with Target do
               begin
               ScaledInput              := Hart2Single(System.Copy(HartAnswer.Data,1,4));
               SmoothedInput            := Hart2Single(System.Copy(HartAnswer.Data,5,4));
               ColdJunctionResistance   := Hart2Single(System.Copy(HartAnswer.Data,9,4));
               ColdJunctionTemperature  := Hart2Single(System.Copy(HartAnswer.Data,13,4));
               ColdJunctionCompensation := Hart2Single(System.Copy(HartAnswer.Data,17,4));
               ProcessVariable          := Hart2Single(System.Copy(HartAnswer.Data,21,4));
               SensorFlags              := ord(HartAnswer.Data[25]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,200,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 201 - Read Secondary Variables
function Read_SecondaryVariables(var HartComm: THartMaster; var Target: THSecondaryVariables; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(201,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=25) then
          begin
          with Target do
               begin
               ProcessVariableScaledInput := Hart2Single(System.Copy(HartAnswer.Data,1,4));
               DampedProcessVariable      := Hart2Single(System.Copy(HartAnswer.Data,5,4));
               LoopCurrent                := Hart2Single(System.Copy(HartAnswer.Data,9,4));
               NoiseFloor                 := Hart2Single(System.Copy(HartAnswer.Data,13,4));
               // ignore not significant 8 data bytes from 17 to 24
               SensorFlags                := ord(HartAnswer.Data[25]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,201,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 202 - Read RAW Input Output Values
function Read_RAWInputOutputValues(var HartComm: THartMaster; var Target: THRAWInputOutputValues; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(202,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=25) then
          begin
          with Target do
               begin
               DACValue       := ord(HartAnswer.Data[1])*16+ord(HartAnswer.Data[2]);
               MainInput      := ord(HartAnswer.Data[3])*16+ord(HartAnswer.Data[4]);
               AuxInput       := ord(HartAnswer.Data[5])*16+ord(HartAnswer.Data[6]);
               MainExcitation := ord(HartAnswer.Data[7])*16+ord(HartAnswer.Data[8]);
               AuxExcitation  := ord(HartAnswer.Data[9])*16+ord(HartAnswer.Data[10]);
               LeadOffTest1   := ord(HartAnswer.Data[11])*16+ord(HartAnswer.Data[12]);
               LeadOffTest2   := ord(HartAnswer.Data[13])*16+ord(HartAnswer.Data[14]);
               LeadOffTest5   := ord(HartAnswer.Data[15])*16+ord(HartAnswer.Data[16]);
               LeadOffTest6   := ord(HartAnswer.Data[17])*16+ord(HartAnswer.Data[18]);
               // ignore not significant 6 data bytes from 19 to 24
               SensorFlags    := ord(HartAnswer.Data[25]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,202,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 203 - Read Calibration Data
function Read_CalibrationData(var HartComm: THartMaster; var Target: THCalibrationData; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(203,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=25) then
          begin
          with Target do
               begin
               RrefCal     := Hart2Single(System.Copy(HartAnswer.Data,1,4));
               Rin5Cal     := Hart2Single(System.Copy(HartAnswer.Data,5,4));
               Rin6Cal     := Hart2Single(System.Copy(HartAnswer.Data,9,4));
               VrefCal     := Hart2Single(System.Copy(HartAnswer.Data,13,4));
               CalFlag     := ord(HartAnswer.Data[17]);
               // ignore not significant 7 data bytes from 18 to 24
               SensorFlags := ord(HartAnswer.Data[25]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,203,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 220 - Write Restore Autotest Defaults
function Write_Restore_AllAutotestDefaults (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Byte(HartComm,220,85,SuppressErrorMessages);
end;

// Command 221 - Write R7 Calibration Value
function Write_Rref_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Single(HartComm,221,Source,SuppressErrorMessages);
end;

// Command 222 - Write R2 Calibration Value
function Write_Rin5_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Single(HartComm,222,Source,SuppressErrorMessages);
end;

// Command 223 - Write R3 Calibration Value
function Write_Rin6_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Single(HartComm,223,Source,SuppressErrorMessages);
end;

// Command 224 - Write IC2/16 Calibration Value
function Write_Vref_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Single(HartComm,224,Source,SuppressErrorMessages);
end;

// Command 225 - Write Calibration Flag
function Write_CalibrationFlag (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Byte(HartComm,225,255,SuppressErrorMessages);
end;

// Command 226 - Write Cold Junction Compensation Calibration Value
function Write_CJC_Calibration (var HartComm: THartMaster; Source: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Single(HartComm,226,Source,SuppressErrorMessages);
end;

//*************************************************************************************************
//**  Read and write functions assigned to the MTL4075 (and posibly to the successors)
//*************************************************************************************************

// Command 205 - Read Trip Parameters for the MTL4075 (and MTL4575)
function Read_TripParameters_x075 (var HartComm: THartMaster; var Target: THx075TripParameters; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(205,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=9) then
          begin
          with Target do
               begin
               TripOutputFunction := ord(HartAnswer.Data[9]); // 0 - Off, 1 - EBD, 2 - Trip
               if (TripOutputFunction in [0..2]) then
                  begin
                  TripValue   := Hart2Single(System.Copy(HartAnswer.Data,1,4));
                  Hysteresis  := Hart2Single(System.Copy(HartAnswer.Data,5,4));
                  end
               else
                  begin
                  TripValue   := 0.0;
                  Hysteresis  := 0.0;
                  end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,205,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 204 - Write (and Read) Trip Parameters for the MTL4075 (and MTL4575)
function WriteRead_TripParameters_x075 (var HartComm: THartMaster; var SourceAndTarget: THx075TripParameters; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := Single2Hart(SourceAndTarget.TripValue) + Single2Hart(SourceAndTarget.Hysteresis) +
       chr(SourceAndTarget.TripOutputFunction);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(204,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=9) then
          begin
          with SourceAndTarget do
               begin
               TripOutputFunction := ord(HartAnswer.Data[9]);
               if (TripOutputFunction in [0..2]) then  // 0 - Off, 1 - EBD, 2 - Trip
                  begin
                  TripValue   := Hart2Single(System.Copy(HartAnswer.Data,1,4));
                  Hysteresis  := Hart2Single(System.Copy(HartAnswer.Data,5,4));
                  end
               else
                  begin
                  TripValue   := 0.0;
                  Hysteresis  := 0.0;
                  end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,204,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

//*************************************************************************************************
//**  Read and write functions assigned to the MTL4575 (and posibly to the successors)
//*************************************************************************************************

// Command 213 - Read Channel Dynamic Variables
function Read_Channel_DynamicVariables (var HartComm: THartMaster; ChannelNumber: byte; var Target: THDynamicVariables; BreakTime: word; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(213,chr(ChannelNumber),BreakTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(System.Length(HartAnswer.Data)<14) and // note that the reply may consist more data
          (ord(HartAnswer.Data[1])=ChannelNumber) then
          begin
          with Target do
               begin
               LoopCurrent         := Hart2Single(System.Copy(HartAnswer.Data,2,4));
               PrimaryUnit         := ord(HartAnswer.Data[6]);
               PrimaryVariable     := Hart2Single(System.Copy(HartAnswer.Data,7,4));
               SecondaryUnit       := ord(HartAnswer.Data[11]);
               SecondaryVariable   := Hart2Single(System.Copy(HartAnswer.Data,12,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and (AnyErrorOccur or // why is ComStatus evalauation required here ?
         ((HartAnswer.CommStatus and CCommandResponseStatus)>0) or ((HartAnswer.DeviceStatus and CFieldDeviceMalfunction)>0)) then
          DisplayCommErrors(HartCommErrors,213,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Commands 140 - Write (and Read) Channel Primary Variable Current
function WriteRead_Channel_Current_Value (var HartComm: THartMaster; ChannelNumber: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := chr(ChannelNumber) + Single2Hart(SourceAndTarget);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(140,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=5) and
          (ord(HartAnswer.Data[1])=ChannelNumber) then
          begin
          SourceAndTarget := Hart2Single(System.Copy(HartAnswer.Data,2,4));
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,140,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Commands 145 - Write (and Read) Channel Trim DAC Zero and DAC Gain
function WriteRead_Channel_TrimmDAC (var HartComm: THartMaster; ChannelNumber, TrimmMode: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := chr(ChannelNumber) + chr(TrimmMode) + Single2Hart(SourceAndTarget);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(145,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=6) and
          (ord(HartAnswer.Data[1])=ChannelNumber) and
          (ord(HartAnswer.Data[2])=TrimmMode) then
          begin
          SourceAndTarget := Hart2Single(System.Copy(HartAnswer.Data,3,4));
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,145,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 148 - Read Channel Transmitter Status Information
function Read_Channel_TransmitterStatus (var HartComm: THartMaster; ChannelNumber: byte; var Target: THTransmitterStatus; SuppressErrorMessages: boolean): boolean;
var I,L: byte;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(148,chr(ChannelNumber),CDefaultWaitTime);
       Target.ComInfo:=HartAnswer.CommStatus;
       Target.DevInfo:=HartAnswer.DeviceStatus;
       L:=System.Length(HartAnswer.Data);
       if not(ExecutionErrorOccur) and
          not (TransferErrorOccur) and
          (L>1) and
          (ord(HartAnswer.Data[1])=ChannelNumber) then
          begin
          SetLength(Target.Status,L);
          for I:=0 to L-1 do
              Target.Status[I]:=HartAnswer.Data[I+1];
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,148,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 151 - Read Input Parameters for the MTL4576
function Read_InputParameters_x576 (var HartComm: THartMaster; Channel: byte; var Target: THInputParameters_x576; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(151,chr(Channel),2*CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=17) then
          begin
          with Target do
               begin
               Channel := ord(HartAnswer.Data[1]);
               SensorType := ord(HartAnswer.Data[2]);
               PVUnits := ord(HartAnswer.Data[3]);
               Linearisation := ord(HartAnswer.Data[4]);
               CJC_Option := ord(HartAnswer.Data[5]);
               Point4mA := Hart2Single(System.Copy(HartAnswer.Data,6,4));
               Point20mA := Hart2Single(System.Copy(HartAnswer.Data,10,4));
               CJC_UserValue := Hart2Single(System.Copy(HartAnswer.Data,14,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,151,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 152 - Write and Read Input Parameters for the MTL4576
function WriteRead_InputParameters_x576 (var HartComm: THartMaster; Channel: byte; var SourceAndTarget: THInputParameters_x576; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := chr(Channel) +
       chr(SourceAndTarget.SensorType) +  chr(SourceAndTarget.PVUnits) + chr(SourceAndTarget.Linearisation) + chr(SourceAndTarget.CJC_Option) +
       Single2Hart(SourceAndTarget.Point4mA) + Single2Hart(SourceAndTarget.Point20mA) + Single2Hart(SourceAndTarget.CJC_UserValue);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(152,S,2*CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=17) then
          begin
          with SourceAndTarget do
               begin
               Channel := ord(HartAnswer.Data[1]);
               SensorType := ord(HartAnswer.Data[2]);
               PVUnits := ord(HartAnswer.Data[3]);
               Linearisation := ord(HartAnswer.Data[4]);
               CJC_Option := ord(HartAnswer.Data[5]);
               Point4mA := Hart2Single(System.Copy(HartAnswer.Data,6,4));
               Point20mA := Hart2Single(System.Copy(HartAnswer.Data,10,4));
               CJC_UserValue := Hart2Single(System.Copy(HartAnswer.Data,14,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,152,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 161 - Read Output Parameters for the MTL4576
function Read_OutputParameters_x576 (var HartComm: THartMaster; Channel: byte; var Target: THOutputParameters_x576; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(161,chr(Channel),CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=23) then
          begin
          with Target do
               begin
               Channel := ord(HartAnswer.Data[1]);
               OpenCircuitAlarmLevel := ord(HartAnswer.Data[2]);
               ShortCircuitAlarmLevel := ord(HartAnswer.Data[3]);
               TransmitterFailureAlarmLevel := ord(HartAnswer.Data[4]);
               ColdJunctionFailureAlarmLevel := ord(HartAnswer.Data[5]);
               NAMUR_MTL := ord(HartAnswer.Data[6]);
               DampingTimeConstant := Hart2Single(System.Copy(HartAnswer.Data,7,4));
               SmoothingTimeConstant := Hart2Single(System.Copy(HartAnswer.Data,11,4));
               EBD_Option := ord(HartAnswer.Data[15]);
               TripLimit := Hart2Single(System.Copy(HartAnswer.Data,16,4));
               TripHysteresis := Hart2Single(System.Copy(HartAnswer.Data,20,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,161,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 162 - Write and Read Output Parameters for the MTL4576
function WriteRead_OutputParameters_x576 (var HartComm: THartMaster; Channel: byte; var SourceAndTarget: THOutputParameters_x576; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := chr(Channel) +
       chr(SourceAndTarget.OpenCircuitAlarmLevel) +  chr(SourceAndTarget.ShortCircuitAlarmLevel) +
       chr(SourceAndTarget.TransmitterFailureAlarmLevel) +  chr(SourceAndTarget.ColdJunctionFailureAlarmLevel) +  chr(SourceAndTarget.NAMUR_MTL) +
       Single2Hart(SourceAndTarget.DampingTimeConstant) + Single2Hart(SourceAndTarget.SmoothingTimeConstant) + chr(SourceAndTarget.EBD_Option) +
       Single2Hart(SourceAndTarget.TripLimit) + Single2Hart(SourceAndTarget.TripHysteresis);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(162,S,2*CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=23) then
          begin
          with SourceAndTarget do
               begin
               Channel := ord(HartAnswer.Data[1]);
               OpenCircuitAlarmLevel := ord(HartAnswer.Data[2]);
               ShortCircuitAlarmLevel := ord(HartAnswer.Data[3]);
               TransmitterFailureAlarmLevel := ord(HartAnswer.Data[4]);
               ColdJunctionFailureAlarmLevel:= ord(HartAnswer.Data[5]);
               NAMUR_MTL := ord(HartAnswer.Data[6]);
               DampingTimeConstant := Hart2Single(System.Copy(HartAnswer.Data,7,4));
               SmoothingTimeConstant := Hart2Single(System.Copy(HartAnswer.Data,11,4));
               EBD_Option := ord(HartAnswer.Data[15]);
               TripLimit := Hart2Single(System.Copy(HartAnswer.Data,16,4));
               TripHysteresis := Hart2Single(System.Copy(HartAnswer.Data,20,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,152,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 208 - Read Common Parameters for the MTL4576
function Read_CommonParameters_x576 (var HartComm: THartMaster; var Target: THCommonParameters_x576; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(208,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=3) then
          begin
          with Target do
               begin
               response_time:= ord(HartAnswer.Data[1]);
               FilterFrequencySelector:= ord(HartAnswer.Data[2]);
               InputOption:= ord(HartAnswer.Data[3]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,208,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 207 - Write and Read Common Parameters for the MTL4576
function WriteRead_CommonParameters_x576 (var HartComm: THartMaster; var SourceAndTarget: THCommonParameters_x576; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  Result:=false;
  S := chr(SourceAndTarget.response_time) + chr(SourceAndTarget.FilterFrequencySelector) + chr(SourceAndTarget.InputOption);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(207,S,CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=3) then
          begin
          with SourceAndTarget do
               begin
               response_time:= ord(HartAnswer.Data[1]);
               FilterFrequencySelector:= ord(HartAnswer.Data[2]);
               InputOption:= ord(HartAnswer.Data[3]);
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,207,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_UserSensorHeader_x576 (var HartComm: THartMaster; var Target: THUserSensorHeader_x576; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(209,chr(0),CDefaultWaitTime*20);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          not(System.Length(HartAnswer.Data)<14) and
          (ord(HartAnswer.Data[1])=0) then
          begin
          with Target do
               begin
               SensorType:=ord(HartAnswer.Data[2]);
               TMIN := Hart2Single(System.Copy(HartAnswer.Data,3,4));
               TMAX := Hart2Single(System.Copy(HartAnswer.Data,7,4));
               TSEG := Hart2Single(System.Copy(HartAnswer.Data,11,4));
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,209,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function Read_UserSensorTable_x576 (var HartComm: THartMaster; var Target: THUserSensorTable; SuppressErrorMessages: boolean): boolean;
var lProfileNumber: byte;
    lError: boolean;
    I: integer;
begin
  Result:=false;
  lError:=false;
  with HartComm do
       begin
       for lProfileNumber:=1 to 5 do
           begin
           HartAnswer:=MasterDialog(209,chr(lProfileNumber),CDefaultWaitTime*20);
           if not(ExecutionErrorOccur) and
              not(TransferErrorOccur) and
              not(System.Length(HartAnswer.Data)<27) and
              (ord(HartAnswer.Data[1])=lProfileNumber) then
              begin
              for I:=0 to 12 do
                  Target[(lProfileNumber-1)*13+I]:=Hart2Integer(System.Copy(HartAnswer.Data,I*2+2,2));
              Result:=true;
              end
           else
              begin
              lError:=true;
              break;
              end;
           end;
       if (lError or AnyErrorOccur) and not(SuppressErrorMessages) then
          DisplayCommErrors (HartCommErrors,209,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

function WriteVerify_UserSensorData_x576 (var HartComm: THartMaster; Source: THUserSensorData_x576; SuppressErrorMessages: boolean): boolean;
var lResponse: THUserSensorData_x576;
    lProfileNumber: byte;
    lError: boolean;
    I: integer;
    S: string;
begin
  Result:=false;
  with HartComm do
       begin
       S := chr(0) + chr(Source.SensorType) +
                     Single2Hart(Source.TMIN) +
                     Single2Hart(Source.TMAX) +
                     Single2Hart(Source.TSEG);
       HartAnswer:=MasterDialog(210,S,CDefaultWaitTime);
       lError:=(ExecutionErrorOccur) or
               (TransferErrorOccur) or
               (ConfigurationErrorOccur) or
               (System.Length(HartAnswer.Data)<>14);
       if not(lError) then
          begin
          with lResponse do
               begin
               SensorType:=ord(HartAnswer.Data[2]);
               TMIN := Hart2Single(System.Copy(HartAnswer.Data,3,4));
               TMAX := Hart2Single(System.Copy(HartAnswer.Data,7,4));
               TSEG := Hart2Single(System.Copy(HartAnswer.Data,11,4));
               end;
          for lProfileNumber:=1 to 5 do
              begin
              S := chr(lProfileNumber) + Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+0],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+1],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+2],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+3],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+4],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+5],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+6],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+7],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+8],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+9],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+10],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+11],2) +
                                         Integer2Hart(Source.CoeffTable[(lProfileNumber-1)*13+12],2);
              HartAnswer:=MasterDialog(210,S,CDefaultWaitTime);
              lError:=(ExecutionErrorOccur) or
                      (TransferErrorOccur) or
                      (ConfigurationErrorOccur) or
                      (System.Length(HartAnswer.Data)<>27);
              if not(lError) then
                 for I:=0 to 12 do
                     lResponse.CoeffTable[(lProfileNumber-1)*13+I]:=Hart2Integer(System.Copy(HartAnswer.Data,I*2+2,2))
              else break;
              end;
          if not(lProfileNumber=6) then
             lError:=true;
          if not(lError) then
             begin
             lError:=(Source.SensorType<>lResponse.SensorType) or
                     (Source.TMIN<>lResponse.TMIN) or
                     (Source.TMAX<>lResponse.TMAX) or
                     (Source.TSEG<>lResponse.TSEG);
             // Add table verification
             Result:=not(lError);
             end;
          end;
       if (lError or AnyErrorOccur) and not(SuppressErrorMessages) then
          DisplayCommErrors (HartCommErrors,210,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

//*************************************************************************************************
//**  Read and write functions assigned to all MTL frequency converters (E01003 and MTL4532)
//*************************************************************************************************

// Command 135 - Write Digital Diagnostics for E01003 and MTL4532
function Write_Digital_Diagnostics (var HartComm: THartMaster; Selector: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Write_One_Byte(HartComm,135,Selector,SuppressErrorMessages);
end;

// Command 138 - Write Pulse Divider Parameters for E01003 and MTL4532
function Write_DividerParameters (var HartComm: THartMaster; Source: THDividerParameters; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := chr(Source.FunctionSelect) +
       Single2Hart(Source.Divider) + Single2Hart(Source.Duration);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(138,S,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       if not(TransferErrorOccur) and
          // not(ExecutionErrorOccur) and
          // not(ConfigurationErrorOccur) and
          (System.Length(HartAnswer.Data)=0) then
          begin
          end;
       if not(SuppressErrorMessages) and (TransferErrorOccur or ConfigurationErrorOccur) then
          DisplayCommErrors(HartCommErrors,138,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       Result:=not(TransferErrorOccur);
       end;
end;

// Command 138 - Write Pulse Divider Parameters for E01003
function Write_DividerParameters_1003 (var HartComm: THartMaster; Source: THDividerParameters; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := chr(Source.FunctionSelect) +
       Single2Hart(Source.Divider) + Single2Hart(Source.Duration);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(138,S,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,138,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;


// Command 139 - Read Pulse Divider Parameters for E01003 and MTL4532
function Read_DividerParameters (var HartComm: THartMaster; var Target: THDividerParameters; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(139,'',CDefaultWaitTime);
       if not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=9) then
          begin
          with Target do
               begin
               FunctionSelect := ord(HartAnswer.Data[1]);
               if (FunctionSelect=CP_Divider_Enabled) or  // MTL4532
                  (FunctionSelect=CP3_Divider_Enabled) then // or E01003
                  begin
                  Divider     := Hart2Single(System.Copy(HartAnswer.Data,2,4));
                  Duration    := Hart2Single(System.Copy(HartAnswer.Data,6,4));
                  end
               else if (FunctionSelect=(CP_Divider_Enabled or CP_Divider_Toggle)) then // MTL4532 only
                       begin
                       Divider     := Hart2Single(System.Copy(HartAnswer.Data,2,4));
                       Duration    := 0.0;
                       end
                    else
                       begin
                       Divider     := 0.0;
                       Duration    := 0.0;
                       end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,139,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 139 - Read Pulse Divider Parameters for E01003
function Read_DividerParameters_1003 (var HartComm: THartMaster; var Target: THDividerParameters; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(139,'',CDefaultWaitTime);
       if not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=9) then
          begin
          with Target do
               begin
               FunctionSelect := ord(HartAnswer.Data[1]);
               if ((FunctionSelect and CP3_Divider_Enabled)>0) then
                  begin
                  Divider     := Hart2Single(System.Copy(HartAnswer.Data,2,4));
                  Duration    := Hart2Single(System.Copy(HartAnswer.Data,6,4));
                  end
               else
                  begin
                  Divider     := 0.0;
                  Duration    := 0.0;
                  end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,139,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 140 - Write Adjust Output Parameters for E01003 and MTL4532
function Write_AdjustOutputParameters (var HartComm: THartMaster; Source: THAdjustOutputParameters; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := chr(Source.AdjMode) + chr(Source.AdjUnits) +
       Single2Hart(Source.OffsetValue) + Single2Hart(Source.GainValue);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(140,S,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,140,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 141 - Read Adjust Output Parameters for E01003 and MTL4532
function Read_AdjustOutputParameters (var HartComm: THartMaster; var Target: THAdjustOutputParameters; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(141,'',CDefaultWaitTime);
       if not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=10) then
          begin
          with Target do
               begin
               AdjMode := ord(HartAnswer.Data[1]);
               if (AdjMode=CHart_Adjust_Soft) or
                  (AdjMode=CHart_Adjust_Hard) then
                  begin
                  AdjUnits    := ord(HartAnswer.Data[2]);
                  OffsetValue := Hart2Single(System.Copy(HartAnswer.Data,3,4));
                  GainValue   := Hart2Single(System.Copy(HartAnswer.Data,7,4));
                  end
               else
                  begin
                  AdjUnits:=CHart_Percent;
                  OffsetValue:=0.0;
                  GainValue:=0.0;
                  end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,141,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 150 - Write Set Deviced To Default for E01003 and MTL4532
function Write_SetDeviceToDefault (var HartComm: THartMaster; SuppressErrorMessages: boolean): boolean;
begin
  with HartComm do
       begin
       HartAnswer:=MasterDialog(150,'Herder'+#0,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,150,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

//*************************************************************************************************
//**  Read and write functions assigned to the eurocard E01003 (and possibly to the MTL4532)
//*************************************************************************************************

// Command 131 - Write Input Source Selection
function Write_InputSourceSelection_1003 (var HartComm: THartMaster; Source: THInputAndLFDParameters_1003; SuppressErrorMessages: boolean): boolean;
begin
  Result:=WriteRead_One_Byte(HartComm,131,Source.FunctionSelect,true);
end;

// Command 132 - Read Input Source Selection
function Read_InputSourceSelection_1003 (var HartComm: THartMaster; var Target: THInputAndLFDParameters_1003; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Read_One_Byte(HartComm,132,Target.FunctionSelect,true);
end;

// Command 133 - Write LeakFlow Off Parameters
function Write_LeakFlowOffParameters_1003 (var HartComm: THartMaster; Source: THLeakFlowOffParameters_1003; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := chr(Source.FunctionSelect) +
       Single2Hart(Source.LeakFlowOffValue) + Single2Hart(Source.LeakFlowOffCurrent);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(133,S,CDefaultWaitTime); // CLongWaitTime
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,133,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 134 - Read LeakFlow Off Parameters
function Read_LeakFlowOffParameters_1003 (var HartComm:THartMaster; var Target:THLeakFlowOffParameters_1003; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(134,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=9) then
          begin
          with Target do
               begin
               FunctionSelect := ord(HartAnswer.Data[1]);
               if ((FunctionSelect and CP3_LeakFlowEnabled)>0) then
                  begin
                  LeakFlowOffValue   := Hart2Single(System.Copy(HartAnswer.Data,2,4));
                  LeakFlowOffCurrent := Hart2Single(System.Copy(HartAnswer.Data,6,4));
                  end
               else
                  begin
                  LeakFlowOffValue   := 0.0;
                  LeakFlowOffCurrent := 0.0;
                  end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,134,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 136 - Write Trip Parameters for E01003
function Write_TripParameters_1003 (var HartComm: THartMaster; Source: THTripParameters_1003; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := chr(Source.FunctionSelect) + chr(Source.TPUnits) +
       Single2Hart(Source.TripValue) + Single2Hart(Source.Hysteresis);
  SetLength(HartAnswer.Data,10);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(136,S,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,136,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 137 - Read Trip Parameters for E01003
function Read_TripParameters_1003 (var HartComm: THartMaster; var Target: THTripParameters_1003; Selector: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(137,chr(Selector),CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=10) then
          begin
          with Target do
               begin
               FunctionSelect := ord(HartAnswer.Data[1]);
               if ((FunctionSelect and CP3_Trip_Enabled)>0) and
                  ((FunctionSelect and CP3_TripB_Pattern)=Selector) then
                  begin
                  TPUnits     := ord(HartAnswer.Data[2]);
                  TripValue   := Hart2Single(System.Copy(HartAnswer.Data,3,4));
                  Hysteresis  := Hart2Single(System.Copy(HartAnswer.Data,7,4));
                  end
               else
                  begin
                  FunctionSelect:=0;
                  TPUnits:=CHart_Percent;
                  TripValue:=0.0;
                  Hysteresis:=0.0;
                  end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,137,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 142 - Write Trimmer Variables Parameters for E01003
function Write_TrimmerVariables (var HartComm: THartMaster; Source: THTrimmerVariables_1003; Selector: byte; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := chr(Selector) + Integer2Hart(Source.OffsetValue,2) + Integer2Hart(Source.GainValue,2);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(142,S,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,142,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 143 - Read Trimmer Variables Parameters for E01003
function Read_TrimmerVariables (var HartComm: THartMaster; var Target: THTrimmerVariables_1003; Selector: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(143,chr(Selector),CDefaultWaitTime);
       if not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=4) then
          begin
          Target.OffsetValue := Hart2Integer(System.Copy(HartAnswer.Data,1,2));
          Target.GainValue := Hart2Integer(System.Copy(HartAnswer.Data,3,2));
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,143,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 155 - Read Internal Values
function Read_Internal_Value (var HartComm: THartMaster; var Target: TByteDynArray; Selector: byte; SuppressErrorMessages: boolean): boolean;
var I: word;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(155,chr(Selector),CDefaultWaitTime);
       if not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=6) then
          begin
          SetLength(Target,6);
          for I:=0 to 5 do Target[I]:=ord(HartAnswer.Data[I+1]);
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,155,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

//*************************************************************************************************
//**  Read and write functions assigned to the MTL4532
//*************************************************************************************************

// Command 133 - Write LeakFlow Off Parameters for MTL4532
function Write_LeakFlowOffParameters_x532 (var HartComm: THartMaster; Source: THLeakFlowOffParameters_x532; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  S := chr(Source.FunctionSelect) + chr(Source.LFUnits) +
       Single2Hart(Source.LeakFlowOffValue) + Single2Hart(Source.LeakFlowOffCurrent);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(133,S,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,133,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 134 - Read LeakFlow Off Parameters for MTL4532
function Read_LeakFlowOffParameters_x532 (var HartComm:THartMaster; var Target: THLeakFlowOffParameters_x532; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(134,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=10) then
          begin
          with Target do
               begin
               FunctionSelect := ord(HartAnswer.Data[1]);
               if (FunctionSelect=CHart_Option_On) then
                  begin
                  LFUnits := ord(HartAnswer.Data[2]);
                  LeakFlowOffValue   := Hart2Single(System.Copy(HartAnswer.Data,3,4));
                  LeakFlowOffCurrent := Hart2Single(System.Copy(HartAnswer.Data,7,4));
                  end
               else
                  begin
                  LFUnits := CHart_Percent;
                  LeakFlowOffValue   := 0.0;
                  LeakFlowOffCurrent := 0.0;
                  end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,134,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 136 - Write Trip Parameters for MTL4532
function Write_TripParameters_x532 (var HartComm: THartMaster; Source: THTripParameters_x532; SuppressErrorMessages: boolean): boolean;
var S: string;
begin
  if (Source.FunctionSelect=0) then S := chr(Source.FunctionSelect)
  else if ((Source.FunctionSelect and CP_TripAOnALarm)>0) and
          ((Source.FunctionSelect and (CP_TripABInfo or CP_TripBInfo))=0) then
          S := chr(Source.FunctionSelect) +
               chr(Source.TPAUnits) + Single2Hart(Source.TripAValue) + Single2Hart(Source.AHysteresis)
  else if ((Source.FunctionSelect and CP_TripBOnPulse)>0) and
          ((Source.FunctionSelect and (CP_TripABInfo or CP_TripAInfo))=0) then
          S := chr(Source.FunctionSelect) +
               chr(Source.TPBUnits) + Single2Hart(Source.TripBValue) + Single2Hart(Source.BHysteresis)
  else if (((Source.FunctionSelect and CP_TripABOnAlarm)>0) and
           ((Source.FunctionSelect and (CP_TripABOnPulse or CP_TripAInfo or CP_TripBInfo))=0)) or
          (((Source.FunctionSelect and CP_TripABOnPulse)>0) and
           ((Source.FunctionSelect and (CP_TripABOnAlarm or CP_TripAInfo or CP_TripBInfo))=0)) then
          S := chr(Source.FunctionSelect) +
               chr(Source.TPAUnits) + Single2Hart(Source.TripAValue) + Single2Hart(Source.AHysteresis) +
               chr(Source.TPBUnits) + Single2Hart(Source.TripBValue) + Single2Hart(Source.BHysteresis);
  with HartComm do
       begin
       HartAnswer:=MasterDialog(136,S,CLongWaitTime);
       // Complete HART reply with empty data and without configuration error expected
       Result:=not(ExecutionErrorOccur) and
               not(TransferErrorOccur) and
               not(ConfigurationErrorOccur) and
               (System.Length(HartAnswer.Data)=0);
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,136,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

// Command 137 - Read Trip Parameters for MTL4532
function Read_TripParameters_x532 (var HartComm: THartMaster; var Target: THTripParameters_x532; SuppressErrorMessages: boolean): boolean;
var DL: integer;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(137,'',CDefaultWaitTime);
       if not(TransferErrorOccur) then
          begin
          with Target do
               begin
               FunctionSelect := ord(HartAnswer.Data[1]);
               DL:=System.Length(HartAnswer.Data);
               case DL of
                    19: begin
                        TPAUnits     := ord(HartAnswer.Data[2]);
                        TripAValue   := Hart2Single(System.Copy(HartAnswer.Data,3,4));
                        AHysteresis  := Hart2Single(System.Copy(HartAnswer.Data,7,4));
                        TPBUnits     := ord(HartAnswer.Data[11]);
                        TripBValue   := Hart2Single(System.Copy(HartAnswer.Data,12,4));
                        BHysteresis  := Hart2Single(System.Copy(HartAnswer.Data,16,4));
                        end;
                    10: if ((FunctionSelect and CP_TripAOnALarm)>0) then
                           begin
                           TPAUnits     := ord(HartAnswer.Data[2]);
                           TripAValue   := Hart2Single(System.Copy(HartAnswer.Data,3,4));
                           AHysteresis  := Hart2Single(System.Copy(HartAnswer.Data,7,4));
                           end
                        else if ((FunctionSelect and CP_TripBOnPulse)>0) then
                                begin
                                TPBUnits     := ord(HartAnswer.Data[2]);
                                TripBValue   := Hart2Single(System.Copy(HartAnswer.Data,3,4));
                                BHysteresis  := Hart2Single(System.Copy(HartAnswer.Data,7,4));
                                end;
                    else FunctionSelect:=0;
                    end;
               end;
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors (HartCommErrors,137,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

  // Read and write functions assigned to the MTL4582B

function Read_ScaledIPT_x582B (var HartComm: THartMaster; var Target: single; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Read_One_Single(HartComm,200,Target,SuppressErrorMessages);
end;

function Read_CalibrationParameters_x582B (var HartComm: THartMaster; Selector: byte; var Target: single; SuppressErrorMessages: boolean): boolean;
var B: byte;
begin
  B:=Selector;
  if (Selector in [0..9]) then
     Result:=(Read_Selected_Single(HartComm, 203, B, Target, SuppressErrorMessages) and (B=Selector))
  else Result:=false;
end;

function WriteRead_CalibrationParameters_x582B (var HartComm: THartMaster; Selector: byte; var SourceAndTarget: single; SuppressErrorMessages: boolean): boolean;
var B: byte;
begin
  B:=Selector;
  if (Selector in [0..9]) then
     Result:=(WriteRead_Selected_Single(HartComm, 204, B, SourceAndTarget, SuppressErrorMessages) and (B=Selector))
  else Result:=false;
end;

function Write_HardwareRevision_x582B (var HartComm: THartMaster; HardwareRevision: byte; SuppressErrorMessages: boolean): boolean;
var B: byte;
begin
  B:=HardwareRevision;
  Result:=(WriteRead_One_Byte(HartComm,211,B, SuppressErrorMessages) and (B=HardwareRevision));
end;

function Read_CalibrationFlag_x582B (var HartComm: THartMaster; var Target: byte; SuppressErrorMessages: boolean): boolean;
begin
  Result:=Read_One_Byte(HartComm,224,Target,SuppressErrorMessages);
end;

function Read_PrimaryVariableOutput_x582B (var HartComm: THartMaster; var Target: THPrimaryVariableRange; SuppressErrorMessages: boolean): boolean;
begin
  Result:=false;
  with HartComm do
       begin
       HartAnswer:=MasterDialog(227,'',CDefaultWaitTime);
       if not(ExecutionErrorOccur) and
          not(TransferErrorOccur) and
          (System.Length(HartAnswer.Data)=9) then
          begin
          Target.PrimaryUnit := ord(HartAnswer.Data[1]);
          Target.UpperLimit  := Hart2Single(System.Copy(HartAnswer.Data,2,4));
          Target.LowerLimit  := Hart2Single(System.Copy(HartAnswer.Data,6,4));
          Result:=true;
          end;
       if not(SuppressErrorMessages) and AnyErrorOccur then
          DisplayCommErrors(HartCommErrors,227,HartAnswer.CommStatus,HartAnswer.DeviceStatus);
       end;
end;

end.








