//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**
//**  ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
//**  HARTDATACONSTANTS.PAS
//**
//**	Description of data constants for applications with HART communication (PCS45)
//**  (derived from the previous unit "DataConstants.pas")
//**
//**  Autor : Georg  Hahn
//**  Datum :  14.11.96 / 10.07.01 / 01.04.03 / 30.08.06 / .... / 10.11.09 / 08.01.10 / 13.06.14 / 10.08.15
//**
//**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit HartDataConstants;

interface

uses DataTypes, HartDataTypes;

const

// The below table consists the "ManufacturersDeviceType" numbers assigned to the specified devices
// The OEM products (MTL5874 and MTL5875) branded with PhoenixContact are not loger listed
// The already assigned product MTL4575B with ID = 10 is terminated and therefore not longer listed
  CDeviceIdentifiers : array [dtNone..dt1003] of byte = (
    0,   // dtNone
    2,   // dt507x
    3,   // dtx073
    6,   // dt5074XK
    7,   // dt4073XK
    8,   // dt4075
    9,   // dt4575
    13,  // dt4573
    11,  // dt4576
    12,  // dt4532
    14,  // dt4582B
    7);  // dt1003

  CDeviceShortNames  : array [dtNone..dt1003] of string[20] = (
    'no device selected',  // dtNone
    'MTL5073 / MTL5074',   // dt507x
    'MTL3073 / MTL4073',   // dtx073
    'MTL5074XK',           // dt5074XK
    'MTL4073XK',           // dt4073XK
    'MTL4075',             // dt4075
    'MTL4575 / MTL5575',   // dtx575
    'MTL4573 / MTL5573',   // dtx573
    'MTL4576 / MTL5576',   // dtx576
    'MTL4532 / MTL5532',   // dtx532
    'MTL4582B / MTL5582B',  // dtc582B
    'E01003');             // dt1003

  CTemperatureConverter:      TDevicesSet = [dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx575,dtx573,dtx576];
  CFrequencyConverter:        TDevicesSet = [dtx532,dt1003];
  CSingleChannel:             TDevicesSet = [dt507x,dtx073,dt5074XK,dt4073XK,dt4075,dtx575,dtx573,dtx532,dt1003];
  CDualChannel:               TDevicesSet = [dtx576];
  CWithEBDDetection:          TDevicesSet = [dt4075,dtx575,dtx573,dtx576];
  CWithShortCircuitDetection: TDevicesSet = [dtx573,dtx576];

// Some constants from HART Communication Foundation Document HCF_SPEC-181

  CCommandResponseStatus  = $80; // Command or Communication Response
  CFieldDeviceMalfunction = $80; // Device Status Malfunction Indication for

  CHart_Manufacturer_MTL  = 64; // Table 8
  CHart_Celsius           = 32; // Table 2
  CHart_Fahrenheit        = 33;
  CHart_Rankine           = 34;
  CHart_Kelvin            = 35;
  CHart_millivolt         = 36;
  CHart_Ohm               = 37;
  CHart_Herz              = 38;
  CHart_milliamper        = 39;
  CHart_Second            = 51;
  CHart_Percent           = 57;

// Some common constant values used in MTL converters

  CHart_Alarm_Off         = 3;  // Commands 128 and 129
  CHart_Alarm_Upscale     = 2;
  CHart_Alarm_Downscale   = 1;

  CHart_Option_Off        = 0;  // Default On/Off assignment
  CHart_Option_On         = 1;

  CHart_x073_Lin_Off      = 1;  // Linearisation = Transfer Function on MTLx073 etc.
  CHart_x073_Lin_On       = 0;

  CHart_Filter_50Hz       = 1;  // Commands 128/129, parameter 2
  CHart_Filter_60Hz       = 2;

  CHart_x575_Slow         = 0;  // 500ms
  CHart_x575_Medium       = 1;  // 200ms
  CHart_x575_Fast         = 2;  // 100ms

  CHart_x575_NE43_Off     = 0;   // Command 209/210 and 128/129, parameter 6
  CHart_x575_NE43_On      = 1;

  CHart_x576_NE43_Off     = 1;   // Command 161/162, parameter 6
  CHart_x576_NE43_On      = 0;

  CHart_FullScale_Off     = 0;   // Command 146/147
  CHart_FullScale_On      = 1;

// Default HART complex data records

  CDef_UID_x073: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 3;  // MTL3073 / MTL4073
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 40; // 1
    HardwareRevision        : 64; // 56=7.0, 64=8.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_x073XK: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 7;  // MTL4073XK
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 41;
    HardwareRevision        : 64; // 56=7.0, 64=8.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_x075: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 8;  // MTL4075
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 1;
    HardwareRevision        : 8;  // 8=1.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_x575: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 9;  // MTL4575 / MTL5575
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 5;
    HardwareRevision        : 40; // 40=5.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_x576: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 11; // MTL4576 / MTL5576
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 4;
    HardwareRevision        : 24; // 24=3.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_x532: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 12; // MTL4532 / MTL5532
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 1;
    HardwareRevision        : 8;  // 8=1.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_1003: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 7;
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 4;
    HardwareRevision        : 11; // 2.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_x573: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 13; // MTL4573 / MTL5573
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 1;
    HardwareRevision        : 8;  // 8=1.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_UID_x582B: THDeviceIdentifier = (
    DeviceTypeCode254       : 254;
    ManufacturerIdentCode   : CHart_Manufacturer_MTL;
    ManufacturersDeviceType : 14; // MTL4582B / MTL5582B
    NumberOfPreambles       : 0;
    RevisionLevelOfUCD      : 0;
    RevisionLevelOfTSD      : 0;
    SoftwareRevision        : 3;
    HardwareRevision        : 16;  // 16=2.0
    Flags                   : 0;
    DeviceIdentNumber       : (0,0,0));

  CDef_TDD: THTagDescriptorDate = (
    TagName    : '        ';
    Descriptor : '                ';
    Day        : 3;
    Month      : 3;
    Year       : 114);  // this value is AC - 1900 (e.g. 2007 - 1900 = 107)

  CDef_PVO_x073: THPrimaryVariableOutput = (
    PVAlarm             : 0;
    TransferFunction    : CHart_x073_Lin_On;
    PVUnits             : CHart_Celsius;
    Point20mA           : 250.0;
    Point4mA            : 0.0;
    DampingTimeConstant : 0.0);

  CDef_PVO_x532: THPrimaryVariableOutput = (
    PVAlarm             : 0;
    TransferFunction    : CHart_x073_Lin_On;
    PVUnits             : CHart_Herz;
    Point20mA           : 1000.0;
    Point4mA            : 0.0;
    DampingTimeConstant : 0.0);

  CDef_x073_THC_Set: THx073ParameterSet = (
    OpenCircuitAlarmLevel         : CHart_Alarm_Upscale;
    TransmitterFailureAlarmLevel  : CHart_Alarm_Downscale;
    FilterFrequencySelector       : CHart_Filter_50Hz;
    InputType                     : 3;  // THC Typ K = 3
    SmoothingTimeConstant         : 0;  // 0 sec
    ColdJunctionCompensationOnOff : CHart_Option_On;
    ColdJunctionFailureAlarmLevel : CHart_Alarm_Downscale);

  CDef_x073_RTD_Set: THx073ParameterSet = (
    OpenCircuitAlarmLevel         : CHart_Alarm_Upscale;
    TransmitterFailureAlarmLevel  : CHart_Alarm_Downscale;
    FilterFrequencySelector       : CHart_Filter_50Hz;
    InputType                     : 8;  // RTD Type Pt100 2W = 8
    SmoothingTimeConstant         : 0;  // 0 sec
    ColdJunctionCompensationOnOff : CHart_Option_Off;
    ColdJunctionFailureAlarmLevel : CHart_Alarm_Off);

  CDef_x075_Trip : THx075TripParameters = (
    TripValue          : 50.0;
    Hysteresis         : 1.0;
    TripOutputFunction : CHart_Option_Off);                 // no Trip function

  CDef_x576_InPar : THInputParameters_x576 = (
    Channel             : 0;
    SensorType          : 3;  // THC Typ K = 3
    PVUnits             : CHart_Celsius;
    Linearisation       : CHart_x073_Lin_On;
    CJC_Option          : CHart_Option_On;
    Point4mA            : 0.0;
    Point20mA           : 250.0;
    CJC_UserValue       : 20.0);

  CDef_x576_OutPar : THOutputParameters_x576 = (
    Channel                      : 0;
    OpenCircuitAlarmLevel        : CHart_Alarm_Off;
    ShortCircuitAlarmLevel       : CHart_Alarm_Off;
    TransmitterFailureAlarmLevel : CHart_Alarm_Off;
    ColdJunctionFailureAlarmLevel: CHart_Alarm_Off;
    NAMUR_MTL                    : CHart_x576_NE43_Off;
    DampingTimeConstant          : 0.0;
    SmoothingTimeConstant        : 2.0; // 2 sec
    EBD_Option                   : CHart_Option_Off;
    TripLimit                    : 50.0;
    TripHysteresis               : 1.0);

  CDef_x576_Common : THCommonParameters_x576 = (
    Response_Time               : CHart_x575_Slow;
    FilterFrequencySelector     : CHart_Filter_50Hz;
    InputOption                 : 0); // two inputs and two outputs

// Bit pattern used in the Frequency Converters MTL4532 and E01003

// Digital numeric values

  CHart_Adjust_Off    = CHart_Option_Off; // refer to the firmware E01003 and MTL4532
  CHart_Adjust_On     = 4;
  CHart_Adjust_Soft   = CHart_Adjust_On;
  CHart_Adjust_Hard   = 9;                // this option is available only on the E01003

// Digital option pattern
  CP_Blank_Pattern    = $00;

// Bit pattern used for the Input configuration
  CP_InputVandI       = CP_Blank_Pattern;
  CP_InputProxy1      = $01;  // Input Source assigned to Proxy
  CP_InputProxy2      = $02;
  CP_PulseToInput     = $04;  // Pulse Output assigned direct to digital input
  CP_PulseToLFD       = $08;
  CP_PulseToInhibit   = $10;  // Inhibit = Proxy2
  CP_AlarmToLFD       = $20;  // Alarm Output assigned to LFD status
  CP_AlarmToInput     = $40;  // Alarm Output assigned to Voltage and Current Input
  CP_AlarmToInhibit   = $80;  // Inhibit = Proxy2
  CP_InputSelector    = CP_InputProxy1 or CP_InputProxy2;
  CP_OutputSelector   = CP_PulseToInput or CP_AlarmToLFD or CP_AlarmToInput;

// Bit pattern used for the signal status configuration (invert inputs)
  CP_StatusLFD1       = $01; // LFD signal from Proxy1
  CP_StatusLFD2       = $02; // LFD signal from Proxy2
  CP_StatusVandI      = $04; // Status signal on V and I
  CP_StatusProxy1     = $08;
  CP_StatusProxy2     = $10;

// Bit pattern used for the Pulse Divider

  CP_Divider_Enabled  = $01; // available only on Pulse Output
  CP_Divider_Toggle   = $02; // only when enabled

// Bit pattern used for the Alarm Output

  CP_TripAOnALarm     = $01;
  CP_TripAInverted    = $02;
  CP_TripAInfo        = CP_TripAOnALarm or CP_TripAInverted;
  CP_TripBOnPulse     = $04; // reserved
  CP_TripBInverted    = $08;
  CP_TripBInfo        = CP_TripBOnPulse or CP_TripBInverted;
  CP_TripABOnAlarm    = $10;
  CP_TripABOnPulse    = $20;
  CP_TripABInverted   = $40;
  CP_TripABInfo       = CP_TripABOnAlarm or CP_TripABOnPulse or CP_TripABInverted;
  CP_InhibitEnabled   = $80;

// Bit pattern used for E01003

  CP3_Off_Pattern     = $00;

  CP3_LFDPresent      = $01;
  CP3_RelayPresent    = $02;

  CP3_LeakFlowEnabled = $04;
  CP3_Divider_Enabled = $04;

  CP3_TripB_Pattern   = $01; // when enabled
  CP3_Trip_OnRelay    = $02; // when enabled
  CP3_TripOutputInfo  = CP3_TripB_Pattern or CP3_Trip_OnRelay;
  CP3_Trip_Inverted   = $04; // when enabled
  CP3_Trip_Enabled    = $08;
  CP3_TripModeInfo    = CP3_Trip_Enabled or CP3_Trip_Inverted;
  CP3_TripReconfig    = $10;

  CP3_Input_PA2IC1    = $01; // $00, $02 and $03 not used
  CP3_LFD_OnRelay     = $04; // when enabled
  CP3_LFD_Enabled     = $08;
  CP3_LFD_ForceTx     = $10; // when enabled

  CDef_AdjustOutput: THAdjustOutputParameters = (
    AdjMode         : CHart_Adjust_Off;     // no Adjustement
    AdjUnits        : CHart_Percent;
    OffsetValue     : 2.0;
    GainValue       : 2.0);

  CDef_Divider: THDividerParameters = (
    FunctionSelect : CHart_Option_Off;      // no Divider
    Divider        : 25.0;
    Duration       : 10.0);

  CDef_x532_LeakOff: THLeakFlowOffParameters_x532 = (
    FunctionSelect    : CHart_Option_Off;    // no LeakFlow
    LFUnits           : CHart_Percent;
    LeakFlowOffValue  : 5.0;  // in process units or percent
    LeakFlowOffCurrent: 4.0); // always in mA

  CDef_1003_LeakOff: THLeakFlowOffParameters_1003 = (
    FunctionSelect    : CHart_Option_Off;    // no LeakFlow
    LeakFlowOffValue  : 5.0;  // in process units or percent
    LeakFlowOffCurrent: 4.0); // always in mA

  CDef_x532_Trip: THTripParameters_x532 = (
    FunctionSelect    : CHart_Option_Off;
    TPAUnits          : CHart_Percent;
    TripAValue        : 20.0;
    AHysteresis       : 1.0;
    TPBUnits          : CHart_Percent;
    TripBValue        : 80.0;
    BHysteresis       : 1.0);

  CDef_1003_TripA: THTripParameters_1003 = (
    FunctionSelect  : CP_Blank_Pattern; // CP_Low_Pattern or CP_OutCh1_Pattern or CP_TripA_Pattern;
    TPUnits         : CHart_Percent;
    TripValue       : 20.0;
    Hysteresis      : 1.0);

  CDef_1003_TripB: THTripParameters_1003 = (
    FunctionSelect  : CP_Blank_Pattern; // CP_High_Pattern or CP_OutCh1_Pattern or CP_TripB_Pattern;
    TPUnits         : CHart_Percent;
    TripValue       : 80.0;
    Hysteresis      : 1.0);

  CDef_1003_Input: THLFDInputParameters_1003 = (
    FunctionSelect  : CP_Blank_Pattern);

  CDef_STS: THTransmitterStatus = (
    ComInfo: 0; DevInfo: 0 );

  CDef_FAN: THFinalAssemblyNumber = (
    Number : (0,0,0));

//*************************************************************************************************

  CDef_DynamicVariables: THDynamicVariables = (
    LoopCurrent             : 12.0;
    PrimaryUnit             : CHart_Percent;
    PrimaryVariable         : 100.0;
    SecondaryUnit           : CHart_Percent;
    SecondaryVariable       : 0.0);

  CDef_PrimaryVariables: THPrimaryVariables = (
    ScaledInput             : 120.0;
    SmoothedInput           : 120.0;
    ColdJunctionResistance  : 30.0;
    ColdJunctionTemperature : 40.0;
    ColdJunctionCompensation: 50.0;
    ProcessVariable         : 60.0;
    SensorFlags             : 0);

  CDef_RAWInputOutputValues: THRAWInputOutputValues = (
    DACValue      : 0;
    MainInput     : 0;
    AuxInput      : 0;
    MainExcitation: 0;
    AuxExcitation : 0;
    LeadOffTest1  : 0;
    LeadOffTest2  : 0;
    LeadOffTest5  : 0;
    LeadOffTest6  : 0;
    SensorFlags   : 0);

  CDef_CalibrationData: THCalibrationData = (
    RrefCal     : 0.0;
    Rin5Cal     : 0.0;
    Rin6Cal     : 0.0;
    VrefCal     : 0.0;
    CalFlag     : 0;
    SensorFlags : 0);

  CDef_SecondaryVariables: THSecondaryVariables = (
    ProcessVariableScaledInput: 123.4;
    DampedProcessVariable     : 123.4;
    LoopCurrent               : 12.0;
    NoiseFloor                : 0.1;
    SensorFlags               : 0);

  CDef_PrimaryVariableRange_x582B: THPrimaryVariableRange = (
    PrimaryUnit             : CHart_Ohm;
    UpperLimit              : 400.0;
    LowerLimit              : 10.0);

implementation

end.
