//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
//**
//**	ORDER DESCRIPTION - CONFIGURATION SOFTWARE WITH HART COMMUNICATION (PCS45)
//**	IMPORTEXPORTPROCS.PAS
//**
//**	Import from and Export to Text Files procedures for PCS45
//**
//**	Autor : Georg  Hahn
//**	Datum :  xx.yy.96 / 11.08.05 / 23.08.06 / 24.10.06 / 29.03.07 / 02.04.14
//**
//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
unit ImportExportProcs;

interface
  uses SysUtils, Classes, ArraysAndGrids, HartDataTypes, HartModules;

  function ImportVeryOldConfigurationFile(SourceFile: TFileName): TMemoryStream;
  function ImportOldConfigurationFile(SourceFile: TFileName): TMemoryStream;
  function ImportConfigurationFile(SourceFile: TFileName): TMemoryStream;

  function ImportTextConfigurationFile(SourceFile: TFileName; SepChar: char): TMemoryStream;

  function ImportExcelConfigurationFile(SourceFile: TFileName): TMemoryStream;

  function ImportUserSensorConfiguration(SourceFile: TFileName; var UserSensorData: THUserSensorData_x576): boolean;

  function ExportConfigurationToGrid(Source: TBasicHartModule): TStringDynGrid;

  procedure PrintStringDynGrid(Source: TStringDynGrid);

implementation

uses Types, DateUtils, LnCRCFileStream, ConversionUtils, StringUtils,
     HartDataConstants, DataTypes, DataConstants, DataInstances,
     LabelNumbers, ConversionProcs, HelpContextNumbers,
     Dialogs, Printers;

type TImpexVersion = (imBefore401, imFrom401);

//***********************************************************************************************//
//**
//** Legacy configuration file import (PCC73 DOS 1.4 and 1.5.1)
//**
//***********************************************************************************************//

// This is conversion procedure between DOS real format (6 bytes)
// and Windows real format (8 bytes)
function Byte6StreamToReal48(Source: Pbyte): real;
var InterResult: real48;
    LTarget: array [0..5] of byte absolute InterResult;
    I: byte;
begin
{$REALCOMPATIBILITY ON}
  for I:=0 to 5 do
      begin
      LTarget[I]:=Source^;
      if I<5 then Inc(Source);
      end;
{$REALCOMPATIBILITY OFF}
  Result:=InterResult;
end;

// This is legacy configuration data file import procedure
// used in the DOS PCC73 version up to 1.4 (file extention *.mtl)
// Note: in the "DOS world" real data were saved as 6 bytes stream
// while in the "Windows world" this data will be saved as 8 bytes.
function ImportVeryOldConfigurationFile(SourceFile: TFileName): TMemoryStream;
var lFileStream: TLnCRCFileStream;
    lMemoryStream: TMemoryStream;
    lDataStream: TDataStream;
    lDx073: THx073;
    I: byte;
begin
  lMemoryStream:=TMemoryStream.Create;
  lMemoryStream.Clear;

  lFileStream:=TLnCRCFileStream.Create(SourceFile,fmOpenRead);

  if (lFileStream.Size=96) then
     begin
     lFileStream.Read(lDataStream,lFileStream.Size);

     // READ:0 - components
     lDx073.MDD.UID.DeviceTypeCode254       := lDataStream[2];
     lDx073.MDD.UID.ManufacturerIdentCode   := lDataStream[3];
     lDx073.MDD.UID.ManufacturersDeviceType := lDataStream[4];
     lDx073.MDD.UID.NumberOfPreambles       := lDataStream[5];
     lDx073.MDD.UID.RevisionLevelOfUCD      := lDataStream[6];
     lDx073.MDD.UID.RevisionLevelOfTSD      := lDataStream[7];
     lDx073.MDD.UID.SoftwareRevision        := lDataStream[8];
     lDx073.MDD.UID.HardwareRevision        := lDataStream[9];
     lDx073.MDD.UID.Flags                   := lDataStream[10];
     lDx073.MDD.UID.DeviceIdentNumber[1]    := lDataStream[11];
     lDx073.MDD.UID.DeviceIdentNumber[2]    := lDataStream[12];
     lDx073.MDD.UID.DeviceIdentNumber[3]    := lDataStream[13];

     // + 20 steps for READ:3 - components

     // READ:13 - components
     for I:=1 to 8 do lDx073.MDD.TDD.TagName[I]:=char(lDataStream[34+I]);
     SetLength(lDx073.MDD.TDD.TagName,lDataStream[34]);
     while System.Length(lDx073.MDD.TDD.TagName)<8 do lDx073.MDD.TDD.TagName:=lDx073.MDD.TDD.TagName+' ';

     for I:=1 to 16 do lDx073.MDD.TDD.Descriptor[I]:=char(lDataStream[43+I]);
     SetLength(lDx073.MDD.TDD.Descriptor,lDataStream[43]);
     while System.Length(lDx073.MDD.TDD.Descriptor)<16 do lDx073.MDD.TDD.Descriptor:=lDx073.MDD.TDD.Descriptor+' ';

     lDx073.MDD.TDD.Day                     := lDataStream[60];
     lDx073.MDD.TDD.Month                   := lDataStream[61];
     lDx073.MDD.TDD.Year                    := lDataStream[62];

     // READ:15 - components
     lDx073.MDD.PVO.PVAlarm                 := lDataStream[63];
     lDx073.MDD.PVO.TransferFunction        := lDataStream[64];
     lDx073.MDD.PVO.PVUnits                 := lDataStream[65];
     lDx073.MDD.PVO.Point20mA:=Byte6StreamToReal48(@lDataStream[66]);
     lDx073.MDD.PVO.Point4mA:=Byte6StreamToReal48(@lDataStream[72]);
     lDx073.MDD.PVO.DampingTimeConstant:=Byte6StreamToReal48(@lDataStream[78]);

     // READ:128 - components
     lDx073.x073Set.OpenCircuitAlarmLevel         := lDataStream[84];
     lDx073.x073Set.TransmitterFailureAlarmLevel  := lDataStream[85];
     lDx073.x073Set.FilterFrequencySelector       := lDataStream[86];
     lDx073.x073Set.InputType                     := lDataStream[87];
     lDx073.x073Set.SmoothingTimeConstant:=trunc(Byte6StreamToReal48(@lDataStream[88]));
     lDx073.x073Set.ColdJunctionCompensationOnOff := lDataStream[94];
     lDx073.x073Set.ColdJunctionFailureAlarmLevel := lDataStream[95];

     // very old data (from DOS PCC73), therefore the device can be only
     // ID=2 (MTL5073 or MTL5074) or ID=3 (MTL3073 or MTL4073)
     if (lDx073.MDD.UID.ManufacturerIdentCode=64) and
        (lDx073.MDD.UID.ManufacturersDeviceType in [2..3]) then
        begin
        lDx073.Parity:=GetDataStreamParityByte(lDx073,SizeOf(THx073)-4);
        lMemoryStream.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
        lMemoryStream.Write(lDx073,SizeOf(THx073));
        end;
     end;

  lFileStream.Free;

  Result:=lMemoryStream;
end;

// This is legacy configuration data file import procedure
// used in the DOS PCC73 version 1.5.1 (file extention *.d73)
// Note: in the "DOS world" real data were saved as 6 bytes stream
// while in the "Windows world" this data will be saved as 8 bytes.
function ImportOldConfigurationFile(SourceFile: TFileName): TMemoryStream;
var lFileStream: TLnCRCFileStream;
    lMemoryStream: TMemoryStream;
    lDataStream: TDataStream;
    HeaderString: string;
    lDataSize: word;
    lDx073: THx073;
    I: byte;
begin
  lMemoryStream:=TMemoryStream.Create;
  lMemoryStream.Clear;

  lFileStream:=TLnCRCFileStream.Create(SourceFile,fmOpenRead);
  if (lFileStream.Size=105) then
     begin
     HeaderString:=lFileStream.ReadStringCount(System.Length(COld_Config_Header));

     lDataSize:=lFileStream.Size-System.Length(COld_Config_Header);
     if (HeaderString=COld_Config_Header) and
        (lFileStream.Seek(System.Length(COld_Config_Header),soFromBeginning)=System.Length(COld_Config_Header)) and
        (lFileStream.Read(lDataStream,lDataSize)=lDataSize) and
        (lDataStream[lDataSize-1]=GetDataStreamParityByte(lDataStream,lDataSize)-1) then
        begin

        // READ:0 - components
        lDx073.MDD.UID.DeviceTypeCode254       := lDataStream[2];
        lDx073.MDD.UID.ManufacturerIdentCode   := lDataStream[3];
        lDx073.MDD.UID.ManufacturersDeviceType := lDataStream[4];
        lDx073.MDD.UID.NumberOfPreambles       := lDataStream[5];
        lDx073.MDD.UID.RevisionLevelOfUCD      := lDataStream[6];
        lDx073.MDD.UID.RevisionLevelOfTSD      := lDataStream[7];
        lDx073.MDD.UID.SoftwareRevision        := lDataStream[8];
        lDx073.MDD.UID.HardwareRevision        := lDataStream[9];
        lDx073.MDD.UID.Flags                   := lDataStream[10];
        lDx073.MDD.UID.DeviceIdentNumber[1]    := lDataStream[11];
        lDx073.MDD.UID.DeviceIdentNumber[2]    := lDataStream[12];
        lDx073.MDD.UID.DeviceIdentNumber[3]    := lDataStream[13];

        // READ:13 - components
        for I:=1 to 8 do lDx073.MDD.TDD.TagName[I]:=char(lDataStream[14+I]);
        SetLength(lDx073.MDD.TDD.TagName,lDataStream[14]);
        while System.Length(lDx073.MDD.TDD.TagName)<8 do lDx073.MDD.TDD.TagName:=lDx073.MDD.TDD.TagName+' ';

        for I:=1 to 16 do lDx073.MDD.TDD.Descriptor[I]:=char(lDataStream[23+I]);
        SetLength(lDx073.MDD.TDD.Descriptor,lDataStream[23]);
        while System.Length(lDx073.MDD.TDD.Descriptor)<16 do lDx073.MDD.TDD.Descriptor:=lDx073.MDD.TDD.Descriptor+' ';

        lDx073.MDD.TDD.Day                     := lDataStream[40];
        lDx073.MDD.TDD.Month                   := lDataStream[41];
        lDx073.MDD.TDD.Year                    := lDataStream[42];

        // READ:15 - components
        lDx073.MDD.PVO.PVAlarm                 := lDataStream[43];
        lDx073.MDD.PVO.TransferFunction        := lDataStream[44];
        lDx073.MDD.PVO.PVUnits                 := lDataStream[45];
        lDx073.MDD.PVO.Point20mA:=Byte6StreamToReal48(@lDataStream[46]);
        lDx073.MDD.PVO.Point4mA:=Byte6StreamToReal48(@lDataStream[52]);
        lDx073.MDD.PVO.DampingTimeConstant:=Byte6StreamToReal48(@lDataStream[58]);

     // READ:128 - components
        lDx073.x073Set.OpenCircuitAlarmLevel         := lDataStream[64];
        lDx073.x073Set.TransmitterFailureAlarmLevel  := lDataStream[65];
        lDx073.x073Set.FilterFrequencySelector       := lDataStream[66];
        lDx073.x073Set.InputType                     := lDataStream[67];
        lDx073.x073Set.SmoothingTimeConstant         := lDataStream[69]; // + (lDataStream[68] * 256); 
        lDx073.x073Set.ColdJunctionCompensationOnOff := lDataStream[70];
        lDx073.x073Set.ColdJunctionFailureAlarmLevel := lDataStream[71];

        // old data (from Win 3.1 PCC73), therefore the device can be only
        // ID=2 (MTL5073 or MTL5074) or ID=3 (MTL3073 or MTL4073)
        if (lDx073.MDD.UID.ManufacturerIdentCode=64) and
           (lDx073.MDD.UID.ManufacturersDeviceType in [2..3]) then
           begin
           lDx073.Parity:=GetDataStreamParityByte(lDx073,SizeOf(THx073)-4);
           lMemoryStream.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
           lMemoryStream.Write(lDx073,SizeOf(THx073));
           end;
        end;
     end;

  lFileStream.Free;

  Result:=lMemoryStream;
end;

//***********************************************************************************************//
//**
//** Text File Configuration Import
//**
//***********************************************************************************************//

function ImportConfigurationFile(SourceFile: TFileName): TMemoryStream;
var lFileStream: TLnCRCFileStream;
    lMemoryStream: TMemoryStream;
    NDI: THDeviceIdentifier;
    HeaderString: string;
    lDT: TDeviceType;
begin
  lMemoryStream:=TMemoryStream.Create;
  lMemoryStream.Clear;

  lFileStream:=TLnCRCFileStream.Create(SourceFile,fmOpenRead);
  HeaderString:=lFileStream.ReadStringCount(System.Length(CNew_Config_Header));

  if (HeaderString=CNew_Config_Header) and
     (lFileStream.Seek(System.Length(CNew_Config_Header),soFromBeginning)=System.Length(CNew_Config_Header)) and
     (lFileStream.Read(NDI,SizeOf(THDeviceIdentifier))=SizeOf(THDeviceIdentifier)) then
     lDT:=IdentToType(NDI)
  else lDT:=dtNone;
  lFileStream.Free;

  if (lDT<>dtNone) then
     lMemoryStream.LoadFromFile(SourceFile);

  Result:=lMemoryStream;
end;

//******************************************************************//

function ImporTHDeviceIdentifier(Source: TStringDynGrid; Index: integer; var Target: THDeviceIdentifier): boolean;
begin
  // All default values like ManufacturerIdentCode are already set
  Target.ManufacturersDeviceType:=DeviceTypeToIdent(GetDevTypeFromString(Source[Index,1]));
  if (Target.ManufacturersDeviceType<>0) and
     (EncodeByteValueFromString (Source[Index+2,1],Target.SoftwareRevision)) and
     (EncodeHardwareRevisionFromString (Source[Index+3,1],Target.HardwareRevision)) then
     Result:=true
  else Result:=false;
end;

function ImporTHTagDescriptorDate(Source: TStringDynGrid; Index: integer; var Target: THTagDescriptorDate): boolean;
var SimpleDate: TSimpleDate;
    S: string;
begin
  Target.TagName:=UpperCase(Source[Index+0,1]);
  while System.Length(Target.TagName)<8 do Target.TagName:=Target.TagName+' ';

  Target.Descriptor:=UpperCase(Source[Index+1,1]);
  while System.Length(Target.Descriptor)<16 do Target.Descriptor:=Target.Descriptor+' ';

  S:=UpperCase(Source[Index+2,1]);
  if (System.Length(S)=10) then SimpleDate:=EncodeLongDateString(S)
  else SimpleDate:=EncodeShortDateString(S);

  if IsValidDate(SimpleDate.Y,SimpleDate.M,SimpleDate.D) then
     begin
     Target.Day:=SimpleDate.D;
     Target.Month:=SimpleDate.M;
     Target.Year:=SimpleDate.Y;
     Result:=true;
     end
  else Result:=false;
end;

function ImportChannelInputParameter(Source: TStringDynGrid; Index: integer; var Target: THInputParameters_x576): boolean;
var lCorruptedData: boolean;
begin

  // Note: encode PVUnits and Linearisation before InputType
  lCorruptedData:=not(EncodePVUnitsFromString(Source[Index+1,1], Target.PVUnits));
  if not(lCorruptedData) then
     lCorruptedData:=not(Encodex073LinearisationFromString(Source[Index+2,1], Target.Linearisation));

  // Note: when PVUnits and Linearisation were encoded correctly then use these information also for proper InputType encoding
  if not(lCorruptedData) then
     begin
     if (System.Length(Source[Index])>2) then
        lCorruptedData:=not(Encodex576InputTypeFromString(Source[Index,1],Source[Index,2],Source[Index+1,1],Source[Index+2,1],dtx576,Target.SensorType))
     else lCorruptedData:=not(Encodex576InputTypeFromString(Source[Index,1],'','','',dtx576,Target.SensorType));
     end;

  if not(lCorruptedData) then
     begin
     lCorruptedData:=not(Encodex576CJCFromString(Source[Index+3,1],Target.CJC_Option));
     if not(lCorruptedData) and
       (System.Length(Source[Index+3])>2) and
       (Target.CJC_Option=2) then // User CJC value
       lCorruptedData:=not(EncodeSingleValueFromString(Source[Index+3,2],Target.CJC_UserValue));
     end;

  if not(lCorruptedData) then
     lCorruptedData:=not(EncodeSingleValueFromString(Source[Index+4,1],Target.Point4mA));
  if not(lCorruptedData) then
     lCorruptedData:=not(EncodeSingleValueFromString(Source[Index+5,1],Target.Point20mA));

  Result:=not(lCorruptedData);
end;


function Importx073Configuration(Source: TStringDynGrid; var Target: TMemoryStream): boolean;
var CorruptedData: boolean;
    ImportData: THx073;
    lDT: TDeviceType;
begin
  ImportData.MDD.UID:=CDef_UID_x073;
  ImportData.MDD.TDD:=CDef_TDD;
  ImportData.MDD.PVO:=CDef_PVO_x073;
  ImportData.x073Set:=CDef_x073_THC_Set;

  lDT:=GetDevTypeFromString(Source[0,1]);

  if (lDT<>dtNone) and not((System.Length(Source)-1)<18) then
     begin
     CorruptedData:=not(ImporTHDeviceIdentifier(Source,0,ImportData.MDD.UID));

     if not(CorruptedData) then
        CorruptedData:=not(ImporTHTagDescriptorDate(Source,4,ImportData.MDD.TDD));

     // Note: encode PVUnits and Linearisation before InputType
     if not(CorruptedData) then
        CorruptedData:=not(EncodePVUnitsFromString(Source[8,1],ImportData.MDD.PVO.PVUnits));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex073LinearisationFromString(Source[9,1],ImportData.MDD.PVO.TransferFunction));

     // Note: when PVUnits and Linearisation were encoded correctly then use these information also for proper InputType encoding
     if not(CorruptedData) then
        begin
        if (System.Length(Source[7])>2) then
           CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],Source[7,2],Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType))
        else CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],'',Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType));
        end;

     if not(CorruptedData) then
        CorruptedData:=not(Encodex073CJCFromString(Source[10,1],ImportData.x073Set.ColdJunctionCompensationOnOff));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[11,1],ImportData.MDD.PVO.Point4mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[12,1],ImportData.MDD.PVO.Point20mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[13,1],ImportData.MDD.PVO.DampingTimeConstant));

     if not(CorruptedData) then
        CorruptedData:=not(EncodeWordValueFromString (Source[14,1],ImportData.x073Set.SmoothingTimeConstant));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex073FilterFreqFromString(Source[15,1],ImportData.x073Set.FilterFrequencySelector));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[16,1],ImportData.x073Set.OpenCircuitAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[17,1],ImportData.x073Set.ColdJunctionFailureAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[18,1],ImportData.x073Set.TransmitterFailureAlarmLevel));

     if not(CorruptedData) then
        begin
        Target.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
        ImportData.Parity:=GetDataStreamParityByte(ImportData,SizeOf(THx073)-4);
        Target.Write(ImportData,SizeOf(THx073));

        Result:=true;
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Importx075Configuration(Source: TStringDynGrid; var Target: TMemoryStream): boolean;
var CorruptedData: boolean;
    ImportData: THx075;
    lDT: TDeviceType;
begin
  ImportData.MDD.UID:=CDef_UID_x075;
  ImportData.MDD.TDD:=CDef_TDD;
  ImportData.MDD.PVO:=CDef_PVO_x073;
  ImportData.x073Set:=CDef_x073_THC_Set;
  ImportData.x075TPA:=CDef_x075_Trip;

  lDT:=GetDevTypeFromString(Source[0,1]);

  if (lDT<>dtNone) and not((System.Length(Source)-1)<19) then
     begin
     CorruptedData:=not(ImporTHDeviceIdentifier(Source,0,ImportData.MDD.UID));

     if not(CorruptedData) then
        CorruptedData:=not(ImporTHTagDescriptorDate(Source,4,ImportData.MDD.TDD));

     // Note: encode PVUnits and Linearisation before InputType
     if not(CorruptedData) then
        CorruptedData:=not(EncodePVUnitsFromString(Source[8,1],ImportData.MDD.PVO.PVUnits));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex073LinearisationFromString(Source[9,1],ImportData.MDD.PVO.TransferFunction));

     // Note: when PVUnits and Linearisation were encoded correctly then use these information also for proper InputType encoding
     if not(CorruptedData) then
        begin
        if (System.Length(Source[7])>2) then
           CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],Source[7,2],Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType))
        else CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],'',Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType));
        end;

     if not(CorruptedData) then
        CorruptedData:=not(Encodex073CJCFromString(Source[10,1],ImportData.x073Set.ColdJunctionCompensationOnOff));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[11,1],ImportData.MDD.PVO.Point4mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[12,1],ImportData.MDD.PVO.Point20mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[13,1],ImportData.MDD.PVO.DampingTimeConstant));

     if not(CorruptedData) then
        CorruptedData:=not(EncodeWordValueFromString (Source[14,1],ImportData.x073Set.SmoothingTimeConstant));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex073FilterFreqFromString(Source[15,1],ImportData.x073Set.FilterFrequencySelector));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[16,1],ImportData.x073Set.OpenCircuitAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[17,1],ImportData.x073Set.ColdJunctionFailureAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[18,1],ImportData.x073Set.TransmitterFailureAlarmLevel));

     if not(CorruptedData) then
        begin
        CorruptedData:=not(Encodex075TripParameterFromString (Source[19,1],ImportData.x075TPA));
        if not(CorruptedData) and // if CorruptedData or not ttTrip then skip
           (x075TripTypeFromHART(ImportData.x075TPA.TripOutputFunction) = ttTrip) then
           // CorruptedData:=not(EncodeHysteresisValueFromString (Source[20,1],ImportData.x075TPA.Hysteresis));
           CorruptedData:=not(EncodeSingleValueFromString (Source[20,1],ImportData.x075TPA.Hysteresis));
        end;

     if not(CorruptedData) then
        begin
        Target.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
        ImportData.Parity:=GetDataStreamParityByte(ImportData,SizeOf(THx075)-4);
        Target.Write(ImportData,SizeOf(THx075));

        Result:=true;
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Importx575Configuration(Source: TStringDynGrid; var Target: TMemoryStream): boolean;
var ImpexType: TImpexVersion;
    CorruptedData: boolean;
    ImportData: THx575;
    lDT: TDeviceType;
    T: integer;
begin
  ImportData.MDD.UID:=CDef_UID_x575;
  ImportData.MDD.TDD:=CDef_TDD;
  ImportData.MDD.PVO:=CDef_PVO_x073;
  ImportData.x073Set:=CDef_x073_THC_Set;
  ImportData.x075TPA:=CDef_x075_Trip;
  ImportData.Speed:=CHart_x575_Slow;
  ImportData.NAMUR:=CHart_x575_NE43_Off;
  T:=0;

  lDT:=GetDevTypeFromString(Source[0,1]);

  if (lDT<>dtNone) and not((System.Length(Source)-1)<20) then
     begin
     CorruptedData:=not(ImporTHDeviceIdentifier(Source,0,ImportData.MDD.UID));

     if not(CorruptedData) then
        CorruptedData:=not(ImporTHTagDescriptorDate(Source,4,ImportData.MDD.TDD));

     // Note: encode PVUnits and Linearisation before InputType
     if not(CorruptedData) then
        CorruptedData:=not(EncodePVUnitsFromString(Source[8,1],ImportData.MDD.PVO.PVUnits));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex073LinearisationFromString(Source[9,1],ImportData.MDD.PVO.TransferFunction));

     // Note: when PVUnits and Linearisation were encoded correctly then use these information also for proper InputType encoding
     if not(CorruptedData) then
        begin
        if (System.Length(Source[7])>2) then
           CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],Source[7,2],Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType))
        else CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],'',Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType));
        end;

     if not(CorruptedData) then
        CorruptedData:=not(Encodex073CJCFromString(Source[10,1],ImportData.x073Set.ColdJunctionCompensationOnOff));

     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[11,1],ImportData.MDD.PVO.Point4mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[12,1],ImportData.MDD.PVO.Point20mA));

     if not(CorruptedData) then
        begin
        // evaluate if the imported file is before 4.01 or from version 4.01
        // Note: by mistake, there was no NAMUR configuration data before 4.01
        if IsFloatingPointString(Source[13,1],'.') or
           IsFloatingPointString(Source[13,1],',') then
           ImpexType:=imBefore401  // data is floating point and assumed damping time contant
        else ImpexType:=imFrom401; // data is not floating point and assumed NAMUR info

        if (ImpexType=imBefore401) then
           begin
           if not(CorruptedData) then
              CorruptedData:=not(EncodeSingleValueFromString(Source[13,1],ImportData.MDD.PVO.DampingTimeConstant));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeWordValueFromString (Source[14,1],ImportData.x073Set.SmoothingTimeConstant));
           if not(CorruptedData) then
              CorruptedData:=not(Encodex073FilterFreqFromString(Source[15,1],ImportData.x073Set.FilterFrequencySelector));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[16,1],ImportData.x073Set.OpenCircuitAlarmLevel));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[17,1],ImportData.x073Set.ColdJunctionFailureAlarmLevel));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[18,1],ImportData.x073Set.TransmitterFailureAlarmLevel));

           if not(CorruptedData) then
              begin
              CorruptedData:=not(Encodex075TripParameterFromString (Source[19,1],ImportData.x075TPA));
              if not(CorruptedData) and // if CorruptedData or not ttTrip then skip
                 (x075TripTypeFromHART(ImportData.x075TPA.TripOutputFunction) = ttTrip) then
                 begin
                 // CorruptedData:=not(EncodeHysteresisValueFromString (Source[20,1],ImportData.x075TPA.Hysteresis));
                 CorruptedData:=not(EncodeSingleValueFromString (Source[20,1],ImportData.x075TPA.Hysteresis));
                 T:=1;
                 end;
              end;
           if not(CorruptedData) then
              CorruptedData:=not(Encodex575ResponseTimeFromString(Source[20+T,1],ImportData.Speed));
           end
        else // ImpexType=imFrom401
           begin
           if not(CorruptedData) then
              CorruptedData:=not(Encodex575OutputCharacteristicFromString(Source[13,1],ImportData.NAMUR));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeSingleValueFromString(Source[14,1],ImportData.MDD.PVO.DampingTimeConstant));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeWordValueFromString (Source[15,1],ImportData.x073Set.SmoothingTimeConstant));
           if not(CorruptedData) then
              CorruptedData:=not(Encodex575ResponseTimeFromString(Source[16,1],ImportData.Speed));
           if not(CorruptedData) then
              CorruptedData:=not(Encodex073FilterFreqFromString(Source[17,1],ImportData.x073Set.FilterFrequencySelector));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[18,1],ImportData.x073Set.OpenCircuitAlarmLevel));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[19,1],ImportData.x073Set.ColdJunctionFailureAlarmLevel));
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[20,1],ImportData.x073Set.TransmitterFailureAlarmLevel));

           if not(CorruptedData) then
              begin
              CorruptedData:=not(Encodex075TripParameterFromString (Source[21,1],ImportData.x075TPA));
              if not(CorruptedData) and // if CorruptedData or not ttTrip then skip
                 (x075TripTypeFromHART(ImportData.x075TPA.TripOutputFunction) = ttTrip) then
                 begin
                 // CorruptedData:=not(EncodeHysteresisValueFromString (Source[22,1],ImportData.x075TPA.Hysteresis));
                 CorruptedData:=not(EncodeSingleValueFromString (Source[22,1],ImportData.x075TPA.Hysteresis));
                 end;
              end;

           end;
        end;

     if not(CorruptedData) then
        begin
        Target.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
        ImportData.Parity:=GetDataStreamParityByte(ImportData,SizeOf(THx575)-4);
        Target.Write(ImportData,SizeOf(THx575));

        Result:=true;
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Importx573Configuration(Source: TStringDynGrid; var Target: TMemoryStream): boolean;
var CorruptedData: boolean;
    ImportData: THx573;
    lDT: TDeviceType;
begin
  ImportData.MDD.UID:=CDef_UID_x573;
  ImportData.MDD.TDD:=CDef_TDD;
  ImportData.MDD.PVO:=CDef_PVO_x073;
  ImportData.x073Set:=CDef_x073_THC_Set;
  ImportData.x075TPA:=CDef_x075_Trip;
  ImportData.Speed:=CHart_x575_Slow;
  ImportData.NAMUR:=CHart_x575_NE43_Off;
  ImportData.ShortCircuitAlarmLevel:=CHart_Alarm_Off;

  lDT:=GetDevTypeFromString(Source[0,1]);

  if (lDT<>dtNone) and not((System.Length(Source)-1)<20) then
     begin
     CorruptedData:=not(ImporTHDeviceIdentifier(Source,0,ImportData.MDD.UID));

     if not(CorruptedData) then
        CorruptedData:=not(ImporTHTagDescriptorDate(Source,4,ImportData.MDD.TDD));

     // Note: encode PVUnits and Linearisation before InputType
     if not(CorruptedData) then
        CorruptedData:=not(EncodePVUnitsFromString(Source[8,1],ImportData.MDD.PVO.PVUnits));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex073LinearisationFromString(Source[9,1],ImportData.MDD.PVO.TransferFunction));

     // Note: when PVUnits and Linearisation were encoded correctly then use these information also for proper InputType encoding
     if not(CorruptedData) then
        begin
        if (System.Length(Source[7])>2) then
           CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],Source[7,2],Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType))
        else CorruptedData:=not(Encodex073InputTypeFromString(Source[7,1],'',Source[8,1],Source[9,1],lDT,ImportData.x073Set.InputType));
        end;

     if not(CorruptedData) then
        CorruptedData:=not(Encodex073CJCFromString(Source[10,1],ImportData.x073Set.ColdJunctionCompensationOnOff));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[11,1],ImportData.MDD.PVO.Point4mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[12,1],ImportData.MDD.PVO.Point20mA));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex575OutputCharacteristicFromString(Source[13,1],ImportData.NAMUR));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[14,1],ImportData.MDD.PVO.DampingTimeConstant));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeWordValueFromString (Source[15,1],ImportData.x073Set.SmoothingTimeConstant));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex575ResponseTimeFromString(Source[16,1],ImportData.Speed));
      if not(CorruptedData) then
        CorruptedData:=not(Encodex073FilterFreqFromString(Source[17,1],ImportData.x073Set.FilterFrequencySelector));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[18,1],ImportData.x073Set.OpenCircuitAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[19,1],ImportData.x073Set.ColdJunctionFailureAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[20,1],ImportData.x073Set.TransmitterFailureAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeAlarmFromString (Source[21,1],ImportData.ShortCircuitAlarmLevel));
     if not(CorruptedData) then
        CorruptedData:=not(Encodex573EBDFromString(Source[22,1],ImportData.x075TPA.TripOutputFunction));

     if not(CorruptedData) then
        begin
        Target.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
        ImportData.Parity:=GetDataStreamParityByte(ImportData,SizeOf(THx573)-4);
        Target.Write(ImportData,SizeOf(THx573));

        Result:=true;
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Importx532Configuration(Source: TStringDynGrid; var Target: TMemoryStream): boolean;
var ImpexType: TImpexVersion;
    CorruptedData: boolean;
    lPM: TPulseOutputMode;
    lAM: TAlarmOutputMode;
    ImportData: THx532;
    IDT: TDeviceType;
    T: integer;
begin
  ImportData.MDD.UID:=CDef_UID_x532;
  ImportData.MDD.TDD:=CDef_TDD;
  ImportData.MDD.PVO:=CDef_PVO_x532;
  ImportData.ADD.AOP:=CDef_AdjustOutput;
  ImportData.ADD.LFP:=CDef_x532_LeakOff;
  ImportData.ADD.TransmitterFailureAlarmLevel:=CHart_Alarm_Off;
  ImportData.ADD.UnderOverRangeMode:=CHart_FullScale_Off;
  ImportData.TPP:=CDef_x532_Trip;
  ImportData.DVP:=CDef_Divider;
  ImportData.InpConf.InputSource:=CP_Blank_Pattern;
  ImportData.InpConf.SignalInvert:=CP_Blank_Pattern;
  T:=0;

  ImpexType:=imFrom401;
  IDT:=GetDevTypeFromString(Source[0,1]);

  if (IDT<>dtNone) and not((System.Length(Source)-1)<17) then
     begin
     CorruptedData:=not(ImporTHDeviceIdentifier(Source,0,ImportData.MDD.UID));

     if not(CorruptedData) then
        CorruptedData:=not(ImporTHTagDescriptorDate(Source,4,ImportData.MDD.TDD));

     // 7 - Input type
     if not(CorruptedData) then
        CorruptedData:=not(Encodex532InputSignalSourceFromString(Source[7,1],ImportData.InpConf.InputSource));

     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[8,1],ImportData.MDD.PVO.Point4mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[9,1],ImportData.MDD.PVO.Point20mA));

     if not(CorruptedData) then
        begin
        // evaluate if the imported file is before 4.01 or from version 4.01
        if IsFloatingPointString(Source[10,1],'.') or
           IsFloatingPointString(Source[10,1],',') then
           ImpexType:=imBefore401  // data is floating point and assumed damping time contant
        else ImpexType:=imFrom401; // data is not floating point and assumed NAMUR info

        if (ImpexType=imBefore401) then
           begin
           CorruptedData:=not(EncodeSingleValueFromString(Source[10,1],ImportData.MDD.PVO.DampingTimeConstant));

           // 11 - output characteristic
           if not(CorruptedData) then
              begin
              if (System.Length(Source[11])>2) then
                 CorruptedData:=not(Encodex532OutputCharacteristicFromString (Source[11,1],Source[11,2],ImportData.ADD.UnderOverRangeMode))
              else CorruptedData:=not(Encodex532OutputCharacteristicFromString (Source[11,1],'',ImportData.ADD.UnderOverRangeMode));
              end;
           end
        else // ImpexType=imFrom401
           begin
           if (System.Length(Source[10])>2) then
              CorruptedData:=not(Encodex532OutputCharacteristicFromString(Source[10,1],Source[10,2],ImportData.ADD.UnderOverRangeMode))
           else CorruptedData:=not(Encodex532OutputCharacteristicFromString(Source[10,1],'',ImportData.ADD.UnderOverRangeMode));

           // 11 - damping time constant
           if not(CorruptedData) then
              CorruptedData:=not(EncodeSingleValueFromString(Source[11,1],ImportData.MDD.PVO.DampingTimeConstant));
           end;
        end;

     if not(CorruptedData) then
        begin

        // 12 - adjust (ev. + 2 lines)
        if not(CorruptedData) then
           CorruptedData:=not(Encodex532AdjustModeFromString(Source[12,1],ImportData.ADD.AOP.AdjMode));
        if not(CorruptedData) and
           (AdjustModeFromHART(ImportData.ADD.AOP.AdjMode)<>tmOff) then
           begin
           CorruptedData:=not(EncodeSingleValueFromString (Source[13,1],ImportData.ADD.AOP.OffsetValue)) or
                          not(EncodeSingleValueFromString (Source[14,1],ImportData.ADD.AOP.GainValue));
           T:=T+2;
           end;

        // 13+T - Leak Flow (ev. +2 lines)
        if not(CorruptedData) then
           CorruptedData:=not( Encodex532LeakFlowOffFromString(Source[13+T,1],ImportData.ADD.LFP.FunctionSelect));
        if not(CorruptedData) and
           (LeakFlowModeFromHART(ImportData.ADD.LFP.FunctionSelect)<>tsOff) then
           begin
           CorruptedData:=not(EncodeSingleValueFromString (Source[14+T,1],ImportData.ADD.LFP.LeakFlowOffValue)) or
                          not(EncodeSingleValueFromString (Source[15+T,1],ImportData.ADD.LFP.LeakFlowOffCurrent));
           T:=T+2;
           end;

        // 14+T - transmitter alarm
        if not(CorruptedData) then
           CorruptedData:=not(EncodeAlarmFromString (Source[14+T,1],ImportData.ADD.TransmitterFailureAlarmLevel));

        // 15+T - pulse input (ev. +1 or +2 lines)
        if not(CorruptedData) then
           CorruptedData:=not(Encodex532PulseOutputModeFromString(Source[15+T,1],ImportData.DVP.FunctionSelect,ImportData.InpConf.InputSource));
        if not(CorruptedData) then
           begin
           lPM:=PulseOutputModex532FromHART(ImportData.DVP.FunctionSelect,ImportData.InpConf.InputSource);
           if (lPM=pmToggle) or (lPM=pmPulses) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString (Source[16+T,1],ImportData.DVP.Divider));
              T:=T+1;
              end;
           if not(CorruptedData) and (lPM=pmPulses) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString (Source[16+T,1],ImportData.DVP.Duration));
              T:=T+1;
              end;
           end;

        // 16+T - alarm input
        if not(CorruptedData) then
           begin
           if (ImpexType=imBefore401) then
              begin
              CorruptedData:=not(Encodex532AlarmOutputModeFromString(Source[16+T,1],ImportData.TPP.FunctionSelect,ImportData.InpConf.InputSource));
              end
           else // ImpexType=imFrom401
              begin
              CorruptedData:=not(Encodex532AlarmOutputModeFromString(Source[16+T,1],ImportData.TPP.FunctionSelect,ImportData.InpConf.InputSource));
              end;
           end;

        if not(CorruptedData) then
           begin
           lAM:=AlarmOutputModeFromHART(ImportData.TPP.FunctionSelect,ImportData.InpConf.InputSource);
           if (lAM=amTrip1) or (lAM=amTrip2) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString (Source[17+T,1],ImportData.TPP.TripAValue)) or
                             not(EncodeSingleValueFromString (Source[18+T,1],ImportData.TPP.AHysteresis));
              if not(CorruptedData) and (lAM=amTrip2) then
                 begin
                 CorruptedData:=not(EncodeSingleValueFromString (Source[19+T,1],ImportData.TPP.TripBValue)) or
                                not(EncodeSingleValueFromString (Source[20+T,1],ImportData.TPP.BHysteresis));
                 T:=T+2;
                 end;
              if not(CorruptedData) then
                 CorruptedData:=not(Encodex532InhibitModeFromString(Source[19+T,1],ImportData.TPP.FunctionSelect));
              end;
           end;
        end;

     if not(CorruptedData) then
        begin
        Target.Write(CNew_Config_Header,System.Length(CNew_Config_Header));

        // Note: the Write procedure for MTLx532 differ from the other modules
        // because of different file structure used in the software version 3.11
        Target.Write(ImportData.MDD,SizeOf(THMainDeviceData));
        Target.Write(ImportData.ADD,SizeOf(THAddDeviceData_x532));
        Target.Write(ImportData.InpConf,SizeOf(THInputConfiguration_x532));
        Target.Write(ImportData.DVP,SizeOf(THDividerParameters));
        Target.Write(ImportData.TPP,SizeOf(THTripParameters_x532));

        ImportData.Parity:=GetDataStreamParityByte(ImportData.MDD,SizeOf(THMainDeviceData)) xor
                           GetDataStreamParityByte(ImportData.ADD,SizeOf(THAddDeviceData_x532)) xor
                           GetDataStreamParityByte(ImportData.InpConf,SizeOf(THInputConfiguration_x532)) xor
                           GetDataStreamParityByte(ImportData.DVP,SizeOf(THDividerParameters)) xor
                           GetDataStreamParityByte(ImportData.TPP,SizeOf(THTripParameters_x532));

        Target.Write(ImportData.Parity,SizeOf(byte));

        Result:=true;
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Import1003Configuration(Source: TStringDynGrid; var Target: TMemoryStream): boolean;
var ImpexType: TImpexVersion;
    CorruptedData: boolean;
    OM: T1003OutputMode;
    ImportData: TH1003;
    IDT: TDeviceType;
    T: integer;
    R: single;
    B: byte;
begin
  ImportData.MDD.UID:=CDef_UID_1003;
  ImportData.MDD.TDD:=CDef_TDD;
  ImportData.MDD.PVO:=CDef_PVO_x532;
  ImportData.ADD.AOP:=CDef_AdjustOutput;
  ImportData.ADD.LFP:=CDef_1003_LeakOff;
  ImportData.ADD.TransmitterFailureAlarmLevel:=CHart_Alarm_Off;
  ImportData.ADD.FullScaleFlag:=CHart_FullScale_Off;
  ImportData.LFD_Inp.FunctionSelect:=CP_Blank_Pattern;
  ImportData.FTD.DVP:=CDef_Divider;
  ImportData.FTD.TPA:=CDef_1003_TripA;
  ImportData.FTD.TPB:=CDef_1003_TripB;
  T:=0;

  ImpexType:=imFrom401;
  IDT:=GetDevTypeFromString(Source[0,1]);

  if (IDT<>dtNone) and not((System.Length(Source)-1)<16) then
     begin
      CorruptedData:=not(ImporTHDeviceIdentifier(Source,0,ImportData.MDD.UID));

     if not(CorruptedData) then
        CorruptedData:=not(ImporTHTagDescriptorDate(Source,4,ImportData.MDD.TDD));

     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[7,1],ImportData.MDD.PVO.Point4mA));
     if not(CorruptedData) then
        CorruptedData:=not(EncodeSingleValueFromString(Source[8,1],ImportData.MDD.PVO.Point20mA));

     if not(CorruptedData) then
        begin
        // evaluate if the imported file is before 4.01 or from version 4.01
        if IsFloatingPointString(DeleteLeadingChars(Source[9,1],' '),'.') or
           IsFloatingPointString(DeleteLeadingChars(Source[9,1],' '),',') then
           ImpexType:=imBefore401  // data is floating point and assumed damping time contant
        else ImpexType:=imFrom401; // data is not floating point and assumed output characteristic info

        if (ImpexType=imBefore401) then
           begin
           // there was a hardware revision export bug in the earlier version
           if (IsFloatingPointString(DeleteLeadingChars(Source[3,1],' '),',') or
               IsFloatingPointString(DeleteLeadingChars(Source[3,1],' '),'.')) and
              EncodeSingleValueFromString (Source[3,1],R) then
              ImportData.MDD.UID.HardwareRevision:=System.Round(R*8)
           else CorruptedData:=true;

           if not(CorruptedData) then
              CorruptedData:=not(EncodeSingleValueFromString(Source[9,1],ImportData.MDD.PVO.DampingTimeConstant));

           // 10 - adjust (ev. + 2 lines)
           if not(CorruptedData) then
              CorruptedData:=not(Encodex532AdjustModeFromString(Source[10,1],ImportData.ADD.AOP.AdjMode));
           if not(CorruptedData) and
              (AdjustModeFromHART(ImportData.ADD.AOP.AdjMode)<>tmOff) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString (Source[11,1],ImportData.ADD.AOP.OffsetValue)) or
                             not(EncodeSingleValueFromString (Source[12,1],ImportData.ADD.AOP.GainValue));
              T:=T+2;
              end;

           // 11+T - Leak Flow (ev. +2 lines)
           if not(CorruptedData) then
              CorruptedData:=not(Encodex532LeakFlowOffFromString(Source[11+T,1],ImportData.ADD.LFP.FunctionSelect));
           if not(CorruptedData) and
              (LeakFlowModeFromHART(ImportData.ADD.LFP.FunctionSelect)<>tsOff) then
              begin
              // different FunctionSelect value for option ON
              ImportData.ADD.LFP.FunctionSelect:=CP3_LeakFlowEnabled;
              CorruptedData:=not(EncodeSingleValueFromString (Source[12+T,1],ImportData.ADD.LFP.LeakFlowOffValue)) or
                             not(EncodeSingleValueFromString (Source[13+T,1],ImportData.ADD.LFP.LeakFlowOffCurrent));
              T:=T+2;
              end;

           // 12+T - transmitter alarm
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[12+T,1],ImportData.ADD.TransmitterFailureAlarmLevel));

           // 13+T - Trip A function
           if not(CorruptedData) then
              CorruptedData:=not(Encode1003OutputModeFromString(Source[13+T,1],OM));
           if not(CorruptedData) and
              (OM<>omOff) then
              begin
              ImportData.FTD.TPA.FunctionSelect:=CP3_Trip_Enabled;
              if (OM=omRelay) then
                 ImportData.FTD.TPA.FunctionSelect:=ImportData.FTD.TPA.FunctionSelect or CP3_Trip_OnRelay;
              CorruptedData:=not(EncodePVUnitsFromString (Source[14+T,1],ImportData.FTD.TPA.TPUnits)) or
                             not(ImportData.FTD.TPA.TPUnits in [CHart_Herz,CHart_Percent]) or
                             not(EncodeSingleValueFromString (Source[15+T,1],ImportData.FTD.TPA.TripValue)) or
                             not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.TPA.Hysteresis)) or
                             not(Encode1003InvertedTrip(Source[17+T,1],ImportData.FTD.TPA.FunctionSelect));
              if not(CorruptedData) and
                (ImportData.FTD.TPA.TPUnits=CHart_Herz) then
                 begin
                 R:=System.Abs(ImportData.MDD.PVO.Point20mA-ImportData.MDD.PVO.Point4mA);
                 ImportData.FTD.TPA.TripValue:=100.0*System.Abs(ImportData.FTD.TPA.TripValue-ImportData.MDD.PVO.Point4mA)/R;
                 ImportData.FTD.TPA.Hysteresis:=100.0*ImportData.FTD.TPA.Hysteresis/R;
                 end;
              T:=T+4;
              end
           else ImportData.FTD.TPA.FunctionSelect:=CP3_Off_Pattern;

           // 14+T - Trip B function
           if not(CorruptedData) then
              CorruptedData:=not(Encode1003OutputModeFromString(Source[14+T,1],OM));
           if not(CorruptedData) and
              (OM<>omOff) then
              begin
              ImportData.FTD.TPB.FunctionSelect:=CP3_Trip_Enabled or CP3_TripB_Pattern;
              if (OM=omRelay) then
                 ImportData.FTD.TPB.FunctionSelect:=ImportData.FTD.TPB.FunctionSelect or CP3_Trip_OnRelay;
              CorruptedData:=not(EncodePVUnitsFromString (Source[15+T,1],ImportData.FTD.TPB.TPUnits)) or
                             not(ImportData.FTD.TPB.TPUnits in [CHart_Herz,CHart_Percent]) or
                             not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.TPB.TripValue)) or
                             not(EncodeSingleValueFromString (Source[17+T,1],ImportData.FTD.TPB.Hysteresis)) or
                             not(Encode1003InvertedTrip(Source[18+T,1],ImportData.FTD.TPB.FunctionSelect));
              if not(CorruptedData) and
                (ImportData.FTD.TPB.TPUnits=CHart_Herz) then
                 begin
                 R:=System.Abs(ImportData.MDD.PVO.Point20mA-ImportData.MDD.PVO.Point4mA);
                 ImportData.FTD.TPB.TripValue:=100.0*System.Abs(ImportData.FTD.TPB.TripValue-ImportData.MDD.PVO.Point4mA)/R;
                 ImportData.FTD.TPB.Hysteresis:=100.0*ImportData.FTD.TPB.Hysteresis/R;
                 end;
              T:=T+4;
              end
           else ImportData.FTD.TPB.FunctionSelect:=CP3_Off_Pattern or CP3_TripB_Pattern;

           // 15+T - pulse divider function
           if not(CorruptedData) then
              CorruptedData:=not(Encode1003OutputModeFromString(Source[15+T,1],OM));
           if not(CorruptedData) and
              (OM=omTransistor) then
              begin
              ImportData.FTD.DVP.FunctionSelect:=CP3_Divider_Enabled;
              CorruptedData:=not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.DVP.Divider)) or
                             not(EncodeSingleValueFromString (Source[17+T,1],ImportData.FTD.DVP.Duration));
              end
           else ImportData.FTD.DVP.FunctionSelect:=CP3_Off_Pattern;

           end
        else // ImpexType=imFrom401
           begin
           // 9 - output characteristic
           if (System.Length(Source[9])>2) then
              CorruptedData:=not(Encodex532OutputCharacteristicFromString (Source[9,1],Source[9,2],ImportData.ADD.FullScaleFlag))
           else CorruptedData:=not(Encodex532OutputCharacteristicFromString (Source[9,1],'',ImportData.ADD.FullScaleFlag));

           // 10 - damping constant
           if not(CorruptedData) then
              CorruptedData:=not(EncodeSingleValueFromString(Source[10,1],ImportData.MDD.PVO.DampingTimeConstant));

           // 11 - adjust (ev. + 2 lines)
           if not(CorruptedData) then
              CorruptedData:=not(Encodex532AdjustModeFromString(Source[11,1],ImportData.ADD.AOP.AdjMode));
           if not(CorruptedData) and
              (AdjustModeFromHART(ImportData.ADD.AOP.AdjMode)<>tmOff) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString (Source[12,1],ImportData.ADD.AOP.OffsetValue)) or
                             not(EncodeSingleValueFromString (Source[13,1],ImportData.ADD.AOP.GainValue));
              T:=T+2;
              end;

           // 12+T - Leak Flow (ev. +2 lines)
           if not(CorruptedData) then
              CorruptedData:=not(Encodex532LeakFlowOffFromString(Source[12+T,1],ImportData.ADD.LFP.FunctionSelect));
           if not(CorruptedData) and
              (LeakFlowModeFromHART(ImportData.ADD.LFP.FunctionSelect)<>tsOff) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString (Source[13+T,1],ImportData.ADD.LFP.LeakFlowOffValue)) or
                             not(EncodeSingleValueFromString (Source[14+T,1],ImportData.ADD.LFP.LeakFlowOffCurrent));
              T:=T+2;
              end;

           // 13+T - transmitter alarm
           if not(CorruptedData) then
              CorruptedData:=not(EncodeAlarmFromString (Source[13+T,1],ImportData.ADD.TransmitterFailureAlarmLevel));

           // 14+T - transistor output (ev. +1 or +2 lines)
           if not(CorruptedData) then
              CorruptedData:=not(Encode1003PulseOutputModeFromString(Source[14+T,1],Source[15+T,0],
                                                                     ImportData.FTD.DVP.FunctionSelect,
                                                                     ImportData.FTD.TPA.FunctionSelect,
                                                                     ImportData.FTD.TPB.FunctionSelect,
                                                                     ImportData.LFD_Inp.FunctionSelect));
           if not(CorruptedData) then
              begin
              if (ImportData.FTD.DVP.FunctionSelect<>CHart_Option_Off) then
                 begin
                 CorruptedData:=not(EncodeSingleValueFromString (Source[15+T,1],ImportData.FTD.DVP.Divider)) or
                                not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.DVP.Duration));
                 T:=T+2;
                 end;
              if (ImportData.FTD.TPA.FunctionSelect<>CP_Blank_Pattern) then
                 begin
                 CorruptedData:=not(EncodeSingleValueFromString (Source[15+T,1],ImportData.FTD.TPA.TripValue)) or
                                not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.TPA.Hysteresis));
                 T:=T+2;
                 end;
              if (ImportData.FTD.TPB.FunctionSelect<>CP_Blank_Pattern) then
                 begin
                 CorruptedData:=not(EncodeSingleValueFromString (Source[15+T,1],ImportData.FTD.TPB.TripValue)) or
                                not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.TPB.Hysteresis));
                 T:=T+2;
                 end;
              end;

           // 15+T - relay output (ev. +1 or +2 lines)
           if not(CorruptedData) then
              CorruptedData:=not(Encode1003AlarmOutputModeFromString(Source[15+T,1],Source[16+T,0],
                                                                     ImportData.FTD.TPA.FunctionSelect,
                                                                     ImportData.FTD.TPB.FunctionSelect,
                                                                     ImportData.LFD_Inp.FunctionSelect));
           if not(CorruptedData) and
              (ImportData.FTD.TPB.FunctionSelect<>CP_Blank_Pattern) then
                 begin
                 if (ImportData.FTD.TPA.FunctionSelect=CP_Blank_Pattern) then
                    begin
                    CorruptedData:=not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.TPB.TripValue)) or
                                   not(EncodeSingleValueFromString (Source[17+T,1],ImportData.FTD.TPB.Hysteresis));

                    end
                 else
                    begin
                    CorruptedData:=not(EncodeSingleValueFromString (Source[16+T,1],ImportData.FTD.TPA.TripValue)) or
                                   not(EncodeSingleValueFromString (Source[17+T,1],ImportData.FTD.TPA.Hysteresis)) or
                                   not(EncodeSingleValueFromString (Source[18+T,1],ImportData.FTD.TPB.TripValue)) or
                                   not(EncodeSingleValueFromString (Source[19+T,1],ImportData.FTD.TPB.Hysteresis));
                    end;
                 end;
           end;
        end;

     if not(CorruptedData) then
        begin
        Target.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
        ImportData.Parity:=GetDataStreamParityByte(ImportData,SizeOf(TH1003)-4);
        Target.Write(ImportData,SizeOf(TH1003));

        Result:=true;
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Importx576Configuration(Source: TStringDynGrid; var Target: TMemoryStream): boolean;
var ImpexType: TImpexVersion;
    CorruptedData: boolean;
    ImportData: THx576;
    IDT: TDeviceType;
    lSLength: word;
    lParity: byte;
begin
  ImportData.MDD.UID:=CDef_UID_x576;
  ImportData.MDD.TDD:=CDef_TDD;
  ImportData.MDD.PVO:=CDef_PVO_x073; //this is not relevant but required for compatibility
  ImportData.INPAR[0]:=CDef_x576_InPar;
  ImportData.INPAR[1]:=CDef_x576_InPar;
  ImportData.OUTPAR[0]:=CDef_x576_OutPar;
  ImportData.OUTPAR[1]:=CDef_x576_OutPar;
  ImportData.COMMPAR:=CDef_x576_Common;

  ImpexType:=imFrom401;
  IDT:=GetDevTypeFromString(Source[0,1]);

  if (IDT<>dtNone) and not((System.Length(Source)-1)<20) then
     begin
     CorruptedData:=not(ImporTHDeviceIdentifier(Source,0,ImportData.MDD.UID));

     if not(CorruptedData) then
        CorruptedData:=not(ImporTHTagDescriptorDate(Source,4,ImportData.MDD.TDD));

     if not(CorruptedData) and
        (System.Length(Source[7])>1) and
        (IsDecimalIntegerString(Source[7,1])) and
        (StrToInt(Source[7,1])=1) then
        begin
        CorruptedData:=not(ImportChannelInputParameter(Source,8,ImportData.INPAR[0]));

        if not(CorruptedData) then
           begin
           // evaluate if the imported file is before 4.01 or from version 4.01
           // Note: In version 4.01 the exported data sequence was chnaged
           if IsFloatingPointString(Source[14,1],'.') or
              IsFloatingPointString(Source[14,1],',') then
              ImpexType:=imBefore401  // data is floating point and assumed damping time contant
           else ImpexType:=imFrom401; // data is not floating point and assumed output characteristic

           if (ImpexType=imBefore401) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString(Source[14,1],ImportData.OUTPAR[0].DampingTimeConstant));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeSingleValueFromString(Source[15,1],ImportData.OUTPAR[0].SmoothingTimeConstant));

              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[16,1],ImportData.OUTPAR[0].OpenCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[17,1],ImportData.OUTPAR[0].ShortCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[18,1],ImportData.OUTPAR[0].ColdJunctionFailureAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[19,1],ImportData.OUTPAR[0].TransmitterFailureAlarmLevel));

              if not(CorruptedData) then
                 CorruptedData:=not(Encodex576OutputCharacteristicFromString (Source[20,1],ImportData.OUTPAR[0].NAMUR_MTL));

              if not(CorruptedData) then
                 CorruptedData:=not(Encodex075TripFunctionFromString (Source[21,1],ImportData.OUTPAR[0].EBD_Option));
              if not(CorruptedData) then // if ttTrip then CorruptedData
                 CorruptedData:=(x075TripTypeFromHART(ImportData.OUTPAR[0].EBD_Option) = ttTrip);
              end
           else // ImpexType=imFrom401
              begin
              CorruptedData:=not(Encodex576OutputCharacteristicFromString (Source[14,1],ImportData.OUTPAR[0].NAMUR_MTL));

              if not(CorruptedData) then
              CorruptedData:=not(EncodeSingleValueFromString(Source[15,1],ImportData.OUTPAR[0].DampingTimeConstant));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeSingleValueFromString(Source[16,1],ImportData.OUTPAR[0].SmoothingTimeConstant));

              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[17,1],ImportData.OUTPAR[0].OpenCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[18,1],ImportData.OUTPAR[0].ShortCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[19,1],ImportData.OUTPAR[0].ColdJunctionFailureAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[20,1],ImportData.OUTPAR[0].TransmitterFailureAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(Encodex573EBDFromString(Source[21,1],ImportData.OUTPAR[0].EBD_Option));
              end;
           end;
        end
     else CorruptedData:=true;

     if not(CorruptedData) and
        (System.Length(Source[22])>1) and
        (IsDecimalIntegerString(Source[22,1])) and
        (StrToInt(Source[22,1])=2) then
        begin
        ImportData.INPAR[1].Channel:=1;
        ImportData.OUTPAR[1].Channel:=1;

        CorruptedData:=not(ImportChannelInputParameter(Source,23,ImportData.INPAR[1]));

        if not(CorruptedData) then
           begin
           if (ImpexType=imBefore401) then
              begin
              CorruptedData:=not(EncodeSingleValueFromString(Source[29,1],ImportData.OUTPAR[1].DampingTimeConstant));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeSingleValueFromString(Source[30,1],ImportData.OUTPAR[1].SmoothingTimeConstant));

              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[31,1],ImportData.OUTPAR[1].OpenCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[32,1],ImportData.OUTPAR[1].ShortCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[33,1],ImportData.OUTPAR[1].ColdJunctionFailureAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[34,1],ImportData.OUTPAR[1].TransmitterFailureAlarmLevel));

              if not(CorruptedData) then
                 CorruptedData:=not(Encodex576OutputCharacteristicFromString (Source[35,1],ImportData.OUTPAR[1].NAMUR_MTL));
              if not(CorruptedData) then
                 CorruptedData:=not(Encodex075TripFunctionFromString (Source[36,1],ImportData.OUTPAR[1].EBD_Option));
              if not(CorruptedData) then // if ttTRip then CorruptedData
                 CorruptedData:=(x075TripTypeFromHART(ImportData.OUTPAR[1].EBD_Option) = ttTrip);
              end
           else // ImpexType=imFrom401
              begin
              CorruptedData:=not(Encodex576OutputCharacteristicFromString (Source[29,1],ImportData.OUTPAR[1].NAMUR_MTL));

              if not(CorruptedData) then
                 CorruptedData:=not(EncodeSingleValueFromString(Source[30,1],ImportData.OUTPAR[1].DampingTimeConstant));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeSingleValueFromString(Source[31,1],ImportData.OUTPAR[1].SmoothingTimeConstant));

              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[32,1],ImportData.OUTPAR[1].OpenCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[33,1],ImportData.OUTPAR[1].ShortCircuitAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[34,1],ImportData.OUTPAR[1].ColdJunctionFailureAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(EncodeAlarmFromString (Source[35,1],ImportData.OUTPAR[1].TransmitterFailureAlarmLevel));
              if not(CorruptedData) then
                 CorruptedData:=not(Encodex573EBDFromString(Source[36,1],ImportData.OUTPAR[1].EBD_Option));
              end;
           end;
        end;

     if not(CorruptedData) then
        begin
        CorruptedData:=not(Encodex575ResponseTimeFromString(Source[37,1],ImportData.COMMPAR.response_time));

        if not(CorruptedData) then
           CorruptedData:=not(Encodex073FilterFreqFromString(Source[38,1],ImportData.COMMPAR.FilterFrequencySelector));

        if not(CorruptedData) then
           CorruptedData:=not(Encodex576InputOptionFromString(Source[39,1],ImportData.COMMPAR.InputOption));
        end;

     if not(CorruptedData) then
        begin
        Target.Write(CNew_Config_Header,System.Length(CNew_Config_Header));
        // Note: the Write procedure for MTLx576 differ from the previous modules
        // because of different file structure used in the software version 3.01
        Target.Write(ImportData.MDD,SizeOf(THMainDeviceData));
        lParity:=GetDataStreamParityByte(ImportData.MDD,SizeOf(THMainDeviceData));
        Target.Write(lParity,SizeOf(byte));

        lSLength:=SizeOf(THInputParameterArray_x576)+SizeOf(THOutputParameterArray_x576)+SizeOf(THCommonParameters_x576)+SizeOf(byte);
        Target.Write(lSLength,SizeOf(word));

        Target.Write(ImportData.INPAR,SizeOf(THInputParameterArray_x576));

        Target.Write(ImportData.OUTPAR,SizeOf(THOutputParameterArray_x576));

        Target.Write(ImportData.COMMPAR,SizeOf(THCommonParameters_x576));

        ImportData.Parity:=GetDataStreamParityByte(ImportData.INPAR,SizeOf(THInputParameterArray_x576)) xor
                           GetDataStreamParityByte(ImportData.OUTPAR,SizeOf(THOutputParameterArray_x576)) xor
                           GetDataStreamParityByte(ImportData.COMMPAR,SizeOf(THCommonParameters_x576));

        Target.Write(ImportData.Parity,SizeOf(byte));

        Result:=true;
        end
     else Result:=false;
     end
  else Result:=false;
end;

procedure MoveUpStringDynGrid(var Grid: TStringDynGrid);
var lGrid: TStringDynGrid;
    I,L: integer;
begin
  L:=System.Length(Grid);
  SetLength(lGrid,0);
  for I:=0 to L-1 do
      AppendArrayToGrid(Grid[I],lGrid);
  SetLength(Grid,0);
  for I:=1 to L-1 do
      AppendArrayToGrid(lGrid[I],Grid);
end;

function ImportTextConfigurationFile(SourceFile: TFileName; SepChar: char): TMemoryStream;
var lFileStream: TLnCRCFileStream;
    lImportArray: TStringDynArray;
    lMemoryStream: TMemoryStream;
    lImportGrid: TStringDynGrid;
    I,L: integer;
    lDT: TDeviceType;
begin
  lMemoryStream:=TMemoryStream.Create;
  lMemoryStream.Clear;

  SetLength(lImportArray,0);
  SetLength(lImportGrid,0);

  lFileStream:=TLnCRCFileStream.Create(SourceFile,fmOpenRead);
  while not(lFileStream.Eof) do
        AppendElementToArray(lFileStream.ReadStringLn,lImportArray);
  lFileStream.Free;

  L:=System.Length(lImportArray);

  if (SepChar<>CTAB) and
     (System.Pos(';',lImportArray[0])>0) then
     SepChar:=';';

  for I:=0 to L-1 do
      AppendArrayToGrid(SplitStringToStringDynArray(lImportArray[I],SepChar),lImportGrid);

  L:=System.Length(lImportGrid);

  if (L>3) and
     (System.Length(lImportGrid[0])=1) and
     not(System.Length(lImportGrid[1])<2) then
     MoveUpStringDynGrid(lImportGrid);

  if not(L<7) and
     not(System.Length(lImportGrid[0])<2) and
     not(System.Length(lImportGrid[1])<2) then
     begin
     lDT:=GetDevTypeFromString(lImportGrid[0,1]);

     if (lDT<>dtNone) and
        CheckManufacturersIdentCodeFromString(lImportGrid[1,1]) then
        begin
        case lDT of
             dt507x,
             dtx073,
             dt5074XK,
             dt4073XK: Importx073Configuration(lImportGrid,lMemoryStream);
             dt4075:   Importx075Configuration(lImportGrid,lMemoryStream);
             dtx575:   Importx575Configuration(lImportGrid,lMemoryStream);
             dtx573:   Importx573Configuration(lImportGrid,lMemoryStream);
             dtx576:   Importx576Configuration(lImportGrid,lMemoryStream);
             dtx532:   Importx532Configuration(lImportGrid,lMemoryStream);
             dt1003:   Import1003Configuration(lImportGrid,lMemoryStream);
             end;
        end;
     end;

  Result:=lMemoryStream;
end;

function ImportExcelConfigurationFile(SourceFile: TFileName): TMemoryStream;
var lMemoryStream: TMemoryStream;
begin
  lMemoryStream:=TMemoryStream.Create;
  lMemoryStream.Clear;

  Result:=lMemoryStream;
end;

function ImportUserSensorConfiguration(SourceFile: TFileName; var UserSensorData: THUserSensorData_x576): boolean;
var lUserSensorData: THUserSensorData_x576;
    lFileStream: TLnCRCFileStream;
    lImportArray: TStringDynArray;
    lFormatError: boolean;
    I: integer;
begin
  Result:=false;
  lFormatError:=false;
  if FileExists(SourceFile) then
     begin
     lFileStream:=TLnCRCFileStream.Create(SourceFile,fmOpenRead);
     lImportArray:=lFileStream.ReadFileToStringDynArrayLn;
     lFileStream.Free;
     if (System.Length(lImportArray)=70) then
        begin
        for I:=0 to 69 do
            lImportArray[I]:=DeleteAllChars(lImportArray[I],' ');
        lUserSensorData.SensorType:=StrToIntDef(lImportArray[0],0);
        if (lUserSensorData.SensorType in [$09,$90,$A0]) then
           begin
           if IsFloatingPointString(lImportArray[1],'.') then
              lUserSensorData.TMIN:=GetRealFromString(lImportArray[1],'.')
           else lFormatError:=true;
           if not(lFormatError) and IsFloatingPointString(lImportArray[2],'.') then
              lUserSensorData.TMAX:=GetRealFromString(lImportArray[2],'.')
           else lFormatError:=true;
           if not(lFormatError) and IsFloatingPointString(lImportArray[3],'.') then
              lUserSensorData.TSEG:=GetRealFromString(lImportArray[3],'.')
           else lFormatError:=true;
           for I:=0 to 64 do
               if not(lFormatError) and IsDecimalIntegerString(lImportArray[I+5]) then
                  lUserSensorData.CoeffTable[I]:=StrToInt(lImportArray[I+5])
               else
                  begin
                  lFormatError:=true;
                  break;
                  end;
           if not(lFormatError) then
              begin
              UserSensorData.SensorType:=lUserSensorData.SensorType;
              UserSensorData.TMIN:=lUserSensorData.TMIN;
              UserSensorData.TMAX:=lUserSensorData.TMAX;
              UserSensorData.TSEG:=lUserSensorData.TSEG;
              UserSensorData.reserved:=0;
              for I:=0 to 64 do
                  UserSensorData.CoeffTable[I]:=lUserSensorData.CoeffTable[I];
              Result:=true;
              end;
           end;
        end;
     end;
end;

//***********************************************************************************************//
//**
//** Text File Configuration Export
//**
//***********************************************************************************************//

procedure ExportTwoStringsToGrid(Title, Value: string;  var Target: TStringDynGrid);
var lExportArray: TStringDynArray;
begin
  SetLength(lExportArray,2);

  lExportArray[0]:=DeleteAcceleratorKey(Title);
  lExportArray[1]:=DeleteAcceleratorKey(Value);
  AppendArrayToGrid(lExportArray,Target);
end;

procedure ExportThreeStringsToGrid(Title, Value, Units: string; var Target: TStringDynGrid);
var lExportArray: TStringDynArray;
begin
  SetLength(lExportArray,3);

  lExportArray[0]:=DeleteAcceleratorKey(Title);
  lExportArray[1]:=Value;
  lExportArray[2]:=Units;
  AppendArrayToGrid(lExportArray,Target);
end;

procedure ExporTHDeviceIdentifier(Source: THDeviceIdentifier; var Target: TStringDynGrid);
begin
  ExportTwoStringsToGrid(LanguageStrings[flDeviceCode],CDeviceShortNames[IdentToType(Source)],Target);

  ExportTwoStringsToGrid(LanguageStrings[flManufacturer],ManufacturersIdentNameFromHART(Source.ManufacturerIdentCode), Target);

  ExportTwoStringsToGrid(LanguageStrings[flSWRevision],IntToStr(Source.SoftwareRevision),Target);

  ExportTwoStringsToGrid(LanguageStrings[flHWRevision],HARTHardwareRevisionToString(Source.HardwareRevision),Target);
end;

procedure ExporTHTagDescriptorDate(Source: THTagDescriptorDate; var Target: TStringDynGrid);
var SimpleDate: TSimpleDate;
begin
  SimpleDate.Y:=Source.Year+1900;
  SimpleDate.M:=Source.Month;
  SimpleDate.D:=Source.Day;

  ExportTwoStringsToGrid(LanguageStrings[flTag],Source.TAGName,Target);
  ExportTwoStringsToGrid(LanguageStrings[flDescription],Source.Descriptor,Target);
  ExportTwoStringsToGrid(LanguageStrings[flDate],FormatShortDateString(SimpleDate),Target);
end;

procedure Export_OnOffSelection_Info(Source: TOnOffSet; SelectionInfo: string; var Target: TStringDynGrid);
begin
  if Source=tsOn then
     ExportTwoStringsToGrid(SelectionInfo,LanguageStrings[flOn],Target)
  else   ExportTwoStringsToGrid(SelectionInfo,LanguageStrings[flOff],Target);
end;

procedure Export_AlarmLevel_Info(Source: TAlarmLevelSet; AlarmInfo: string; var Target: TStringDynGrid);
begin
  case Source of
       taLow:   ExportTwoStringsToGrid(AlarmInfo,LanguageStrings[flLowAlarm],Target);
       taHigh:  ExportTwoStringsToGrid(AlarmInfo,LanguageStrings[flHighAlarm],Target);
       taOff:   ExportTwoStringsToGrid(AlarmInfo,LanguageStrings[flAlarmOff],Target);
       end;
end;

procedure Export_x073_Info1(DevIdent: THDeviceIdentifier; PVOutput: THPrimaryVariableOutput; x073Source: THx073ParameterSet; var Target: TStringDynGrid);
begin
  ExportTwoStringsToGrid(LanguageStrings[flInputType],TempInputTypeNameFromHARTInfo(IdentToType(DevIdent), x073Source.InputType, PVOutput.TransferFunction),Target);
  ExportTwoStringsToGrid(LanguageStrings[flProcessUnits],LanguageStrings[flCelsius+PVUnitsFromHART(PVOutput.PVUnits)],Target);
  Export_OnOffSelection_Info(LinearFromHART(PVOutput.TransferFunction),LanguageStrings[flLinearisation],Target);
  Export_OnOffSelection_Info(CJCompFromHART(x073Source.ColdJunctionCompensationOnOff),LanguageStrings[flColdJnCompensation],Target);
end;

procedure ExportPVOInfo(Source: THPrimaryVariableOutput; var Target: TStringDynGrid);
begin
  ExportThreeStringsToGrid(LanguageStrings[fl4mAPoint], FloatToStrF(Source.Point4mA,ffFixed,6,1), LanguageStrings[flCelsius+PVUnitsFromHART(Source.PVUnits)], Target);
  ExportThreeStringsToGrid(LanguageStrings[fl20mAPoint], FloatToStrF(Source.Point20mA,ffFixed,6,1), LanguageStrings[flCelsius+PVUnitsFromHART(Source.PVUnits)], Target);
end;

procedure ExportOutputFilteringInfo(Source: THPrimaryVariableOutput; x073Source: THx073ParameterSet; var Target: TStringDynGrid);
begin
  ExportThreeStringsToGrid(LanguageStrings[flDampingConst], FloatToStrF(Source.DampingTimeConstant,ffFixed,4,1), 's', Target);
  ExportThreeStringsToGrid(LanguageStrings[flSmoothingConst],IntToStr(x073Source.SmoothingTimeConstant),'s',Target);
end;

procedure Export_x073_Info2(x073Source: THx073ParameterSet; var Target: TStringDynGrid);
begin
  ExportTwoStringsToGrid(LanguageStrings[flFilterFreq],LanguageStrings[fl50Hz+integer(FrequencyFromHART(x073Source.FilterFrequencySelector))],Target);

  Export_AlarmLevel_Info(AlarmLevelFromHART(x073Source.OpenCircuitAlarmLevel),LanguageStrings[flAlarmOC],Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(x073Source.ColdJunctionFailureAlarmLevel),LanguageStrings[flAlarmCJC],Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(x073Source.TransmitterFailureAlarmLevel),LanguageStrings[flAlarmTX],Target);
end;

procedure Export_x075_TripInfo(Source: THx075TripParameters; var Target: TStringDynGrid);
var EBDInfo: string;
begin
  EBDInfo:=TripFunctionNameFromHART(Source.TripOutputFunction,Source.TripValue);
  if (x075TripTypeFromHART(Source.TripOutputFunction)<>ttTrip) then
     ExportTwoStringsToGrid(LanguageStrings[flEBDdetectionTrip],EBDInfo,Target)
  else //
     begin // = ttTrip
     ExportThreeStringsToGrid(LanguageStrings[flThreshold],EBDInfo,'%',Target);
     ExportThreeStringsToGrid(LanguageStrings[flHysteresis],FloatToStrF(Source.Hysteresis,ffFixed,3,1),'%',Target);
     end;
end;

procedure Export_x575_ResponseInfo(Source: byte; var Target: TStringDynGrid);
begin
  case Source of
       2:   ExportTwoStringsToGrid(LanguageStrings[flResponseTime],LanguageStrings[flFast],Target);
       1:   ExportTwoStringsToGrid(LanguageStrings[flResponseTime],LanguageStrings[flMedium],Target);
       else ExportTwoStringsToGrid(LanguageStrings[flResponseTime],LanguageStrings[flSlow],Target);
       end;
end;

procedure Export_x575_NE43(Source: byte; var Target: TStringDynGrid);
begin
  if (Source=1) then
     ExportTwoStringsToGrid(LanguageStrings[flOutputCharacteristic],LanguageStrings[flOutNE43],Target)
  else ExportTwoStringsToGrid(LanguageStrings[flOutputCharacteristic],LanguageStrings[flOutMTL],Target);
end;

procedure Export_x576_NE43(Source: byte; var Target: TStringDynGrid);
begin
  if (Source=0) then
     ExportTwoStringsToGrid(LanguageStrings[flOutputCharacteristic],LanguageStrings[flOutNE43],Target)
  else ExportTwoStringsToGrid(LanguageStrings[flOutputCharacteristic],LanguageStrings[flOutMTL],Target);
end;

procedure Export_x532_InputInfo(Source: byte; var Target: TStringDynGrid);
begin
  if ((Source and $03)=0) then
     ExportTwoStringsToGrid(LanguageStrings[flInputSource],LanguageStrings[flCurrentOrVoltage],Target)
  else ExportTwoStringsToGrid(LanguageStrings[flInputSource],LanguageStrings[flSwitchProximity],Target);
end;

procedure Export_x532_OutputInfo(FSInfo, NE43Info: byte; var Target: TStringDynGrid);
var OI: string;
begin
  if (FSInfo=1) then
     OI:=LanguageStrings[flOut0to20mA]
  else
     begin
     if (NE43Info>0) then
        OI:=LanguageStrings[flOut4to20mA]+' (NE43)'
     else OI:=LanguageStrings[flOut4to20mA]+' (MTL)';
     end;
  ExportTwoStringsToGrid(LanguageStrings[flOutputCharacteristic],OI,Target);
end;

procedure Export_x532_AdjustmentInfo(Source: THAdjustOutputParameters; var Target: TStringDynGrid);
begin
  if (AdjustModeFromHART(Source.AdjMode)=tmOff) then
     ExportTwoStringsToGrid(LanguageStrings[flAdjustOutput],LanguageStrings[flOff],Target)
  else
     begin
     ExportTwoStringsToGrid(LanguageStrings[flAdjustOutput],LanguageStrings[flOn],Target);
     ExportThreeStringsToGrid('    '+LanguageStrings[flAdjustOffset],FloatToStrF(Source.OffsetValue,ffFixed,4,1),'%',Target);
     ExportThreeStringsToGrid('    '+LanguageStrings[flAdjustGain],FloatToStrF(Source.GainValue,ffFixed,4,1),'%',Target);
     end;
end;

procedure Export_x532_LeakFlowInfo(Source: THLeakFlowOffParameters_x532; var Target: TStringDynGrid);
begin
  if (LeakFlowModeFromHART(Source.FunctionSelect)=tsOff) then
     ExportTwoStringsToGrid(LanguageStrings[flLeakFlowOption],LanguageStrings[flOff],Target)
  else
     begin
     ExportTwoStringsToGrid(LanguageStrings[flLeakFlowOption],LanguageStrings[flOn],Target);
     // .. may be we should check "LFUnits" and allow only % entry ? ..
     ExportThreeStringsToGrid('    '+LanguageStrings[flLeakFlowThreshold],FloatToStrF(Source.LeakFlowOffValue,ffFixed,4,1),'%',Target);
     ExportThreeStringsToGrid('    '+LanguageStrings[flLeakFlowCurrent],FloatToStrF(Source.LeakFlowOffCurrent,ffFixed,4,1),LanguageStrings[flmiliAmp],Target);
     end;
end;

procedure Export_x532_DividerInfo(Source: THDividerParameters; InpConf: byte; var Target: TStringDynGrid);
begin
  if ((Source.FunctionSelect and CP_Divider_Enabled)=0) then
     begin
     if ((InpConf and CP_PulseToInput)=0) then
        ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flOff],Target)
     else ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flPulseDirect],Target);
     end
  else // (Source.FunctionSelect and CP_Divider_Enabled)>0
     begin
     if ((Source.FunctionSelect and CP_Divider_Toggle)>0) then
         ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flToggleOutput],Target)
     else ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flFixedPulse],Target);
     ExportTwoStringsToGrid('    '+LanguageStrings[flDivisorValue],FloatToStrF(Source.Divider,ffFixed,5,0),Target);
     if ((Source.FunctionSelect and CP_Divider_Toggle)=0) then
        ExportThreeStringsToGrid('    '+LanguageStrings[flPulseDuration],FloatToStrF(Source.Duration,ffFixed,4,1),'ms',Target);
     end;
end;

procedure Export_x532_TripInfo(Source: THTripParameters_x532; InpConf: THInputConfiguration_x532; var Target: TStringDynGrid);
var UnitString: string;
begin
  case (Source.FunctionSelect and (CP_TripAOnALarm or CP_TripABOnAlarm)) of
       CP_TripAOnALarm:
            begin
            if ((Source.FunctionSelect and CP_TripAInverted)>0) then
                ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flInvertedShort],Target)  // flSimpleTrip
            else ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flNormalShort],Target);

            if (ValueUnitsFromHART(Source.TPAUnits))=tuPercent then
               UnitString:=LanguageStrings[flPercent]
            else UnitString:=LanguageStrings[flHerz];

            ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold],FloatToStrF(Source.TripAValue,ffFixed,4,1),UnitString,Target);
            ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis],FloatToStrF(Source.AHysteresis,ffFixed,4,1),UnitString,Target);

            if ((Source.FunctionSelect and CP_InhibitEnabled)>0) then
               begin
               if ((InpConf.SignalInvert and CP_StatusProxy2)>0) then
                  ExportThreeStringsToGrid('    '+'Inhibit',LanguageStrings[flOn],LanguageStrings[flInvertedShort],Target)
               else ExportThreeStringsToGrid('    '+'Inhibit',LanguageStrings[flOn],LanguageStrings[flNormalShort],Target);
               end
            else ExportTwoStringsToGrid('    '+'Inhibit',LanguageStrings[flOff],Target);
            end;
       CP_TripABOnAlarm:
            begin
            if ((Source.FunctionSelect and CP_TripABInverted)>0) then
                ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flInvertedShort],Target) // flWindowTrip
            else ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flNormalShort],Target);

            if (ValueUnitsFromHART(Source.TPAUnits))=tuPercent then
               UnitString:=LanguageStrings[flPercent]
            else UnitString:=LanguageStrings[flHerz];

            ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold]+' 1',FloatToStrF(Source.TripAValue,ffFixed,4,1),UnitString,Target);
            ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis]+' 1',FloatToStrF(Source.AHysteresis,ffFixed,4,1),UnitString,Target);

            if (ValueUnitsFromHART(Source.TPBUnits))=tuPercent then
               UnitString:=LanguageStrings[flPercent]
            else UnitString:=LanguageStrings[flHerz];

            ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold]+' 2',FloatToStrF(Source.TripBValue,ffFixed,4,1),UnitString,Target);
            ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis]+' 2',FloatToStrF(Source.BHysteresis,ffFixed,4,1),UnitString,Target);

            if ((Source.FunctionSelect and CP_InhibitEnabled)>0) then
               begin
               if ((InpConf.SignalInvert and CP_StatusProxy2)>0) then
                  ExportThreeStringsToGrid('    '+'Inhibit',LanguageStrings[flOn],LanguageStrings[flInvertedShort],Target)
               else ExportThreeStringsToGrid('    '+'Inhibit',LanguageStrings[flOn],LanguageStrings[flNormalShort],Target);
               end
            else ExportTwoStringsToGrid('    '+'Inhibit',LanguageStrings[flOff],Target);
            end
       else // no Trip function
          begin
          if ((InpConf.InputSource and CP_AlarmToLFD)>0) then
             ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flLFDShort],Target)
          else ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flOff],Target);
          end;
       end;
end;

procedure Export_1003_AdjustmentInfo(Source: THAdjustOutputParameters; var Target: TStringDynGrid);
begin
  if (AdjustModeFromHART(Source.AdjMode)=tmOff) then
     ExportTwoStringsToGrid(LanguageStrings[flAdjustOutput],LanguageStrings[flOff],Target)
  else
     begin
     if (AdjustModeFromHART(Source.AdjMode)=tmHard) then
        ExportTwoStringsToGrid(LanguageStrings[flAdjustOutput],LanguageStrings[flHardAdjust],Target)
     else ExportTwoStringsToGrid(LanguageStrings[flAdjustOutput],LanguageStrings[flOn],Target); // flSoftAdjust
     ExportThreeStringsToGrid('    '+LanguageStrings[flAdjustOffset],FloatToStrF(Source.OffsetValue,ffFixed,4,1),'%',Target);
     ExportThreeStringsToGrid('    '+LanguageStrings[flAdjustGain],FloatToStrF(Source.GainValue,ffFixed,4,1),'%',Target);
     end;
end;

procedure Export_1003_LeakFlowInfo(Source: THLeakFlowOffParameters_1003; var Target: TStringDynGrid);
begin
  // Note: change 1003 LeakFlow option to x532 format
  if (Source.FunctionSelect=CP3_LeakFlowEnabled) then
     Source.FunctionSelect:=CHart_Option_On
  else Source.FunctionSelect:=CHart_Option_Off;
  if (LeakFlowModeFromHART(Source.FunctionSelect)=tsOff) then
     ExportTwoStringsToGrid(LanguageStrings[flLeakFlowOption],LanguageStrings[flOff],Target)
  else
     begin
     ExportTwoStringsToGrid(LanguageStrings[flLeakFlowOption],LanguageStrings[flOn],Target);
     // .. may be we should check "LFUnits" and allow only % entry ? ..
     ExportThreeStringsToGrid('    '+LanguageStrings[flLeakFlowThreshold],FloatToStrF(Source.LeakFlowOffValue,ffFixed,4,1),'%',Target);
     ExportThreeStringsToGrid('    '+LanguageStrings[flLeakFlowCurrent],FloatToStrF(Source.LeakFlowOffCurrent,ffFixed,4,1),LanguageStrings[flmiliAmp],Target);
     end;
end;

procedure Export_1003_DividerInfo(Source_TPA,Source_TPB: THTripParameters_1003; Source_DP: THDividerParameters; InpConf: byte; var Target: TStringDynGrid);
var UnitString: string;
begin
  // Note: it is sufficient to check only "no Trip A on Transistor Output" here
  // becuase Trip B can be only together with Trip A
  if (((Source_TPA.FunctionSelect and CP3_Trip_Enabled)=0) or
      ((Source_TPA.FunctionSelect and CP3_Trip_OnRelay)>0)) then
     begin
     if ((Source_DP.FunctionSelect and CP3_Divider_Enabled)=0) then
        begin
        if ((InpConf and CP3_LFD_Enabled)=0) or
           ((InpConf and CP3_LFD_OnRelay)>0) then
           ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flOff],Target)
        else ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flLFDShort],Target);
        end
     else // (Source_DP.FunctionSelect and CP_Divider_Enabled)>0
        begin
        ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flFixedPulse],Target);
        ExportTwoStringsToGrid('    '+LanguageStrings[flDivisorValue],FloatToStrF(Source_DP.Divider,ffFixed,5,0),Target);
        ExportThreeStringsToGrid('    '+LanguageStrings[flPulseDuration],FloatToStrF(Source_DP.Duration,ffFixed,4,1),'ms',Target);
        end;
     end
  else // there is at least one Trip on Relay output
     begin
     // Note: it is sufficient now to check only "Trip B on Transistor Output"
     // to detect window comparator TripAB on Transistor Output
     if ((Source_TPB.FunctionSelect and CP3_Trip_Enabled)>0) and
        ((Source_TPB.FunctionSelect and CP3_Trip_OnRelay)=0) then
        begin
        if ((Source_TPA.FunctionSelect and CP3_Trip_Inverted)>0) then
            ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flInvertedShort],Target) // flWindowTrip
        else ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flNormalShort],Target);

        if (ValueUnitsFromHART(Source_TPA.TPUnits))=tuPercent then
           UnitString:=LanguageStrings[flPercent]
        else UnitString:=LanguageStrings[flHerz];

        ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold]+' 1',FloatToStrF(Source_TPA.TripValue,ffFixed,4,1),UnitString,Target);
        ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis]+' 1',FloatToStrF(Source_TPA.Hysteresis,ffFixed,4,1),UnitString,Target);

        if (ValueUnitsFromHART(Source_TPB.TPUnits))=tuPercent then
           UnitString:=LanguageStrings[flPercent]
        else UnitString:=LanguageStrings[flHerz];

        ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold]+' 2',FloatToStrF(Source_TPB.TripValue,ffFixed,4,1),UnitString,Target);
        ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis]+' 2',FloatToStrF(Source_TPB.Hysteresis,ffFixed,4,1),UnitString,Target);
        end
     else // single comparator TripA on Transistor Output
        begin
        if ((Source_TPA.FunctionSelect and CP3_Trip_Inverted)>0) then
            ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flInvertedShort],Target)  // flSimpleTrip
        else ExportTwoStringsToGrid(LanguageStrings[flPulseOutput],LanguageStrings[flNormalShort],Target);

        if (ValueUnitsFromHART(Source_TPA.TPUnits))=tuPercent then
           UnitString:=LanguageStrings[flPercent]
        else UnitString:=LanguageStrings[flHerz];

        ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold],FloatToStrF(Source_TPA.TripValue,ffFixed,4,1),UnitString,Target);
        ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis],FloatToStrF(Source_TPA.Hysteresis,ffFixed,4,1),UnitString,Target);
        end;
     end;
end;

procedure Export_1003_TripInfo(Source_TPA,Source_TPB: THTripParameters_1003; InpConf: byte; var Target: TStringDynGrid);
var UnitString: string;
begin
  // Note: it is sufficient to check only "no Trip B on Relay Output" here
  // becuase Trip A can be only together with Trip B
  if (((Source_TPB.FunctionSelect and CP3_Trip_Enabled)=0) or
      ((Source_TPB.FunctionSelect and CP3_Trip_OnRelay)=0)) then
     begin
     if ((InpConf and CP3_LFD_Enabled)=0) or
        ((InpConf and CP3_LFD_OnRelay)=0) then // no LFD on Relay Output
        ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flOff],Target)
     else ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flLFDShort],Target)
     end
  else // there is at least one Trip on Relay output
     begin
     // Note: it is sufficient now to check only "Trip A on Relay Output"
     // to detect window comparator TripAB on Relay Output
     if ((Source_TPA.FunctionSelect and CP3_Trip_Enabled)>0) and
        ((Source_TPA.FunctionSelect and CP3_Trip_OnRelay)>0) then
        begin
        if ((Source_TPA.FunctionSelect and CP3_Trip_Inverted)>0) then
            ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flInvertedShort],Target) // flWindowTrip
        else ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flNormalShort],Target);

        if (ValueUnitsFromHART(Source_TPA.TPUnits))=tuPercent then
           UnitString:=LanguageStrings[flPercent]
        else UnitString:=LanguageStrings[flHerz];

        ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold]+' 1',FloatToStrF(Source_TPA.TripValue,ffFixed,4,1),UnitString,Target);
        ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis]+' 1',FloatToStrF(Source_TPA.Hysteresis,ffFixed,4,1),UnitString,Target);

        if (ValueUnitsFromHART(Source_TPB.TPUnits))=tuPercent then
           UnitString:=LanguageStrings[flPercent]
        else UnitString:=LanguageStrings[flHerz];

        ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold]+' 2',FloatToStrF(Source_TPB.TripValue,ffFixed,4,1),UnitString,Target);
        ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis]+' 2',FloatToStrF(Source_TPB.Hysteresis,ffFixed,4,1),UnitString,Target);
        end
     else // single comparator TripB on Relay Output
        begin
        if ((Source_TPB.FunctionSelect and CP3_Trip_Inverted)>0) then
            ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flInvertedShort],Target)  // flSimpleTrip
        else ExportTwoStringsToGrid(LanguageStrings[flAlarmOutput],LanguageStrings[flNormalShort],Target);

        if (ValueUnitsFromHART(Source_TPB.TPUnits))=tuPercent then
           UnitString:=LanguageStrings[flPercent]
        else UnitString:=LanguageStrings[flHerz];

        ExportThreeStringsToGrid('    '+LanguageStrings[flThreshold],FloatToStrF(Source_TPB.TripValue,ffFixed,4,1),UnitString,Target);
        ExportThreeStringsToGrid('    '+LanguageStrings[flHysteresis],FloatToStrF(Source_TPB.Hysteresis,ffFixed,4,1),UnitString,Target);
        end;
     end;
end;

procedure ExportChannelInputParameters_x576(DevType: TDeviceType; Source: THInputParameters_x576; var Target: TStringDynGrid);
begin
  ExportTwoStringsToGrid(LanguageStrings[flInputType],TempInputTypeNameFromHARTInfo(DevType, Source.SensorType, Source.Linearisation),Target);
  ExportTwoStringsToGrid(LanguageStrings[flProcessUnits],LanguageStrings[flCelsius+PVUnitsFromHART(Source.PVUnits)],Target);
  Export_OnOffSelection_Info(LinearFromHART(Source.Linearisation),LanguageStrings[flLinearisation],Target);
  Export_OnOffSelection_Info(CJCompFromHART(Source.CJC_Option),LanguageStrings[flColdJnCompensation],Target);

  ExportThreeStringsToGrid(LanguageStrings[fl4mAPoint], FloatToStrF(Source.Point4mA,ffFixed,6,1), LanguageStrings[flCelsius+PVUnitsFromHART(Source.PVUnits)], Target);
  ExportThreeStringsToGrid(LanguageStrings[fl20mAPoint], FloatToStrF(Source.Point20mA,ffFixed,6,1), LanguageStrings[flCelsius+PVUnitsFromHART(Source.PVUnits)], Target);
end;

procedure Export_x573_TripInfo(Source: byte; var Target: TStringDynGrid);
var EBDType: Tx075TripType;
    EBDInfo: string;
begin
  EBDType:=x075TripTypeFromHART(Source);
  if (EBDType<>ttTrip) then
     begin
     if (EBDType=ttEBD) then
        EBDInfo:=LanguageStrings[flOn]
     else EBDInfo:=LanguageStrings[flOff];
     ExportTwoStringsToGrid(LanguageStrings[flEBDdetection],EBDInfo,Target);
     end;
end;

procedure ExportChannelOutputParameters_x576(Source: THOutputParameters_x576; var Target: TStringDynGrid);
begin
  Export_x576_NE43(Source.NAMUR_MTL, Target);
  ExportThreeStringsToGrid(LanguageStrings[flDampingConst], FloatToStrF(Source.DampingTimeConstant,ffFixed,4,1), 's', Target);
  ExportThreeStringsToGrid(LanguageStrings[flSmoothingConst], FloatToStrF(Source.SmoothingTimeConstant,ffFixed,4,1), 's', Target);

  Export_AlarmLevel_Info(AlarmLevelFromHART(Source.OpenCircuitAlarmLevel),LanguageStrings[flAlarmOC],Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(Source.ShortCircuitAlarmLevel),LanguageStrings[flAlarmSC],Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(Source.ColdJunctionFailureAlarmLevel),LanguageStrings[flAlarmCJC],Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(Source.TransmitterFailureAlarmLevel),LanguageStrings[flAlarmTX],Target);

  Export_x573_TripInfo(Source.EBD_Option,Target);
end;

procedure ExportTHCommonParameters_x576(Source: THCommonParameters_x576; var Target: TStringDynGrid);
var InputOption: Tx576InputOptionSet;
begin
  Export_x575_ResponseInfo(Source.response_time,Target);
  ExportTwoStringsToGrid(LanguageStrings[flFilterFreq],LanguageStrings[fl50Hz+integer(FrequencyFromHART(Source.FilterFrequencySelector))],Target);

  InputOption:=InputOptionx576FromHART(Source.InputOption);
  case InputOption of
    is2In2Out: ExportTwoStringsToGrid('Input Option',LanguageStrings[fl2in2out],Target);
    is1In2Out: ExportTwoStringsToGrid('Input Option',LanguageStrings[fl1in2out],Target);
    end;
end;

procedure Export_x073_ToGrid(Source: THartModule_x073; var Target: TStringDynGrid);
begin
  ExporTHDeviceIdentifier(Source.BasicDeviceData.UID,Target);
  ExporTHTagDescriptorDate(Source.BasicDeviceData.TDD,Target);
  Export_x073_Info1(Source.BasicDeviceData.UID,Source.PrimaryVariableOutputData, Source.x073ParameterSet,Target);
  ExportPVOInfo(Source.PrimaryVariableOutputData,Target);
  ExportOutputFilteringInfo(Source.PrimaryVariableOutputData,Source.x073ParameterSet,Target);
  Export_x073_Info2(Source.x073ParameterSet,Target);
end;

procedure Export_x075_ToGrid(Source: THartModule_x075; var Target: TStringDynGrid);
begin
  ExporTHDeviceIdentifier(Source.BasicDeviceData.UID,Target);
  ExporTHTagDescriptorDate(Source.BasicDeviceData.TDD,Target);
  Export_x073_Info1(Source.BasicDeviceData.UID,Source.PrimaryVariableOutputData, Source.x073ParameterSet,Target);
  ExportPVOInfo(Source.PrimaryVariableOutputData,Target);
  ExportOutputFilteringInfo(Source.PrimaryVariableOutputData,Source.x073ParameterSet,Target);
  Export_x073_Info2(Source.x073ParameterSet,Target);
  Export_x075_TripInfo(Source.x075TripParameters,Target);
end;

procedure Export_x575_ToGrid(Source: THartModule_x575; var Target: TStringDynGrid);
begin
  ExporTHDeviceIdentifier(Source.BasicDeviceData.UID,Target);
  ExporTHTagDescriptorDate(Source.BasicDeviceData.TDD,Target);
  Export_x073_Info1(Source.BasicDeviceData.UID,Source.PrimaryVariableOutputData, Source.x073ParameterSet,Target);
  ExportPVOInfo(Source.PrimaryVariableOutputData,Target);
  Export_x575_NE43(Source.NE43Mode, Target);
  ExportOutputFilteringInfo(Source.PrimaryVariableOutputData,Source.x073ParameterSet,Target);
  Export_x575_ResponseInfo(Source.ResponseTime,Target);
  Export_x073_Info2(Source.x073ParameterSet,Target);
  Export_x075_TripInfo(Source.x075TripParameters,Target);
end;

procedure Export_x573_ToGrid(Source: THartModule_x573; var Target: TStringDynGrid);
begin
  ExporTHDeviceIdentifier(Source.BasicDeviceData.UID,Target);
  ExporTHTagDescriptorDate(Source.BasicDeviceData.TDD,Target);
  Export_x073_Info1(Source.BasicDeviceData.UID,Source.PrimaryVariableOutputData, Source.x073ParameterSet,Target);
  ExportPVOInfo(Source.PrimaryVariableOutputData,Target);
  Export_x575_NE43(Source.NE43Mode, Target);
  ExportOutputFilteringInfo(Source.PrimaryVariableOutputData,Source.x073ParameterSet,Target);
  Export_x575_ResponseInfo(Source.ResponseTime,Target);
  Export_x073_Info2(Source.x073ParameterSet,Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(Source.ShortCircuitAlarmLevel),LanguageStrings[flAlarmSC],Target);
  Export_x573_TripInfo(Source.x075TripParameters.TripOutputFunction,Target);
end;

procedure Export_x576_ToGrid(Source: THartModule_x576; var Target: TStringDynGrid);
begin
  ExporTHDeviceIdentifier(Source.BasicDeviceData.UID,Target);
  ExporTHTagDescriptorDate(Source.BasicDeviceData.TDD,Target);
  ExportTwoStringsToGrid(LanguageStrings[flChannel],'1',Target);
  ExportChannelInputParameters_x576(Source.DeviceType,Source.InputParameters[0],Target);
  ExportChannelOutputParameters_x576(Source.OutputParameters[0],Target);
  ExportTwoStringsToGrid(LanguageStrings[flChannel],'2',Target);
  ExportChannelInputParameters_x576(Source.DeviceType,Source.InputParameters[1],Target);
  ExportChannelOutputParameters_x576(Source.OutputParameters[1],Target);
  ExportTHCommonParameters_x576(Source.CommonParameters,Target);
end;

procedure Export_x532_ToGrid(Source: THartModule_x532; var Target: TStringDynGrid);
begin
  ExporTHDeviceIdentifier(Source.BasicDeviceData.UID,Target);
  ExporTHTagDescriptorDate(Source.BasicDeviceData.TDD,Target);
  Export_x532_InputInfo(Source.InputConfiguration.InputSource,Target);
  ExportPVOInfo(Source.PrimaryVariableOutputData,Target);
  Export_x532_OutputInfo(Source.FullScale,Source.NE43Mode,Target);
  ExportThreeStringsToGrid(LanguageStrings[flDampingConst], FloatToStrF(Source.PrimaryVariableOutputData.DampingTimeConstant,ffFixed,4,1), 's', Target);
  Export_x532_AdjustmentInfo(Source.AdjustOutputParameters,Target);
  Export_x532_LeakFlowInfo(Source.LeakFlowOffParameters,Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(Source.TransmitterFailureAlarmLevel),LanguageStrings[flAlarmTX],Target);
  Export_x532_DividerInfo(Source.DividerParameters,THInputConfiguration_x532(Source.InputConfiguration).InputSource,Target);
  Export_x532_TripInfo(Source.TripParameters,THInputConfiguration_x532(Source.InputConfiguration),Target);
end;

procedure Export_1003_ToGrid(Source: THartModule_1003; var Target: TStringDynGrid);
begin
  ExporTHDeviceIdentifier(Source.BasicDeviceData.UID,Target);
  ExporTHTagDescriptorDate(Source.BasicDeviceData.TDD,Target);
  ExportPVOInfo(Source.PrimaryVariableOutputData,Target);
  Export_x532_OutputInfo(Source.FullScale,0,Target);
  ExportThreeStringsToGrid(LanguageStrings[flDampingConst], FloatToStrF(Source.PrimaryVariableOutputData.DampingTimeConstant,ffFixed,4,1), 's', Target);
  Export_1003_AdjustmentInfo(Source.AdjustOutputParameters,Target);
  Export_1003_LeakFlowInfo(Source.LeakFlowOffParameters,Target);
  Export_AlarmLevel_Info(AlarmLevelFromHART(Source.TransmitterFailureAlarmLevel),LanguageStrings[flAlarmTX],Target);
  Export_1003_DividerInfo(Source.TripAParameters,Source.TripBParameters,Source.DividerParameters,Source.InputAndLFDMode,Target);
  Export_1003_TripInfo(Source.TripAParameters,Source.TripBParameters,Source.InputAndLFDMode,Target);
end;

function ExportConfigurationToGrid(Source: TBasicHartModule): TStringDynGrid;
var lExportGrid: TStringDynGrid;
    lDT: TDeviceType;
begin
  SetLength(lExportGrid,0);
  lDT:=IdentToType(Source.BasicDeviceData.UID);

  if (lDT<>dtNone) then
     begin
     case lDT of
          dt507x,
          dtx073,
          dt5074XK,
          dt4073XK: Export_x073_ToGrid(THartModule_x073(Source),lExportGrid);
          dt4075:   Export_x075_ToGrid(THartModule_x075(Source),lExportGrid);
          dtx575:   Export_x575_ToGrid(THartModule_x575(Source),lExportGrid);
          dtx573:   Export_x573_ToGrid(THartModule_x573(Source),lExportGrid);
          dtx576:   Export_x576_ToGrid(THartModule_x576(Source),lExportGrid);
          dtx532:   Export_x532_ToGrid(THartModule_x532(Source),lExportGrid);
          dt1003:   Export_1003_ToGrid(THartModule_1003(Source),lExportGrid);
          end;
     ExportTwoStringsToGrid(LanguageStrings[mlCreationDate],DateToStr(Date),lExportGrid);
     end;

  Result:=lExportGrid;
end;

procedure PrintStringDynGrid(Source: TStringDynGrid);
var I, K, Y, Tab1, Tab2, Tab3, Step: integer;
    InterResult: real;
begin
  K:=System.Length(Source);

  if (K>0) then
     begin
     InterResult:=Printer.PageWidth/2323;
     Tab1:=round(300*InterResult);
     Tab2:=round(1150*InterResult);
     Tab3:=round(1400*InterResult);
     if (MOD45.DevType<>dtx576) then
        Step:=round(80*InterResult)
     else Step:=round(70*InterResult);
     Printer.BeginDoc; Y:=round(120*InterResult);
     with Printer.Canvas do
          begin
          Font.Name:='Arial';
          Font.Size:=14;
          TextOut (Tab1, Y, LanguageStrings[mlPrintConfigData]); Y:=Y+(Step*3) DIV 2;
          Font.Size:=12;

          for I:=0 to K-2 do
              begin
              if (System.Length(Source[I])>0) then
                 Printer.Canvas.TextOut (Tab1, Y, Source[I,0]);
              if (System.Length(Source[I])>1) then
                 Printer.Canvas.TextOut (Tab2, Y, Source[I,1]);
              if (System.Length(Source[I])>2) then
                 Printer.Canvas.TextOut (Tab3, Y, Source[I,2]);
              Y:=Y+Step;
              end;

          Y:=Y+Step DIV 2;

          if (System.Length(Source[K-1])>0) then
             Printer.Canvas.TextOut (Tab1, Y, Source[K-1,0]);
          if (System.Length(Source[K-1])>1) then
             Printer.Canvas.TextOut (Tab2, Y, Source[K-1,1]);
          end;

     if (Printer.PrinterIndex > 0)then
        MessageDlg(LanguageStrings[msSendToPrinter]+Printer.Printers[Printer.PrinterIndex],mtInformation,[mbOk],hcNoHelpTopic);
     Printer.EndDoc;
     end
  else
     begin
     MessageDlg(LanguageStrings[msNothingToPrint]+Printer.Printers[Printer.PrinterIndex],mtInformation,[mbOk],hcNoHelpTopic);
     end;
end;

end.
