{******************************************************************************
*******************************************************************************
*******************************************************************************
***
***	ORDER DESCRIPTION - CONFIGURATION SOFTWARE PCS45
***	LOADSTREAMDATA.PAS
***
***	Configuration data load subroutines for all file formates (also "old")
***
***	Autor : Georg  Hahn
***	Datum :  14.11.1996 / 28.09.00 / 31.03.03 / 23.08.05 / 24.10.06 / 09.03.07
***          25.10.08 / 21.02.11
***
*******************************************************************************
*******************************************************************************
******************************************************************************}
unit LoadStreamData;

interface

uses Classes, LnCRCFileStream, DataTypes, HartDataTypes;

  function Read_Very_Old_Configuration (Source: TLnCRCFileStream; var Target: TDx073): boolean;
  function Read_Old_Configuration (Source: TLnCRCFileStream; var Target: TDx073): boolean;
  function Read_x073_Configuration (Source: TLnCRCFileStream; var Target: TDx073; var TypeInfo: TDeviceType): boolean;
  function Read_x075_Configuration (Source: TLnCRCFileStream; var Target: TDx075; var TypeInfo: TDeviceType): boolean;
  function Read_x575_Configuration (Source: TLnCRCFileStream; var Target: TDx575; var TypeInfo: TDeviceType): boolean;

  procedure Old1003_To_New1003(Source: TDOld1003; var Target: TD1003);
  procedure New1003_To_Old1003(Source: TD1003; var Target: TDOld1003);
  function Read_1003_Configuration (Source: TLnCRCFileStream; var Target: TD1003; var TypeInfo: TDeviceType): boolean;
  function Read_4532_Configuration (Source: TLnCRCFileStream; var Target: TD4532): boolean;

implementation

uses SysUtils, ConversionUtils, DataConstants, ConversionProcs;

type

 // TVeryOldx073 record was used from PCC73 DOS-versions before 1.5.1 to save configuration data on disk.
 // PCS45 for Windows uses it only as a basis for load (translation) subroutine to accept files with
 // "very old" format *.MTL

  TVeryOldx073 = record
    load : (off_,on_,demo);
    pwd  : (yes,no);

    // READ:0
    V254 : byte;
    ManufacturersID : byte;
    ManufacturersDeviceType : byte;
    NRP : byte;
    UCR : byte;
    TSRH : byte;
    SoftwareRevision : byte;
    HardwareRevision : byte;
    FLAGSH : byte;
    DeviceIdentity : Array[1..3] of byte;

    // READ:3
    LoopCurrent : real;
    TemperatureUnit : byte;
    Temperature : real;
    CJUnit : byte;
    CJTemperature : real;

    // READ:13
    TagName : String[8];
    Descriptor : String[16];
    Day : byte;
    Month : byte;
    Year : byte;

    // READ:15
    Alarm : byte;
    Linearisation : byte;
    PVUnits : byte;
    Point20mA : real;
    Point4mA : real;
    DampingTimeConstant : real;

    // READ:128
    OpenCircuitAlarmLevel : byte;
    TransmitterFailureAlarmLevel : byte;
    FilterFrequencySelector : byte;
    InputType : byte;
    SmoothingTimeConstant : real;
    ColdJunctionCompensationOnOff : byte;
    ColdJunctionFailureAlarmLevel : byte;
  end;

function Read_Very_Old_Configuration (Source: TLnCRCFileStream; var Target: TDx073): boolean;
var VeryOld: TDataStream;
    I: byte;
begin
  Source.Read(VeryOld,Source.Size);

  // READ:0 - components
  Target.MDD.UID.DeviceTypeCode254       := VeryOld[2];
  Target.MDD.UID.ManufacturerIdentCode  := VeryOld[3];
  // Target.MDD.UID.ManufacturersDeviceType := VeryOld[4];
  Target.MDD.UID.ManufacturersDeviceType := 3;
  // very old data (from DOS PCC73) can be only 3 = 4073
  Target.MDD.UID.NumberOfPreambles       := VeryOld[5];
  Target.MDD.UID.RevisionLevelOfUCD      := VeryOld[6];
  Target.MDD.UID.RevisionLevelOfTSD      := VeryOld[7];
  Target.MDD.UID.SoftwareRevision        := VeryOld[8];
  Target.MDD.UID.HardwareRevision        := VeryOld[9];
  Target.MDD.UID.Flags                   := VeryOld[10];
  Target.MDD.UID.DeviceIdentNumber[1]    := VeryOld[11];
  Target.MDD.UID.DeviceIdentNumber[2]    := VeryOld[12];
  Target.MDD.UID.DeviceIdentNumber[3]    := VeryOld[13];

  // + 20 steps for READ:3 - components

  // READ:13 - components
  for I:=1 to 8 do Target.MDD.TDD.TagName[I]:=char(VeryOld[34+I]);
  SetLength(Target.MDD.TDD.TagName,VeryOld[34]);
  while System.Length(Target.MDD.TDD.TagName)<8 do Target.MDD.TDD.TagName:=Target.MDD.TDD.TagName+' ';


  for I:=1 to 16 do Target.MDD.TDD.Descriptor[I]:=char(VeryOld[43+I]);
  SetLength(Target.MDD.TDD.Descriptor,VeryOld[43]);
  while System.Length(Target.MDD.TDD.Descriptor)<16 do Target.MDD.TDD.Descriptor:=Target.MDD.TDD.Descriptor+' ';

  Target.MDD.TDD.Day                     := VeryOld[60];
  Target.MDD.TDD.Month                   := VeryOld[61];
  Target.MDD.TDD.Year                    := VeryOld[62];

  // READ:15 - components
  Target.MDD.PVI.PVAlarm                 := VeryOld[63];
  Target.MDD.PVI.TransferFunction        := VeryOld[64];
  Target.MDD.PVI.PVUnits                 := VeryOld[65];
  Target.MDD.PVI.Point20mA:=Byte6StreamToReal48(@VeryOld[66]);
  Target.MDD.PVI.Point4mA:=Byte6StreamToReal48(@VeryOld[72]);
  Target.MDD.PVI.DampingTimeConstant:=Byte6StreamToReal48(@VeryOld[78]);

  // READ:128 - components
  Target.x073.OpenCircuitAlarmLevel         := VeryOld[84];
  Target.x073.TransmitterFailureAlarmLevel  := VeryOld[85];
  Target.x073.FilterFrequencySelector       := VeryOld[86];
  Target.x073.InputType                     := VeryOld[87];
  Target.x073.SmoothingTimeConstant:=trunc(Byte6StreamToReal48(@VeryOld[88]));
  Target.x073.ColdJunctionCompensationOnOff := VeryOld[94];
  Target.x073.ColdJunctionFailureAlarmLevel  := VeryOld[95];

  Result:=(Target.MDD.UID.ManufacturerIdentCode=64) and
          (Target.MDD.UID.ManufacturersDeviceType in [2..7]);
end;

 // TOldx073 record was used from PCC73 DOS-version 1.5.1 to save configuration data on disk.
 // PCS45 for Windows uses it only as a basis for load (translation) subroutine to accept files
 // with "old" format *.D73. The most significant difference is, that Win32 uses 8 byte
 // to save real data while DOS have used 6 byte.

type

  TOldx073 = record
    DevType: word;
    MDD    : TMainDeviceData;
    x073   : Tx073ParameterSet;
    Checksum: byte;
    end;

function Read_Old_Configuration (Source: TLnCRCFileStream; var Target: TDx073): boolean;
var Old: TExtendedDataStream;
    HeaderString: string;
    DataSize: word;
    I: byte;
begin
  HeaderString:=Source.ReadStringCount(System.Length(Old_Config_Header));
  Source.Seek(System.Length(Old_Config_Header),soFromBeginning);
  if (HeaderString=Old_Config_Header) and ((Source.Size-Source.Position)<SizeOf(Old)) then
     begin
     FillChar(Old.Bytes,SizeOf(Old.Bytes),$FF);
     DataSize:=Source.Read(Old,Source.Size-Source.Position);
     if (DataSize=73) and // and (Old.MarkValue=73)
        (Old.Bytes[DataSize-3]=GetDataStreamParityByte(Old,DataSize)) then
        begin

        Target.MDD.UID.DeviceTypeCode254       := Old.Bytes[0];
        Target.MDD.UID.ManufacturerIdentCode  := Old.Bytes[1];
        Target.MDD.UID.ManufacturersDeviceType := Old.Bytes[2];
        Target.MDD.UID.NumberOfPreambles       := Old.Bytes[3];
        Target.MDD.UID.RevisionLevelOfUCD      := Old.Bytes[4];
        Target.MDD.UID.RevisionLevelOfTSD      := Old.Bytes[5];
        Target.MDD.UID.SoftwareRevision        := Old.Bytes[6];
        Target.MDD.UID.HardwareRevision        := Old.Bytes[7];
        Target.MDD.UID.Flags                   := Old.Bytes[8];
        Target.MDD.UID.DeviceIdentNumber[1]    := Old.Bytes[9];
        Target.MDD.UID.DeviceIdentNumber[2]    := Old.Bytes[10];
        Target.MDD.UID.DeviceIdentNumber[3]    := Old.Bytes[11];

        for I:=1 to 8 do Target.MDD.TDD.TagName[I]:=char(Old.Bytes[12+I]);
        SetLength(Target.MDD.TDD.TagName,Old.Bytes[12]);
        while System.Length(Target.MDD.TDD.TagName)<8 do Target.MDD.TDD.TagName:=Target.MDD.TDD.TagName+' ';

        for I:=1 to 16 do Target.MDD.TDD.Descriptor[I]:=char(Old.Bytes[21+I]);
        SetLength(Target.MDD.TDD.Descriptor,Old.Bytes[21]);
        while System.Length(Target.MDD.TDD.Descriptor)<16 do Target.MDD.TDD.Descriptor:=Target.MDD.TDD.Descriptor+' ';

        Target.MDD.TDD.Day                     := Old.Bytes[38];
        Target.MDD.TDD.Month                   := Old.Bytes[39];
        Target.MDD.TDD.Year                    := Old.Bytes[40];

        Target.MDD.PVI.PVAlarm                 := Old.Bytes[41];
        Target.MDD.PVI.TransferFunction        := Old.Bytes[42];
        Target.MDD.PVI.PVUnits                 := Old.Bytes[43];
        Target.MDD.PVI.Point20mA:=Byte6StreamToReal48(@Old.Bytes[44]);
        Target.MDD.PVI.Point4mA:=Byte6StreamToReal48(@Old.Bytes[50]);
        Target.MDD.PVI.DampingTimeConstant:=Byte6StreamToReal48(@Old.Bytes[56]);

        Target.x073.OpenCircuitAlarmLevel         := Old.Bytes[62];
        Target.x073.TransmitterFailureAlarmLevel  := Old.Bytes[63];
        Target.x073.FilterFrequencySelector       := Old.Bytes[64];
        Target.x073.InputType                     := Old.Bytes[65];
        Target.x073.SmoothingTimeConstant         := Old.Bytes[67]; // + Old.Bytes[66] * 256
        Target.x073.ColdJunctionCompensationOnOff := Old.Bytes[68];
        Target.x073.ColdJunctionFailureAlarmLevel := Old.Bytes[69];

        Result:=(Target.MDD.UID.ManufacturerIdentCode=64) and
                (Target.MDD.UID.ManufacturersDeviceType in [2..7]);
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Read_x073_Configuration (Source: TLnCRCFileStream; var Target: TDx073; var TypeInfo: TDeviceType): boolean;
begin
  Source.Seek(System.Length(New_Config_Header),soFromBeginning);
  if ((Source.Size-Source.Position)=SizeOf(Target)) then
     begin
     FillChar(Target,SizeOf(Target),$FF);
     if (Source.Read(Target,SizeOf(Target))=SizeOf(Target)) and
        (Target.Parity=GetDataStreamParityByte(Target,SizeOf(Target)-3)) then
        begin
        while System.Length(Target.MDD.TDD.TagName)<8 do Target.MDD.TDD.TagName:=Target.MDD.TDD.TagName+' ';
        while System.Length(Target.MDD.TDD.Descriptor)<16 do Target.MDD.TDD.Descriptor:=Target.MDD.TDD.Descriptor+' ';
        Result:=(Target.MDD.UID.ManufacturerIdentCode=64) and
                (Target.MDD.UID.ManufacturersDeviceType in [2..7]);
        TypeInfo:=IdentToType(Target.MDD.UID);
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Read_x075_Configuration (Source: TLnCRCFileStream; var Target: TDx075; var TypeInfo: TDeviceType): boolean;
begin
  Source.Seek(System.Length(New_Config_Header),soFromBeginning);
  if ((Source.Size-Source.Position)=SizeOf(Target)) then
     begin
     FillChar(Target,SizeOf(Target),$FF);
     if (Source.Read(Target,SizeOf(Target))=SizeOf(Target)) and
        (Target.Parity=GetDataStreamParityByte(Target,SizeOf(Target)-3)) then
        begin
        while System.Length(Target.MDD.TDD.TagName)<8 do Target.MDD.TDD.TagName:=Target.MDD.TDD.TagName+' ';
        while System.Length(Target.MDD.TDD.Descriptor)<16 do Target.MDD.TDD.Descriptor:=Target.MDD.TDD.Descriptor+' ';
        Result:=(Target.MDD.UID.ManufacturerIdentCode=64) and
                (Target.MDD.UID.ManufacturersDeviceType=8);
        TypeInfo:=IdentToType(Target.MDD.UID);
        end
     else Result:=false;
     end
  else Result:=false;
end;

function Read_x575_Configuration (Source: TLnCRCFileStream; var Target: TDx575; var TypeInfo: TDeviceType): boolean;
var SourceSize, TargetSize: Longint;
    SourceType: integer;
begin
  Source.Seek(System.Length(New_Config_Header),soFromBeginning);
  SourceSize:=Source.Size-Source.Position; TargetSize:=SizeOf(Target);

  if (TargetSize=SourceSize) or (TargetSize=SourceSize+1) then
     begin
     FillChar(Target,TargetSize,$FF);
     SourceSize:=Source.Read(Target,TargetSize);
     if (TargetSize=SourceSize) and
        (Target.Parity=GetDataStreamParityByte(Target,SizeOf(Target)-3)) then SourceType:=1 // actual Record
     else if ((TargetSize=SourceSize) and (Target.NAMUR=GetDataStreamParityByte(Target,SizeOf(Target)-3))) or
             ((TargetSize=SourceSize+1) and (Target.NAMUR=GetDataStreamParityByte(Target,SizeOf(Target)-(3+1)))) then SourceType:=2 // previous 4575 record
          else SourceType:=0;
     if (SourceType in [1,2]) then
        begin
        while System.Length(Target.MDD.TDD.TagName)<8 do Target.MDD.TDD.TagName:=Target.MDD.TDD.TagName+' ';
        while System.Length(Target.MDD.TDD.Descriptor)<16 do Target.MDD.TDD.Descriptor:=Target.MDD.TDD.Descriptor+' ';
        Result:=(Target.MDD.UID.ManufacturerIdentCode=64) and
                (Target.MDD.UID.ManufacturersDeviceType=9);
        TypeInfo:=IdentToType(Target.MDD.UID);
        if (SourceType=2) then
           begin
           Target.Parity:=Target.NAMUR;
           Target.NAMUR:=0;
           end;
        end
     else Result:=false;
     end
  else Result:=false;
end;

procedure Old1003_To_New1003(Source: TDOld1003; var Target: TD1003);
begin
  Target.MDD:=Source.MDD;
  Target.ADD.TRP:=Source.ADD.TRP;
  Target.ADD.CCF:=Source.ADD.CCF;
  Target.ADD.TransmitterFailureAlarmLevel:=Source.ADD.TransmitterFailureAlarmLevel;
  Target.ADD.FullScaleFlag:=Def_FullScale;
  Target.LFD_Inp:=Def_Input;
  Target.FTD:=Source.FTD;
end;

procedure New1003_To_Old1003(Source: TD1003; var Target: TDOld1003);
begin
  Target.MDD:=Source.MDD;
  Target.ADD.TRP:=Source.ADD.TRP;
  Target.ADD.CCF:=Source.ADD.CCF;
  Target.ADD.TransmitterFailureAlarmLevel:=Source.ADD.TransmitterFailureAlarmLevel;
  Target.FTD:=Source.FTD;
end;

function Read_1003_Configuration (Source: TLnCRCFileStream; var Target: TD1003; var TypeInfo: TDeviceType): boolean;
var Old1003: TDOld1003;
    Success: boolean;
begin
  Source.Seek(System.Length(New_Config_Header),soFromBeginning);
  if ((Source.Size-Source.Position)=SizeOf(Target)) then
     begin
     FillChar(Target,SizeOf(Target),$FF);
     if (Source.Read(Target,SizeOf(Target))=SizeOf(Target)) and
        (Target.Parity=GetDataStreamParityByte(Target,SizeOf(Target)-3)) then
        begin
        Success:=(Target.MDD.UID.ManufacturerIdentCode=64) and
                 (Target.MDD.UID.ManufacturersDeviceType=7);
        TypeInfo:=dt1003_new;
        Result:=Success;
        end
     else Result:=false;
     end
  else if ((Source.Size-Source.Position)=SizeOf(Old1003)) then
          begin
          FillChar(Old1003,SizeOf(Old1003),$FF);
          if (Source.Read(Old1003,SizeOf(Old1003))=SizeOf(Old1003)) and
             (Old1003.Parity=GetDataStreamParityByte(Old1003,SizeOf(Old1003)-3)) then
             begin
             Success:=(Old1003.MDD.UID.ManufacturerIdentCode=64) and
                      (Old1003.MDD.UID.ManufacturersDeviceType=7);
             if Success then Old1003_To_New1003(Old1003, Target);
             TypeInfo:=dt1003_old;
             Result:=Success;
             end
          else Result:=false;
          end
       else Result:=false;
end;

function Read_4532_Configuration (Source: TLnCRCFileStream; var Target: TD4532): boolean;
begin
  Source.Seek(System.Length(New_Config_Header),soFromBeginning);
  if ((Source.Size-Source.Position)=SizeOf(Target)) then
     begin
     FillChar(Target,SizeOf(Target),$FF);
     if (Source.Read(Target,SizeOf(Target))=SizeOf(Target)) and
        (Target.Parity=GetDataStreamParityByte(Target,SizeOf(Target)-3)) then // Get_Parity_Byte
        begin
        Result:=(Target.MDD.UID.ManufacturerIdentCode=64) and
                (Target.MDD.UID.ManufacturersDeviceType=6);
        end
     else Result:=false;
     end
  else Result:=false;
end;

end.

